@@ -26,12 +26,20 @@
  * with the assistance of funding from US Joint Forces Command Joint Concept Development &
  * Experimentation (J9) http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 23007 $
+ * Further assistance for enhancements from Imagicle spa
+ *
+ * $Revision: 28621 $
  * $Author: rjongbloed $
- * $Date: 2009-06-28 23:13:04 -0500 (Sun, 28 Jun 2009) $
+ * $Date: 2012-12-02 20:33:10 -0600 (Sun, 02 Dec 2012) $
  */
 
-#include <ptlib.h>
+#ifndef PTLIB_CLI_H
+#define PTLIB_CLI_H
+
+#include <ptbuildopts.h>
+
+#if P_CLI
+
 #include <ptlib/sockets.h>
 
 #include <list>
@@ -84,10 +92,10 @@ class PCLI : public PObject
            PCLI::m_newLine.
 
            The GetErrorCode() function should be consulted after Write() returns
-           PFalse to determine what caused the failure.
+           false to determine what caused the failure.
 
            @return
-           PTrue if at least len bytes were written to the channel.
+           true if at least len bytes were written to the channel.
          */
         virtual PBoolean Write(
           const void * buf, ///< Pointer to a block of memory to write.
@@ -106,6 +114,12 @@ class PCLI : public PObject
           */
         void Stop();
 
+        /**Run command line interpreter context.
+           This will execute OnStart() then ReadAndProcessInput() until it
+           returns false, then OnStop().
+          */
+        bool Run();
+
         /**Call back frunction for when context is started.
            This is usually called from within a background thread.
 
@@ -120,14 +134,20 @@ class PCLI : public PObject
           */
         virtual void OnStop();
 
+        /**Write prompt (depending on state) to channel.
+          */
+        virtual bool WritePrompt();
+
         /**Read a character from the attached channel an process.
            If the character was successfully read then ProcessInput() is called.
           */
         virtual bool ReadAndProcessInput();
 
         /**Process a character read from the channel.
+           Returns false if have error and processing is to cease.
           */
-        virtual void ProcessInput(char ch);
+        virtual bool ProcessInput(int ch);
+        virtual bool ProcessInput(const PString & line);
 
         /**Call back for a command line was completed and ENTER pressed.
            The default behaviour processes the line into a PArgList and deals
@@ -146,7 +166,7 @@ class PCLI : public PObject
 
         /**Indicate is currently processing a command.
           */
-        bool IsProcessingCommand() const { return m_processingCommand; }
+        bool IsProcessingCommand() const { return m_state == e_ProcessingCommand; }
       //@}
 
       protected:
@@ -154,10 +174,17 @@ class PCLI : public PObject
 
         PCLI      & m_cli;
         PString     m_commandLine;
-        bool        m_ignoreNextLF;
+        bool        m_ignoreNextEOL;
         PStringList m_commandHistory;
         PThread   * m_thread;
-        bool        m_processingCommand;
+
+        enum State {
+          e_Username,
+          e_Password,
+          e_CommandEntry,
+          e_ProcessingCommand
+        } m_state;
+        PString m_enteredUsername;
     };
 
     /**This class is an enhancement to PArgList to add context.
@@ -195,7 +222,6 @@ class PCLI : public PObject
 
       protected:
         Context & m_context;
-        PString   m_command;
         PString   m_usage;
 
       friend class PCLI;
@@ -223,8 +249,8 @@ class PCLI : public PObject
        that have been added will have their background threads started.
 
        If runInBackground is false, then there must only be one context added
-       and that context is continuously read until it's channel is closed or
-       returns end of file.
+       and that context is continuously read until it is stopped, it's channel
+       is closed or returns end of file.
       */
     virtual bool Start(
       bool runInBackground = true   ///< Spawn a thread to read and interpret commands
@@ -236,14 +262,17 @@ class PCLI : public PObject
       */
     virtual void Stop();
 
-    /**Open a command line interpreter context.
+    /**Start a command line interpreter context.
+       If \p runInBackground is true then the context is immediately started in
+       a thread. If false then the context is set up and will await a call to
+       the Start() function.
       */
-    bool StartContext(
+    Context * StartContext(
       PChannel * channel,           ///< Channel to read/write
       bool autoDelete = true,       ///< Automatically delete channel on exit
       bool runInBackground = true   ///< Spawn a thread to read and interpret commands
-    );
-    bool StartContext(
+    ) { return StartContext(channel, channel, autoDelete, autoDelete, runInBackground); }
+    Context * StartContext(
       PChannel * readChannel,      ///< Channel to be used for both read operations.
       PChannel * writeChannel,     ///< Channel to be used for both write operations.
       bool autoDeleteRead = true,  ///< Automatically delete the read channel
@@ -251,6 +280,39 @@ class PCLI : public PObject
       bool runInBackground = true   ///< Spawn a thread to read and interpret commands
     );
 
+    /** Start default foreground context.
+        Default action returns NULL.
+      */
+    virtual Context * StartForeground();
+
+    /**Run a command line interpreter context.
+       This will run the interpreter synchonously, continuously reading from
+       the channel supplied until the context is stopped or the channel is
+       closed or returns end of file.
+      */
+    bool Run(
+      PChannel & channel            ///< Channel to read/write
+    ) { return Run(&channel, &channel, false, false); }
+    bool Run(
+      PChannel * channel,           ///< Channel to read/write
+      bool autoDelete = true        ///< Automatically delete channel on exit
+    ) { return Run(channel, channel, autoDelete, autoDelete); }
+    bool Run(
+      PChannel * readChannel,      ///< Channel to be used for both read operations.
+      PChannel * writeChannel,     ///< Channel to be used for both write operations.
+      bool autoDeleteRead = true,  ///< Automatically delete the read channel
+      bool autoDeleteWrite = true  ///< Automatically delete the write channel
+    );
+
+    /** Run a context.
+       This will run the interpreter synchonously, continuously reading from
+       the channel supplied until the context is stopped or the channel is
+       closed or returns end of file.
+    */
+    bool RunContext(
+      Context * context
+    ) { return context != NULL && context->Run(); }
+
     /**Create a new context.
        Users may use this to create derived classes for their own use.
       */
@@ -285,6 +347,23 @@ class PCLI : public PObject
       Arguments & line
     );
 
+    /**Received a login name/pasword to be verified.
+       Note that the m_username or m_password field must be non-empty for a
+       log in sequence to occur, even if this function is overridden and the
+       memnbers not actually used for validation.
+
+       If the m_username is an empty string, but m_password is not, then it
+       the username not prompted for, just the password is required. The
+       reverse is also poassible and only a username entry required.
+
+       Default returns true if parameters are equal to m_username, m_password
+       respectively.
+      */
+    virtual bool OnLogIn(
+      const PString & username,
+      const PString & password
+    );
+
     /**Set a string to all command line interpreter contexts.
       */
     void Broadcast(
@@ -305,7 +384,8 @@ class PCLI : public PObject
       const char * command,       ///< Command(s) to register
       const PNotifier & notifier, ///< Callback to execute when command interpreted
       const char * help = NULL,   ///< Help text on command (what it does)
-      const char * usage = NULL   ///< Usage text on command (syntax/options)
+      const char * usage = NULL,  ///< Usage text on command (syntax/options)
+      const char * argSpec = NULL ///< Argument specification to pass to PArgList
     );
 
     /**Show help for registered commands to the context.
@@ -317,16 +397,6 @@ class PCLI : public PObject
 
   /**@name Member access */
   //@{
-    /**Get prompt used for command line interpreter.
-       Default is "CLI> ".
-      */
-    const PString & GetPrompt() const { return m_prompt; }
-
-    /**Set prompt used for command line interpreter.
-       Default is "CLI> ".
-      */
-    void SetPrompt(const PString & prompt) { m_prompt = prompt; }
-
     /**Get new line string output at the end of every line.
        Default is "\n".
       */
@@ -337,21 +407,115 @@ class PCLI : public PObject
       */
     void SetNewLine(const PString & newLine) { m_newLine = newLine; }
 
+    /**Get flag for echo is required for entered characters.
+       Default is false.
+      */
+    bool GetRequireEcho() const { return m_requireEcho; }
+
+    /**Set flag for echo is required for entered characters.
+       Default is false.
+      */
+    void SetRequireEcho(bool requireEcho) { m_requireEcho = requireEcho; }
+
+    /**Get characters used for editing (backspace/delete) command lines.
+       Default is "\b\x7f".
+      */
+    const PString & GetEditCharacters() const { return m_editCharacters; }
+
+    /**Set characters used for editing (backspace/delete) command lines.
+       Default is "\b\x7f".
+      */
+    void SetEditCharacters(const PString & editCharacters) { m_editCharacters = editCharacters; }
+
+    /**Get prompt used for command line interpreter.
+       Default is "CLI> ".
+      */
+    const PString & GetPrompt() const { return m_prompt; }
+
+    /**Set prompt used for command line interpreter.
+       Default is "CLI> ".
+      */
+    void SetPrompt(const PString & prompt) { m_prompt = prompt; }
+
+    /**Get prompt used for login (if enabled).
+       Default is "Username: ".
+      */
+    const PString & GetUsernamePrompt() const { return m_usernamePrompt; }
+
+    /**Set prompt used for login (if enabled).
+       Default is "Username: ".
+      */
+    void SetUsernamePrompt(const PString & prompt) { m_usernamePrompt = prompt; }
+
+    /**Get prompt used for password (if enabled).
+       Default is "Password: ".
+      */
+    const PString & GetPasswordPrompt() const { return m_passwordPrompt; }
+
+    /**Set prompt used for password (if enabled).
+       Default is "Password: ".
+      */
+    void SetPasswordPrompt(const PString & prompt) { m_passwordPrompt = prompt; }
+
+    /**Get username for log in validation.
+       Default is empty string, disabling entry of username/password.
+      */
+    const PString & GetUsername() const { return m_username; }
+
+    /**Set username for log in validation.
+       Default is empty string, disabling entry of username/password.
+      */
+    void SetUsername(const PString & username) { m_username = username; }
+
+    /**Get password for log in validation.
+       Default is empty string, disabling entry of password.
+      */
+    const PString & GetPassword() const { return m_password; }
+
+    /**Set password for log in validation.
+       Default is empty string, disabling entry of password.
+      */
+    void SetPassword(const PString & password) { m_password = password; }
+
+    /**Get command to be used for comment lines.
+       Default is "#\n;\n//".
+      */
+    const PCaselessString & GetCommentCommand() const { return m_commentCommand; }
+
+    /**Set command to be used for comment lines.
+       Default is "#\n;\n//".
+      */
+    void SetCommentCommand(const PCaselessString & commentCommand) { m_commentCommand = commentCommand; }
+
+    /**Get command to be used to exit session.
+       Default is "exit\nquit".
+      */
+    const PCaselessString & GetExitCommand() const { return m_exitCommand; }
+
+    /**Set command to be used to exit session.
+       Default is "exit\nquit".
+      */
+    void SetExitCommand(const PCaselessString & exitCommand) { m_exitCommand = exitCommand; }
+
     /**Get command to be used to display help.
-       Default is "?".
+       Default is "?\nhelp".
       */
     const PCaselessString & GetHelpCommand() const { return m_helpCommand; }
 
     /**Set command to be used to display help.
-       Default is "?".
+       Default is "?\nhelp".
       */
     void SetHelpCommand(const PCaselessString & helpCommand) { m_helpCommand = helpCommand; }
 
     /**Get help on help.
+       This string is output before the individual help is output for each command.
+       Default describes ?, !, !n, !! followed by "Command available are:".
       */
     const PString & GetHelpOnHelp() const { return m_helpOnHelp; }
 
     /**Set help on help.
+       This string is output before the individual help is output for each command.
+       Default describes ?, !, !n, !! followed by "Command available are:".
       */
     void SetHelpOnHelp(const PCaselessString & helpOnHelp) { m_helpOnHelp = helpOnHelp; }
 
@@ -376,41 +540,58 @@ class PCLI : public PObject
     void SetHistoryCommand(const PCaselessString & historyCommand) { m_historyCommand = historyCommand; }
 
     /**Get error message for if there is no history.
+       Default is "No command history".
       */
     const PString & GetNoHistoryError() const { return m_noHistoryError; }
 
     /**Set error message for if there is no history.
+       Default is "No command history".
       */
     void SetNoHistoryError(const PString & noHistoryError) { m_noHistoryError = noHistoryError; }
 
-    /**Get usage prefix for if Arguments::WriteError() called.
+    /**Get usage prefix for if Arguments::WriteUsage() called.
+       Default is "Usage: ".
       */
     const PString & GetCommandUsagePrefix() const { return m_commandUsagePrefix; }
 
-    /**Set usage prefix for if Arguments::WriteError() called.
+    /**Set usage prefix for if Arguments::WriteUsage() called.
+       Default is "Usage: ".
       */
     void SetCommandUsagePrefix(const PString & commandUsagePrefix) { m_commandUsagePrefix = commandUsagePrefix; }
 
     /**Get error prefix for if Arguments::WriteError() called.
+       Default is ": error: ", always prefixed by command name.
       */
     const PString & GetCommandErrorPrefix() const { return m_commandErrorPrefix; }
 
     /**Set error prefix for if Arguments::WriteError() called.
+       Default is ": error: ", always prefixed by command name.
       */
     void SetCommandErrorPrefix(const PString & commandErrorPrefix) { m_commandErrorPrefix = commandErrorPrefix; }
 
     /**Get error message for if unknown command is entered.
+       Default is "Unknown command".
       */
     const PString & GetUnknownCommandError() const { return m_unknownCommandError; }
 
     /**Set error message for if unknown command is entered.
+       Default is "Unknown command".
       */
     void SetUnknownCommandError(const PString & unknownCommandError) { m_unknownCommandError = unknownCommandError; }
   //@}
 
+
   protected:
-    PString         m_prompt;
     PString         m_newLine;
+    bool            m_requireEcho;
+    PString         m_editCharacters;
+    PString         m_prompt;
+    PString         m_usernamePrompt;
+    PString         m_passwordPrompt;
+    PString         m_username;
+    PString         m_password;
+    PCaselessString m_commentCommand;
+    PCaselessString m_exitCommand;
     PCaselessString m_helpCommand;
     PString         m_helpOnHelp;
     PCaselessString m_repeatCommand;
@@ -424,6 +605,7 @@ class PCLI : public PObject
       PNotifier m_notifier;
       PString   m_help;
       PString   m_usage;
+      PString   m_argSpec;
     };
     typedef std::map<PString, InternalCommand> CommandMap_t;
     CommandMap_t m_commands;
@@ -450,12 +632,23 @@ class PCLIStandard : public PCLI
 
   /**@name Overrides from PCLI */
   //@{
-    /**Start a command line interpreter.
-       As for ancestor function, however if no contexts have been added, then
-       on that takes a PConsoleChannel is automatically added.
+    /** Start default foreground context.
+        Default behaviour returns a context using stdin/stdout.
       */
-    virtual bool Start(
-      bool runInBackground = true   ///< Spawn a thread to read and interpret commands
+    virtual Context * StartForeground();
+  //@}
+
+  /**@name Operations */
+  //@{
+    /**Run a script file, output going to stdout.
+       This will also suppress output of the prompt.
+      */
+    bool RunScript(
+      PChannel & channel            ///< Channel to read from
+    ) { return RunScript(&channel, false); }
+    bool RunScript(
+      PChannel * channel,           ///< Channel to read from
+      bool autoDelete = true        ///< Automatically delete channel on exit
     );
   //@}
 };
@@ -523,11 +716,13 @@ class PCLISocket : public PCLI
     /**Get the port we are listing on.
       */
     WORD GetPort() const { return m_listenSocket.GetPort(); }
+  //@}
 
   protected:
     PDECLARE_NOTIFIER(PThread, PCLISocket, ThreadMain);
     bool HandleSingleThreadForAll();
     bool HandleIncoming();
+    virtual PTCPSocket * CreateSocket();
 
     bool m_singleThreadForAll;
 
@@ -537,3 +732,33 @@ class PCLISocket : public PCLI
     typedef std::map<PSocket *, Context *> ContextMap_t;
     ContextMap_t m_contextBySocket;
 };
+
+
+#if P_TELNET
+/**Command Line Interpreter over Telnet sockets.
+   This class allows for access and automatic creation of command line
+   interpreter contexts from incoming Telnet connections on a listening port.
+  */
+class PCLITelnet : public PCLISocket
+{
+  public:
+  /**@name Construction */
+  //@{
+    PCLITelnet(
+      WORD port = 0,
+      const char * prompt = NULL,
+      bool singleThreadForAll = false
+    );
+  //@}
+
+  protected:
+    virtual PTCPSocket * CreateSocket();
+};
+#endif // P_TELNET
+
+#endif // P_CLI
+
+#endif // PTLIB_CLI_H
+
+
+// End Of File ///////////////////////////////////////////////////////////////
