@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23897 $
+ * $Revision: 29414 $
  * $Author: rjongbloed $
- * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
+ * $Date: 2013-04-02 15:39:30 -0500 (Tue, 02 Apr 2013) $
  */
 
 
@@ -41,7 +41,7 @@
    encoding mechanism as defined in RFC1521.
 
    To encode a large block of data use the following seqeunce:
-<code>
+<pre><code>
       PBase64 base;
       base.StartEncoding();
       while (Read(dataChunk)) {
@@ -49,23 +49,23 @@
         out << base.GetEncodedString();
       }
       out << base.CompleteEncoding();
-</code>
+</code></pre>
 
     if smaller blocks that fit easily in memory are to be encoded the
-    #Encode()# functions can be used to everything in one go.
+    <code>Encode()</code> functions can be used to everything in one go.
 
     To decode a large block of data use the following sequence:
-<code>
+<pre><code>
 
       PBase64 base;
       base.StartDecoding();
       while (Read(str) && ProcessDecoding(str))
         Write(base.GetDecodedData());
       Write(base.GetDecodedData());
-</code>
+</code></pre>
 
     if smaller blocks that fit easily in memory are to be decoded the
-    #Decode()# functions can be used to everything in one go.
+    <code>Decode()</code> functions can be used to everything in one go.
  */
 class PBase64 : public PObject
 {
@@ -73,12 +73,15 @@ class PBase64 : public PObject
 
   public:
     /** Construct a base 64 encoder/decoder and initialise both encode and
-       decode members as in #StartEncoding()# and #StartDecoding()#.
+       decode members as in <code>StartEncoding()</code> and <code>StartDecoding()</code>.
      */
     PBase64();
 
     void StartEncoding(
-      PBoolean useCRLFs = PTrue  // Use CR, LF pairs in end of line characters.
+      bool useCRLFs = true  ///< Use CR, LF pairs in end of line characters.
+    );
+    void StartEncoding(
+      const char * endOfLine  ///< String to use for end of line.
     );
     // Begin a base 64 encoding operation, initialising the object instance.
 
@@ -105,8 +108,7 @@ class PBase64 : public PObject
     PString GetEncodedString();
 
     /** Complete the base 64 encoding and return the remainder of the encoded
-       Base64 string. Previous data may have been already removed by the
-       #GetInterim()# function.
+       Base64 string.
     
        @return
        Base64 encoded string for the processed data.
@@ -115,17 +117,21 @@ class PBase64 : public PObject
 
 
     static PString Encode(
-      const PString & str     // String to be encoded to Base64
+      const PString & str,          ///< String to be encoded to Base64
+      const char * endOfLine = "\n" ///< String to use for end of line.
     );
     static PString Encode(
-      const char * cstr       // C String to be encoded to Base64
+      const char * cstr,            ///< C String to be encoded to Base64
+      const char * endOfLine = "\n" ///< String to use for end of line.
     );
     static PString Encode(
-      const PBYTEArray & data // Data block to be encoded to Base64
+      const PBYTEArray & data,      ///< Data block to be encoded to Base64
+      const char * endOfLine = "\n" ///< String to use for end of line.
     );
     static PString Encode(
-      const void * dataBlock, // Pointer to data to be encoded to Base64
-      PINDEX length           // Length of the data block.
+      const void * dataBlock,       ///< Pointer to data to be encoded to Base64
+      PINDEX length,                ///< Length of the data block.
+      const char * endOfLine = "\n" ///< String to use for end of line.
     );
     // Encode the data in memory to Base 64 data returnin the string.
 
@@ -136,7 +142,7 @@ class PBase64 : public PObject
     /** Incorporate the specified data into the base 64 decoding.
     
        @return
-       PTrue if block was last in the Base64 encoded string.
+       true if block was last in the Base64 encoded string.
      */
     PBoolean ProcessDecoding(
       const PString & str      // String to be encoded
@@ -169,8 +175,8 @@ class PBase64 : public PObject
     /** Convert a printable text string to binary data using the Internet MIME
        standard base 64 content transfer encoding.
 
-       The base64 string is checked and PTrue returned if all perfectly correct.
-       If PFalse is returned then the string had extraneous or illegal
+       The base64 string is checked and true returned if all perfectly correct.
+       If false is returned then the string had extraneous or illegal
        characters in it that were ignored. This does not mean that the data is
        not valid, only that it is suspect.
     
@@ -196,13 +202,12 @@ class PBase64 : public PObject
     void OutputBase64(const BYTE * data);
 
     PString encodedString;
-    PINDEX  encodeLength;
     BYTE    saveTriple[3];
     PINDEX  saveCount;
     PINDEX  nextLine;
-    PBoolean    useCRLFs;
+    PString endOfLine;
 
-    PBoolean       perfectDecode;
+    bool       perfectDecode;
     PINDEX     quadPosition;
     PBYTEArray decodedData;
     PINDEX     decodeSize;
@@ -216,15 +221,12 @@ class PMessageDigest : public PObject
     /// Create a new message digestor
     PMessageDigest();
 
-    class Result {
+    class Result : public PBYTEArray {
       public:
-        PINDEX GetSize() const          { return value.GetSize(); }
-        const BYTE * GetPointer() const { return (const BYTE *)value; }
+        virtual void PrintOn(ostream & strm) const;
 
-      private:
-        PBYTEArray value;
-        friend class PMessageDigest5;
-        friend class PMessageDigestSHA1;
+        PString AsBase64() const { return PBase64::Encode(*this, ""); }
+        PString AsHex() const;
     };
 
     /// Begin a Message Digest operation, initialising the object instance.
@@ -272,6 +274,46 @@ class PMessageDigest : public PObject
 };
 
 
+/** HMAC template class.
+ Create a class that implemented a HMAC as per RFC 2104 using an arbitrary hash class
+ */
+
+class PHMAC : public PObject
+{
+  public:
+    typedef PMessageDigest::Result Result;
+    virtual void Hash(const BYTE * data, PINDEX len, Result & result) = 0;
+
+    virtual PString Encode(const BYTE * data, PINDEX len);
+    virtual PString Encode(const PBYTEArray & data);
+    virtual PString Encode(const PString & str);
+
+    virtual void Process(const BYTE * data, PINDEX len, Result & result);
+    virtual void Process(const PBYTEArray & data, Result & result);
+    virtual void Process(const PString & str, Result & result);
+
+  protected:
+    virtual int GetL() const = 0;
+    virtual int GetB() const { return 64; }
+    virtual void Initialise(const BYTE * key, PINDEX len);
+    virtual void InternalProcess(const BYTE * data, PINDEX len, PHMAC::Result & result);
+
+    PBYTEArray m_key;
+};
+
+template <class hash_class>
+class PHMACTemplate : public PHMAC
+{
+  public:
+    PHMACTemplate(const PString & key)                   { Initialise((const BYTE *)(const char *)key, key.GetLength()); }
+    PHMACTemplate(const PBYTEArray & key)                { Initialise((const BYTE *)key, key.GetSize()); }
+    PHMACTemplate(const BYTE * key, PINDEX len)          { Initialise(key, len); }
+
+    virtual void Hash(const BYTE * data, PINDEX len, Result & result)
+    { hash_class::Encode(data, len, result); }
+    virtual int GetL() const { return 20; };
+};
+
 /** MD5 Message Digest.
  A class to produce a Message Digest for a block of text/data using the
  MD5 algorithm as defined in RFC1321 by Ronald Rivest of MIT Laboratory
@@ -389,6 +431,8 @@ class PMessageDigest5 : public PMessageDigest
     PUInt64 count;
 };
 
+typedef PHMACTemplate<PMessageDigest5> PHMAC_MD5;
+
 #if P_SSL
 
 /** SHA1 Digest.
@@ -464,6 +508,8 @@ class PMessageDigestSHA1 : public PMessageDigest
     void * shaContext;
 };
 
+typedef PHMACTemplate<PMessageDigestSHA1> PHMAC_SHA1;
+
 #endif
 
 /**This abstract class defines an encryption/decryption algortihm.
@@ -786,7 +832,7 @@ class PSecureConfig : public PConfig
        they are copied to the secured variables.
 
        @return
-       PTrue if secure key values are valid.
+       true if secure key values are valid.
      */
 
     void ResetPending();
