@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22374 $
+ * $Revision: 28975 $
  * $Author: rjongbloed $
- * $Date: 2009-04-06 21:41:26 -0500 (Mon, 06 Apr 2009) $
+ * $Date: 2013-01-23 20:28:34 -0600 (Wed, 23 Jan 2013) $
  */
 
 #ifndef PTLIB_DELAYCHAN_H
@@ -75,12 +75,12 @@ class PAdaptiveDelay : public PObject
 
        If @a maximumSlip is 0, this feature is disabled.
       */
-    void SetMaximumSlip(PTimeInterval maximumSlip)
-    { jitterLimit = maximumSlip; }
+    void SetMaximumSlip(unsigned maximumSlip)
+    { m_jitterLimit = -(int)maximumSlip; }
 
     /**Get the current slip time. */
     PTimeInterval GetMaximumSlip() const
-    { return jitterLimit; }
+    { return -m_jitterLimit; }
   //@}
 
   /**@name Functionality */
@@ -96,7 +96,7 @@ class PAdaptiveDelay : public PObject
        than later" strategy).
 
        @return
-       PTrue if we are "too late" of @a time milliseconds (unrelated to
+       true if we are "too late" of @a time milliseconds (unrelated to
        the maximum slip time).
       */
     PBoolean Delay(int time);
@@ -108,11 +108,10 @@ class PAdaptiveDelay : public PObject
   //@}
  
   protected:
-    PBoolean   firstTime;
-    PTime  targetTime;
-
-    PTimeInterval  jitterLimit;
-    PTimeInterval  minimumDelay;
+    PTimeInterval  m_jitterLimit;
+    PTimeInterval  m_minimumDelay;
+    PTime          m_targetTime;
+    bool           m_firstTime;
 };
 
 
@@ -183,11 +182,11 @@ class PDelayChannel : public PIndirectChannel
        of bytes read.
 
        The GetErrorCode() function should be consulted after Read() returns
-       PFalse to determine what caused the failure.
+       false to determine what caused the failure.
 
        @return
-       PTrue indicates that at least one character was read from the channel.
-       PFalse means no bytes were read due to timeout or some other I/O error.
+       true indicates that at least one character was read from the channel.
+       false means no bytes were read due to timeout or some other I/O error.
      */
     virtual PBoolean Read(
       void * buf,   ///< Pointer to a block of memory to receive the read bytes.
@@ -199,9 +198,9 @@ class PDelayChannel : public PIndirectChannel
        of bytes written.
 
        The GetErrorCode() function should be consulted after Write() returns
-       PFalse to determine what caused the failure.
+       false to determine what caused the failure.
 
-       @return PTrue if at least len bytes were written to the channel.
+       @return true if at least len bytes were written to the channel.
      */
     virtual PBoolean Write(
       const void * buf, ///< Pointer to a block of memory to write.
