@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23951 $
+ * $Revision: 27547 $
  * $Author: rjongbloed $
- * $Date: 2010-01-20 00:51:49 -0600 (Wed, 20 Jan 2010) $
+ * $Date: 2012-04-27 22:17:28 -0500 (Fri, 27 Apr 2012) $
  */
 
 #ifndef PTLIB_HTML_H
@@ -41,7 +41,7 @@
 // PHTML
 
 /** This class describes a HyperText markup Language string as used by the
-   World Wide Web and the #PURL# and #PHTTPSocket# class.
+   World Wide Web and the <code>PURL</code> and <code>PHTTP</code> class.
    
    All of the standard stream I/O operators, manipulators etc will operate on
    the PString class.
@@ -139,6 +139,16 @@ class PHTML : public PStringStream
     void Toggle(ElementInSet elmt);
 
 
+    class Escaped {
+      public:
+        Escaped(const char * str) : m_str(str) { }
+      private:
+        void Output(ostream & strm) const;
+        const char * m_str;
+      friend ostream & operator<<(ostream & strm, const Escaped & e) { e.Output(strm); return strm; }
+    };
+    static PString Escape(const char * str);
+
     class Element {
       public: 
         virtual ~Element() {}
