@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25387 $
+ * $Revision: 28997 $
  * $Author: rjongbloed $
- * $Date: 2011-03-22 22:51:09 -0500 (Tue, 22 Mar 2011) $
+ * $Date: 2013-01-25 20:37:28 -0600 (Fri, 25 Jan 2013) $
  */
 
 #ifndef PTLIB_HTTP_H
@@ -176,6 +176,9 @@ class PHTTP : public PInternetProtocol
     };
 
     enum StatusCode {
+      BadResponse = 1,             ///<   1 = response is malformed
+      ContentProcessorError,       ///< Content Processer returned error
+      UnknownTransferEncoding,     ///< Unknown "Transfer-Encoding" extension
       Continue = 100,              ///< 100 - Continue
       SwitchingProtocols,          ///< 101 - upgrade allowed
       RequestOK = 200,             ///< 200 - request has succeeded
@@ -212,6 +215,7 @@ class PHTTP : public PInternetProtocol
     };
 
     // Common MIME header tags
+    static const PCaselessString & HostTag();
     static const PCaselessString & AllowTag();
     static const PCaselessString & AuthorizationTag();
     static const PCaselessString & ContentEncodingTag();
@@ -245,6 +249,9 @@ class PHTTP : public PInternetProtocol
     /** Create a TCP/IP HTTP protocol channel.
      */
     PHTTP();
+    PHTTP(
+      const char * defaultServiceName  ///< Service name for the protocol.
+    );
 
     /** Parse a response line string into a response code and any extra info
        on the line. Results are placed into the member variables
@@ -468,13 +475,13 @@ class PHTTPClient : public PHTTP
     );
 
     /// Write a HTTP command to server
-    PBoolean WriteCommand(
+    bool WriteCommand(
       Commands cmd,
       const PString & url,
       PMIMEInfo & outMIME,
       const PString & dataBody
     );
-    PBoolean WriteCommand(
+    bool WriteCommand(
       const PString & cmdName,
       const PString & url,
       PMIMEInfo & outMIME,
@@ -482,18 +489,67 @@ class PHTTPClient : public PHTTP
     );
 
     /// Read a response from the server
-    PBoolean ReadResponse(
+    bool ReadResponse(
       PMIMEInfo & replyMIME
     );
 
+    /// Call back to process the body of the HTTP command
+    struct ContentProcessor
+    {
+      virtual ~ContentProcessor() { }
+      virtual void * GetBuffer(PINDEX & size) = 0;
+      virtual bool Process(const void * data, PINDEX length) = 0;
+    };
+
     /// Read the body of the HTTP command
-    PBoolean ReadContentBody(
-      PMIMEInfo & replyMIME,
-      PBYTEArray & body
+    bool ReadContentBody(
+      PMIMEInfo & replyMIME,        ///< Reply MIME from server
+      ContentProcessor & processor  ///< Processor for received body
     );
-    PBoolean ReadContentBody(
-      PMIMEInfo & replyMIME,
-      PString & body
+
+    /// Read the body of the HTTP command, throwing it away
+    bool ReadContentBody(
+      PMIMEInfo & replyMIME   ///< Reply MIME from server
+    );
+
+    /// Read the body of the HTTP command as a string
+    bool ReadContentBody(
+      PMIMEInfo & replyMIME,        ///< Reply MIME from server
+      PString & body                ///< Received body as a string
+    );
+
+    /// Read the body of the HTTP command as a binary block
+    bool ReadContentBody(
+      PMIMEInfo & replyMIME,        ///< Reply MIME from server
+      PBYTEArray & body             ///< Received body as binary data
+    );
+
+
+    /** Start getting the document specified by the URL.
+        This does not return until completed, and continuously calls the
+        ContentProcessor with chunks of data.
+
+       @return true if document has successfully been transferred.
+     */
+    bool GetDocument(
+      const PURL & url,             ///< Universal Resource Locator for document.
+      ContentProcessor & processor  ///< Processor for received body
+    );
+
+    /** Start getting the document specified by the URL.
+        It is expected that ReadContentBody() is called after this returns to
+        get, or throw away, the data body received.
+
+       @return true if document is being transferred.
+     */
+    bool GetDocument(
+      const PURL & url,         ///< Universal Resource Locator for document.
+      PMIMEInfo & outMIME,      ///< MIME info in request
+      PMIMEInfo & replyMIME     ///< MIME info in response
+    );
+    bool GetDocument(
+      const PURL & url,         ///< Universal Resource Locator for document.
+      PMIMEInfo & replyMIME     ///< MIME info in response
     );
 
 
@@ -502,10 +558,9 @@ class PHTTPClient : public PHTTP
         An empty string for the contentType parameter means that any content
         type is acceptable.
 
-       @return
-       true if document is being transferred.
+        @return true if document is being transferred.
      */
-    PBoolean GetTextDocument(
+    bool GetTextDocument(
       const PURL & url,         ///< Universal Resource Locator for document.
       PString & document,       ///< Body read
       const PString & contentType = PString::Empty() ///< Content-Type header to expect
@@ -513,33 +568,62 @@ class PHTTPClient : public PHTTP
 
     /** Get the document specified by the URL.
 
-       @return
-       true if document is being transferred.
+        An empty string for the contentType parameter means that any content
+        type is acceptable.
+
+        @return true if document is being transferred.
      */
-    PBoolean GetDocument(
+    bool GetBinaryDocument(
       const PURL & url,         ///< Universal Resource Locator for document.
-      PMIMEInfo & outMIME,      ///< MIME info in request
-      PMIMEInfo & replyMIME     ///< MIME info in response
+      PBYTEArray & document,    ///< Body read
+      const PString & contentType = PString::Empty() ///< Content-Type header to expect
     );
 
+
     /** Get the header for the document specified by the URL.
 
        @return
        true if document header is being transferred.
      */
-    PBoolean GetHeader(
+    bool GetHeader(
       const PURL & url,         ///< Universal Resource Locator for document.
       PMIMEInfo & outMIME,      ///< MIME info in request
       PMIMEInfo & replyMIME     ///< MIME info in response
     );
+    bool GetHeader(
+      const PURL & url,         ///< Universal Resource Locator for document.
+      PMIMEInfo & replyMIME     ///< MIME info in response
+    );
 
 
+    /** Post the "application/x-www-form-urlencoded" data specified to the URL.
+        The data returned is thrown away.
+
+       @return true if document is being transferred.
+     */
+    bool PostData(
+      const PURL & url,         ///< Universal Resource Locator for document.
+      const PStringToString & data  ///< Information posted to the HTTP server.
+    );
+
     /** Post the data specified to the URL.
+        The data returned is thrown away.
 
-       @return
-       true if document is being transferred.
+       @return true if document is being transferred.
      */
-    PBoolean PostData(
+    bool PostData(
+      const PURL & url,         ///< Universal Resource Locator for document.
+      PMIMEInfo & outMIME,      ///< MIME info in request
+      const PString & data    ///< Information posted to the HTTP server.
+    );
+
+    /** Start a post of the data specified to the URL.
+        It is expected that ReadContentBody() is called after this returns to
+        get, or throw away, the data body received.
+
+       @return true if document is being transferred.
+     */
+    bool PostData(
       const PURL & url,       ///< Universal Resource Locator for document.
       PMIMEInfo & outMIME,    ///< MIME info in request
       const PString & data,   ///< Information posted to the HTTP server.
@@ -547,11 +631,11 @@ class PHTTPClient : public PHTTP
     );
 
     /** Post the data specified to the URL.
+        The body of the reply is returned
 
-       @return
-       true if document is being transferred.
+       @return true if document is has been transferred.
      */
-    PBoolean PostData(
+    bool PostData(
       const PURL & url,       ///< Universal Resource Locator for document.
       PMIMEInfo & outMIME,    ///< MIME info in request
       const PString & data,   ///< Information posted to the HTTP server.
@@ -567,7 +651,7 @@ class PHTTPClient : public PHTTP
     bool PutTextDocument(
       const PURL & url,         ///< Universal Resource Locator for document.
       const PString & document,       ///< Body read
-      const PString & contentType = "text/plain" ///< Content-Type header to use
+      const PString & contentType = PMIMEInfo::TextPlain() ///< Content-Type header to use
     );
 
     /** Put the document specified by the URL.
@@ -597,6 +681,14 @@ class PHTTPClient : public PHTTP
       const PString & password
     );
 
+#if P_SSL
+    void SetSSLCredentials(
+      const PString & authority,
+      const PString & certificate,
+      const PString & privateKey
+    );
+#endif
+
     /// Set persistent connection mode
     void SetPersistent(
       bool persist = true
@@ -606,16 +698,17 @@ class PHTTPClient : public PHTTP
     bool GetPersistent() const { return m_persist; }
 
   protected:
-    PBoolean AssureConnect(const PURL & url, PMIMEInfo & outMIME);
-    bool InternalReadContentBody(
-      PMIMEInfo & replyMIME,
-      PAbstractArray * body
-    );
+    bool AssureConnect(const PURL & url, PMIMEInfo & outMIME);
 
     PString m_userAgentName;
     bool    m_persist;
     PString m_userName;
     PString m_password;
+#if P_SSL
+    PString m_authority;    // Directory, file or data
+    PString m_certificate;  // File or data
+    PString m_privateKey;   // File or data
+#endif
     PHTTPClientAuthentication * m_authentication;
 };
 
@@ -915,6 +1008,19 @@ class PHTTPServer : public PHTTP
       */
     PHTTPConnectionInfo & GetConnectionInfo() { return connectInfo; }
 
+    /**Called when a request is received. 
+       By default, this intercepts the GET, HEAD and POST commands
+       and calls OnGET, OnHEAD and OnPOST. For all other command,
+       OnUnknown is called
+       */
+    virtual bool OnCommand(
+      PINDEX cmd,
+      const PURL & url, 
+      const PString & args,
+      PHTTPConnectionInfo & connectInfo
+    );
+
+
   protected:
     void Construct();
 
@@ -1223,6 +1329,10 @@ class PHTTPResource : public PObject
      */
     const PURL & GetURL() const { return baseURL; }
 
+    /**Get relative hot link for this resouce.
+      */
+    PString GetHotLink() const { return baseURL.AsString(PURL::PathOnly); }
+
     /** Get the current content type for the resource.
 
        @return
@@ -1882,9 +1992,6 @@ class PHTTPDirRequest : public PHTTPFileRequest
 };
 
 
-PFACTORY_LOAD(PURL_HttpLoader);
-
-
 #endif // P_HTTP
 
 #endif // PTLIB_HTTP_H
