@@ -23,14 +23,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22915 $
+ * $Revision: 28207 $
  * $Author: rjongbloed $
- * $Date: 2009-06-17 23:13:31 -0500 (Wed, 17 Jun 2009) $
+ * $Date: 2012-08-16 19:09:54 -0500 (Thu, 16 Aug 2012) $
  */
 
 #ifndef PTLIB_HTTPSVC_H
 #define PTLIB_HTTPSVC_H
 
+#if P_HTTPFORMS
+
 #include <ptlib/svcproc.h>
 #include <ptlib/sockets.h>
 #include <ptclib/httpform.h>
@@ -113,12 +115,18 @@ class PHTTPServiceProcess : public PServiceProcess
     virtual void OnConfigChanged() = 0;
     virtual PBoolean Initialise(const char * initMsg) = 0;
 
-    PBoolean ListenForHTTP(
+    bool ListenForHTTP(
       WORD port,
       PSocket::Reusability reuse = PSocket::CanReuseAddress,
       PINDEX stackSize = 0x4000
     );
-    PBoolean ListenForHTTP(
+    bool ListenForHTTP(
+      const PString & interfaces,
+      WORD port,
+      PSocket::Reusability reuse = PSocket::CanReuseAddress,
+      PINDEX stackSize = 0x4000
+    );
+    bool ListenForHTTP(
       PSocket * listener,
       PSocket::Reusability reuse = PSocket::CanReuseAddress,
       PINDEX stackSize = 0x4000
@@ -138,8 +146,8 @@ class PHTTPServiceProcess : public PServiceProcess
     const PTEACypher::Key & GetProductKey() const { return productKey; }
     const PStringArray & GetSecuredKeys() const { return securedKeys; }
     const PTEACypher::Key & GetSignatureKey() const { return signatureKey; }
-    PBoolean ShouldIgnoreSignatures() const { return ignoreSignatures; }
-    void SetIgnoreSignatures(PBoolean ig) { ignoreSignatures = ig; }
+    bool ShouldIgnoreSignatures() const { return ignoreSignatures; }
+    void SetIgnoreSignatures(bool ig) { ignoreSignatures = ig; }
 
     static PHTTPServiceProcess & Current();
 
@@ -152,14 +160,14 @@ class PHTTPServiceProcess : public PServiceProcess
     PBoolean ProcessHTTP(PTCPSocket & socket);
 
   protected:
-    PSocket  * httpListeningSocket;
+    PSocketList m_httpListeningSockets;
     PHTTPSpace httpNameSpace;
     PString    macroKeyword;
 
     PTEACypher::Key productKey;
     PStringArray    securedKeys;
     PTEACypher::Key signatureKey;
-    PBoolean            ignoreSignatures;
+    bool            ignoreSignatures;
 
     PTime      compilationDate;
     PString    manufacturersHomePage;
@@ -188,6 +196,7 @@ class PHTTPServiceProcess : public PServiceProcess
 
 /////////////////////////////////////////////////////////////////////
 
+
 class PConfigPage : public PHTTPConfig
 {
   PCLASSINFO(PConfigPage, PHTTPConfig)
@@ -199,7 +208,7 @@ class PConfigPage : public PHTTPConfig
     );
     PConfigPage(
       PHTTPServiceProcess & app,
-      const PString & title,
+      const PString & url,
       const PString & section,
       const PHTTPAuthority & auth
     );
@@ -373,7 +382,7 @@ class PServiceMacro : public PObject
 #define PCREATE_SERVICE_MACRO(name, request, args) \
   class PServiceMacro_##name : public PServiceMacro { \
     public: \
-      PServiceMacro_##name() : PServiceMacro(#name, PFalse) { } \
+      PServiceMacro_##name() : PServiceMacro(#name, false) { } \
       PString Translate(PHTTPRequest &, const PString &, const PString &) const; \
   }; \
   static const PServiceMacro_##name serviceMacro_##name; \
@@ -384,7 +393,7 @@ class PServiceMacro : public PObject
 #define PCREATE_SERVICE_MACRO_BLOCK(name, request, args, block) \
   class PServiceMacro_##name : public PServiceMacro { \
     public: \
-      PServiceMacro_##name() : PServiceMacro(#name, PTrue) { } \
+      PServiceMacro_##name() : PServiceMacro(#name, true) { } \
       PString Translate(PHTTPRequest &, const PString &, const PString &) const; \
   }; \
   static const PServiceMacro_##name serviceMacro_##name; \
@@ -426,15 +435,15 @@ class PServiceHTTPFile : public PHTTPFile
 {
   PCLASSINFO(PServiceHTTPFile, PHTTPFile)
   public:
-    PServiceHTTPFile(const PString & filename, PBoolean needSig = PFalse)
+    PServiceHTTPFile(const PString & filename, PBoolean needSig = false)
       : PHTTPFile(filename) { needSignature = needSig; }
-    PServiceHTTPFile(const PString & filename, const PFilePath & file, PBoolean needSig = PFalse)
+    PServiceHTTPFile(const PString & filename, const PFilePath & file, PBoolean needSig = false)
       : PHTTPFile(filename, file) { needSignature = needSig; }
-    PServiceHTTPFile(const PString & filename, const PString & file, PBoolean needSig = PFalse)
+    PServiceHTTPFile(const PString & filename, const PString & file, PBoolean needSig = false)
       : PHTTPFile(filename, file) { needSignature = needSig; }
-    PServiceHTTPFile(const PString & filename, const PHTTPAuthority & auth, PBoolean needSig = PFalse)
+    PServiceHTTPFile(const PString & filename, const PHTTPAuthority & auth, PBoolean needSig = false)
       : PHTTPFile(filename, auth) { needSignature = needSig; }
-    PServiceHTTPFile(const PString & filename, const PFilePath & file, const PHTTPAuthority & auth, PBoolean needSig = PFalse)
+    PServiceHTTPFile(const PString & filename, const PFilePath & file, const PHTTPAuthority & auth, PBoolean needSig = false)
       : PHTTPFile(filename, file, auth) { needSignature = needSig; }
 
     void OnLoadedText(PHTTPRequest &, PString & text);
@@ -451,10 +460,10 @@ class PServiceHTTPDirectory : public PHTTPDirectory
 {
   PCLASSINFO(PServiceHTTPDirectory, PHTTPDirectory)
   public:
-    PServiceHTTPDirectory(const PURL & url, const PDirectory & dirname, PBoolean needSig = PFalse)
+    PServiceHTTPDirectory(const PURL & url, const PDirectory & dirname, PBoolean needSig = false)
       : PHTTPDirectory(url, dirname) { needSignature = needSig; }
 
-    PServiceHTTPDirectory(const PURL & url, const PDirectory & dirname, const PHTTPAuthority & auth, PBoolean needSig = PFalse)
+    PServiceHTTPDirectory(const PURL & url, const PDirectory & dirname, const PHTTPAuthority & auth, PBoolean needSig = false)
       : PHTTPDirectory(url, dirname, auth) { needSignature = needSig; }
 
     void OnLoadedText(PHTTPRequest &, PString & text);
@@ -468,6 +477,8 @@ class PServiceHTTPDirectory : public PHTTPDirectory
 };
 
 
+#endif // P_HTTPFORMS
+
 #endif // PTLIB_HTTPSVC_H
 
 
