@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25387 $
+ * $Revision: 26840 $
  * $Author: rjongbloed $
- * $Date: 2011-03-22 22:51:09 -0500 (Tue, 22 Mar 2011) $
+ * $Date: 2012-01-08 01:22:58 -0600 (Sun, 08 Jan 2012) $
  */
 
 #ifndef PTLIB_PMIME_H
@@ -35,11 +35,11 @@
 #pragma interface
 #endif
 
-#include <ptclib/inetprot.h>
 #include <ptclib/cypher.h>
 
 
 class PMultiPartList;
+class PInternetProtocol;
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -61,6 +61,12 @@ class PMIMEInfo : public PStringOptions
     PMIMEInfo(
       PInternetProtocol & socket   ///< Application socket to read MIME info.
     );
+    PMIMEInfo(
+      const PStringToString & dict
+    );
+    PMIMEInfo(
+      const PString & str
+    );
 
 
   // Overrides from class PObject
@@ -98,6 +104,10 @@ class PMIMEInfo : public PStringOptions
       PInternetProtocol & socket   ///< Application socket to write MIME info.
     ) const;
 
+    /**Return a string presentation of the MIME.
+      */
+    PString AsString() const;
+
     /**Add a MIME field given a "name: value" format string.
        Note that if the field name was already in the MIME dictionary then
        this will append the new value after a '\\n' character to the previous
@@ -112,11 +122,16 @@ class PMIMEInfo : public PStringOptions
     bool AddMIME(
       const PString & fieldName, ///< MIME field name
       const PString & fieldValue ///< MIME field value
-    );
+    ) { return InternalAddMIME(fieldName, fieldValue); }
     bool AddMIME(
       const PMIMEInfo & mime
     );
 
+    virtual bool InternalAddMIME(
+      const PString & fieldName, ///< MIME field name
+      const PString & fieldValue ///< MIME field value
+    );
+
     /** Get a complex MIME field.
         This will parse a complex MIME field of the general form:
 
@@ -192,6 +207,8 @@ class PMIMEInfo : public PStringOptions
     static const PCaselessString & ContentDescriptionTag();
     static const PCaselessString & ContentIdTag();
 
+    static const PCaselessString & TextPlain();
+
 
     /** Set an association between a file type and a MIME content type. The
        content type is then sent for any file in the directory sub-tree that
