@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 27039 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2012-02-22 01:19:46 -0600 (Wed, 22 Feb 2012) $
  */
 
 
@@ -48,7 +48,6 @@
 
 /**
  SOAP Message classes
- ####################
  */
 
 //! SOAP message according to http://www.w3.org/TR/SOAP/
@@ -58,13 +57,13 @@ class PSOAPMessage : public PXML
 public:
   
   //! Construct a SOAP message 
-  PSOAPMessage( int options = PXMLParser::Indent + PXMLParser::NewLineAfterElement );
+  PSOAPMessage(Options options = Indent + NewLineAfterElement);
 
   //! Construct a SOAP message with method name and namespace already provided
   PSOAPMessage( const PString & method, const PString & nameSpace );
 
   //! Set the method name and namespace
-  void SetMethod( const PString & name, const PString & nameSpace );
+  void SetMethod( const PString & name, const PString & nameSpace, const PString & methodPrefix = "m:" );
 
   //! Get the method name and namespace
   void GetMethod( PString & name, PString & nameSpace );
@@ -73,7 +72,7 @@ public:
   void AddParameter( PString name, PString type, PString value );
 
   //! Add a parameter using a PXMLElement
-  void AddParameter( PXMLElement* parameter, PBoolean dirty = PTrue );
+  void AddParameter( PXMLElement* parameter, PBoolean dirty = true );
 
   //! Get parameter "name" with type "string"
   PBoolean GetParameter( const PString & name, PString & value );
@@ -84,12 +83,6 @@ public:
   //! Get parameter "name"
   PXMLElement* GetParameter( const PString & name );
 
-  //! Print the contents of this SOAP message on an ostream
-  void PrintOn(ostream & strm) const;
-
-  //! Output the SOAP message to a string
-  PString AsString( void );
-  
   //! Parse a string for a valid SOAP message
   PBoolean Load(const PString & str);
 
@@ -122,7 +115,6 @@ private:
 
 /**
  SOAP Server classes
- ####################
  */
 
 class PSOAPServerRequestResponse : public PObject 
@@ -195,7 +187,6 @@ class PSOAPServerResource : public PHTTPResource
 
 /**
  SOAP client classes
- ####################
  */
 
 class PSOAPClient : public PObject
