@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23931 $
+ * $Revision: 28819 $
  * $Author: rjongbloed $
- * $Date: 2010-01-13 06:19:08 -0600 (Wed, 13 Jan 2010) $
+ * $Date: 2013-01-09 00:17:37 -0600 (Wed, 09 Jan 2013) $
  */
 
 #ifndef PTLIB_PSOCKBUN_H
@@ -40,17 +40,24 @@
 #include <ptlib/ipsock.h>
 #include <ptlib/sockets.h>
 #include <ptlib/safecoll.h>
-#include <list>
-
-
-class PNatMethod;
-class PInterfaceMonitorClient;
-class PInterfaceFilter;
+#include <ptclib/pnat.h>
+#include <map>
 
 
 #define PINTERFACE_MONITOR_FACTORY_NAME "InterfaceMonitor"
 
 
+//////////////////////////////////////////////////
+
+class PInterfaceFilter : public PObject {
+  PCLASSINFO(PInterfaceFilter, PObject);
+  
+  public:
+    virtual PIPSocket::InterfaceTable FilterInterfaces(const PIPSocket::Address & destination,
+                                                       PIPSocket::InterfaceTable & interfaces) const = 0;
+};
+
+
 //////////////////////////////////////////////////
 
 /** This class is a singleton that will monitor the network interfaces on a
@@ -63,7 +70,7 @@ class PInterfaceFilter;
   */
 class PInterfaceMonitor : public PProcessStartup
 {
-  PCLASSINFO(PInterfaceMonitor, PProcessStartup);
+    PCLASSINFO(PInterfaceMonitor, PProcessStartup);
   public: 
     enum {
       DefaultRefreshInterval = 60000
@@ -76,7 +83,7 @@ class PInterfaceMonitor : public PProcessStartup
     virtual ~PInterfaceMonitor();
 
     /// Return the singleton interface for the network monitor
-    static PInterfaceMonitor & GetInstance();
+    PFACTORY_GET_SINGLETON(PProcessStartupFactory, PInterfaceMonitor);
     
     /// Change the refresh interval
     void SetRefreshInterval (unsigned refresh);
@@ -101,18 +108,19 @@ class PInterfaceMonitor : public PProcessStartup
         ip%name, eg "10.0.1.11%3Com 3C90x Ethernet Adapter" or "192.168.0.10%eth0"
       */
     PStringArray GetInterfaces(
-      bool includeLoopBack = false,  /// Flag for if loopback is to included in list
+      bool includeLoopBack = false,  ///< Flag for if loopback is to included in list
       const PIPSocket::Address & destination = PIPSocket::GetDefaultIpAny()
+                          ///< Optional destination for selecting specific interface
     );
 
     /** Returns whether destination is reachable through binding or not.
         The default behaviour returns true unless there is an interface
-        filter installed an the filter does not return 'binding' among
+        filter installed an the filter does not return <code>binding</code> among
         it's interfaces.
       */
     bool IsValidBindingForDestination(
-      const PIPSocket::Address & binding,
-      const PIPSocket::Address & destination
+      const PIPSocket::Address & binding,     ///< Interface binding to test
+      const PIPSocket::Address & destination  ///< Destination to test against the <code>binding</code>
     );
 
     /** Return information about an active interface given the descriptor
@@ -120,8 +128,8 @@ class PInterfaceMonitor : public PProcessStartup
        e.g. "10.0.1.11" or "%eth0" may be used.
       */
     bool GetInterfaceInfo(
-      const PString & iface,  /// Interface desciptor name
-      InterfaceEntry & info   /// Information on the interface
+      const PString & iface,  ///< Interface desciptor name
+      InterfaceEntry & info   ///< Information on the interface
     ) const;
     
     /** Returns whether the descriptor string equals the interface entry.
@@ -129,35 +137,76 @@ class PInterfaceMonitor : public PProcessStartup
         e.g. "10.0.1.11" or "%eth0" may be used.
       */
     static bool IsMatchingInterface(
-      const PString & iface,        /// Interface descriptor
-      const InterfaceEntry & entry  /// Interface entry
+      const PString & iface,        ///< Interface descriptor
+      const InterfaceEntry & entry  ///< Interface entry
     );
-    
+
+    /**Information on the interface change.
+      */
+    struct InterfaceChange : public InterfaceEntry
+    {
+      InterfaceChange(const InterfaceEntry & entry, bool added)
+        : InterfaceEntry(entry)
+        , m_added(added)
+#if P_NAT
+        , m_natMethod(NULL)
+#endif
+        { }
+
+      const bool m_added;
+
+#if P_NAT
+      InterfaceChange(const PNatMethod * natMethod, bool added)
+        : m_added(added), m_natMethod(natMethod) { }
+
+      const PNatMethod * const m_natMethod;
+#endif
+    };
+
+    /// Type for disposition notifiers
+    typedef PNotifierTemplate<InterfaceChange> Notifier;
+
+    /// Macro to declare correctly typed interface notifier
+    #define PDECLARE_InterfaceNotifier(cls, fn) PDECLARE_NOTIFIER2(PInterfaceMonitor, cls, fn, PInterfaceMonitor::InterfaceChange)
+
+    /// Macro to create correctly typed interface notifier
+    #define PCREATE_InterfaceNotifier(fn) PCREATE_NOTIFIER2(fn, PInterfaceMonitor::InterfaceChange)
+
+    enum {
+      DefaultPriority = 50,
+    };
+
+    /**Add a notifier for interface changes.
+      */
+    void AddNotifier(
+      const Notifier & notifier,   ///< Notifier to be called by interface monitor
+      unsigned priority = DefaultPriority   ///< Priority for notification
+    );
+    void RemoveNotifier(
+      const Notifier & notifier    ///< Notifier to be called by interface monitor
+    );
+
     /** Sets the monitor's interface filter. Note that the monitor instance
         handles deletion of the filter.
       */
     void SetInterfaceFilter(PInterfaceFilter * filter);
     bool HasInterfaceFilter() const { return m_interfaceFilter != NULL; }
-    
-    virtual void RefreshInterfaceList();
-    
+
+#if P_NAT
     void OnRemoveNatMethod(const PNatMethod * natMethod);
+#endif
 
   protected:
     virtual void OnShutdown();
 
     void UpdateThreadMain();
 
-    void AddClient(PInterfaceMonitorClient *);
-    void RemoveClient(PInterfaceMonitorClient *);
-    
+    virtual void RefreshInterfaceList();
     virtual void OnInterfacesChanged(const PIPSocket::InterfaceTable & addedInterfaces, const PIPSocket::InterfaceTable & removedInterfaces);
 
-    typedef PSmartPtr<PInterfaceMonitorClient> ClientPtr;
-
-    typedef std::list<PInterfaceMonitorClient *> ClientList_T;
-    ClientList_T m_clients;
-    PMutex       m_clientsMutex;
+    typedef std::multimap<unsigned, Notifier> Notifiers;
+    Notifiers m_notifiers;
+    PMutex    m_notifiersMutex;
 
     PIPSocket::InterfaceTable m_interfaces;
     PMutex                    m_interfacesMutex;
@@ -174,78 +223,6 @@ class PInterfaceMonitor : public PProcessStartup
 };
 
 
-//////////////////////////////////////////////////
-
-/** This is a base class for clients of the PInterfaceMonitor singleton object.
-    The OnAddInterface() and OnRemoveInterface() functions are called in the
-    context of a thread that is monitoring interfaces. The client object is
-    locked for Read/Write before these functions are called.
-  */
-class PInterfaceMonitorClient : public PSafeObject
-{
-  PCLASSINFO(PInterfaceMonitorClient, PSafeObject);
-  public:
-    enum {
-      DefaultPriority = 50,
-    };
-    PInterfaceMonitorClient(PINDEX priority = DefaultPriority);
-    ~PInterfaceMonitorClient();
-
-    typedef PIPSocket::InterfaceEntry InterfaceEntry;
-
-    /** Get an array of all current interface descriptors, possibly including
-        the loopback (127.0.0.1) interface. Note the names are of the form
-        ip%name, eg "10.0.1.11%3Com 3C90x Ethernet Adapter" or "192.168.0.10%eth0".
-        If destination is not 'any' and a filter is set, filters the interface list
-        before returning it.
-      */
-    virtual PStringArray GetInterfaces(
-      bool includeLoopBack = false,  /// Flag for if loopback is to included in list
-      const PIPSocket::Address & destination = PIPSocket::GetDefaultIpAny() /// destination
-    );
-
-    /** Return information about an active interface given the descriptor
-       string. Note that when searchin the descriptor may be a partial match
-       e.g. "10.0.1.11" or "%eth0" may be used.
-      */
-    virtual PBoolean GetInterfaceInfo(
-      const PString & iface,  /// Interface desciptor name
-      InterfaceEntry & info   /// Information on the interface
-    ) const;
-    
-    /**Returns the priority of this client. A higher value means higher priority.
-       Higher priority clients get their callback functions called first. Clients
-       with the same priority get called in the order of their insertion.
-      */
-    PINDEX GetPriority() const { return priority; }
-
-  protected:
-    /// Call back function for when an interface has been added to the system
-    virtual void OnAddInterface(const InterfaceEntry & entry) = 0;
-
-    /// Call back function for when an interface has been removed from the system
-    virtual void OnRemoveInterface(const InterfaceEntry & entry) = 0;
-    
-    /// Called when a NAT method is about to be destroyed
-    virtual void OnRemoveNatMethod(const PNatMethod * /*natMethod*/) { }
-    
-    PINDEX priority;
-
-  friend class PInterfaceMonitor;
-};
-
-
-//////////////////////////////////////////////////
-
-class PInterfaceFilter : public PObject {
-  PCLASSINFO(PInterfaceFilter, PObject);
-  
-  public:
-    virtual PIPSocket::InterfaceTable FilterInterfaces(const PIPSocket::Address & destination,
-                                                       PIPSocket::InterfaceTable & interfaces) const = 0;
-};
-
-
 //////////////////////////////////////////////////
 
 /** This is a base class for UDP socket(s) that are monitored for interface
@@ -253,16 +230,18 @@ class PInterfaceFilter : public PObject {
     bound to an IP address and/or interface name. The second will dynamically
     open/close ports as interfaces are added and removed from the system.
   */
-class PMonitoredSockets : public PInterfaceMonitorClient
+class PMonitoredSockets : public PSafeObject
 {
-  PCLASSINFO(PMonitoredSockets, PInterfaceMonitorClient);
+    PCLASSINFO(PMonitoredSockets, PSafeObject);
   protected:
     PMonitoredSockets(
-      bool reuseAddr,
-      PNatMethod * natMethod
+      bool reuseAddr    ///< Flag for sharing socket/exclusve use
+      P_NAT_PARAM(PNatMethod * natMethod)  ///< NAT method to use to create sockets.
     );
 
   public:
+    typedef PIPSocket::InterfaceEntry InterfaceEntry;
+
     /** Open the socket(s) using the specified port. If port is zero then a
         system allocated port is used. In this case and when multiple
         interfaces are supported, all sockets use the same dynamic port value.
@@ -284,24 +263,41 @@ class PMonitoredSockets : public PInterfaceMonitorClient
 
     /// Get the local address for the given interface.
     virtual PBoolean GetAddress(
-      const PString & iface,        /// Interface to get address for
-      PIPSocket::Address & address, /// Address of interface
-      WORD & port,                  /// Port listening on
-      PBoolean usingNAT                 /// Require NAT address/port
+      const PString & iface,        ///< Interface to get address for
+      PIPSocket::Address & address, ///< Address of interface
+      WORD & port,                  ///< Port listening on
+      PBoolean usingNAT             ///< Require NAT address/port
     ) const = 0;
 
+    struct BundleParams {
+      BundleParams()
+        : m_buffer(NULL)
+        , m_length(0)
+        , m_addr(0)
+        , m_port(0)
+        , m_lastCount(0)
+        , m_errorCode(PChannel::NoError)
+        , m_errorNumber(0)
+      { }
+
+      void * m_buffer;              ///< Data to read/write
+      PINDEX m_length;              ///< Maximum length of data
+      PIPSocket::Address m_addr;    ///< Remote IP address data came from, or is written to
+      WORD m_port;                  ///< Remote port data came from, or is written to
+      PString m_iface;              ///< Interface to use for read or write, also then one data was read on
+      PINDEX m_lastCount;           ///< Actual length of data read/written
+      PTimeInterval m_timeout;      ///< Time to wait for data
+      PChannel::Errors m_errorCode; ///< Error code for read/write
+      int m_errorNumber;            ///< Error number (OS specific) for read/write
+    };
+
     /** Write to the remote address/port using the socket(s) available. If the
         iface parameter is empty, then the data is written to all socket(s).
         Otherwise the iface parameter indicates the specific interface socket
         to write the data to.
       */
-    virtual PChannel::Errors WriteToBundle(
-      const void * buffer,              /// Data to write
-      PINDEX length,                    /// Length of data
-      const PIPSocket::Address & addr,  /// Remote IP address to write to
-      WORD port,                        /// Remote port to write to
-      const PString & iface,            /// Interface to use for writing
-      PINDEX & lastWriteCount
+    virtual void WriteToBundle(
+      BundleParams & param ///< Info on data to write
     ) = 0;
 
     /** Read fram a remote address/port using the socket(s) available. If the
@@ -310,48 +306,83 @@ class PMonitoredSockets : public PInterfaceMonitorClient
         Otherwise the iface parameter indicates the specific interface socket
         to read the data from.
       */
-    virtual PChannel::Errors ReadFromBundle(
-      void * buffer,                /// Data to read
-      PINDEX length,                /// Maximum length of data
-      PIPSocket::Address & addr,    /// Remote IP address data came from
-      WORD & port,                  /// Remote port data came from
-      PString & iface,              /// Interface to use for read, also one data was read on
-      PINDEX & lastReadCount,       /// Actual length of data read
-      const PTimeInterval & timeout /// Time to wait for data
+    virtual void ReadFromBundle(
+      BundleParams & param ///< Info on data to read
     ) = 0;
 
+    /** Get an array of all current interface descriptors, possibly including
+        the loopback (127.0.0.1) interface. Note the names are of the form
+        ip%name, eg "10.0.1.11%3Com 3C90x Ethernet Adapter" or "192.168.0.10%eth0".
+        If destination is not 'any' and a filter is set, filters the interface list
+        before returning it.
+      */
+    virtual PStringArray GetInterfaces(
+      bool includeLoopBack = false,  ///< Flag for if loopback is to included in list
+      const PIPSocket::Address & destination = PIPSocket::GetDefaultIpAny()
+                          ///< Optional destination for selecting specific interface
+    );
+
+    /** Return information about an active interface given the descriptor
+       string. Note that when searchin the descriptor may be a partial match
+       e.g. "10.0.1.11" or "%eth0" may be used.
+      */
+    virtual bool GetInterfaceInfo(
+      const PString & iface,  ///< Interface desciptor name
+      InterfaceEntry & info   ///< Information on the interface
+    ) const;
+
+#if P_NAT
     /// Set the NAT method, eg STUN client pointer
     void SetNatMethod(
       PNatMethod * method
     ) { natMethod = method; }
 
-
     // Get the current NAT method, eg STUN client pointer
     PNatMethod * GetNatMethod() const { return natMethod; }
 
+#endif
+
     /** Create a new monitored socket instance based on the interface
         descriptor. This will create a multiple or single socket derived class
-        of PMonitoredSockets depending on teh iface parameter.
+        of PMonitoredSockets depending on the iface parameter.
+
+        If the iface parameter starts with a '%' or "*%" then a bundle is
+        created for all IP addresses for a specific interface.
+
+        If the iface has a non INADDR_ANY address value, with or without a
+        %iface qualifier, then only that IP address is listened to.
+
+        If the iface resolves to a specific INADDR_ANY or INADDR6_ANY, then
+        only that version will be listened to, on all interfaces or 
+
+        Finally if empty string or "*", than all IP versions and all interfaces
+        are used.
       */
     static PMonitoredSockets * Create(
-      const PString & iface,            /// Interface name to create socket for
-      bool reuseAddr = false,           /// Re-use or exclusive port number
-      PNatMethod * natMethod = NULL     /// NAT method
+      const PString & iface,              ///< Interface name to create socket for
+      bool reuseAddr = false              ///< Re-use or exclusive port number
+      P_NAT_PARAM(PNatMethod * natMethod = NULL)     ///< NAT method
     );
 
   protected:
+#if P_NAT
     virtual void OnRemoveNatMethod(
       const PNatMethod * natMethod
     );
+#endif
 
     struct SocketInfo {
       SocketInfo()
         : socket(NULL)
         , inUse(false)
       { }
+      void Read(PMonitoredSockets & bundle, BundleParams & param);
+      void Write(BundleParams & param);
+
       PUDPSocket * socket;
       bool         inUse;
     };
+    friend struct SocketInfo;
 
     bool CreateSocket(
       SocketInfo & info,
@@ -365,37 +396,17 @@ class PMonitoredSockets : public PInterfaceMonitorClient
       bool usingNAT
     ) const;
 
-    PChannel::Errors WriteToSocket(
-      const void * buf,
-      PINDEX len,
-      const PIPSocket::Address & addr,
-      WORD port,
-      const SocketInfo & info,
-      PINDEX & lastWriteCount
-    );
-    PChannel::Errors ReadFromSocket(
-      SocketInfo & info,
-      void * buf,
-      PINDEX len,
-      PIPSocket::Address & addr,
-      WORD & port,
-      PINDEX & lastReadCount,
-      const PTimeInterval & timeout
-    );
-    PChannel::Errors ReadFromSocket(
+    void ReadFromSocketList(
       PSocket::SelectList & readers,
       PUDPSocket * & socket,
-      void * buf,
-      PINDEX len,
-      PIPSocket::Address & addr,
-      WORD & port,
-      PINDEX & lastReadCount,
-      const PTimeInterval & timeout
+      BundleParams & param
     );
 
     WORD          localPort;
     bool          reuseAddress;
+#if P_NAT
     PNatMethod  * natMethod;
+#endif
 
     bool          opened;
     PUDPSocket    interfaceAddedSignal;
@@ -411,14 +422,14 @@ typedef PSafePtr<PMonitoredSockets> PMonitoredSocketsPtr;
   */
 class PMonitoredSocketChannel : public PChannel
 {
-  PCLASSINFO(PMonitoredSocketChannel, PChannel);
+    PCLASSINFO(PMonitoredSocketChannel, PChannel);
   public:
   /**@name Construction */
   //@{
     /// Construct a monitored socket bundle channel
     PMonitoredSocketChannel(
-      const PMonitoredSocketsPtr & sockets,  /// Monitored socket bundle to use in channel
-      bool shared                            /// Monitored socket is shared by other channels
+      const PMonitoredSocketsPtr & sockets,  ///< Monitored socket bundle to use in channel
+      bool shared                            ///< Monitored socket is shared by other channels
     );
   //@}
 
@@ -434,9 +445,9 @@ class PMonitoredSocketChannel : public PChannel
       PINDEX length
     );
 
-    virtual PBoolean Write(
     /** Override of PChannel functions to allow connectionless writes
      */
+    virtual PBoolean Write(
       const void * buffer,
       PINDEX length
     );
@@ -448,7 +459,7 @@ class PMonitoredSocketChannel : public PChannel
         The iface parameter can be a partial descriptor eg "%eth0".
       */
     void SetInterface(
-      const PString & iface   /// Interface descriptor
+      const PString & iface   ///< Interface descriptor
     );
 
     /// Get the current interface descriptor being used/
@@ -457,26 +468,26 @@ class PMonitoredSocketChannel : public PChannel
     /** Get the local IP address and port for the currently selected interface.
       */
     bool GetLocal(
-      PIPSocket::Address & address, /// IP address of local interface
-      WORD & port,                  /// Port listening on
-      bool usingNAT                 /// Require NAT address/port
+      PIPSocket::Address & address, ///< IP address of local interface
+      WORD & port,                  ///< Port listening on
+      bool usingNAT                 ///< Require NAT address/port
     );
 
     /// Set the remote address/port for all Write() functions
     void SetRemote(
-      const PIPSocket::Address & address, /// Remote IP address
-      WORD port                           /// Remote port number
+      const PIPSocket::Address & address, ///< Remote IP address
+      WORD port                           ///< Remote port number
     );
 
     /// Set the remote address/port for all Write() functions
     void SetRemote(
-      const PString & hostAndPort /// String of the form host[:port]
+      const PString & hostAndPort ///< String of the form host[:port]
     );
 
     /// Get the current remote address/port for all Write() functions
     void GetRemote(
-      PIPSocket::Address & addr,  /// Remote IP address
-      WORD & port                 /// Remote port number
+      PIPSocket::Address & addr,  ///< Remote IP address
+      WORD & port                 ///< Remote port number
     ) const { addr = remoteAddress; port = remotePort; }
 
     /** Set flag for receiving UDP data from any remote address. If the flag
@@ -484,7 +495,7 @@ class PMonitoredSocketChannel : public PChannel
         remote address and port is ignored.
       */
     void SetPromiscuous(
-      bool flag   /// New flag
+      bool flag   ///< New flag
     ) { promiscuousReads = flag; }
 
     /// Get flag for receiving UDP data from any remote address
@@ -492,8 +503,8 @@ class PMonitoredSocketChannel : public PChannel
 
     /// Get the IP address and port of the last received UDP data.
     void GetLastReceived(
-      PIPSocket::Address & addr,  /// Remote IP address
-      WORD & port                 /// Remote port number
+      PIPSocket::Address & addr,  ///< Remote IP address
+      WORD & port                 ///< Remote port number
     ) const { addr = lastReceivedAddress; port = lastReceivedPort; }
 
     /// Get the interface the last received UDP data was recieved on.
@@ -528,11 +539,25 @@ class PMonitoredSocketBundle : public PMonitoredSockets
   PCLASSINFO(PMonitoredSocketBundle, PMonitoredSockets);
   public:
     PMonitoredSocketBundle(
-      bool reuseAddr = false,
-      PNatMethod  * natMethod = NULL
+      const PString & fixedInterface, ///< Interface name for bundle, "" is all interfaces
+      unsigned ipVersion,             ///< Version of IP to listen
+      bool reuseAddr                  ///< Flag for sharing socket/exclusve use
+      P_NAT_PARAM(PNatMethod * natMethod = NULL)  ///< NAT method to use to create sockets.
     );
     ~PMonitoredSocketBundle();
 
+    /** Get an array of all current interface descriptors, possibly including
+        the loopback (127.0.0.1) interface. Note the names are of the form
+        ip%name, eg "10.0.1.11%3Com 3C90x Ethernet Adapter" or "192.168.0.10%eth0".
+        If destination is not 'any' and a filter is set, filters the interface list
+        before returning it.
+      */
+    virtual PStringArray GetInterfaces(
+      bool includeLoopBack = false,  ///< Flag for if loopback is to included in list
+      const PIPSocket::Address & destination = PIPSocket::GetDefaultIpAny()
+                          ///< Optional destination for selecting specific interface
+    );
+
     /** Open the socket(s) using the specified port. If port is zero then a
         system allocated port is used. In this case and when multiple
         interfaces are supported, all sockets use the same dynamic port value.
@@ -548,10 +573,10 @@ class PMonitoredSocketBundle : public PMonitoredSockets
 
     /// Get the local address for the given interface.
     virtual PBoolean GetAddress(
-      const PString & iface,        /// Interface to get address for
-      PIPSocket::Address & address, /// Address of interface
-      WORD & port,                  /// Port listening on
-      PBoolean usingNAT                 /// Require NAT address/port
+      const PString & iface,        ///< Interface to get address for
+      PIPSocket::Address & address, ///< Address of interface
+      WORD & port,                  ///< Port listening on
+      PBoolean usingNAT             ///< Require NAT address/port
     ) const;
 
     /** Write to the remote address/port using the socket(s) available. If the
@@ -559,13 +584,8 @@ class PMonitoredSocketBundle : public PMonitoredSockets
         Otherwise the iface parameter indicates the specific interface socket
         to write the data to.
       */
-    virtual PChannel::Errors WriteToBundle(
-      const void * buf,
-      PINDEX len,
-      const PIPSocket::Address & addr,
-      WORD port,
-      const PString & iface,
-      PINDEX & lastWriteCount
+    virtual void WriteToBundle(
+      BundleParams & param ///< Info on data to write
     );
 
     /** Read fram a remote address/port using the socket(s) available. If the
@@ -574,29 +594,21 @@ class PMonitoredSocketBundle : public PMonitoredSockets
         Otherwise the iface parameter indicates the specific interface socket
         to read the data from.
       */
-    virtual PChannel::Errors ReadFromBundle(
-      void * buf,
-      PINDEX len,
-      PIPSocket::Address & addr,
-      WORD & port,
-      PString & iface,
-      PINDEX & lastReadCount,
-      const PTimeInterval & timeout
+    virtual void ReadFromBundle(
+      BundleParams & param ///< Info on data to read
     );
 
   protected:
-    /// Call back function for when an interface has been added to the system
-    virtual void OnAddInterface(const InterfaceEntry & entry);
-
-    /// Call back function for when an interface has been removed from the system
-    virtual void OnRemoveInterface(const InterfaceEntry & entry);
+    PDECLARE_InterfaceNotifier(PMonitoredSocketBundle, OnInterfaceChange);
 
     typedef std::map<std::string, SocketInfo> SocketInfoMap_T;
 
     void OpenSocket(const PString & iface);
     void CloseSocket(SocketInfoMap_T::iterator iterSocket);
 
-    SocketInfoMap_T socketInfoMap;
+    SocketInfoMap_T m_socketInfoMap;
+    PCaselessString m_fixedInterface;
+    unsigned        m_ipVersion;
 };
 
 
@@ -611,9 +623,9 @@ class PSingleMonitoredSocket : public PMonitoredSockets
   PCLASSINFO(PSingleMonitoredSocket, PMonitoredSockets);
   public:
     PSingleMonitoredSocket(
-      const PString & theInterface,
-      bool reuseAddr = false,
-      PNatMethod  * natMethod = NULL
+      const PString & theInterface, ///< Interface to bind to
+      bool reuseAddr    ///< Flag for sharing socket/exclusve use
+      P_NAT_PARAM(PNatMethod * natMethod = NULL)  ///< NET method to use to create sockets.
     );
     ~PSingleMonitoredSocket();
 
@@ -622,8 +634,9 @@ class PSingleMonitoredSocket : public PMonitoredSockets
         ip%name, eg "10.0.1.11%3Com 3C90x Ethernet Adapter" or "192.168.0.10%eth0"
       */
     virtual PStringArray GetInterfaces(
-      PBoolean includeLoopBack = false,  /// Flag for if loopback is to included in list
+      bool includeLoopBack = false,  ///< Flag for if loopback is to included in list
       const PIPSocket::Address & destination = PIPSocket::GetDefaultIpAny()
+                          ///< Optional destination for selecting specific interface
     );
 
     /** Open the socket(s) using the specified port. If port is zero then a
@@ -641,10 +654,10 @@ class PSingleMonitoredSocket : public PMonitoredSockets
 
     /// Get the local address for the given interface.
     virtual PBoolean GetAddress(
-      const PString & iface,        /// Interface to get address for
-      PIPSocket::Address & address, /// Address of interface
-      WORD & port,                  /// Port listening on
-      PBoolean usingNAT                 /// Require NAT address/port
+      const PString & iface,        ///< Interface to get address for
+      PIPSocket::Address & address, ///< Address of interface
+      WORD & port,                  ///< Port listening on
+      PBoolean usingNAT             ///< Require NAT address/port
     ) const;
 
     /** Write to the remote address/port using the socket(s) available. If the
@@ -652,13 +665,8 @@ class PSingleMonitoredSocket : public PMonitoredSockets
         Otherwise the iface parameter indicates the specific interface socket
         to write the data to.
       */
-    virtual PChannel::Errors WriteToBundle(
-      const void * buf,
-      PINDEX len,
-      const PIPSocket::Address & addr,
-      WORD port,
-      const PString & iface,
-      PINDEX & lastWriteCount
+    virtual void WriteToBundle(
+      BundleParams & param ///< Info on data to write
     );
 
     /** Read fram a remote address/port using the socket(s) available. If the
@@ -667,29 +675,19 @@ class PSingleMonitoredSocket : public PMonitoredSockets
         Otherwise the iface parameter indicates the specific interface socket
         to read the data from.
       */
-    virtual PChannel::Errors ReadFromBundle(
-      void * buf,
-      PINDEX len,
-      PIPSocket::Address & addr,
-      WORD & port,
-      PString & iface,
-      PINDEX & lastReadCount,
-      const PTimeInterval & timeout
+    virtual void ReadFromBundle(
+      BundleParams & param ///< Info on data to read
     );
 
 
   protected:
-    /// Call back function for when an interface has been added to the system
-    virtual void OnAddInterface(const InterfaceEntry & entry);
-
-    /// Call back function for when an interface has been removed from the system
-    virtual void OnRemoveInterface(const InterfaceEntry & entry);
+    PDECLARE_InterfaceNotifier(PSingleMonitoredSocket, OnInterfaceChange);
 
     bool IsInterface(const PString & iface) const;
 
-    PString        theInterface;
-    InterfaceEntry theEntry;
-    SocketInfo     theInfo;
+    PString        m_interface;
+    InterfaceEntry m_entry;
+    SocketInfo     m_info;
 };
 
 
