@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
- * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Revision: 28941 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 08:18:28 -0600 (Fri, 18 Jan 2013) $
  */
 
 #ifndef PTLIB_PSSL_H
@@ -41,8 +41,11 @@
 struct ssl_st;
 struct ssl_ctx_st;
 struct x509_st;
+struct X509_name_st;
 struct evp_pkey_st;
 struct dh_st;
+struct aes_key_st;
+struct SHAstate_st;
 
 enum PSSLFileTypes {
   PSSLFileTypePEM,
@@ -99,12 +102,19 @@ class PSSLPrivateKey : public PObject
     PSSLPrivateKey(
       const PSSLPrivateKey & privKey
     );
+    PSSLPrivateKey(
+      evp_pkey_st * privKey,
+      bool duplicate = true
+    );
 
     /**Create a copy of the private key.
       */
     PSSLPrivateKey & operator=(
       const PSSLPrivateKey & privKay
     );
+    PSSLPrivateKey & operator=(
+      evp_pkey_st * privKay
+    );
 
     /**Destroy and release storage for private key.
       */
@@ -112,7 +122,11 @@ class PSSLPrivateKey : public PObject
 
     /**Get internal OpenSSL private key structure.
       */
-    operator evp_pkey_st *() const { return key; }
+    operator evp_pkey_st *() const { return m_pkey; }
+
+    /**Set internal OpenSSL private key structure.
+      */
+    void Attach(evp_pkey_st * key);
 
     /**Create a new private key.
      */
@@ -122,6 +136,16 @@ class PSSLPrivateKey : public PObject
       void *cb_arg = NULL                       ///< Argument passed to callback
     );
 
+    /**Return true if is a valid private key.
+     */
+    bool IsValid() const { return m_pkey != NULL; }
+
+    /**Set the certificate as binary ASN1 DER encoded data.
+      */
+    bool SetData(
+      const PBYTEArray & data
+    );
+
     /**Get the certificate as binary ASN1 DER encoded data.
       */
     PBYTEArray GetData() const;
@@ -130,6 +154,12 @@ class PSSLPrivateKey : public PObject
       */
     PString AsString() const;
 
+    /**Set the certificate from ASN1 DER base64 encoded data.
+      */
+    bool Parse(
+      const PString & keyStr
+    );
+
     /**Load private key from file.
        The type of the private key can be specified explicitly, or if
        PSSLFileTypeDEFAULT it will be determined from the file extension,
@@ -147,13 +177,14 @@ class PSSLPrivateKey : public PObject
       */
     PBoolean Save(
       const PFilePath & keyFile,  ///< Private key file
-      PBoolean append = PFalse,        ///< Append to file
+      PBoolean append = false,        ///< Append to file
       PSSLFileTypes fileType = PSSLFileTypeDEFAULT  ///< Type of file to write
     );
 
 
   protected:
-    evp_pkey_st * key;
+    void FreePrivateKey();
+    evp_pkey_st * m_pkey;
 };
 
 
@@ -163,7 +194,7 @@ class PSSLPrivateKey : public PObject
   */
 class PSSLCertificate : public PObject
 {
-  PCLASSINFO(PSSLCertificate, PObject);
+    PCLASSINFO(PSSLCertificate, PObject);
   public:
     /**Create an empty certificate.
       */
@@ -203,12 +234,19 @@ class PSSLCertificate : public PObject
     PSSLCertificate(
       const PSSLCertificate & cert
     );
+    PSSLCertificate(
+      x509_st * cert,
+      bool duplicate = true
+    );
 
     /**Create a copy of the certificate.
       */
     PSSLCertificate & operator=(
       const PSSLCertificate & cert
     );
+    PSSLCertificate & operator=(
+      x509_st * cert
+    );
 
     /**Destroy and release storage for certificate.
       */
@@ -216,8 +254,17 @@ class PSSLCertificate : public PObject
 
     /**Get internal OpenSSL X509 structure.
       */
-    operator x509_st *() const { return certificate; }
+    operator x509_st *() const { return m_certificate; }
 
+    /**Set internal OpenSSL X509 structure.
+      */
+    void Attach(x509_st * cert);
+
+    /**Return true if is a valid certificate.
+     */
+    bool IsValid() const { return m_certificate != NULL; }
+
+    
     /**Create a new root certificate.
        The subject name is a string of the form "/name=value/name=value" where
        name is a short name for the field and value is a string value for the
@@ -231,6 +278,12 @@ class PSSLCertificate : public PObject
       const PSSLPrivateKey & key  ///< Key to sign certificate with
     );
 
+    /**Set the certificate as binary ASN1 DER encoded data.
+      */
+    bool SetData(
+      const PBYTEArray & data
+    );
+
     /**Get the certificate as binary ASN1 DER encoded data.
       */
     PBYTEArray GetData() const;
@@ -239,6 +292,12 @@ class PSSLCertificate : public PObject
       */
     PString AsString() const;
 
+    /**Set the certificate from ASN1 DER base64 encoded data.
+      */
+    bool Parse(
+      const PString & certStr
+    );
+
     /**Load certificate from file.
        The type of the certificate key can be specified explicitly, or if
        PSSLFileTypeDEFAULT it will be determined from the file extension,
@@ -256,13 +315,48 @@ class PSSLCertificate : public PObject
       */
     PBoolean Save(
       const PFilePath & keyFile,  ///< Certificate key file
-      PBoolean append = PFalse,        ///< Append to file
+      PBoolean append = false,        ///< Append to file
       PSSLFileTypes fileType = PSSLFileTypeDEFAULT  ///< Type of file to write
     );
 
+    class X509_Name : public PObject {
+        PCLASSINFO(X509_Name, PObject);
+      public:
+        X509_Name(X509_name_st * name = NULL) : m_name(name) { }
+
+        Comparison Compare(const PObject & other) const;
+        void PrintOn(ostream & strm) const;
+
+        bool IsValid() const { return m_name != NULL; }
+
+        PString GetCommonName() const;
+        PString GetNID(int id) const;
+        PString AsString(
+          int indent = -1  // Negative means single line
+        ) const;
+
+      protected:
+        X509_name_st * m_name;
+    };
+
+    /**Get certificate issuer name.
+      */
+    bool GetIssuerName(X509_Name & name) const;
+
+    /**Get certificate subject name.
+      */
+    bool GetSubjectName(X509_Name & name) const;
+    PString GetSubjectName() const;
+
+    /**Get certificate alternate subject name.
+      */
+    PString GetSubjectAltName() const;
+
+    virtual void PrintOn(ostream & strm) const { strm << GetSubjectName(); }
 
   protected:
-    x509_st * certificate;
+    void FreeCertificate();
+    x509_st * m_certificate;
 };
 
 
@@ -332,6 +426,52 @@ class PSSLDiffieHellman : public PObject
 };
 
 
+#ifdef P_SSL_AES
+/// AES encryption scheme
+class PAESContext : public PObject
+{
+  PCLASSINFO(PAESContext, PObject);
+  public:
+    PAESContext();
+    PAESContext(bool encrypt, const void * key, PINDEX numBits);
+    ~PAESContext();
+
+    void SetEncrypt(const void * key, PINDEX numBits);
+    void SetDecrypt(const void * key, PINDEX numBits);
+
+    void Encrypt(const void * in, void * out);
+    void Decrypt(const void * in, void * out);
+
+  protected:
+    aes_key_st * m_key;
+};
+#endif // P_SSL_AES
+
+
+/// SHA1 digest scheme
+class PSHA1Context : public PObject
+{
+  PCLASSINFO(PSHA1Context, PObject);
+  public:
+    PSHA1Context();
+    ~PSHA1Context();
+
+    enum { BlockSize = 64 };
+
+    void Update(const void * data, PINDEX length);
+    void Update(const PString & str) { Update((const char *)str, str.GetLength()); }
+
+    typedef BYTE Digest[20];
+    void Finalise(Digest result);
+
+    static void Process(const void * data, PINDEX length, Digest result);
+    static void Process(const PString & str, Digest result) { Process((const char *)str, str.GetLength(), result); }
+
+  protected:
+    SHAstate_st * m_context;
+};
+
+
 /**Context for SSL channels.
    This class embodies a common environment for all connections made via SSL
    using the PSSLChannel class. It includes such things as the version of SSL
@@ -341,7 +481,6 @@ class PSSLContext {
   public:
     enum Method {
       SSLv23,
-      SSLv2,
       SSLv3,
       TLSv1
     };
@@ -352,7 +491,7 @@ class PSSLContext {
        session ID is a completely arbitrary block of data. If sessionId is
        non NULL and idSize is zero, then sessionId is assumed to be a pointer
        to a C string.
-       The default SSL method is SSLv23
+       The default SSL method is TLSv1
       */
     PSSLContext(
       const void * sessionId = NULL,  ///< Pointer to session ID
@@ -370,47 +509,83 @@ class PSSLContext {
 
     /**Get the internal SSL context structure.
       */
-    operator ssl_ctx_st *() const { return context; }
+    operator ssl_ctx_st *() const { return m_context; }
 
-    /**Set the path to locate CA certificates.
+    /**Set the locations for CA certificates used to verify peer certificates.
       */
-    PBoolean SetCAPath(
-      const PDirectory & caPath   ///< Directory for CA certificates
+    bool SetVerifyLocations(
+      const PFilePath & caFile, ///< File for CA certificates
+      const PDirectory & caDir  ///< Directory for CA certificates
     );
 
-    /**Set the CA certificate file.
+    /**Set the CA certificate used to verify peer certificates.
       */
-    PBoolean SetCAFile(
-      const PFilePath & caFile    ///< CA certificate file
+    bool SetVerifyCertificate(
+      const PSSLCertificate & cert
+    );
+
+    P_DECLARE_ENUM(VerifyMode,
+      VerifyNone,
+      VerifyPeer,
+      VerifyPeerMandatory
+    );
+
+    /**Set certificate verification mode for connection.
+      */
+    void SetVerifyMode(
+      VerifyMode mode,    ///< New verification mode
+      unsigned depth = 9  ///< Verification depth (max number of certs in chain)
+    );
+
+    /**Set certificate verification mode for connection.
+      */
+    VerifyMode GetVerifyMode() const;
+
+    /**Set the CA certificate(s) to send to client from server.
+      */
+    bool AddClientCA(
+      const PSSLCertificate & certificate
+    );
+    bool AddClientCA(
+      const PList<PSSLCertificate> & certificates
     );
 
     /**Use the certificate specified.
       */
-    PBoolean UseCertificate(
+    bool UseCertificate(
       const PSSLCertificate & certificate
     );
 
     /**Use the private key specified.
       */
-    PBoolean UsePrivateKey(
+    bool UsePrivateKey(
       const PSSLPrivateKey & key
     );
 
     /**Use the Diffie-Hellman parameters specified.
       */
-    PBoolean UseDiffieHellman(
+    bool UseDiffieHellman(
       const PSSLDiffieHellman & dh
     );
 
     /**Set the available ciphers to those listed.
       */
-    PBoolean SetCipherList(
+    bool SetCipherList(
       const PString & ciphers   ///< List of cipher names.
     );
 
+    /**Set the credentials for the context.
+      */
+    bool SetCredentials(
+      const PString & authority,    ///< Certificate Authority directory, file or data
+      const PString & certificate,  ///< Local certificate file or data
+      const PString & privateKey,   ///< Private key file or data for local certificate
+      bool create = false           ///< If certificate/provateKey are file paths and do not exist, then create.
+    );
+
   protected:
     void Construct(Method method, const void * sessionId, PINDEX idSize);
-    ssl_ctx_st * context;
+    ssl_ctx_st * m_context;
 };
 
 
@@ -425,7 +600,7 @@ class PSSLChannel : public PIndirectChannel
       */
     PSSLChannel(
       PSSLContext * context = NULL,   ///< Context for SSL channel
-      PBoolean autoDeleteContext = PFalse  ///< Flag for context to be automatically deleted.
+      PBoolean autoDeleteContext = false  ///< Flag for context to be automatically deleted.
     );
     PSSLChannel(
       PSSLContext & context           ///< Context for SSL channel
@@ -439,9 +614,9 @@ class PSSLChannel : public PIndirectChannel
     virtual PBoolean Read(void * buf, PINDEX len);
     virtual PBoolean Write(const void * buf, PINDEX len);
     virtual PBoolean Close();
-    virtual PBoolean Shutdown(ShutdownValue) { return PTrue; }
+    virtual PBoolean Shutdown(ShutdownValue) { return true; }
     virtual PString GetErrorText(ErrorGroup group = NumErrorGroups) const;
-    virtual PBoolean ConvertOSError(int error, ErrorGroup group = LastGeneralError);
+    virtual PBoolean ConvertOSError(P_INT_PTR error, ErrorGroup group = LastGeneralError);
 
     // New functions
     /**Accept a new inbound connection (server).
@@ -460,7 +635,7 @@ class PSSLChannel : public PIndirectChannel
       */
     PBoolean Accept(
       PChannel * channel,     ///< Channel to attach to.
-      PBoolean autoDelete = PTrue  ///< Flag for if channel should be automatically deleted.
+      PBoolean autoDelete = true  ///< Flag for if channel should be automatically deleted.
     );
 
 
@@ -480,7 +655,16 @@ class PSSLChannel : public PIndirectChannel
       */
     PBoolean Connect(
       PChannel * channel,     ///< Channel to attach to.
-      PBoolean autoDelete = PTrue  ///< Flag for if channel should be automatically deleted.
+      PBoolean autoDelete = true  ///< Flag for if channel should be automatically deleted.
+    );
+
+    /**Set the CA certificate(s) to send to client from server.
+      */
+    bool AddClientCA(
+      const PSSLCertificate & certificate
+    );
+    bool AddClientCA(
+      const PList<PSSLCertificate> & certificates
     );
 
     /**Use the certificate specified.
@@ -495,21 +679,36 @@ class PSSLChannel : public PIndirectChannel
       const PSSLPrivateKey & key
     );
 
-    enum VerifyMode {
-      VerifyNone,
-      VerifyPeer,
-      VerifyPeerMandatory,
-    };
+    /**Get the available ciphers.
+      */
+    PString GetCipherList() const;
 
+    typedef PSSLContext::VerifyMode VerifyMode;
+
+    /**Set certificate verification mode for connection.
+      */
     void SetVerifyMode(
       VerifyMode mode
     );
 
-    PSSLContext * GetContext() const { return context; }
+    /**Get the peer certificate, if there is one.
+       If SetVerifyMode() has been called with VerifyPeer then this will
+       return true if the remote does not offer a certiciate. If set to
+       VerifyPeerMandatory, then it will return false. In both cases it will
+       return false if the certificate is offered but cannot be authenticated.
+      */
+    bool GetPeerCertificate(
+      PSSLCertificate & certificate,
+      PString * error = NULL
+    );
+
+    PSSLContext * GetContext() const { return m_context; }
 
     virtual PBoolean RawSSLRead(void * buf, PINDEX & len);
 
   protected:
+    void Construct(PSSLContext * ctx, PBoolean autoDel);
+
     /**This callback is executed when the Open() function is called with
        open channels. It may be used by descendent channels to do any
        handshaking required by the protocol that channel embodies.
@@ -517,14 +716,14 @@ class PSSLChannel : public PIndirectChannel
        The default behaviour "connects" the channel to the OpenSSL library.
 
        @return
-       Returns PTrue if the protocol handshaking is successful.
+       Returns true if the protocol handshaking is successful.
      */
     virtual PBoolean OnOpen();
 
   protected:
-    PSSLContext * context;
-    PBoolean          autoDeleteContext;
-    ssl_st      * ssl;
+    PSSLContext * m_context;
+    bool          m_autoDeleteContext;
+    ssl_st      * m_ssl;
 };
 
 #endif // PTLIB_PSSL_H
