@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 27469 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2012-04-18 14:33:35 -0500 (Wed, 18 Apr 2012) $
  */
 
 #ifndef PTLIB_PWAVFILEDEV_H
@@ -94,12 +94,15 @@ class PSoundChannel_WAVFile : public PSoundChannel
 
 protected:
     bool ReadSamples(void * data, PINDEX size);
+    bool ReadSample(short & data);
 
     PWAVFile       m_WAVFile;
     PAdaptiveDelay m_Pacing;
     bool           m_autoRepeat;
     unsigned       m_sampleRate;
     PINDEX         m_bufferSize;
+    PShortArray    m_sampleBuffer;
+    PINDEX         m_samplePosition;
 };
 
 
