@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 28521 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2012-11-15 18:29:10 -0600 (Thu, 15 Nov 2012) $
  */
 
 #ifndef PTLIB_PXMLRPC_H
@@ -80,7 +80,7 @@ class PXMLRPC : public PObject
 
     PXMLRPC(
       const PURL & url,
-      unsigned options = 0
+      PXML::Options options = PXML::NoOptions
     );
 
     void SetTimeout(const PTimeInterval & t) { timeout = t; }
@@ -103,7 +103,7 @@ class PXMLRPC : public PObject
     PINDEX        faultCode;
     PString       faultText;
     PTimeInterval timeout;
-    unsigned      options;
+    PXML::Options m_options;
 };
 
 /////////////////////////////////////////////////////////////////
@@ -221,7 +221,7 @@ class PXMLRPCVariableBase : public PObject {
     const char * type;
 
   private:
-    PXMLRPCVariableBase(const PXMLRPCVariableBase &) { }
+    PXMLRPCVariableBase(const PXMLRPCVariableBase &) : PObject() { }
 };
 
 
@@ -267,7 +267,7 @@ class PXMLRPCStructBase : public PObject {
     PXMLRPCStructBase();
     PXMLRPCStructBase & operator=(const PXMLRPCStructBase &);
   private:
-    PXMLRPCStructBase(const PXMLRPCStructBase &) { }
+    PXMLRPCStructBase(const PXMLRPCStructBase &) : PObject() { }
 
   public:
     void PrintOn(ostream & strm) const;
@@ -277,7 +277,7 @@ class PXMLRPCStructBase : public PObject {
     PXMLRPCVariableBase * GetVariable(const char * name) const { return variablesByName.GetAt(name); }
 
     void AddVariable(PXMLRPCVariableBase * var);
-    static PXMLRPCStructBase & GetInitialiser() { return *PAssertNULL(initialiserInstance); }
+    static PXMLRPCStructBase & GetInitialiser();
 
   protected:
     void EndConstructor();
