@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24177 $
+ * $Revision: 29306 $
  * $Author: rjongbloed $
- * $Date: 2010-04-05 06:52:04 -0500 (Mon, 05 Apr 2010) $
+ * $Date: 2013-03-24 23:14:19 -0500 (Sun, 24 Mar 2013) $
  */
 
 #ifndef PTLIB_SHTTPSVC_H
@@ -52,9 +52,9 @@ class PSecureHTTPServiceProcess : public PHTTPServiceProcess
 
     virtual PHTTPServer * CreateHTTPServer(PTCPSocket & socket);
 
-    PBoolean SetServerCertificate(
+    bool SetServerCertificate(
       const PFilePath & certFile,
-      PBoolean create = false,
+      bool create = false,
       const char * dn = NULL
     );
 
@@ -63,9 +63,10 @@ class PSecureHTTPServiceProcess : public PHTTPServiceProcess
     virtual PString CreateNonSSLMessage(const PString & url);
     virtual PString CreateRedirectMessage(const PString & url);
 
+    void DisableSSL();
+
   protected:
-    PSSLContext * sslContext;
-    PBoolean          disableSSL;
+    PSSLContext * m_sslContext;
 };
 
 
