@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24410 $
+ * $Revision: 29661 $
  * $Author: rjongbloed $
- * $Date: 2010-05-27 18:43:58 -0500 (Thu, 27 May 2010) $
+ * $Date: 2013-05-08 00:02:01 -0500 (Wed, 08 May 2013) $
  */
 
 #ifndef PTLIB_PURL_H
@@ -38,16 +38,24 @@
 
 #if P_URL
 
+#include <ptlib/pfactory.h>
+
+
+class PURLScheme;
+class PURLLegacyScheme;
+
 
 //////////////////////////////////////////////////////////////////////////////
 // PURL
 
-class PURLLegacyScheme;
-
 /**
  This class describes a Universal Resource Locator.
  This is the desciption of a resource location as used by the World Wide
- Web and the #PHTTPSocket# class.
+ Web and the <code>PHTTPSocket</code> class.
+
+ Note, this has been extended to be for URI's (Universal Resource  Identifier)
+ which is a superset of URL's. But for backward compatinility reasons the name
+ is still PURL.
  */
 class PURL : public PObject
 {
@@ -78,17 +86,17 @@ class PURL : public PObject
     /**Compare the two URLs and return their relative rank.
 
      @return
-       #LessThan#, #EqualTo# or #GreaterThan#
+       <code>LessThan</code>, <code>EqualTo</code> or <code>GreaterThan</code>
        according to the relative rank of the objects.
      */
     virtual Comparison Compare(
       const PObject & obj   ///< Object to compare against.
     ) const;
 
-    /**This function yields a hash value required by the #PDictionary#
+    /**This function yields a hash value required by the <code>PDictionary</code>
        class. A descendent class that is required to be the key of a dictionary
        should override this function. The precise values returned is dependent
-       on the semantics of the class. For example, the #PString# class
+       on the semantics of the class. For example, the <code>PString</code> class
        overrides it to provide a hash function for distinguishing text strings.
 
        The default behaviour is to return the value zero.
@@ -127,14 +135,14 @@ class PURL : public PObject
 
     /**Print/String output representation formats. */
     enum UrlFormat {
-      /// Translate to a string as a full URL
-      FullURL,      
-      /// Translate to a string as only path
-      PathOnly,     
-      /// Translate to a string with no scheme or host
-      URIOnly,      
-      /// Translate to a string with scheme and host/port
-      HostPortOnly  
+      FullURL,       ///< Output full URI
+      PathOnly,      ///< Translate to a string as only path
+      LocationOnly,  ///< Translate to a string with the location (scheme and user/pass/host/port)
+      RelativeOnly,  ///< Translate to a string with no scheme or host, just the relative part
+
+      // for backward compatibility
+      HostPortOnly = LocationOnly,
+      URIOnly = RelativeOnly
     };
 
     /**Convert the URL object into its string representation. The parameter
@@ -159,8 +167,12 @@ class PURL : public PObject
       LoginTranslation,
       /// Translate the path field for a URL.
       PathTranslation,
-      /// Translate the query parameters field for a URL.
-      QueryTranslation
+      /// Translate the query variable field for a URL.
+      QueryTranslation,
+      /// Translate the parameter variables field for a URL.
+      ParameterTranslation,
+      /// Translate the quoted parameter variables field for a URL.
+      QuotedParameterTranslation
     };
 
     /**Translate a string from general form to one that can be included into
@@ -191,23 +203,34 @@ class PURL : public PObject
     static void SplitVars(
       const PString & str,    ///< String to split into variables.
       PStringToString & vars, ///< Dictionary of variable names and values.
-      char sep1,              ///< Separater between pairs
-      char sep2               ///< Separater between key and value
+      char sep1 = ';',        ///< Separater between pairs
+      char sep2 = '=',        ///< Separater between key and value
+      TranslationType type = ParameterTranslation ///< Type of translation.
     );
 
     /** Split a string in &= form to a dictionary of names and values. */
     static void SplitQueryVars(
       const PString & queryStr,   ///< String to split into variables.
       PStringToString & queryVars ///< Dictionary of variable names and values.
-    ) { SplitVars(queryStr, queryVars, '&', '='); }
+    ) { SplitVars(queryStr, queryVars, '&', '=', QueryTranslation); }
 
+    /** Construct string from a dictionary using separators.
+      */
+    static void OutputVars(
+      ostream & strm,               ///< Stream to output dictionary to
+      const PStringToString & vars, ///< Dictionary of variable names and values.
+      char sep0 = ';',              ///< First separater before all ('\0' means none)
+      char sep1 = ';',              ///< Separater between pairs
+      char sep2 = '=',              ///< Separater between key and value
+      TranslationType type = ParameterTranslation ///< Type of translation.
+    );
 
 
     /// Get the scheme field of the URL.
     const PCaselessString & GetScheme() const { return scheme; }
 
     /// Set the scheme field of the URL
-    void SetScheme(const PString & scheme);
+    bool SetScheme(const PString & scheme);
 
     /// Get the username field of the URL.
     const PString & GetUserName() const { return username; }
@@ -230,12 +253,15 @@ class PURL : public PObject
     /// Get the port field of the URL.
     WORD GetPort() const { return port; }
 
-    /// Set the port field in the URL.
+    /// Set the port field in the URL. Zero resets to default.
     void SetPort(WORD newPort);
     
     /// Get if explicit port is specified.
     PBoolean GetPortSupplied() const { return portSupplied; }
 
+    /// Get the hostname and optional port fields of the URL.
+    PString GetHostPort() const;
+
     /// Get if path is relative or absolute
     PBoolean GetRelativePath() const { return relativePath; }
 
@@ -254,6 +280,12 @@ class PURL : public PObject
     /// Append segment to the path field of the URL.
     void AppendPath(const PString & segment);
 
+    /// Change segment in the path field of the URL.
+    void ChangePath(
+      const PString & segment, ///< New value for segment, empty means remove
+      PINDEX idx = P_MAX_INDEX ///< Segment index, P_MAX_INDEX means last segment
+    );
+
     /// Get the parameter (;) field of the URL.
     PString GetParameters() const;
 
@@ -262,7 +294,7 @@ class PURL : public PObject
 
     /// Get the parameter (;) field(s) of the URL as a string dictionary.
     /// Note the values have already been translated using UntranslateString
-    const PStringToString & GetParamVars() const { return paramVars; }
+    const PStringOptions & GetParamVars() const { return paramVars; }
 
     /// Set the parameter (;) field(s) of the URL as a string dictionary.
     /// Note the values will be translated using TranslateString
@@ -276,7 +308,7 @@ class PURL : public PObject
       bool emptyDataDeletes = true  ///< If true, and data empty string, key is removed
     );
 
-    /// Get the fragment (##) field of the URL.
+    /// Get the fragment (\#) field of the URL.
     const PString & GetFragment() const { return fragment; }
 
     /// Get the Query (?) field of the URL as a string.
@@ -288,7 +320,7 @@ class PURL : public PObject
 
     /// Get the Query (?) field of the URL as a string dictionary.
     /// Note the values have already been translated using UntranslateString
-    const PStringToString & GetQueryVars() const { return queryVars; }
+    const PStringOptions & GetQueryVars() const { return queryVars; }
 
     /// Set the Query (?) field(s) of the URL as a string dictionary.
     /// Note the values will be translated using TranslateString
@@ -304,22 +336,64 @@ class PURL : public PObject
     /// Set the contents of URL, data left after all elemetns are parsed out
     void SetContents(const PString & str);
 
-    /// Return PTrue if the URL is an empty string.
+    /// Return true if the URL is an empty string.
     PBoolean IsEmpty() const { return urlString.IsEmpty(); }
 
 
+    struct LoadParams {
+      LoadParams(
+        const PString & requiredContentType = PString::Empty(),
+        const PTimeInterval & timeout = PMaxTimeInterval
+      ) : m_requiredContentType(requiredContentType)
+        , m_timeout(timeout)
+      {
+      }
+
+      PString       m_requiredContentType;
+      PTimeInterval m_timeout;
+
+      PString       m_username;     // Basic authentication
+      PString       m_password;
+#if P_SSL
+      PString       m_authority;    // Directory, file or data
+      PString       m_certificate;  // File or data
+      PString       m_privateKey;   // File or data
+#endif
+    };
+    /**Get the resource the URL is pointing at.
+       The data returned is obtained according to the scheme and the factory
+       PURLLoaderFactory.
+      */
+    bool LoadResource(
+      PString & data,  ///< Resource data as a string
+      const LoadParams & params = LoadParams()  ///< Parameters for load
+    ) const;
+    bool LoadResource(
+      PBYTEArray & data,  ///< Resource data as a binary blob
+      const LoadParams & params = LoadParams()  ///< Parameters for load
+    ) const;
+
+    // For backward compatibility
+    template <class T>
+    bool LoadResource(
+      T & data,  ///< Resource data as a string
+      const PString & requiredContentType = PString::Empty(), ///< Expected content type where applicable
+      const PTimeInterval & timeout = PMaxTimeInterval        ///< Timeout to wait for resource
+    ) const { return LoadResource(data, LoadParams(requiredContentType, timeout)); }
+
     /**Open the URL in a browser.
 
        @return
        The browser was successfully opened. This does not mean the URL exists and was
        displayed.
      */
-    static PBoolean OpenBrowser(
+    bool OpenBrowser() const { return OpenBrowser(AsString()); }
+    static bool OpenBrowser(
       const PString & url   ///< URL to open
     );
   //@}
 
-    PBoolean LegacyParse(const PString & url, const PURLLegacyScheme * schemeInfo);
+    bool LegacyParse(const char * str, const PURLLegacyScheme * schemeInfo);
     PString LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schemeInfo) const;
 
   protected:
@@ -329,20 +403,22 @@ class PURL : public PObject
       const char * defaultScheme ///< Default scheme for URL
     );
     void Recalculate();
+
+    const PURLScheme * schemeInfo;
     PString urlString;
 
     PCaselessString scheme;
-    PString username;
-    PString password;
+    PString         username;
+    PString         password;
     PCaselessString hostname;
-    WORD port;
-    PBoolean portSupplied;          /// port was supplied in string input
-    PBoolean relativePath;
-    PStringArray path;
-    PStringToString paramVars;
-    PString fragment;
-    PStringToString queryVars;
-    PString m_contents;  // Anything left after parsing other elements
+    WORD            port;
+    bool            portSupplied;          /// port was supplied in string input
+    bool            relativePath;
+    PStringArray    path;
+    PStringOptions paramVars;
+    PString         fragment;
+    PStringOptions queryVars;
+    PString         m_contents;  // Anything left after parsing other elements
 };
 
 
@@ -353,11 +429,14 @@ class PURLScheme : public PObject
 {
   PCLASSINFO(PURLScheme, PObject);
   public:
-    virtual PString GetName() const = 0;
-    virtual PBoolean Parse(const PString & url, PURL & purl) const = 0;
+    virtual bool Parse(const char * cstr, PURL & url) const = 0;
     virtual PString AsString(PURL::UrlFormat fmt, const PURL & purl) const = 0;
+    virtual WORD GetDefaultPort() const { return 0; }
 };
 
+typedef PFactory<PURLScheme> PURLSchemeFactory;
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PURLLegacyScheme
 
@@ -365,7 +444,6 @@ class PURLLegacyScheme : public PURLScheme
 {
   public:
     PURLLegacyScheme(
-      const char * s,
       bool user    = false,
       bool pass    = false,
       bool host    = false,
@@ -378,8 +456,7 @@ class PURLLegacyScheme : public PURLScheme
       bool rel     = false,
       WORD port    = 0
     )
-      : scheme(s)
-      , hasUsername           (user)
+      : hasUsername           (user)
       , hasPassword           (pass)
       , hasHostPort           (host)
       , defaultToUserIfNoAt   (def)
@@ -392,16 +469,18 @@ class PURLLegacyScheme : public PURLScheme
       , defaultPort           (port)
     { }
 
-    PBoolean Parse(const PString & url, PURL & purl) const
-    { return purl.LegacyParse(url, this); }
+    bool Parse(const char * cstr, PURL & url) const
+    {
+      return url.LegacyParse(cstr, this);
+    }
 
-    PString AsString(PURL::UrlFormat fmt, const PURL & purl) const
-    { return purl.LegacyAsString(fmt, this); }
+    PString AsString(PURL::UrlFormat fmt, const PURL & url) const
+    {
+      return url.LegacyAsString(fmt, this);
+    }
 
-    PString GetName() const     
-    { return scheme; }
+    virtual WORD GetDefaultPort() const { return defaultPort; }
 
-    PString scheme;
     bool hasUsername;
     bool hasPassword;
     bool hasHostPort;
@@ -415,6 +494,66 @@ class PURLLegacyScheme : public PURLScheme
     WORD defaultPort;
 };
 
+/** Define a scheme based on basic legacy syntax.
+    The full HTTP style URL is:
+      scheme://user:pass@host:port/path#fragment;params?query
+    the various flags indicate if the subsection is present in the scheme.
+
+  */
+#define PURL_LEGACY_SCHEME(schemeName, \
+                           hasUsername,           /* URL scheme has a ysername */ \
+                           hasPassword,           /* URL scheme has a password */ \
+                           hasHostPort,           /* URL scheme has a host:port */ \
+                           defaultToUserIfNoAt,   /* URL scheme is username if no @, otherwise host:port */ \
+                           defaultHostToLocal,    /* URL scheme defaults to PIPSocket::GetHostName() if not present */ \
+                           hasQuery,              /* URL scheme has a query secton */ \
+                           hasParameters,         /* URL scheme has a parameter section */ \
+                           hasFragments,          /* URL scheme has a fragment section */ \
+                           hasPath,               /* URL scheme has a path */ \
+                           relativeImpliesScheme, /* URL scheme has relative path (no //) then scheme: is not output */ \
+                           defaultPort)           /* URL scheme default port if not specified in host:port */ \
+  class PURLLegacyScheme_##schemeName : public PURLLegacyScheme \
+  { \
+    public: \
+      PURLLegacyScheme_##schemeName() \
+        : PURLLegacyScheme(hasUsername, \
+                           hasPassword, \
+                           hasHostPort, \
+                           defaultToUserIfNoAt, \
+                           defaultHostToLocal, \
+                           hasQuery, \
+                           hasParameters, \
+                           hasFragments, \
+                           hasPath, \
+                           relativeImpliesScheme, \
+                           defaultPort) \
+        { } \
+  }; \
+  static PURLSchemeFactory::Worker<PURLLegacyScheme_##schemeName> schemeName##Factory(#schemeName, true); \
+
+
+
+//////////////////////////////////////////////////////////////////////////////
+// PURLLoader
+
+class PURLLoader : public PObject
+{
+  PCLASSINFO(PURLLoader, PObject);
+  public:
+    virtual bool Load(PString & str, const PURL & url, const PURL::LoadParams & params) const = 0;
+    virtual bool Load(PBYTEArray & data, const PURL & url, const PURL::LoadParams & params) const = 0;
+};
+
+typedef PFactory<PURLLoader> PURLLoaderFactory;
+
+#if P_HTTP
+  PFACTORY_LOAD(PURL_HttpLoader);
+#endif // P_HTTP
+#if P_FTP
+  PFACTORY_LOAD(PURL_FtpLoader);
+#endif // P_HTTP
+
+
 #endif // P_URL
 
 #endif // PTLIB_PURL_H
