@@ -26,35 +26,28 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 27066 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2012-02-28 02:23:49 -0600 (Tue, 28 Feb 2012) $
  */
 
 #ifndef PTLIB_PTLIB_H
 #define PTLIB_PTLIB_H
 
 #ifdef __GNUC__
-
 #pragma interface
-
-#if !defined(__USE_STD__) && __GNUC__ >= 3
-#define __USE_STD__
-#endif
-
 #endif
 
 #ifdef __NUCLEUS_PLUS__
 #include "nucpp.h"
 #endif
 
-#ifdef __USE_STD__
-//using namespace std;
-#endif
-
 #include "ptbuildopts.h"
+
+#include <ptlib/object.h>
 #include <ptlib/contain.h>
 
+
 ///////////////////////////////////////////////////////////////////////////////
 // PTime
 
