@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 28621 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2012-12-02 20:33:10 -0600 (Sun, 02 Dec 2012) $
  */
 
 #ifndef PTLIB_ARGLIST_H
@@ -62,33 +62,33 @@ class PArgList : public PObject
     PArgList(
       const char * theArgPtr = NULL,        ///< A string constituting the arguments 
       const char * argumentSpecPtr = NULL,  ///< The specification C string for argument options. See description for details.
-      PBoolean optionsBeforeParams = PTrue       ///< Parse options only before parameters 
+      PBoolean optionsBeforeParams = true       ///< Parse options only before parameters 
     );
     /** Create an argument list. */
     PArgList(
       const PString & theArgStr,             ///< A string constituting the arguments 
       const char * argumentSpecPtr = NULL,   ///< The specification C string for argument options. See description for details.
-      PBoolean optionsBeforeParams = PTrue        ///< Parse options only before parameters 
+      PBoolean optionsBeforeParams = true        ///< Parse options only before parameters 
     );
     /** Create an argument list. */
     PArgList(
       const PString & theArgStr,             ///< A string constituting the arguments 
       const PString & argumentSpecStr,       ///< The specification string for argument options. See description for details.
-      PBoolean optionsBeforeParams = PTrue        ///< Parse options only before parameters 
+      PBoolean optionsBeforeParams = true        ///< Parse options only before parameters 
     );
     /** Create an argument list. */
     PArgList(
       int theArgc,                           ///< Count of argument strings in theArgv 
       char ** theArgv,                       ///< An array of strings constituting the arguments 
       const char * argumentSpecPtr = NULL,   ///< The specification C string for argument options. See description for details.
-      PBoolean optionsBeforeParams = PTrue        ///< Parse options only before parameters 
+      PBoolean optionsBeforeParams = true        ///< Parse options only before parameters 
     );
     /** Create an argument list. */
     PArgList(
       int theArgc,                           ///< Count of argument strings in theArgv 
       char ** theArgv,                       ///< An array of strings constituting the arguments 
       const PString & argumentSpecStr,       ///< The specification string for argument options. See description for details.
-      PBoolean optionsBeforeParams = PTrue        ///< Parse options only before parameters 
+      PBoolean optionsBeforeParams = true        ///< Parse options only before parameters 
     );
   //@}
 
@@ -125,12 +125,34 @@ class PArgList : public PObject
       const PStringArray & theArgs ///< A string array constituting the arguments
     );
 
+    /** Set the comand name
+      */
+    void SetCommandName(
+       const PString & name
+    ) { m_commandName = name; }
+
+    /** Get the command name
+      */
+    const PString & GetCommandName() const { return m_commandName; }
+
     /** Parse the arguments.
        Parse the standard C program arguments into an argument of options and
-       parameters. Consecutive calls with #optionsBeforeParams# set
-       to PTrue will parse out different options and parameters. If SetArgs()
-       function is called then the Parse() function will restart from the
-       beginning of the argument list.
+       parameters.
+
+       If \p optionsBeforeParams is set to true then options will be parsed up
+       until the first parameter. Consecutive calls maybe made using \p
+       theArgumentSpec of NULL and sets of options/parametes may be parsed,
+       e.g. -f -g first one -h -k second. A -- indicates the end of the first
+       parsed set if there are no parameters, e.g. -f -g -- -h -k second
+
+       If \p optionsBeforeParams is set to false then options and parameters
+       may be mixed in any order. If a -- is seen then option parsing is
+       stopped and all remaining arguments are passed as parameters, e.g. -a
+       -b -- "-param starting with minus sign".
+
+       If SetArgs() function is called then the Parse() function will restart
+       from the beginning of the argument list. SetArgs(0, NULL) may be used
+       to reset parsing without altering the supplied arguments.
 
        The specification string consists of case significant letters for each
        option. If the letter is followed by a '-' character then a long name
@@ -142,20 +164,49 @@ class PArgList : public PObject
        an associated string but this MUST follow the letter immediately, if
        it is present at all.
 
-       For example, "ab:c" allows for "-a -b arg -barg -c" and
-       "a-an-arg.b-option:c;" allows for "-a --an-arg --option arg -c -copt".
+       If the option is followed by a space character then all text up until
+       a new line '\\n' is used as a usage/help text for the option.
 
-       @return PTrue if there is at least one parameter after parsing.
+       Any text comainted with '[' and ']' characters will provide a section
+       break in the usage/help text.
+
+       For example, "ab:c" allows for "-a -b arg -barg -c" and
+       "a-an-arg. help\nb-option:c;" allows for "-a --an-arg --option arg -c -copt".
+
+       @return true if there is at least one parameter after parsing. Note if
+               false is returned, IsParsed() should be used to determined if
+               there was a parsing error, or there was just no parameters.
      */
-    virtual PBoolean Parse(
-      const char * theArgumentSpec,    ///< The specification string for argument options. See description for details.
-      PBoolean optionsBeforeParams = PTrue  ///< Parse options only before parameters
+    virtual bool Parse(
+      const char * theArgumentSpec = NULL, ///< The specification string for argument options. See description for details.
+      PBoolean optionsBeforeParams = true  ///< Parse options only before parameters
     );
     /** Parse the arguments. */
-    virtual PBoolean Parse(
+    virtual bool Parse(
       const PString & theArgumentStr,  ///< The specification string for argument options. See description for details.       
-      PBoolean optionsBeforeParams = PTrue  ///< Parse options only before parameters
+      PBoolean optionsBeforeParams = true  ///< Parse options only before parameters
     );
+
+    /**Determine if already parsed at least once.
+      */
+    bool IsParsed() const { return !m_options.empty(); }
+
+    /**Return error message after a call to Parse().
+       The return value will be an empty string if there was no error in
+       parsing the command line.
+      */
+    const PString & GetParseError() const { return m_parseError; }
+
+    /**Output usage text for parsed arguments.
+       If \p usage is non-null then text of the form "usage: processname args"
+       is output wgere args is the text in the \p usage parameter. If \p usage
+       has multiple lines in it, then each argument pattern is output.
+      */
+    ostream & Usage(
+      ostream & strm,    ///< Stream to output usage text.
+      const char * usage = NULL ///< Usage first line
+    ) const;
+    PString Usage() const;
   //@}
 
   /**@name Getting parsed arguments */
@@ -180,7 +231,7 @@ class PArgList : public PObject
     /** Get if option present.
       Determines whether the option was specified on the command line.
 
-       @return PTrue if the option was present.
+       @return true if the option was present.
      */
     PBoolean HasOption(
       char optionChar             ///< Character letter code for the option 
@@ -219,7 +270,7 @@ class PArgList : public PObject
 
     /** Get the argument count.
        Get the number of parameters that may be obtained via the
-       #GetParameter()# function. Note that this does not include options
+       <code>GetParameter()</code> function. Note that this does not include options
        and option strings.
 
        @return count of parameters.
@@ -274,54 +325,42 @@ class PArgList : public PObject
     );
   //@}
 
-  /**@name Errors */
-  //@{
-    /** This function is called when access to illegal parameter index is made
-       in the GetParameter function. The default behaviour is to output a
-       message to the standard #PError# stream.
-     */
-    virtual void IllegalArgumentIndex(
-      PINDEX idx ///< Number of the parameter that was accessed. 
-    ) const;
-
-    /** This function is called when an unknown option was specified on the
-       command line. The default behaviour is to output a message to the
-       standard #PError# stream.
-     */
-    virtual void UnknownOption(
-      const PString & option   ///< Option that was illegally placed on command line. 
-    ) const;
-
-    /** This function is called when an option that requires an associated
-       string was specified on the command line but no associated string was
-       provided. The default behaviour is to output a message to the standard
-       #PError# stream.
-     */
-    virtual void MissingArgument(
-      const PString & option  ///< Option for which the associated string was missing. 
-    ) const;
-  //@}
-
   protected:
-    /// The original program arguments.
-    PStringArray argumentArray;
-    /// The specification letters for options
-    PString      optionLetters;
-    /// The specification strings for options
-    PStringArray optionNames;
-    /// The count of the number of times an option appeared in the command line.
-    PIntArray    optionCount;
-    /// The array of associated strings to options.
-    PStringArray optionString;
-    /// The index of each .
-    PIntArray    parameterIndex;
-    /// Shift count for the parameters in the argument list.
-    int          shift;
+    PString m_commandName;
+    PString m_parseError;         // An error was detected during parsing of arguments
+    PStringArray m_argumentArray; // The original program arguments.
 
-  private:
-    PBoolean ParseOption(PINDEX idx, PINDEX offset, PINDEX & arg, const PIntArray & canHaveOptionString);
-    PINDEX GetOptionCountByIndex(PINDEX idx) const;
-    PString GetOptionStringByIndex(PINDEX idx, const char * dflt) const;
+    enum OptionType {
+      NoString,
+      HasString,
+      StringWithLetter
+    };
+    struct OptionSpec {
+      OptionSpec() : m_letter('\0'), m_type(NoString), m_count(0) { }
+      char       m_letter;
+      PString    m_name;
+      PString    m_usage;
+      PString    m_section;
+      OptionType m_type;
+
+      unsigned   m_count;
+      PString    m_string;
+    };
+    vector<OptionSpec> m_options;
+
+    /// The index of each parameter.
+    PIntArray m_parameterIndex;
+
+    /// Shift count for the parameters in the argument list.
+    int m_shift;
+    int m_argsParsed;
+
+    // Internal stuff
+    bool InternalSpecificationError(bool isError, const PString & msg);
+    size_t InternalFindOption(const PString & name) const;
+    int InternalParseOption(const PString & opt, PINDEX offset, PINDEX & arg);
+    PINDEX InternalGetOptionCountByIndex(size_t idx) const;
+    PString InternalGetOptionStringByIndex(size_t idx, const char * dflt) const;
 };
 
 
