@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 29415 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2013-04-02 16:41:13 -0500 (Tue, 02 Apr 2013) $
  */
 
 #ifndef PTLIB_CHANNEL_H
@@ -47,14 +47,14 @@ class PChannel;
 
 /* Buffer class used in PChannel stream.
 This class is necessary for implementing the standard C++ iostream interface
-on #PChannel# classes and its descendents. It is an internal class and
+on <code>PChannel</code> classes and its descendents. It is an internal class and
 should not ever be used by application writers.
 */
 class PChannelStreamBuffer : public streambuf {
 
   protected:
     /* Construct the streambuf for standard streams on a channel. This is used
-       internally by the #PChannel# class.
+       internally by the <code>PChannel</code> class.
      */
     PChannelStreamBuffer(
       PChannel * chan   // Channel the buffer operates on.
@@ -87,9 +87,9 @@ class PChannelStreamBuffer : public streambuf {
    serial port, pipe, network socket or even just a simple file. Anything that
    requires opening and closing then reading and/or writing from.
 
-   A descendent would typically have constructors and an #Open()# function which
-   enables access to the I/O channel it represents. The #Read()# and
-   #Write()# functions would then be overridden to the platform and I/O
+   A descendent would typically have constructors and an open function which
+   enables access to the I/O channel it represents. The <code>Read()</code> and
+   <code>Write()</code> functions would then be overridden to the platform and I/O
    specific mechanisms required.
 
    The general model for a channel is that the channel accepts and/or supplies
@@ -97,22 +97,23 @@ class PChannelStreamBuffer : public streambuf {
    functions that allow certain types of transfer. These include direct
    transfers, buffered transfers (via iostream) or asynchronous transfers.
 
-   The model also has the fundamental state of the channel being {\it open}
-   or {\it closed}. A channel instance that is closed contains sufficient
+   The model also has the fundamental state of the channel being <code>open</code>
+   or <code>closed</code>. A channel instance that is closed contains sufficient
    information to describe the channel but does not allocate or lock any
    system resources. An open channel allocates or locks the particular system
    resource. The act of opening a channel is a key event that may fail. In this
    case the channel remains closed and error values are set.
  */
-class PChannel : public PObject, public iostream {
+class PChannel : public PObject, public P_IOSTREAM {
   PCLASSINFO(PChannel, PObject);
 
-  public:
   /**@name Construction */
   //@{
+  protected:
       /// Create the channel.
     PChannel();
 
+  public:
       /// Close down the channel.
     ~PChannel();
   //@}
@@ -123,9 +124,9 @@ class PChannel : public PObject, public iostream {
        eg strcmp(), is used.
 
        @return
-       comparison of the two objects, #EqualTo# for same,
-       #LessThan# for #obj# logically less than the
-       object and #GreaterThan# for #obj# logically
+       comparison of the two objects, <code>EqualTo</code> for same,
+       <code>LessThan</code> for <code>obj</code> logically less than the
+       object and <code>GreaterThan</code> for <code>obj</code> logically
        greater than the object.
      */
     virtual Comparison Compare(
@@ -152,30 +153,30 @@ class PChannel : public PObject, public iostream {
   //@{
     /** Determine if the channel is currently open.
        This indicates that read and write operations can be executed on the
-       channel. For example, in the #PFile# class it returns if the file is
+       channel. For example, in the <code>PFile</code> class it returns if the file is
        currently open.
 
-       @return PTrue if the channel is open.
+       @return true if the channel is open.
      */
     virtual PBoolean IsOpen() const;
 
     /** Get the platform and I/O channel type name of the channel. For example,
-       it would return the filename in #PFile# type channels.
+       it would return the filename in <code>PFile</code> type channels.
 
        @return the name of the channel.
      */
-    virtual PString GetName() const;
+    virtual PString GetName() const { return channelName; }
 
     /** Get the integer operating system handle for the channel.
 
        @return
        standard OS descriptor integer.
      */
-    int GetHandle() const;
+    P_INT_PTR GetHandle() const;
 
     /** Get the base channel of channel indirection using PIndirectChannel.
        This function returns the eventual base channel for reading of a series
-       of indirect channels provided by descendents of #PIndirectChannel#.
+       of indirect channels provided by descendents of <code>PIndirectChannel</code>.
 
        The behaviour for this function is to return "this".
        
@@ -186,7 +187,7 @@ class PChannel : public PObject, public iostream {
 
     /** Get the base channel of channel indirection using PIndirectChannel.
        This function returns the eventual base channel for writing of a series
-       of indirect channels provided by descendents of #PIndirectChannel#.
+       of indirect channels provided by descendents of <code>PIndirectChannel</code>.
 
        The behaviour for this function is to return "this".
        
@@ -196,10 +197,96 @@ class PChannel : public PObject, public iostream {
     virtual PChannel * GetBaseWriteChannel() const;
   //@}
 
+  /**@name Error functions */
+  //@{
+    /** Normalised error codes.
+        The error result of the last file I/O operation in this object.
+     */
+    enum Errors {
+      NoError,        ///< No error
+      NotFound,       ///< Open fail due to device or file not found
+      FileExists,     ///< Open fail due to file already existing
+      DiskFull,       ///< Write fail due to disk full
+      AccessDenied,   ///< Operation fail due to insufficient privilege
+      DeviceInUse,    ///< Open fail due to device already open for exclusive use
+      BadParameter,   ///< Operation fail due to bad parameters
+      NoMemory,       ///< Operation fail due to insufficient memory
+      NotOpen,        ///< Operation fail due to channel not being open yet
+      Timeout,        ///< Operation failed due to a timeout
+      Interrupted,    ///< Operation was interrupted
+      BufferTooSmall, ///< Operations buffer was too small for data.
+      Miscellaneous,  ///< Miscellaneous error.
+      ProtocolFailure,///< High level protocol failure
+      Unavailable,    ///< Cannot use channel at this time
+      NumNormalisedErrors
+    };
+
+    /**Error groups.
+       To aid in multithreaded applications where reading and writing may be
+       happening simultaneously, read and write errors are separated from
+       other errors.
+      */
+    enum ErrorGroup {
+      LastReadError,      ///< Error during Read() operation
+      LastWriteError,     ///< Error during Write() operation
+      LastGeneralError,   ///< Error during other operation, eg Open()
+      NumErrorGroups
+    };
+
+    /** Get normalised error code.
+      Return the error result of the last file I/O operation in this object.
+      @return Normalised error code.
+      */
+    Errors GetErrorCode(
+      ErrorGroup group = NumErrorGroups   ///< Error group to get
+    ) const;
+
+    /** Get OS errro code.
+      Return the operating system error number of the last file I/O
+      operation in this object.
+      @return Operating System error code.
+      */
+    int GetErrorNumber(
+      ErrorGroup group = NumErrorGroups   ///< Error group to get
+    ) const;
+
+    /** Get error message description.
+        Return a string indicating the error message that may be displayed to
+       the user. The error for the last I/O operation in this object is used.
+       @return Operating System error description string.
+     */
+    virtual PString GetErrorText(
+      ErrorGroup group = NumErrorGroups   ///< Error group to get
+    ) const;
+
+    /** Get error message description.
+       Return a string indicating the error message that may be displayed to
+       the user. The <code>osError</code> parameter is used unless zero, in which case
+       the <code>lastError</code> parameter is used.
+       @return Operating System error description string.
+     */
+    static PString GetErrorText(
+      Errors lastError,   ///< Error code to translate.
+      int osError = 0     ///< OS error number to translate.
+    );
+
+    /** Convert an operating system error into platform independent error.
+       This will set the lastError and osError member variables for access by
+       GetErrorCode() and GetErrorNumber().
+       
+       @return true if there was no error.
+     */
+    static PBoolean ConvertOSError(
+      P_INT_PTR libcReturnValue,
+      Errors & lastError,
+      int & osError
+    );
+  //@}
+
   /**@name Reading functions */
   //@{
     /** Set the timeout for read operations. This may be zero for immediate
-       return of data through to #PMaxTimeInterval# which will wait forever for
+       return of data through to <code>PMaxTimeInterval</code> which will wait forever for
        the read request to be filled.
        
        Note that this function may not be available, or meaningfull, for all
@@ -223,11 +310,11 @@ class PChannel : public PObject, public iostream {
        of bytes read.
 
        The GetErrorCode() function should be consulted after Read() returns
-       PFalse to determine what caused the failure.
+       false to determine what caused the failure.
 
        @return
-       PTrue indicates that at least one character was read from the channel.
-       PFalse means no bytes were read due to timeout or some other I/O error.
+       true indicates that at least one character was read from the channel.
+       false means no bytes were read due to timeout or some other I/O error.
      */
     virtual PBoolean Read(
       void * buf,   ///< Pointer to a block of memory to receive the read bytes.
@@ -265,45 +352,22 @@ class PChannel : public PObject, public iostream {
        all of the bytes have been read, or an error occurs.
 
        @return
-       PTrue if the read of #len# bytes was sucessfull.
+       true if the read of <code>len</code> bytes was sucessfull.
      */
     PBoolean ReadBlock(
       void * buf,   ///< Pointer to a block of memory to receive the read bytes.
       PINDEX len    ///< Maximum number of bytes to read into the buffer.
     );
 
-    /** Read #len# character into a string from the channel. This
+    /** Read <code>len</code> character into a string from the channel. This
        function simply uses ReadBlock(), so all remarks pertaining to that
        function also apply to this one.
 
        @return
        String that was read.
      */
-    PString ReadString(PINDEX len);
-
-    /** Begin an asynchronous read from channel. The read timeout is used as in
-       other read operations, in this case calling the OnReadComplete()
-       function.
-
-       Note that if the channel is not capable of asynchronous read then this
-       will do a sychronous read is in the Read() function with the addition
-       of calling the OnReadComplete() before returning.
-
-       @return
-       PTrue if the read was sucessfully queued.
-     */
-    virtual PBoolean ReadAsync(
-      void * buf,   ///< Pointer to a block of memory to receive the read bytes.
-      PINDEX len    ///< Maximum number of bytes to read into the buffer.
-    );
-
-    /** User callback function for when a #ReadAsync()# call has completed or
-       timed out. The original pointer to the buffer passed in ReadAsync() is
-       passed to the function.
-     */
-    virtual void OnReadComplete(
-      void * buf, ///< Pointer to a block of memory that received the read bytes.
-      PINDEX len  ///< Actual number of bytes to read into the buffer.
+    PString ReadString(
+      PINDEX len  ///< Length of string data to read.
     );
   //@}
 
@@ -335,22 +399,41 @@ class PChannel : public PObject, public iostream {
        of bytes written.
 
        The GetErrorCode() function should be consulted after Write() returns
-       PFalse to determine what caused the failure.
+       false to determine what caused the failure.
 
        @return
-       PTrue if at least len bytes were written to the channel.
+       true if at least len bytes were written to the channel.
      */
     virtual PBoolean Write(
       const void * buf, ///< Pointer to a block of memory to write.
       PINDEX len        ///< Number of bytes to write.
     );
 
+	/** Low level write to the channel with marker.	(H323plus Implementation)
+       This function will block until the requested number of characters	 
+       are written or the write timeout is reached. The GetLastWriteCount()	 
+       function returns the actual number of bytes written. By default it	 
+       calls the Write(void *,len) function	 
+ 	 
+       The GetErrorCode() function should be consulted after Write() returns	 
+       false to determine what caused the failure.	 
+ 	 
+       @return	 
+       true if at least len bytes were written to the channel.	 
+      */	 
+     virtual PBoolean Write(	 
+       const void * buf,       ///< Pointer to a block of memory to write.	 
+       PINDEX len,             ///< Number of bytes to write.	 
+       const void * /*mark*/   ///< pointer for AVsync & render buffering.	 
+     ) { return Write(buf,len); }	 
+ 
+
     /** Get the number of bytes written by the last Write() call.
        
        Note that the number of bytes written may often be less, or even more,
        than that asked for. A common case of it being less is where the disk
        is full. An example of where the bytes written is more is as follows.
-       On a #PTextFile# channel on the MSDOS platform, there is
+       On a <code>PTextFile</code> channel on the MSDOS platform, there is
        translation of \n to CR/LF pairs. This will result in the number of
        bytes returned being more than that requested.
 
@@ -365,7 +448,7 @@ class PChannel : public PObject, public iostream {
        Note that this asserts if the value is not in the range 0..255.
 
        @return
-       PTrue if the byte was successfully written.
+       true if the byte was successfully written.
      */
     PBoolean WriteChar(int c);
 
@@ -373,32 +456,118 @@ class PChannel : public PObject, public iostream {
        function so all comments on that function also apply.
 
        @return
-       PTrue if the character written.
+       true if the character written.
      */
-    PBoolean WriteString(const PString & str);
+    PBoolean WriteString(
+      const PString & str   ///< String to write
+    );
+  //@}
+
+  /**@name Asynchronous I/O functions */
+  //@{
+    class AsyncContext;
+    typedef PNotifierTemplate<PChannel::AsyncContext &> AsyncNotifier;
+    #define PDECLARE_AsyncNotifier(cls, fn) PDECLARE_NOTIFIER2(PChannel, cls, fn, PChannel::AsyncContext &)
+    #define PCREATE_AsyncNotifier(fn) PCREATE_NOTIFIER2(fn, PChannel::AsyncContext &)
+
+#ifdef _MSC_VER
+  #pragma pack(push)
+  #if P_64BIT
+    #pragma pack(8)
+  #else
+    #pragma pack(4)
+  #endif
+#endif
+    /** Parameters for asynchronous I/O operation.
+      */
+#ifdef _WIN32
+    typedef OVERLAPPED AsyncContextBase;
+#elif defined _AIO_H
+    typedef struct aiocb AsyncContextBase;
+#else
+    class AsyncContextBase { off_t m_offset; };
+#endif
+    class AsyncContext : public AsyncContextBase
+    {
+      public:
+        AsyncContext(
+          void * buf = NULL,
+          PINDEX len = 0,
+          const AsyncNotifier & notifier = AsyncNotifier()
+        );
+
+        /** Set the offset to do the read/write operation.
+          */
+        void SetOffset(off_t offset);
+
+        void        * m_buffer;       ///< Pointer to a block of memory to receive the read bytes.
+        PINDEX        m_length;       ///< Maximum number of bytes to read into the buffer.
+        AsyncNotifier m_notifier;     ///< Notification function for when asynchronous operation complete.
+
+        Errors        m_errorCode;    ///< Error returned after operation completed.
+        int           m_errorNumber;  ///< OS error returned after operation completed.
+
+        // Internal stuff
+        PChannel * m_channel;
+        typedef void (PChannel::*CompletionFunction)(AsyncContext &);
+        CompletionFunction m_onComplete;
+        bool Initialise(PChannel * channel, CompletionFunction onComplete);
+        void OnIOComplete(PINDEX length, int errorNumber);
+    };
+#ifdef _MSC_VER
+  #pragma pack(pop)
+#endif
+
+    /** Begin an asynchronous read from channel. The read timeout is used as in
+       other read operations, in this case calling the OnReadComplete()
+       function.
+
+       If the channel is not capable of asynchronous read then returns false.
+
+       Note: the life time of the context and the m_buffer within that context
+       must exceed the duration of the asynchronous operation. After
+       OnReadComplete has been called, these objects may be destroyed.
+
+       @return
+       true if the read was sucessfully queued.
+     */
+    virtual bool ReadAsync(
+      AsyncContext & context ///< Context for asynchronous operation
+    );
+
+    /** User callback function for when a <code>ReadAsync()</code> call has completed or
+       timed out.
+
+       The default behaviour calls the m_callback field of context.
+     */
+    virtual void OnReadComplete(
+      AsyncContext & context ///< Context for asynchronous operation
+    );
 
     /** Begin an asynchronous write from channel. The write timeout is used as
        in other write operations, in this case calling the OnWriteComplete()
-       function. Note that if the channel is not capable of asynchronous write
-       then this will do a sychronous write as in the Write() function with
-       the addition of calling the OnWriteComplete() before returning.
+       function.
+
+       If the channel is not capable of asynchronous write then returns false.
+
+       Note: the life time of the context and the m_buffer within that context
+       must exceed the duration of the asynchronous operation. After
+       OnWriteComplete has been called, these objects may be destroyed.
 
        @return
-       PTrue of the write operation was succesfully queued.
+       true of the write operation was succesfully queued.
      */
-    virtual PBoolean WriteAsync(
-      const void * buf, ///< Pointer to a block of memory to write.
-      PINDEX len        ///< Number of bytes to write.
+    virtual bool WriteAsync(
+      AsyncContext & context ///< Context for asynchronous operation
     );
 
     /** User callback function for when a WriteAsync() call has completed or
-       timed out. The original pointer to the buffer passed in WriteAsync() is
-       passed in here and the len parameter is the actual number of characters
-       written.
+       timed out.
+
+       The default behaviour calls the m_callback field of context.
      */
     virtual void OnWriteComplete(
-      const void * buf, ///< Pointer to a block of memory to write.
-      PINDEX len        ///< Number of bytes to write.
+      AsyncContext & context ///< Context for asynchronous operation
     );
   //@}
 
@@ -406,7 +575,7 @@ class PChannel : public PObject, public iostream {
   //@{
     /** Close the channel, shutting down the link to the data source.
 
-       @return PTrue if the channel successfully closed.
+       @return true if the channel successfully closed.
      */
     virtual PBoolean Close();
 
@@ -418,19 +587,35 @@ class PChannel : public PObject, public iostream {
 
     /** Close one or both of the data streams associated with a channel.
 
-       The default behavour is to do nothing and return PFalse.
+       The default behavour is to do nothing and return false.
 
        @return
-       PTrue if the shutdown was successfully performed.
+       true if the shutdown was successfully performed.
      */
     virtual PBoolean Shutdown(
       ShutdownValue option
     );
 
+    /**Set local echo mode.
+       For some classes of channel, e.g. PConsoleChannel, data read by this
+       channel is automatically echoed. This disables the function so things
+       like password entry can work.
+
+       Default behaviour does nothing and return true if the channel is open.
+      */
+    virtual bool SetLocalEcho(
+      bool localEcho
+    );
+
+    /**Flow Control information 
+       Pass data to the channel for flowControl determination.
+      */
+    virtual bool FlowControl(const void * flowData);
+
     /**Set the iostream buffer size for reads and writes.
 
        @return
-       PTrue if the new buffer size was set.
+       true if the new buffer size was set.
       */
     PBoolean SetBufferSize(
       PINDEX newSize    ///< New buffer size
@@ -441,39 +626,38 @@ class PChannel : public PObject, public iostream {
        the C language.
 
        The escape commands are:
-\begin{description}
-          \item[#\a#]    alert (ascii value 7)
-          \item[#\b#]    backspace (ascii value 8)
-          \item[#\f#]    formfeed (ascii value 12)
-          \item[#\n#]    newline (ascii value 10)
-          \item[#\r#]    return (ascii value 13)
-          \item[#\t#]    horizontal tab (ascii value 9)
-          \item[#\v#]    vertical tab (ascii value 11)
-          \item[#\\#]    backslash
-          \item[#\ooo#]  where ooo is octal number, ascii value ooo
-          \item[#\xhh#]  where hh is hex number (ascii value 0xhh)
-          \item[#\0#]    null character (ascii zero)
-          \item[#\dns#]  delay for n seconds
-          \item[#\dnm#]  delay for n milliseconds
-          \item[#\s#]    characters following this, up to a \w
-                                     command or the end of string, are to be
-                                     sent to modem
-          \item[#\wns#]  characters following this, up to a \s, \d
-                                     or another \w or the end of the string are
-                                     expected back from the modem. If the
-                                     string is not received within n seconds,
-                                     a failed command is registered. The
-                                     exception to this is if the command is at
-                                     the end of the string or the next
-                                     character in the string is the \s, \d or
-                                     \w in which case all characters are
-                                     ignored from the modem until n seconds of
-                                     no data.
-          \item[#\wnm#]  as for above but timeout is in
-                                     milliseconds.
-\end{description}
+          <table border=0>
+          <tr><td>\\a    <td>alert (ascii value 7)
+          <tr><td>\\b    <td>backspace (ascii value 8)
+          <tr><td>\\f    <td>formfeed (ascii value 12)
+          <tr><td>\\n    <td>newline (ascii value 10)
+          <tr><td>\\r    <td>return (ascii value 13)
+          <tr><td>\\t    <td>horizontal tab (ascii value 9)
+          <tr><td>\\v    <td>vertical tab (ascii value 11)
+          <tr><td>\\\\   <td> backslash
+          <tr><td>\\ooo  <td>where ooo is octal number, ascii value ooo
+          <tr><td>\\xhh  <td>where hh is hex number (ascii value 0xhh)
+          <tr><td>\\0    <td>null character (ascii zero)
+          <tr><td>\\dns  <td>delay for n seconds
+          <tr><td>\\dnm  <td>delay for n milliseconds
+          <tr><td>\\s    <td>characters following this, up to a \\w
+                          command or the end of string, are to be
+                          sent to modem
+          <tr><td>\\wns  <td>characters following this, up to a \\s, \\d
+                          or another \\w or the end of the string are
+                          expected back from the modem. If the
+                          string is not received within n seconds,
+                          a failed command is registered. The
+                          exception to this is if the command is at
+                          the end of the string or the next
+                          character in the string is the \\s, \\d or
+                          \\w in which case all characters are
+                          ignored from the modem until n seconds of
+                          no data.
+          <tr><td>\\wnm  <td>as for above but timeout is in milliseconds.
+          </table>
        @return
-       PTrue if the command string was completely processed.
+       true if the command string was completely processed.
      */
     PBoolean SendCommandString(
       const PString & command  ///< Command to send to the channel
@@ -486,146 +670,6 @@ class PChannel : public PObject, public iostream {
     void AbortCommandString();
   //@}
 
-  /**@name Error functions */
-  //@{
-    /** Normalised error codes.
-        The error result of the last file I/O operation in this object.
-     */
-    enum Errors {
-      NoError,
-      /// Open fail due to device or file not found
-      NotFound,       
-      /// Open fail due to file already existing
-      FileExists,     
-      /// Write fail due to disk full
-      DiskFull,       
-      /// Operation fail due to insufficient privilege
-      AccessDenied,   
-      /// Open fail due to device already open for exclusive use
-      DeviceInUse,    
-      /// Operation fail due to bad parameters
-      BadParameter,   
-      /// Operation fail due to insufficient memory
-      NoMemory,       
-      /// Operation fail due to channel not being open yet
-      NotOpen,        
-      /// Operation failed due to a timeout
-      Timeout,        
-      /// Operation was interrupted
-      Interrupted,    
-      /// Operations buffer was too small for data.
-      BufferTooSmall, 
-      /// Miscellaneous error.
-      Miscellaneous,
-      /// High level protocol failure
-      ProtocolFailure,
-      NumNormalisedErrors
-    };
-
-    /**Error groups.
-       To aid in multithreaded applications where reading and writing may be
-       happening simultaneously, read and write errors are separated from
-       other errors.
-      */
-    enum ErrorGroup {
-      LastReadError,      ///< Error during Read() operation
-      LastWriteError,     ///< Error during Write() operation
-      LastGeneralError,   ///< Error during other operation, eg Open()
-      NumErrorGroups
-    };
-
-    /** Get normalised error code.
-      Return the error result of the last file I/O operation in this object.
-      @return Normalised error code.
-      */
-    Errors GetErrorCode(
-      ErrorGroup group = NumErrorGroups   ///< Error group to get
-    ) const;
-
-    /** Get OS errro code.
-      Return the operating system error number of the last file I/O
-      operation in this object.
-      @return Operating System error code.
-      */
-    int GetErrorNumber(
-      ErrorGroup group = NumErrorGroups   ///< Error group to get
-    ) const;
-
-      /** Get error message description.
-        Return a string indicating the error message that may be displayed to
-       the user. The error for the last I/O operation in this object is used.
-      @return Operating System error description string.
-       */
-    virtual PString GetErrorText(
-      ErrorGroup group = NumErrorGroups   ///< Error group to get
-    ) const;
-
-      /** Get error message description.
-        Return a string indicating the error message that may be displayed to
-       the user. The #osError# parameter is used unless zero, in which case
-       the #lastError# parameter is used.
-      @return Operating System error description string.
-       */
-    static PString GetErrorText(
-      Errors lastError,   ///< Error code to translate.
-      int osError = 0     ///< OS error number to translate.
-    );
-  //@}
-
-    /** Convert an operating system error into platform independent error.
-       This will set the lastError and osError member variables for access by
-       GetErrorCode() and GetErrorNumber().
-       
-       @return PTrue if there was no error.
-     */
-    static PBoolean ConvertOSError(
-      int libcReturnValue,
-      Errors & lastError,
-      int & osError
-    );
-
-  /**@name Scattered read/write functions */
-  //@{
-    /** Structure that defines a "slice" of memory to be written to
-     */
-#if P_HAS_RECVMSG
-    typedef iovec Slice;
-#else
-    struct Slice {
-      void * iov_base;
-      size_t iov_len;
-    };
-#endif
-
-    typedef std::vector<Slice> VectorOfSlice;
-
-    /** Low level scattered read from the channel. This is identical to Read except 
-        that the data will be read into a series of scattered memory slices. By default,
-        this call will default to calling Read multiple times, but this may be 
-        implemented by operating systems to do a real scattered read
-
-       @return
-       PTrue indicates that at least one character was read from the channel.
-       PFalse means no bytes were read due to timeout or some other I/O error.
-     */
-    virtual PBoolean Read(
-      const VectorOfSlice & slices    // slices to read to
-    );
-
-    /** Low level scattered write to the channel. This is identical to Write except 
-        that the data will be written from a series of scattered memory slices. By default,
-        this call will default to calling Write multiple times, but this can be actually
-        implemented by operating systems to do a real scattered write
-
-       @return
-       PTrue indicates that at least one character was read from the channel.
-       PFalse means no bytes were read due to timeout or some other I/O error.
-     */
-    virtual PBoolean Write(
-      const VectorOfSlice & slices    // slices to read to
-    );
-  //@}
-
   protected:
     PChannel(const PChannel &);
     PChannel & operator=(const PChannel &);
@@ -634,18 +678,18 @@ class PChannel : public PObject, public iostream {
 
     /** Convert an operating system error into platform independent error.
       The internal error codes are set by this function. They may be obtained
-      via the #GetErrorCode()# and #GetErrorNumber()# functions.
+      via the <code>GetErrorCode()</code> and <code>GetErrorNumber()</code> functions.
        
-       @return PTrue if there was no error.
+       @return true if there was no error.
      */
     virtual PBoolean ConvertOSError(
-      int libcReturnValue,
+      P_INT_PTR libcReturnValue,                ///< Return value from standard library
       ErrorGroup group = LastGeneralError ///< Error group to set
     );
 
   public:
     /**Set error values to those specified.
-       Return PTrue if errorCode is NoError, PFalse otherwise
+       Return true if errorCode is NoError, false otherwise
       */
     PBoolean SetErrorValues(
       Errors errorCode,   ///< Error code to translate.
@@ -656,11 +700,11 @@ class PChannel : public PObject, public iostream {
   protected:
     /** Read a character with specified timeout.
       This reads a single character from the channel waiting at most the
-      amount of time specified for it to arrive. The #timeout# parameter
+      amount of time specified for it to arrive. The <code>timeout</code> parameter
       is adjusted for amount of time it actually took, so it can be used
       for a multiple character timeout.
 
-       @return PTrue if there was no error.
+       @return true if there was no error.
      */
     int ReadCharWithTimeout(
       PTimeInterval & timeout  // Timeout for read.
@@ -677,7 +721,7 @@ class PChannel : public PObject, public iostream {
 
     // Member variables
     /// The operating system file handle return by standard open() function.
-    int os_handle;
+    P_INT_PTR os_handle;
     /// The platform independant error code.
     Errors lastErrorCode[NumErrorGroups+1];
     /// The operating system error number (eg as returned by errno).
@@ -690,7 +734,8 @@ class PChannel : public PObject, public iostream {
     PTimeInterval readTimeout;
     /// Timeout for write operations.
     PTimeInterval writeTimeout;
-
+    /// Name of channel
+    PString channelName;
 
   private:
     // New functions for class
@@ -712,6 +757,19 @@ class PChannel : public PObject, public iostream {
 };
 
 
+/** A channel that does nothing.
+  */
+class PNullChannel : public PChannel
+{
+    PCLASSINFO(PNullChannel, PChannel)
+  public:
+    PNullChannel();
+
+    PBoolean Read(void *, PINDEX);
+    PBoolean Write(const void *, PINDEX);
+};
+
+
 #endif // PTLIB_CHANNEL_H
 
 
