@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23897 $
+ * $Revision: 27039 $
  * $Author: rjongbloed $
- * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
+ * $Date: 2012-02-22 01:19:46 -0600 (Wed, 22 Feb 2012) $
  */
 
 
@@ -43,6 +43,7 @@
 #include <sys/stat.h>
 #endif
 
+#include <ptlib/bitwise_enum.h>
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -66,75 +67,65 @@ class PFile : public PChannel
   //@{
     /**Create a file object but do not open it. It does not initially have a
        valid file name. However, an attempt to open the file using the
-       #Open()# function will generate a unique temporary file.
+       <code>Open()</code> function will generate a unique temporary file.
      */
     PFile();
 
     /**When a file is opened, it may restrict the access available to
        operations on the object instance. A value from this enum is passed to
-       the #Open()# function to set the mode.
+       the <code>Open()</code> function to set the mode.
      */
     enum OpenMode {
-      /// File can be read but not written.
-      ReadOnly,
-      /// File can be written but not read.
-      WriteOnly,
-      /// File can be both read and written.
-      ReadWrite
+      ReadOnly,   ///< File can be read but not written.
+      WriteOnly,  ///< File can be written but not read.
+      ReadWrite   ///< File can be both read and written.
     };
 
     /**When a file is opened, a number of options may be associated with the
        open file. These describe what action to take on opening the file and
        what to do on closure. A value from this enum is passed to the
-       #Open()# function to set the options.
+       <code>Open()</code> function to set the options.
 
-       The #ModeDefault# option will use the following values:
-          \arg \c ReadOnly  #MustExist#
-          \arg \c WriteOnly #Create | Truncate#
-          \arg \c ReadWrite #Create#
+       The <code>ModeDefault</code> option will use the following values:
+          \arg \c ReadOnly  <code>MustExist</code>
+          \arg \c WriteOnly <code>Create | Truncate</code>
+          \arg \c ReadWrite <code>Create</code>
      */
-    enum OpenOptions {
-      /// File options depend on the OpenMode parameter.
-      ModeDefault = -1, 
-      /// File open fails if file does not exist.
-      MustExist = 0,    
-      /// File is created if it does not exist.
-      Create = 1,       
-      /// File is set to zero length if it already exists.
-      Truncate = 2,     
-      /// File open fails if file already exists.
-      Exclusive = 4,    
-      /// File is temporary and is to be deleted when closed.
-      Temporary = 8,
-      /// File may not be read by another process.
-      DenySharedRead = 16,
-      /// File may not be written by another process.
-      DenySharedWrite = 32
-    };
+    P_DECLARE_BITWISE_ENUM_EX(OpenOptions, 6,
+        (MustExist,       ///< File open fails if file does not exist.
+         Create,          ///< File is created if it does not exist.
+         Truncate,        ///< File is set to zero length if it already exists.
+         Exclusive,       ///< File open fails if file already exists.
+         Temporary,       ///< File is temporary and is to be deleted when closed.
+         DenySharedRead,  ///< File may not be read by another process.
+         DenySharedWrite  ///< File may not be written by another process.
+        ), 
+        ModeDefault = -1  ///< File options depend on the OpenMode parameter.
+    );
 
     /**Create a unique temporary file name, and open the file in the specified
        mode and using the specified options. Note that opening a new, unique,
        temporary file name in ReadOnly mode will always fail. This would only
        be usefull in a mode and options that will create the file.
 
-       The #PChannel::IsOpen()# function may be used after object
+       The <code>PChannel::IsOpen()</code> function may be used after object
        construction to determine if the file was successfully opened.
      */
     PFile(
       OpenMode mode,          ///< Mode in which to open the file.
-      int opts = ModeDefault  ///< #OpenOptions enum# for open operation.
+      OpenOptions opts = ModeDefault  ///< <code>OpenOptions</code> enum# for open operation.
     );
 
     /**Create a file object with the specified name and open it in the
        specified mode and with the specified options.
 
-       The #PChannel::IsOpen()# function may be used after object
+       The <code>PChannel::IsOpen()</code> function may be used after object
        construction to determine if the file was successfully opened.
      */
     PFile(
       const PFilePath & name,    ///< Name of file to open.
       OpenMode mode = ReadWrite, ///< Mode in which to open the file.
-      int opts = ModeDefault     ///< #OpenOptions enum# for open operation.
+      OpenOptions opts = ModeDefault     ///< <code>OpenOptions</code> enum# for open operation.
     );
 
     /// Close the file on destruction.
@@ -145,7 +136,7 @@ class PFile : public PChannel
   /**@name Overrides from class PObject */
   //@{
     /**Determine the relative rank of the two objects. This is essentially the
-       string comparison of the #PFilePath# names of the files.
+       string comparison of the <code>PFilePath</code> names of the files.
 
        @return
        relative rank of the file paths.
@@ -159,7 +150,7 @@ class PFile : public PChannel
   /**@name Overrides from class PChannel */
   //@{
     /**Get the platform and I/O channel type name of the channel. For example,
-       it would return the filename in #PFile# type channels.
+       it would return the filename in <code>PFile</code> type channels.
 
        @return
        the name of the channel.
@@ -171,11 +162,11 @@ class PFile : public PChannel
        of bytes read.
 
        The GetErrorCode() function should be consulted after Read() returns
-       PFalse to determine what caused the failure.
+       false to determine what caused the failure.
 
        @return
-       PTrue indicates that at least one character was read from the channel.
-       PFalse means no bytes were read due to timeout or some other I/O error.
+       true indicates that at least one character was read from the channel.
+       false means no bytes were read due to timeout or some other I/O error.
      */
     virtual PBoolean Read(
       void * buf,   ///< Pointer to a block of memory to receive the read bytes.
@@ -187,9 +178,9 @@ class PFile : public PChannel
        of bytes written.
 
        The GetErrorCode() function should be consulted after Write() returns
-       PFalse to determine what caused the failure.
+       false to determine what caused the failure.
 
-       @return PTrue if at least len bytes were written to the channel.
+       @return true if at least len bytes were written to the channel.
      */
     virtual PBoolean Write(
       const void * buf, ///< Pointer to a block of memory to write.
@@ -197,7 +188,7 @@ class PFile : public PChannel
     );
 
     /** Close the file channel.
-        @return PTrue if close was OK.
+        @return true if close was OK.
       */
     virtual PBoolean Close();
   //@}
@@ -210,7 +201,7 @@ class PFile : public PChannel
        file system.
 
        @return
-       PTrue if the file exists.
+       true if the file exists.
      */
     static PBoolean Exists(
       const PFilePath & name  ///< Name of file to see if exists.
@@ -221,7 +212,7 @@ class PFile : public PChannel
        of the object actually exists within the platforms file system.
 
        @return
-       PTrue if the file exists.
+       true if the file exists.
      */
     PBoolean Exists() const;
 
@@ -229,10 +220,10 @@ class PFile : public PChannel
        Determine if the file specified may be opened in the specified mode. This would
        check the current access rights to the file for the mode. For example,
        for a file that is read only, using mode == ReadWrite would return
-       PFalse but mode == ReadOnly would return PTrue.
+       false but mode == ReadOnly would return true.
 
        @return
-       PTrue if a file open would succeed.
+       true if a file open would succeed.
      */
     static PBoolean Access(
       const PFilePath & name, ///< Name of file to have its access checked.
@@ -244,57 +235,57 @@ class PFile : public PChannel
        instance of the object may be opened in the specified mode. This would
        check the current access rights to the file for the mode. For example,
        for a file that is read only, using mode == ReadWrite would return
-       PFalse but mode == ReadOnly would return PTrue.
+       false but mode == ReadOnly would return true.
 
        @return
-       PTrue if a file open would succeed.
+       true if a file open would succeed.
      */
     PBoolean Access(
       OpenMode mode         ///< Mode in which the file open would be done.
     );
 
-    /**Delete the specified file. If #force# is PFalse and the file
+    /**Delete the specified file. If <code>force</code> is false and the file
        is protected against being deleted then the function fails. If
-       #force# is PTrue then the protection is ignored. What
+       <code>force</code> is true then the protection is ignored. What
        constitutes file deletion protection is platform dependent, eg on DOS
        is the Read Only attribute and on a Novell network it is a Delete
        trustee right. Some protection may not be able to overridden with the
-       #force# parameter at all, eg on a Unix system and you are
+       <code>force</code> parameter at all, eg on a Unix system and you are
        not the owner of the file.
 
        @return
-       PTrue if the file was deleted.
+       true if the file was deleted.
      */
     static PBoolean Remove(
       const PFilePath & name,   // Name of file to delete.
-      PBoolean force = PFalse      // Force deletion even if file is protected.
+      PBoolean force = false      // Force deletion even if file is protected.
     );
     static PBoolean Remove(
       const PString & name,   // Name of file to delete.
-      PBoolean force = PFalse      // Force deletion even if file is protected.
+      PBoolean force = false      // Force deletion even if file is protected.
     );
 
-    /**Delete the current file. If #force# is PFalse and the file
+    /**Delete the current file. If <code>force</code> is false and the file
        is protected against being deleted then the function fails. If
-       #force# is PTrue then the protection is ignored. What
+       <code>force</code> is true then the protection is ignored. What
        constitutes file deletion protection is platform dependent, eg on DOS
        is the Read Only attribute and on a Novell network it is a Delete
        trustee right. Some protection may not be able to overridden with the
-       #force# parameter at all, eg on a Unix system and you are
+       <code>force</code> parameter at all, eg on a Unix system and you are
        not the owner of the file.
 
        @return
-       PTrue if the file was deleted.
+       true if the file was deleted.
      */
     PBoolean Remove(
-      PBoolean force = PFalse      // Force deletion even if file is protected.
+      PBoolean force = false      // Force deletion even if file is protected.
     );
 
     /**Change the specified files name. This does not move the file in the
        directory hierarchy, it only changes the name of the directory entry.
 
-       The #newname# parameter must consist only of the file name
-       part, as returned by the #PFilePath::GetFileName()# function. Any
+       The <code>newname</code> parameter must consist only of the file name
+       part, as returned by the <code>PFilePath::GetFileName()</code> function. Any
        other file path parts will cause an error.
 
        The first form uses the file path specification associated with the
@@ -303,12 +294,12 @@ class PFile : public PChannel
        arbitrary file specified by name.
 
        @return
-       PTrue if the file was renamed.
+       true if the file was renamed.
      */
     static PBoolean Rename(
       const PFilePath & oldname,  ///< Old name of the file.
       const PString & newname,    ///< New name for the file.
-      PBoolean force = PFalse
+      PBoolean force = false
         ///< Delete file if a destination exists with the same name.
     );
 
@@ -316,8 +307,8 @@ class PFile : public PChannel
        This does not move the file in the
        directory hierarchy, it only changes the name of the directory entry.
 
-       The #newname# parameter must consist only of the file name
-       part, as returned by the #PFilePath::GetFileName()# function. Any
+       The <code>newname</code> parameter must consist only of the file name
+       part, as returned by the <code>PFilePath::GetFileName()</code> function. Any
        other file path parts will cause an error.
 
        The first form uses the file path specification associated with the
@@ -326,34 +317,34 @@ class PFile : public PChannel
        arbitrary file specified by name.
 
        @return
-       PTrue if the file was renamed.
+       true if the file was renamed.
      */
     PBoolean Rename(
       const PString & newname,  ///< New name for the file.
-      PBoolean force = PFalse
+      PBoolean force = false
         ///< Delete file if a destination exists with the same name.
     );
 
     /**Make a copy of the specified file.
 
        @return
-       PTrue if the file was renamed.
+       true if the file was renamed.
      */
     static PBoolean Copy(
       const PFilePath & oldname,  ///< Old name of the file.
       const PFilePath & newname,  ///< New name for the file.
-      PBoolean force = PFalse
+      PBoolean force = false
         ///< Delete file if a destination exists with the same name.
     );
 
     /**Make a copy of the current file.
 
        @return
-       PTrue if the file was renamed.
+       true if the file was renamed.
      */
     PBoolean Copy(
       const PFilePath & newname,  ///< New name for the file.
-      PBoolean force = PFalse
+      PBoolean force = false
         ///< Delete file if a destination exists with the same name.
     );
 
@@ -364,12 +355,12 @@ class PFile : public PChannel
        it is across file systems then a copy and a delete is performed.
 
        @return
-       PTrue if the file was moved.
+       true if the file was moved.
      */
     static PBoolean Move(
       const PFilePath & oldname,  ///< Old path and name of the file.
       const PFilePath & newname,  ///< New path and name for the file.
-      PBoolean force = PFalse
+      PBoolean force = false
         ///< Delete file if a destination exists with the same name.
     );
 
@@ -380,18 +371,18 @@ class PFile : public PChannel
        it is across file systems then a copy and a delete is performed.
 
        @return
-       PTrue if the file was moved.
+       true if the file was moved.
      */
     PBoolean Move(
       const PFilePath & newname,  ///< New path and name for the file.
-      PBoolean force = PFalse
+      PBoolean force = false
         ///< Delete file if a destination exists with the same name.
     );
   //@}
 
   /**@name File channel functions */
   //@{
-    /**Get the full path name of the file. The #PFilePath# object
+    /**Get the full path name of the file. The <code>PFilePath</code> object
        describes the full file name specification for the particular platform.
 
        @return
@@ -399,7 +390,7 @@ class PFile : public PChannel
      */
     const PFilePath & GetFilePath() const;
 
-    /**Set the full path name of the file. The #PFilePath# object
+    /**Set the full path name of the file. The <code>PFilePath</code> object
        describes the full file name specification for the particular platform.
      */
     void SetFilePath(
@@ -412,31 +403,31 @@ class PFile : public PChannel
        it is closed.
        
        If there has not been a filename attached to the file object (via
-       #SetFilePath()#, the #name# parameter or a previous
+       <code>SetFilePath()</code>, the <code>name</code> parameter or a previous
        open) then a new unique temporary filename is generated.
 
        @return
-       PTrue if the file was successfully opened.
+       true if the file was successfully opened.
      */
     virtual PBoolean Open(
       OpenMode mode = ReadWrite,  // Mode in which to open the file.
-      int opts = ModeDefault      // Options for open operation.
+      OpenOptions opts = ModeDefault      // Options for open operation.
     );
 
     /**Open the specified file name in the specified mode and with
        the specified options. If the file object already has an open file then
        it is closed.
        
-       Note: if #mode# is StandardInput, StandardOutput or StandardError,
-       then the #name# parameter is ignored.
+       Note: if <code>mode</code> is StandardInput, StandardOutput or StandardError,
+       then the <code>name</code> parameter is ignored.
 
        @return
-       PTrue if the file was successfully opened.
+       true if the file was successfully opened.
      */
     virtual PBoolean Open(
       const PFilePath & name,    // Name of file to open.
       OpenMode mode = ReadWrite, // Mode in which to open the file.
-      int opts = ModeDefault     // #OpenOptions enum# for open operation.
+      OpenOptions opts = ModeDefault     // <code>OpenOptions</code> enum# for open operation.
     );
       
     /**Get the current size of the file.
@@ -450,7 +441,7 @@ class PFile : public PChannel
        expanding the file, or truncating it if being made shorter.
 
        @return
-       PTrue if the file size was changed to the length specified.
+       true if the file size was changed to the length specified.
      */
     virtual PBoolean SetLength(
       off_t len   // New length of file.
@@ -467,14 +458,14 @@ class PFile : public PChannel
     };
 
     /**Set the current active position in the file for the next read or write
-       operation. The #pos# variable is a signed number which is
-       added to the specified origin. For #origin == PFile::Start#
-       only positive values for #pos# are meaningful. For
-       #origin == PFile::End# only negative values for
-       #pos# are meaningful.
+       operation. The <code>pos</code> variable is a signed number which is
+       added to the specified origin. For <code>origin == PFile::Start</code>
+       only positive values for <code>pos</code> are meaningful. For
+       <code>origin == PFile::End</code> only negative values for
+       <code>pos</code> are meaningful.
 
        @return
-       PTrue if the new file position was set.
+       true if the new file position was set.
      */
     virtual PBoolean SetPosition(
       off_t pos,                         ///< New position to set.
@@ -490,38 +481,38 @@ class PFile : public PChannel
     virtual off_t GetPosition() const;
 
     /**Determine if the current file position is at the end of the file. If
-       this is PTrue then any read operation will fail.
+       this is true then any read operation will fail.
 
        @return
-       PTrue if at end of file.
+       true if at end of file.
      */
     PBoolean IsEndOfFile() const;
       
     /**Get information (eg protection, timestamps) on the specified file.
 
        @return
-       PTrue if the file info was retrieved.
+       true if the file info was retrieved.
      */
     static PBoolean GetInfo(
       const PFilePath & name,  // Name of file to get the information on.
       PFileInfo & info
-      // #PFileInfo# structure to receive the information.
+      // <code>PFileInfo</code> structure to receive the information.
     );
 
     /**Get information (eg protection, timestamps) on the current file.
 
        @return
-       PTrue if the file info was retrieved.
+       true if the file info was retrieved.
      */
     PBoolean GetInfo(
       PFileInfo & info
-      // #PFileInfo# structure to receive the information.
+      // <code>PFileInfo</code> structure to receive the information.
     );
 
     /**Set permissions on the specified file.
 
        @return
-       PTrue if the file was renamed.
+       true if the file was renamed.
      */
     static PBoolean SetPermissions(
       const PFilePath & name,   // Name of file to change the permission of.
@@ -530,7 +521,7 @@ class PFile : public PChannel
     /**Set permissions on the current file.
 
        @return
-       PTrue if the file was renamed.
+       true if the file was renamed.
      */
     PBoolean SetPermissions(
       int permissions           // New permissions mask for the file.
@@ -539,11 +530,9 @@ class PFile : public PChannel
 
   protected:
     // Member variables
-    /// The fully qualified path name for the file.
-    PFilePath path;
 
-    /// File is to be removed when closed.
-    PBoolean removeOnClose;
+    PFilePath path;         ///< The fully qualified path name for the file.
+    PBoolean removeOnClose; ///< File is to be removed when closed.
 
 
 // Include platform dependent part of class
