@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23897 $
+ * $Revision: 28339 $
  * $Author: rjongbloed $
- * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
+ * $Date: 2012-09-11 02:43:32 -0500 (Tue, 11 Sep 2012) $
  */
 
 #ifndef PTLIB_LISTS_H
@@ -62,7 +62,7 @@ struct PListInfo
 };
 
 /**This class is a collection of objects which are descendents of the
-   #PObject# class. It is implemeted as a doubly linked list.
+   <code>PObject</code> class. It is implemeted as a doubly linked list.
 
    The implementation of a list allows very fast inserting and deleting of
    objects in the collection, but has severe penalties for random access. All
@@ -73,13 +73,13 @@ struct PListInfo
    access via ordinal index is made sequentially there is little overhead.
 
    The PAbstractList class would very rarely be descended from directly by
-   the user. The #PDECLARE_LIST# and #PLIST# macros would normally
+   the user. The <code>PDECLARE_LIST</code> and <code>PLIST</code> macros would normally
    be used to create descendent classes. They will instantiate the template
-   based on #PList# or directly declare and define the class (using
+   based on <code>PList</code> or directly declare and define the class (using
    inline functions) if templates are not being used.
 
-   The #PList# class or #PDECLARE_LIST# macro will define the
-   correctly typed operators for subscript access (#operator[]#).
+   The <code>PList</code> class or <code>PDECLARE_LIST</code> macro will define the
+   correctly typed operators for subscript access (operator[]).
  */
 class PAbstractList : public PCollection
 {
@@ -105,25 +105,27 @@ class PAbstractList : public PCollection
 
        \arg \c LessThan if the instances object value at an
        ordinal position is less than the corresponding objects value in the
-       #obj# parameters list.
+       <code>obj</code> parameters list.
                           
        This is also returned if all objects are equal and the instances list
-       length is less than the #obj# parameters list length.
+       length is less than the <code>obj</code> parameters list length.
 
        \arg \c GreaterThan if the instances object value at an
        ordinal position is greater than the corresponding objects value in the
-       #obj# parameters list.
+       <code>obj</code> parameters list.
                           
        This is also returned if all objects are equal and the instances list
-       length is greater than the #obj# parameters list length.
+       length is greater than the <code>obj</code> parameters list length.
 
        @return
-       comparison of the two objects, #EqualTo# for same,
-       #LessThan# for #obj# logically less than the
-       object and #GreaterThan# for #obj# logically
+       comparison of the two objects, <code>EqualTo</code> for same,
+       <code>LessThan</code> for <code>obj</code> logically less than the
+       object and <code>GreaterThan</code> for <code>obj</code> logically
        greater than the object.
      */
-    virtual Comparison Compare(const PObject & obj) const;
+    virtual Comparison Compare(
+      const PObject & obj   ///< Object being compared to
+    ) const;
 
   /**@name Overrides from class PContainer */
   //@{
@@ -132,7 +134,7 @@ class PAbstractList : public PCollection
        set in any other way.
 
        @return
-       Always PTrue.
+       Always true.
      */
     virtual PBoolean SetSize(
       PINDEX newSize  ///< New size for the list, this is ignored.
@@ -151,13 +153,23 @@ class PAbstractList : public PCollection
       PObject * obj   ///< New object to place into the collection.
     );
 
+    /**Append a new object to the collection. This places a new link at the
+       "tail" of the list.
+    
+       @return
+       index of the newly added object.
+     */
+    virtual void Prepend(
+      PObject * obj   ///< New object to place into the collection.
+    );
+
     /**Insert a new object immediately before the specified object. If the
        object to insert before is not in the collection then the equivalent of
-       the #Append()# function is performed.
+       the <code>Append()</code> function is performed.
        
        Note that the object values are compared for the search of the
-       #before# parameter, not the pointers. So the objects in the
-       collection must correctly implement the #PObject::Compare()#
+       <code>before</code> parameter, not the pointers. So the objects in the
+       collection must correctly implement the <code>PObject::Compare()</code>
        function.
 
        @return
@@ -170,12 +182,12 @@ class PAbstractList : public PCollection
 
     /**Insert a new object at the specified ordinal index. If the index is
        greater than the number of objects in the collection then the
-       equivalent of the #Append()# function is performed.
+       equivalent of the <code>Append()</code> function is performed.
 
        @return
        index of the newly inserted object.
      */
-    virtual PINDEX InsertAt(
+    P_DEPRECATED virtual PINDEX InsertAt(
       PINDEX index,   ///< Index position in collection to place the object.
       PObject * obj   ///< New object to place into the collection.
     );
@@ -184,12 +196,31 @@ class PAbstractList : public PCollection
        is set then the object is also deleted.
 
        @return
-       PTrue if the object was in the collection.
+       true if the object was in the collection.
      */
     virtual PBoolean Remove(
       const PObject * obj   ///< Existing object to remove from the collection.
     );
 
+    /**Remove the head object from the list. If the AllowDeleteObjects option
+       is set then the object is also deleted.
+
+       @return
+       pointer to the object being removed, or NULL if it was deleted.
+     */
+    virtual PObject * RemoveHead()
+      { return RemoveElement(info->head); }
+
+    /**Remove the tail object from the list. If the AllowDeleteObjects option
+       is set then the object is also deleted.
+
+       @return
+       pointer to the object being removed, or NULL if it was deleted.
+     */
+    virtual PObject * RemoveTail()
+      { return RemoveElement(info->tail); }
+
+
     /**Remove the object at the specified ordinal index from the collection.
        If the AllowDeleteObjects option is set then the object is also deleted.
 
@@ -199,22 +230,22 @@ class PAbstractList : public PCollection
        @return
        pointer to the object being removed, or NULL if it was deleted.
      */
-    virtual PObject * RemoveAt(
+    P_DEPRECATED virtual PObject * RemoveAt(
       PINDEX index   ///< Index position in collection to place the object.
     );
 
     /**Set the object at the specified ordinal position to the new value. This
        will overwrite the existing entry. 
        This method will NOT delete the old object independently of the 
-       AllowDeleteObjects option. Use #ReplaceAt()# instead.
+       AllowDeleteObjects option. Use <code>ReplaceAt()</code> instead.
 
        Note if the index is beyond the size of the collection then the
        function will assert.
 
        @return
-       PTrue if the object was successfully added.
+       true if the object was successfully added.
      */
-    virtual PBoolean SetAt(
+    P_DEPRECATED virtual PBoolean SetAt(
       PINDEX index,   ///< Index position in collection to set.
       PObject * val   ///< New value to place into the collection.
     );
@@ -227,9 +258,9 @@ class PAbstractList : public PCollection
        function will assert.
        
        @return
-       PTrue if the object was successfully replaced.
+       true if the object was successfully replaced.
      */   
-    virtual PBoolean ReplaceAt(
+    P_DEPRECATED virtual PBoolean ReplaceAt(
       PINDEX index,   ///< Index position in collection to set.
       PObject * val   ///< New value to place into the collection.
     );
@@ -244,7 +275,7 @@ class PAbstractList : public PCollection
        @return
        pointer to object at the specified index.
      */
-    virtual PObject * GetAt(
+    P_DEPRECATED virtual PObject * GetAt(
       PINDEX index  ///< Index position in the collection of the object.
     ) const;
 
@@ -261,7 +292,7 @@ class PAbstractList : public PCollection
 
     /**Search the collection for the specified value of the object. The object
        values are compared, not the pointers.  So the objects in the
-       collection must correctly implement the #PObject::Compare()#
+       collection must correctly implement the <code>PObject::Compare()</code>
        function. A simple linear search from "head" of the list is performed.
 
        @return
@@ -274,34 +305,10 @@ class PAbstractList : public PCollection
 
 
   protected:
-    /**Get the object at the specified ordinal position. If the index was
-       greater than the size of the collection then this asserts.
-
-       The object accessed in this way is remembered by the class and further
-       access will be fast. Access to elements one either side of that saved
-       element, and the head and tail of the list, will always be fast.
-
-       @return
-       reference to object at the specified index.
-     */
-    PINLINE PObject & GetReferenceAt(
-      PINDEX index  ///< Ordinal index of the list element to set as current.
-    ) const;
-
-    /**Move the internal "cursor" to the index position specified. This
-       function will optimise the sequential move taking into account the
-       previous current position and the position at the head and tail of the
-       list. Whichever of these three points is closes is used as the starting
-       point for a sequential move to the required index.
-
-       @return
-       PTrue if the index could be set as the current element.
-     */
-    PBoolean SetCurrent(
-      PINDEX index,           ///< Ordinal index of the list element to set as current.
-      PListElement * & lastElement ///< pointer to final element
-    ) const;
-
+    PINLINE PObject & GetReferenceAt(PINDEX index) const;
+    PListElement * FindElement(PINDEX index) const;
+    PListElement * FindElement(const PObject & obj, PINDEX * index) const;
+    void InsertElement(PListElement * element, PObject * obj);
     PObject * RemoveElement(PListElement * element);
 
     // The types below cannot be nested as DevStudio 2005 AUTOEXP.DAT doesn't like it
@@ -313,7 +320,7 @@ class PAbstractList : public PCollection
 /**This template class maps the PAbstractList to a specific object type. The
    functions in this class primarily do all the appropriate casting of types.
 
-   Note that if templates are not used the #PDECLARE_LIST# macro will
+   Note that if templates are not used the <code>PDECLARE_LIST</code> macro will
    simulate the template instantiation.
  */
 template <class T> class PList : public PAbstractList
@@ -343,63 +350,67 @@ template <class T> class PList : public PAbstractList
 
   /**@name Iterators */
   //@{
-    class iterator_base : public std::iterator<std::bidirectional_iterator_tag, T> {
-    protected:
-      iterator_base(PListElement * e) : element(e) { }
-      PListElement * element;
+    typedef T value_type;
 
-      void Next() { element = PAssertNULL(element)->next; }
-      void Prev() { element = PAssertNULL(element)->prev; }
-      T * Ptr() const { return  (T *)PAssertNULL(element)->data; }
+    class iterator_base : public std::iterator<std::bidirectional_iterator_tag, value_type> {
+      protected:
+        iterator_base(PListElement * e) : element(e) { }
+        PListElement * element;
 
-    public:
-      bool operator==(const iterator_base & it) const { return element == it.element; }
-      bool operator!=(const iterator_base & it) const { return element != it.element; }
+        void Next() { this->element = PAssertNULL(this->element)->next; }
+        void Prev() { this->element = PAssertNULL(this->element)->prev; }
+        value_type * Ptr() const { return  dynamic_cast<value_type *>(PAssertNULL(this->element)->data); }
+
+      public:
+        bool operator==(const iterator_base & it) const { return this->element == it.element; }
+        bool operator!=(const iterator_base & it) const { return this->element != it.element; }
+
+      friend class PList<T>;
     };
 
     class iterator : public iterator_base {
-    public:
-      iterator(PListElement * e = NULL) : iterator_base(e) { }
+      public:
+        iterator(PListElement * e = NULL) : iterator_base(e) { }
 
-      iterator operator++()    {                      iterator_base::Next(); return *this; }
-      iterator operator--()    {                      iterator_base::Prev(); return *this; }
-      iterator operator++(int) { iterator it = *this; iterator_base::Next(); return it;    }
-      iterator operator--(int) { iterator it = *this; iterator_base::Prev(); return it;    }
+        iterator operator++()    {                      this->Next(); return *this; }
+        iterator operator--()    {                      this->Prev(); return *this; }
+        iterator operator++(int) { iterator it = *this; this->Next(); return it;    }
+        iterator operator--(int) { iterator it = *this; this->Prev(); return it;    }
 
-      T * operator->() const { return  iterator_base::Ptr(); }
-      T & operator* () const { return *iterator_base::Ptr(); }
+        value_type * operator->() const { return  this->Ptr(); }
+        value_type & operator* () const { return *this->Ptr(); }
     };
 
-    iterator begin()  { return info->head; }
+    iterator begin()  { return this->info->head; }
     iterator end()    { return iterator(); }
-    iterator rbegin() { return info->tail; }
+    iterator rbegin() { return this->info->tail; }
     iterator rend()   { return iterator(); }
-
+    iterator find(const value_type & obj) { return this->FindElement(obj, NULL); }
+    void insert(const iterator & pos, value_type * obj) { this->InsertElement(pos.element, obj); }
 
     class const_iterator : public iterator_base {
-    public:
-      const_iterator(PListElement * e = NULL) : iterator_base(e) { }
+      public:
+        const_iterator(PListElement * e = NULL) : iterator_base(e) { }
 
-      const_iterator operator++()    {                            iterator_base::Next(); return *this; }
-      const_iterator operator--()    {                            iterator_base::Prev(); return *this; }
-      const_iterator operator++(int) { const_iterator it = *this; iterator_base::Next(); return it;    }
-      const_iterator operator--(int) { const_iterator it = *this; iterator_base::Prev(); return it;    }
+        const_iterator operator++()    {                            this->Next(); return *this; }
+        const_iterator operator--()    {                            this->Prev(); return *this; }
+        const_iterator operator++(int) { const_iterator it = *this; this->Next(); return it;    }
+        const_iterator operator--(int) { const_iterator it = *this; this->Prev(); return it;    }
 
-      const T * operator->() const { return  iterator_base::Ptr(); }
-      const T & operator* () const { return *iterator_base::Ptr(); }
+        const value_type * operator->() const { return  this->Ptr(); }
+        const value_type & operator* () const { return *this->Ptr(); }
     };
 
-    const_iterator begin()  const { return info->head; }
+    const_iterator begin()  const { return this->info->head; }
     const_iterator end()    const { return const_iterator(); }
-    const_iterator rbegin() const { return info->tail; }
+    const_iterator rbegin() const { return this->info->tail; }
     const_iterator rend()   const { return iterator(); }
+    const_iterator find(const value_type & obj) const { return this->FindElement(obj, NULL); }
 
-    T & front() const { return *(T *)PAssertNULL(info->head)->data; }
-    T & back() const { return *(T *)PAssertNULL(info->tail)->data; }
-    void erase(const iterator & it) { Remove(&*it); }
-    void erase(const const_iterator & it) { Remove(&*it); }
-
-    typedef T value_type;
+    value_type & front() const { return dynamic_cast<value_type &>(*PAssertNULL(this->info->head)->data); }
+    value_type & back() const { return dynamic_cast<value_type &>(*PAssertNULL(this->info->tail)->data); }
+    void erase(const iterator & it) { this->RemoveElement(it.element); }
+    void erase(const const_iterator & it) { this->RemoveElement(it.element); }
   //@}
 
   /**@name New functions for class */
@@ -413,10 +424,11 @@ template <class T> class PList : public PAbstractList
        element, and the head and tail of the list, will always be fast.
 
        @return
-       reference to the object at #index# position.
+       reference to the object at <code>index</code> position.
      */
-    T & operator[](PINDEX index) const
-      { return (T &)GetReferenceAt(index); }
+    T & operator[](
+      PINDEX index  ///< Index for entry
+    ) const { return dynamic_cast<T &>(this->GetReferenceAt(index)); }
   //@}
 
   protected:
@@ -431,9 +443,9 @@ template <class T> class PList : public PAbstractList
    class declaration off so no additional members can be added.
 
    If the compilation is using templates then this macro produces a typedef
-   of the #PList# template class.
+   of the <code>PList</code> template class.
 
-   See the #PList# class and #PDECLARE_LIST# macro for more
+   See the <code>PList</code> class and <code>PDECLARE_LIST</code> macro for more
    information.
  */
 #define PLIST(cls, T) typedef PList<T> cls
@@ -443,11 +455,11 @@ template <class T> class PList : public PAbstractList
    customised for a particular object type <b>T</b>.
 
    If the compilation is using templates then this macro produces a descendent
-   of the #PList# template class. If templates are not being used then the
+   of the <code>PList</code> template class. If templates are not being used then the
    macro defines a set of inline functions to do all casting of types. The
    resultant classes have an identical set of functions in either case.
 
-   See the #PList# and #PAbstractList# classes for more information.
+   See the <code>PList</code> and <code>PAbstractList</code> classes for more information.
  */
 #define PDECLARE_LIST(cls, T) \
   PLIST(cls##_PTemplate, T); \
@@ -471,7 +483,7 @@ template <class T> class PList : public PAbstractList
    removed or when all references to the set are destroyed. This is different
    from the default on most collection classes.
 
-   Note that if templates are not used the #PDECLARE_QUEUE# macro will
+   Note that if templates are not used the <code>PDECLARE_QUEUE</code> macro will
    simulate the template instantiation.
  */
 template <class T> class PQueue : public PAbstractList
@@ -514,7 +526,7 @@ template <class T> class PQueue : public PAbstractList
        first object added to the queue or NULL if queue empty.
      */
     virtual T * Dequeue()
-      { if (GetSize() == 0) return NULL; else return (T *)PAbstractList::RemoveAt(0);}
+      { return dynamic_cast<T *>(PAbstractList::RemoveHead()); }
   //@}
 
   protected:
@@ -531,9 +543,9 @@ template <class T> class PQueue : public PAbstractList
    closes the class declaration off so no additional members can be added.
 
    If the compilation is using templates then this macro produces a typedef
-   of the #PQueue# template class.
+   of the <code>PQueue</code> template class.
 
-   See the #PList# class and #PDECLARE_QUEUE# macro for more
+   See the <code>PList</code> class and <code>PDECLARE_QUEUE</code> macro for more
    information.
  */
 #define PQUEUE(cls, T) typedef PQueue<T> cls
@@ -545,11 +557,11 @@ template <class T> class PQueue : public PAbstractList
    that allows the list to be used as a first in first out queue.
 
    If the compilation is using templates then this macro produces a descendent
-   of the #PQueue# template class. If templates are not being used then
+   of the <code>PQueue</code> template class. If templates are not being used then
    the macro defines a set of inline functions to do all casting of types. The
    resultant classes have an identical set of functions in either case.
 
-   See the #PQueue# and #PAbstractList# classes for more information.
+   See the <code>PQueue</code> and <code>PAbstractList</code> classes for more information.
  */
 #define PDECLARE_QUEUE(cls, T) \
   PQUEUE(cls##_PTemplate, T); \
@@ -573,7 +585,7 @@ template <class T> class PQueue : public PAbstractList
    removed or when all references to the set are destroyed. This is different
    from the default on most collection classes.
 
-   Note that if templates are not used the #PDECLARE_STACK# macro will
+   Note that if templates are not used the <code>PDECLARE_STACK</code> macro will
    simulate the template instantiation.
  */
 template <class T> class PStack : public PAbstractList
@@ -605,12 +617,12 @@ template <class T> class PStack : public PAbstractList
   /**@name New functions for class */
   //@{
     /**Add an object to the stack. This object will be on "top" of the stack
-       and will be the object returned by the #Pop()#
+       and will be the object returned by the <code>Pop()</code>
        function.
      */
     virtual void Push(
       T * obj    ///< Object to add to the stack.
-    ) { PAbstractList::InsertAt(0, obj); }
+    ) { PAbstractList::Prepend(obj); }
 
     /**Remove the last object pushed onto the stack.
 
@@ -618,7 +630,7 @@ template <class T> class PStack : public PAbstractList
        object on top of the stack.
      */
     virtual T * Pop()
-      { return (T *)PAbstractList::RemoveAt(0); }
+      { return dynamic_cast<T *>(PAbstractList::RemoveHead()); }
 
     /**Get the element that is currently on top of the stack without removing
        it.
@@ -627,13 +639,13 @@ template <class T> class PStack : public PAbstractList
        reference to object on top of the stack.
      */
     virtual T & Top()
-      { PAssert(GetSize() > 0, PStackEmpty); return *(T *)GetAt(0); }
+      { PAssert(this->GetSize() > 0, PStackEmpty); return dynamic_cast<T &>(*this->info->head->data); }
   //@}
 
   protected:
     PStack(int dummy, const PStack * c)
       : PAbstractList(dummy, c)
-      { reference->deleteObjects = c->reference->deleteObjects; }
+      { this->reference->deleteObjects = c->reference->deleteObjects; }
 };
 
 
@@ -644,9 +656,9 @@ template <class T> class PStack : public PAbstractList
    closes the class declaration off so no additional members can be added.
 
    If the compilation is using templates then this macro produces a typedef
-   of the #PStack# template class.
+   of the <code>PStack</code> template class.
 
-   See the #PStack# class and #PDECLARE_STACK# macro for more
+   See the <code>PStack</code> class and <code>PDECLARE_STACK</code> macro for more
    information.
  */
 #define PSTACK(cls, T) typedef PStack<T> cls
@@ -658,11 +670,11 @@ template <class T> class PStack : public PAbstractList
    that allows the list to be used as a last in first out stack.
 
    If the compilation is using templates then this macro produces a descendent
-   of the #PStack# template class. If templates are not being used then
+   of the <code>PStack</code> template class. If templates are not being used then
    the macro defines a set of inline functions to do all casting of types. The
    resultant classes have an identical set of functions in either case.
 
-   See the #PStack# and #PAbstractList# classes for more information.
+   See the <code>PStack</code> and <code>PAbstractList</code> classes for more information.
  */
 #define PDECLARE_STACK(cls, T) \
   PSTACK(cls##_PTemplate, T); \
@@ -682,38 +694,39 @@ template <class T> class PStack : public PAbstractList
 
 struct PSortedListElement
 {
-  PSortedListElement * parent;
-  PSortedListElement * left;
-  PSortedListElement * right;
-  PObject            * data;
-  PINDEX               subTreeSize;
-  enum { Red, Black }  colour;
+  PSortedListElement(PSortedListElement * nil = NULL, PObject * obj = NULL);
+
+  PSortedListElement * m_parent;
+  PSortedListElement * m_left;
+  PSortedListElement * m_right;
+  PObject            * m_data;
+  PINDEX               m_subTreeSize;
+  enum { Red, Black }  m_colour;
 
   PDECLARE_POOL_ALLOCATOR();
 };
 
 struct PSortedListInfo
 {
-  PSortedListInfo();
+  PSortedListInfo() : m_root(&nil) { }
 
-  PSortedListElement * root;
-  //PSortedListElement * lastElement;
-  //PINDEX               lastIndex;
   PSortedListElement   nil;
+  PSortedListElement * m_root;
 
-  PSortedListElement * Successor(const PSortedListElement * node) const;
-  PSortedListElement * Predecessor(const PSortedListElement * node) const;
+  PSortedListElement * Successor(PSortedListElement * node) const;
+  PSortedListElement * Predecessor(PSortedListElement * node) const;
   PSortedListElement * OrderSelect(PSortedListElement * node, PINDEX index) const;
-
-  typedef PSortedListElement Element;
+  PSortedListElement * OrderSelect(PINDEX index) const { return OrderSelect(m_root, index); }
+  PINDEX ValueSelect(PSortedListElement * node, const PObject & obj, PSortedListElement * & element) const;
+  PINDEX ValueSelect(const PObject & obj, PSortedListElement * & element) const { return ValueSelect(m_root, obj, element); }
 
   PDECLARE_POOL_ALLOCATOR();
 };
 
 /**This class is a collection of objects which are descendents of the
-   #PObject# class. It is implemeted as a Red-Black binary tree to
+   <code>PObject</code> class. It is implemeted as a Red-Black binary tree to
    maintain the objects in rank order. Note that this requires that the
-   #PObject::Compare()# function be fully implemented oin objects
+   <code>PObject::Compare()</code> function be fully implemented oin objects
    contained in the collection.
 
    The implementation of a sorted list allows fast inserting and deleting as
@@ -727,14 +740,14 @@ struct PSortedListInfo
    other access incurs a minimum overhead, but not insignificant.
 
    The PAbstractSortedList class would very rarely be descended from directly
-   by the user. The #PDECLARE_LIST# and #PLIST# macros would normally
+   by the user. The <code>PDECLARE_LIST</code> and <code>PLIST</code> macros would normally
    be used to create descendent classes. They will instantiate the template
-   based on #PSortedList# or directly declare and define the class (using
+   based on <code>PSortedList</code> or directly declare and define the class (using
    inline functions) if templates are not being used.
 
-   The #PSortedList# class or #PDECLARE_SORTED_LIST# macro will
+   The <code>PSortedList</code> class or <code>PDECLARE_SORTED_LIST</code> macro will
    define the correctly typed operators for subscript access
-   (#operator[]#).
+   (operator[]).
  */
 class PAbstractSortedList : public PCollection
 {
@@ -761,22 +774,22 @@ class PAbstractSortedList : public PCollection
 
        \arg \c LessThan if the instances object value at an
        ordinal position is less than the corresponding objects value in the
-       #obj# parameters list.
+       <code>obj</code> parameters list.
                           
        This is also returned if all objects are equal and the instances list
-       length is less than the #obj# parameters list length.
+       length is less than the <code>obj</code> parameters list length.
 
        \arg \c GreaterThan if the instances object value at an
        ordinal position is greater than the corresponding objects value in the
-       #obj# parameters list.
+       <code>obj</code> parameters list.
                           
        This is also returned if all objects are equal and the instances list
-       length is greater than the #obj# parameters list length.
+       length is greater than the <code>obj</code> parameters list length.
 
        @return
-       comparison of the two objects, #EqualTo# for same,
-       #LessThan# for #obj# logically less than the
-       object and #GreaterThan# for #obj# logically
+       comparison of the two objects, <code>EqualTo</code> for same,
+       <code>LessThan</code> for <code>obj</code> logically less than the
+       object and <code>GreaterThan</code> for <code>obj</code> logically
        greater than the object.
      */
     virtual Comparison Compare(const PObject & obj) const;
@@ -789,7 +802,7 @@ class PAbstractSortedList : public PCollection
        set in any other way.
 
        @return
-       Always PTrue.
+       Always true.
      */
     virtual PBoolean SetSize(
       PINDEX newSize  // New size for the sorted list, this is ignored.
@@ -811,7 +824,7 @@ class PAbstractSortedList : public PCollection
     /**Add a new object to the collection.
     
        The object is always placed in the correct ordinal position in the list.
-       It is not placed at the specified position. The #before#
+       It is not placed at the specified position. The <code>before</code>
        parameter is ignored.
 
        @return
@@ -825,7 +838,7 @@ class PAbstractSortedList : public PCollection
     /**Add a new object to the collection.
     
        The object is always placed in the correct ordinal position in the list.
-       It is not placed at the specified position. The #index#
+       It is not placed at the specified position. The <code>index</code>
        parameter is ignored.
 
        @return
@@ -844,7 +857,7 @@ class PAbstractSortedList : public PCollection
        same instance of the object that is in the collection.
 
        @return
-       PTrue if the object was in the collection.
+       true if the object was in the collection.
      */
     virtual PBoolean Remove(
       const PObject * obj   // Existing object to remove from the collection.
@@ -864,18 +877,18 @@ class PAbstractSortedList : public PCollection
     );
 
     /**Remove all of the elements in the collection. This operates by
-       continually calling #RemoveAt()# until there are no objects left.
+       continually calling <code>RemoveAt()</code> until there are no objects left.
 
        The objects are removed from the last, at index
-       #(GetSize()-1)# toward the first at index zero.
+       <code>(GetSize()-1)</code> toward the first at index zero.
      */
     virtual void RemoveAll();
 
-    /**This method simply returns PFalse as the list order is mantained by the 
-       class. Kept to mimic #PAbstractList# interface.
+    /**This method simply returns false as the list order is mantained by the 
+       class. Kept to mimic <code>PAbstractList</code> interface.
        
        @return
-       PFalse allways
+       false allways
      */
     virtual PBoolean SetAt(
       PINDEX index,   // Index position in collection to set.
@@ -906,14 +919,10 @@ class PAbstractSortedList : public PCollection
     virtual PINDEX GetObjectsIndex(
       const PObject * obj
     ) const;
-    virtual PINDEX GetObjectsIndex(
-      const PObject * obj,
-      PSortedListElement * & lastElement
-    ) const;
 
     /**Search the collection for the specified value of the object. The object
        values are compared, not the pointers.  So the objects in the
-       collection must correctly implement the #PObject::Compare()#
+       collection must correctly implement the <code>PObject::Compare()</code>
        function. A binary search is employed to locate the entry.
 
        @return
@@ -924,20 +933,18 @@ class PAbstractSortedList : public PCollection
     ) const;
   //@}
 
-    // The type below cannot be nested as DevStudio 2005 AUTOEXP.DAT doesn't like it
-    typedef PSortedListElement Element;
-
   protected:
-    
+
     // New functions for class
-    void RemoveElement(Element * node);
-    void LeftRotate(Element * node);
-    void RightRotate(Element * node);
-    void DeleteSubTrees(Element * node, PBoolean deleteObject);
-    PINDEX ValueSelect(const Element * node, const PObject & obj, const Element ** lastElement) const;
+    void RemoveElement(PSortedListElement * node);
+    void LeftRotate(PSortedListElement * node);
+    void RightRotate(PSortedListElement * node);
+    void DeleteSubTrees(PSortedListElement * node, bool deleteObject);
+    PSortedListElement * FindElement(const PObject & obj, PINDEX * index) const;
+    PSortedListElement * FindElement(const PObject * obj, PINDEX * index) const;
 
     // The type below cannot be nested as DevStudio 2005 AUTOEXP.DAT doesn't like it
-    PSortedListInfo * info;
+    PSortedListInfo * m_info;
 };
 
 
@@ -945,7 +952,7 @@ class PAbstractSortedList : public PCollection
    The functions in this class primarily do all the appropriate casting of
    types.
 
-   Note that if templates are not used the #PDECLARE_SORTED_LIST# macro
+   Note that if templates are not used the <code>PDECLARE_SORTED_LIST</code> macro
    will simulate the template instantiation.
  */
 template <class T> class PSortedList : public PAbstractSortedList
@@ -983,10 +990,101 @@ template <class T> class PSortedList : public PAbstractSortedList
        access will be fast.
 
        @return
-       reference to the object at #index# position.
+       reference to the object at <code>index</code> position.
      */
-    T & operator[](PINDEX index) const
-      { return *(T *)GetAt(index); }
+    T & operator[](
+      PINDEX index  ///< Index for entry
+    ) const { return dynamic_cast<T &>(*this->GetAt(index)); }
+  //@}
+
+  /**@name Iterators */
+  //@{
+    typedef T value_type;
+    friend class iterator_base;
+
+  private:
+    void NextElement(PSortedListElement * & element)
+    {
+      element = this->m_info->Successor(element);
+      if (element == &this->m_info->nil)
+        element = NULL;
+    }
+
+    void PrevElement(PSortedListElement * & element)
+    {
+      element = this->m_info->Predecessor(element);
+      if (element == &this->m_info->nil)
+        element = NULL;
+    }
+
+    class iterator_base : public std::iterator<std::bidirectional_iterator_tag, value_type> {
+      protected:
+        PSortedList<T>     * m_list;
+        PSortedListElement * m_element;
+
+        iterator_base(PSortedList<T> * l, PSortedListElement * e) : m_list(l), m_element(e) { }
+
+        bool Valid() const { return PAssert(this->m_list != NULL && this->m_element != NULL && this->m_element != &m_list->m_info->nil, PInvalidArrayIndex); }
+        void Next() { if (Valid()) this->m_list->NextElement(this->m_element); }
+        void Prev() { if (Valid()) this->m_list->PrevElement(this->m_element); }
+        value_type * Ptr() const { return dynamic_cast<value_type *>(Valid() ? this->m_element->m_data : NULL); }
+
+      public:
+        bool operator==(const iterator_base & it) const { return this->m_element == it.m_element; }
+        bool operator!=(const iterator_base & it) const { return this->m_element != it.m_element; }
+
+      friend class PSortedList<T>;
+    };
+
+  public:
+    class iterator : public iterator_base {
+      public:
+        iterator() : iterator_base(NULL, NULL) { }
+        iterator(PSortedList<T> * l, PSortedListElement * e) : iterator_base(l, e) { }
+
+        iterator operator++()    {                      this->Next(); return *this; }
+        iterator operator--()    {                      this->Prev(); return *this; }
+        iterator operator++(int) { iterator it = *this; this->Next(); return it;    }
+        iterator operator--(int) { iterator it = *this; this->Prev(); return it;    }
+
+        value_type * operator->() const { return  this->Ptr(); }
+        value_type & operator* () const { return *this->Ptr(); }
+    };
+
+    iterator begin()  { return IsEmpty() ? iterator() : iterator(this, this->m_info->OrderSelect(1));                 }
+    iterator end()    { return             iterator();                                                                }
+    iterator rbegin() { return IsEmpty() ? iterator() : iterator(this, this->m_info->OrderSelect(this->GetSize())); }
+    iterator rend()   { return             iterator();                                                                }
+
+    class const_iterator : public iterator_base {
+      public:
+        const_iterator() : iterator_base(NULL, NULL) { }
+        const_iterator(PSortedList<T> * l, PSortedListElement * e) : iterator_base(l, e) { }
+
+        const_iterator operator++()    {                            this->Next(); return *this; }
+        const_iterator operator--()    {                            this->Prev(); return *this; }
+        const_iterator operator++(int) { const_iterator it = *this; this->Next(); return it;    }
+        const_iterator operator--(int) { const_iterator it = *this; this->Prev(); return it;    }
+
+        const value_type * operator->() const { return  this->Ptr(); }
+        const value_type & operator* () const { return *this->Ptr(); }
+    };
+
+    const_iterator begin()  const { return IsEmpty() ? const_iterator() : const_iterator(this, this->m_info->OrderSelect(1));                 }
+    const_iterator end()    const { return             const_iterator();                                                                            }
+    const_iterator rbegin() const { return IsEmpty() ? const_iterator() : const_iterator(this, this->m_info->OrderSelect(this->GetSize())); }
+    const_iterator rend()   const { return             const_iterator();                                                                            }
+
+    value_type & front() { return *this->begin(); }
+    value_type & back()  { return *this->rbegin(); }
+    const value_type & front() const { return *this->begin(); }
+    const value_type & back()  const { return *this->rbegin(); }
+
+          iterator find(const value_type & obj)       { return       iterator(this, this->FindElement(obj, NULL));                      }
+    const_iterator find(const value_type & obj) const { return const_iterator(this, this->FindElement(obj, NULL)); }
+
+    void erase(const iterator & it)       { PAssert(this == it.m_list, PLogicError); this->RemoveElement(it.m_element); }
+    void erase(const const_iterator & it) { PAssert(this == it.m_list, PLogicError); this->RemoveElement(it.m_element); }
   //@}
 
   protected:
@@ -1001,9 +1099,9 @@ template <class T> class PSortedList : public PAbstractSortedList
    class declaration off so no additional members can be added.
 
    If the compilation is using templates then this macro produces a typedef
-   of the #PSortedList# template class.
+   of the <code>PSortedList</code> template class.
 
-   See the #PSortedList# class and #PDECLARE_SORTED_LIST# macro for
+   See the <code>PSortedList</code> class and <code>PDECLARE_SORTED_LIST</code> macro for
    more information.
  */
 #define PSORTED_LIST(cls, T) typedef PSortedList<T> cls
@@ -1014,11 +1112,11 @@ template <class T> class PSortedList : public PAbstractSortedList
    customised for a particular object type <b>T</b>.
 
    If the compilation is using templates then this macro produces a descendent
-   of the #PSortedList# template class. If templates are not being used
+   of the <code>PSortedList</code> template class. If templates are not being used
    then the macro defines a set of inline functions to do all casting of types.
    The resultant classes have an identical set of functions in either case.
 
-   See the #PSortedList# and #PAbstractSortedList# classes for more
+   See the <code>PSortedList</code> and <code>PAbstractSortedList</code> classes for more
    information.
  */
 #define PDECLARE_SORTED_LIST(cls, T) \
