@@ -26,16 +26,21 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Revision: 28935 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:12:14 -0600 (Fri, 18 Jan 2013) $
  */
 
 
 ///////////////////////////////////////////////////////////////////////////////
 // PChannel
 
-// nothing to do
+  protected:
+    virtual HANDLE GetAsyncReadHandle() const;
+    virtual HANDLE GetAsyncWriteHandle() const;
+
+   public:
+     int GetOSHandleAsInt() const { return PtrToLong((void*)os_handle); }
 
 
 // End Of File ///////////////////////////////////////////////////////////////
