@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28062 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
  */
 
 
@@ -39,6 +39,8 @@
     virtual PBoolean IsOpen() const;
   protected:
     PROCESS_INFORMATION info;
-    HANDLE hToChild, hFromChild, hStandardError;
+    PWin32Handle m_hToChild;
+    PWin32Handle m_hFromChild;
+    PWin32Handle m_hStandardError;
 
 // End Of File ///////////////////////////////////////////////////////////////
