@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21682 $
+ * $Revision: 28089 $
  * $Author: rjongbloed $
- * $Date: 2008-12-02 23:47:17 +0000 (Tue, 02 Dec 2008) $
+ * $Date: 2012-07-22 23:03:49 -0500 (Sun, 22 Jul 2012) $
  */
 
 
@@ -38,38 +38,16 @@
   public:
     ~PProcess();
 
-    void SignalTimerChange();
-    // Signal to the timer thread that a change was made.
-
     virtual PBoolean IsServiceProcess() const;
     virtual PBoolean IsGUIProcess() const;
     void WaitOnExitConsoleWindow();
 
   private:
-    PLIST(ThreadList, PThread);
-    ThreadList autoDeleteThreads;
-    PMutex deleteThreadMutex;
-
-    class HouseKeepingThread : public PThread
-    {
-      PCLASSINFO(HouseKeepingThread, PThread)
-        public:
-        HouseKeepingThread();
-        void Main();
-        PSyncPoint breakBlock;
-    };
-    HouseKeepingThread * houseKeeper;
-    // Thread for doing timers, thread clean up etc.
-
   friend PThread * PThread::Current();
-  friend void HouseKeepingThread::Main();
   friend UINT __stdcall PThread::MainFunction(void * thread);
   friend class PServiceProcess;
   friend class PApplication;
-#ifndef _WIN32_WCE
-  friend int PASCAL WinMain(HINSTANCE, HINSTANCE, LPSTR, int);
-#else
-  friend int PASCAL WinMain(HINSTANCE, HINSTANCE, LPTSTR, int);
-#endif
+
+  friend PDEFINE_WINMAIN(hInstance, hPrevInstance, lpCmdLine, nCmdShow);
 
 // End Of File ///////////////////////////////////////////////////////////////
