@@ -26,23 +26,22 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28062 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
 // PSemaphore
 
   public:
-    HANDLE GetHandle() const { return handle; }
-    unsigned GetInitialVal() const  { return initialVal; }
-    unsigned GetMaxCountVal() const { return maxCountVal; }
+    HANDLE GetHandle() const { return m_handle; }
+    unsigned GetInitialVal() const  { return m_initialVal; }
+    unsigned GetMaxCountVal() const { return m_maxCountVal; }
 
   protected:
-    PSemaphore(HANDLE h);
-    HANDLE handle;
-    unsigned initialVal;
-    unsigned maxCountVal;
+    unsigned     m_maxCountVal;
+    unsigned     m_initialVal;
+    PWin32Handle m_handle;
 
 // End Of File ///////////////////////////////////////////////////////////////
