@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28062 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
  */
 
 
@@ -36,6 +36,9 @@
 // PSyncPoint
 
   public:
-    virtual void Signal();
+    HANDLE GetHandle() const { return m_handle; }
+
+  protected:
+    PWin32Handle m_handle;
 
 // End Of File ///////////////////////////////////////////////////////////////
