@@ -26,27 +26,32 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20418 $
+ * $Revision: 28951 $
  * $Author: rjongbloed $
- * $Date: 2008-06-07 05:15:18 +0000 (Sat, 07 Jun 2008) $
+ * $Date: 2013-01-21 02:32:18 -0600 (Mon, 21 Jan 2013) $
  */
 
 
 ///////////////////////////////////////////////////////////////////////////////
 // PThread
 
+#define PTHREAD_ID_FMT "%u"
+
   public:
-    HANDLE GetHandle() const { return threadHandle; }
+    HANDLE GetHandle() const { return m_threadHandle; }
     void Win32AttachThreadInput();
 
   protected:
-    HANDLE threadHandle;
-    UINT   threadId;
+    PWin32Handle m_threadHandle;
 
   private:
-    PINDEX originalStackSize;
-
-    void CleanUp();
     static UINT __stdcall MainFunction(void * thread);
 
+#if defined(P_WIN_COM)
+  public:
+    bool CoInitialise();
+  private:
+    bool m_comInitialised;
+#endif
+  
 // End Of File ///////////////////////////////////////////////////////////////
