@@ -20,9 +20,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22722 $
+ * $Revision: 25555 $
  * $Author: rjongbloed $
- * $Date: 2009-05-26 06:59:58 -0500 (Tue, 26 May 2009) $
+ * $Date: 2011-04-13 03:52:01 -0500 (Wed, 13 Apr 2011) $
  */
 
 #include <ptbuildopts.h>
@@ -128,10 +128,6 @@ class PVideoInputDevice_Application : public PVideoInputDevice
       PINDEX * bytesReturned         ///< OPtional bytes returned.
     );
 
-    /**Try all known video formats & see which ones are accepted by the video driver
-     */
-    virtual PBoolean TestAllFormats();
-
     /**Set the Handle of the window you wish to capture
       */
     void AttachCaptureWindow(
