@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23897 $
+ * $Revision: 28062 $
  * $Author: rjongbloed $
- * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
+ * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
  */
 
 #ifndef PTLIB_MUTEX_H
@@ -44,7 +44,7 @@
 /**This class defines a thread mutual exclusion object. A mutex is where a
    piece of code or data cannot be accessed by more than one thread at a time.
    To prevent this the PMutex is used in the following manner:
-<code>
+<pre><code>
       PMutex mutex;
 
       ...
@@ -56,42 +56,59 @@
       mutex.Signal();
 
       ...
-</code>
-    The first thread will pass through the #Wait()# function, a second
+</code></pre>
+    The first thread will pass through the <code>Wait()</code> function, a second
     thread will block on that function until the first calls the
-    #Signal()# function, releasing the second thread.
+    <code>Signal()</code> function, releasing the second thread.
  */
 
-/*
- * On Windows, It is convenient for PTimedMutex to be an ancestor of PSemaphore
- * But that is the only platform where it is - every other platform (i.e. Unix)
- * uses different constructs for these objects, so there is no need for a PTimedMute
- * to carry around all of the PSemaphore members
- */
-
-#ifdef _WIN32
-class PTimedMutex : public PSemaphore
-{
-  PCLASSINFO(PTimedMutex, PSemaphore);
-#else
 class PTimedMutex : public PSync
 {
-  PCLASSINFO(PTimedMutex, PSync)
-#endif
-
+    PCLASSINFO(PTimedMutex, PSync)
   public:
     /* Create a new mutex.
        Initially the mutex will not be "set", so the first call to Wait() will
        never wait.
      */
     PTimedMutex();
+
+    /**Copy constructor is allowed but does not copy, allocating a new mutex.
+       Two copies of the same mutex information would be very bad.
+      */
     PTimedMutex(const PTimedMutex & mutex);
 
+    /**Assignment operator is allowed but does nothing.
+       Overwriting the old mutex information would be very bad.
+      */
+    PTimedMutex & operator=(const PTimedMutex &) { return *this; }
+
+    /**Block until the synchronisation object is available.
+     */
+    virtual void Wait();
+
+    /**Block, for a time, until the synchronisation object is available.
+
+       @return
+       true if lock is acquired, false if timed out
+     */
+    virtual PBoolean Wait(
+      const PTimeInterval & timeout // Amount of time to wait.
+    );
+
+    /**Signal that the synchronisation object is available.
+     */
+    virtual void Signal();
+
     /** Try to enter the critical section for exlusive access. Does not wait.
         @return true if cirical section entered, leave/Signal must be called.
       */
     PINLINE bool Try() { return Wait(0); }
 
+
+  private:
+    PThreadIdentifier m_lockerId;
+    PAtomicInteger    m_lockCount;
+
 // Include platform dependent part of class
 #ifdef _WIN32
 #include "msos/ptlib/mutex.h"
@@ -103,8 +120,14 @@ class PTimedMutex : public PSync
 // On Windows, critical sections are recursive and so we can use them for mutexes
 // The only Posix mutex that is recursive is pthread_mutex, so we have to use that
 #ifdef _WIN32
+/** \class PMutex
+    Synonym for PCriticalSection
+  */
 typedef PCriticalSection PMutex;
 #else
+/** \class PMutex
+    Synonym for PTimedMutex
+  */
 typedef PTimedMutex PMutex;
 #define	PCriticalSection PTimedMutex
 #endif
