@@ -7,9 +7,9 @@
  *
  * Contributor(s): Snark at GnomeMeeting
  *
- * $Revision: 21791 $
+ * $Revision: 26992 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 06:07:11 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2012-02-16 00:25:26 -0600 (Thu, 16 Feb 2012) $
  */
 
 #ifndef PTLIB_PLUGIN_H
@@ -22,36 +22,25 @@
 
 #include <ptlib/pfactory.h>
 
-template <class AbstractClass, typename KeyType = PString>
+template <class AbstractClass, typename KeyType = PDefaultPFactoryKey>
 class PDevicePluginFactory : public PFactory<AbstractClass, KeyType>
 {
   public:
-    class Worker : public PFactory<AbstractClass, KeyType>::WorkerBase 
+    typedef PFactory<AbstractClass, KeyType> Base_T;
+    typedef typename Base_T::WorkerBase WorkerBase_T;
+
+    class Worker : protected WorkerBase_T
     {
       public:
         Worker(const KeyType & key, bool singleton = false)
-          : PFactory<AbstractClass, KeyType>::WorkerBase(singleton)
+          : WorkerBase_T(singleton)
         {
-          PFactory<AbstractClass, KeyType>::Register(key, this);
+          Base_T::Register(key, this);
         }
 
         ~Worker()
         {
-          typedef typename PFactory<AbstractClass, KeyType>::WorkerBase WorkerBase_T;
-          typedef std::map<KeyType, WorkerBase_T *> KeyMap_T;
-          KeyType key;
-
-          KeyMap_T km = PFactory<AbstractClass, KeyType>::GetKeyMap();
-
-          typename KeyMap_T::const_iterator entry;
-          for (entry = km.begin(); entry != km.end(); ++entry) {
-            if (entry->second == this) {
-              key = entry->first;
-              break;
-            }
-          }
-          if (key != NULL)
-            PFactory<AbstractClass, KeyType>::Unregister(key);
+          Base_T::Unregister(this);
         }
 
       protected:
@@ -78,7 +67,7 @@ class PDevicePluginAdapter : public PDevicePluginAdapterBase
     void CreateFactory(const PString & device)
     {
       if (!(Factory_T::IsRegistered(device)))
-        new Worker_T(device, PFalse);
+        new Worker_T(device, false);
     }
 };
 
@@ -177,13 +166,13 @@ class PPlugin_##serviceType##_##serviceName##_Registration { \
 PCREATE_PLUGIN_REGISTERER(serviceName, serviceType, descriptor) \
 PPlugin_##serviceType##_##serviceName##_Registration \
   PPlugin_##serviceType##_##serviceName##_Registration_Instance(&PPluginManager::GetPluginManager()); \
+  int PPlugin_##serviceType##_##serviceName##_link() { return 0; }
 
-#define PWLIB_STATIC_LOAD_PLUGIN(serviceName, serviceType) \
-  class PPlugin_##serviceType##_##serviceName##_Registration; \
-  extern PPlugin_##serviceType##_##serviceName##_Registration PPlugin_##serviceType##_##serviceName##_Registration_Instance; \
-  static PPlugin_##serviceType##_##serviceName##_Registration * PPlugin_##serviceType##_##serviceName##_Registration_Static_Library_Loader = &PPlugin_##serviceType##_##serviceName##_Registration_Instance;
+#define PPLUGIN_STATIC_LOAD(serviceName, serviceType) \
+  extern int PPlugin_##serviceType##_##serviceName##_link(); \
+  int const PPlugin_##serviceType##_##serviceName##_loader = PPlugin_##serviceType##_##serviceName##_link();
 
-// Win32 only has static plugins at present, maybe one day ...
+// always define static plugins in Windows, since otherwise they seem not to work
 #ifndef P_FORCE_STATIC_PLUGIN
   #define P_FORCE_STATIC_PLUGIN 1
 #endif
@@ -194,15 +183,26 @@ PPlugin_##serviceType##_##serviceName##_Registration \
 #define PCREATE_PLUGIN_STATIC(serviceName, serviceType, descriptor) \
 static void __attribute__ (( constructor )) PWLIB_StaticLoader_##serviceName##_##serviceType() \
 { PPluginManager::GetPluginManager().RegisterService(#serviceName, #serviceType, descriptor); } \
+  int PPlugin_##serviceType##_##serviceName##_link() { return 0; }
 
 #else
 #define PCREATE_PLUGIN_STATIC(serviceName, serviceType, descriptor) \
 extern int PWLIB_gStaticLoader__##serviceName##_##serviceType; \
 static int PWLIB_StaticLoader_##serviceName##_##serviceType() \
 { PPluginManager::GetPluginManager().RegisterService(#serviceName, #serviceType, descriptor); return 1; } \
-int PWLIB_gStaticLoader__##serviceName##_##serviceType =  PWLIB_StaticLoader_##serviceName##_##serviceType(); 
+  int PWLIB_gStaticLoader__##serviceName##_##serviceType =  PWLIB_StaticLoader_##serviceName##_##serviceType(); \
+  int PPlugin_##serviceType##_##serviceName##_link() { return 0; }
+#endif
+
+#define PPLUGIN_STATIC_LOAD(serviceName, serviceType) \
+  extern int PPlugin_##serviceType##_##serviceName##_link(); \
+  int const PPlugin_##serviceType##_##serviceName##_loader = PPlugin_##serviceType##_##serviceName##_link();
+
+#ifndef P_SHAREDLIB
+#ifndef P_FORCE_STATIC_PLUGIN
+  #define P_FORCE_STATIC_PLUGIN 1
+#endif
 #endif
-#define PWLIB_STATIC_LOAD_PLUGIN(serviceName, serviceType) 
 
 #endif
 
