@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22165 $
+ * $Revision: 29812 $
  * $Author: rjongbloed $
- * $Date: 2009-03-06 00:24:25 +0000 (Fri, 06 Mar 2009) $
+ * $Date: 2013-05-27 19:43:21 -0500 (Mon, 27 May 2013) $
  */
  
 #ifndef PTLIB_SAFE_COLLECTION_H
@@ -36,6 +36,12 @@
 #endif
 
 
+#include <ptlib/syncthrd.h>
+
+
+class PTimer;
+
+
 /** This class defines a thread-safe object in a collection.
 
   This is part of a set of classes to solve the general problem of a
@@ -92,7 +98,29 @@
   automatic unlocking of objects ones the pointer goes out of scope. It may
   also be used to lock each object of a collection in turn.
 
-  The enumeration
+  The enumeration of a PSafeCollection of PSafeObjects utilises the PSafePtr
+  class in a classic "for loop" manner.
+
+  <CODE>
+    for (PSafePtr<MyClass> iter(collection, PSafeReadWrite); iter != NULL; ++iter)
+      iter->Process();
+  </CODE>
+
+  There is one piece if important behaviour in the above. If while enumerating
+  a specic object in the collection, that object is "safely deleted", you are
+  guaranteed that the object is still usable and has not been phsyically
+  deleted, however it will no longer be in the collection, so the enumeration
+  will stop as it can no longer determine where in the collection it was.
+
+  What to do in this case is to take a "snapshot" at a point of time that can be safely and completely
+  iterated over:
+
+  <CODE>
+    PSafeList<MyClass> collCopy = collection;
+    for (PSafePtr<MyClass> iter(callCopy, PSafeReadWrite); iter != NULL; ++iter)
+      iter->Process();
+  </CODE>
+
  */
 class PSafeObject : public PObject
 {
@@ -114,7 +142,7 @@ class PSafeObject : public PObject
        deallocated) as the caller thread is using the object, but not
        necessarily at this time locking it.
 
-       If the function returns PFalse, then the object has been flagged for
+       If the function returns false, then the object has been flagged for
        deletion and the calling thread should immediately cease using the
        object.
 
@@ -136,7 +164,7 @@ class PSafeObject : public PObject
        It is recommended that the PSafePtr<> class is used to manage this
        rather than the application calling this function directly.
 
-       @return PTrue if reference count has reached zero and is not being
+       @return true if reference count has reached zero and is not being
                safely deleted elsewhere ie SafeRemove() not called
       */
     PBoolean SafeDereference();
@@ -148,7 +176,7 @@ class PSafeObject : public PObject
        occur and no read/write lock can be present for any read only locks to
        occur.
 
-       If the function returns PFalse, then the object has been flagged for
+       If the function returns false, then the object has been flagged for
        deletion and the calling thread should immediately cease use of the
        object, possibly executing the SafeDereference() function to remove
        any references it may have acquired.
@@ -179,7 +207,7 @@ class PSafeObject : public PObject
        occur and no read/write lock can be present for any read only locks to
        occur.
 
-       If the function returns PFalse, then the object has been flagged for
+       If the function returns false, then the object has been flagged for
        deletion and the calling thread should immediately cease use of the
        object, possibly executing the SafeDereference() function to remove
        any references it may have acquired.
@@ -235,6 +263,12 @@ class PSafeObject : public PObject
        @return true if object may be deleted.
       */
     virtual bool GarbageCollection();
+
+    /**Get count of references to this object.
+       Note this returns the value outside of any mutexes, so it could change
+       at any moment. Care must be exercised in its use.
+      */
+    unsigned GetSafeReferenceCount() const { return safeReferenceCount; }
   //@}
 
   private:
@@ -345,10 +379,20 @@ class PSafeCollection : public PObject
     );
 
   public:
+    /** Output the contents of the object to the stream. The exact output is
+       dependent on the exact semantics of the descendent class. This is
+       primarily used by the standard <code>#operator<<</code> function.
+
+       The default behaviour is to print the class name.
+     */
+    virtual void PrintOn(
+      ostream &strm   // Stream to print the object into.
+    ) const;
+
     /**Remove all objects in collection.
       */
     virtual void RemoveAll(
-      PBoolean synchronous = PFalse  ///< Wait till objects are deleted before returning
+      PBoolean synchronous = false  ///< Wait till objects are deleted before returning
     );
 
     /**Disallow the automatic delete any objects that have been removed.
@@ -356,17 +400,17 @@ class PSafeCollection : public PObject
        deletion using PSafeObject::SafeRemove() and DeleteObject().
       */
     void AllowDeleteObjects(
-      PBoolean yes = PTrue   ///< New value for flag for deleting objects
+      PBoolean yes = true   ///< New value for flag for deleting objects
     ) { deleteObjects = yes; }
 
     /**Disallow the automatic delete any objects that have been removed.
        Objects are simply removed from the collection and not marked for
        deletion using PSafeObject::SafeRemove() and DeleteObject().
       */
-    void DisallowDeleteObjects() { deleteObjects = PFalse; }
+    void DisallowDeleteObjects() { deleteObjects = false; }
 
     /**Delete any objects that have been removed.
-       Returns PTrue if all objects in the collection have been removed and
+       Returns true if all objects in the collection have been removed and
        their pending deletions carried out.
       */
     virtual PBoolean DeleteObjectsToBeRemoved();
@@ -397,15 +441,22 @@ class PSafeCollection : public PObject
   //@}
 
   protected:
+    void CopySafeCollection(PCollection * other);
+    void CopySafeDictionary(PAbstractDictionary * other);
+    bool SafeAddObject(PSafeObject * obj, PSafeObject * old);
     void SafeRemoveObject(PSafeObject * obj);
     PDECLARE_NOTIFIER(PTimer, PSafeCollection, DeleteObjectsTimeout);
 
-    PCollection  *     collection;
+    PCollection      * collection;
     mutable PMutex     collectionMutex;
-    PBoolean               deleteObjects;
+    bool               deleteObjects;
     PList<PSafeObject> toBeRemoved;
     PMutex             removalMutex;
-    PTimer             deleteObjectsTimer;
+    PTimer           * m_deleteObjectsTimer;
+
+  private:
+    PSafeCollection(const PSafeCollection & other) : PObject(other) { }
+    void operator=(const PSafeCollection &) { }
 
   friend class PSafePtrBase;
 };
@@ -426,9 +477,6 @@ enum PSafetyMode {
 
   NOTE: the PSafePtr will allow safe and mutexed access to objects but is not
   thread safe itself! You should not share PSafePtr instances across threads.
-  You can assign a PSafePtr to another instance across a thread boundary
-  provided it is on a reference and no read only or read/write locks are
-  present.
 
   See the PSafeObject class for more details.
  */
@@ -498,21 +546,40 @@ class PSafePtrBase : public PObject
        Note this is not a value comparison and will only return EqualTo if the
        two PSafePtrBase instances are pointing to the same instance.
       */
-    Comparison Compare(
+    virtual Comparison Compare(
       const PObject & obj   ///< Other instance to compare against
     ) const;
+
+    /** Output the contents of the object to the stream. The exact output is
+       dependent on the exact semantics of the descendent class. This is
+       primarily used by the standard <code>#operator<<</code> function.
+
+       The default behaviour is to print the class name.
+     */
+    virtual void PrintOn(
+      ostream &strm   // Stream to print the object into.
+    ) const;
   //@}
 
   /**@name Operations */
   //@{
     /**Set the pointer to NULL, unlocking/dereferencing existing pointer value.
       */
-    void SetNULL();
+    virtual void SetNULL();
 
-    /**Return PTrue if pointer is NULL.
+    /**Return true if pointer is NULL.
       */
     bool operator!() const { return currentObject == NULL; }
 
+    /**Return pointer to safe object.
+      */
+    PSafeObject * GetObject() const { return currentObject; }
+
+    /**Return pointer to safe object.
+      */
+    template <class T>
+    T * GetObjectAs() const { return dynamic_cast<T *>(currentObject); }
+
     /**Get the locking mode used by this pointer.
       */
     PSafetyMode GetSafetyMode() const { return lockMode; }
@@ -523,23 +590,20 @@ class PSafePtrBase : public PObject
        deletion and the calling thread should immediately cease use of the
        object. This instance pointer will be set to NULL.
       */
-    PBoolean SetSafetyMode(
+    virtual PBoolean SetSafetyMode(
       PSafetyMode mode  ///< New locking mode
     );
-
-    /**Get the associated collection this pointer may be contained in.
-      */
-    const PSafeCollection * GetCollection() const { return collection; }
   //@}
 
-    void Assign(const PSafePtrBase & ptr);
-    void Assign(const PSafeCollection & safeCollection);
-    void Assign(PSafeObject * obj);
-    void Assign(PINDEX idx);
+    virtual void Assign(const PSafePtrBase & ptr);
+    virtual void Assign(const PSafeCollection & safeCollection);
+    virtual void Assign(PSafeObject * obj);
+    virtual void Assign(PINDEX idx);
 
   protected:
-    void Next();
-    void Previous();
+    virtual void Next();
+    virtual void Previous();
+    virtual void DeleteObject(PSafeObject * obj);
 
     enum EnterSafetyModeOption {
       WithReference,
@@ -553,6 +617,9 @@ class PSafePtrBase : public PObject
     };
     void ExitSafetyMode(ExitSafetyModeOption ref);
 
+    virtual void LockPtr() { }
+    virtual void UnlockPtr() { }
+
   protected:
     const PSafeCollection * collection;
     PSafeObject           * currentObject;
@@ -560,6 +627,126 @@ class PSafePtrBase : public PObject
 };
 
 
+/** This class defines a base class for thread-safe pointer to an object.
+
+  This is part of a set of classes to solve the general problem of a
+  collection (eg a PList or PDictionary) of objects that needs to be a made
+  thread safe. Any thread can add, read, write or remove an object with both
+  the object and the database of objects itself kept thread safe.
+
+  NOTE: unlikel PSafePtrBase, pointers based on this class are thread safe
+  themseleves, at the expense of performance on every operation.
+
+  See the PSafeObject class for more details.
+ */
+class PSafePtrMultiThreaded : public PSafePtrBase
+{
+    PCLASSINFO(PSafePtrMultiThreaded, PSafePtrBase);
+
+  /**@name Construction */
+  //@{
+  protected:
+    /**Create a new pointer to a PSafeObject.
+       An optional locking mode may be provided to lock the object for reading
+       or writing and automatically unlock it on destruction.
+
+       Note that this version is not associated with a collection so the ++
+       and -- operators will not work.
+     */
+    PSafePtrMultiThreaded(
+      PSafeObject * obj = NULL,         ///< Physical object to point to.
+      PSafetyMode mode = PSafeReference ///< Locking mode for the object
+    );
+
+    /**Create a new pointer to a PSafeObject.
+       An optional locking mode may be provided to lock the object for reading
+       or writing and automatically unlock it on destruction.
+
+       The idx'th entry of the collection is pointed to by this object. If the
+       idx is beyond the size of the collection, the pointer is NULL.
+     */
+    PSafePtrMultiThreaded(
+      const PSafeCollection & safeCollection, ///< Collection pointer will enumerate
+      PSafetyMode mode,                       ///< Locking mode for the object
+      PINDEX idx                              ///< Index into collection to point to
+    );
+
+    /**Create a new pointer to a PSafeObject.
+       An optional locking mode may be provided to lock the object for reading
+       or writing and automatically unlock it on destruction.
+
+       The obj parameter is only set if it contained in the collection,
+       otherwise the pointer is NULL.
+     */
+    PSafePtrMultiThreaded(
+      const PSafeCollection & safeCollection, ///< Collection pointer will enumerate
+      PSafetyMode mode,                       ///< Locking mode for the object
+      PSafeObject * obj                       ///< Inital object in collection to point to
+    );
+
+    /**Copy the pointer to the PSafeObject.
+       This will create a copy of the pointer with the same locking mode and
+       lock on the PSafeObject. It will also increment the reference count on
+       the PSafeObject as well.
+      */
+    PSafePtrMultiThreaded(
+      const PSafePtrMultiThreaded & enumerator   ///< Pointer to copy
+    );
+
+  public:
+    /**Unlock and dereference the PSafeObject this is pointing to.
+      */
+    ~PSafePtrMultiThreaded();
+  //@}
+
+  /**@name Overrides from class PObject */
+  //@{
+    /**Compare the pointers.
+       Note this is not a value comparison and will only return EqualTo if the
+       two PSafePtrBase instances are pointing to the same instance.
+      */
+    virtual Comparison Compare(
+      const PObject & obj   ///< Other instance to compare against
+    ) const;
+  //@}
+
+  /**@name Operations */
+  //@{
+    /**Set the pointer to NULL, unlocking/dereferencing existing pointer value.
+      */
+    virtual void SetNULL();
+
+    /**Change the locking mode used by this pointer.
+
+       If the function returns false, then the object has been flagged for
+       deletion and the calling thread should immediately cease use of the
+       object. This instance pointer will be set to NULL.
+      */
+    virtual PBoolean SetSafetyMode(
+      PSafetyMode mode  ///< New locking mode
+    );
+  //@}
+
+    virtual void Assign(const PSafePtrMultiThreaded & ptr);
+    virtual void Assign(const PSafePtrBase & ptr);
+    virtual void Assign(const PSafeCollection & safeCollection);
+    virtual void Assign(PSafeObject * obj);
+    virtual void Assign(PINDEX idx);
+
+  protected:
+    virtual void Next();
+    virtual void Previous();
+    virtual void DeleteObject(PSafeObject * obj);
+
+    virtual void LockPtr() { m_mutex.Wait(); }
+    virtual void UnlockPtr();
+
+  protected:
+    mutable PMutex m_mutex;
+    PSafeObject  * m_objectToDelete;
+};
+
+
 /** This class defines a thread-safe enumeration of object in a collection.
 
   This is part of a set of classes to solve the general problem of a
@@ -567,23 +754,21 @@ class PSafePtrBase : public PObject
   thread safe. Any thread can add, read, write or remove an object with both
   the object and the database of objects itself kept thread safe.
 
-  There are two modes of safe pointer, one that is enumerating a collection
+  There are two modes of safe pointer: one that is enumerating a collection;
   and one that is independent of the collection that the safe object is in.
-  There are some subtle semantics that must be observed in each of these two
+  There are subtle semantics that must be observed in each of these two
   modes especially when switching from one to the other.
 
-  NOTE: the PSafePtr will allow safe and mutexed access to objects but is not
-  thread safe itself! You should not share PSafePtr instances across threads.
-  You can assign a PSafePtr to another instance across a thread boundary
-  provided it is on a reference and no read only or read/write locks are
-  present.
+  NOTE: the PSafePtr will allow safe and mutexed access to objects but may not
+  be thread safe itself! This depends on the base class being used. If the more
+  efficient PSafePtrBase class is used you should not share PSafePtr instances
+  across threads.
 
-  See the PSafeObject class for more details. Especially in regard to
+  See the PSafeObject class for more details, especially in regards to
   enumeration of collections.
  */
-template <class T> class PSafePtr : public PSafePtrBase
+template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseClass
 {
-    PCLASSINFO(PSafePtr, PSafePtrBase);
   public:
   /**@name Construction */
   //@{
@@ -597,7 +782,7 @@ template <class T> class PSafePtr : public PSafePtrBase
     PSafePtr(
       T * obj = NULL,                   ///< Physical object to point to.
       PSafetyMode mode = PSafeReference ///< Locking mode for the object
-    ) : PSafePtrBase(obj, mode) { }
+    ) : BaseClass(obj, mode) { }
 
     /**Create a new pointer to a PSafeObject.
        An optional locking mode may be provided to lock the object for reading
@@ -610,7 +795,7 @@ template <class T> class PSafePtr : public PSafePtrBase
       const PSafeCollection & safeCollection, ///< Collection pointer will enumerate
       PSafetyMode mode = PSafeReadWrite,      ///< Locking mode for the object
       PINDEX idx = 0                          ///< Index into collection to point to
-    ) : PSafePtrBase(safeCollection, mode, idx) { }
+    ) : BaseClass(safeCollection, mode, idx) { }
 
     /**Create a new pointer to a PSafeObject.
        An optional locking mode may be provided to lock the object for reading
@@ -623,7 +808,7 @@ template <class T> class PSafePtr : public PSafePtrBase
       const PSafeCollection & safeCollection, ///< Collection pointer will enumerate
       PSafetyMode mode,                       ///< Locking mode for the object
       PSafeObject * obj                       ///< Inital object in collection to point to
-    ) : PSafePtrBase(safeCollection, mode, obj) { }
+    ) : BaseClass(safeCollection, mode, obj) { }
 
     /**Copy the pointer to the PSafeObject.
        This will create a copy of the pointer with the same locking mode and
@@ -632,7 +817,7 @@ template <class T> class PSafePtr : public PSafePtrBase
       */
     PSafePtr(
       const PSafePtr & ptr   ///< Pointer to copy
-    ) : PSafePtrBase(ptr) { }
+    ) : BaseClass(ptr) { }
 
     /**Copy the pointer to the PSafeObject.
        This will create a copy of the pointer with the same locking mode and
@@ -641,7 +826,7 @@ template <class T> class PSafePtr : public PSafePtrBase
       */
     PSafePtr & operator=(const PSafePtr & ptr)
       {
-        Assign(ptr);
+        BaseClass::Assign(ptr);
         return *this;
       }
 
@@ -651,7 +836,7 @@ template <class T> class PSafePtr : public PSafePtrBase
       */
     PSafePtr & operator=(const PSafeCollection & safeCollection)
       {
-        Assign(safeCollection);
+        BaseClass::Assign(safeCollection);
         return *this;
       }
 
@@ -672,7 +857,7 @@ template <class T> class PSafePtr : public PSafePtrBase
      */
     PSafePtr & operator=(T * obj)
       {
-        Assign(obj);
+        this->Assign(obj);
         return *this;
       }
 
@@ -687,24 +872,37 @@ template <class T> class PSafePtr : public PSafePtrBase
      */
     PSafePtr & operator=(PINDEX idx)
       {
-        Assign(idx);
+        BaseClass::Assign(idx);
         return *this;
       }
+
+    /**Set the safe pointer to the specified object.
+       This will return a PSafePtr for the previous value of this. It does
+       this in a thread safe manner so "test and set" semantics are obeyed.
+      */
+    PSafePtr Set(T * obj)
+      {
+        this->LockPtr();
+        PSafePtr oldPtr = *this;
+        this->Assign(obj);
+        this->UnlockPtr();
+        return oldPtr;
+      }
   //@}
 
   /**@name Operations */
   //@{
     /**Return the physical pointer to the object.
       */
-    operator T*()    const { return  (T *)currentObject; }
+    operator T*()    const { return  dynamic_cast<T *>(BaseClass::currentObject); }
 
     /**Return the physical pointer to the object.
       */
-    T & operator*()  const { return *(T *)PAssertNULL(currentObject); }
+    T & operator*()  const { return *dynamic_cast<T *>(PAssertNULL(BaseClass::currentObject)); }
 
     /**Allow access to the physical object the pointer is pointing to.
       */
-    T * operator->() const { return  (T *)PAssertNULL(currentObject); }
+    T * operator->() const { return  dynamic_cast<T *>(PAssertNULL(BaseClass::currentObject)); }
 
     /**Post-increment the pointer.
        This requires that the pointer has been created with a PSafeCollection
@@ -712,8 +910,8 @@ template <class T> class PSafePtr : public PSafePtrBase
       */
     T * operator++(int)
       {
-        T * previous = (T *)currentObject;
-        Next();
+        T * previous = dynamic_cast<T *>(BaseClass::currentObject);
+        BaseClass::Next();
         return previous;
       }
 
@@ -723,8 +921,8 @@ template <class T> class PSafePtr : public PSafePtrBase
       */
     T * operator++()
       {
-        Next();
-        return (T *)currentObject;
+        BaseClass::Next();
+        return dynamic_cast<T *>(BaseClass::currentObject);
       }
 
     /**Post-decrement the pointer.
@@ -733,8 +931,8 @@ template <class T> class PSafePtr : public PSafePtrBase
       */
     T * operator--(int)
       {
-        T * previous = (T *)currentObject;
-        Previous();
+        T * previous = dynamic_cast<T *>(BaseClass::currentObject);
+        BaseClass::Previous();
         return previous;
       }
 
@@ -744,25 +942,10 @@ template <class T> class PSafePtr : public PSafePtrBase
       */
     T * operator--()
       {
-        Previous();
-        return (T *)currentObject;
+        BaseClass::Previous();
+        return dynamic_cast<T *>(BaseClass::currentObject);
       }
   //@}
-
-  /**Cast the pointer to a different type. The pointer being cast to MUST
-     be a derived class or NULL is returned.
-    */
-      /*
-  template <class Base>
-  static PSafePtr<T> DownCast(const PSafePtr<Base> & oldPtr)
-  {
-    PSafePtr<T> newPtr;
-    Base * realPtr = oldPtr;
-    if (realPtr != NULL && PIsDescendant(realPtr, T))
-      newPtr.Assign(oldPtr);
-    return newPtr;
-  }
-  */
 };
 
 
@@ -793,7 +976,7 @@ PSafePtr<Derived> PSafePtrCast(const PSafePtr<Base> & oldPtr)
  */
 template <class Coll, class Base> class PSafeColl : public PSafeCollection
 {
-    PCLASSINFO(PSafeColl, PSafeCollection);
+    PCLASSINFO_WITH_CLONE(PSafeColl, PSafeCollection);
   public:
   /**@name Construction */
   //@{
@@ -802,6 +985,30 @@ template <class Coll, class Base> class PSafeColl : public PSafeCollection
     PSafeColl()
       : PSafeCollection(new Coll)
       { }
+
+    /**Copy constructor for safe collection.
+       Note the left hand side will always have DisallowDeleteObjects() set.
+      */
+    PSafeColl(const PSafeColl & other)
+      : PSafeCollection(new Coll)
+    {
+      PWaitAndSignal lock2(other.collectionMutex);
+      CopySafeCollection(dynamic_cast<Coll *>(other.collection));
+    }
+
+    /**Assign one safe collection to another.
+       Note the left hand side will always have DisallowDeleteObjects() set.
+      */
+    PSafeColl & operator=(const PSafeColl & other)
+    {
+      if (&other != this) {
+        RemoveAll(true);
+        PWaitAndSignal lock1(collectionMutex);
+        PWaitAndSignal lock2(other.collectionMutex);
+        CopySafeCollection(dynamic_cast<Coll *>(other.collection));
+      }
+      return *this;
+    }
   //@}
 
   /**@name Operations */
@@ -812,12 +1019,12 @@ template <class Coll, class Base> class PSafeColl : public PSafeCollection
       */
     virtual PSafePtr<Base> Append(
       Base * obj,       ///< Object to add to safe collection.
-      PSafetyMode mode = PSafeReference
+      PSafetyMode mode = PSafeReference   ///< Safety mode for returned locked PSafePtr
     ) {
         PWaitAndSignal mutex(collectionMutex);
-        if (!obj->SafeReference())
-          return NULL;
-        return PSafePtr<Base>(*this, mode, collection->Append(obj));
+        if (SafeAddObject(obj, NULL))
+          return PSafePtr<Base>(*this, mode, collection->Append(obj));
+        return NULL;
       }
 
     /**Remove an object to the collection.
@@ -924,7 +1131,7 @@ template <class Base> class PSafeSortedList : public PSafeColl<PSortedList<Base>
  */
 template <class Coll, class Key, class Base> class PSafeDictionaryBase : public PSafeCollection
 {
-    PCLASSINFO(PSafeDictionaryBase, PSafeCollection);
+    PCLASSINFO_WITH_CLONE(PSafeDictionaryBase, PSafeCollection);
   public:
   /**@name Construction */
   //@{
@@ -932,6 +1139,30 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
       */
     PSafeDictionaryBase()
       : PSafeCollection(new Coll) { }
+
+    /**Copy constructor for safe collection.
+       Note the left hand side will always have DisallowDeleteObjects() set.
+      */
+    PSafeDictionaryBase(const PSafeDictionaryBase & other)
+      : PSafeCollection(new Coll)
+    {
+      PWaitAndSignal lock2(other.collectionMutex);
+      CopySafeDictionary(dynamic_cast<Coll *>(other.collection));
+    }
+
+    /**Assign one safe collection to another.
+       Note the left hand side will always have DisallowDeleteObjects() set.
+      */
+    PSafeDictionaryBase & operator=(const PSafeDictionaryBase & other)
+    {
+      if (&other != this) {
+        RemoveAll(true);
+        PWaitAndSignal lock1(collectionMutex);
+        PWaitAndSignal lock2(other.collectionMutex);
+        CopySafeDictionary(dynamic_cast<Coll *>(other.collection));
+      }
+      return *this;
+    }
   //@}
 
   /**@name Operations */
@@ -943,9 +1174,8 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
     virtual void SetAt(const Key & key, Base * obj)
       {
         collectionMutex.Wait();
-        SafeRemove(((Coll *)collection)->GetAt(key));
-        if (obj->SafeReference())
-          ((Coll *)collection)->SetAt(key, obj);
+        if (SafeAddObject(obj, dynamic_cast<Coll &>(*collection).GetAt(key)))
+          dynamic_cast<Coll &>(*collection).SetAt(key, obj);
         collectionMutex.Signal();
       }
 
@@ -961,7 +1191,7 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
       const Key & key   ///< Key to fund object to delete
     ) {
         PWaitAndSignal mutex(collectionMutex);
-        return SafeRemove(((Coll *)collection)->GetAt(key));
+        return SafeRemove(dynamic_cast<Coll &>(*collection).GetAt(key));
       }
 
     /**Determine of the dictionary contains an entry for the key.
@@ -970,7 +1200,7 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
       const Key & key
     ) {
         PWaitAndSignal lock(collectionMutex);
-        return ((Coll *)collection)->Contains(key);
+        return dynamic_cast<Coll &>(*collection).Contains(key);
       }
 
     /**Get the instance in the collection of the index.
@@ -993,13 +1223,24 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
     virtual PSafePtr<Base> FindWithLock(
       const Key & key,
       PSafetyMode mode = PSafeReadWrite
-    ) {
+    ) const {
         collectionMutex.Wait();
-        PSafePtr<Base> ptr(*this, PSafeReference, ((Coll *)collection)->GetAt(key));
+        PSafePtr<Base> ptr(*this, PSafeReference, dynamic_cast<Coll &>(*collection).GetAt(key));
         collectionMutex.Signal();
         ptr.SetSafetyMode(mode);
         return ptr;
       }
+
+    /**Get an array containing all the keys for the dictionary.
+      */
+    PArray<Key> GetKeys() const
+    {
+      PArray<Key> keys;
+      collectionMutex.Wait();
+      dynamic_cast<Coll &>(*collection).AbstractGetKeys(keys);
+      collectionMutex.Signal();
+      return keys;
+    }
   //@}
 };
 
@@ -1008,10 +1249,133 @@ template <class Coll, class Key, class Base> class PSafeDictionaryBase : public
   See the PSafeObject class for more details. Especially in regard to
   enumeration of collections.
  */
-template <class Key, class Base> class PSafeDictionary : public PSafeDictionaryBase<PDictionary<Key, Base>, Key, Base>
+template <class K, class D>
+ class PSafeDictionary : public PSafeDictionaryBase<PDictionary<K, D>, K, D>
 {
   public:
-    typedef PSafePtr<Base> value_type;
+    typedef K key_type;
+    typedef D data_type;
+    typedef PSafePtr<D> value_type;
+    typedef PSafeDictionary<K, D> dict_type;
+
+  /**@name Iterators */
+  //@{
+    class iterator;
+    class const_iterator;
+    class iterator_base {
+      protected:
+        K * m_internal_first;  // Must be first two members
+        value_type m_internal_second;
+
+        const dict_type * m_dictionary;
+        PArray<K>   m_keys;
+        PINDEX      m_position;
+
+        iterator_base()
+          : m_internal_first(NULL)
+          , m_internal_second(NULL)
+          , m_dictionary(NULL)
+          , m_position(P_MAX_INDEX)
+        {
+        }
+
+        iterator_base(const dict_type * dict)
+          : m_dictionary(dict)
+          , m_keys(dict->GetKeys())
+        {
+          this->SetPosition(0);
+        }
+
+        iterator_base(const dict_type * dict, const K & key)
+          : m_dictionary(dict)
+          , m_keys(dict->GetKeys())
+        {
+          this->SetPosition(m_keys.GetValuesIndex(key));
+        }
+
+        bool SetPosition(PINDEX position)
+        {
+          if (position >= this->m_keys.GetSize()) {
+            this->m_position = P_MAX_INDEX;
+            this->m_internal_first = NULL;
+            this->m_internal_second.SetNULL();
+            return false;
+          }
+
+          this->m_position = position;
+          this->m_internal_first  = &this->m_keys[position];
+          this->m_internal_second = this->m_dictionary->FindWithLock(*this->m_internal_first, PSafeReference);
+          return this->m_internal_second == NULL;
+        }
+
+        void Next() { while (this->SetPosition(this->m_position+1)) { } }
+        void Prev() { while (this->SetPosition(this->m_position > 0 ? this->m_position+1 : P_MAX_INDEX)) { } }
+
+      public:
+        bool operator==(const iterator_base & it) const { return this->m_position == it.m_position; }
+        bool operator!=(const iterator_base & it) const { return this->m_position != it.m_position; }
+    };
+
+    class iterator_pair {
+      public:
+        const K & first;
+        value_type second;
+
+      private:
+        iterator_pair() : first(reinterpret_cast<const K &>(0)) { }
+    };
+
+    class iterator : public iterator_base, public std::iterator<std::forward_iterator_tag, iterator_pair> {
+      protected:
+        iterator(dict_type * dict) : iterator_base(dict) { }
+        iterator(dict_type * dict, const K & key) : iterator_base(dict, key) { }
+
+      public:
+        iterator() { }
+
+        iterator operator++()    {                      this->Next(); return *this; }
+        iterator operator--()    {                      this->Prev(); return *this; }
+        iterator operator++(int) { iterator it = *this; this->Next(); return it;    }
+        iterator operator--(int) { iterator it = *this; this->Prev(); return it;    }
+
+        const iterator_pair * operator->() const { return  reinterpret_cast<const iterator_pair *>(this); }
+        const iterator_pair & operator* () const { return *reinterpret_cast<const iterator_pair *>(this); }
+
+      friend class PSafeDictionary<K, D>;
+    };
+
+    iterator begin() { return iterator(this); }
+    iterator end()   { return iterator(); }
+    iterator find(const K & key) { return iterator(this, key); }
+
+
+    class const_iterator : public iterator_base, public std::iterator<std::forward_iterator_tag, iterator_pair> {
+      protected:
+        const_iterator(const dict_type * dict) : iterator_base(dict) { }
+        const_iterator(const dict_type * dict, const K & key) : iterator_base(dict, key) { }
+
+      public:
+        const_iterator() { }
+        const_iterator(const typename dict_type::iterator & it) : iterator_base(it) { }
+
+        const_iterator operator++()    {                            this->Next(); return *this; }
+        const_iterator operator--()    {                            this->Prev(); return *this; }
+        const_iterator operator++(int) { const_iterator it = *this; this->Next(); return it;    }
+        const_iterator operator--(int) { const_iterator it = *this; this->Prev(); return it;    }
+
+        const iterator_pair * operator->() const { return  reinterpret_cast<const iterator_pair *>(this); }
+        const iterator_pair & operator* () const { return *reinterpret_cast<const iterator_pair *>(this); }
+
+      friend class PSafeDictionary<K, D>;
+    };
+
+    const_iterator begin() const { return const_iterator(this); }
+    const_iterator end()   const { return const_iterator(); }
+    const_iterator find(const K & key) const { return const_iterator(this, key); }
+
+    void erase(const       iterator & it) { this->RemoveAt(it->first); }
+    void erase(const const_iterator & it) { this->RemoveAt(it->first); }
+  //@}
 };
 
 
