@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23897 $
+ * $Revision: 28930 $
  * $Author: rjongbloed $
- * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
+ * $Date: 2013-01-18 02:37:32 -0600 (Fri, 18 Jan 2013) $
  */
 
 #ifndef PTLIB_SOCKET_H
@@ -40,10 +40,15 @@
 
 #include <ptlib/channel.h>
 
+#ifdef P_OPENBSD
+#include <sys/uio.h>
+#endif
+
 #ifdef __NUCLEUS_PLUS__
 #include <sys/socket.h>
 #endif
 
+
 class PSocket;
 
 PLIST(PSocketList, PSocket);
@@ -73,7 +78,7 @@ class PSocket : public PChannel
        Use the SetReadTimeout() function to set a maximum time for the Connect().
 
        @return
-       PTrue if the channel was successfully connected to the remote host.
+       true if the channel was successfully connected to the remote host.
      */
     virtual PBoolean Connect(
       const PString & address   ///< Address of remote machine to connect to.
@@ -97,7 +102,7 @@ class PSocket : public PChannel
        SetPort() or SetService() function.
 
        @return
-       PTrue if the channel was successfully opened.
+       true if the channel was successfully opened.
      */
     virtual PBoolean Listen(
       unsigned queueSize = 5,  ///< Number of pending accepts that may be queued.
@@ -125,7 +130,7 @@ class PSocket : public PChannel
        The default behaviour is to assert.
 
        @return
-       PTrue if the channel was successfully opened.
+       true if the channel was successfully opened.
      */
     virtual PBoolean Accept(
       PSocket & socket          ///< Listening socket making the connection.
@@ -134,7 +139,7 @@ class PSocket : public PChannel
     /**Close one or both of the data streams associated with a socket.
 
        @return
-       PTrue if the shutdown was performed
+       true if the shutdown was performed
      */
     virtual PBoolean Shutdown(
       ShutdownValue option   ///< Flag for shutdown of read, write or both.
@@ -147,7 +152,7 @@ class PSocket : public PChannel
        options of the class SOL_SOCKET.
 
        @return
-       PTrue if the option was successfully set.
+       true if the option was successfully set.
      */
     PBoolean SetOption(
       int option,             ///< Option to set.
@@ -159,7 +164,7 @@ class PSocket : public PChannel
        options of the class SOL_SOCKET.
 
        @return
-       PTrue if the option was successfully set.
+       true if the option was successfully set.
      */
     PBoolean SetOption(
       int option,             ///< Option to set.
@@ -172,7 +177,7 @@ class PSocket : public PChannel
        options of the class SOL_SOCKET.
 
        @return
-       PTrue if the option was successfully retrieved.
+       true if the option was successfully retrieved.
      */
     PBoolean GetOption(
       int option,             ///< Option to get.
@@ -184,7 +189,7 @@ class PSocket : public PChannel
        options of the class SOL_SOCKET.
 
        @return
-       PTrue if the option was successfully retrieved.
+       true if the option was successfully retrieved.
      */
     PBoolean GetOption(
       int option,             ///< Option to get.
@@ -372,7 +377,7 @@ class PSocket : public PChannel
        has data available.
 
        @return
-       PTrue if the select was successful or timed out, PFalse if an error
+       true if the select was successful or timed out, false if an error
        occurred. If a timeout occurred then the lists returned will be empty.
 
        For the versions taking sockets directly instead of lists the integer
@@ -401,6 +406,84 @@ class PSocket : public PChannel
     inline static DWORD Net2Host(DWORD v) { return ntohl(v); }
   //@}
 
+
+  /**@name Scattered read/write functions */
+  //@{
+    /** Structure that defines a "slice" of memory to be written to
+     */
+#if _WIN32
+    struct Slice : public WSABUF 
+    {
+      Slice()
+      { buf = (char *)0; len = 0; }
+
+      Slice(void * v, const size_t len)
+      { SetBase(v); SetLength(len); }
+
+      Slice(const void * v, const size_t len)
+      { SetBase(v); SetLength(len); }
+
+      void SetBase(const void * v)   { buf = (char *)v; }
+      void SetBase(void * v)         { buf = (char *)v; }
+      void * GetBase() const         { return buf; }
+
+      void SetLength(size_t v)  { len = (ULONG)v; }
+      size_t GetLength() const  { return len; }
+    };
+#else
+#if P_HAS_RECVMSG
+    struct Slice : public iovec
+    {
+#else
+    struct Slice 
+    {
+      protected:
+        void * iov_base;
+        size_t iov_len;
+      public:
+#endif
+      Slice()
+      { SetBase(NULL); SetLength(0); }
+
+      Slice(void * v, size_t len)
+      { SetBase(v); SetLength(len); }
+
+      void SetBase(void * v) { iov_base = v; }
+      void * GetBase() const   { return iov_base; }
+      void SetLength(size_t v) { iov_len = v; }
+      size_t GetLength() const  { return iov_len; }
+    };
+#endif
+
+    /** Low level scattered read from the channel. This is identical to Read except 
+        that the data will be read into a series of scattered memory slices. By default,
+        this call will default to calling Read multiple times, but this may be 
+        implemented by operating systems to do a real scattered read
+
+       @return
+       true indicates that at least one character was read from the channel.
+       false means no bytes were read due to timeout or some other I/O error.
+     */
+    virtual bool Read(
+      Slice * slices,    // slices to read to
+      size_t sliceCount
+    );
+
+    /** Low level scattered write to the channel. This is identical to Write except 
+        that the data will be written from a series of scattered memory slices. By default,
+        this call will default to calling Write multiple times, but this can be actually
+        implemented by operating systems to do a real scattered write
+
+       @return
+       true indicates that at least one character was read from the channel.
+       false means no bytes were read due to timeout or some other I/O error.
+     */
+    virtual bool Write(
+      const Slice * slices,  // slices to write from
+      size_t sliceCount
+    );
+  //@}
+
   protected:
     /*This function calls os_socket() with the correct parameters for the
        socket protocol type.
@@ -416,26 +499,29 @@ class PSocket : public PChannel
     int os_socket(int af, int type, int proto);
     PBoolean os_connect(
       struct sockaddr * sin,
-      PINDEX size
+      socklen_t size
     );
-    PBoolean os_recvfrom(
-      void * buf,
-      PINDEX len,
+
+    PBoolean os_vread(
+      Slice * slices,
+      size_t sliceCount,
       int flags,
       struct sockaddr * from,
-      PINDEX * fromlen
+      socklen_t * fromlen
     );
-    PBoolean os_sendto(
-      const void * buf,
-      PINDEX len,
+
+    PBoolean os_vwrite(
+      const Slice * slices,
+      size_t sliceCount,
       int flags,
       struct sockaddr * to,
-      PINDEX tolen
+      socklen_t tolen
     );
+
     PBoolean os_accept(
       PSocket & listener,
       struct sockaddr * addr,
-      PINDEX * size
+      socklen_t * size
     );
 
 
@@ -443,12 +529,6 @@ class PSocket : public PChannel
     /// Port to be used by the socket when opening the channel.
     WORD port;
 
-#if P_HAS_RECVMSG
-    PBoolean catchReceiveToAddr;
-    virtual void SetLastReceiveAddr(void * /*addr*/, int /*addrLen*/)
-    { }
-#endif
-
 // Include platform dependent part of class
 #ifdef _WIN32
 #include "msos/ptlib/socket.h"
@@ -492,7 +572,15 @@ class P_fd_set {
     fd_set * set;
 
   private:
+#ifdef __MINGW32__
+  // avoid compile error when this declaration is private
+  // this could be a gcc bug
+  public:
     P_fd_set(const P_fd_set &) {}
+  private:
+#else
+    P_fd_set(const P_fd_set &) {}
+#endif
     void operator=(const P_fd_set &) {}
 };
 
@@ -527,20 +615,6 @@ class P_timeval {
     PBoolean infinite;
 };
 
-#ifdef _WIN32
-class PWinSock : public PSocket
-{
-  PCLASSINFO(PWinSock, PSocket)
-// Must be one and one only instance of this class, and it must be static!.
-  public:
-    PWinSock();
-    ~PWinSock();
-  private:
-    virtual PBoolean OpenSocket();
-    virtual const char * GetProtocolName() const;
-};
-#endif
-
 
 #endif // PTLIB_SOCKET_H
 
