@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23897 $
- * $Author: rjongbloed $
- * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
+ * $Revision: 28941 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 08:18:28 -0600 (Fri, 18 Jan 2013) $
  */
 
 
@@ -46,7 +46,7 @@
    entity that is abstracted for use here. Very little manipulation of the
    sounds are possible.
 
-   The most common sound to use is the static function #Beep()# which
+   The most common sound to use is the static function <code>Beep()</code> which
    emits the system standard "warning" or "attention" sound.
  */
 class PSound : public PBYTEArray
@@ -92,7 +92,7 @@ class PSound : public PBYTEArray
        format conversions between file and driver are performed.
 
        @return
-       PTrue if the sound is loaded successfully.
+       true if the sound is loaded successfully.
      */
     PBoolean Load(
       const PFilePath & filename   ///< Sound file to load.
@@ -102,7 +102,7 @@ class PSound : public PBYTEArray
        object.
 
        @return
-       PTrue if the sound is saved successfully.
+       true if the sound is saved successfully.
      */
     PBoolean Save(
       const PFilePath & filename   ///< Sound file to load.
@@ -152,17 +152,17 @@ class PSound : public PBYTEArray
 
   /**@name Miscellaneous functions */
   //@{
-    /**Play a sound file to the default device. If the #wait#
-       parameter is PTrue then the function does not return until the file has
-       been played. If PFalse then the sound play is begun asynchronously and
+    /**Play a sound file to the default device. If the <code>wait</code>
+       parameter is true then the function does not return until the file has
+       been played. If false then the sound play is begun asynchronously and
        the function returns immediately.
 
        @return
-       PTrue if the sound is playing or has played.
+       true if the sound is playing or has played.
      */
     static PBoolean PlayFile(
       const PFilePath & file, ///< Sound file to play.
-      PBoolean wait = PTrue        ///< Flag to play sound synchronously.
+      PBoolean wait = true        ///< Flag to play sound synchronously.
     );
 
     /// Play the "standard" warning beep for the platform.
@@ -186,12 +186,12 @@ class PSound : public PBYTEArray
 
 
 /**
-   Abstract class for a generalised sound channel, 
-   and an implementation of PSoundChannel for old code that is not plugin-aware.
+   Abstract class for a generalised sound channel, and an implementation of
+   PSoundChannel for old code that is not plugin-aware.
    When instantiated, it selects the first plugin of the base class 
    "PSoundChannel"
 
-   As an abstract class, this represents a sound schannel. Drivers for real, 
+   As an abstract class, this represents a sound channel. Drivers for real, 
    platform dependent sound hardware will be ancestors of this class and 
    can be found in the plugins section of PTLib.
 
@@ -201,13 +201,12 @@ class PSound : public PBYTEArray
    both read and write audio data to once instance of a PSoundChannel
    class.
 
-   PSoundChanel instances are designed to be reentrant. The actual
+   PSoundChannel instances are designed to be reentrant. The actual
    usage model employed is left to the developer. One model could be
-   where one thread is responsible for 
-   construction, setup, opening and read/write operations. After creating and
-   eventually opening the channel this thread is responsible for
-   handling read/writes fast enough to avoid gaps in the generated  audio
-   stream.
+   where one thread is responsible for construction, setup, opening and 
+   read/write operations. After creating and eventually opening the channel
+   this thread is responsible for handling read/writes fast enough to avoid
+   gaps in the generated audio stream.
 
    Remaining operations may beinvoked from other threads.
    This includes Close() and actually gathering the necessary data to
@@ -238,7 +237,7 @@ class PSound : public PBYTEArray
    Similarly for reading, an entire buffer must be read before any of it is
    available to a Read() call. Note that once a buffer is filled you can read
    it a byte at a time if desired, but as soon as all the data in the buffer
-   is used returned, the next read will wait until the entire next buffer is
+   is used, the next read will wait until the entire next buffer is
    read from the hardware. So again, tailor the number and size of buffers to
    the application. To avoid being blocked until the buffer fills, you can use
    the StartRecording() function to initiate the buffer filling, and the
@@ -257,6 +256,7 @@ class PSoundChannel : public PChannel
   /**@name Construction */
   //@{
     enum Directions {
+      Closed = -1,
       Recorder,
       Player
     };
@@ -342,13 +342,9 @@ class PSoundChannel : public PChannel
 
     /**Get the name for the default sound devices/driver that is on this
        platform. Note that a named device may not necessarily do both
-       playing and recording so the arrays returned with the #dir#
+       playing and recording so the string returned with the <code>dir</code>
        parameter in each value is not necessarily the same.
 
-       This will return a list of uniqie device names across all of the available
-       drivers. If two drivers have identical names for devices, then the string
-       returned will be of the form driver+'\\t'+device.
-
        @return
        A platform dependent string for the sound player/recorder.
      */
@@ -358,9 +354,13 @@ class PSoundChannel : public PChannel
 
     /**Get the list of all devices name for the default sound devices/driver that is on this
        platform. Note that a named device may not necessarily do both
-       playing and recording so the arrays returned with the #dir#
+       playing and recording so the arrays returned with the <code>dir</code>
        parameter in each value is not necessarily the same.
 
+       This will return a list of unique device names across all of the available
+       drivers. If two drivers have identical names for devices, then the string
+       returned will be of the form driver+'\\t'+device.
+
        @return
        Platform dependent strings for the sound player/recorder.
      */
@@ -373,7 +373,7 @@ class PSoundChannel : public PChannel
        platform specific and is as returned in the GetDevices() function.
 
        @return
-       PTrue if the sound device is valid for playing/recording.
+       true if the sound device is valid for playing/recording.
      */
     virtual PBoolean Open(
       const PString & device,       ///< Name of sound driver/device
@@ -386,13 +386,13 @@ class PSoundChannel : public PChannel
     /**Test if this instance of PSoundChannel is open.
 
        @return
-       PTrue if this instance is open.
+       true if this instance is open.
      */
     virtual PBoolean IsOpen() const;
 
     /** Close the channel, shutting down the link to the data source. 
 
-       @return PTrue if the channel successfully closed.
+       @return true if the channel successfully closed.
      */
     virtual PBoolean Close();
 
@@ -401,17 +401,31 @@ class PSoundChannel : public PChannel
        @return
        integer value of the handle.
      */
-    virtual int GetHandle() const;
+    virtual P_INT_PTR GetHandle() const;
 
     /// Get the name of the open channel
     virtual PString GetName() const;
 
+    /// Get the direction of the channel
+    Directions GetDirection() const
+    {
+      return activeDirection;
+    }
+
+    /// Get text representing the direction of the channel
+    static const char * GetDirectionText(Directions dir);
+
+    virtual const char * GetDirectionText() const
+    {
+      return GetDirectionText(activeDirection);
+    }
+
     /** Abort the background playing/recording of the sound channel.
-	There will be a logic assertion if you attempt to Abort a
-	sound channel operation, when the device is currently closed.
+        There will be a logic assertion if you attempt to Abort a
+        sound channel operation, when the device is currently closed.
 
        @return
-       PTrue if the sound has successfully been aborted.
+       true if the sound has successfully been aborted.
      */
     virtual PBoolean Abort();
   //@}
@@ -425,7 +439,7 @@ class PSoundChannel : public PChannel
        by information in the file being played.
 
        @return
-       PTrue if the format is valid.
+       true if the format is valid.
      */
     virtual PBoolean SetFormat(
       unsigned numChannels = 1,     ///< Number of channels eg mono/stereo
@@ -448,7 +462,7 @@ class PSoundChannel : public PChannel
        power of two, so 20000 => 32768. 
 
        @return
-       PTrue if the sound device is valid for playing/recording.
+       true if the sound device is valid for playing/recording.
      */
     virtual PBoolean SetBuffers(
       PINDEX size,      ///< Size of each buffer
@@ -458,7 +472,7 @@ class PSoundChannel : public PChannel
     /**Get the internal buffers for the sound channel I/O. 
 
        @return
-       PTrue if the buffer size were obtained.
+       true if the buffer size were obtained.
      */
     virtual PBoolean GetBuffers(
       PINDEX & size,    // Size of each buffer
@@ -470,28 +484,47 @@ class PSoundChannel : public PChannel
     };
 
     /**Set the volume of the play/read process.
-       The volume range is 0 == quiet, 100 == LOUDEST. The volume is a
+       The volume range is 0 == muted, 100 == LOUDEST. The volume is a
        logarithmic scale mapped from the lowest gain possible on the device to
-       the highest gain
+       the highest gain.
         
        @return
-       PTrue if there were no errors.
+       true if there were no errors.
     */
     virtual PBoolean SetVolume(
       unsigned volume   ///< New volume level
     );
 
     /**Get the volume of the play/read process.
-       The volume range is 0 == quiet, 100 == LOUDEST. The volume is a
+       The volume range is 0 == muted, 100 == LOUDEST. The volume is a
        logarithmic scale mapped from the lowest gain possible on the device to
        the highest gain.
 
        @return
-       PTrue if there were no errors.
+       true if there were no errors.
     */
     virtual PBoolean GetVolume(
       unsigned & volume   ///< Variable to receive volume level.
     );
+
+    /**Set the mute state of the play/read process.
+        
+       @return
+       true if there were no errors.
+    */
+    virtual bool SetMute(
+      bool mute   ///< New mute state
+    );
+
+    /**Get the mute state of the play/read process.
+
+       @return
+       true if there were no errors.
+    */
+    virtual bool GetMute(
+      bool & mute   ///< Variable to receive mute state.
+    );
+
   //@}
 
   /**@name Play functions */
@@ -499,33 +532,33 @@ class PSoundChannel : public PChannel
 
     /** Low level write (or play) to the channel. 
 
-	It will generate a logical assertion if you attempt write to a
-	channel set up for recording.
+        It will generate a logical assertion if you attempt write to a
+        channel set up for recording.
 
-       @param buf is a pointer to the data to be written to the
-       channel.  It is an error for this pointer to be NULL. A logical
-       assert will be generated when buf is NULL.
+        @param buf is a pointer to the data to be written to the
+        channel.  It is an error for this pointer to be NULL. A logical
+        assert will be generated when buf is NULL.
 
-       @param len Nr of bytes to send. If len equals the buffer size
+        @param len Nr of bytes to send. If len equals the buffer size
         set by SetBuffers() it will block for
         (1000*len)/(samplesize*samplerate) ms. Typically, the sample
         size is 2 bytes.  If len == 0, this will return immediately,
         where the return value is equal to the value of IsOpen().
  
-       @return PTrue if len bytes were written to the channel,
-         otherwise PFalse. The GetErrorCode() function should be 
-        consulted after Write() returns PFalse to determine what 
+        @return true if len bytes were written to the channel,
+        otherwise false. The GetErrorCode() function should be 
+        consulted after Write() returns false to determine what 
         caused the failure.
-
      */
     virtual PBoolean Write(const void * buf, PINDEX len);
 
+
     /** Get number of bytes written in last Write() operation. */
     virtual PINDEX GetLastWriteCount() const;
 
-    /**Play a sound to the open device. If the #wait# parameter is
-       PTrue then the function does not return until the file has been played.
-       If PFalse then the sound play is begun asynchronously and the function
+    /**Play a sound to the open device. If the <code>wait</code> parameter is
+       true then the function does not return until the file has been played.
+       If false then the sound play is begun asynchronously and the function
        returns immediately.
 
        Note:  if the driver is closed while playing the sound, the play 
@@ -536,17 +569,17 @@ class PSoundChannel : public PChannel
        performed.
 
        @return
-       PTrue if the sound is playing or has played.
+       true if the sound is playing or has played.
      */
 
     virtual PBoolean PlaySound(
       const PSound & sound,   ///< Sound to play.
-      PBoolean wait = PTrue        ///< Flag to play sound synchronously.
+      PBoolean wait = true        ///< Flag to play sound synchronously.
     );
 
-    /**Play a sound file to the open device. If the #wait#
-       parameter is PTrue then the function does not return until the file has
-       been played. If PFalse then the sound play is begun asynchronously and
+    /**Play a sound file to the open device. If the <code>wait</code>
+       parameter is true then the function does not return until the file has
+       been played. If false then the sound play is begun asynchronously and
        the function returns immediately.
 
        Note if the driver is closed of the object destroyed then the sound
@@ -557,18 +590,18 @@ class PSoundChannel : public PChannel
        performed.
 
        @return
-       PTrue if the sound is playing or has played.
+       true if the sound is playing or has played.
      */
     virtual PBoolean PlayFile(
       const PFilePath & file, ///< Sound file to play.
-      PBoolean wait = PTrue        ///< Flag to play sound synchronously.
+      PBoolean wait = true        ///< Flag to play sound synchronously.
     );
 
     /**Indicate if the sound play begun with PlayBuffer() or PlayFile() has
        completed.
 
        @return
-       PTrue if the sound has completed playing.
+       true if the sound has completed playing.
      */
     virtual PBoolean HasPlayCompleted();
 
@@ -576,7 +609,7 @@ class PSoundChannel : public PChannel
        PlayFile() has completed. 
 
        @return
-       PTrue if the sound has successfully completed playing.
+       true if the sound has successfully completed playing.
      */
     virtual PBoolean WaitForPlayCompletion();
 
@@ -589,26 +622,26 @@ class PSoundChannel : public PChannel
        reached. The GetLastReadCount() function returns the actual number
        of bytes read.
 
-	It will generate a logical assertion if you attempt to read
-	from a PSoundChannel that is setup for playing.
+       It will generate a logical assertion if you attempt to read
+       from a PSoundChannel that is setup for playing.
 
        The GetErrorCode() function should be consulted after Read() returns
-       PFalse to determine what caused the failure.
+       false to determine what caused the failure.
 
-        @param len Nr of bytes to endeaveour to read from the sound
-        device. If len equals the buffer size set by SetBuffers() it
-        will block for (1000*len)/(samplesize*samplerate)
-        ms. Typically, the sample size is 2 bytes.  If len == 0, this
-        will return immediately, where the return value is equal to
-        the value of IsOpen().
+       @param len Nr of bytes to endeaveour to read from the sound
+       device. If len equals the buffer size set by SetBuffers() it
+       will block for (1000*len)/(samplesize*samplerate)
+       ms. Typically, the sample size is 2 bytes.  If len == 0, this
+       will return immediately, where the return value is equal to
+       the value of IsOpen().
 
        @param buf is a pointer to the empty data area, which will
        contain the data collected from the sound device.  It is an
        error for this pointer to be NULL. A logical assert will be
        generated when buf is NULL.
 
-       @return PTrue indicates that at least one character was read
-       from the channel.  PFalse means no bytes were read due to some
+       @return true indicates that at least one character was read
+       from the channel.  false means no bytes were read due to some
        I/O error, (which includes timeout or some other thread closed
        the device).
      */
@@ -635,7 +668,7 @@ class PSoundChannel : public PChannel
        without blocking.
 
        @return
-       PTrue if the sound has been recorded.
+       true if the sound has been recorded.
      */
     virtual PBoolean RecordSound(
       PSound & sound ///< Sound recorded
@@ -651,7 +684,7 @@ class PSoundChannel : public PChannel
        without blocking.
 
        @return
-       PTrue if the sound has been recorded.
+       true if the sound has been recorded.
      */
     virtual PBoolean RecordFile(
       const PFilePath & file ///< Sound file recorded
@@ -661,7 +694,7 @@ class PSoundChannel : public PChannel
        initiate the recording.
 
        @return
-       PTrue if the sound driver has successfully started recording.
+       true if the sound driver has successfully started recording.
      */
     virtual PBoolean StartRecording();
 
@@ -670,7 +703,7 @@ class PSoundChannel : public PChannel
        the buffer size.
 
        @return
-       PTrue if the sound driver has filled a buffer.
+       true if the sound driver has filled a buffer.
      */
     virtual PBoolean IsRecordBufferFull();
 
@@ -680,7 +713,7 @@ class PSoundChannel : public PChannel
        a new recording.
 
        @return
-       PTrue if the sound driver has filled a buffer.
+       true if the sound driver has filled a buffer.
      */
     virtual PBoolean AreAllRecordBuffersFull();
 
@@ -689,7 +722,7 @@ class PSoundChannel : public PChannel
        is less than the buffer size.
 
        @return
-       PTrue if the sound driver has filled a buffer.
+       true if the sound driver has filled a buffer.
      */
     virtual PBoolean WaitForRecordBufferFull();
 
@@ -699,13 +732,14 @@ class PSoundChannel : public PChannel
        again to start a new recording.
 
        @return
-       PTrue if the sound driver has filled a buffer.
+       true if the sound driver has filled a buffer.
      */
     virtual PBoolean WaitForAllRecordBuffersFull();
   //@}
 
   protected:
-    PSoundChannel * baseChannel;
+    PSoundChannel * m_baseChannel;
+    PReadWriteMutex m_baseMutex;
 
     /**This is the direction that this sound channel is opened for use
        in.  Should the user attempt to used this opened class instance
@@ -736,7 +770,7 @@ template <class className> class PSoundChannelPluginServiceDescriptor : public P
   PPLUGIN_STATIC_LOAD(BeOS, PSoundChannel);
 #endif
 
-#if defined(P_DIRECTSOUND) && ! defined(P_DIRECTSOUND_WINCE)
+#if defined(P_DIRECTSOUND)
   PPLUGIN_STATIC_LOAD(DirectSound, PSoundChannel);
 #endif
 
