@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21791 $
+ * $Revision: 29970 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 00:07:11 -0600 (Fri, 12 Dec 2008) $
+ * $Date: 2013-06-16 20:14:51 -0500 (Sun, 16 Jun 2013) $
  */
 
 #ifndef PTLIB_THREAD_H
@@ -43,23 +43,22 @@
 #endif
 
 #include <ptlib/mutex.h>
+#include <set>
+
 
 class PSemaphore;
 
-#define PThreadIdentifer PThreadIdentifier
-
-typedef P_THREADIDENTIFIER PThreadIdentifier;
 
 ///////////////////////////////////////////////////////////////////////////////
 // PThread
 
-/** This class defines a thread of execution in the system. A \it thread is
+/** This class defines a thread of execution in the system. A <i>thread</i> is
    an independent flow of processor instructions. This differs from a
-   \it process which also embodies a program address space and resource
+   <i>process</i> which also embodies a program address space and resource
    allocation. So threads can share memory and resources as they run in the
    context of a given process. A process always contains at least one thread.
-   This is reflected in this library by the #PProcess class being
-   descended from the #PThread class.
+   This is reflected in this library by the <code>PProcess</code> class being
+   descended from the <code>PThread</code> class.
 
    The implementation of a thread is platform dependent, but it is
    assumed that the platform has some support for native threads.
@@ -97,50 +96,49 @@ class PThread : public PObject
       NoAutoDeleteThread  
     };
 
-    /** Create a new thread instance. Unless the #startSuspended
-       parameter is #PTrue, the threads #Main() function is called to
+    /** Create a new thread instance. Unless the <code>startSuspended</code>
+       parameter is <code>true</code>, the threads <code>Main()</code> function is called to
        execute the code for the thread.
        
        Note that the exact timing of the execution of code in threads can
        never be predicted. Thus you you can get a race condition on
        intialising a descendent class. To avoid this problem a thread is
-       always started suspended. You must call the #Resume() function after
+       always started suspended. You must call the <code>Resume()</code> function after
        your descendent class construction is complete.
 
        If synchronisation is required between threads then the use of
        semaphores is essential.
 
-       If the #deletion is set to #AutoDeleteThread
-       then the #PThread is assumed to be allocated with the new operator and
+       If the <code>deletion</code> is set to <code>AutoDeleteThread</code>
+       then the <code>PThread</code> is assumed to be allocated with the new operator and
        may be freed using the delete operator as soon as the thread is
        terminated or executes to completion (usually the latter).
 
-       The stack size argument retained only for source code compatibility for
-       previous implementations. It is not used in the current code and
-       may be removed in subsequent versions.
+       The stack size argument is highly platform specific and should alamost
+       always be zero for the default.
      */
     PThread(
-      PINDEX,                 ///< Not used - previously stack size
+      PINDEX stack,  ///< Stack size on some platforms, 0 == default
       AutoDeleteFlag deletion = AutoDeleteThread,
         ///< Automatically delete PThread instance on termination of thread.
       Priority priorityLevel = NormalPriority,  ///< Initial priority of thread.
       const PString & threadName = PString::Empty() ///< The name of the thread (for Debug/Trace)
     );
 
-    /** Destroy the thread, this simply calls the #Terminate() function
+    /** Destroy the thread, this simply calls the <code>Terminate()</code> function
        with all its restrictions and penalties. See that function for more
        information.
 
        Note that the correct way for a thread to terminate is to return from
-       the #Main() function.
+       the <code>Main()</code> function.
      */
     ~PThread();
   //@}
 
-  /**@name Overrides from #PObject */
+  /**@name Overrides from <code>PObject</code> */
   //@{
     /**Standard stream print function.
-       The #PObject class has a << operator defined that calls this function
+       The <code>PObject</code> class has a << operator defined that calls this function
        polymorphically.
       */
     void PrintOn(
@@ -164,8 +162,8 @@ class PThread : public PObject
        called causing at the very least the possiblity of memory leaks.
 
        Note that the correct way for a thread to terminate is to return from
-       the #Main() function or self terminate by calling
-       #Terminate() within the context of the thread which can then
+       the <code>Main()</code> function or self terminate by calling
+       <code>Terminate()</code> within the context of the thread which can then
        assure that all resources are cleaned up.
      */
     virtual void Terminate();
@@ -173,7 +171,7 @@ class PThread : public PObject
     /** Determine if the thread has been terminated or ran to completion.
 
        @return
-       #PTrue if the thread has been terminated.
+       <code>true</code> if the thread has been terminated.
      */
     virtual PBoolean IsTerminated() const;
 
@@ -184,7 +182,7 @@ class PThread : public PObject
     /** Block and wait for the thread to terminate.
 
        @return
-       #PFalse if the thread has not terminated and the timeout has expired, #PTrue otherwise.
+       <code>false</code> if the thread has not terminated and the timeout has expired, <code>true</code> otherwise.
      */
     PBoolean WaitForTermination(
       const PTimeInterval & maxWait  ///< Maximum time to wait for termination.
@@ -192,35 +190,35 @@ class PThread : public PObject
 
     /** Suspend or resume the thread.
     
-       If #susp is #PTrue this increments an internal count of
+       If <code>susp</code> is <code>true</code> this increments an internal count of
        suspensions that must be matched by an equal number of calls to
-       #Resume()# or #Suspend(PFalse) before the
+       <code>Resume()</code> or <code>Suspend(false)</code> before the
        thread actually executes again.
 
-       If #susp is #PFalse then this decrements the internal count of
+       If <code>susp</code> is <code>false</code> then this decrements the internal count of
        suspensions. If the count is <= 0 then the thread will run. Note that
        the thread will not be suspended until an equal number of
-       #Suspend(PTrue) calls are made.
+       <code>Suspend(true)</code> calls are made.
      */
     virtual void Suspend(
-      PBoolean susp = PTrue    ///< Flag to suspend or resume a thread.
+      PBoolean susp = true    ///< Flag to suspend or resume a thread.
     );
 
     /** Resume thread execution, this is identical to
-       #Suspend(PFalse).
+       <code>Suspend(false)</code>.
 
       The Resume() method may be called from within the constructor of a
-      PThread descendant.  However, the #Resume() should be in the
+      PThread descendant.  However, the <code>Resume()</code> should be in the
       constructor of the most descendant class. So, if you have a
       class B (which is descended of PThread), and a class C (which is
-      descended of B), placing the call to #Resume() in the constructor of B is
+      descended of B), placing the call to <code>Resume()</code> in the constructor of B is
       unwise.
 
-      If you do place a call to #Resume() in the constructor, it
+      If you do place a call to <code>Resume()</code> in the constructor, it
       should be at the end of the constructor, after all the other
       initialisation in the constructor.
 
-      The reason the call to #Resume() should be at the end of the
+      The reason the call to <code>Resume()</code> should be at the end of the
       construction process is simple - you want the thread to start
       when all the variables in the class have been correctly
       initialised.
@@ -228,11 +226,11 @@ class PThread : public PObject
     virtual void Resume();
 
     /** Determine if the thread is currently suspended. This checks the
-       suspension count and if greater than zero returns #PTrue for a suspended
+       suspension count and if greater than zero returns <code>true</code> for a suspended
        thread.
 
        @return
-       #PTrue if thread is suspended.
+       <code>true</code> if thread is suspended.
      */
     virtual PBoolean IsSuspended() const;
 
@@ -291,9 +289,24 @@ class PThread : public PObject
       * if called by the owning thread. Calling this function for another
       * thread that may be terminating is a very bad idea.
       */
-    virtual PThreadIdentifier GetThreadId() const;
+    virtual PThreadIdentifier GetThreadId() const { return m_threadId; }
     static PThreadIdentifier GetCurrentThreadId();
 
+    /// Times for execution of the thread.
+    struct Times
+    {
+      PTimeInterval m_real;     ///< Total real time since thread start in milliseconds.
+      PTimeInterval m_kernel;   ///< Total kernel CPU time in milliseconds.
+      PTimeInterval m_user;     ///< Total user CPU time in milliseconds.
+      friend ostream & operator<<(ostream & strm, const Times & times);
+    };
+
+    /** Get the thread execution times.
+     */
+    bool GetTimes(
+      Times & times   ///< Times for thread execution.
+    );
+
     /** User override function for the main execution routine of the thread. A
        descendent class must provide the code that will be executed in the
        thread within this function.
@@ -323,8 +336,8 @@ class PThread : public PObject
     static void Yield();
 
     /**Create a simple thread executing the specified notifier.
-       This creates a simple #PThread class that automatically executes the
-       function defined by the #PNotifier in the context of a new thread.
+       This creates a simple <code>PThread</code> class that automatically executes the
+       function defined by the <code>PNotifier</code> in the context of a new thread.
       */
     static PThread * Create(
       const PNotifier & notifier,     ///< Function to execute in thread.
@@ -333,7 +346,7 @@ class PThread : public PObject
         ///< Automatically delete PThread instance on termination of thread.
       Priority priorityLevel = NormalPriority,  ///< Initial priority of thread.
       const PString & threadName = PString::Empty(), ///< The name of the thread (for Debug/Trace)
-      PINDEX stackSize = 65536         ///< Stack size on some platforms
+      PINDEX stackSize = 0            ///< Stack size on some platforms, 0 == default
     );
     static PThread * Create(
       const PNotifier & notifier,     ///< Function to execute in thread.
@@ -341,15 +354,36 @@ class PThread : public PObject
     ) { return Create(notifier, 0, NoAutoDeleteThread, NormalPriority, threadName); }
   //@}
   
-    bool IsAutoDelete() const { return autoDelete; }
+    bool IsAutoDelete() const { return m_type == e_IsAutoDelete; }
+
+    /// Thread local storage base class, see PThreadLocalStorage for template.
+    class LocalStorageBase
+    {
+      public:
+        virtual ~LocalStorageBase() { }
+      protected:
+        LocalStorageBase() { }
+        void StorageDestroyed();
+        virtual void * Allocate() const = 0;
+        virtual void Deallocate(void * ptr) const = 0;
+        virtual void * GetStorage() const;
+      private:
+        void ThreadDestroyed(PThread * thread) const;
+        typedef std::map<PThread *, void *> StorageMap;
+        mutable StorageMap m_storage;
+        PMutex m_mutex;
+      friend class PThread;
+    };
+    friend class LocalStorageBase;
 
   private:
-    PThread();
-    // Create a new thread instance as part of a #PProcess class.
+    PThread(bool isProcess);
+    // Create a new thread instance as part of a <code>PProcess</code> class.
+
+    void InternalDestroy();
 
     friend class PProcess;
     friend class PExternalThread;
-    friend class PHouseKeepingThread;
     // So a PProcess can get at PThread() constructor but nothing else.
 
     PThread(const PThread &) : PObject () { }
@@ -358,30 +392,18 @@ class PThread : public PObject
     PThread & operator=(const PThread &) { return *this; }
     // Empty assignment operator to prevent copying of thread instances.
 
-    PBoolean autoDelete;
-    // Automatically delete the thread on completion.
+  protected:
+    enum Type { e_IsAutoDelete, e_IsManualDelete, e_IsProcess, e_IsExternal } m_type;
 
-    // Give the thread a name for debugging purposes.
-    PString threadName;
-    PMutex threadNameMutex;
+    PINDEX m_originalStackSize;
 
-#if PTRACING
-  public:
-    struct TraceInfo {
-      TraceInfo()
-      { traceBlockIndentLevel = 0; }
+    PString m_threadName; // Give the thread a name for debugging purposes.
+    PMutex  m_threadNameMutex;
 
-      PStack<PStringStream> traceStreams;
-      unsigned traceLevel;
-      unsigned traceBlockIndentLevel;
-    };
+    PThreadIdentifier m_threadId;
 
-#ifndef P_HAS_THREADLOCAL_STORAGE
-  private:
-    friend class PTrace;
-    TraceInfo traceInfo;
-#endif // P_HAS_THREADLOCAL_STORAGE
-#endif // PTRACING
+    typedef std::list<const LocalStorageBase *> LocalStorageList;
+    LocalStorageList m_localStorage;
 
 // Include platform dependent part of class
 #ifdef _WIN32
@@ -391,19 +413,9 @@ class PThread : public PObject
 #endif
 };
 
-// Include definition of platform dependent thread ID format
-#if defined(_WIN32) && !defined(_WIN32_WCE)
-  #define PTHREAD_ID_FMT ":%u"
-#else
-  #define PTHREAD_ID_FMT ":0x%x"
-#endif
-
-#ifdef _MSC_VER
-#pragma warning(disable:4355)
-#endif
 
 /** Define some templates to simplify the declaration
-  * of simple #PThread descendants with one or two paramaters 
+  * of simple <code>PThread</code> descendants with one or two paramaters 
   */
 
 /*
@@ -420,25 +432,60 @@ class PThread : public PObject
  */
 class PThreadMain : public PThread
 {
-  PCLASSINFO(PThreadMain, PThread);
+    PCLASSINFO(PThreadMain, PThread);
   public:
     typedef void (*FnType)(); 
-    PThreadMain(FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThreadMain(FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
       , m_function(function)
-      { PThread::Resume(); }
-    PThreadMain(const char * file, int line, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,  NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_function(function)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+
     virtual void Main()
-      { (*m_function)(); }
+    {
+      (*m_function)();
+    }
 
   protected:
     FnType m_function;
 };
 
+
+/*
+   This template automates calling a global function using a functor
+   It is used as follows:
+
+   struct Functor {
+     void operator()(PThread & thread) { ... code in here }
+   }
+
+   ...
+   Functor arg;
+   new PThreadFunctor<Functor>(arg)
+ */
+template<typename Functor>
+class PThreadFunctor : public PThread
+{
+    PCLASSINFO(PThreadFunctor, PThread);
+  public:
+    PThreadFunctor(Functor & funct, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+      , m_funct(funct)
+    {
+      PThread::Resume();
+    }
+
+    virtual void Main()
+    {
+      m_funct(*this);
+    }
+
+  protected:
+    Functor & m_funct;
+};
+
+
 /*
    This template automates calling a global function with one argument within it's own thread.
    It is used as follows:
@@ -454,23 +501,22 @@ class PThreadMain : public PThread
 template<typename Arg1Type>
 class PThread1Arg : public PThread
 {
-  PCLASSINFO(PThread1Arg, PThread);
+    PCLASSINFO(PThread1Arg, PThread);
   public:
     typedef void (*FnType)(Arg1Type arg1);
 
-    PThread1Arg(Arg1Type arg1, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread1Arg(Arg1Type arg1, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
       , m_function(function)
       , m_arg1(arg1)
-    { PThread::Resume(); }
-    PThread1Arg(const char * file, int line, Arg1Type arg1, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_function(function)
-      , m_arg1(arg1)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+
     virtual void Main()
-      { (*m_function)(m_arg1); }
+    {
+      (*m_function)(m_arg1);
+    }
 
   protected:
     FnType   m_function;
@@ -493,24 +539,22 @@ class PThread1Arg : public PThread
 template<typename Arg1Type, typename Arg2Type>
 class PThread2Arg : public PThread
 {
-  PCLASSINFO(PThread2Arg, PThread);
+    PCLASSINFO(PThread2Arg, PThread);
   public:
     typedef void (*FnType)(Arg1Type arg1, Arg2Type arg2); 
-    PThread2Arg(Arg1Type arg1, Arg2Type arg2, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread2Arg(Arg1Type arg1, Arg2Type arg2, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
       , m_function(function)
       , m_arg1(arg1)
       , m_arg2(arg2)
-      { PThread::Resume(); }
-    PThread2Arg(const char * file, int line, Arg1Type arg1, Arg2Type arg2, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_function(function)
-      , m_arg1(arg1)
-      , m_arg2(arg2)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+    
     virtual void Main()
-      { (*m_function)(m_arg1, m_arg2); }
+    {
+      (*m_function)(m_arg1, m_arg2);
+    }
 
   protected:
     FnType   m_function;
@@ -536,23 +580,20 @@ class PThread3Arg : public PThread
   PCLASSINFO(PThread3Arg, PThread);
   public:
     typedef void (*FnType)(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3); 
-    PThread3Arg(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread3Arg(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
       , m_function(function)
       , m_arg1(arg1)
       , m_arg2(arg2)
       , m_arg3(arg3)
-      { PThread::Resume(); }
-    PThread3Arg(const char * file, int line, Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_function(function)
-      , m_arg1(arg1)
-      , m_arg2(arg2)
-      , m_arg3(arg3)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+    
     virtual void Main()
-      { (*m_function)(m_arg1, m_arg2, m_arg3); }
+    {
+      (*m_function)(m_arg1, m_arg2, m_arg3);
+    }
 
   protected:
     FnType   m_function;
@@ -580,27 +621,34 @@ class PThread3Arg : public PThread
 template <typename ObjType>
 class PThreadObj : public PThread
 {
-  public:
-  PCLASSINFO(PThreadObj, PThread);
+    PCLASSINFO(PThreadObj, PThread);
   public:
     typedef void (ObjType::*ObjTypeFn)(); 
-    PThreadObj(ObjType & obj, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+
+    PThreadObj(
+      ObjType & obj,
+      ObjTypeFn function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000,
+                autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,
+                priority,
+                name)
       , m_object(obj)
       , m_function(function)
-      { PThread::Resume(); }
-    PThreadObj(const char * file, int line, ObjType & obj, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_object(obj)
-      , m_function(function)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+
     void Main()
-      { (m_object.*m_function)(); }
+    {
+      (m_object.*m_function)();
+    }
 
   protected:
     ObjType & m_object;
-    ObjTypeFn m_function;
+    ObjTypeFn P_ALIGN_FIELD(m_function, 8);
 };
 
 
@@ -623,60 +671,76 @@ class PThreadObj : public PThread
 template <class ObjType, typename Arg1Type>
 class PThreadObj1Arg : public PThread
 {
-  PCLASSINFO(PThreadObj1Arg, PThread);
+    PCLASSINFO(PThreadObj1Arg, PThread);
   public:
     typedef void (ObjType::*ObjTypeFn)(Arg1Type); 
-    PThreadObj1Arg(ObjType & obj, Arg1Type arg1, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+
+    PThreadObj1Arg(
+      ObjType & obj,
+      Arg1Type arg1,
+      ObjTypeFn function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000,
+                autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,
+                priority,
+                name)
       , m_object(obj)
       , m_function(function)
       , m_arg1(arg1)
-      { PThread::Resume(); }
-    PThreadObj1Arg(const char * file, int line, ObjType & obj, Arg1Type arg1, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_object(obj)
-      , m_function(function)
-      , m_arg1(arg1)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+
     void Main()
-    { (m_object.*m_function)(m_arg1); }
+    {
+      (m_object.*m_function)(m_arg1);
+    }
 
   protected:
     ObjType & m_object;
-    ObjTypeFn m_function;
+    ObjTypeFn P_ALIGN_FIELD(m_function, 8);
     Arg1Type  m_arg1;
 };
 
 template <class ObjType, typename Arg1Type, typename Arg2Type>
 class PThreadObj2Arg : public PThread
 {
-  PCLASSINFO(PThreadObj2Arg, PThread);
+    PCLASSINFO(PThreadObj2Arg, PThread);
   public:
-    typedef void (ObjType::*ObjTypeFn)(Arg1Type, Arg2Type); 
-    PThreadObj2Arg(ObjType & obj, Arg1Type arg1, Arg2Type arg2, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    typedef void (ObjType::*ObjTypeFn)(Arg1Type, Arg2Type);
+
+    PThreadObj2Arg(
+      ObjType & obj,
+      Arg1Type arg1,
+      Arg2Type arg2,
+      ObjTypeFn function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000,
+                autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,
+                priority,
+                name)
       , m_object(obj)
       , m_function(function)
       , m_arg1(arg1)
       , m_arg2(arg2)
-      { PThread::Resume(); }
-    PThreadObj2Arg(const char * file, int line, ObjType & obj, Arg1Type arg1, Arg2Type arg2, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_object(obj)
-      , m_function(function)
-      , m_arg1(arg1)
-      , m_arg2(arg2)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+
     void Main()
-      { (m_object.*m_function)(m_arg1, m_arg2); }
+    {
+      (m_object.*m_function)(m_arg1, m_arg2);
+    }
 
   protected:
     ObjType & m_object;
-    ObjTypeFn m_function;
+    ObjTypeFn P_ALIGN_FIELD(m_function, 8);
     Arg1Type  m_arg1;
-    Arg1Type  m_arg2;
+    Arg2Type  m_arg2;
 };
 
 
@@ -685,79 +749,26 @@ class PThreadObj2Arg : public PThread
 // PThreadLocalStorage
 //
 
-#ifdef _WIN32
-
-#define P_HAS_THREADLOCAL_STORAGE 1
-
 template <class Storage_T>
-class PThreadLocalStorage
+class PThreadLocalStorage : public PThread::LocalStorageBase
 {
   public:
-    typedef DWORD Key_T;
-    typedef Storage_T value_type;
+    typedef Storage_T * Ptr_T;
 
-    PThreadLocalStorage()
-    { key = TlsAlloc(); }
-
-    ~PThreadLocalStorage()
-    { TlsFree(key);  }
-
-    Key_T GetKey() const
-    { return key; }
-
-    value_type * Get()
-    { return (value_type *) TlsGetValue(key); }
-
-    void Set(value_type * v)
-    { TlsSetValue(key, (LPVOID)v); }
+    ~PThreadLocalStorage()        { this->StorageDestroyed(); }
+    Ptr_T Get() const             { return  (Ptr_T)this->GetStorage(); }
+    operator Ptr_T() const        { return  (Ptr_T)this->GetStorage(); }
+    Ptr_T operator->() const      { return  (Ptr_T)this->GetStorage(); }
+    Storage_T & operator*() const { return *(Ptr_T)this->GetStorage(); }
 
   protected:
-    DWORD key;
+    virtual void * Allocate() const           { return new Storage_T(); }
+    virtual void Deallocate(void * ptr) const { delete (Ptr_T)ptr; }
 };
 
-#elif defined(P_PTHREADS)
 
-#include <pthread.h>
+#define P_HAS_THREADLOCAL_STORAGE 1  // For backward compatbility
 
-#define P_HAS_THREADLOCAL_STORAGE 1
-
-template <class Storage_T>
-class PThreadLocalStorage
-{
-  public:
-    typedef pthread_key_t Key_T;
-    typedef Storage_T value_type;
-
-    PThreadLocalStorage()
-    { pthread_key_create(&key, NULL); }
-
-    ~PThreadLocalStorage()
-    { pthread_key_delete(key); }
-
-    Key_T GetKey() const
-    { return key; }
-
-    value_type * Get()
-    { return (value_type *)pthread_getspecific(key); }
-
-    void Set(value_type * v)
-    { pthread_setspecific(key, v); }
-
-  private:
-    Key_T key;
-};
-
-#else
-
-#undef P_HAS_THREADLOCAL_STORAGE 1
-#warning("Thread local storage not supported");
-
-#endif
-
-
-#ifdef _MSC_VER
-#pragma warning(default:4355)
-#endif
 
 #endif // PTLIB_THREAD_H
 
