@@ -26,33 +26,20 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28062 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2012-07-19 04:45:52 -0500 (Thu, 19 Jul 2012) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
 // PMutex
 
 #if defined(P_PTHREADS) || defined(VX_TASKS)
+  public:
     virtual ~PTimedMutex();
-    mutable pthread_mutex_t mutex;
-#endif
-
-#if defined(P_PTHREADS) || defined(__BEOS__) || defined(P_MAC_MPTHREADS) || defined(VX_TASKS)
-    virtual void Wait();
-    virtual PBoolean Wait(const PTimeInterval & timeout);
-    virtual void Signal();
-    virtual PBoolean WillBlock() const;
-
   protected:
-
-#  if defined(P_PTHREADS) && !defined(VX_TASKS)
-#    if P_HAS_RECURSIVE_MUTEX == 0
-       mutable PAtomicInteger lockCount;
-#    endif
-#  endif
-
+    mutable pthread_mutex_t m_mutex;
+    void Construct();
 #endif
 
 
