@@ -26,16 +26,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 29362 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2013-03-28 02:36:25 -0500 (Thu, 28 Mar 2013) $
  */
 
   protected:
-    int toChildPipe[2];
-    int fromChildPipe[2];
-    int stderrChildPipe[2];
-    int childPid;
-    int retVal;
+    int m_toChildPipe[2];
+    int m_fromChildPipe[2];
+    int m_stderrChildPipe[2];
+    int m_childPID;
+    int m_returnCode;
 
 // End Of File ////////////////////////////////////////////////////////////////
