@@ -25,9 +25,9 @@
  *                 Thorsten Westheider (thorsten.westheider@teleos-web.de)
  *                 Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 22477 $
+ * $Revision: 28399 $
  * $Author: rjongbloed $
- * $Date: 2009-04-29 01:03:51 -0500 (Wed, 29 Apr 2009) $
+ * $Date: 2012-09-25 03:35:40 -0500 (Tue, 25 Sep 2012) $
  */
 
 #ifndef PTLIB_CONVERT_H
@@ -94,15 +94,21 @@ class PColourConverter : public PObject
       const PVideoFrameInfo & dst  ///< Destination frame info
     );
 
+    /// Print description of converter
+    virtual void PrintOn(
+      ostream & strm
+    ) const;
+
     /**Get the video conversion vertical flip state
      */
-    PBoolean GetVFlipState() 
-      { return verticalFlip; }
+    bool GetVFlipState() const
+    { return m_verticalFlip; }
     
     /**Set the video conversion vertical flip state
      */
-    void SetVFlipState(PBoolean vFlipState) 
-      { verticalFlip = vFlipState; }
+    void SetVFlipState(
+      bool vFlipState  ///< New state for flipping images
+    ) { m_verticalFlip = vFlipState; }
     
     /**Set the frame size to be used.
 
@@ -116,10 +122,10 @@ class PColourConverter : public PObject
     /**Set the source frame info to be used.
 
        Default behaviour sets the srcFrameWidth and srcFrameHeight variables and
-       recalculates the frame buffer size in bytes then returns PTrue if the size
+       recalculates the frame buffer size in bytes then returns true if the size
        was calculated correctly.
 
-       Returns PFalse if the colour formats do not agree.
+       Returns false if the colour formats do not agree.
     */
     virtual PBoolean SetSrcFrameInfo(
       const PVideoFrameInfo & info   ///< New info for frame
@@ -129,9 +135,9 @@ class PColourConverter : public PObject
 
        Default behaviour sets the dstFrameWidth and dstFrameHeight variables,
        and the scale / crop preference. It then recalculates the frame buffer
-       size in bytes then returns PTrue if the size was calculated correctly.
+       size in bytes then returns true if the size was calculated correctly.
 
-       Returns PFalse if the colour formats do not agree.
+       Returns false if the colour formats do not agree.
     */
     virtual PBoolean SetDstFrameInfo(
       const PVideoFrameInfo & info  ///< New info for frame
@@ -152,7 +158,7 @@ class PColourConverter : public PObject
     /**Set the source frame size to be used.
 
        Default behaviour sets the srcFrameWidth and srcFrameHeight variables and
-       recalculates the frame buffer size in bytes then returns PTrue if the size
+       recalculates the frame buffer size in bytes then returns true if the size
        was calculated correctly.
     */
     virtual PBoolean SetSrcFrameSize(
@@ -164,39 +170,39 @@ class PColourConverter : public PObject
 
        Default behaviour sets the dstFrameWidth and dstFrameHeight variables,
        and the scale / crop preference. It then recalculates the frame buffer
-       size in bytes then returns PTrue if the size was calculated correctly.
+       size in bytes then returns true if the size was calculated correctly.
     */
     virtual PBoolean SetDstFrameSize(
-      unsigned width,  ///< New width of target frame
-      unsigned height ///< New height of target frame
+      unsigned width,   ///< New width of target frame
+      unsigned height   ///< New height of target frame
     );
     virtual PBoolean SetDstFrameSize(
-      unsigned width,  ///< New width of target frame
-      unsigned height, ///< New height of target frame
-      PBoolean bScale
+      unsigned width,   ///< New width of target frame
+      unsigned height,  ///< New height of target frame
+      PBoolean bScale   ///< Indicate if scaling or cropping is to be used
     );
 
     /**Get the source colour format.
       */
-    const PString & GetSrcColourFormat() { return srcColourFormat; }
+    const PString & GetSrcColourFormat() { return m_srcColourFormat; }
 
     /**Get the destination colour format.
       */
-    const PString & GetDstColourFormat() { return dstColourFormat; }
+    const PString & GetDstColourFormat() { return m_dstColourFormat; }
 
     /**Get the maximum frame size in bytes for source frames.
 
        Note a particular device may be able to provide variable length
        frames (eg motion JPEG) so will be the maximum size of all frames.
       */
-    PINDEX GetMaxSrcFrameBytes() { return srcFrameBytes; }
+    PINDEX GetMaxSrcFrameBytes() { return m_srcFrameBytes; }
 
     /**Get the maximum frame size in bytes for destination frames.
 
        Note a particular device may be able to provide variable length
        frames (eg motion JPEG) so will be the maximum size of all frames.
       */
-    PINDEX GetMaxDstFrameBytes() { return dstFrameBytes; }
+    PINDEX GetMaxDstFrameBytes() { return m_dstFrameBytes; }
 
 
     /**Convert from one colour format to another.
@@ -205,7 +211,7 @@ class PColourConverter : public PObject
        where srcFrameBuffer and dstFrameBuffer are the same, if the conversion
        algorithm allows for that to occur without an intermediate frame store.
 
-       The function should return PFalse if srcFrameBuffer and dstFrameBuffer
+       The function should return false if srcFrameBuffer and dstFrameBuffer
        are the same and that form pf conversion is not allowed
     */
     virtual PBoolean Convert(
@@ -217,30 +223,30 @@ class PColourConverter : public PObject
     virtual PBoolean Convert(
       const BYTE * srcFrameBuffer,  ///< Frame store for source pixels
       BYTE * dstFrameBuffer,        ///< Frame store for destination pixels
-      unsigned int srcFrameBytes,
+      unsigned int srcFrameBytes,   ///< Bytes used in source frame buffer
       PINDEX * bytesReturned = NULL ///< Bytes written to dstFrameBuffer
     ) = 0;
 
     /**Convert from one colour format to another.
        This version will copy the data from one frame buffer to the same frame
        buffer. Not all conversions can do this so an intermediate store and
-       copy may be required. If the noIntermediateFrame parameter is PTrue
+       copy may be required. If the noIntermediateFrame parameter is true
        and the conversion cannot be done in place then the function returns
-       PFalse. If the in place conversion can be done then that parameter is
+       false. If the in place conversion can be done then that parameter is
        ignored.
 
        Note that the frame should be large enough to take the destination
        pixels.
 
        Default behaviour calls Convert() from the frameBuffer to itself, and
-       if that returns PFalse then calls it again (provided noIntermediateFrame
-       is PFalse) using an intermediate store, copying the intermediate store
+       if that returns false then calls it again (provided noIntermediateFrame
+       is false) using an intermediate store, copying the intermediate store
        back to the original frame store.
     */
     virtual PBoolean ConvertInPlace(
       BYTE * frameBuffer,               ///< Frame buffer to translate data
       PINDEX * bytesReturned = NULL,    ///< Bytes written to frameBuffer
-      PBoolean noIntermediateFrame = PFalse  ///< Flag to use intermediate store
+      PBoolean noIntermediateFrame = false  ///< Flag to use intermediate store
     );
 
 
@@ -273,20 +279,20 @@ class PColourConverter : public PObject
       unsigned & height ///< Height of source frame
     ) const;
 
-    unsigned GetSrcFrameWidth()  const { return srcFrameWidth;  }
-    unsigned GetSrcFrameHeight() const { return srcFrameHeight; }
-    unsigned GetDstFrameWidth()  const { return dstFrameWidth;  }
-    unsigned GetDstFrameHeight() const { return dstFrameHeight; }
+    unsigned GetSrcFrameWidth()  const { return m_srcFrameWidth;  }
+    unsigned GetSrcFrameHeight() const { return m_srcFrameHeight; }
+    unsigned GetDstFrameWidth()  const { return m_dstFrameWidth;  }
+    unsigned GetDstFrameHeight() const { return m_dstFrameHeight; }
 
     /**Set the resize mode to be used.
     */
     void SetResizeMode(
       PVideoFrameInfo::ResizeMode mode
-    ) { if (mode < PVideoFrameInfo::eMaxResizeMode) resizeMode = mode; }
+    ) { if (mode < PVideoFrameInfo::eMaxResizeMode) m_resizeMode = mode; }
 
     /**Get the resize mode to be used.
     */
-    PVideoFrameInfo::ResizeMode GetResizeMode() const { return resizeMode; }
+    PVideoFrameInfo::ResizeMode GetResizeMode() const { return m_resizeMode; }
 
     /**Convert RGB to YUV.
       */
@@ -310,34 +316,45 @@ class PColourConverter : public PObject
       PVideoFrameInfo::ResizeMode resizeMode
     );
 
+    /**Rotate the video buffer image.
+       At this time, the \p angle may be 90, -90 or 180.
+       Note: if dstYUV is NULL for an in-place rotation, then there may be a
+       performance hit due to allocating an internal buffer and copying back
+       to the source memory.
+      */
+    static bool RotateYUV420P(
+      int angle, unsigned width, unsigned height, BYTE * srcYUV, BYTE * dstYUV = NULL
+    );
+
+    /**Fill a rectangle of the video buffer with the specified colour.
+      */
     static bool FillYUV420P(
-      unsigned x, unsigned y, int width, int height,
+      unsigned x, unsigned y, unsigned width, unsigned height,
       unsigned frameWidth, unsigned frameHeight, BYTE * yuv,
       unsigned r, unsigned g, unsigned b
     );
 
   protected:
-    PString  srcColourFormat;
-    PString  dstColourFormat;
-    unsigned srcFrameWidth;
-    unsigned srcFrameHeight;
-    unsigned srcFrameBytes;
+    void Construct(
+      const PVideoFrameInfo & src, ///< Source frame info (colour formet, size etc)
+      const PVideoFrameInfo & dst  ///< Destination frame info
+    );
+
+    PString  m_srcColourFormat;
+    PString  m_dstColourFormat;
+    unsigned m_srcFrameWidth;
+    unsigned m_srcFrameHeight;
+    PINDEX   m_srcFrameBytes;
 
     // Needed for resizing
-    unsigned dstFrameWidth;
-    unsigned dstFrameHeight;
-    unsigned dstFrameBytes;
+    unsigned m_dstFrameWidth;
+    unsigned m_dstFrameHeight;
+    PINDEX   m_dstFrameBytes;
 
-    PVideoFrameInfo::ResizeMode resizeMode;
-     
-    PBoolean     verticalFlip;
+    PVideoFrameInfo::ResizeMode m_resizeMode;
+    bool                        m_verticalFlip;
 
-    PBYTEArray intermediateFrameStore;
-
-#ifndef P_MACOSX
-      /* Use by the jpeg decompressor */
-    struct jdec_private *jdec;
-#endif
+    PBYTEArray m_intermediateFrameStore;
 
   friend class PColourConverterRegistration;
 };
@@ -363,8 +380,8 @@ static class cls##_Registration : public PColourConverterRegistration { \
 } p_##cls##_registration_instance; \
 PColourConverter * cls##_Registration::Create(const PVideoFrameInfo & src, const PVideoFrameInfo & dst) const \
   { return new cls(src, dst); } \
-PBoolean cls::Convert(const BYTE *srcFrameBuffer, BYTE *dstFrameBuffer, unsigned int p_srcFrameBytes, PINDEX * bytesReturned) \
-  { srcFrameBytes = p_srcFrameBytes;return Convert(srcFrameBuffer, dstFrameBuffer, bytesReturned); } \
+PBoolean cls::Convert(const BYTE *srcFrameBuffer, BYTE *dstFrameBuffer, unsigned int srcFrameBytes, PINDEX * bytesReturned) \
+  { m_srcFrameBytes = srcFrameBytes;return Convert(srcFrameBuffer, dstFrameBuffer, bytesReturned); } \
 PBoolean cls::Convert(const BYTE *srcFrameBuffer, BYTE *dstFrameBuffer, PINDEX * bytesReturned)
 
 
