@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 23962 $
+ * $Revision: 29331 $
  * $Author: rjongbloed $
- * $Date: 2010-01-21 22:39:06 -0600 (Thu, 21 Jan 2010) $
+ * $Date: 2013-03-26 20:22:40 -0500 (Tue, 26 Mar 2013) $
  */
 
 
@@ -48,16 +48,14 @@ class PColourConverter;
 
 class PVideoFrameInfo : public PObject
 {
-  PCLASSINFO(PVideoFrameInfo, PObject);
-
+    PCLASSINFO(PVideoFrameInfo, PObject);
   public:
-    enum ResizeMode
-    {
-        eScale,
-        eCropCentre,
-        eCropTopLeft,
-        eMaxResizeMode
-    };
+    P_DECLARE_ENUM_EX(ResizeMode,eMaxResizeMode,
+      eScale,0,
+      eCropCentre,
+      eCropTopLeft
+    );
+    friend ostream & operator<<(ostream & strm, ResizeMode mode);
 
     enum StandardSizes {
       SQCIFWidth = 128,  SQCIFHeight = 96,
@@ -65,11 +63,14 @@ class PVideoFrameInfo : public PObject
       CIFWidth   = 352,  CIFHeight   = 288,
       CIF4Width  = 704,  CIF4Height  = 576,
       CIF16Width = 1408, CIF16Height = 1152,
-      i480Width  = 640,  i480Height  = 480,
-      p720Width  = 960,  p720Height  = 720,
+      HD480Width = 704,  HD480Height = 480,
+      i480Width  = 704,  i480Height  = 480,
+      HD720Width = 1280, HD720Height = 720,
+      p720Width  = 1280, p720Height  = 720,
+      HD1080Width= 1920, HD1080Height= 1080,
       i1080Width = 1920, i1080Height = 1080,
       HDTVWidth  = 1920, HDTVHeight  = 1080,
-      MaxWidth   = 1920, MaxHeight   = 1152
+      MaxWidth   = 1920, MaxHeight   = 1200
     };
 
     /// Construct video frame information
@@ -82,10 +83,33 @@ class PVideoFrameInfo : public PObject
       ResizeMode      resizeMode = eScale
     );
 
+    /** Compare the two PVideoFrameInfo and return their relative rank.
+        This ranking is by the relative area of the frame resolution, and
+        frame rate if resolution equal. The final check for equality is on
+        the colourFormat. The SAR and resize mode take no part.
+
+       @return
+       <code>LessThan</code>, <code>EqualTo</code> or <code>GreaterThan</code>
+       according to the relative rank of the objects.
+     */
+    virtual Comparison Compare(
+      const PObject & obj   // Object to compare against.
+    ) const;
+
+    /** Output the contents of the object to the stream. The exact output is
+       dependent on the exact semantics of the descendent class. This is
+       primarily used by the standard <code>#operator<<</code> function.
+
+       The default behaviour is to print the class name.
+     */
+    virtual void PrintOn(
+      ostream & strm   // Stream to print the object into.
+    ) const;
+
     /**Set the frame size to be used.
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
@@ -95,11 +119,11 @@ class PVideoFrameInfo : public PObject
     /**Get the frame size being used.
 
        Default behaviour returns the value of the frameWidth and frameHeight
-       variable and returns PTrue.
+       variable and returns true.
     */
     virtual PBoolean GetFrameSize(
-      unsigned & width,
-      unsigned & height
+      unsigned & width, ///< Current width of frame
+      unsigned & height ///< Current height of frame
     ) const;
 
     /** Get the width of the frame being used.
@@ -114,10 +138,42 @@ class PVideoFrameInfo : public PObject
     */
     virtual unsigned GetFrameHeight() const;
 
+    /**Set the Storage Aspect Ratio size to be used.
+
+       Default behaviour sets the sarWidth and sarHeight variables and
+       returns true.
+    */
+    virtual PBoolean SetFrameSar(
+      unsigned width,   ///< New SAR width of frame
+      unsigned height   ///< New SAR height of frame
+    );
+
+     /**Get the Storage Aspect Ratio size being used.
+
+       Default behaviour returns the value of the sarWidth and sarHeight
+       variable and returns true.
+    */
+    virtual PBoolean GetSarSize(
+      unsigned & width, ///< Current SAR width of frame
+      unsigned & height ///< Current SAR height of frame
+    ) const;
+
+    /** Get the width of the Storage Aspect Ratio being used.
+
+        Default behaviour returns the value of the sarWidth variable
+    */
+    virtual unsigned GetSarWidth() const;
+
+    /** Get the height of the Storage Aspect Ratio being used.
+
+        Default behaviour returns the value of the sarHeight variable
+    */
+    virtual unsigned GetSarHeight() const;
+    
     /**Set the video frame rate to be used on the device.
 
        Default behaviour sets the value of the frameRate variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameRate(
       unsigned rate  ///< Frames  per second
@@ -132,7 +188,7 @@ class PVideoFrameInfo : public PObject
     /**Set the colour format to be used.
 
        Default behaviour sets the value of the colourFormat variable and then
-       returns PTrue if not an empty string.
+       returns true if not an empty string.
     */
     virtual PBoolean SetColourFormat(
       const PString & colourFormat // New colour format for device.
@@ -197,9 +253,17 @@ class PVideoFrameInfo : public PObject
       unsigned height     ///< Height to convert
     );
 
+    /**Get all "known" image size names.
+       Returns all standard names for sizes, e.g. "qcif", "cif", "vga",
+       "hd1080" etc.
+      */
+    static PStringArray GetSizeNames();
+
   protected:
     unsigned   frameWidth;
     unsigned   frameHeight;
+    unsigned   sarWidth;
+    unsigned   sarHeight;
     unsigned   frameRate;
     PString    colourFormat;
     ResizeMode resizeMode;
@@ -211,55 +275,60 @@ class PVideoControlInfo : public PObject
   PCLASSINFO(PVideoControlInfo, PObject);
 
  public:
+    P_DECLARE_STREAMABLE_ENUM(Types,
+      Pan,
+      Tilt,
+      Zoom
+    );
 
-	 typedef enum {
-        ControlPan,
-        ControlTilt,
-        ControlZoom
-     } InputControlType;
+    Types m_type;
+    long  m_min;
+    long  m_max;
+    long  m_step;
+    long  m_default;
+    long  m_flags;
+    long  m_current;
 
- 	static PString AsString(const InputControlType & type);
+    PVideoControlInfo()
+      : m_type(EndTypes)
+      , m_min(0)
+      , m_max(0)
+      , m_step(0)
+      , m_default(0)
+      , m_flags(0)
+      , m_current(0)
+    { }
 
-	  InputControlType type;
-	  long					  min;
-	  long					  max;
-	  long					  step;
-	  long					  def;
-	  long					  flags;
-	  long					  current;
+    bool IsValid() const { return m_max > 0 && m_min < m_max; }
 };
 
 
 /**This class defines a video Input device control (Camera controls PTZ)
 */
 
-class PVideoInputControl : public PVideoControlInfo
+class PVideoInputControl : public PObject
 {
-    PCLASSINFO(PVideoInputControl, PVideoControlInfo);
+    PCLASSINFO(PVideoInputControl, PObject);
 
 public:
-	~PVideoInputControl();
+  ~PVideoInputControl();
 
-	virtual PBoolean Pan(long value, bool absolute = false );
-	virtual PBoolean Tilt(long value, bool absolute = false);
-	virtual PBoolean Zoom(long value, bool absolute = false);
+  virtual PBoolean Pan (long value, bool absolute = false);
+  virtual PBoolean Tilt(long value, bool absolute = false);
+  virtual PBoolean Zoom(long value, bool absolute = false);
 
-	long GetPan();
-	long GetTilt();
-	long GetZoom();
+  virtual long GetPan()  const { return m_control[PVideoControlInfo::Pan ].m_current; }
+  virtual long GetTilt() const { return m_control[PVideoControlInfo::Tilt].m_current; }
+  virtual long GetZoom() const { return m_control[PVideoControlInfo::Zoom].m_current; }
 
-	void Reset();
-	void SetCurrentPosition(const InputControlType ctype, long current);
+  virtual void Reset();
 
-	typedef std::list<PVideoControlInfo> InputDeviceControls;
+  bool IsValid(PVideoControlInfo::Types type) const { return m_control[type].IsValid(); }
+  const PVideoControlInfo & GetControl(PVideoControlInfo::Types type) const { return m_control[type]; }
 
 protected:
-	PBoolean GetVideoControlInfo(const InputControlType ctype, PVideoControlInfo & control);
-	PBoolean GetDefaultPosition(const InputControlType ctype, long & def);
-	PBoolean GetCurrentPosition(const InputControlType ctype, long & current);
-
-	std::list<PVideoControlInfo> m_info;
-	PMutex ccmutex;
+  PVideoControlInfo m_control[PVideoControlInfo::NumTypes];
+  PMutex            m_mutex;
 
 };
 
@@ -271,17 +340,17 @@ class PVideoInteractionInfo : public PObject
 
  public:
 
-	 typedef enum {
-        InteractKey,		/// Register remote KeyPresses
-        InteractMouse,		/// Register remote Mouse Movements\Clicks
-        InteractNavigate,	/// Register remote Navigation commands
-		InteractRTSP,		/// Register remote RTSP (Real Time Streaming Protocol) Inputs
-		InteractOther		/// Register remote application specific Inputs
+   typedef enum {
+        InteractKey,    /// Register remote KeyPresses
+        InteractMouse,    /// Register remote Mouse Movement Clicks
+        InteractNavigate,  /// Register remote Navigation commands
+    InteractRTSP,    /// Register remote RTSP (Real Time Streaming Protocol) Inputs
+    InteractOther    /// Register remote application specific Inputs
      } InputInteractType;
 
- 	static PString AsString(const InputInteractType & type);
+   static PString AsString(const InputInteractType & type);
 
-	InputInteractType type;
+  InputInteractType type;
 };
 
 
@@ -307,7 +376,7 @@ class PVideoInteractionInfo : public PObject
      "YUV410P"  YUV 4:1:0 planar
      "MJPEG"    Motion JPEG
      "UYVY422"  YUV 4:2:2 packed as U Y V Y U Y V Y ...
-     "UYV444    YUV 4:4:4 packed as U Y V   U Y V   ...
+     "UYV444"   YUV 4:4:4 packed as U Y V   U Y V   ...
                 They are used in IEEE 1394 digital cameras. The specification
                 is found at
 http://www.1394ta.org/Download/Technology/Specifications/2000/IIDC_Spec_v1_30.pdf
@@ -345,6 +414,17 @@ class PVideoDevice : public PVideoFrameInfo
       */
     virtual PStringArray GetDeviceNames() const = 0;
 
+    struct Attributes {
+      Attributes();
+
+      int m_brightness;
+      int m_contrast;
+      int m_saturation;
+      int m_hue;
+      int m_gamma;
+      int m_exposure;
+    };
+
     struct OpenArgs {
       OpenArgs();
 
@@ -361,25 +441,21 @@ class PVideoDevice : public PVideoFrameInfo
       bool        convertSize;
       ResizeMode  resizeMode;
       bool        flip;
-      int         brightness;
-      int         whiteness;
-      int         contrast;
-      int         colour;
-      int         hue;
+      Attributes  m_attributes;
     };
 
     /**Open the device given the device name.
       */
     virtual PBoolean OpenFull(
       const OpenArgs & args,      ///< Parameters to set on opened device
-      PBoolean startImmediate = PTrue  ///< Immediately start device
+      PBoolean startImmediate = true  ///< Immediately start device
     );
 
     /**Open the device given the device name.
       */
     virtual PBoolean Open(
       const PString & deviceName,   ///< Device name to open
-      PBoolean startImmediate = PTrue    ///< Immediately start device
+      PBoolean startImmediate = true    ///< Immediately start device
     ) = 0;
 
     /**Determine if the device is currently open.
@@ -406,7 +482,7 @@ class PVideoDevice : public PVideoFrameInfo
     /**Set the video format to be used.
 
        Default behaviour sets the value of the videoFormat variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetVideoFormat(
       VideoFormat videoFormat   ///< New video format
@@ -429,7 +505,7 @@ class PVideoDevice : public PVideoFrameInfo
        special value of -1 will find the first working channel number.
 
        Default behaviour sets the value of the channelNumber variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetChannel(
       int channelNumber  ///< New channel number for device.
@@ -441,6 +517,15 @@ class PVideoDevice : public PVideoFrameInfo
     */
     virtual int GetChannel() const;
 
+    /**Set the frame info to be used, trying converters if available.
+
+       This function will simply call SetColourFormatConverter(),
+       SetFrameSizeConverter() and SetFrameRate().
+    */
+    virtual bool SetFrameInfoConverter(
+      const PVideoFrameInfo & info // New frame infofor device.
+    );
+
     /**Set the colour format to be used, trying converters if available.
 
        This function will set the colour format on the device to one that
@@ -452,12 +537,12 @@ class PVideoDevice : public PVideoFrameInfo
     );
 
     /**Get the video conversion vertical flip state.
-       Default action is to return PFalse.
+       Default action is to return false.
      */
     virtual PBoolean GetVFlipState();
 
     /**Set the video conversion vertical flip state.
-       Default action is to return PFalse.
+       Default action is to return false.
      */
     virtual PBoolean SetVFlipState(
       PBoolean newVFlipState    ///< New vertical flip state
@@ -466,7 +551,7 @@ class PVideoDevice : public PVideoFrameInfo
     /**Get the minimum & maximum size of a frame on the device.
 
        Default behaviour returns the value 1 to UINT_MAX for both and returns
-       PFalse.
+       false.
     */
     virtual PBoolean GetFrameSizeLimits(
       unsigned & minWidth,   ///< Variable to receive minimum width
@@ -495,17 +580,29 @@ class PVideoDevice : public PVideoFrameInfo
     virtual PBoolean SetFrameSizeConverter(
       unsigned width,                   ///< New width of frame
       unsigned height,                  ///< New height of frame
-	  PBoolean  /*bScaleNotCrop*/           ///< Not used.
-	  )  { return SetFrameSizeConverter(width,height,eScale); }
+    PBoolean  /*bScaleNotCrop*/           ///< Not used.
+    )  { return SetFrameSizeConverter(width,height,eScale); }
 
 
+    /**Set the nearest available frame size to be used.
+
+       Note that devices may not be able to produce the requested size, so
+       this function picks the nearest available size.
+
+       Default behaviour simply calls SetFrameSize().
+    */
+    virtual PBoolean SetNearestFrameSize(
+      unsigned width,   ///< New width of frame
+      unsigned height   ///< New height of frame
+    );
+
     /**Set the frame size to be used.
 
        Note that devices may not be able to produce the requested size, and
        this function will fail.  See SetFrameSizeConverter().
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       then sets the converter sizes.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
@@ -513,19 +610,29 @@ class PVideoDevice : public PVideoFrameInfo
     );
 
     /**Get the frame size being used.
-
-       Default behaviour returns the value of the frameWidth and frameHeight
-       variable and returns PTrue.
+       If a converter exists, the destination frame size of the converter
+       is returned, not the underlying physical device. If you wish the
+       physical device frame size use PVideoFrameInfo::GetFrameSize().
     */
     virtual PBoolean GetFrameSize(
       unsigned & width,
       unsigned & height
     ) const;
 
-    /**Get the maximum frame size in bytes.
+    /**Get the colour format being used.
+       If a converter exists, the destination colour format of the converter
+       is returned, not the underlying physical device. If you wish the
+       physical device colout format use PVideoFrameInfo::GetColourFormat().
+      */
+    virtual const PString& GetColourFormat() const;
 
-       Note a particular device may be able to provide variable length
-       frames (eg motion JPEG) so will be the maximum size of all frames.
+    /**Get the maximum frame size in bytes.
+       This gets the maximum of the physical device or the converted frame
+       size and colour format. It is guranteed to be enought memory space for
+       any video grab or output.
+
+       Note, a particular device may be able to provide variable length
+       frames (eg motion JPEG) so this will be the maximum size of all frames.
       */
     virtual PINDEX GetMaxFrameBytes() = 0;
 
@@ -539,62 +646,19 @@ class PVideoDevice : public PVideoFrameInfo
      */
     virtual PBoolean CanCaptureVideo() const = 0;
 
-    /**Get the brightness of the image. 0xffff-Very bright. -1 is unknown.
+    /**Get video attributes.
      */
-    virtual int GetBrightness();
-
-    /**Set brightness of the image. 0xffff-Very bright.
-     */
-    virtual PBoolean SetBrightness(unsigned newBrightness);
-
-
-    /**Get the whiteness of the image. 0xffff-Very white. -1 is unknown.
-     */
-    virtual int GetWhiteness();
-
-    /**Set whiteness of the image. 0xffff-Very white.
-     */
-    virtual PBoolean SetWhiteness(unsigned newWhiteness);
-
-
-    /**Get the colour of the image. 0xffff-lots of colour. -1 is unknown.
-     */
-    virtual int GetColour();
-
-    /**Set colour of the image. 0xffff-lots of colour.
-     */
-    virtual PBoolean SetColour(unsigned newColour);
-
-
-    /**Get the contrast of the image. 0xffff-High contrast. -1 is unknown.
-     */
-    virtual int GetContrast();
-
-    /**Set contrast of the image. 0xffff-High contrast.
-     */
-    virtual PBoolean SetContrast(unsigned newContrast);
-
-
-    /**Get the hue of the image. 0xffff-High hue. -1 is unknown.
-     */
-    virtual int GetHue();
-
-    /**Set hue of the image. 0xffff-High hue.
-     */
-    virtual PBoolean SetHue(unsigned newHue);
-    
-    
-    /**Return whiteness, brightness, colour, contrast and hue in one call.
-     */
-    virtual PBoolean GetParameters(
-      int *whiteness,
-      int *brightness,
-      int *colour,
-      int *contrast,
-      int *hue
+    virtual bool GetAttributes(
+      Attributes & attributes
     );
 
-    
+    /**Set video attributes.
+     */
+    virtual bool SetAttributes(
+      const Attributes & attributes
+    );
+
+
     /** Set VideoFormat and VideoChannel in one ioctl
      */
     virtual PBoolean SetVideoChannelFormat (
@@ -627,11 +691,18 @@ class PVideoDevice : public PVideoFrameInfo
     PColourConverter * converter;
     PBYTEArray         frameStore;
 
-    int          frameBrightness; // 16 bit entity, -1 is no value
-    int          frameWhiteness;
-    int          frameContrast;
-    int          frameColour;
-    int          frameHue;
+  private:
+    P_REMOVE_VIRTUAL(int, GetBrightness(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetBrightness(unsigned), false);
+    P_REMOVE_VIRTUAL(int, GetWhiteness(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetWhiteness(unsigned), false);
+    P_REMOVE_VIRTUAL(int, GetColour(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetColour(unsigned), false);
+    P_REMOVE_VIRTUAL(int, GetContrast(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetContrast(unsigned), false);
+    P_REMOVE_VIRTUAL(int, GetHue(), 0);
+    P_REMOVE_VIRTUAL(PBoolean, SetHue(unsigned), false);
+    P_REMOVE_VIRTUAL(PBoolean, GetParameters(int *, int *, int *, int *, int *), false);
 };
 
 
@@ -660,7 +731,7 @@ class PVideoOutputDevice : public PVideoDevice
        If driverName is an empty string or the value "*" then this will return
        a list of unique device names across all of the available drivers. If
        two drivers have identical names for devices, then the string returned
-       will be of the form driver+'\t'+device.
+       will be of the form driver+'\\t'+device.
     */
     static PStringArray GetDriversDeviceNames(
       const PString & driverName,         ///< Name of driver
@@ -692,7 +763,7 @@ class PVideoOutputDevice : public PVideoDevice
     static PVideoOutputDevice *CreateOpenedDevice(
       const PString & driverName,         ///< Name of driver
       const PString & deviceName,         ///< Name of device
-      PBoolean startImmediate = PTrue,         ///< Immediately start display
+      PBoolean startImmediate = true,         ///< Immediately start display
       PPluginManager * pluginMgr = NULL   ///< Plug in manager, use default if NULL
     );
 
@@ -700,20 +771,20 @@ class PVideoOutputDevice : public PVideoDevice
     */
     static PVideoOutputDevice *CreateOpenedDevice(
       const OpenArgs & args,              ///< Parameters to set on opened device
-      PBoolean startImmediate = PTrue          ///< Immediately start display
+      PBoolean startImmediate = true          ///< Immediately start display
     );
 
     /**Close the device.
       */
-    virtual PBoolean Close() { return PTrue; }
+    virtual PBoolean Close() { return true; }
 
     /**Start the video device I/O display.
       */
-    virtual PBoolean Start() { return PTrue; }
+    virtual PBoolean Start() { return true; }
 
     /**Stop the video device I/O display.
       */
-    virtual PBoolean Stop() { return PTrue; }
+    virtual PBoolean Stop() { return true; }
 
     /** Is the device a camera, and obtain video
      */
@@ -722,22 +793,42 @@ class PVideoOutputDevice : public PVideoDevice
     /**Set a section of the output frame buffer.
       */
     virtual PBoolean SetFrameData(
-      unsigned x,
-      unsigned y,
-      unsigned width,
-      unsigned height,
-      const BYTE * data,
-      PBoolean endFrame = PTrue
+      unsigned x,               ///< Horizontal position in frame where data is put
+      unsigned y,               ///< Vertical position in frame where data is put
+      unsigned width,           ///< Width of area in frame where data is put
+      unsigned height,          ///< Height of area in frame where data is put
+      const BYTE * data,        ///< Data to put into the video frame store
+      PBoolean endFrame = true  ///< Indicate no more data for this video frame
     ) = 0;
     virtual PBoolean SetFrameData(
-      unsigned x,
-      unsigned y,
-      unsigned width,
-      unsigned height,
-      const BYTE * data,
-      PBoolean endFrame,
-      unsigned flags
+      unsigned x,               ///< Horizontal position in frame where data is put
+      unsigned y,               ///< Vertical position in frame where data is put
+      unsigned width,           ///< Width of area in frame where data is put
+      unsigned height,          ///< Height of area in frame where data is put
+      const BYTE * data,        ///< Data to put into the video frame store
+      PBoolean endFrame,        ///< Indicate no more data for this video frame
+      bool & keyFrameNeeded     ///< Indicates bad video and a new key frame is required
     );
+    virtual PBoolean SetFrameData(
+      unsigned x,               ///< Horizontal position in frame where data is put
+      unsigned y,               ///< Vertical position in frame where data is put
+      unsigned width,           ///< Width of area in frame where data is put
+      unsigned height,          ///< Height of area in frame where data is put
+      unsigned sarWidth,        ///< Aspect ratio width of area in frame where data is put
+      unsigned sarHeight,       ///< Aspect ratio height of area in frame where data is put
+      const BYTE * data,        ///< Data to put into the video frame store
+      PBoolean endFrame,        ///< Indicate no more data for this video frame
+      bool & keyFrameNeeded,    ///< Indicates bad video and a new key frame is required
+      const void * mark
+    );
+
+    /**Allow the outputdevice decide whether the 
+        decoder should ignore decode hence not render
+        any output. 
+
+        Returns: false if to decode and render.
+      */
+    virtual PBoolean DisableDecode();
 
     /**Get the position of the output device, where relevant. For devices such as
        files, this always returns zeros. For devices such as Windows, this is the
@@ -775,14 +866,14 @@ class PVideoOutputDeviceRGB : public PVideoOutputDevice
     PVideoOutputDeviceRGB();
 
     /**Set the colour format to be used.
-       Note that this function does not do any conversion. If it returns PTrue
+       Note that this function does not do any conversion. If it returns true
        then the video device does the colour format in native mode.
 
        To utilise an internal converter use the SetColourFormatConverter()
        function.
 
        Default behaviour sets the value of the colourFormat variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetColourFormat(
       const PString & colourFormat // New colour format for device.
@@ -794,7 +885,7 @@ class PVideoOutputDeviceRGB : public PVideoOutputDevice
        this function will fail.  See SetFrameSizeConverter().
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
@@ -816,7 +907,7 @@ class PVideoOutputDeviceRGB : public PVideoOutputDevice
       unsigned width,
       unsigned height,
       const BYTE * data,
-      PBoolean endFrame = PTrue
+      PBoolean endFrame = true
     );
 
     /**Indicate frame may be displayed.
@@ -848,7 +939,7 @@ class PVideoOutputDevicePPM : public PVideoOutputDeviceRGB
       */
     virtual PBoolean Open(
       const PString & deviceName,   ///< Device name (filename base) to open
-      PBoolean startImmediate = PTrue    ///< Immediately start device
+      PBoolean startImmediate = true    ///< Immediately start device
     );
 
     /**Determine if the device is currently open.
@@ -899,7 +990,7 @@ class PVideoInputDevice : public PVideoDevice
        If driverName is an empty string or the value "*" then this will return
        a list of unique device names across all of the available drivers. If
        two drivers have identical names for devices, then the string returned
-       will be of the form driver+'\t'+device.
+       will be of the form driver+'\\t'+device.
     */
     static PStringArray GetDriversDeviceNames(
       const PString & driverName,         ///< Name of driver
@@ -934,7 +1025,7 @@ class PVideoInputDevice : public PVideoDevice
     static PVideoInputDevice *CreateOpenedDevice(
       const PString & driverName,         ///< Name of driver
       const PString & deviceName,         ///< Name of device
-      PBoolean startImmediate = PTrue,         ///< Immediately start grabbing
+      PBoolean startImmediate = true,         ///< Immediately start grabbing
       PPluginManager * pluginMgr = NULL   ///< Plug in manager, use default if NULL
     );
 
@@ -942,14 +1033,14 @@ class PVideoInputDevice : public PVideoDevice
     */
     static PVideoInputDevice *CreateOpenedDevice(
       const OpenArgs & args,              ///< Parameters to set on opened device
-      PBoolean startImmediate = PTrue          ///< Immediately start display
+      PBoolean startImmediate = true          ///< Immediately start display
     );
 
-	typedef struct {
-	   std::list<PVideoFrameInfo> framesizes;
-	   std::list<PVideoControlInfo> controls;
-	   std::list<PVideoInteractionInfo> interactions;
-	} Capabilities;
+  typedef struct {
+     std::list<PVideoFrameInfo> framesizes;
+     std::list<PVideoControlInfo> controls;
+     std::list<PVideoInteractionInfo> interactions;
+  } Capabilities;
 
     /**Retrieve a list of Device Capabilities
       */
@@ -974,15 +1065,20 @@ class PVideoInputDevice : public PVideoDevice
       PPluginManager * pluginMgr = NULL     ///< Plug in manager, use default if NULL
     );
 
+    /**Get the devices video Input controls
+        By Default return NULL;
+      */
+    virtual PVideoInputControl * GetVideoInputControls();
+
     /**Open the device given the device name.
       */
     virtual PBoolean Open(
       const PString & deviceName,   ///< Device name to open
-      PBoolean startImmediate = PTrue    ///< Immediately start device
+      PBoolean startImmediate = true    ///< Immediately start device
     ) = 0;
 
     virtual PBoolean Close(
-    ) { return PTrue; }
+    ) { return true; }
 
     /** Is the device a camera, and obtain video
      */
@@ -992,6 +1088,18 @@ class PVideoInputDevice : public PVideoDevice
       */
     virtual PBoolean IsCapturing() = 0;
 
+    /**Set the nearest available frame size to be used.
+
+       Note that devices may not be able to produce the requested size, so
+       this function picks the nearest available size.
+
+       Default behaviour simply calls SetFrameSize().
+    */
+    virtual PBoolean SetNearestFrameSize(
+      unsigned width,   ///< New width of frame
+      unsigned height   ///< New height of frame
+    );
+
     /**Grab a frame.
       */
     virtual PBoolean GetFrame(
@@ -1003,7 +1111,8 @@ class PVideoInputDevice : public PVideoDevice
     virtual PBoolean GetFrameData(
       BYTE * buffer,                 ///< Buffer to receive frame
       PINDEX * bytesReturned,        ///< Optional bytes returned.
-      unsigned int & flags           ///< optional flags returned
+      bool & keyFrame         /**< On input, forces generation of key frame,
+                                   On return indicates key frame generated */
     );
     virtual PBoolean GetFrameData(
       BYTE * buffer,                 ///< Buffer to receive frame
@@ -1015,16 +1124,39 @@ class PVideoInputDevice : public PVideoDevice
     virtual PBoolean GetFrameDataNoDelay(
       BYTE * buffer,                 ///< Buffer to receive frame
       PINDEX * bytesReturned,       ///< Optional bytes returned.
-      unsigned int & flags           ///< optional flags returned
+      bool & keyFrame         /**< On input, forces generation of key frame,
+                                   On return indicates key frame generated */
     );
     virtual PBoolean GetFrameDataNoDelay(
       BYTE * buffer,                 ///< Buffer to receive frame
       PINDEX * bytesReturned = NULL  ///< Optional bytes returned.
     ) = 0;
 
-    /**Try all known video formats & see which ones are accepted by the video driver
-     */
-    virtual PBoolean TestAllFormats() = 0;
+    /**Pass data to the inputdevice for flowControl determination.
+      */
+    virtual bool FlowControl(const void * flowData);
+
+    /**Set the capture modes for implementations that support them.
+       For example with Video For Windows, this is used to select picture (0)
+       or video (1) modes.
+
+       In picture-mode the implementation requests a single frame from the
+       connected camera device. The camera device then does nothing until the
+       frame has been processed and the next is requested.
+
+       In video-mode the camera continuously sends new frames.
+
+       The default implementation does nothing but returns false.
+      */
+    virtual bool SetCaptureMode(unsigned mode);
+
+    /**Returns the current capture mode. See SetCaptureMode() for more details.
+       A return value of -1 indicates an error or the mode is not supported.
+    */
+    virtual int GetCaptureMode() const;
+
+    P_REMOVE_VIRTUAL(PBoolean, GetFrameData(BYTE *, PINDEX *, unsigned &), false);
+    P_REMOVE_VIRTUAL(PBoolean, GetFrameDataNoDelay(BYTE *, PINDEX *, unsigned &), false);
 };
 
 
@@ -1064,9 +1196,6 @@ PPLUGIN_STATIC_LOAD(FakeVideo, PVideoInputDevice);
   PPLUGIN_STATIC_LOAD(DirectShow, PVideoInputDevice);
 #endif
 
-#ifdef P_DIRECTSHOW2
-  PPLUGIN_STATIC_LOAD(DirectShow2, PVideoInputDevice);
-#endif
 
 ////////////////////////////////////////////////////////
 //
@@ -1084,11 +1213,16 @@ template <class className> class PVideoOutputPluginServiceDescriptor : public PD
   static PVideoOutputPluginServiceDescriptor<PVideoOutputDevice_##name> PVideoOutputDevice_##name##_descriptor; \
   PCREATE_PLUGIN(name, PVideoOutputDevice, &PVideoOutputDevice_##name##_descriptor)
 
+#if P_VFW_CAPTURE
 #if _WIN32
   PPLUGIN_STATIC_LOAD(Window, PVideoOutputDevice);
 #endif
+#endif
 
 #if P_SDL
+  #if P_MACOSX
+    #include <SDL_main.h>
+  #endif
   PPLUGIN_STATIC_LOAD(SDL, PVideoOutputDevice);
 #endif
 
@@ -1118,3 +1252,4 @@ class PVideoFont : public PObject
 #endif   // PTLIB_PVIDEOIO_H
 
 // End Of File ///////////////////////////////////////////////////////////////
+
