@@ -99,7 +99,7 @@
  * src/ptlib/common/vconvert.cxx
  *
  * ToDo or Bugs:
- * This module does not implement GetBrightness() etc.  1394 cameras
+ * This module does not implement GetAttributes() etc.  1394 cameras
  * can do both automatic control and manual control of brightness
  * etc., and they are usually set to automatic
  * control. Get/SetBrightness() etc. cannot access manual/automatic
@@ -131,9 +131,9 @@
  *
  *
  *
- * $Revision: 20649 $
- * $Author: dsandras $
- * $Date: 2008-08-10 21:49:13 +0200 (dim 10 aoû 2008) $
+ * $Revision: 29042 $
+ * $Author: rjongbloed $
+ * $Date: 2013-01-29 19:09:11 -0600 (Tue, 29 Jan 2013) $
  */
 
 #pragma implementation "videoio1394dc.h"
@@ -166,7 +166,7 @@ static int num_captured;
 PVideoInputDevice_1394DC::PVideoInputDevice_1394DC()
 {
   handle = NULL;
-  is_capturing = PFalse;
+  is_capturing = false;
   capturing_duration = 10000; // arbitrary large value suffices
 }
 
@@ -209,85 +209,52 @@ PBoolean PVideoInputDevice_1394DC::Open(const PString & devName, PBoolean startI
 {
   if (!kernel_version_ok()) {
     PTRACE(0, "The Linux kernel version is too old.");
-    return PFalse;
+    return false;
   }
 
   if (IsOpen()) {
     PTRACE(0, "You cannot open PVideoInputDevice_1394DC twice.");
-    return PFalse;
+    return false;
   }
 
   if (devName == "/dev/raw1394")
-    UseDMA = PFalse;
+    UseDMA = false;
   // Don't forget /dev/video1394/0
   else if (strncmp(devName, "/dev/video1394", 14) == 0)
-    UseDMA = PTrue;
+    UseDMA = true;
   else {
     PTRACE(0, "devName must be /dev/raw1394 or /dev/video1394");
-    return PFalse;
+    return false;
   }
 
   // See if devName is accessible.
   if (!PFile::Exists(devName)) {
     PTRACE(1, devName << " is not accessible.");
-    return PFalse;
+    return false;
   }
 
-  /*-----------------------------------------------------------------------
-   *  Open ohci and asign handle to it
-   *-----------------------------------------------------------------------*/
-  handle = dc1394_create_handle(0);
+  handle = dc1394_new();
+
   if (handle==NULL)
   {
-    PTRACE(0, "Unable to aquire a raw1394 handle\ndid you insmod the drivers?\n");
-    return PFalse;
+    PTRACE(0, "Unable to aquire a dc1394 handle, did you insmod the drivers?\n");
+    return false;
   }
 
-  /*-----------------------------------------------------------------------
-   *  get the camera nodes and describe them as we find them
-   *-----------------------------------------------------------------------*/
-  int numNodes = raw1394_get_nodecount(handle);
-  camera_nodes = dc1394_get_camera_nodes(handle,&numCameras,0);
-  if (numCameras<1)
+  if (dc1394_camera_enumerate(handle, &camera_list))
+  {
+    PTRACE(0, "Failed to enumerate cameras\n");
+    dc1394_free(handle);
+    handle = NULL;
+    return false;
+  }
+  
+  if (camera_list->num == 0)
   {
     PTRACE(0, "no cameras found :(\n");
-    dc1394_destroy_handle(handle);
+    dc1394_free(handle);
     handle = NULL;
-    return PFalse;
-  }
-
-  /*-----------------------------------------------------------------------
-   *  to prevent the iso-transfer bug from raw1394 system, check if
-   *  camera is highest node. For details see 
-   *  http://linux1394.sourceforge.net/faq.html#DCbusmgmt
-   *  and
-   *  http://sourceforge.net/tracker/index.php?func=detail&aid=435107&group_id=8157&atid=108157
-   *-----------------------------------------------------------------------*/
-  for (int i=0; i<numCameras; i++) {
-    if( camera_nodes[i] == numNodes-1) {
-      PTRACE(0,"Sorry, your camera is the highest numbered node\n"
-             "of the bus, and has therefore become the root node.\n"
-             "The root node is responsible for maintaining \n"
-             "the timing of isochronous transactions on the IEEE \n"
-             "1394 bus.  However, if the root node is not cycle master \n"
-             "capable (it doesn't have to be), then isochronous \n"
-             "transactions will not work.  The host controller card is \n"
-             "cycle master capable, however, most cameras are not.\n"
-             "\n"
-             "The quick solution is to add the parameter \n"
-             "attempt_root=1 when loading the OHCI driver as a \n"
-             "module.  So please do (as root):\n"
-             "\n"
-             "   rmmod ohci1394\n"
-             "   insmod ohci1394 attempt_root=1\n"
-             "\n"
-             "for more information see the FAQ at \n"
-             "http://linux1394.sourceforge.net/faq.html#DCbusmgmt\n"
-             "\n");
-      dc1394_destroy_handle(handle);
-      handle = NULL;
-      return PFalse;
-    }
+    return false;
   }
 
   frameHeight = 240;
@@ -296,22 +263,24 @@ PBoolean PVideoInputDevice_1394DC::Open(const PString & devName, PBoolean startI
   capturing_duration = 10000; // arbitrary large value suffices
   deviceName = devName;
 
+#if 0
   // select the specified input and video format
   if (!SetChannel(channelNumber) ||
       !SetVideoFormat(videoFormat)) {
     PTRACE(1, "SetChannel() or SetVideoFormat() failed");
     Close();
-    return PFalse;
+    return false;
   }
 
   if (startImmediate && !Start()) {
     Close();
-    return PFalse;
+    return false;
   }
 
   // Verify the format that the card accept
   quadlet_t supported_framerates;
   supportedFormat = 0;
+
   if (dc1394_query_supported_framerates(handle, camera_nodes[channelNumber],
 	FORMAT_VGA_NONCOMPRESSED, MODE_320x240_YUV422,
 	&supported_framerates) == DC1394_SUCCESS) {
@@ -323,9 +292,10 @@ PBoolean PVideoInputDevice_1394DC::Open(const PString & devName, PBoolean startI
 	&supported_framerates) == DC1394_SUCCESS) {
      supportedFormat |= DC1394_FORMAT_160x120;
   }
-  
+#endif
+
   PTRACE(3, "Successfully opended\n");
-  return PTrue;
+  return true;
 }
 
 
@@ -340,25 +310,27 @@ PBoolean PVideoInputDevice_1394DC::Close()
   if (IsOpen()) {
     if (IsCapturing())
       Stop();
-    dc1394_destroy_handle(handle);
+    dc1394_free(handle);
     handle = NULL;
-    return PTrue;
+    return true;
   } else
-    return PFalse;
+    return false;
 }
 
 PBoolean PVideoInputDevice_1394DC::Start()
 {
   int dc1394_mode;
-  if (!IsOpen()) return PFalse;
-  if (is_capturing) return PTrue;
+  if (!IsOpen()) return false;
+  if (is_capturing) return true;
+
+#if 0      
   if (frameWidth == 320 && frameHeight == 240)
     dc1394_mode = MODE_320x240_YUV422;
   else if (frameWidth == 160 && frameHeight == 120)
     dc1394_mode = MODE_160x120_YUV444;
   else {
     PTRACE(1, "Frame size is neither 320x240 or 160x120" << frameWidth << "x" << frameHeight);
-    return PFalse;
+    return false;
   }
   PTRACE(1, deviceName << " " << channelNumber);
 
@@ -366,8 +338,8 @@ PBoolean PVideoInputDevice_1394DC::Start()
   if (dc1394_query_supported_framerates(handle, camera_nodes[channelNumber],
           FORMAT_VGA_NONCOMPRESSED, dc1394_mode,
           &supported_framerates) != DC1394_SUCCESS) {
-    PTRACE(1, "dc1394_query_supported_framerates() failed.");
-    return PFalse;
+    PTRACE(1, "dc1394_video_get_supported_framerates() failed.");
+    return false;
   }
 
   int framerate;
@@ -387,7 +359,7 @@ PBoolean PVideoInputDevice_1394DC::Start()
     framerate = FRAMERATE_1_875;
   else {
     PTRACE(1, "Frame rate " << supported_framerates << " is not supported");
-    return PFalse;
+    return false;
   }  
 
   // In order to compile the following line, you need libdc1394 0.9.0 or later.
@@ -409,17 +381,30 @@ PBoolean PVideoInputDevice_1394DC::Start()
                            P_DC1394_DEFAULT_SPEED,
                            framerate,
        &camera)!=DC1394_SUCCESS))
+#else
+  camera = dc1394_camera_new(handle, camera_list->ids[0].guid);
+
+  if (!camera)
+  {
+    PTRACE(0,"The camera \"" << camera_list->ids[0].guid
+             << "\"could not be selected.\n");
+    return false;
+  }
+    
+  if (dc1394_capture_setup(camera, 4, DC1394_CAPTURE_FLAGS_DEFAULT))
+#endif
   {
     PTRACE(0,"unable to setup camera-\n"
              "check " __FILE__ " to make sure\n"
              "that the video mode,framerate and format are\n"
              "supported by your camera\n");
-    return PFalse;
+    return false;
   }
 
   /*-----------------------------------------------------------------------
    *  have the camera start sending us data
    *-----------------------------------------------------------------------*/
+#if 0      
   if (dc1394_start_iso_transmission(handle,camera.node)
       !=DC1394_SUCCESS) 
   {
@@ -428,31 +413,43 @@ PBoolean PVideoInputDevice_1394DC::Start()
       dc1394_dma_release_camera(handle,&camera);
     else
       dc1394_release_camera(handle,&camera);
-    return PFalse;
+#else
+  if (dc1394_video_set_transmission(camera, DC1394_ON))
+  {
+    PTRACE(0, "unable to start camera iso transmission\n");
+#endif
+    return false;
   }
-  is_capturing = PTrue;
+  is_capturing = true;
 #ifdef ESTIMATE_CAPTURE_PERFORMANCE
   PTime now;
   start_time = now.GetTimestamp();
   num_captured = 0;
 #endif
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PVideoInputDevice_1394DC::Stop()
 {
   if (IsCapturing()) {
+#if 0      
     dc1394_stop_iso_transmission(handle,camera.node);
     if (UseDMA) {
     dc1394_dma_unlisten(handle, &camera);
     dc1394_dma_release_camera(handle,&camera);
     } else
       dc1394_release_camera(handle,&camera);
-    is_capturing = PFalse;
-    return PTrue;
+#else
+    dc1394_video_set_transmission(camera, DC1394_OFF);
+    dc1394_capture_stop(camera);
+    dc1394_camera_free(camera);
+#endif
+
+      is_capturing = false;
+    return true;
   } else
-    return PFalse;
+    return false;
 }
 
 
@@ -491,76 +488,9 @@ PBoolean PVideoInputDevice_1394DC::SetVideoFormat(VideoFormat newFormat)
 {
   if (!PVideoDevice::SetVideoFormat(newFormat)) {
     PTRACE(3,"PVideoDevice::SetVideoFormat\t failed for format "<<newFormat);
-    return PFalse;
+    return false;
   }
-  return PTrue;
-}
-
-int PVideoInputDevice_1394DC::GetBrightness()
-{
-  return -1;
-}
-
-
-PBoolean PVideoInputDevice_1394DC::SetBrightness(unsigned newBrightness)
-{
-  return PFalse;
-}
-
-
-int PVideoInputDevice_1394DC::GetHue()
-{
-  return -1;
-}
-
-
-PBoolean PVideoInputDevice_1394DC::SetHue(unsigned newHue)
-{
-  return PFalse;
-}
-
-
-int PVideoInputDevice_1394DC::GetContrast()
-{
-  return -1;
-}
-
-
-PBoolean PVideoInputDevice_1394DC::SetContrast(unsigned newContrast)
-{
-  return PFalse;
-}
-
-PBoolean PVideoInputDevice_1394DC::SetColour(unsigned newColour) 
-{
-  return -1;
-}
-
-int PVideoInputDevice_1394DC::GetColour()
-{
-  return -1;
-}
-
-
-PBoolean PVideoInputDevice_1394DC::SetWhiteness(unsigned newWhiteness) 
-{
-  return PFalse;
-}
-
-int PVideoInputDevice_1394DC::GetWhiteness()
-{
-  return -1;
-}
-
-
-PBoolean PVideoInputDevice_1394DC::GetParameters (int *whiteness, int *brightness,
-                                       int *colour, int *contrast, int *hue)
-{
-  *whiteness = -1;
-  *brightness = -1;
-  *colour = -1;
-  *hue = -1;
-  return PFalse;
+  return true;
 }
 
 
@@ -572,13 +502,13 @@ int PVideoInputDevice_1394DC::GetNumChannels()
 
 PBoolean PVideoInputDevice_1394DC::SetChannel(int newChannel)
 {
-  if (PVideoDevice::SetChannel(newChannel) == PFalse)
-    return PFalse;
+  if (PVideoDevice::SetChannel(newChannel) == false)
+    return false;
   if(IsCapturing()) {
     Stop();
     Start();
   }
-  return PTrue;
+  return true;
 }
 
 
@@ -586,9 +516,9 @@ PBoolean PVideoInputDevice_1394DC::SetChannel(int newChannel)
 PBoolean PVideoInputDevice_1394DC::SetFrameRate(unsigned rate)
 {
   if (!PVideoDevice::SetFrameRate(rate))
-    return PFalse;
+    return false;
 
-  return PTrue;
+  return true;
 }
 
 
@@ -601,7 +531,7 @@ PBoolean PVideoInputDevice_1394DC::GetFrameSizeLimits(unsigned & minWidth,
   maxWidth = 320;
   minHeight = 120;
   maxHeight = 240;
-  return PTrue;
+  return true;
 }
 
 
@@ -613,13 +543,14 @@ PINDEX PVideoInputDevice_1394DC::GetMaxFrameBytes()
 
 PBoolean PVideoInputDevice_1394DC::GetFrameDataNoDelay(BYTE * buffer, PINDEX * bytesReturned)
 {
-  if (!IsCapturing()) return PFalse;
+  if (!IsCapturing()) return false;
 
+#if 0
   PTRACE(3, "We are going to single capture.\n");
   if ((UseDMA && dc1394_dma_single_capture(&camera)!=DC1394_SUCCESS) ||
       (!UseDMA && dc1394_single_capture(handle,&camera)!=DC1394_SUCCESS)){
     PTRACE(1, "dc1394_single_capture() failed.");
-    return PFalse;
+    return false;
   }
   
   PTRACE(3, "single captured, try to convert\n");
@@ -630,8 +561,18 @@ PBoolean PVideoInputDevice_1394DC::GetFrameDataNoDelay(BYTE * buffer, PINDEX * b
     converter->Convert((const BYTE *)camera.capture_buffer, buffer, bytesReturned);
   else {
     PTRACE(1, "Converter must exist. Something goes wrong.");
-    return PFalse;
+    return false;
   }
+#else
+  dc1394video_frame_t* frame;
+  
+  if (dc1394_capture_dequeue(camera, DC1394_CAPTURE_POLICY_WAIT, &frame)) {
+    PTRACE(1, "Could not capture a frame");
+    return false;
+  }
+  
+  // Is further data conversion needed?
+#endif
 
 #ifdef ESTIMATE_CAPTURE_PERFORMANCE
   ++num_captured;
@@ -640,9 +581,12 @@ PBoolean PVideoInputDevice_1394DC::GetFrameDataNoDelay(BYTE * buffer, PINDEX * b
   ::fprintf(stderr, "time %f, num_captured=%d, fps=%f\n", capturing_time, num_captured, num_captured/capturing_time);
 #endif
 
+#if 0
   if (UseDMA)
     dc1394_dma_done_with_buffer(&camera);
-  return PTrue;
+#endif
+
+  return true;
 }
 
 PBoolean PVideoInputDevice_1394DC::GetFrameData(BYTE * buffer, PINDEX * bytesReturned)
@@ -659,7 +603,7 @@ void PVideoInputDevice_1394DC::ClearMapping()
 
 PBoolean PVideoInputDevice_1394DC::TestAllFormats()
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -673,16 +617,16 @@ PBoolean PVideoInputDevice_1394DC::SetFrameSize(unsigned width, unsigned height)
 {
   if (width == 320 && height == 240) {
     if (!(supportedFormat & DC1394_FORMAT_320x240))
-      return PFalse;
+      return false;
     colourFormat = "UYVY422";
   }
   else if (width == 160 && height == 120) {
     if (!(supportedFormat & DC1394_FORMAT_160x120))
-      return PFalse;
+      return false;
     colourFormat = "UYV444";
   }
   else
-    return PFalse;
+    return false;
 
   frameWidth = width;
   frameHeight = height;
@@ -694,7 +638,7 @@ PBoolean PVideoInputDevice_1394DC::SetFrameSize(unsigned width, unsigned height)
     Start();
   }
 
-  return PTrue;
+  return true;
 }
 
 
