@@ -18,9 +18,9 @@
  * Classes to support streaming video input from IEEE 1394 cameras.
  * Detailed explanation can be found at src/ptlib/unix/video4dc1394.cxx
  *
- * $Revision: 20385 $
+ * $Revision: 29042 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2013-01-29 19:09:11 -0600 (Tue, 29 Jan 2013) $
  */
 
 
@@ -32,8 +32,7 @@
 #pragma interface
 #endif
 
-#include <libraw1394/raw1394.h>
-#include <libdc1394/dc1394_control.h>
+#include <dc1394/dc1394.h>
 #include <ptclib/delaychan.h>
 
 /** This class defines a video input device that
@@ -55,7 +54,7 @@ class PVideoInputDevice_1394DC : public PVideoInputDevice
       */
     PBoolean Open(
       const PString & deviceName,   /// Device name to open
-      PBoolean startImmediate = PTrue    /// Immediately start device
+      PBoolean startImmediate = true    /// Immediately start device
     );
 
     /**Determine of the device is currently open.
@@ -107,56 +106,6 @@ class PVideoInputDevice_1394DC : public PVideoInputDevice
     );
 
 
-    /**Get the brightness of the image. 0xffff-Very bright.
-     */
-    int GetBrightness();
-
-    /**Set brightness of the image. 0xffff-Very bright.
-     */
-    PBoolean SetBrightness(unsigned newBrightness);
-
-
-    /**Get the whiteness of the image. 0xffff-Very white.
-     */
-    int GetWhiteness();
-
-    /**Set whiteness of the image. 0xffff-Very white.
-     */
-    PBoolean SetWhiteness(unsigned newWhiteness);
-
-
-    /**Get the colour of the image. 0xffff-lots of colour.
-     */
-    int GetColour();
-
-    /**Set colour of the image. 0xffff-lots of colour.
-     */
-    PBoolean SetColour(unsigned newColour);
-
-
-    /**Get the contrast of the image. 0xffff-High contrast.
-     */
-    int GetContrast();
-
-    /**Set contrast of the image. 0xffff-High contrast.
-     */
-    PBoolean SetContrast(unsigned newContrast);
-
-
-    /**Get the hue of the image. 0xffff-High hue.
-     */
-    int GetHue();
-
-    /**Set hue of the image. 0xffff-High hue.
-     */
-    PBoolean SetHue(unsigned newHue);
-    
-    
-    /**Return whiteness, brightness, colour, contrast and hue in one call.
-     */
-    PBoolean GetParameters (int *whiteness, int *brightness, 
-				int *colour, int *contrast, int *hue);
-
     /**Get the minimum & maximum size of a frame on the device.
     */
     PBoolean GetFrameSizeLimits(
@@ -194,12 +143,12 @@ class PVideoInputDevice_1394DC : public PVideoInputDevice
 
  protected:
     PINDEX frameBytes;
-    raw1394handle_t handle;
+    dc1394_t* handle;
     PBoolean is_capturing;
     PBoolean UseDMA;
-    nodeid_t * camera_nodes;
+    dc1394camera_list_t* camera_list;
     int numCameras;
-    dc1394_cameracapture camera;
+    dc1394camera_t* camera;
     int capturing_duration;
     int supportedFormat;
     PAdaptiveDelay m_pacing;
