@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include "precompile.h"
@@ -48,7 +48,7 @@ class MyUDPSocket : public PUDPSocket
     {
       BYTE buffer[1024];
       Read(buffer, 1024);
-      return PTrue;
+      return true;
     }
 
 };
