@@ -4,9 +4,9 @@
  * PWLib application header file for sound test.
  *
  *
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
  
 #ifndef _AUDIO_MAIN_H
@@ -62,7 +62,7 @@ class TestAudio : public PThread
     TestAudio(TestAudioDevice &master);
     virtual ~TestAudio();
 
-    virtual void Terminate() { keepGoing = PFalse; }
+    virtual void Terminate() { keepGoing = false; }
     void LowerVolume();
     void RaiseVolume();
     
