@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21486 $
+ * $Revision: 29702 $
  * $Author: rjongbloed $
- * $Date: 2008-11-09 17:38:52 -0600 (Sun, 09 Nov 2008) $
+ * $Date: 2013-05-13 22:41:08 -0500 (Mon, 13 May 2013) $
  */
 
 #include "precompile.h"
@@ -79,7 +79,7 @@ void IPV6Test::Main()
   if (args.HasOption('v')) {
     cout << "Product Name: " << GetName() << endl
          << "Manufacturer: " << GetManufacturer() << endl
-         << "Version     : " << GetVersion(PTrue) << endl
+         << "Version     : " << GetVersion(true) << endl
          << "System      : " << GetOSName() << '-'
          << GetOSHardware() << ' '
          << GetOSVersion() << endl;
@@ -87,7 +87,7 @@ void IPV6Test::Main()
   }
 
 #if ! P_HAS_IPV6
-  cout << "error: IPV6 not included in PWLib" << endl;
+  cout << "error: IPV6 not included in PTLib" << endl;
 #else
 
   if (args.HasOption('4')) {
@@ -97,6 +97,7 @@ void IPV6Test::Main()
 
   if (args.HasOption('d')) {
     PString name = args.GetOptionString('d');
+    PIPSocket::SetDefaultIpAddressFamily(AF_INET6);
     PIPSocket::Address addr;
     if (!PIPSocket::GetHostAddress(name, addr)) 
       PError << "error: hostname \"" << name << "\" not found" << endl;
@@ -214,23 +215,55 @@ void IPV6Test::Main()
   }
   {
     // test #8 - check if interface table contains IPV6 addresses
-    cout << "test #8: check if interface table contains IPV6 addresses";
+    cout << "\n\ntest #8: check if interface table contains IPV6 addresses";
 
     PIPSocket::InterfaceTable if_table;
-    PIPSocket::GetInterfaceTable( if_table );
+    if (!PIPSocket::GetInterfaceTable(if_table))
+      cout << "Could not get interface table." << endl;
 
     // Display the interface table
     cout << endl;
-    cout << "The interface table has " << if_table.GetSize()
-         <<" entries" << endl;
+    cout << "The interface table has " << if_table.GetSize() <<" entries" << endl;
 
+    bool hasIPv6 = false;
     for (PINDEX i=0; i < if_table.GetSize(); i++) {
       PIPSocket::InterfaceEntry if_entry = if_table[i];
-      cout << i << " " << if_entry << endl;
+      cout << i << '\t' << if_entry;
+
+      if (if_entry.GetAddress().GetVersion() == 6) {
+        hasIPv6 = true;
+
+        PIPSocket::Address ipv6 = if_entry.GetAddress();
+        if (ipv6.IsLinkLocal())
+          cout << ", \"link local\"";
+        cout << ", scope = " << ipv6.GetIPV6Scope();
+      }
+
+      cout << '\n';
     }
-    cout << "Please do manual check ...";
-    cout << endl;
+    cout << "Interface table " << (hasIPv6 ? "has" : "doesn't have") << " IPv6 entries" << endl;
   }
+  {
+    // test #8b - check if route table contains IPV6 addresses
+    cout << "\n\ntest #8b: check if route table contains IPV6 addresses" << endl;
+
+    PIPSocket::RouteTable rt_table;
+    if (!PIPSocket::GetRouteTable(rt_table))
+      cout << "Could not get route table." << endl;
+
+    // Display the route table
+    cout << "The route table has " << rt_table.GetSize() <<" entries" << endl;
+
+    bool hasIPv6 = false;
+    for (PINDEX i=0; i < rt_table.GetSize(); i++) {
+      PIPSocket::RouteEntry rt_entry = rt_table[i];
+      if (rt_entry.GetDestination().GetVersion() == 6)
+        hasIPv6 = true;
+      cout << i << '\t' << rt_entry << '\n';
+    }
+    cout << "Route table " << (hasIPv6 ? "has" : "doesn't have") << " IPv6 entries" << endl;
+  }
+#if P_URL
   {
     // test #9 - see if URLs decode correctly
     cout << "test #9: Please do manual check if parsing IPV6 URLs works" << endl;
@@ -238,13 +271,22 @@ void IPV6Test::Main()
     PURL url("h323:@[::ffff:220.244.81.10]:1234");
     PString addrStr = url.GetHostName();
     PIPSocket::Address addr;
-    PIPSocket::GetHostAddress(addrStr, addr);
+    if (!PIPSocket::GetHostAddress(addrStr, addr))
+      cout << "Could not get IP address for " << addrStr << endl;
     WORD port = url.GetPort();
     cout << "  host string = " << addrStr << " (should be [::ffff:220.244.81.10])\n"
          << "  address     = " << addr    << " (should be ::ffff:220.244.81.10)\n"
          << "  port        = " << port    << " (should be 1234)\n";
   }
-#endif
+#endif // P_URL
+
+  // test #10 - check PIPSocket::GetGatewayAddress()
+  cout << "test #10: PIPSocket::GetGatewayAddress() - IPv4="
+        << PIPSocket::GetGatewayAddress(4)
+        << ", IPv6="
+        << PIPSocket::GetGatewayAddress(6)
+        << endl;
+#endif // P_HAS_IPV6
 }
 
 // End of File ///////////////////////////////////////////////////////////////
