@@ -21,9 +21,9 @@
  *
  * The author of this code is Derek J Smithies
  * *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Revision: 29155 $
+ * $Author: ededu $
+ * $Date: 2013-02-20 04:26:07 -0600 (Wed, 20 Feb 2013) $
  */
 
 /*! \mainpage Thread safe test of PSafeXXX
@@ -42,9 +42,6 @@
 \author  Derek J Smithies
 
 
-$Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
-
-
 \section secContents Contents
 
 \li \ref secClassListings 
