@@ -23,23 +23,21 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26456 $
+ * $Revision: 28872 $
  * $Author: rjongbloed $
- * $Date: 2011-09-22 00:27:10 -0500 (Thu, 22 Sep 2011) $
+ * $Date: 2013-01-13 05:56:55 -0600 (Sun, 13 Jan 2013) $
  */
 
 #include "precompile.h"
 #include "main.h"
 #include "version.h"
 
+#include  <ptlib/videoio.h>
+#include  <ptlib/vconvert.h>
+
 
 PCREATE_PROCESS(VidTest);
 
-#include  <ptlib/video.h>
-#include  <ptlib/vconvert.h>
-#include  <ptclib/vsdl.h>
-
-
 VidTest::VidTest()
   : PProcess("PwLib Video Example", "vidtest", 1, 0, ReleaseCode, 0)
   , m_grabber(NULL)
@@ -53,43 +51,23 @@ void VidTest::Main()
 {
   PArgList & args = GetArguments();
 
-  args.Parse("h-help."
-             "-input-driver:"
-             "I-input-device:"
-             "-input-format:"
-             "-input-channel:"
-             "-output-driver:"
-             "O-output-device:"
-             "T-time:"
+  args.Parse("-input-driver: video grabber driver.\n"
+             "I-input-device: video grabber device.\n"
+             "-input-format: video grabber format (\"pal\"/\"ntsc\")\n"
+             "-input-channel: video grabber channel.\n"
+             "-output-driver: video display driver to use.\n"
+             "O-output-device: video display device to use.\n"
+             "T-time: time in seconds to run test, no command line\n"
 #if PTRACING
-             "o-output:"             "-no-output."
-             "t-trace."              "-no-trace."
-#endif
-       );
-
-#if PTRACING
-  PTrace::Initialise(args.GetOptionCount('t'),
-                     args.HasOption('o') ? (const char *)args.GetOptionString('o') : NULL,
-         PTrace::Blocks | PTrace::Timestamp | PTrace::Thread | PTrace::FileAndLine);
+             "o-output: file name for output of log messages\n"
+             "t-trace. degree of verbosity in log (more times for more detail)\n"
 #endif
+             "h-help."
+             );
 
   if (args.HasOption('h')) {
-    PError << "usage: vidtest [options] [descriptor ...]\n"
-              "\n"
-              "Available options are:\n"
-              "   --help                 : print this help message.\n"
-              "   --input-driver  drv    : video grabber driver.\n"
-              "   -I --input-device dev  : video grabber device.\n"
-              "   --input-format  fmt    : video grabber format (\"pal\"/\"ntsc\")\n"
-              "   --input-channel num    : video grabber channel.\n"
-              "   --output-driver drv    : video display driver to use.\n"
-              "   -O --output-device dev : video display device to use.\n"
-              "   -T --time              : time in seconds to run test, no command line\n"
-#if PTRACING
-              "   -o or --output file   : file name for output of log messages\n"       
-              "   -t or --trace         : degree of verbosity in log (more times for more detail)\n"     
-#endif
-              "\n"
+    args.Usage(cerr, "[options] [descriptor ...]")
+           << "\n"
               "The dscriptor arguments are of the form:\n"
               "    [colour ':' ] size [ '@' rate][ \"/crop\" ].\n"
               "The size component is size is one of the prefined names \"qcif\", \"cif\", \"vga\"\n"
@@ -104,10 +82,13 @@ void VidTest::Main()
               "If they cannot do the format natively, or there are intermediate formats, then\n"
               "video converters are provided.\n"
               "\n"
-              " e.g. ./vidtest --input-device fake --input-channel 2 YUV420P/qcif" << endl << endl;
+              " e.g. ./vidtest --input-device fake --input-channel 2 YUV420P/qcif"
+           << endl;
     return;
   }
 
+  PTRACE_INITIALISE(args, PTrace::Blocks|PTrace::Timestamp|PTrace::Thread|PTrace::FileAndLine);
+
 
   /////////////////////////////////////////////////////////////////////
 
@@ -142,7 +123,9 @@ void VidTest::Main()
   if (m_grabber->GetDeviceCapabilities(&caps)) {
     cout << "Grabber " << inputDeviceName << " capabilities." << endl;
     for (std::list<PVideoFrameInfo>::const_iterator r = caps.framesizes.begin(); r != caps.framesizes.end(); ++r)
-      cout << "    " << r->GetColourFormat() << ' ' << r->GetFrameWidth() << 'x' << r->GetFrameHeight() << ' ' << r->GetFrameRate() << "fps\n";
+      cout << "    " << r->GetColourFormat() << ' '
+           << r->GetFrameWidth() << 'x' << r->GetFrameHeight()
+           << ' ' << r->GetFrameRate() << "fps\n";
     cout << endl;
   }
   else
@@ -253,20 +236,6 @@ void VidTest::Main()
 
   /////////////////////////////////////////////////////////////////////
 
-  if (!m_grabber->SetFrameRate(grabberInfo.GetFrameRate())) {
-    cerr << "Video input device could not be set to frame rate " << grabberInfo.GetFrameRate() << endl;
-    return;
-  }
-  cout << "Grabber frame rate set to " << m_grabber->GetFrameRate() << endl;
-
-
-  if (!m_grabber->SetFrameSizeConverter(grabberInfo.GetFrameWidth(), grabberInfo.GetFrameHeight(), grabberInfo.GetResizeMode())) {
-    cerr << "Video input device could not be set to size " << grabberInfo.GetFrameWidth() << 'x' << grabberInfo.GetFrameHeight() << endl;
-    return;
-  }
-  cout << "Grabber frame size set to " << m_grabber->GetFrameWidth() << 'x' << m_grabber->GetFrameHeight() << endl;
-
-
   if (!m_grabber->SetColourFormatConverter(grabberInfo.GetColourFormat()) ) {
     cerr << "Video input device could not be set to colour format \"" << grabberInfo.GetColourFormat() << '"' << endl;
     return;
@@ -280,19 +249,26 @@ void VidTest::Main()
   cout << ')' << endl;
 
 
-  /////////////////////////////////////////////////////////////////////
-
-  if  (!m_display->SetFrameSizeConverter(displayInfo.GetFrameWidth(), displayInfo.GetFrameHeight(), displayInfo.GetResizeMode())) {
-    cerr << "Video output device could not be set to size " << displayInfo.GetFrameWidth() << 'x' << displayInfo.GetFrameHeight() << endl;
+  if (!m_grabber->SetFrameSizeConverter(grabberInfo.GetFrameWidth(), grabberInfo.GetFrameHeight(), grabberInfo.GetResizeMode())) {
+    cerr << "Video input device could not be set to size " << grabberInfo.GetFrameWidth() << 'x' << grabberInfo.GetFrameHeight() << endl;
     return;
   }
+  cout << "Grabber frame size set to " << m_grabber->GetFrameWidth() << 'x' << m_grabber->GetFrameHeight() << endl;
 
-  cout << "Display frame size set to " << m_display->GetFrameWidth() << 'x' << m_display->GetFrameHeight() << endl;
+
+  if (!m_grabber->SetFrameRate(grabberInfo.GetFrameRate())) {
+    cerr << "Video input device could not be set to frame rate " << grabberInfo.GetFrameRate() << endl;
+    return;
+  }
+  cout << "Grabber frame rate set to " << m_grabber->GetFrameRate() << endl;
+
+
+  /////////////////////////////////////////////////////////////////////
 
   if (!m_display->SetColourFormatConverter(displayInfo.GetColourFormat())) {
     cerr << "Video output device could not be set to colour format \"" << displayInfo.GetColourFormat() << '"' << endl;
     return;
- }
+  }
 
   cout << "Diaplay colour format set to " << m_display->GetColourFormat() << " (";
   if (displayInfo.GetColourFormat() == m_display->GetColourFormat())
@@ -302,6 +278,14 @@ void VidTest::Main()
   cout << ')' << endl;
 
 
+  if  (!m_display->SetFrameSizeConverter(displayInfo.GetFrameWidth(), displayInfo.GetFrameHeight(), displayInfo.GetResizeMode())) {
+    cerr << "Video output device could not be set to size " << displayInfo.GetFrameWidth() << 'x' << displayInfo.GetFrameHeight() << endl;
+    return;
+  }
+
+  cout << "Display frame size set to " << m_display->GetFrameWidth() << 'x' << m_display->GetFrameHeight() << endl;
+
+
   /////////////////////////////////////////////////////////////////////
 
   PThread::Create(PCREATE_NOTIFIER(GrabAndDisplay), 0,
