@@ -10,11 +10,13 @@
  * Copied by Derek Smithies, 1)removed all the wxwidget stuff.
  *                           2)turned into a console application.
  *
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
+#include <ptlib.h>
+
 #include "main.h"
 #include "version.h"
 
@@ -30,8 +32,8 @@
 XMPPFrame::XMPPFrame()
   : m_Roster(new XMPP::Roster), m_Client(NULL)
 {
-  m_Roster->RosterChangedHandlers().Add(new PCREATE_SMART_NOTIFIER(OnRosterChanged));
-  isReadyForUse = PFalse;
+  m_Roster->RosterChangedHandlers().Add(PCREATE_NOTIFIER(OnRosterChanged));
+  isReadyForUse = false;
 }
 
 
@@ -45,12 +47,12 @@ void XMPPFrame::OnConnect()
   //  cout << "On Connect is running " << endl;
 
   m_Client = new XMPP::C2S::StreamHandler(XMPPConsole::Current().GetServer(), XMPPConsole::Current().GetPassword());
-  m_Client->SessionEstablishedHandlers().Add(new PCREATE_SMART_NOTIFIER(OnSessionEstablished));
-  m_Client->SessionReleasedHandlers().Add   (new PCREATE_SMART_NOTIFIER(OnSessionReleased));
-  m_Client->MessageHandlers().Add           (new PCREATE_SMART_NOTIFIER(OnMessage));
-  m_Client->ErrorHandlers().Add             (new PCREATE_SMART_NOTIFIER(OnError));
-  m_Client->PresenceHandlers().Add          (new PCREATE_SMART_NOTIFIER(OnPresence));
-  m_Client->IQHandlers().Add                (new PCREATE_SMART_NOTIFIER(OnIQ));
+  m_Client->SessionEstablishedHandlers().Add(PCREATE_NOTIFIER(OnSessionEstablished));
+  m_Client->SessionReleasedHandlers().Add   (PCREATE_NOTIFIER(OnSessionReleased));
+  m_Client->MessageHandlers().Add           (PCREATE_NOTIFIER(OnMessage));
+  m_Client->ErrorHandlers().Add             (PCREATE_NOTIFIER(OnError));
+  m_Client->PresenceHandlers().Add          (PCREATE_NOTIFIER(OnPresence));
+  m_Client->IQHandlers().Add                (PCREATE_NOTIFIER(OnIQ));
   m_Client->Start();
 }
 
@@ -78,7 +80,7 @@ void XMPPFrame::OnQuit()
   OnDisconnect();
 }
 
-void XMPPFrame::OnIQ(XMPP::IQ& pdu, INT)
+void XMPPFrame::OnIQ(XMPP::IQ& , INT)
 {
   // cout << "On IQ for " << pdu << endl;
 }
@@ -117,7 +119,7 @@ void XMPPFrame::OnPresence(XMPP::Presence& pdu, INT)
 #endif
 }
 
-void XMPPFrame::OnSessionEstablished(XMPP::C2S::StreamHandler& client, INT)
+void XMPPFrame::OnSessionEstablished(XMPP::C2S::StreamHandler& , INT)
 {
   cout << "On Session Established." << endl;
 
@@ -131,10 +133,10 @@ void XMPPFrame::OnReadyForUse(PTimer &, INT)
 {
   //  cout << "Called 1 second after session established " << endl;
 
-  isReadyForUse = PTrue;
+  isReadyForUse = true;
 }
 
-void XMPPFrame::OnSessionReleased(XMPP::C2S::StreamHandler& client, INT)
+void XMPPFrame::OnSessionReleased(XMPP::C2S::StreamHandler& , INT)
 {
   cout << "Disconnected" << endl;
   m_Roster->Detach();
@@ -149,7 +151,7 @@ void XMPPFrame::OnMessage(XMPP::Message& msg, INT)
     cout << " Message received. Subject:" << msg.GetSubject() << "    body:" << msg.GetBody() << "  from: " << msg.GetFrom() << endl;
 }
 
-void XMPPFrame::OnRosterChanged(XMPP::Roster& rost, INT)
+void XMPPFrame::OnRosterChanged(XMPP::Roster& , INT)
 {
   //  cout << " ON Roster changed " << endl;
   //cout << endl;
@@ -239,7 +241,7 @@ void XMPPConsole::Main()
 #ifdef PMEMORY_CHECK
              "-setallocationbreakpoint:"
 #endif       
-	     , PFalse);
+	     , false);
   
   #if PMEMORY_CHECK
   if (args.HasOption("setallocationbreakpoint"))
@@ -279,7 +281,7 @@ void XMPPConsole::Main()
     cout << endl
 	 << "Product Name: " <<  (const char *)GetName() << endl
 	 << "Manufacturer: " <<  (const char *)GetManufacturer() << endl
-	 << "Version     : " <<  (const char *)GetVersion(PTrue) << endl
+	 << "Version     : " <<  (const char *)GetVersion(true) << endl
 	 << "System      : " <<  (const char *)GetOSName() << " - "
 	 << (const char *)GetOSHardware() << " - "
 	 << (const char *)GetOSVersion()  << endl
@@ -337,7 +339,7 @@ void UserInterface::Main()
  
   console.SetReadTimeout(P_MAX_INDEX);
   for (;;) {
-    char ch = console.ReadChar();
+    int ch = console.ReadChar();
  
     switch (tolower(ch)) {
 
@@ -440,7 +442,7 @@ void UserInterface::ProcessDirectedMessage(PString & message)
     return;
   }
 
-  PStringArray bits = message.Tokenise(" ", PFalse);
+  PStringArray bits = message.Tokenise(" ", false);
   if (bits.GetSize() < 3) {
     cout << "a directed message consists of a node number to call, subject, and a message" << endl;
     return;
