@@ -25,9 +25,9 @@
  *
  * $Log: asner.cxx,v $
  * Revision 1.93  2005/11/30 12:47:41  csoutheren
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -71,15 +71,15 @@ inline PBoolean CheckByteOffset(PINDEX offset, PINDEX upper = MaximumStringSize)
   return (0 <= offset && offset <= upper);
 }
 
-static PINDEX FindNameByValue(const PASN_Names *names, unsigned namesCount, PINDEX value)
+static unsigned FindNameByValue(const PASN_Names *names, unsigned namesCount, unsigned value)
 {
   if (names != NULL) {
-    for (unsigned int i = 0;i < namesCount;i++) {
+    for (unsigned i = 0;i < namesCount;i++) {
       if (names[i].value == value)
         return i;
     }
   }
-  return P_MAX_INDEX;
+  return UINT_MAX;
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -438,10 +438,10 @@ void PASN_Integer::Encode(PASN_Stream & strm) const
 ///////////////////////////////////////////////////////////////////////
 
 PASN_Enumeration::PASN_Enumeration(unsigned val)
-: PASN_Object(UniversalEnumeration, UniversalTagClass, PFalse),names(NULL),namesCount(0)
+: PASN_Object(UniversalEnumeration, UniversalTagClass, false),names(NULL),namesCount(0)
 {
   value = val;
-  maxEnumValue = P_MAX_INDEX;
+  maxEnumValue = UINT_MAX;
 }
 
 
@@ -495,8 +495,8 @@ PObject * PASN_Enumeration::Clone() const
 
 void PASN_Enumeration::PrintOn(ostream & strm) const
 {
-  PINDEX idx = FindNameByValue(names, namesCount, value);
-  if (idx != P_MAX_INDEX)
+  unsigned idx = FindNameByValue(names, namesCount, value);
+  if (idx != UINT_MAX)
     strm << names[idx].name;
   else
     strm << '<' << value << '>';
@@ -728,7 +728,7 @@ PBoolean PASN_ObjectId::CommonDecode(PASN_Stream & strm, unsigned dataLen)
 
   // handle zero length strings correctly
   if (dataLen == 0)
-    return PTrue;
+    return true;
 
   unsigned subId;
 
@@ -740,7 +740,7 @@ PBoolean PASN_ObjectId::CommonDecode(PASN_Stream & strm, unsigned dataLen)
     subId = 0;
     do {    /* shift and add in low order 7 bits */
       if (strm.IsAtEnd())
-        return PFalse;
+        return false;
       byte = strm.ByteDecode();
       subId = (subId << 7) + (byte & 0x7f);
       dataLen--;
@@ -768,7 +768,7 @@ PBoolean PASN_ObjectId::CommonDecode(PASN_Stream & strm, unsigned dataLen)
     value[1] = subId-80;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -910,17 +910,17 @@ void PASN_BitString::SetData(unsigned nBits, const BYTE * buf, PINDEX size)
 PBoolean PASN_BitString::SetSize(unsigned nBits)
 {
   if (!CheckByteOffset(nBits))
-    return PFalse;
+    return false;
 
   if (constraint == Unconstrained)
     totalBits = nBits;
   else if (totalBits < (unsigned)lowerLimit) {
     if (lowerLimit < 0)
-      return PFalse;
+      return false;
     totalBits = lowerLimit;
   } else if ((unsigned)totalBits > upperLimit) {
     if (upperLimit > (unsigned)MaximumSetSize)
-      return PFalse;
+      return false;
     totalBits = upperLimit;
   } else
     totalBits = nBits;
@@ -932,7 +932,7 @@ bool PASN_BitString::operator[](PINDEX bit) const
 {
   if ((unsigned)bit < totalBits)
     return (bitData[bit>>3] & (1 << (7 - (bit&7)))) != 0;
-  return PFalse;
+  return false;
 }
 
 
@@ -978,7 +978,7 @@ PObject * PASN_BitString::Clone() const
 
 void PASN_BitString::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  int indent = (int)strm.precision() + 2;
   ios::fmtflags flags = strm.flags();
 
   if (totalBits > 128)
@@ -1118,7 +1118,7 @@ PString PASN_OctetString::AsString() const
 {
   if (value.IsEmpty())
     return PString();
-  return PString((const char *)(const BYTE *)value, value.GetSize());
+  return PString(value);
 }
 
 
@@ -1139,7 +1139,7 @@ PObject * PASN_OctetString::Clone() const
 
 void PASN_OctetString::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  int indent = (int)strm.precision() + 2;
   ios::fmtflags flags = strm.flags();
 
   strm << ' ' << value.GetSize() << " octets {\n"
@@ -1187,16 +1187,16 @@ PINDEX PASN_OctetString::GetDataLength() const
 PBoolean PASN_OctetString::SetSize(PINDEX newSize)
 {
   if (!CheckByteOffset(newSize, MaximumStringSize))
-    return PFalse;
+    return false;
 
   if (constraint != Unconstrained) {
     if (newSize < (PINDEX)lowerLimit) {
       if (lowerLimit < 0)
-        return PFalse;
+        return false;
       newSize = lowerLimit;
     } else if ((unsigned)newSize > upperLimit) {
       if (upperLimit > (unsigned)MaximumStringSize)
-        return PFalse;
+        return false;
       newSize = upperLimit;
     }
   }
@@ -1479,34 +1479,27 @@ PASN_BMPString & PASN_BMPString::operator=(const PASN_BMPString & other)
 PBoolean PASN_BMPString::IsLegalCharacter(WORD ch)
 {
   if (ch < firstChar)
-    return PFalse;
+    return false;
 
   if (ch > lastChar)
-    return PFalse;
+    return false;
 
   if (characterSet.IsEmpty())
-    return PTrue;
+    return true;
 
   const wchar_t * wptr = characterSet;
   PINDEX count = characterSet.GetSize();
   while (count-- > 0) {
     if (*wptr == ch)
-      return PTrue;
+      return true;
     wptr++;
   }
 
-  return PFalse;
+  return false;
 }
 
-
-PASN_BMPString & PASN_BMPString::operator=(const PWCharArray & array)
+void PASN_BMPString::SetValueRaw(const wchar_t * array, PINDEX paramSize)
 {
-  PINDEX paramSize = array.GetSize();
-
-  // Remove trailing NULL character, if present
-  if (paramSize > 0 && array[paramSize-1] == 0)
-    paramSize--;
-
   // Can't copy any more than the upper constraint
   if ((unsigned)paramSize > upperLimit)
     paramSize = upperLimit;
@@ -1526,6 +1519,18 @@ PASN_BMPString & PASN_BMPString::operator=(const PWCharArray & array)
   while (count < newSize)
     value[count++] = firstChar;
 
+}
+
+PASN_BMPString & PASN_BMPString::operator=(const PWCharArray & array)
+{
+  PINDEX paramSize = array.GetSize();
+
+  // Remove trailing NULL character, if present
+  if (paramSize > 0 && array[paramSize-1] == 0)
+    paramSize--;
+
+  SetValueRaw(array, paramSize);
+ 
   return *this;
 }
 
@@ -1601,7 +1606,7 @@ PObject::Comparison PASN_BMPString::Compare(const PObject & obj) const
 
 void PASN_BMPString::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  int indent = (int)strm.precision() + 2;
   PINDEX sz = value.GetSize();
   strm << ' ' << sz << " characters {\n";
   PINDEX i = 0;
@@ -1840,7 +1845,7 @@ PString PASN_Choice::GetTagName() const
 PBoolean PASN_Choice::CheckCreate() const
 {
   if (choice != NULL)
-    return PTrue;
+    return true;
 
   return ((PASN_Choice *)this)->CreateObject();
 }
@@ -1956,7 +1961,7 @@ PBoolean PASN_Choice::IsPrimitive() const
 {
   if (CheckCreate())
     return choice->IsPrimitive();
-  return PFalse;
+  return false;
 }
 
 
@@ -2078,7 +2083,7 @@ PObject * PASN_Sequence::Clone() const
 
 void PASN_Sequence::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  int indent = (int)strm.precision() + 2;
   strm << "{\n";
   for (PINDEX i = 0; i < fields.GetSize(); i++) {
     strm << setw(indent+6) << "field[" << i << "] <";
@@ -2122,7 +2127,7 @@ PINDEX PASN_Sequence::GetDataLength() const
 
 PBoolean PASN_Sequence::IsPrimitive() const
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -2228,21 +2233,21 @@ PASN_Array & PASN_Array::operator=(const PASN_Array & other)
 PBoolean PASN_Array::SetSize(PINDEX newSize)
 {
   if (newSize > MaximumArraySize)
-    return PFalse;
+    return false;
 
   PINDEX originalSize = array.GetSize();
   if (!array.SetSize(newSize))
-    return PFalse;
+    return false;
 
   for (PINDEX i = originalSize; i < newSize; i++) {
     PASN_Object * obj = CreateObject();
     if (obj == NULL)
-      return PFalse;
+      return false;
 
     array.SetAt(i, obj);
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -2256,7 +2261,7 @@ PObject::Comparison PASN_Array::Compare(const PObject & obj) const
 
 void PASN_Array::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  int indent = (int)strm.precision() + 2;
   strm << array.GetSize() << " entries {\n";
   for (PINDEX i = 0; i < array.GetSize(); i++)
     strm << setw(indent+1) << "[" << i << "]=" << setprecision(indent) << array[i] << '\n';
@@ -2296,7 +2301,7 @@ PINDEX PASN_Array::GetDataLength() const
 
 PBoolean PASN_Array::IsPrimitive() const
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -2343,7 +2348,7 @@ void PASN_Stream::Construct()
 
 void PASN_Stream::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  int indent = (int)strm.precision() + 2;
   strm << " size=" << GetSize()
        << " pos=" << byteOffset << '.' << (8-bitOffset)
        << " {\n";
