@@ -26,22 +26,28 @@
  * with the assistance of funding from US Joint Forces Command Joint Concept Development &
  * Experimentation (J9) http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 23799 $
- * $Author: rjongbloed $
- * $Date: 2009-11-24 23:24:03 -0600 (Tue, 24 Nov 2009) $
+ * Further assistance for enhancements from Imagicle spa
+ *
+ * $Revision: 28935 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:12:14 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
 #include <ptclib/cli.h>
 
+#if P_CLI
+
+#include <ptclib/telnet.h>
+
 
 ///////////////////////////////////////////////////////////////////////////////
 
 PCLI::Context::Context(PCLI & cli)
   : m_cli(cli)
-  , m_ignoreNextLF(false)
+  , m_ignoreNextEOL(false)
   , m_thread(NULL)
-  , m_processingCommand(false)
+  , m_state(cli.GetUsername().IsEmpty() ? (cli.GetPassword().IsEmpty() ? e_CommandEntry : e_Password) : e_Username)
 {
 }
 
@@ -116,9 +122,22 @@ void PCLI::Context::Stop()
 }
 
 
+bool PCLI::Context::Run()
+{
+  if (!IsOpen())
+    return false;
+
+  OnStart();
+  while (ReadAndProcessInput())
+    ;
+  OnStop();
+  return true;
+}
+
+
 void PCLI::Context::OnStart()
 {
-  *this << m_cli.GetPrompt() << ::flush;
+  WritePrompt();
 }
 
 
@@ -127,6 +146,26 @@ void PCLI::Context::OnStop()
 }
 
 
+bool PCLI::Context::WritePrompt()
+{
+  switch (m_state) {
+    case e_Username :
+      if (!m_cli.GetUsername().IsEmpty())
+        return WriteString(m_cli.GetUsernamePrompt());
+      // Do next case
+
+    case e_Password :
+      SetLocalEcho(false);
+      if (!m_cli.GetPassword().IsEmpty())
+        return WriteString(m_cli.GetPasswordPrompt());
+      // Do next case
+
+    default :
+      return WriteString(m_cli.GetPrompt());
+  }
+}
+
+
 bool PCLI::Context::ReadAndProcessInput()
 {
   if (!IsOpen())
@@ -138,43 +177,99 @@ bool PCLI::Context::ReadAndProcessInput()
     return false;
   }
   
-  ProcessInput((char)ch);
+  return ProcessInput(ch);
+}
+
+bool PCLI::Context::ProcessInput(const PString & str)
+{
+  PStringArray lines = str.Lines();
+
+  PINDEX i;
+  for (i = 0; i < lines.GetSize(); ++i) {
+    PINDEX j;
+    PString & line = lines[i];
+    for (j = 0; j < line.GetLength(); ++j)
+      if (!ProcessInput(line[j]))
+        return false;
+    if (!ProcessInput('\n'))
+      return false;
+  }
   return true;
 }
 
-
-void PCLI::Context::ProcessInput(char ch)
+bool PCLI::Context::ProcessInput(int ch)
 {
-  switch (ch) {
-    case '\n' :
-      if (m_ignoreNextLF) {
-        m_ignoreNextLF = false;
-        return;
-      }
-      break;
-
-    case '\r' :
-      m_ignoreNextLF = true;
-      break;
-
-    case '\b' : // Backspace
-    case '\x7f' : // Delete
+  if (ch != '\n' && ch != '\r') {
+    if (m_cli.GetEditCharacters().Find((char)ch) != P_MAX_INDEX) {
       if (!m_commandLine.IsEmpty()) {
         m_commandLine.Delete(m_commandLine.GetLength()-1, 1);
-        WriteString("\b \b");
+        if (m_cli.GetRequireEcho() && m_state != e_Password) {
+          if (!WriteString("\b \b"))
+            return false;
+        }
       }
-      // Do next case
+    }
+    else if (ch > 0 && ch < 256 && isprint(ch)) {
+      m_commandLine += (char)ch;
 
+      if (m_cli.GetRequireEcho() && m_state != e_Password) {
+        if (!WriteChar(ch))
+          return false;
+      }
+    }
+
+    m_ignoreNextEOL = false;
+    return true;
+  }
+
+  if (m_ignoreNextEOL) {
+    m_ignoreNextEOL = false;
+    return true;
+  }
+
+  m_ignoreNextEOL = true;
+
+  switch (m_state) {
+    case e_Username :
+      if (m_cli.GetPassword().IsEmpty()) {
+        if (m_cli.OnLogIn(m_commandLine, PString::Empty()))
+          m_state = e_CommandEntry;
+      }
+      else {
+        m_enteredUsername = m_commandLine;
+        m_state = e_Password;
+      }
+      break;
+
+    case e_Password :
+      if (!WriteString(m_cli.GetNewLine()))
+        return false;
+
+      if (m_cli.OnLogIn(m_enteredUsername, m_commandLine))
+        m_state = e_CommandEntry;
+      else
+        m_state = m_cli.GetUsername().IsEmpty() ? (m_cli.GetPassword().IsEmpty() ? e_CommandEntry : e_Password) : e_Username;
+
+      SetLocalEcho(m_state != e_Password);
+      m_enteredUsername.MakeEmpty();
+      break;
+
     default :
-      if (isprint(ch))
-        m_commandLine += ch;
-      m_ignoreNextLF = false;
-      return;
+      OnCompletedLine();
   }
 
-  OnCompletedLine();
   m_commandLine.MakeEmpty();
-  WriteString(m_cli.GetPrompt());
+  return WritePrompt();
+}
+
+
+static bool CheckInternalCommand(const PCaselessString & line, const PCaselessString & cmds)
+{
+  PINDEX pos = cmds.Find(line);
+  if (pos == P_MAX_INDEX)
+    return false;
+  char terminator = cmds[pos + line.GetLength()];
+  return terminator == '\n' || terminator == '\0';
 }
 
 
@@ -186,6 +281,19 @@ void PCLI::Context::OnCompletedLine()
 
   PTRACE(4, "PCLI\tProcessing command line \"" << line << '"');
 
+  if (CheckInternalCommand(line, m_cli.GetExitCommand())) {
+    Stop();
+    return;
+  }
+
+  if (m_cli.GetCommentCommand().Find(line[0]) != P_MAX_INDEX) {
+    PStringArray comments = m_cli.GetCommentCommand().Lines();
+    for (PINDEX i = 0; i < comments.GetSize(); ++i) {
+      if (line.NumCompare(comments[i]) == EqualTo)
+        return;
+    }
+  }
+
   if (line.NumCompare(m_cli.GetRepeatCommand()) == EqualTo) {
     if (m_commandHistory.IsEmpty()) {
       *this << m_cli.GetNoHistoryError() << endl;
@@ -195,7 +303,7 @@ void PCLI::Context::OnCompletedLine()
     line = m_commandHistory.back();
   }
 
-  if (line == m_cli.GetHistoryCommand()) {
+  if (CheckInternalCommand(line, m_cli.GetHistoryCommand())) {
     unsigned cmdNum = 1;
     for (PStringList::iterator cmd = m_commandHistory.begin(); cmd != m_commandHistory.end(); ++cmd)
       *this << cmdNum++ << ' ' << *cmd << '\n';
@@ -213,30 +321,23 @@ void PCLI::Context::OnCompletedLine()
     line = m_commandHistory[cmdNum-1];
   }
 
-  if (line == m_cli.GetHelpCommand())
+  if (CheckInternalCommand(line, m_cli.GetHelpCommand()))
     m_cli.ShowHelp(*this);
   else {
     Arguments args(*this, line);
-    m_processingCommand = true;
+    m_state = e_ProcessingCommand;
     m_cli.OnReceivedLine(args);
-    m_processingCommand = false;
+    m_state = e_CommandEntry;
   }
 
   m_commandHistory += line;
 }
 
 
-void PCLI::Context::ThreadMain(PThread &, INT)
+void PCLI::Context::ThreadMain(PThread &, P_INT_PTR)
 {
   PTRACE(4, "PCLI\tContext thread started");
-
-  if (IsOpen()) {
-    OnStart();
-    while (ReadAndProcessInput())
-      ;
-    OnStop();
-  }
-
+  Run();
   PTRACE(4, "PCLI\tContext thread ended");
 }
 
@@ -252,15 +353,18 @@ PCLI::Arguments::Arguments(Context & context, const PString & rawLine)
 
 PCLI::Context & PCLI::Arguments::WriteUsage()
 {
-  if (!m_usage.IsEmpty())
-    m_context << m_context.GetCLI().GetCommandUsagePrefix() << m_usage << endl;
+  if (!m_usage.IsEmpty()) {
+    m_context << m_context.GetCLI().GetCommandUsagePrefix() << m_usage << '\n';
+    Usage(m_context);
+    m_context << flush;
+  }
   return m_context;
 }
 
 
 PCLI::Context & PCLI::Arguments::WriteError(const PString & error)
 {
-  m_context << m_command << m_context.GetCLI().GetCommandErrorPrefix();
+  m_context << GetCommandName() << m_context.GetCLI().GetCommandErrorPrefix();
   if (!error.IsEmpty())
     m_context << error << endl;
   return m_context;
@@ -270,10 +374,16 @@ PCLI::Context & PCLI::Arguments::WriteError(const PString & error)
 ///////////////////////////////////////////////////////////////////////////////
 
 PCLI::PCLI(const char * prompt)
-  : m_prompt(prompt != NULL ? prompt : "CLI> ")
-  , m_newLine("\r\n")
-  , m_helpCommand("?")
-  , m_helpOnHelp("Use ? to display help\n"
+  : m_newLine("\r\n")
+  , m_requireEcho(false)
+  , m_editCharacters("\b\x7f")
+  , m_prompt(prompt != NULL ? prompt : "CLI> ")
+  , m_usernamePrompt("Username: ")
+  , m_passwordPrompt("Password: ")
+  , m_commentCommand("#\n;\n//")
+  , m_exitCommand("exit\nquit")
+  , m_helpCommand("?\nhelp")
+  , m_helpOnHelp("Use ? or 'help' to display help\n"
                  "Use ! to list history of commands\n"
                  "Use !n to repeat the n'th command\n"
                  "Use !! to repeat last command\n"
@@ -306,21 +416,19 @@ bool PCLI::Start(bool runInBackground)
     return true;
   }
 
+  if (m_contextList.empty())
+    StartForeground();
+
   if (m_contextList.size() != 1) {
     PTRACE(2, "PCLI\tCan only start in foreground if have one context.");
     return false;
   }
 
   Context * context = m_contextList.front();
-  if (!context->IsOpen()) {
-    PTRACE(2, "PCLI\tCannot start foreground processing, context not open.");
-    return false;
-  }
-
-  context->OnStart();
-  while (context->ReadAndProcessInput())
-    ;
-  return true;
+  bool result = context->Run();
+  RemoveContext(context);
+  PTRACE_IF(2, !result, "PCLI\tCannot start foreground processing, context not open.");
+  return result;
 }
 
 
@@ -335,42 +443,50 @@ void PCLI::Stop()
 }
 
 
-bool PCLI::StartContext(PChannel * channel, bool autoDelete, bool runInBackground)
+PCLI::Context * PCLI::StartContext(PChannel * readChannel,
+                                   PChannel * writeChannel,
+                                   bool autoDeleteRead,
+                                   bool autoDeleteWrite,
+                                   bool runInBackground)
 {
   PCLI::Context * context = AddContext();
   if (context == NULL)
-    return false;
-
-  if (!context->Open(channel, autoDelete)) {
-    PTRACE(2, "PCLI\tCould not open context: " << context->GetErrorText());
-    return false;
-  }
-
-  if (runInBackground)
-    return context->Start();
-
-  return true;
-}
-
-
-bool PCLI::StartContext(PChannel * readChannel,
-                        PChannel * writeChannel,
-                        bool autoDeleteRead,
-                        bool autoDeleteWrite,
-                        bool runInBackground)
-{
-  PCLI::Context * context = AddContext();
-  if (context == NULL)
-    return false;
+    return NULL;
   
   if (!context->Open(readChannel, writeChannel, autoDeleteRead, autoDeleteWrite)) {
     PTRACE(2, "PCLI\tCould not open context: " << context->GetErrorText());
-    return false;
+    RemoveContext(context);
+    return NULL;
   }
 
-  if (runInBackground)
-    return context->Start();
+  if (runInBackground) {
+    if (!context->Start()) {
+      RemoveContext(context);
+      return NULL;
+    }
+  }
 
+  return context;
+}
+
+
+PCLI::Context * PCLI::StartForeground()
+{
+  return NULL;
+}
+
+
+bool PCLI::Run(PChannel * readChannel,
+               PChannel * writeChannel,
+               bool autoDeleteRead,
+               bool autoDeleteWrite)
+{
+  Context * context = StartContext(readChannel, writeChannel, autoDeleteRead, autoDeleteWrite, false);
+  if (context == NULL)
+    return false;
+
+  context->Run();
+  RemoveContext(context);
   return true;
 }
 
@@ -449,8 +565,17 @@ void PCLI::OnReceivedLine(Arguments & args)
     CommandMap_t::iterator cmd = m_commands.find(names);
     if (cmd != m_commands.end()) {
       args.Shift(nesting);
-      args.m_command = cmd->first;
+      args.SetCommandName(cmd->first);
       args.m_usage = cmd->second.m_usage;
+
+      if (!cmd->second.m_argSpec.IsEmpty()) {
+        args.Parse(cmd->second.m_argSpec, true);
+        if (!args.IsParsed()) {
+          args.WriteUsage() << args.GetParseError();
+          return;
+        }
+      }
+
       cmd->second.m_notifier(args, 0);
       return;
     }
@@ -460,14 +585,21 @@ void PCLI::OnReceivedLine(Arguments & args)
 }
 
 
+bool PCLI::OnLogIn(const PString & username, const PString & password)
+{
+  return m_username == username && m_password == password;
+}
+
+
 void PCLI::Broadcast(const PString & message) const
 {
   for (ContextList_t::const_iterator iter = m_contextList.begin(); iter != m_contextList.end(); ++iter)
     **iter << message << endl;
+  PTRACE(4, "PCLI\tBroadcast \"" << message << '"');
 }
 
 
-bool PCLI::SetCommand(const char * command, const PNotifier & notifier, const char * help, const char * usage)
+bool PCLI::SetCommand(const char * command, const PNotifier & notifier, const char * help, const char * usage, const char * argSpec)
 {
   if (!PAssert(command != NULL && *command != '\0' && !notifier.IsNULL(), PInvalidParameter))
     return false;
@@ -491,6 +623,7 @@ bool PCLI::SetCommand(const char * command, const PNotifier & notifier, const ch
       cmd.m_help = help;
       if (usage != NULL && *usage != '\0')
         cmd.m_usage = names & usage;
+      cmd.m_argSpec = argSpec;
     }
   }
 
@@ -521,7 +654,7 @@ void PCLI::ShowHelp(Context & context)
       context << left << setw(maxCommandLength) << cmd->first << "   ";
 
       if (cmd->second.m_help.IsEmpty())
-        context << GetCommandUsagePrefix(); // Earlier condiiton says must have usage
+        context << GetCommandUsagePrefix(); // Earlier conditon says must have usage
       else {
         lines = cmd->second.m_help.Lines();
         context << lines[0];
@@ -548,13 +681,21 @@ PCLIStandard::PCLIStandard(const char * prompt)
 }
 
 
-bool PCLIStandard::Start(bool runInBackground)
+PCLI::Context * PCLIStandard::StartForeground()
 {
-  if (m_contextList.empty())
-    StartContext(new PConsoleChannel(PConsoleChannel::StandardInput),
-                 new PConsoleChannel(PConsoleChannel::StandardOutput),
-                 true, true, runInBackground);
-  return PCLI::Start(runInBackground);
+  return StartContext(new PConsoleChannel(PConsoleChannel::StandardInput),
+                      new PConsoleChannel(PConsoleChannel::StandardOutput),
+                      true, true, false);
+}
+
+
+bool PCLIStandard::RunScript(PChannel * channel, bool autoDelete)
+{
+  PString prompt = GetPrompt();
+  SetPrompt(PString::Empty());
+  bool result = Run(channel, new PConsoleChannel(PConsoleChannel::StandardOutput), autoDelete, true);
+  SetPrompt(prompt);
+  return result;
 }
 
 
@@ -656,7 +797,7 @@ bool PCLISocket::Listen(WORD port)
 }
 
 
-void PCLISocket::ThreadMain(PThread &, INT)
+void PCLISocket::ThreadMain(PThread &, P_INT_PTR)
 {
   PTRACE(4, "PCLI\tServer thread started on port " << GetPort());
 
@@ -689,8 +830,11 @@ bool PCLISocket::HandleSingleThreadForAll()
         if (iterContext != m_contextBySocket.end()) {
           char buffer[1024];
           if (socket->Read(buffer, sizeof(buffer)-1)) {
-            for (PINDEX i = 0; i < socket->GetLastReadCount(); ++i)
-              iterContext->second->ProcessInput(buffer[i]);
+            PINDEX count = socket->GetLastReadCount();
+            for (PINDEX i = 0; i < count; ++i) {
+              if (!iterContext->second->ProcessInput(buffer[i]))
+                socket->Close();
+            }
           }
           else
             socket->Close();
@@ -705,7 +849,7 @@ bool PCLISocket::HandleSingleThreadForAll()
 
 bool PCLISocket::HandleIncoming()
 {
-  PTCPSocket * socket = new PTCPSocket;
+  PTCPSocket * socket = CreateSocket();
   if (socket->Accept(m_listenSocket)) {
     PTRACE(3, "PCLI\tIncoming connection from " << socket->GetPeerHostName());
     Context * context = CreateContext();
@@ -723,3 +867,31 @@ bool PCLISocket::HandleIncoming()
   delete socket;
   return false;
 }
+
+
+PTCPSocket * PCLISocket::CreateSocket()
+{
+  return new PTCPSocket();
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+
+#if P_TELNET
+
+PCLITelnet::PCLITelnet(WORD port, const char * prompt, bool singleThreadForAll)
+  : PCLISocket(port, prompt, singleThreadForAll)
+{
+}
+
+
+PTCPSocket * PCLITelnet::CreateSocket()
+{
+  return new PTelnetSocket();
+}
+
+#endif // P_TELNET
+
+#endif // P_CLI
+
+///////////////////////////////////////////////////////////////////////////////
