@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23089 $
+ * $Revision: 29414 $
  * $Author: rjongbloed $
- * $Date: 2009-07-08 11:27:38 -0500 (Wed, 08 Jul 2009) $
+ * $Date: 2013-04-02 15:39:30 -0500 (Tue, 02 Apr 2013) $
  */
 
 #ifdef __GNUC__
@@ -51,11 +51,19 @@ PBase64::PBase64()
 }
 
 
-void PBase64::StartEncoding(PBoolean useCRLF)
+void PBase64::StartEncoding(bool useCRLF)
 {
-  encodedString = "";
-  encodeLength = nextLine = saveCount = 0;
-  useCRLFs = useCRLF;
+  encodedString.MakeEmpty();
+  nextLine = saveCount = 0;
+  endOfLine = useCRLF ? "\r\n" : "\n";
+}
+
+
+void PBase64::StartEncoding(const char * eol)
+{
+  encodedString.MakeEmpty();
+  nextLine = saveCount = 0;
+  endOfLine = eol;
 }
 
 
@@ -67,7 +75,7 @@ void PBase64::ProcessEncoding(const PString & str)
 
 void PBase64::ProcessEncoding(const char * cstr)
 {
-  ProcessEncoding((const BYTE *)cstr, strlen(cstr));
+  ProcessEncoding((const BYTE *)cstr, (int)strlen(cstr));
 }
 
 
@@ -82,17 +90,17 @@ static const char Binary2Base64[65] =
 
 void PBase64::OutputBase64(const BYTE * data)
 {
-  char * out = encodedString.GetPointer(((encodeLength+7)&~255) + 256);
+  encodedString.SetMinSize(((encodedString.GetLength()+7)&~255) + 256);
 
-  out[encodeLength++] = Binary2Base64[data[0] >> 2];
-  out[encodeLength++] = Binary2Base64[((data[0]&3)<<4) | (data[1]>>4)];
-  out[encodeLength++] = Binary2Base64[((data[1]&15)<<2) | (data[2]>>6)];
-  out[encodeLength++] = Binary2Base64[data[2]&0x3f];
+  encodedString += Binary2Base64[data[0] >> 2];
+  encodedString += Binary2Base64[((data[0]&3)<<4) | (data[1]>>4)];
+  encodedString += Binary2Base64[((data[1]&15)<<2) | (data[2]>>6)];
+  encodedString += Binary2Base64[data[2]&0x3f];
 
-  if (++nextLine > 18) { // 76 columns
-    if (useCRLFs)
-      out[encodeLength++] = '\r';
-    out[encodeLength++] = '\n';
+  PINDEX len = endOfLine.GetLength();
+  if (++nextLine > (76-len)/4) {
+    for (PINDEX i = 0; i < len; ++i)
+      encodedString += endOfLine[i];
     nextLine = 0;
   }
 }
@@ -136,56 +144,56 @@ void PBase64::ProcessEncoding(const void * dataPtr, PINDEX length)
 PString PBase64::GetEncodedString()
 {
   PString retval = encodedString;
-  encodedString = "";
-  encodeLength = 0;
+  encodedString.MakeEmpty();
   return retval;
 }
 
 
 PString PBase64::CompleteEncoding()
 {
-  char * out = encodedString.GetPointer(encodeLength + 5)+encodeLength;
+  encodedString.SetMinSize(encodedString.GetLength() + 5);
 
   switch (saveCount) {
     case 1 :
-      *out++ = Binary2Base64[saveTriple[0] >> 2];
-      *out++ = Binary2Base64[(saveTriple[0]&3)<<4];
-      *out++ = '=';
-      *out   = '=';
+      encodedString += Binary2Base64[saveTriple[0] >> 2];
+      encodedString += Binary2Base64[(saveTriple[0]&3)<<4];
+      encodedString += '=';
+      encodedString += '=';
       break;
 
     case 2 :
-      *out++ = Binary2Base64[saveTriple[0] >> 2];
-      *out++ = Binary2Base64[((saveTriple[0]&3)<<4) | (saveTriple[1]>>4)];
-      *out++ = Binary2Base64[((saveTriple[1]&15)<<2)];
-      *out   = '=';
+      encodedString += Binary2Base64[saveTriple[0] >> 2];
+      encodedString += Binary2Base64[((saveTriple[0]&3)<<4) | (saveTriple[1]>>4)];
+      encodedString += Binary2Base64[((saveTriple[1]&15)<<2)];
+      encodedString += '=';
   }
 
   return encodedString;
 }
 
 
-PString PBase64::Encode(const PString & str)
+PString PBase64::Encode(const PString & str, const char * endOfLine)
 {
-  return Encode((const char *)str);
+  return Encode((const char *)str, str.GetLength(), endOfLine);
 }
 
 
-PString PBase64::Encode(const char * cstr)
+PString PBase64::Encode(const char * cstr, const char * endOfLine)
 {
-  return Encode((const BYTE *)cstr, strlen(cstr));
+  return Encode((const BYTE *)cstr, (PINDEX)strlen(cstr), endOfLine);
 }
 
 
-PString PBase64::Encode(const PBYTEArray & data)
+PString PBase64::Encode(const PBYTEArray & data, const char * endOfLine)
 {
-  return Encode(data, data.GetSize());
+  return Encode(data, data.GetSize(), endOfLine);
 }
 
 
-PString PBase64::Encode(const void * data, PINDEX length)
+PString PBase64::Encode(const void * data, PINDEX length, const char * endOfLine)
 {
   PBase64 encoder;
+  encoder.StartEncoding(endOfLine);
   encoder.ProcessEncoding(data, length);
   return encoder.CompleteEncoding();
 }
@@ -193,7 +201,7 @@ PString PBase64::Encode(const void * data, PINDEX length)
 
 void PBase64::StartDecoding()
 {
-  perfectDecode = PTrue;
+  perfectDecode = true;
   quadPosition = 0;
   decodedData.SetSize(0);
   decodeSize = 0;
@@ -231,21 +239,21 @@ PBoolean PBase64::ProcessDecoding(const char * cstr)
     BYTE value = Base642Binary[(BYTE)*cstr++];
     switch (value) {
       case 96 : // end of string
-        return PFalse;
+        return false;
 
       case 97 : // '=' sign
         if (quadPosition == 3 || (quadPosition == 2 && *cstr == '=')) {
           quadPosition = 0;  // Reset this to zero, as have a perfect decode
-          return PTrue; // Stop decoding now as must be at end of data
+          return true; // Stop decoding now as must be at end of data
         }
-        perfectDecode = PFalse;  // Ignore '=' sign but flag decode as suspect
+        perfectDecode = false;  // Ignore '=' sign but flag decode as suspect
         break;
 
       case 98 : // CRLFs
         break;  // Ignore totally
 
       case 99 :  // Illegal characters
-        perfectDecode = PFalse;  // Ignore rubbish but flag decode as suspect
+        perfectDecode = false;  // Ignore rubbish but flag decode as suspect
         break;
 
       default : // legal value from 0 to 63
@@ -299,7 +307,7 @@ PString PBase64::Decode(const PString & str)
 {
   PBYTEArray data;
   Decode(str, data);
-  return PString((const char *)(const BYTE *)data, data.GetSize());
+  return PString(data);
 }
 
 
@@ -335,7 +343,7 @@ void PMessageDigest::Process(const PString & str)
 
 void PMessageDigest::Process(const char * cstr)
 {
-  Process(cstr, strlen(cstr));
+  Process(cstr, (int)strlen(cstr));
 }
 
 
@@ -353,7 +361,7 @@ PString PMessageDigest::CompleteDigest()
 {
   Result result;
   CompleteDigest(result);
-  return PBase64::Encode(result.GetPointer(), result.GetSize());
+  return result.AsBase64();
 }
 
 void PMessageDigest::CompleteDigest(Result & result)
@@ -362,6 +370,28 @@ void PMessageDigest::CompleteDigest(Result & result)
 }
 
 
+void PMessageDigest::Result::PrintOn(ostream & strm) const
+{
+  if ((strm.flags()&ios::basefield) == ios::dec)
+    strm << AsBase64();
+  else {
+    char oldFill = strm.fill();
+    strm.fill('0');
+    for (PINDEX i = 0; i < GetSize(); ++i)
+      strm << setw(2) << (unsigned)GetAt(i);
+    strm.fill(oldFill);
+  }
+}
+
+
+PString PMessageDigest::Result::AsHex() const
+{
+  PStringStream strm;
+  strm << hex << *this;
+  return strm;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PMessageDigest5
 
@@ -573,7 +603,7 @@ void PMessageDigest5::InternalCompleteDigest(Result & result)
   Process(&countBytes, sizeof(countBytes));
 
   // Store state in digest
-  PUInt32l * valuep = (PUInt32l *)result.value.GetPointer(4 * sizeof(PUInt32l));
+  PUInt32l * valuep = (PUInt32l *)result.GetPointer(4 * sizeof(PUInt32l));
   for (PINDEX i = 0; i < PARRAYSIZE(state); i++)
     valuep[i] = state[i];
 
@@ -597,13 +627,13 @@ void PMessageDigest5::Encode(const PString & str, Result & result)
 
 PString PMessageDigest5::Encode(const char * cstr)
 {
-  return Encode((const BYTE *)cstr, strlen(cstr));
+  return Encode((const BYTE *)cstr, (int)strlen(cstr));
 }
 
 
 void PMessageDigest5::Encode(const char * cstr, Result & result)
 {
-  Encode((const BYTE *)cstr, strlen(cstr), result);
+  Encode((const BYTE *)cstr, (int)strlen(cstr), result);
 }
 
 
@@ -623,7 +653,7 @@ PString PMessageDigest5::Encode(const void * data, PINDEX length)
 {
   Result result;
   Encode(data, length, result);
-  return PBase64::Encode(result.GetPointer(), result.GetSize());
+  return result.AsBase64();
 }
 
 
@@ -643,7 +673,7 @@ void PMessageDigest5::Encode(const PString & str, Code & result)
 
 void PMessageDigest5::Encode(const char * cstr, Code & result)
 {
-  Encode((const BYTE *)cstr, strlen(cstr), result);
+  Encode((const BYTE *)cstr, (int)strlen(cstr), result);
 }
 
 void PMessageDigest5::Encode(const PBYTEArray & data, Code & result)
@@ -719,7 +749,7 @@ void PMessageDigestSHA1::InternalCompleteDigest(Result & result)
   if (shaContext == NULL)
     return;
 
-  SHA1_Final(result.value.GetPointer(20), (SHA_CTX *)shaContext);
+  SHA1_Final(result.GetPointer(20), (SHA_CTX *)shaContext);
   delete ((SHA_CTX *)shaContext);
   shaContext = NULL;
 }
@@ -765,7 +795,7 @@ PString PMessageDigestSHA1::Encode(const void * data, PINDEX length)
 {
   Result result;
   Encode(data, length, result);
-  return PBase64::Encode(result.GetPointer(), result.GetSize());
+  return result.AsBase64();
 }
 
 
@@ -827,7 +857,7 @@ void PCypher::Encode(const void * data, PINDEX length, PBYTEArray & coded)
 {
   PAssert((blockSize%8) == 0, PUnsupportedFeature);
 
-  Initialise(PTrue);
+  Initialise(true);
 
   const BYTE * in = (const BYTE *)data;
   BYTE * out = coded.GetPointer(
@@ -870,14 +900,14 @@ PBoolean PCypher::Decode(const PString & cypher, PString & clear)
 
   PBYTEArray clearText;
   if (!Decode(cypher, clearText))
-    return PFalse;
+    return false;
 
   if (clearText.IsEmpty())
-    return PTrue;
+    return true;
 
   PINDEX sz = clearText.GetSize();
-  memcpy(clear.GetPointer(sz+1), (const BYTE *)clearText, sz);
-  return PTrue;
+  memcpy(clear.GetPointerAndSetLength(sz), (const BYTE *)clearText, sz);
+  return true;
 }
 
 
@@ -885,7 +915,7 @@ PBoolean PCypher::Decode(const PString & cypher, PBYTEArray & clear)
 {
   PBYTEArray coded;
   if (!PBase64::Decode(cypher, coded))
-    return PFalse;
+    return false;
   return Decode(coded, clear);
 }
 
@@ -916,9 +946,9 @@ PBoolean PCypher::Decode(const PBYTEArray & coded, PBYTEArray & clear)
 {
   PAssert((blockSize%8) == 0, PUnsupportedFeature);
   if (coded.IsEmpty() || (coded.GetSize()%blockSize) != 0)
-    return PFalse;
+    return false;
 
-  Initialise(PFalse);
+  Initialise(false);
 
   const BYTE * in = coded;
   PINDEX length = coded.GetSize();
@@ -932,11 +962,11 @@ PBoolean PCypher::Decode(const PBYTEArray & coded, PBYTEArray & clear)
 
   if (blockSize != 1) {
     if (*--out >= blockSize)
-      return PFalse;
+      return false;
     clear.SetSize(length - blockSize + *out);
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1018,6 +1048,75 @@ void PTEACypher::DecodeBlock(const void * in, void * out)
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+//
+// PHMAC
+//
+
+void PHMAC::Initialise(const BYTE * key, PINDEX oldLen)
+{
+  // ensure the key is at least one block long and pad out if necessary
+  if ((int)oldLen < GetB()) {
+    m_key.SetSize(oldLen);
+    memcpy(m_key.GetPointer(), key,  oldLen);
+  }
+  else if ((int)oldLen > GetB()) {
+    Result result;
+    Hash((const BYTE *)m_key, m_key.GetSize(), result);
+    m_key.SetSize(result.GetSize());
+    memcpy(m_key.GetPointer(), result.GetPointer(), result.GetSize());
+  }
+}
+
+void PHMAC::InternalProcess(const BYTE * data, PINDEX len, PHMAC::Result & result)
+{
+  PINDEX i, l;
+  BYTE * s;
+  BYTE * d;
+
+  // construct key XOR ipad. This will always
+  PBYTEArray buffer(GetB() + len);
+  s = m_key.GetPointer();
+  l = m_key.GetSize();
+  d = buffer.GetPointer();
+  for (i = 0; i < l; ++i)
+    *d++ = 0x36 ^ *s++;
+  for (;i < (PINDEX)GetB(); ++i)
+    *d++ = 0x36;
+
+  // append text
+  memcpy(d, data, len);
+
+  // hash 
+  Result hash1;
+  Hash(buffer, buffer.GetSize(), hash1);
+
+  // create key XOR opad
+  buffer.SetSize(GetB() + hash1.GetSize());
+  s = m_key.GetPointer();
+  l = m_key.GetSize();
+  d = buffer.GetPointer();
+  for (i = 0; i < l; ++i)
+    *d++ = 0x5c ^ *s++;
+  for (;i < (PINDEX)GetB(); ++i)
+    *d++ = 0x5c;
+
+  // append hash
+  memcpy(d, hash1.GetPointer(), hash1.GetSize());
+
+  // hash 
+  Hash(buffer.GetPointer(), buffer.GetSize(), result);
+}
+
+
+PString PHMAC::Encode(const BYTE * data, PINDEX len) { Result result; InternalProcess(data, len, result);            return result.AsBase64(); }
+PString PHMAC::Encode(const PBYTEArray & data)       { Result result; InternalProcess(data, data.GetSize(), result); return result.AsBase64(); }
+PString PHMAC::Encode(const PString & str)           { Result result; InternalProcess(str, str.GetLength(), result); return result.AsBase64(); }
+
+void PHMAC::Process(const BYTE * data, PINDEX len, PHMAC::Result & result)   { InternalProcess(data, len, result); }
+void PHMAC::Process(const PBYTEArray & data, PHMAC::Result & result)         { InternalProcess(data, data.GetSize(), result); }
+void PHMAC::Process(const PString & str, PHMAC::Result & result)             { InternalProcess(str, str.GetLength(), result); }
+
 ///////////////////////////////////////////////////////////////////////////////
 // PSecureConfig
 
@@ -1067,24 +1166,24 @@ void PSecureConfig::GetProductKey(PTEACypher::Key & prodKey) const
 PSecureConfig::ValidationState PSecureConfig::GetValidation() const
 {
   PString str;
-  PBoolean allEmpty = PTrue;
+  PBoolean allEmpty = true;
   PMessageDigest5 digestor;
   for (PINDEX i = 0; i < securedKeys.GetSize(); i++) {
     str = GetString(securedKeys[i]);
     if (!str.IsEmpty()) {
       digestor.Process(str.Trim());
-      allEmpty = PFalse;
+      allEmpty = false;
     }
   }
   str = GetString(expiryDateKey);
   if (!str.IsEmpty()) {
     digestor.Process(str);
-    allEmpty = PFalse;
+    allEmpty = false;
   }
   str = GetString(optionBitsKey);
   if (!str.IsEmpty()) {
     digestor.Process(str);
-    allEmpty = PFalse;
+    allEmpty = false;
   }
 
   PString vkey = GetString(securityKey);
@@ -1116,17 +1215,17 @@ PSecureConfig::ValidationState PSecureConfig::GetValidation() const
 PBoolean PSecureConfig::ValidatePending()
 {
   if (GetValidation() != Pending)
-    return PFalse;
+    return false;
 
   PString vkey = GetString(securityKey);
   if (vkey.IsEmpty())
-    return PTrue;
+    return true;
 
   PMessageDigest5::Code code;
   BYTE info[sizeof(code)+1+sizeof(DWORD)];
   PTEACypher crypt(productKey);
   if (crypt.Decode(vkey, info, sizeof(info)) != sizeof(info))
-    return PFalse;
+    return false;
 
   PTime expiryDate(0, 0, 0,
             1, info[sizeof(code)]&15, (info[sizeof(code)]>>4)+1996, PTime::GMT);
@@ -1148,7 +1247,7 @@ PBoolean PSecureConfig::ValidatePending()
   digestor.Complete(code);
 
   if (memcmp(info, &code, sizeof(code)) != 0)
-    return PFalse;
+    return false;
 
   SetString(expiryDateKey, expiry);
   SetString(optionBitsKey, options);
@@ -1161,7 +1260,7 @@ PBoolean PSecureConfig::ValidatePending()
   }
   DeleteKey(pendingPrefix + securityKey);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1172,7 +1271,7 @@ void PSecureConfig::ResetPending()
       DeleteKey(securedKeys[i]);
   }
   else {
-    SetBoolean(pendingPrefix + securityKey, PTrue);
+    SetBoolean(pendingPrefix + securityKey, true);
 
     for (PINDEX i = 0; i < securedKeys.GetSize(); i++) {
       PString str = GetString(securedKeys[i]);
