@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23990 $
- * $Author: rjongbloed $
- * $Date: 2010-01-29 00:10:55 -0600 (Fri, 29 Jan 2010) $
+ * $Revision: 28931 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 06:31:11 -0600 (Fri, 18 Jan 2013) $
  */
 
 #ifdef __GNUC__
@@ -37,7 +37,7 @@
 #define new PNEW
 
 
-#if P_DNS
+#if P_DNS_RESOLVER && P_URL
 
 #ifdef  _WIN32
 #define PATH_SEP   ";"
@@ -105,6 +105,33 @@ struct NAPTR_DNS {
   PString GetReplacement() const    { return PString(GetReplacementBase()+1, GetReplacementLen()); }
 };
 
+
+void ResolveNAPTR(PDNS_RECORD dnsRecord, PDNS::NAPTRRecord & record)
+{
+#ifdef _WIN32
+  if (PProcess::IsOSVersion(6)) {
+    DNS_NAPTR_DATA * naptr = (DNS_NAPTR_DATA *)&dnsRecord->Data;
+    record.order       = naptr->wOrder; 
+    record.preference  = naptr->wPreference; 
+    record.flags       = naptr->pFlags; 
+    record.service     = naptr->pService; 
+    record.regex       = naptr->pRegularExpression; 
+    record.replacement = naptr->pReplacement; 
+  }
+  else
+#endif
+  {
+    NAPTR_DNS * naptr = (NAPTR_DNS *)&dnsRecord->Data; 
+    record.order       = naptr->order; 
+    record.preference  = naptr->preference; 
+    record.flags       = naptr->GetFlags(); 
+    record.service     = naptr->GetService(); 
+    record.regex       = naptr->GetRegex(); 
+    record.replacement = naptr->GetReplacement(); 
+  }
+}
+
+
 PDNS::NAPTRRecord * PDNS::NAPTRRecordList::HandleDNSRecord(PDNS_RECORD dnsRecord, PDNS_RECORD /*results*/)
 {
   PDNS::NAPTRRecord * record = NULL;
@@ -115,25 +142,8 @@ PDNS::NAPTRRecord * PDNS::NAPTRRecordList::HandleDNSRecord(PDNS_RECORD dnsRecord
       ) {
     record = new NAPTRRecord();
 
-#if WINVER >= 0x600  // Windows Vista/7
-    DNS_NAPTR_DATA * naptr = &dnsRecord->Data.Naptr;
+	ResolveNAPTR(dnsRecord, *record);
 
-    record->order       = naptr->wOrder;
-    record->preference  = naptr->wPreference;
-    record->flags       = naptr->pFlags;
-    record->service     = naptr->pService;
-    record->regex       = naptr->pRegularExpression;
-    record->replacement = naptr->pReplacement;
-#else
-    NAPTR_DNS * naptr = (NAPTR_DNS *)&dnsRecord->Data;
-
-    record->order       = naptr->order;
-    record->preference  = naptr->preference;
-    record->flags       = naptr->GetFlags();
-    record->service     = naptr->GetService();
-    record->regex       = naptr->GetRegex();
-    record->replacement = naptr->GetReplacement();
-#endif
   }
 
   return record;
@@ -154,7 +164,7 @@ PDNS::NAPTRRecord * PDNS::NAPTRRecordList::GetFirst(const char * service)
 
   currentPos   = 0;
   lastOrder = operator[](0).order;
-  orderLocked = PFalse;
+  orderLocked = false;
 
   return GetNext(service);
 }
@@ -178,7 +188,7 @@ PDNS::NAPTRRecord * PDNS::NAPTRRecordList::GetNext(const char * service)
       lastOrder   = record.order;
       if (record.order == lastOrder) {
         if ((service == NULL) || (record.service *= service)) {
-          orderLocked = PTrue;
+          orderLocked = true;
           return &record;
         }
       }
@@ -235,7 +245,7 @@ static PString ApplyRegex(const PString & orig, const PString & regexStr)
 
   // construct the regular expression
   PRegularExpression regex;
-  int regexFlags = PRegularExpression::Extended;
+  PRegularExpression::CompileOptions regexFlags = PRegularExpression::Extended;
   if (flags.Find('i') != P_MAX_INDEX)
     regexFlags += PRegularExpression::IgnoreCase;
   if (!regex.Compile(strings[0], regexFlags)) {
@@ -254,7 +264,7 @@ static PString ApplyRegex(const PString & orig, const PString & regexStr)
   PString value = strings[1];
   for (pos = 0; pos < value.GetLength(); pos++) {
     if (value[pos] == '\\' && pos < value.GetLength()-1) {
-      int var = value[pos+1]-'1'+1;   
+      PINDEX var = value[pos+1]-'1'+1;   
       PString str;
       if (var >= 0 && var < starts.GetSize() && var < ends.GetSize())
         str = orig(starts[var], ends[var]);
@@ -267,7 +277,7 @@ static PString ApplyRegex(const PString & orig, const PString & regexStr)
 
 static PStringArray & GetENUMServers()
 {
-  static const char * defaultDomains[] = { "87840.com","e164.voxgratia.net","e164.org","e164.arpa"};
+  static const char * defaultDomains[] = { "e164.org","e164.arpa"};
   static PStringArray servers(
           sizeof(defaultDomains)/sizeof(defaultDomains[0]),
           defaultDomains
@@ -303,7 +313,7 @@ PBoolean PDNS::ENUMLookup(const PString & e164,
 
 static PBoolean InternalENUMLookup(const PString & e164, const PString & service, PDNS::NAPTRRecordList & records, PString & returnStr)
 {
-  PBoolean result = PFalse;
+  PBoolean result = false;
 
   // get the first record that matches the service. 
   PDNS::NAPTRRecord * rec = records.GetFirst(service);
@@ -315,39 +325,39 @@ static PBoolean InternalENUMLookup(const PString & e164, const PString & service
       break;
 
     // process the flags
-    PBoolean handled  = PFalse;
-    PBoolean terminal = PTrue;
+    PBoolean handled  = false;
+    PBoolean terminal = true;
 
     for (PINDEX f = 0; !handled && f < rec->flags.GetLength(); ++f) {
       switch (tolower(rec->flags[f])) {
 
         // do an SRV lookup
         case 's':
-          terminal = PTrue;
-          handled = PFalse;
+          terminal = true;
+          handled = false;
           break;
 
         // do an A lookup
         case 'a':
-          terminal = PTrue;
-          handled = PFalse;
+          terminal = true;
+          handled = false;
           break;
 
         // apply regex and do the lookup
         case 'u':
           returnStr = ApplyRegex(e164, rec->regex);
-          result   = PTrue;
-          terminal = PTrue;
-          handled  = PTrue;
+          result   = true;
+          terminal = true;
+          handled  = true;
           break;
 
         // handle in a protocol specific way - not supported
         case 'p':
-          handled = PFalse;
+          handled = false;
           break;
   
         default:
-          handled = PFalse;
+          handled = false;
       }
     }
 
@@ -408,10 +418,10 @@ PBoolean PDNS::ENUMLookup(
       continue;
 
     if (InternalENUMLookup(e164, service, records, returnStr))
-      return PTrue;
+      return true;
   }
 
-  return PFalse;
+  return false;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -430,7 +440,7 @@ static PStringArray & GetRDSServers()
 
 static PBoolean RewriteDomain(const PString & original, PDNS::NAPTRRecordList & records, PString & returnStr)
 {
-   PBoolean result = PFalse;
+   PBoolean result = false;
 
   // get the first record that matches the service. 
   PDNS::NAPTRRecord * rec = records.GetFirst();
@@ -442,14 +452,14 @@ static PBoolean RewriteDomain(const PString & original, PDNS::NAPTRRecordList &
       break;
 
     // process the flags
-    PBoolean handled  = PFalse;
+    PBoolean handled  = false;
 
 	// General domain rewrites has no flag
     if (rec->flags.IsEmpty()) {
         returnStr = ApplyRegex(original, rec->regex);
 		if (returnStr.GetLength() > 0) {
-            result   = PTrue;
-            handled  = PTrue;
+            result   = true;
+            handled  = true;
 			break;
 		}
 	} else {
@@ -470,7 +480,7 @@ static PBoolean RewriteDomain(const PString & original, PDNS::NAPTRRecordList &
 
 static PBoolean InternalRDSLookup(const PString & rds, const PString & service, PDNS::NAPTRRecordList & records, PString & returnStr)
 {
-  PBoolean result = PFalse;
+  PBoolean result = false;
 
   // get the first record that matches the service. 
   PDNS::NAPTRRecord * rec = records.GetFirst(service);
@@ -482,8 +492,8 @@ static PBoolean InternalRDSLookup(const PString & rds, const PString & service,
       break;
 
     // process the flags
-    PBoolean handled  = PFalse;
-    PBoolean terminal = PTrue;
+    PBoolean handled  = false;
+    PBoolean terminal = true;
 
     for (PINDEX f = 0; !handled && f < rec->flags.GetLength(); ++f) {
       switch (tolower(rec->flags[f])) {
@@ -492,20 +502,20 @@ static PBoolean InternalRDSLookup(const PString & rds, const PString & service,
         case 's':
 		  // apply regex and do the lookup
           returnStr = ApplyRegex(rds, rec->regex);
-          result   = PTrue;
-          terminal = PTrue;
-          handled  = PTrue;
+          result   = true;
+          terminal = true;
+          handled  = true;
           break;
  
         case 'a':            // A lookup
         case 'u':            // U Lookup
-          terminal = PTrue;
-          handled = PFalse;
+          terminal = true;
+          handled = false;
           break;
  
         case 'p':           // P specific
 		default:
-          handled = PFalse;
+          handled = false;
           break;
       }
     }
@@ -603,14 +613,14 @@ PBoolean PDNS::RDSLookup(
 		
 	if (retStr.GetSize() > 0) {   // We have found records 
 		returnStr = retStr;
-	    return PTrue;
+	    return true;
 	}
   }
 
-  return PFalse;
+  return false;
 }
 
 
-#endif
+#endif // P_DNS_RESOLVER && P_URL
 
 // End of File ///////////////////////////////////////////////////////////////
