@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24840 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2010-10-28 23:07:18 -0500 (Thu, 28 Oct 2010) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -38,18 +38,19 @@
 
 #include <ptlib/sockets.h>
 #include <ptclib/random.h>
+
+#if P_ASN
 #include <ptclib/asner.h>
+#endif
 
 
 #define new PNEW
 
-#define GUID_SIZE 16
-
 
 ///////////////////////////////////////////////////////////////////////////////
 
 PGloballyUniqueID::PGloballyUniqueID()
-  : PBYTEArray(GUID_SIZE)
+  : PBYTEArray(Size)
 {
   static PMutex mutex;
   PWaitAndSignal wait(mutex);
@@ -112,7 +113,7 @@ PGloballyUniqueID::PGloballyUniqueID()
   theArray[9] = (BYTE)clockSequence;
 
   static PEthSocket::Address macAddress;
-  static PBoolean needMacAddress = PTrue;
+  static PBoolean needMacAddress = true;
   if (needMacAddress) {
     PIPSocket::InterfaceTable interfaces;
     if (PIPSocket::GetInterfaceTable(interfaces)) {
@@ -121,7 +122,7 @@ PGloballyUniqueID::PGloballyUniqueID()
         if (!macAddrStr && macAddrStr != "44-45-53-54-00-00") { /* not Win32 PPP device */
           macAddress = macAddrStr;
           if (macAddress != NULL) {
-            needMacAddress = PFalse;
+            needMacAddress = false;
             break;
           }
         }
@@ -134,7 +135,7 @@ PGloballyUniqueID::PGloballyUniqueID()
       macAddress.ls.s = (WORD)rand;
       macAddress.b[0] |= '\x80';
 
-      needMacAddress = PFalse;
+      needMacAddress = false;
     }
   }
 
@@ -143,7 +144,7 @@ PGloballyUniqueID::PGloballyUniqueID()
 
 
 PGloballyUniqueID::PGloballyUniqueID(const char * cstr)
-  : PBYTEArray(GUID_SIZE)
+  : PBYTEArray(Size)
 {
   if (cstr != NULL && *cstr != '\0') {
     PStringStream strm(cstr);
@@ -153,26 +154,33 @@ PGloballyUniqueID::PGloballyUniqueID(const char * cstr)
 
 
 PGloballyUniqueID::PGloballyUniqueID(const PString & str)
-  : PBYTEArray(GUID_SIZE)
+  : PBYTEArray(Size)
 {
   PStringStream strm(str);
   ReadFrom(strm);
 }
 
 
+PGloballyUniqueID::PGloballyUniqueID(const void * data, PINDEX size)
+  : PBYTEArray(Size)
+{
+  memcpy(theArray, PAssertNULL(data), PMIN(size, GetSize()));
+}
+
+
 #if P_ASN
 PGloballyUniqueID::PGloballyUniqueID(const PASN_OctetString & newId)
   : PBYTEArray(newId)
 {
-  PAssert(GetSize() == GUID_SIZE, PInvalidParameter);
-  SetSize(GUID_SIZE);
+  PAssert(GetSize() == Size, PInvalidParameter);
+  PBYTEArray::SetSize(Size);
 }
 #endif
 
 
 PObject * PGloballyUniqueID::Clone() const
 {
-  PAssert(GetSize() == GUID_SIZE, "PGloballyUniqueID is invalid size");
+  PAssert(GetSize() == Size, "PGloballyUniqueID is invalid size");
 
   return new PGloballyUniqueID(*this);
 }
@@ -180,7 +188,7 @@ PObject * PGloballyUniqueID::Clone() const
 
 PINDEX PGloballyUniqueID::HashFunction() const
 {
-  PAssert(GetSize() == GUID_SIZE, "PGloballyUniqueID is invalid size");
+  PAssert(GetSize() == Size, "PGloballyUniqueID is invalid size");
 
   DWORD * words = (DWORD *)theArray;
   DWORD sum = words[0] + words[1] + words[2] + words[3];
@@ -190,7 +198,7 @@ PINDEX PGloballyUniqueID::HashFunction() const
 
 void PGloballyUniqueID::PrintOn(ostream & strm) const
 {
-  PAssert(GetSize() == GUID_SIZE, "PGloballyUniqueID is invalid size");
+  PAssert(GetSize() == Size, "PGloballyUniqueID is invalid size");
 
   char fillchar = strm.fill();
   strm << hex << setfill('0')
@@ -216,14 +224,14 @@ void PGloballyUniqueID::PrintOn(ostream & strm) const
 
 void PGloballyUniqueID::ReadFrom(istream & strm)
 {
-  PAssert(GetSize() == GUID_SIZE, "PGloballyUniqueID is invalid size");
-  SetSize(16);
+  PAssert(GetSize() == Size, "PGloballyUniqueID is invalid size");
+  PBYTEArray::SetSize(Size);
 
   strm >> ws;
 
   PINDEX count = 0;
 
-  while (count < 2*GUID_SIZE) {
+  while (count < 2*Size) {
     if (isxdigit(strm.peek())) {
       char digit = (char)(strm.get() - '0');
       if (digit >= 10) {
@@ -245,8 +253,8 @@ void PGloballyUniqueID::ReadFrom(istream & strm)
       break;
   }
 
-  if (count < 2*GUID_SIZE) {
-    memset(theArray, 0, GUID_SIZE);
+  if (count < 2*Size) {
+    memset(theArray, 0, Size);
     strm.clear(ios::failbit);
   }
 }
@@ -262,9 +270,9 @@ PString PGloballyUniqueID::AsString() const
 
 PBoolean PGloballyUniqueID::IsNULL() const
 {
-  PAssert(GetSize() == GUID_SIZE, "PGloballyUniqueID is invalid size");
+  PAssert(GetSize() == Size, "PGloballyUniqueID is invalid size");
 
-  return memcmp(theArray, "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", 16) == 0;
+  return memcmp(theArray, "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", Size) == 0;
 }
 
 
