@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #ifdef __GNUC__
@@ -124,6 +124,37 @@ void PHTML::Toggle(ElementInSet elmt)
 }
 
 
+void PHTML::Escaped::Output(ostream & strm) const
+{
+  for (const char * ptr = m_str; *ptr != '\0'; ++ptr) {
+    switch (*ptr) {
+      case '"' :
+        strm << "&quot;";
+        break;
+      case '<' :
+        strm << "&lt;";
+        break;
+      case '>' :
+        strm << "&gt;";
+        break;
+      case '&' :
+        strm << "&amp;";
+        break;
+      default :
+        strm << *ptr;
+    }
+  }
+}
+
+
+PString PHTML::Escape(const char * str)
+{
+  PStringStream strm;
+  strm << Escaped(str);
+  return strm;
+}
+
+
 void PHTML::Element::Output(PHTML & html) const
 {
   PAssert(reqElement == NumElementsInSet || html.Is(reqElement),
@@ -288,7 +319,7 @@ void PHTML::Heading::AddAttr(PHTML & html) const
   PAssert(num >= 1 && num <= 6, "Bad heading number");
   html << num;
   if (srcString != NULL)
-    html << " SRC=\"" << srcString << '"';
+    html << " SRC=\"" << Escaped(srcString) << '"';
   if (seqNum > 0)
     html << " SEQNUM=" << seqNum;
   if (skipSeq > 0)
@@ -331,7 +362,7 @@ PHTML::HotLink::HotLink(const char * href, const char * attr)
 void PHTML::HotLink::AddAttr(PHTML & html) const
 {
   if (hrefString != NULL && *hrefString != '\0')
-    html << " HREF=\"" << hrefString << '"';
+    html << " HREF=\"" << Escaped(hrefString) << '"';
   else
     PAssert(html.Is(InAnchor), PInvalidParameter);
 }
@@ -346,7 +377,7 @@ PHTML::Target::Target(const char * name, const char * attr)
 void PHTML::Target::AddAttr(PHTML & html) const
 {
   if (nameString != NULL && *nameString != '\0')
-    html << " NAME=\"" << nameString << '"';
+    html << " NAME=\"" << Escaped(nameString) << '"';
 }
 
 
@@ -365,7 +396,7 @@ PHTML::ImageElement::ImageElement(const char * n,
 void PHTML::ImageElement::AddAttr(PHTML & html) const
 {
   if (srcString != NULL)
-    html << " SRC=\"" << srcString << '"';
+    html << " SRC=\"" << Escaped(srcString) << '"';
 }
 
 
@@ -392,7 +423,7 @@ void PHTML::Image::AddAttr(PHTML & html) const
 {
   PAssert(srcString != NULL && *srcString != '\0', PInvalidParameter);
   if (altString != NULL)
-    html << " ALT=\"" << altString << '"';
+    html << " ALT=\"" << Escaped(altString) << '"';
   if (width != 0)
     html << " WIDTH=" << width;
   if (height != 0)
@@ -552,7 +583,7 @@ void PHTML::DefinitionItem::Output(PHTML & html) const
 PHTML::TableStart::TableStart(const char * attr)
   : Element("TABLE", attr, InTable, InBody, BothCRLF)
 {
-  borderFlag = PFalse;
+  borderFlag = false;
 }
 
 PHTML::TableStart::TableStart(BorderCodes border, const char * attr)
@@ -626,11 +657,11 @@ void PHTML::Form::AddAttr(PHTML & html) const
   if (methodString != NULL)
     html << " METHOD=" << methodString;
   if (actionString != NULL)
-    html << " ACTION=\"" << actionString << '"';
+    html << " ACTION=\"" << Escaped(actionString) << '"';
   if (mimeTypeString != NULL)
-    html << " ENCTYPE=\"" << mimeTypeString << '"';
+    html << " ENCTYPE=\"" << Escaped(mimeTypeString) << '"';
   if (scriptString != NULL)
-    html << " SCRIPT=\"" << scriptString << '"';
+    html << " SCRIPT=\"" << Escaped(scriptString) << '"';
 }
 
 
@@ -669,7 +700,7 @@ void PHTML::Select::AddAttr(PHTML & html) const
 {
   if (!html.Is(InSelect)) {
     PAssert(nameString != NULL && *nameString != '\0', PInvalidParameter);
-    html << " NAME=\"" << nameString << '"';
+    html << " NAME=\"" << Escaped(nameString) << '"';
   }
   FieldElement::AddAttr(html);
 }
@@ -678,7 +709,7 @@ void PHTML::Select::AddAttr(PHTML & html) const
 PHTML::Option::Option(const char * attr)
   : FieldElement("OPTION", attr, NumElementsInSet, NoCRLF, Enabled)
 {
-  selectedFlag = PFalse;
+  selectedFlag = false;
 }
 
 PHTML::Option::Option(SelectionCodes select,
@@ -692,7 +723,7 @@ PHTML::Option::Option(DisableCodes disabled,
                       const char * attr)
   : FieldElement("OPTION", attr, NumElementsInSet, NoCRLF, disabled)
 {
-  selectedFlag = PFalse;
+  selectedFlag = false;
 }
 
 PHTML::Option::Option(SelectionCodes select,
@@ -725,7 +756,7 @@ PHTML::FormField::FormField(const char * n,
 void PHTML::FormField::AddAttr(PHTML & html) const
 {
   PAssert(nameString != NULL && *nameString != '\0', PInvalidParameter);
-  html << " NAME=\"" << nameString << '"';
+  html << " NAME=\"" << Escaped(nameString) << '"';
   FieldElement::AddAttr(html);
 }
 
@@ -787,7 +818,7 @@ void PHTML::HiddenField::AddAttr(PHTML & html) const
 {
   InputField::AddAttr(html);
   PAssert(valueString != NULL, PInvalidParameter);
-  html << " VALUE=\"" << valueString << '"';
+  html << " VALUE=\"" << Escaped(valueString) << '"';
 }
 
 
@@ -855,11 +886,12 @@ PHTML::InputText::InputText(const char * type,
 void PHTML::InputText::AddAttr(PHTML & html) const
 {
   InputField::AddAttr(html);
-  html << " SIZE=" << width;
+  if (width > 0)
+    html << " SIZE=" << width;
   if (length > 0)
     html << " MAXLENGTH=" << length;
   if (value != NULL)
-    html << " VALUE=\"" << value << '"';
+    html << " VALUE=\"" << Escaped(value) << '"';
 }
 
 
@@ -905,7 +937,7 @@ PHTML::RadioButton::RadioButton(const char * fname,
   : InputField("radio", fname, Enabled, attr)
 {
   valueString = value;
-  checkedFlag = PFalse;
+  checkedFlag = false;
 }
 
 PHTML::RadioButton::RadioButton(const char * fname,
@@ -915,7 +947,7 @@ PHTML::RadioButton::RadioButton(const char * fname,
   : InputField("radio", fname, disabled, attr)
 {
   valueString = value;
-  checkedFlag = PFalse;
+  checkedFlag = false;
 }
 
 PHTML::RadioButton::RadioButton(const char * fname,
@@ -945,7 +977,7 @@ void PHTML::RadioButton::AddAttr(PHTML & html) const
 {
   InputField::AddAttr(html);
   PAssert(valueString != NULL, PInvalidParameter);
-  html << " VALUE=\"" << valueString << "\"";
+  html << " VALUE=\"" << Escaped(valueString) << '"';
   if (checkedFlag)
     html << " CHECKED";
 }
@@ -972,11 +1004,27 @@ PHTML::CheckBox::CheckBox(const char * fname,
 }
 
 
-PHTML::InputRange::InputRange(const char * fname,
-                              int min, int max, int value,
-                              DisableCodes disabled,
-                              const char * attr)
-  : InputField("range", fname, disabled, attr)
+PHTML::InputNumber::InputNumber(const char * fname,
+                                int min, int max, int value,
+                                DisableCodes disabled,
+                                const char * attr)
+  : InputField("number", fname, disabled, attr)
+{
+  Construct(min, max, value);
+}
+
+PHTML::InputNumber::InputNumber(const char * type,
+                                const char * fname,
+                                int min, int max,
+                                int value,
+                                DisableCodes disabled,
+                                const char * attr)
+  : InputField(type, fname, disabled, attr)
+{
+  Construct(min, max, value);
+}
+
+void PHTML::InputNumber::Construct(int min, int max, int value)
 {
   PAssert(min <= max, PInvalidParameter);
   minValue = min;
@@ -989,7 +1037,7 @@ PHTML::InputRange::InputRange(const char * fname,
     initValue = value;
 }
 
-void PHTML::InputRange::AddAttr(PHTML & html) const
+void PHTML::InputNumber::AddAttr(PHTML & html) const
 {
   InputField::AddAttr(html);
   PINDEX max = PMAX(-minValue, maxValue);
@@ -1001,7 +1049,16 @@ void PHTML::InputRange::AddAttr(PHTML & html) const
   html << " SIZE=" << width
        << " MIN=" << minValue
        << " MAX=" << maxValue
-       << " VALUE=\"" << initValue << "\"";
+       << " VALUE=\"" << initValue << '"';
+}
+
+
+PHTML::InputRange::InputRange(const char * fname,
+                              int min, int max, int value,
+                              DisableCodes disabled,
+                              const char * attr)
+  : InputNumber("range", fname, min, max, value, disabled, attr)
+{
 }
 
 
@@ -1018,7 +1075,7 @@ void PHTML::InputFile::AddAttr(PHTML & html) const
 {
   InputField::AddAttr(html);
   if (acceptString != NULL)
-    html << " ACCEPT=\"" << acceptString << '"';
+    html << " ACCEPT=\"" << Escaped(acceptString) << '"';
 }
 
 
@@ -1045,7 +1102,7 @@ void PHTML::InputImage::AddAttr(PHTML & html) const
 {
   InputField::AddAttr(html);
   if (srcString != NULL)
-    html << " SRC=\"" << srcString << '"';
+    html << " SRC=\"" << Escaped(srcString) << '"';
 }
 
 
@@ -1082,7 +1139,7 @@ void PHTML::ResetButton::AddAttr(PHTML & html) const
 {
   InputImage::AddAttr(html);
   if (titleString != NULL)
-    html << " VALUE=\"" << titleString << '"';
+    html << " VALUE=\"" << Escaped(titleString) << '"';
 }
 
 
