@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25012 $
+ * $Revision: 28998 $
  * $Author: rjongbloed $
- * $Date: 2011-01-06 01:01:23 -0600 (Thu, 06 Jan 2011) $
+ * $Date: 2013-01-25 21:11:10 -0600 (Fri, 25 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -96,11 +96,11 @@ int PHTTPClient::ExecuteCommand(const PString & cmdName,
 
   bool needAuthentication = true;
   PURL adjustableURL = url;
-  for (PINDEX retry = 0; retry < 3; retry++) {
+  for (int retry = 3; retry > 0; --retry) {
     if (!AssureConnect(adjustableURL, outMIME))
       break;
 
-    if (!WriteCommand(cmdName, url.AsString(PURL::URIOnly), outMIME, dataBody)) {
+    if (!WriteCommand(cmdName, url.AsString(PURL::RelativeOnly), outMIME, dataBody)) {
       lastResponseCode = -1;
       lastResponseInfo = GetErrorText(LastWriteError);
       break;
@@ -112,64 +112,70 @@ int PHTTPClient::ExecuteCommand(const PString & cmdName,
 
     // Await a response, if all OK exit loop
     if (ReadResponse(replyMIME) && (lastResponseCode != Continue || ReadResponse(replyMIME))) {
+      if (IsOK(lastResponseCode))
+        return lastResponseCode;
+
       switch (lastResponseCode) {
         case MovedPermanently:
         case MovedTemporarily:
           adjustableURL = replyMIME("Location");
-          if (!adjustableURL.IsEmpty())
-            return lastResponseCode;
+          if (adjustableURL.IsEmpty())
+            continue;
           break;
 
         case UnAuthorised:
           if (needAuthentication && replyMIME.Contains("WWW-Authenticate") && !(m_userName.IsEmpty() && m_password.IsEmpty())) {
-            needAuthentication = false;
-
             // authenticate 
             PString errorMsg;
             PHTTPClientAuthentication * newAuth = PHTTPClientAuthentication::ParseAuthenticationRequired(false, replyMIME, errorMsg);
-            if (newAuth == NULL)
-              return false;
+            if (newAuth != NULL) {
+              newAuth->SetUsername(m_userName);
+              newAuth->SetPassword(m_password);
 
-            newAuth->SetUsername(m_userName);
-            newAuth->SetPassword(m_password);
+              delete m_authentication;
+              m_authentication = newAuth;
+              needAuthentication = false;
+              continue;
+            }
 
-            delete m_authentication;
-            m_authentication = newAuth;
-            break;
+            lastResponseInfo += " - " + errorMsg;
           }
           // Do next case
 
         default:
-          return lastResponseCode;
+          break;
       }
+
+      retry = 0; // No more retries
     }
     else {
       // If not persisting, we have no oppurtunity to write again, just error out
       if (!m_persist)
         break;
 
-      // ... we close the channel and allow AssureConnet() to reopen it.
+      // ... we close the channel and allow AssureConnect() to reopen it.
       Close();
     }
   }
 
+  PTRACE_IF(3, !IsOK(lastResponseCode), "HTTP", "Error " << lastResponseCode << ' ' << lastResponseInfo);
   return lastResponseCode;
 }
 
 
-PBoolean PHTTPClient::WriteCommand(Commands cmd,
-                               const PString & url,
-                               PMIMEInfo & outMIME,
-                               const PString & dataBody)
+bool PHTTPClient::WriteCommand(Commands cmd,
+                        const PString & url,
+                            PMIMEInfo & outMIME,
+                        const PString & dataBody)
 {
   return WriteCommand(commandNames[cmd], url, outMIME, dataBody);
 }
 
 
-PBoolean PHTTPClient::WriteCommand(const PString & cmdName,
-                                   const PString & url,
-                                       PMIMEInfo & outMIME,
-                                   const PString & dataBody)
+bool PHTTPClient::WriteCommand(const PString & cmdName,
+                               const PString & url,
+                                   PMIMEInfo & outMIME,
+                               const PString & dataBody)
 {
   ostream & this_stream = *this;
   PINDEX len = dataBody.GetLength();
@@ -185,7 +191,7 @@ PBoolean PHTTPClient::WriteCommand(const PString & cmdName,
 
 #if PTRACING
   if (PTrace::CanTrace(3)) {
-    ostream & strm = PTrace::Begin(3, __FILE__, __LINE__);
+    ostream & strm = PTRACE_BEGIN(3);
     strm << "HTTP\tSending ";
     if (PTrace::CanTrace(4))
       strm << '\n';
@@ -197,7 +203,7 @@ PBoolean PHTTPClient::WriteCommand(const PString & cmdName,
     if (PTrace::CanTrace(4)) {
       strm << '\n' << outMIME;
       if (!dataBody.IsEmpty()) {
-        int amt = PTrace::CanTrace(5) ? 10000 : 100;
+        PINDEX amt = PTrace::CanTrace(5) ? 10000 : 100;
         strm << dataBody.Left(amt);
         if (len > amt)
           strm << "\n....";
@@ -214,7 +220,7 @@ PBoolean PHTTPClient::WriteCommand(const PString & cmdName,
 }
 
 
-PBoolean PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
+bool PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
 {
   PString http = ReadString(7);
   if (!http) {
@@ -224,7 +230,7 @@ PBoolean PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
       lastResponseCode = PHTTP::RequestOK;
       lastResponseInfo = "HTTP/0.9";
       PTRACE(3, "HTTP\tRead HTTP/0.9 OK");
-      return PTrue;
+      return true;
     }
 
     if (http[0] == '\n')
@@ -237,15 +243,15 @@ PBoolean PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
 
       PString body;
       if (lastResponseCode >= 300) {
-        if (replyMIME.GetInteger(ContentLengthTag(), INT_MAX) > MaxTraceContentSize)
-          InternalReadContentBody(replyMIME, NULL); // Waste body
-        else
+        if ((int)replyMIME.GetInteger(ContentLengthTag(), INT_MAX) <= MaxTraceContentSize)
           ReadContentBody(replyMIME, body);
+        else
+          ReadContentBody(replyMIME); // Waste body
       }
 
 #if PTRACING
       if (PTrace::CanTrace(3)) {
-        ostream & strm = PTrace::Begin(3, __FILE__, __LINE__);
+        ostream & strm = PTRACE_BEGIN(3);
         strm << "HTTP\tResponse ";
         if (PTrace::CanTrace(4))
           strm << '\n';
@@ -263,7 +269,7 @@ PBoolean PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
         lastResponseInfo += '\n' + body;
 
       if (readOK)
-        return PTrue;
+        return true;
     }
   }
  
@@ -275,41 +281,119 @@ PBoolean PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
     SetErrorValues(ProtocolFailure, 0, LastReadError);
   }
 
-  return PFalse;
+  return false;
 }
 
 
-PBoolean PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PString & body)
+struct PHTTPClient_DummyProcessor : public PHTTPClient::ContentProcessor
 {
-  PCharArray rawBody;
-  if (!InternalReadContentBody(replyMIME, &rawBody))
-    return false;
+  BYTE m_body[4096];
 
-  body = PString(rawBody, rawBody.GetSize());
-  return true;
-}
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    size = sizeof(m_body);
+    return m_body;
+  }
+
+  virtual bool Process(const void *, PINDEX)
+  {
+    return true;
+  }
+};
 
 
-PBoolean PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PBYTEArray & body)
+bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME)
 {
-  return InternalReadContentBody(replyMIME, &body);
+  PHTTPClient_DummyProcessor processor;
+  return ReadContentBody(replyMIME, processor);
 }
 
 
-PBoolean PHTTPClient::InternalReadContentBody(PMIMEInfo & replyMIME, PAbstractArray * body)
+struct PHTTPClient_StringProcessor : public PHTTPClient::ContentProcessor
+{
+  PString & m_body;
+
+  PHTTPClient_StringProcessor(PString & body)
+    : m_body(body)
+  {
+  }
+
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    PINDEX oldLength = m_body.GetLength();
+    char * ptr = m_body.GetPointerAndSetLength(oldLength+size);
+    return ptr != NULL ? ptr+oldLength : NULL;
+  }
+
+  virtual bool Process(const void *, PINDEX)
+  {
+    return true;
+  }
+};
+
+
+bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PString & body)
+{
+  PHTTPClient_StringProcessor processor(body);
+  return ReadContentBody(replyMIME, processor);
+}
+
+
+struct PHTTPClient_BinaryProcessor : public PHTTPClient::ContentProcessor
+{
+  PBYTEArray & m_body;
+
+  PHTTPClient_BinaryProcessor(PBYTEArray & body)
+    : m_body(body)
+  {
+  }
+
+  virtual void * GetBuffer(PINDEX & size)
+  {
+    PINDEX oldSize = m_body.GetSize();
+    BYTE * ptr = m_body.GetPointer(oldSize+size);
+    return ptr != NULL ? ptr+oldSize : NULL;
+  }
+
+  virtual bool Process(const void *, PINDEX)
+  {
+    return true;
+  }
+};
+
+bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PBYTEArray & body)
+{
+  PHTTPClient_BinaryProcessor processor(body);
+  return ReadContentBody(replyMIME, processor);
+}
+
+
+bool PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, ContentProcessor & processor)
 {
   PCaselessString encoding = replyMIME(TransferEncodingTag());
 
   if (encoding != ChunkedTag()) {
     if (replyMIME.Contains(ContentLengthTag())) {
       PINDEX length = replyMIME.GetInteger(ContentLengthTag());
-      if (body != NULL) {
-        body->SetSize(length);
-        return ReadBlock(body->GetPointer(), length);
+
+      PINDEX size = length;
+      void * ptr = processor.GetBuffer(size);
+      if (ptr == NULL) {
+        lastResponseCode = ContentProcessorError;
+        lastResponseInfo = "No buffer from HTTP content processor";
+        return false;
       }
-      while (length-- > 0) {
-        if (ReadChar() < 0)
+
+      if (length == size)
+        return ReadBlock(ptr, length);
+
+      while (length > 0 && Read(ptr, PMIN(length, size))) {
+        if (!processor.Process(ptr, GetLastReadCount())) {
+          lastResponseCode = ContentProcessorError;
+          lastResponseInfo = "Content processing error";
           return false;
+        }
+        length -= GetLastReadCount();
       }
       return true;
     }
@@ -317,80 +401,104 @@ PBoolean PHTTPClient::InternalReadContentBody(PMIMEInfo & replyMIME, PAbstractAr
     if (!(encoding.IsEmpty())) {
       lastResponseCode = -1;
       lastResponseInfo = "Unknown Transfer-Encoding extension";
-      return PFalse;
+      return false;
     }
 
-    if (body != NULL) {
-      // Must be raw, read to end file variety
-      static const PINDEX ChunkSize = 2048;
-      PINDEX bytesRead = 0;
-      while (ReadBlock((char *)body->GetPointer(bytesRead+ChunkSize)+bytesRead, ChunkSize))
-        bytesRead += GetLastReadCount();
+    PINDEX size = 8192;
+    void * ptr = processor.GetBuffer(size);
+    if (ptr == NULL) {
+      lastResponseCode = ContentProcessorError;
+      lastResponseInfo = "No buffer from HTTP content processor";
+      return false;
+    }
 
-      body->SetSize(bytesRead + GetLastReadCount());
-    }
-    else {
-      while (ReadChar() >= 0)
-        ;
+    // Must be raw, read to end file variety
+    while (Read(ptr, size)) {
+      if (!processor.Process(ptr, GetLastReadCount())) {
+        lastResponseCode = ContentProcessorError;
+        lastResponseInfo = "Content processing error";
+        return false;
+      }
     }
+
     return GetErrorCode(LastReadError) == NoError;
   }
 
   // HTTP1.1 chunked format
-  PINDEX bytesRead = 0;
   for (;;) {
     // Read chunk length line
     PString chunkLengthLine;
     if (!ReadLine(chunkLengthLine))
-      return PFalse;
+      return false;
 
     // A zero length chunk is end of output
     PINDEX chunkLength = chunkLengthLine.AsUnsigned(16);
     if (chunkLength == 0)
       break;
 
-    if (body != NULL) {
-      // Read the chunk
-      if (!ReadBlock((char *)body->GetPointer(bytesRead+chunkLength)+bytesRead, chunkLength))
-        return PFalse;
-      bytesRead+= chunkLength;
+    PINDEX size = chunkLength;
+    void * ptr = processor.GetBuffer(size);
+    if (ptr == NULL) {
+      lastResponseCode = ContentProcessorError;
+      lastResponseInfo = "No buffer from HTTP content processor";
+      return false;
+    }
+
+    if (chunkLength == size) {
+      if (!ReadBlock(ptr, chunkLength))
+        return false;
     }
     else {
-      while (chunkLength-- > 0) {
-        if (ReadChar() < 0)
+      // Read the chunk
+      while (chunkLength > 0 && Read(ptr, PMIN(chunkLength, size))) {
+        if (!processor.Process(ptr, GetLastReadCount())) {
+          lastResponseCode = ContentProcessorError;
+          lastResponseInfo = "Content processing error";
           return false;
+        }
+        chunkLength -= GetLastReadCount();
       }
     }
 
     // Read the trailing CRLF
     if (!ReadLine(chunkLengthLine))
-      return PFalse;
+      return false;
   }
 
   // Read the footer
   PString footer;
   do {
     if (!ReadLine(footer))
-      return PFalse;
+      return false;
   } while (replyMIME.AddMIME(footer));
 
-  return PTrue;
+  return true;
 }
 
 
-PBoolean PHTTPClient::GetTextDocument(const PURL & url,
-                                      PString & document,
-                                      const PString & requiredContentType)
+static bool CheckContentType(const PMIMEInfo & replyMIME, const PString & requiredContentType)
+{
+  PCaselessString actualContentType = replyMIME(PHTTPClient::ContentTypeTag());
+  if (requiredContentType.IsEmpty() || actualContentType.IsEmpty() ||
+      actualContentType.NumCompare(requiredContentType, requiredContentType.Find(';')) == PObject::EqualTo)
+    return true;
+
+  PTRACE(2, "HTTP\tIncorrect Content-Type for document: expecting "
+         << requiredContentType << ", got " << actualContentType);
+  return false;
+}
+
+
+bool PHTTPClient::GetTextDocument(const PURL & url,
+                                     PString & document,
+                               const PString & requiredContentType)
 {
   PMIMEInfo outMIME, replyMIME;
   if (!GetDocument(url, outMIME, replyMIME))
-    return PFalse;
+    return false;
 
-  PCaselessString actualContentType = replyMIME(ContentTypeTag());
-  if (!requiredContentType.IsEmpty() && !actualContentType.IsEmpty() &&
-        actualContentType.NumCompare(requiredContentType, requiredContentType.Find(';')) != EqualTo) {
-    PTRACE(2, "HTTP\tIncorrect Content-Type for document: expecting " << requiredContentType << ", got " << actualContentType);
-    InternalReadContentBody(replyMIME, NULL); // Waste body
+  if (!CheckContentType(replyMIME, requiredContentType)) {
+    ReadContentBody(replyMIME); // Waste body
     return false;
   }
 
@@ -405,42 +513,98 @@ PBoolean PHTTPClient::GetTextDocument(const PURL & url,
 }
 
 
-PBoolean PHTTPClient::GetDocument(const PURL & url,
-                              PMIMEInfo & outMIME,
-                              PMIMEInfo & replyMIME)
+bool PHTTPClient::GetBinaryDocument(const PURL & url,
+                                    PBYTEArray & document,
+                                    const PString & requiredContentType)
+{
+  PMIMEInfo outMIME, replyMIME;
+  if (!GetDocument(url, outMIME, replyMIME))
+    return false;
+
+  if (!CheckContentType(replyMIME, requiredContentType)) {
+    ReadContentBody(replyMIME); // Waste body
+    return false;
+  }
+
+  if (!ReadContentBody(replyMIME, document)) {
+    PTRACE(2, "HTTP\tRead of body failed");
+    return false;
+  }
+
+  PTRACE_IF(4, !document.IsEmpty(), "HTTP\tReceived " << document.GetSize() << " byte body\n");
+  return true;
+}
+
+
+bool PHTTPClient::GetDocument(const PURL & url, ContentProcessor & processor)
+{
+  PMIMEInfo outMIME, replyMIME;
+  return IsOK(ExecuteCommand(GET, url, outMIME, PString::Empty(), replyMIME)) &&
+         ReadContentBody(replyMIME, processor);
+}
+
+
+bool PHTTPClient::GetDocument(const PURL & url, PMIMEInfo & replyMIME)
+{
+  PMIMEInfo outMIME;
+  return IsOK(ExecuteCommand(GET, url, outMIME, PString::Empty(), replyMIME));
+}
+
+
+bool PHTTPClient::GetDocument(const PURL & url, PMIMEInfo & outMIME, PMIMEInfo & replyMIME)
 {
   return IsOK(ExecuteCommand(GET, url, outMIME, PString::Empty(), replyMIME));
 }
 
 
-PBoolean PHTTPClient::GetHeader(const PURL & url,
-                            PMIMEInfo & outMIME,
-                            PMIMEInfo & replyMIME)
+bool PHTTPClient::GetHeader(const PURL & url, PMIMEInfo & replyMIME)
+{
+  PMIMEInfo outMIME;
+  return IsOK(ExecuteCommand(HEAD, url, outMIME, PString::Empty(), replyMIME));
+}
+
+
+bool PHTTPClient::GetHeader(const PURL & url, PMIMEInfo & outMIME, PMIMEInfo & replyMIME)
 {
   return IsOK(ExecuteCommand(HEAD, url, outMIME, PString::Empty(), replyMIME));
 }
 
 
-PBoolean PHTTPClient::PostData(const PURL & url,
-                           PMIMEInfo & outMIME,
-                           const PString & data,
-                           PMIMEInfo & replyMIME)
+bool PHTTPClient::PostData(const PURL & url, const PStringToString & data)
 {
-  PString dataBody = data;
-  if (!outMIME.Contains(ContentTypeTag())) {
+  PStringStream entityBody;
+  PURL::OutputVars(entityBody, data, '\0', '&', '=', PURL::QueryTranslation);
+  entityBody << "\r\n"; // Add CRLF for compatibility with some CGI servers.
+
+  PMIMEInfo outMIME;
+  return PostData(url, outMIME, entityBody);
+}
+
+
+bool PHTTPClient::PostData(const PURL & url, PMIMEInfo & outMIME, const PString & data)
+{
+  PMIMEInfo replyMIME;
+  return PostData(url, outMIME, data, replyMIME) && ReadContentBody(replyMIME);
+}
+
+
+bool PHTTPClient::PostData(const PURL & url,
+                            PMIMEInfo & outMIME,
+                        const PString & data,
+                            PMIMEInfo & replyMIME)
+{
+  if (!outMIME.Contains(ContentTypeTag()))
     outMIME.SetAt(ContentTypeTag(), "application/x-www-form-urlencoded");
-    dataBody += "\r\n"; // Add CRLF for compatibility with some CGI servers.
-  }
 
   return IsOK(ExecuteCommand(POST, url, outMIME, data, replyMIME));
 }
 
 
-PBoolean PHTTPClient::PostData(const PURL & url,
-                           PMIMEInfo & outMIME,
-                           const PString & data,
-                           PMIMEInfo & replyMIME,
-                           PString & body)
+bool PHTTPClient::PostData(const PURL & url,
+                            PMIMEInfo & outMIME,
+                        const PString & data,
+                            PMIMEInfo & replyMIME,
+                              PString & body)
 {
   return PostData(url, outMIME, data, replyMIME) && ReadContentBody(replyMIME, body);
 }
@@ -471,7 +635,7 @@ bool PHTTPClient::DeleteDocument(const PURL & url)
 }
 
 
-PBoolean PHTTPClient::AssureConnect(const PURL & url, PMIMEInfo & outMIME)
+bool PHTTPClient::AssureConnect(const PURL & url, PMIMEInfo & outMIME)
 {
   PString host = url.GetHostName();
 
@@ -491,21 +655,29 @@ PBoolean PHTTPClient::AssureConnect(const PURL & url, PMIMEInfo & outMIME)
         lastResponseCode = -2;
         lastResponseInfo = tcp->GetErrorText();
         delete tcp;
-        return PFalse;
+        return false;
       }
 
-      PSSLChannel * ssl = new PSSLChannel;
+      PSSLContext * context = new PSSLContext;
+      if (!context->SetCredentials(m_authority, m_certificate, m_privateKey)) {
+        lastResponseCode = -2;
+        lastResponseInfo = "Could not set certificates";
+        delete context;
+        return false;
+      }
+
+      PSSLChannel * ssl = new PSSLChannel(context);
       if (!ssl->Connect(tcp)) {
         lastResponseCode = -2;
         lastResponseInfo = ssl->GetErrorText();
         delete ssl;
-        return PFalse;
+        return false;
       }
 
       if (!Open(ssl)) {
         lastResponseCode = -2;
         lastResponseInfo = GetErrorText();
-        return PFalse;
+        return false;
       }
     }
     else
@@ -514,12 +686,11 @@ PBoolean PHTTPClient::AssureConnect(const PURL & url, PMIMEInfo & outMIME)
     if (!Connect(host, url.GetPort())) {
       lastResponseCode = -2;
       lastResponseInfo = GetErrorText();
-      return PFalse;
+      return false;
     }
   }
 
   // Have connection, so fill in the required MIME fields
-  static char HostTag[] = "Host";
   if (!outMIME.Contains(HostTag)) {
     if (!host)
       outMIME.SetAt(HostTag, host);
@@ -530,25 +701,31 @@ PBoolean PHTTPClient::AssureConnect(const PURL & url, PMIMEInfo & outMIME)
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
-void PHTTPClient::SetAuthenticationInfo(
-  const PString & userName,
-  const PString & password
-)
+void PHTTPClient::SetAuthenticationInfo(const PString & userName,const PString & password)
 {
   m_userName = userName;
   m_password = password;
 }
 
 
+#if P_SSL
+void PHTTPClient::SetSSLCredentials(const PString & authority, const PString & certificate, const PString & privateKey)
+{
+  m_authority = authority;
+  m_certificate = certificate;
+  m_privateKey = privateKey;
+}
+#endif
+
 ////////////////////////////////////////////////////////////////////////////////////
 
 PHTTPClientAuthentication::PHTTPClientAuthentication()
 {
-  isProxy = PFalse;
+  isProxy = false;
 }
 
 
@@ -639,7 +816,7 @@ PBoolean PHTTPClientBasicAuthentication::Authorise(AuthObject & authObject) cons
   PBase64 digestor;
   digestor.StartEncoding();
   digestor.ProcessEncoding(username + ":" + password);
-  PString result = digestor.GetEncodedString();
+  PString result = digestor.CompleteEncoding();
 
   PStringStream auth;
   auth << "Basic " << result;
@@ -694,10 +871,6 @@ PObject::Comparison PHTTPClientDigestAuthentication::Compare(const PObject & oth
   if (result != EqualTo)
     return result;
 
-  result = nonce.Compare(otherAuth->nonce);
-  if (result != EqualTo)
-    return result;
-
   return PHTTPClientAuthentication::Compare(other);
 }
 
@@ -710,7 +883,7 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
   opaque.MakeEmpty();
   algorithm = NumAlgorithms;
 
-  qopAuth = qopAuthInt = PFalse;
+  qopAuth = qopAuthInt = false;
   cnonce.MakeEmpty();
   nonceCount.SetValue(1);
 
@@ -726,7 +899,7 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
       algorithm = (Algorithm)(algorithm+1);
       if (algorithm >= PHTTPClientDigestAuthentication::NumAlgorithms) {
         PTRACE(1, "HTTP\tUnknown digest algorithm " << str);
-        return PFalse;
+        return false;
       }
     }
   }
@@ -734,13 +907,13 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
   authRealm = GetAuthParam(auth, "realm");
   if (authRealm.IsEmpty()) {
     PTRACE(1, "HTTP\tNo realm in authentication");
-    return PFalse;
+    return false;
   }
 
   nonce = GetAuthParam(auth, "nonce");
   if (nonce.IsEmpty()) {
     PTRACE(1, "HTTP\tNo nonce in authentication");
-    return PFalse;
+    return false;
   }
 
   opaque = GetAuthParam(auth, "opaque");
@@ -751,7 +924,7 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
   PString qopStr = GetAuthParam(auth, "qop");
   if (!qopStr.IsEmpty()) {
     PTRACE(3, "HTTP\tAuthentication contains qop-options " << qopStr);
-    PStringList options = qopStr.Tokenise(',', PTrue);
+    PStringList options = qopStr.Tokenise(',', true);
     qopAuth    = options.GetStringsIndex("auth") != P_MAX_INDEX;
     qopAuthInt = options.GetStringsIndex("auth-int") != P_MAX_INDEX;
     cnonce = PGloballyUniqueID().AsString();
@@ -762,7 +935,7 @@ PBoolean PHTTPClientDigestAuthentication::Parse(const PString & p_auth, PBoolean
   stale = staleStr.Find("true") != P_MAX_INDEX;
 
   isProxy = proxy;
-  return PTrue;
+  return true;
 }
 
 
@@ -836,7 +1009,7 @@ PBoolean PHTTPClientDigestAuthentication::Authorise(AuthObject & authObject) con
          << "response=\"" << AsHex(response) << "\", "
          << "cnonce=\"" << cnonce << "\", "
          << "nc=" << nc << ", "
-         << "qop=\"" << qop << '"';
+         << "qop=" << qop;
   }
   else {
     digestor.Process(AsHex(a2));
@@ -848,32 +1021,43 @@ PBoolean PHTTPClientDigestAuthentication::Authorise(AuthObject & authObject) con
     auth << ", opaque=\"" << opaque << "\"";
 
   authObject.GetMIME().SetAt(isProxy ? "Proxy-Authorization" : "Authorization", auth);
-  return PTrue;
+  return true;
 }
 
+
+static void AuthError(PString & errorMsg, const char * errText, const PString & scheme)
+{
+  if (!errorMsg.IsEmpty())
+    errorMsg += ", ";
+  errorMsg += errText;
+  errorMsg += " scheme \"";
+  errorMsg += scheme;
+  errorMsg += '"';
+}
+
+
 PHTTPClientAuthentication * PHTTPClientAuthentication::ParseAuthenticationRequired(bool isProxy, const PMIMEInfo & replyMIME, PString & errorMsg)
 {
-  PString line = replyMIME(isProxy ? "Proxy-Authenticate" : "WWW-Authenticate");
+  PStringArray lines = replyMIME(isProxy ? "Proxy-Authenticate" : "WWW-Authenticate").Lines();
 
   // find authentication
-  PINDEX pos = line.Find(' ');
-  PString scheme = line.Left(pos).Trim().ToLower();
-  PHTTPClientAuthentication * newAuth = PHTTPClientAuthenticationFactory::CreateInstance(scheme);
-  if (newAuth == NULL) {
-    delete newAuth;
-    errorMsg = "Unknown authentication scheme " + scheme;
-    return NULL;
+  for (PINDEX i = 0; i < lines.GetSize(); ++i) {
+    PString line = lines[i];
+    PString scheme = line.Left(line.Find(' ')).Trim().ToLower();
+    PHTTPClientAuthentication * newAuth = PHTTPClientAuthenticationFactory::CreateInstance(scheme);
+    if (newAuth == NULL)
+      AuthError(errorMsg, "Unknown authentication", scheme);
+    else {
+      // parse the new authentication scheme
+      if (newAuth->Parse(line, isProxy))
+        return newAuth;
+
+      delete newAuth;
+      AuthError(errorMsg, "Failed to parse authentication for", scheme);
+    }
   }
 
-  // parse the new authentication scheme
-  if (!newAuth->Parse(line, isProxy)) {
-    delete newAuth;
-    errorMsg = "Failed to parse authentication for scheme " + scheme;
-    return NULL;
-  }
-
-  // switch authentication schemes
-  return newAuth;
+  return NULL;
 }
 
 
@@ -915,23 +1099,35 @@ class PURL_HttpLoader : public PURLLoader
 {
     PCLASSINFO(PURL_HttpLoader, PURLLoader);
   public:
-    virtual bool Load(const PURL & url, PString & str, const PString & requiredContentType)
+    virtual bool Load(PString & str, const PURL & url, const PURL::LoadParams & params) const
     {
       PHTTPClient http;
-      return http.GetTextDocument(url, str, requiredContentType);
+      http.SetPersistent(false);
+      http.SetReadTimeout(params.m_timeout);
+      http.SetAuthenticationInfo(params.m_username, params.m_password);
+#if P_SSL
+      http.SetSSLCredentials(params.m_authority, params.m_certificate, params.m_privateKey);
+#endif
+      return http.GetTextDocument(url, str, params.m_requiredContentType);
     }
 
-    virtual bool Load(const PURL & url, PBYTEArray & data, const PString & requiredContentType)
+    virtual bool Load(PBYTEArray & data, const PURL & url, const PURL::LoadParams & params) const
     {
       PHTTPClient http;
+      http.SetPersistent(false);
+      http.SetReadTimeout(params.m_timeout);
+      http.SetAuthenticationInfo(params.m_username, params.m_password);
+#if P_SSL
+      http.SetSSLCredentials(params.m_authority, params.m_certificate, params.m_privateKey);
+#endif
       PMIMEInfo outMIME, replyMIME;
       if (!http.GetDocument(url, outMIME, replyMIME))
         return false;
 
       PCaselessString actualContentType = replyMIME(PHTTP::ContentTypeTag());
-      if (!requiredContentType.IsEmpty() && !actualContentType.IsEmpty() &&
-            actualContentType.NumCompare(requiredContentType, requiredContentType.Find(';')) != EqualTo) {
-        PTRACE(2, "HTTP\tIncorrect Content-Type for document: expecting " << requiredContentType << ", got " << actualContentType);
+      if (!params.m_requiredContentType.IsEmpty() && !actualContentType.IsEmpty() &&
+            actualContentType.NumCompare(params.m_requiredContentType, params.m_requiredContentType.Find(';')) != EqualTo) {
+        PTRACE(2, "HTTP\tIncorrect Content-Type for document: expecting " << params.m_requiredContentType << ", got " << actualContentType);
         return false;
       }
 
