@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21551 $
+ * $Revision: 28989 $
  * $Author: rjongbloed $
- * $Date: 2008-11-19 00:09:07 +0000 (Wed, 19 Nov 2008) $
+ * $Date: 2013-01-25 00:26:28 -0600 (Fri, 25 Jan 2013) $
  */
 
 #ifdef __GNUC__
@@ -51,7 +51,7 @@ PHTTPField::PHTTPField(const char * nam, const char * titl, const char * hlp)
     title(titl != NULL ? titl : nam),
     help(hlp != NULL ? hlp : "")
 {
-  notInHTML = PTrue;
+  notInHTML = true;
 }
 
 
@@ -105,16 +105,16 @@ static PBoolean FindSpliceBlock(const PRegularExpression & startExpr,
   start = finish = P_MAX_INDEX;
 
   if (!text.FindRegEx(startExpr, pos, len, offset))
-    return PFalse;
+    return false;
 
   PINDEX endpos, endlen;
   if (!text.FindRegEx(endExpr, endpos, endlen, pos+len))
-    return PTrue;
+    return true;
 
   start = pos + len;
   finish = endpos - 1;
   len = endpos - pos + endlen;
-  return PTrue;
+  return true;
 }
 
 
@@ -142,10 +142,10 @@ static PBoolean FindSpliceName(const PCaselessString & text,
     static PRegularExpression NameExpr("name[ \t\r\n]*=[ \t\r\n]*\"[^\"]*\"",
                                        PRegularExpression::Extended|PRegularExpression::IgnoreCase);
     if ((pos = text.FindRegEx(NameExpr, start)) == P_MAX_INDEX)
-      return PFalse;
+      return false;
 
     if (pos >= finish)
-      return PFalse;
+      return false;
 
     pos = text.Find('"', pos) + 1;
     end = text.Find('"', pos) - 1;
@@ -177,16 +177,16 @@ static PBoolean FindSpliceFieldName(const PString & text,
                                       "<[a-z]+[ \t\r\n][^>]*name[ \t\r\n]*=[ \t\r\n]*\"[^\"]*\"[^>]*>",
                                       PRegularExpression::Extended|PRegularExpression::IgnoreCase);
   if (!text.FindRegEx(FieldName, pos, len, offset))
-    return PFalse;
+    return false;
 
   PINDEX nameStart, nameEnd;
   if (!FindSpliceName(text, pos, pos+len-1, nameStart, nameEnd))
-    return PFalse;
+    return false;
 
   name = text(nameStart, nameEnd);
   pos = nameStart;
   len = nameEnd - nameStart + 1;
-  return PTrue;
+  return true;
 }
 
 
@@ -224,7 +224,7 @@ static PBoolean FindInputValue(const PString & text, PINDEX & before, PINDEX & a
                                   PRegularExpression::Extended|PRegularExpression::IgnoreCase);
   PINDEX pos = text.FindRegEx(Value);
   if (pos == P_MAX_INDEX)
-    return PFalse;
+    return false;
 
   before = text.Find('"', pos);
   if (before != P_MAX_INDEX)
@@ -237,17 +237,19 @@ static PBoolean FindInputValue(const PString & text, PINDEX & before, PINDEX & a
     while (text[after] != '\0' && text[after] != '>' && !isspace(text[after]))
       after++;
   }
-  return PTrue;
+  return true;
 }
 
 
 PString PHTTPField::GetHTMLInput(const PString & input) const
 {
+  PStringStream adjusted;
   PINDEX before, after;
   if (FindInputValue(input, before, after))
-    return input(0, before) + GetValue(PFalse) + input.Mid(after);
-
-  return "<input value=\"" + GetValue(PFalse) + "\"" + input.Mid(6);
+    adjusted << input(0, before) << PHTML::Escaped(GetValue(false)) << input.Mid(after);
+  else
+    adjusted << "<INPUT VALUE=\"" << PHTML::Escaped(GetValue(false)) << '"' << input.Mid(6);
+  return adjusted;
 }
 
 
@@ -329,7 +331,7 @@ PString PHTTPField::GetHTMLSelect(const PString & selection) const
   PString text = selection;
   PStringArray dummy1;
   PINDEX dummy2 = P_MAX_INDEX;
-  AdjustSelectOptions(text, 0, P_MAX_INDEX, GetValue(PFalse), dummy1, dummy2);
+  AdjustSelectOptions(text, 0, P_MAX_INDEX, GetValue(false), dummy1, dummy2);
   return text;
 }
 
@@ -365,10 +367,10 @@ void PHTTPField::LoadFromConfig(PConfig & cfg)
   PString section, key;
   switch (SplitConfigKey(fullName, section, key)) {
     case 1 :
-      SetValue(cfg.GetString(key, GetValue(PTrue)));
+      SetValue(cfg.GetString(key, GetValue(true)));
       break;
     case 2 :
-      SetValue(cfg.GetString(section, key, GetValue(PTrue)));
+      SetValue(cfg.GetString(section, key, GetValue(true)));
   }
 }
 
@@ -388,7 +390,7 @@ void PHTTPField::SaveToConfig(PConfig & cfg) const
 
 PBoolean PHTTPField::Validated(const PString &, PStringStream &) const
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -409,7 +411,7 @@ PBoolean PHTTPField::ValidateAll(const PStringToString & data, PStringStream & m
 {
   if (data.Contains(fullName))
     return Validated(data[fullName], msg);
-  return PTrue;
+  return true;
 }
 
 
@@ -418,8 +420,10 @@ PBoolean PHTTPField::ValidateAll(const PStringToString & data, PStringStream & m
 
 PHTTPCompositeField::PHTTPCompositeField(const char * nam,
                                          const char * titl,
-                                         const char * hlp)
+                                         const char * hlp,
+                                         bool includeHeaders)
   : PHTTPField(nam, titl, hlp)
+  , m_includeHeaders(includeHeaders)
 {
 }
 
@@ -476,11 +480,20 @@ PHTTPField * PHTTPCompositeField::NewField() const
 
 void PHTTPCompositeField::GetHTMLTag(PHTML & html) const
 {
+  if (m_includeHeaders) {
+    html << PHTML::TableStart("border=1 cellspacing=0 cellpadding=8");
+    GetHTMLHeading(html);
+    html << PHTML::TableRow();
+  }
+
   for (PINDEX i = 0; i < fields.GetSize(); i++) {
-    if (i != 0 && html.Is(PHTML::InTable))
+    if (m_includeHeaders || (i != 0 && html.Is(PHTML::InTable)))
       html << PHTML::TableData("NOWRAP ALIGN=CENTER");
     fields[i].GetHTMLTag(html);
   }
+
+  if (m_includeHeaders)
+    html << PHTML::TableEnd();
 }
 
 
@@ -517,7 +530,7 @@ void PHTTPCompositeField::GetHTMLHeading(PHTML & html) const
 {
   html << PHTML::TableRow();
   for (PINDEX i = 0; i < fields.GetSize(); i++)
-    html << PHTML::TableHeader() << fields[i].GetTitle();
+    html << PHTML::TableHeader() << PHTML::Escaped(fields[i].GetTitle());
 }
 
 
@@ -570,9 +583,9 @@ PBoolean PHTTPCompositeField::ValidateAll(const PStringToString & data,
 {
   for (PINDEX i = 0; i < fields.GetSize(); i++)
     if (!fields[i].ValidateAll(data, msg))
-      return PFalse;
+      return false;
 
-  return PTrue;
+  return true;
 }
 
 
@@ -620,10 +633,10 @@ void PHTTPSubForm::GetHTMLTag(PHTML & html) const
     value = "New";
   html << PHTML::HotLink(subFormName +
             "?subformprefix=" + PURL::TranslateString(fullName, PURL::QueryTranslation))
-       << value << PHTML::HotLink();
+       << PHTML::Escaped(value) << PHTML::HotLink();
 
   if (secondary != P_MAX_INDEX)
-    html << PHTML::TableData("NOWRAP") << fields[secondary].GetValue();
+    html << PHTML::TableData("NOWRAP") << PHTML::Escaped(fields[secondary].GetValue());
 }
 
 
@@ -716,7 +729,7 @@ void PHTTPFieldArray::AddArrayControlBox(PHTML & html, PINDEX fld) const
   PStringArray options = GetArrayControlOptions(fld, fields.GetSize()-1, orderedArray);
   html << PHTML::Select(fields[fld].GetName() + ArrayControlBox);
   for (PINDEX i = 0; i < options.GetSize(); i++)
-    html << PHTML::Option(i == 0 ? PHTML::Selected : PHTML::NotSelected) << options[i];
+    html << PHTML::Option(i == 0 ? PHTML::Selected : PHTML::NotSelected) << PHTML::Escaped(options[i]);
   html << PHTML::Select();
 }
 
@@ -888,7 +901,7 @@ void PHTTPFieldArray::SetAllValues(const PStringToString & data)
   for (i = 0; i < fields.GetSize(); i++)
     newFields.SetAt(i, fields.GetAt(i));
 
-  PBoolean lastFieldIsSet = PFalse;
+  PBoolean lastFieldIsSet = false;
   PINDEX size = fields.GetSize();
   for (i = 0; i < size; i++) {
     PHTTPField * fieldPtr = &fields[i];
@@ -920,14 +933,14 @@ void PHTTPFieldArray::SetAllValues(const PStringToString & data)
       if (i == size-1) {
         newFields.RemoveAt(pos);
         newFields.InsertAt(0, fieldPtr);
-        lastFieldIsSet = PTrue;
+        lastFieldIsSet = true;
       }
     }
     else if (control == ArrayControlAddBottom || control == ArrayControlAdd) {
       if (i == size-1) {
         newFields.RemoveAt(pos);
         newFields.Append(fieldPtr);
-        lastFieldIsSet = PTrue;
+        lastFieldIsSet = true;
       }
     }
     else if (control == ArrayControlIgnore) {
@@ -978,7 +991,7 @@ PStringArray PHTTPFieldArray::GetStrings(PConfig & cfg)
   PStringArray values(GetSize());
 
   for (PINDEX i = 0; i < GetSize(); i++)
-    values[i] = fields[i].GetValue(PFalse);
+    values[i] = fields[i].GetValue(false);
 
   return values;
 }
@@ -998,42 +1011,74 @@ void PHTTPFieldArray::SetStrings(PConfig & cfg, const PStringArray & values)
 // PHTTPStringField
 
 PHTTPStringField::PHTTPStringField(const char * name,
-                                   PINDEX siz,
+                                   PINDEX maxLen,
                                    const char * initVal,
-                                   const char * help)
-  : PHTTPField(name, NULL, help),
-    value(initVal != NULL ? initVal : ""),
-    initialValue(value)
+                                   const char * help,
+                                   int r,
+                                   int c)
+  : PHTTPField(name, NULL, help)
+  , value(initVal != NULL ? initVal : "")
+  , initialValue(value)
+  , maxLength(maxLen)
+  , rows(r)
+  , columns(c)
 {
-  size = siz;
 }
 
 
 PHTTPStringField::PHTTPStringField(const char * name,
                                    const char * title,
-                                   PINDEX siz,
+                                   PINDEX maxLen,
                                    const char * initVal,
-                                   const char * help)
-  : PHTTPField(name, title, help),
-    value(initVal != NULL ? initVal : ""),
-    initialValue(value)
+                                   const char * help,
+                                   int r,
+                                   int c)
+  : PHTTPField(name, title, help)
+  , value(initVal != NULL ? initVal : "")
+  , initialValue(value)
+  , maxLength(maxLen)
+  , rows(r)
+  , columns(c)
 {
-  size = siz;
 }
 
 
 PHTTPField * PHTTPStringField::NewField() const
 {
-  return new PHTTPStringField(baseName, title, size, initialValue, help);
+  return new PHTTPStringField(baseName, title, maxLength, initialValue, help);
 }
 
 
 void PHTTPStringField::GetHTMLTag(PHTML & html) const
 {
-  if (size < 128)
-    html << PHTML::InputText(fullName, size, value);
+  const int DefaultColumns = 80;
+  int r, c;
+  if (rows == 0) {
+    if (columns != 0) {
+      c = columns;
+      r = (maxLength+c-1)/c;
+    }
+    else if (maxLength < DefaultColumns*2) {
+      c = maxLength;
+      r = 1;
+    }
+    else {
+      c = DefaultColumns;
+      r = (maxLength+c-1)/c;
+    }
+  }
+  else {
+    r = rows;
+    if (columns != 0)
+      c = columns;
+    else
+      c = (maxLength+r-1)/r;
+  }
+
+  if (r <= 1)
+    html << PHTML::InputText(fullName, c, maxLength);
   else
-    html << PHTML::TextArea(fullName, (size+79)/80, 80) << value << PHTML::TextArea(fullName);
+    html << PHTML::TextArea(fullName, r, c) << PHTML::Escaped(value) << PHTML::TextArea(fullName);
 }
 
 
@@ -1076,13 +1121,13 @@ PHTTPPasswordField::PHTTPPasswordField(const char * name,
 
 PHTTPField * PHTTPPasswordField::NewField() const
 {
-  return new PHTTPPasswordField(baseName, title, size, initialValue, help);
+  return new PHTTPPasswordField(baseName, title, maxLength, initialValue, help);
 }
 
 
 void PHTTPPasswordField::GetHTMLTag(PHTML & html) const
 {
-  html << PHTML::InputPassword(fullName, size, value);
+  html << PHTML::InputPassword(fullName, maxLength);
 }
 
 
@@ -1117,6 +1162,46 @@ PString PHTTPPasswordField::Decrypt(const PString & pword)
 }
 
 
+//////////////////////////////////////////////////////////////////////////////
+// PHTTPDateField
+
+PHTTPDateField::PHTTPDateField(const char * name, const PTime & initVal, PTime::TimeFormat fmt)
+  : PHTTPStringField(name, 30, initVal.AsString(fmt))
+  , m_format(fmt)
+{
+}
+
+
+PHTTPField * PHTTPDateField::NewField() const
+{
+  return new PHTTPDateField(baseName, value);
+}
+
+
+void PHTTPDateField::SetValue(const PString & newValue)
+{
+  PTime test(newValue);
+  if (test.IsValid())
+    value = test.AsString(m_format);
+  else
+    value = newValue;
+}
+
+
+PBoolean PHTTPDateField::Validated(const PString & newValue, PStringStream & msg) const
+{
+  if (newValue.IsEmpty())
+    return true;
+
+  PTime test(newValue);
+  if (test.IsValid())
+    return true;
+
+  msg << "Invalid time specification.";
+  return false;
+}
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PHTTPIntegerField
 
@@ -1154,7 +1239,7 @@ PHTTPField * PHTTPIntegerField::NewField() const
 
 void PHTTPIntegerField::GetHTMLTag(PHTML & html) const
 {
-  html << PHTML::InputRange(fullName, low, high, value) << "  " << units;
+  html << PHTML::InputNumber(fullName, low, high, value) << "  " << PHTML::Escaped(units);
 }
 
 
@@ -1200,11 +1285,11 @@ PBoolean PHTTPIntegerField::Validated(const PString & newVal, PStringStream & ms
 {
   int val = newVal.AsInteger();
   if (val >= low && val <= high)
-    return PTrue;
+    return true;
 
   msg << "The field \"" << GetName() << "\" should be between "
       << low << " and " << high << ".<BR>";
-  return PFalse;
+  return false;
 }
 
 
@@ -1464,7 +1549,7 @@ void PHTTPRadioField::GetHTMLTag(PHTML & html) const
   for (PINDEX i = 0; i < values.GetSize(); i++)
     html << PHTML::RadioButton(fullName, values[i],
                         values[i] == value ? PHTML::Checked : PHTML::UnChecked)
-         << titles[i]
+         << PHTML::Escaped(titles[i])
          << PHTML::BreakLine();
 }
 
@@ -1506,13 +1591,14 @@ void PHTTPRadioField::SetValue(const PString & newVal)
 PHTTPSelectField::PHTTPSelectField(const char * name,
                                    const PStringArray & valueArray,
                                    PINDEX initVal,
-                                   const char * help)
-  : PHTTPField(name, NULL, help),
-    values(valueArray)
+                                   const char * help,
+                                   bool enumeration)
+  : PHTTPField(name, NULL, help)
+  , m_values(valueArray)
+  , m_enumeration(enumeration)
+  , m_initialValue(initVal)
+  , m_value(initVal < m_values.GetSize() ? m_values[initVal] : PString::Empty())
 {
-  initialValue = initVal;
-  if (initVal < values.GetSize())
-    value = values[initVal];
 }
 
 
@@ -1520,13 +1606,14 @@ PHTTPSelectField::PHTTPSelectField(const char * name,
                                    PINDEX count,
                                    const char * const * valueStrings,
                                    PINDEX initVal,
-                                   const char * help)
-  : PHTTPField(name, NULL, help),
-    values(count, valueStrings)
+                                   const char * help,
+                                   bool enumeration)
+  : PHTTPField(name, NULL, help)
+  , m_values(count, valueStrings)
+  , m_enumeration(enumeration)
+  , m_initialValue(initVal)
+  , m_value(initVal < m_values.GetSize() ? m_values[initVal] : PString::Empty())
 {
-  initialValue = initVal;
-  if (initVal < count)
-    value = values[initVal];
 }
 
 
@@ -1534,13 +1621,14 @@ PHTTPSelectField::PHTTPSelectField(const char * name,
                                    const char * title,
                                    const PStringArray & valueArray,
                                    PINDEX initVal,
-                                   const char * help)
-  : PHTTPField(name, title, help),
-    values(valueArray)
+                                   const char * help,
+                                   bool enumeration)
+  : PHTTPField(name, title, help)
+  , m_values(valueArray)
+  , m_enumeration(enumeration)
+  , m_initialValue(initVal)
+  , m_value(initVal < m_values.GetSize() ? m_values[initVal] : PString::Empty())
 {
-  initialValue = initVal;
-  if (initVal < values.GetSize())
-    value = values[initVal];
 }
 
 
@@ -1549,28 +1637,30 @@ PHTTPSelectField::PHTTPSelectField(const char * name,
                                    PINDEX count,
                                    const char * const * valueStrings,
                                    PINDEX initVal,
-                                   const char * help)
-  : PHTTPField(name, title, help),
-    values(count, valueStrings)
+                                   const char * help,
+                                   bool enumeration)
+  : PHTTPField(name, title, help)
+  , m_values(count, valueStrings)
+  , m_enumeration(enumeration)
+  , m_initialValue(initVal)
+  , m_value(initVal < m_values.GetSize() ? m_values[initVal] : PString::Empty())
 {
-  initialValue = initVal;
-  if (initVal < values.GetSize())
-    value = values[initVal];
 }
 
 
 PHTTPField * PHTTPSelectField::NewField() const
 {
-  return new PHTTPSelectField(baseName, title, values, initialValue, help);
+  return new PHTTPSelectField(baseName, title, m_values, m_initialValue, help);
 }
 
 
 void PHTTPSelectField::GetHTMLTag(PHTML & html) const
 {
   html << PHTML::Select(fullName);
-  for (PINDEX i = 0; i < values.GetSize(); i++)
-    html << PHTML::Option(values[i] == value ? PHTML::Selected : PHTML::NotSelected)
-         << values[i];
+  for (PINDEX i = 0; i < m_values.GetSize(); i++)
+    html << PHTML::Option(m_values[i] == m_value ? PHTML::Selected : PHTML::NotSelected,
+                          m_enumeration ? psprintf("value=\"%u\"", i) : PString::Empty())
+         << PHTML::Escaped(m_values[i]);
   html << PHTML::Select();
 }
 
@@ -1578,18 +1668,18 @@ void PHTTPSelectField::GetHTMLTag(PHTML & html) const
 PString PHTTPSelectField::GetValue(PBoolean dflt) const
 {
   if (dflt)
-    if (initialValue < values.GetSize())
-      return values[initialValue];
+    if (m_initialValue < m_values.GetSize())
+      return m_values[m_initialValue];
     else
-      return PString();
+      return PString::Empty();
   else
-    return value;
+    return m_value;
 }
 
 
 void PHTTPSelectField::SetValue(const PString & newVal)
 {
-  value = newVal;
+  m_value = newVal;
 }
 
 
@@ -1664,13 +1754,13 @@ static PBoolean FindSpliceField(const PRegularExpression & startExpr,
   field = NULL;
 
   if (!FindSpliceBlock(startExpr, endExpr, text, offset, pos, len, start, finish))
-    return PFalse;
+    return false;
 
   PINDEX endBlock = start != finish ? (start-1) : (pos+len-1);
   PINDEX namePos, nameEnd;
   if (FindSpliceName(text, pos, endBlock, namePos, nameEnd))
     field = rootField.LocateName(text(namePos, nameEnd));
-  return PTrue;
+  return true;
 }
 
 
@@ -1775,7 +1865,7 @@ void PHTTPForm::OnLoadedText(PHTTPRequest & request, PString & text)
                                        PRegularExpression::Extended|PRegularExpression::IgnoreCase);
   while (FindSpliceField(ValueRegEx, PRegularExpression(), text, pos+len, fields, pos, len, start, finish, field)) {
     if (field != NULL)
-      text.Splice(field->GetValue(), pos, len);
+      text.Splice(PHTML::Escape(field->GetValue()), pos, len);
   }
 
   pos = len = 0;
@@ -1807,7 +1897,7 @@ void PHTTPForm::OnLoadedText(PHTTPRequest & request, PString & text)
   static PRegularExpression TextEndRegEx("</textarea[^>]*>", PRegularExpression::IgnoreCase);
   while (FindSpliceField(TextRegEx, TextEndRegEx, text, pos+len, fields, pos, len, start, finish, field)) {
     if (field != NULL)
-      text.Splice(field->GetValue(), start, finish-start+1);
+      text.Splice(PHTML::Escape(field->GetValue()), start, finish-start+1);
   }
 }
 
@@ -1817,7 +1907,7 @@ PHTTPField * PHTTPForm::Add(PHTTPField * fld)
   if (PAssertNULL(fld) == NULL)
     return NULL;
 
-  PAssert(!fieldNames[fld->GetName()], "Field already on form!");
+  PAssert(!fieldNames[fld->GetName()], "Field " + fld->GetName() + " already on form!");
   fieldNames += fld->GetName();
   fields.Append(fld);
   return fld;
@@ -1849,7 +1939,7 @@ void PHTTPForm::BuildHTML(PHTML & html, BuildOptions option)
     if (field.NotYetInHTML()) {
       html << PHTML::TableRow()
            << PHTML::TableData("align=right")
-           << field.GetTitle()
+           << PHTML::Escaped(field.GetTitle())
            << PHTML::TableData("align=left")
            << "<!--#form html " << field.GetName() << "-->"
            << PHTML::TableData()
@@ -1920,7 +2010,7 @@ PBoolean PHTTPForm::Post(PHTTPRequest & request,
       while (FindSpliceAccepted(msg, pos, pos, len, start, finish))
         msg.Delete(pos, len);
 
-      PBoolean appendErrors = PTrue;
+      PBoolean appendErrors = true;
       pos = 0;
       while (FindSpliceErrors(msg, pos, pos, len, start, finish)) {
         PString block = msg(start, finish);
@@ -1932,7 +2022,7 @@ PBoolean PHTTPForm::Post(PHTTPRequest & request,
         else
           block += errors;
         msg.Splice(block, pos, len);
-        appendErrors = PFalse;
+        appendErrors = false;
       }
 
       if (appendErrors)
@@ -1940,7 +2030,7 @@ PBoolean PHTTPForm::Post(PHTTPRequest & request,
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -2036,14 +2126,14 @@ PBoolean PHTTPConfig::Post(PHTTPRequest & request,
 
   PHTTPForm::Post(request, data, msg);
   if (request.code != PHTTP::RequestOK)
-    return PTrue;
+    return true;
 
   if (sectionField != NULL)
     section = sectionPrefix + sectionField->GetValue() + sectionSuffix;
 
   PString sectionName = request.url.GetQueryVars()("section", section);
   if (sectionName.IsEmpty())
-    return PTrue;
+    return true;
 
   PConfig cfg(sectionName);
 
@@ -2086,7 +2176,7 @@ PBoolean PHTTPConfig::Post(PHTTPRequest & request,
   }
 
   section = sectionName;
-  return PTrue;
+  return true;
 }
 
 
@@ -2133,9 +2223,9 @@ PHTTPConfigSectionList::PHTTPConfigSectionList(const PURL & url,
   : PHTTPString(url, auth),
     sectionPrefix(prefix),
     additionalValueName(valueName),
-    newSectionLink(newSection.AsString(PURL::URIOnly)),
+    newSectionLink(newSection.AsString(PURL::RelativeOnly)),
     newSectionTitle(newTitle),
-    editSectionLink(editSection.AsString(PURL::URIOnly) +
+    editSectionLink(editSection.AsString(PURL::RelativeOnly) +
                       "?section=" + PURL::TranslateString(prefix, PURL::QueryTranslation))
 {
   if (heading.Is(PHTML::InBody))
@@ -2163,12 +2253,12 @@ void PHTTPConfigSectionList::OnLoadedText(PHTTPRequest &, PString & text)
           html << PHTML::TableRow()
                << PHTML::TableData()
                << PHTML::HotLink(editSectionLink + PURL::TranslateString(name, PURL::QueryTranslation))
-               << name
+               << PHTML::Escaped(name)
                << PHTML::HotLink();
           if (!additionalValueName)
             html << PHTML::TableData()
                  << PHTML::HotLink(editSectionLink + PURL::TranslateString(name, PURL::QueryTranslation))
-                 << cfg.GetString(nameList[i], additionalValueName, "")
+                 << PHTML::Escaped(cfg.GetString(nameList[i], additionalValueName, ""))
                  << PHTML::HotLink();
           html << PHTML::TableData() << PHTML::SubmitButton("Remove", name);
         }
@@ -2177,7 +2267,7 @@ void PHTTPConfigSectionList::OnLoadedText(PHTTPRequest &, PString & text)
       html << PHTML::TableRow()
            << PHTML::TableData()
            << PHTML::HotLink(newSectionLink)
-           << newSectionTitle
+           << PHTML::Escaped(newSectionTitle)
            << PHTML::HotLink()
            << PHTML::TableEnd()
            << PHTML::Form();
@@ -2195,12 +2285,12 @@ void PHTTPConfigSectionList::OnLoadedText(PHTTPRequest &, PString & text)
           text.Splice(repeat, pos, 0);
           text.Replace("<!--#form hotlink-->",
                        editSectionLink + PURL::TranslateString(name, PURL::QueryTranslation),
-                       PTrue, pos);
+                       true, pos);
           if (!additionalValueName)
             text.Replace("<!--#form additional-->",
                          cfg.GetString(nameList[i], additionalValueName, ""),
-                         PTrue, pos);
-          text.Replace("<!--#form section-->", name, PTrue, pos);
+                         true, pos);
+          text.Replace("<!--#form section-->", name, true, pos);
           pos = text.Find(FormListInclude, pos);
         }
       }
@@ -2227,7 +2317,7 @@ PBoolean PHTTPConfigSectionList::Post(PHTTPRequest &,
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
