@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24367 $
+ * $Revision: 29190 $
  * $Author: rjongbloed $
- * $Date: 2010-05-21 22:15:14 -0500 (Fri, 21 May 2010) $
+ * $Date: 2013-03-03 18:10:49 -0600 (Sun, 03 Mar 2013) $
  */
 
 #include <ptlib.h>
@@ -45,7 +45,7 @@
 #define STRANGE_NETSCAPE_BUG  3
 
 // maximum delay between characters whilst reading a line of text
-#define READLINE_TIMEOUT  30
+static const PTimeInterval ReadLineTimeout(0, 30);
 
 #define DEFAULT_PERSIST_TIMEOUT 30
 #define DEFAULT_PERSIST_TRANSATIONS 10
@@ -110,7 +110,7 @@ PBoolean PHTTPSpace::AddResource(PHTTPResource * res, AddOptions overwrite)
 
     if (node->resource != NULL) {
       delete res;
-      return PFalse;   // Already a resource in tree in partial path
+      return false;   // Already a resource in tree in partial path
     }
 
     PINDEX pos = node->children.GetValuesIndex(path[i]);
@@ -122,18 +122,18 @@ PBoolean PHTTPSpace::AddResource(PHTTPResource * res, AddOptions overwrite)
 
   if (!node->children.IsEmpty()) {
     delete res;
-    return PFalse;   // Already a resource in tree further down path.
+    return false;   // Already a resource in tree further down path.
   }
 
   if (overwrite == ErrorOnExist && node->resource != NULL) {
     delete res;
-    return PFalse;   // Already a resource in tree at leaf
+    return false;   // Already a resource in tree at leaf
   }
 
   delete node->resource;
   node->resource = res;
 
-  return PTrue;
+  return true;
 }
 
 
@@ -147,28 +147,28 @@ PBoolean PHTTPSpace::DelResource(const PURL & url)
 
     PINDEX pos = node->children.GetValuesIndex(path[i]);
     if (pos == P_MAX_INDEX)
-      return PFalse;
+      return false;
 
     node = &node->children[pos];
 
     // If have resource and not last node, then trying to remove something
     // further down the tree than a leaf node.
     if (node->resource != NULL && i < (path.GetSize()-1))
-      return PFalse;
+      return false;
   }
 
   if (!node->children.IsEmpty())
-    return PFalse;   // Still a resource in tree further down path.
+    return false;   // Still a resource in tree further down path.
 
   if (node->parent != NULL) {
     do {
       Node * par = node->parent;
       par->children.Remove(node);
       node = par;
-    } while (node != NULL && node->children.IsEmpty());
+    } while (node->parent != NULL && node->children.IsEmpty());
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -226,7 +226,7 @@ PHTTPServer::PHTTPServer(const PHTTPSpace & space)
 void PHTTPServer::Construct()
 {
   transactionCount = 0;
-  SetReadLineTimeout(PTimeInterval(0, READLINE_TIMEOUT));
+  SetReadLineTimeout(ReadLineTimeout);
 }
 
 
@@ -242,7 +242,7 @@ PBoolean PHTTPServer::ProcessCommand()
 
   // this will only return false upon timeout or completely invalid command
   if (!ReadCommand(cmd, args))
-    return PFalse;
+    return false;
 
   connectInfo.commandCode = (Commands)cmd;
   if (cmd < NumCommands)
@@ -256,11 +256,11 @@ PBoolean PHTTPServer::ProcessCommand()
   // if no tokens, error
   if (args.IsEmpty()) {
     OnError(BadRequest, args, connectInfo);
-    return PFalse;
+    return false;
   }
 
   if (!connectInfo.Initialise(*this, args))
-      return PFalse;
+    return false;
 
   // now that we've decided we did receive a HTTP request, increment the
   // count of transactions
@@ -274,18 +274,20 @@ PBoolean PHTTPServer::ProcessCommand()
   // mangle it into a proper URL and do NOT close the connection.
   // for all other commands, close the read connection if not persistent
   if (cmd == CONNECT) 
-    connectInfo.url = "https://" + args;
+    connectInfo.url.Parse("https://" + args);
   else {
-    connectInfo.url = args;
+    connectInfo.url.Parse(args, "http");
     if (connectInfo.url.GetPort() == 0)
       connectInfo.url.SetPort(myPort);
   }
 
-  PBoolean persist;
+  bool persist;
   
   // make sure the form info is reset for each new operation
   connectInfo.ResetMultipartFormInfo();
 
+  PTRACE(5, "HTTPServer\tTransaction " << connectInfo.commandCode << ' ' << connectInfo.GetURL());
+
   // If the incoming URL is of a proxy type then call OnProxy() which will
   // probably just go OnError(). Even if a full URL is provided in the
   // command we should check to see if it is a local server request and process
@@ -299,51 +301,63 @@ PBoolean PHTTPServer::ProcessCommand()
     persist = OnProxy(connectInfo);
   else {
     connectInfo.entityBody = ReadEntityBody();
-
-    // Handle the local request
-    PStringToString postData;
-    switch (cmd) {
-      case GET :
-        persist = OnGET(url, connectInfo.GetMIME(), connectInfo);
-        break;
-
-      case HEAD :
-        persist = OnHEAD(url, connectInfo.GetMIME(), connectInfo);
-        break;
-
-      case POST :
-        if (!connectInfo.DecodeMultipartFormInfo())
-          PURL::SplitQueryVars(connectInfo.entityBody, postData); // x-www-form-urlencoded
-        persist = OnPOST(url, connectInfo.GetMIME(), postData, connectInfo);
-        break;
-
-      case P_MAX_INDEX:
-      default:
-        persist = OnUnknown(args, connectInfo);
-    }
+    persist = OnCommand(cmd, url, args, connectInfo);
   }
 
   flush();
 
   // if the function just indicated that the connection is to persist,
-  // and so did the client, then return PTrue. Note that all of the OnXXXX
-  // routines above must make sure that their return value is PFalse if
+  // and so did the client, then return true. Note that all of the OnXXXX
+  // routines above must make sure that their return value is false if
   // if there was no ContentLength field in the response. This ensures that
   // we always close the socket so the client will get the correct end of file
   if (persist && connectInfo.IsPersistent()) {
     unsigned max = connectInfo.GetPersistenceMaximumTransations();
     if (max == 0 || transactionCount < max)
-      return PTrue;
+      return true;
   }
 
   PTRACE(5, "HTTPServer\tConnection end: " << connectInfo.IsPersistent());
 
-  // close the output stream now and return PFalse
+  // close the output stream now and return false
   Shutdown(ShutdownWrite);
-  return PFalse;
+  return false;
 }
 
 
+bool PHTTPServer::OnCommand(PINDEX cmd, const PURL & url, const PString & args, PHTTPConnectionInfo & connectInfo)
+{
+  bool persist = false;
+
+  // Handle the local request
+  switch (cmd) {
+    case GET :
+      persist = OnGET(url, connectInfo.GetMIME(), connectInfo);
+      break;
+
+    case HEAD :
+      persist = OnHEAD(url, connectInfo.GetMIME(), connectInfo);
+      break;
+
+    case POST :
+    {
+      PStringToString postData;
+      if (!connectInfo.DecodeMultipartFormInfo())
+        PURL::SplitQueryVars(connectInfo.entityBody, postData); // x-www-form-urlencoded
+      persist = OnPOST(url, connectInfo.GetMIME(), postData, connectInfo);
+      break;
+    }
+
+    case P_MAX_INDEX:
+    default:
+      persist = OnUnknown(args, connectInfo);
+  }
+
+  return persist;
+}
+
+
+
 PString PHTTPServer::ReadEntityBody()
 {
   if (connectInfo.GetMajorVersion() < 1)
@@ -354,16 +368,12 @@ PString PHTTPServer::ReadEntityBody()
   // a content length of > 0 means read explicit length
   // a content length of < 0 means read until EOF
   // a content length of 0 means read nothing
-  int count = 0;
-  if (contentLength > 0) {
+  if (contentLength > 0)
     entityBody = ReadString((PINDEX)contentLength);
-  } else if (contentLength == -2) {
-    ReadLine(entityBody, PFalse);
-  } else if (contentLength < 0) {
-    while (Read(entityBody.GetPointer(count+1000)+count, 1000))
-      count += GetLastReadCount();
-    entityBody.SetSize(count+1);
-  }
+  else if (contentLength == -2)
+    ReadLine(entityBody, false);
+  else if (contentLength < 0)
+    entityBody = ReadString(P_MAX_INDEX);
 
   // close the connection, if not persistent
   if (!connectInfo.IsPersistent()) {
@@ -508,7 +518,7 @@ PBoolean PHTTPServer::StartResponse(StatusCode code,
                                 long bodySize)
 {
   if (connectInfo.majorVersion < 1) 
-    return PFalse;
+    return false;
 
   httpStatusCodeStruct dummyInfo;
   const httpStatusCodeStruct * statusInfo;
@@ -517,7 +527,7 @@ PBoolean PHTTPServer::StartResponse(StatusCode code,
   else {
     dummyInfo.text = "";
     dummyInfo.code = code;
-    dummyInfo.allowedBody = PTrue;
+    dummyInfo.allowedBody = true;
     dummyInfo.majorVersion = connectInfo.majorVersion;
     dummyInfo.minorVersion = connectInfo.minorVersion;
     statusInfo = &dummyInfo;
@@ -527,7 +537,7 @@ PBoolean PHTTPServer::StartResponse(StatusCode code,
   *this << "HTTP/" << connectInfo.majorVersion << '.' << connectInfo.minorVersion
         << ' ' << statusInfo->code << ' ' << statusInfo->text << "\r\n";
 
-  PBoolean chunked = PFalse;
+  PBoolean chunked = false;
 
   // If do not have user set content length, decide if we should add one
   if (!headers.Contains(ContentLengthTag())) {
@@ -539,10 +549,10 @@ PBoolean PHTTPServer::StartResponse(StatusCode code,
     }
     else {
       // v1.1 or later, see if will use chunked output
-      chunked = bodySize == P_MAX_INDEX;
+      chunked = (PINDEX)bodySize == P_MAX_INDEX;
       if (chunked)
         headers.SetAt(TransferEncodingTag(), ChunkedTag());
-      else if (bodySize >= 0 && bodySize < P_MAX_INDEX)
+      else if ((bodySize >= 0) && ((PINDEX)bodySize < P_MAX_INDEX))
         headers.SetAt(ContentLengthTag(), bodySize);
     }
   }
@@ -564,22 +574,23 @@ PBoolean PHTTPServer::StartResponse(StatusCode code,
 void PHTTPServer::SetDefaultMIMEInfo(PMIMEInfo & info,
                      const PHTTPConnectionInfo & connectInfo)
 {
-  PTime now;
-  if (!info.Contains(DateTag()))
-    info.SetAt(DateTag(), now.AsString(PTime::RFC1123, PTime::GMT));
-  if (!info.Contains(MIMEVersionTag()))
-    info.SetAt(MIMEVersionTag(), "1.0");
-  if (!info.Contains(ServerTag()))
-    info.SetAt(ServerTag(), GetServerName());
+  if (!info.Contains(DateTag))
+    info.SetAt(DateTag, PTime().AsString(PTime::RFC1123, PTime::GMT));
+
+  if (!info.Contains(MIMEVersionTag))
+    info.SetAt(MIMEVersionTag, "1.0");
+
+  if (!info.Contains(ServerTag))
+    info.SetAt(ServerTag, GetServerName());
 
   if (connectInfo.IsPersistent()) {
     if (connectInfo.IsProxyConnection()) {
       PTRACE(5, "HTTPServer\tSetting proxy persistent response");
-      info.SetAt(ProxyConnectionTag(), KeepAliveTag());
+      info.SetAt(ProxyConnectionTag, KeepAliveTag());
     }
     else {
       PTRACE(5, "HTTPServer\tSetting direct persistent response");
-      info.SetAt(ConnectionTag(), KeepAliveTag());
+      info.SetAt(ConnectionTag, KeepAliveTag());
     }
   }
 }
@@ -664,7 +675,7 @@ void PHTTPAuthority::DecodeBasicAuthority(const PString & authInfo,
 
 PBoolean PHTTPAuthority::IsActive() const
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -811,48 +822,43 @@ PBoolean PHTTPConnectionInfo::Initialise(PHTTPServer & server, PString & args)
 {
   // if only one argument, then it must be a version 0.9 simple request
   PINDEX lastSpacePos = args.FindLast(' ');
-  static const PCaselessString httpId = "HTTP/";
-  if (lastSpacePos == P_MAX_INDEX || httpId != args(lastSpacePos+1, lastSpacePos+5)) {
+  if (lastSpacePos == P_MAX_INDEX || strncasecmp(&args[lastSpacePos+1], "HTTP/", 5) != 0) {
     majorVersion = 0;
     minorVersion = 9;
-    return PTrue;
+    return true;
   }
 
   // otherwise, attempt to extract a version number
-  PCaselessString verStr = args.Mid(lastSpacePos + 6);
-  PINDEX dotPos = verStr.Find('.');
-  if (dotPos == 0 || dotPos >= verStr.GetLength()) {
-    server.OnError(PHTTP::BadRequest, "Malformed version number: " + verStr, *this);
-    return PFalse;
+  PINDEX dotPos = args.Find('.', lastSpacePos+6);
+  if (dotPos == 0 || dotPos == P_MAX_INDEX) {
+    server.OnError(PHTTP::BadRequest, "Malformed version number: " + args, *this);
+    return false;
   }
 
   // should actually check if the text contains only digits, but the
   // chances of matching everything else and it not being a valid number
   // are pretty small, so don't bother
-  majorVersion = (int)verStr.Left(dotPos).AsInteger();
-  minorVersion = (int)verStr.Mid(dotPos+1).AsInteger();
+  majorVersion = atoi(&args[lastSpacePos+6]);
+  minorVersion = atoi(&args[dotPos+1]);
   args.Delete(lastSpacePos, P_MAX_INDEX);
 
   // build our connection info reading MIME info until an empty line is
   // received, or EOF
   if (!mimeInfo.Read(server))
-    return PFalse;
+    return false;
 
   wasPersistent = isPersistent;
   isPersistent = false;
 
-  PString str;
-
   // check for Proxy-Connection and Connection strings
-  isProxyConnection = mimeInfo.Contains(PHTTP::ProxyConnectionTag());
-  if (isProxyConnection)
-    str = mimeInfo[PHTTP::ProxyConnectionTag()];
-  else if (mimeInfo.Contains(PHTTP::ConnectionTag()))
-    str = mimeInfo[PHTTP::ConnectionTag()];
+  PString str = mimeInfo(PHTTP::ProxyConnectionTag());
+  isProxyConnection = !str.IsEmpty();
+  if (!isProxyConnection)
+    str = mimeInfo(PHTTP::ConnectionTag());
 
   // get any connection options
   if (!str) {
-    PStringArray tokens = str.Tokenise(", \r\n", PFalse);
+    PStringArray tokens = str.Tokenise(", \r\n", false);
     for (PINDEX z = 0; !isPersistent && z < tokens.GetSize(); z++)
       isPersistent = isPersistent || (tokens[z] *= PHTTP::KeepAliveTag());
   }
@@ -870,18 +876,17 @@ PBoolean PHTTPConnectionInfo::Initialise(PHTTPServer & server, PString & args)
   // If the client didn't specify a persistent connection, then use the
   // ContentLength if there is one or read until end of file if there isn't
   if (!isPersistent)
-    entityBodyLength = mimeInfo.GetInteger(PHTTP::ContentLengthTag(),
-                                           (commandCode == PHTTP::POST) ? -2 : 0);
+    entityBodyLength = mimeInfo.GetInteger(PHTTP::ContentLengthTag, (commandCode == PHTTP::POST) ? -2 : 0);
   else {
-    entityBodyLength = mimeInfo.GetInteger(PHTTP::ContentLengthTag(), -1);
+    entityBodyLength = mimeInfo.GetInteger(PHTTP::ContentLengthTag, -1);
     if (entityBodyLength < 0) {
       PTRACE(5, "HTTPServer\tPersistent connection has no content length");
       entityBodyLength = 0;
-      mimeInfo.SetAt(PHTTP::ContentLengthTag(), "0");
+      mimeInfo.SetAt(PHTTP::ContentLengthTag, "0");
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -895,7 +900,7 @@ void PHTTPConnectionInfo::SetMIME(const PString & tag, const PString & value)
 PBoolean PHTTPConnectionInfo::IsCompatible(int major, int minor) const
 {
   if (minor == 0 && major == 0)
-    return PTrue;
+    return true;
   else
     return (majorVersion > major) ||
            ((majorVersion == major) && (minorVersion >= minor));
@@ -950,7 +955,7 @@ PBoolean PHTTPResource::OnGET(PHTTPServer & server,
                       const PMIMEInfo & info,
             const PHTTPConnectionInfo & connectInfo)
 {
-  return OnGETOrHEAD(server, url, info, connectInfo, PTrue);
+  return OnGETOrHEAD(server, url, info, connectInfo, true);
 }
 
 
@@ -959,7 +964,7 @@ PBoolean PHTTPResource::OnHEAD(PHTTPServer & server,
                       const PMIMEInfo & info,
             const PHTTPConnectionInfo & connectInfo)
 {
-  return OnGETOrHEAD(server, url, info, connectInfo, PFalse);
+  return OnGETOrHEAD(server, url, info, connectInfo, false);
 }
 
 
@@ -980,15 +985,14 @@ PBoolean PHTTPResource::OnGETOrHEAD(PHTTPServer & server,
                                          connectInfo.GetMultipartFormInfo(),
                                          server);
 
-  PBoolean retVal = PTrue;
+  PBoolean retVal = true;
   if (CheckAuthority(server, *request, connectInfo)) {
-    retVal = PFalse;
+    retVal = false;
     server.SetDefaultMIMEInfo(request->outMIME, connectInfo);
 
-    PTime expiryDate;
+    PTime expiryDate(0);
     if (GetExpirationDate(expiryDate))
-      request->outMIME.SetAt(PHTTP::ExpiresTag(),
-                              expiryDate.AsString(PTime::RFC1123, PTime::GMT));
+      request->outMIME.Set(PHTTP::ExpiresTag, expiryDate.AsString(PTime::RFC1123, PTime::GMT));
 
     if (!LoadHeaders(*request)) 
       retVal = server.OnError(request->code, url.AsString(), connectInfo);
@@ -1029,7 +1033,7 @@ PBoolean PHTTPResource::OnPOST(PHTTPServer & server,
 
   request->entityBody = connectInfo.GetEntityBody();
 
-  PBoolean persist = PTrue;
+  PBoolean persist = true;
   if (CheckAuthority(server, *request, connectInfo)) {
     server.SetDefaultMIMEInfo(request->outMIME, connectInfo);
     persist = OnPOSTData(*request, data);
@@ -1072,7 +1076,7 @@ PBoolean PHTTPResource::CheckAuthority(PHTTPServer & server,
                      const PHTTPConnectionInfo & connectInfo)
 {
   if (authority == NULL)
-    return PTrue;
+    return true;
 
   return CheckAuthority(*authority, server, request, connectInfo);
 }
@@ -1084,13 +1088,13 @@ PBoolean PHTTPResource::CheckAuthority(PHTTPAuthority & authority,
                         const PHTTPConnectionInfo & connectInfo)
 {
   if (!authority.IsActive())
-    return PTrue;
+    return true;
 
 
   // if this is an authorisation request...
   if (request.inMIME.Contains(PHTTP::AuthorizationTag()) &&
       authority.Validate(request, request.inMIME[PHTTP::AuthorizationTag()]))
-    return PTrue;
+    return true;
 
   // it must be a request for authorisation
   PMIMEInfo headers;
@@ -1122,7 +1126,7 @@ PBoolean PHTTPResource::CheckAuthority(PHTTPAuthority & authority,
   server.StartResponse(PHTTP::UnAuthorised, headers, reply.GetLength());
   server.WriteString(reply);
 
-  return PFalse;
+  return false;
 }
 
 
@@ -1142,13 +1146,13 @@ void PHTTPResource::ClearAuthority()
 
 PBoolean PHTTPResource::IsModifiedSince(const PTime &)
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PHTTPResource::GetExpirationDate(PTime &)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -1175,8 +1179,8 @@ static void WriteChunkedDataToServer(PHTTPServer & server, PCharArray & data)
 
 void PHTTPResource::SendData(PHTTPRequest & request)
 {
-  if (!request.outMIME.Contains(PHTTP::ContentTypeTag()) && !contentType)
-    request.outMIME.SetAt(PHTTP::ContentTypeTag(), contentType);
+  if (!request.outMIME.Contains(PHTTP::ContentTypeTag) && !contentType)
+    request.outMIME.SetAt(PHTTP::ContentTypeTag, contentType);
 
   PCharArray data;
   if (LoadData(request, data)) {
@@ -1199,7 +1203,7 @@ void PHTTPResource::SendData(PHTTPRequest & request)
   }
   else {
     request.server.StartResponse(request.code, request.outMIME, data.GetSize());
-    request.server.Write(data, data.GetSize());
+    request.server.write(data, data.GetSize());
   }
 }
 
@@ -1209,8 +1213,8 @@ PBoolean PHTTPResource::LoadData(PHTTPRequest & request, PCharArray & data)
   PString text = LoadText(request);
   OnLoadedText(request, text);
   if(data.SetSize(text.GetLength()))
-    memcpy(data.GetPointer(), text.GetPointer(), text.GetLength()); // Lose the trailing '\0'
-  return PFalse;
+    memcpy(data.GetPointer(), (const char *)text, text.GetLength()); // Lose the trailing '\0'
+  return false;
 }
 
 
@@ -1234,7 +1238,7 @@ PBoolean PHTTPResource::Post(PHTTPRequest & request,
   request.code = PHTTP::MethodNotAllowed;
   msg = "Error in POST";
   msg << "Post to this resource is not allowed" << PHTML::Body();
-  return PTrue;
+  return true;
 }
 
 
@@ -1288,7 +1292,7 @@ PHTTPString::PHTTPString(const PURL & url,
 PBoolean PHTTPString::LoadHeaders(PHTTPRequest & request)
 {
   request.contentSize = string.GetLength();
-  return PTrue;
+  return true;
 }
 
 
@@ -1378,11 +1382,11 @@ PBoolean PHTTPFile::LoadHeaders(PHTTPRequest & request)
 
   if (!file.Open(filePath, PFile::ReadOnly)) {
     request.code = PHTTP::NotFound;
-    return PFalse;
+    return false;
   }
 
   request.contentSize = file.GetLength();
-  return PTrue;
+  return true;
 }
 
 
@@ -1407,22 +1411,21 @@ PBoolean PHTTPFile::LoadData(PHTTPRequest & request, PCharArray & data)
     PAssert(file.Read(data.GetPointer(count), count), PLogicError);
 
   if (!file.IsEndOfFile())
-    return PTrue;
+    return true;
 
   file.Close();
-  return PFalse;
+  return false;
 }
 
 
 PString PHTTPFile::LoadText(PHTTPRequest & request)
 {
-  PFile & file = ((PHTTPFileRequest&)request).file;
-  PAssert(file.IsOpen(), PLogicError);
-  PINDEX count = file.GetLength();
   PString text;
-  if (count > 0)
-    PAssert(file.Read(text.GetPointer(count+1), count), PLogicError);
-  PAssert(file.Close(), PLogicError);
+  PFile & file = ((PHTTPFileRequest&)request).file;
+  if (PAssert(file.IsOpen(), PLogicError)) {
+    text = file.ReadString(file.GetLength());
+    PAssert(file.Close(), PLogicError);
+  }
   return text;
 }
 
@@ -1478,10 +1481,10 @@ PHTTPTailFile::PHTTPTailFile(const PURL & url,
 PBoolean PHTTPTailFile::LoadHeaders(PHTTPRequest & request)
 {
   if (!PHTTPFile::LoadHeaders(request))
-    return PFalse;
+    return false;
 
   request.contentSize = P_MAX_INDEX;
-  return PTrue;
+  return true;
 }
 
 
@@ -1494,7 +1497,7 @@ PBoolean PHTTPTailFile::LoadData(PHTTPRequest & request, PCharArray & data)
 
   while (file.GetPosition() >= file.GetLength()) {
     if (!request.server.Write(NULL, 0))
-      return PFalse;
+      return false;
     PThread::Sleep(200);
   }
 
@@ -1507,7 +1510,7 @@ PBoolean PHTTPTailFile::LoadData(PHTTPRequest & request, PCharArray & data)
 // PHTTPDirectory
 
 PHTTPDirectory::PHTTPDirectory(const PURL & url, const PDirectory & dir)
-  : PHTTPFile(url, 0), basePath(dir), allowDirectoryListing(PTrue)
+  : PHTTPFile(url, 0), basePath(dir), allowDirectoryListing(true)
 {
 }
 
@@ -1515,7 +1518,7 @@ PHTTPDirectory::PHTTPDirectory(const PURL & url, const PDirectory & dir)
 PHTTPDirectory::PHTTPDirectory(const PURL & url,
                                const PDirectory & dir,
                                const PHTTPAuthority & auth)
-  : PHTTPFile(url, PString(), auth), basePath(dir), allowDirectoryListing(PTrue)
+  : PHTTPFile(url, PString(), auth), basePath(dir), allowDirectoryListing(true)
 {
 }
 
@@ -1564,24 +1567,24 @@ PBoolean PHTTPDirectory::FindAuthorisations(const PDirectory & dir, PString & re
 {
   PFilePath fn = dir + accessFilename;
   PTextFile file;
-  PBoolean first = PTrue;
+  PBoolean first = true;
   if (file.Open(fn, PFile::ReadOnly)) {
     PString line;
     while (file.ReadLine(line)) {
       if (first) {
         realm = line.Trim();
-        first = PFalse;
+        first = false;
       } else {
         PStringArray tokens = line.Tokenise(':');
         if (tokens.GetSize() > 1)
           authorisations.SetAt(tokens[0].Trim(), tokens[1].Trim());
       }
     }
-    return PTrue;
+    return true;
   }
     
   if (dir.IsRoot() || (dir == basePath))
-    return PFalse;
+    return false;
 
   return FindAuthorisations(dir.GetParent(), realm, authorisations);
 }
@@ -1596,7 +1599,7 @@ PBoolean PHTTPDirectory::CheckAuthority(PHTTPServer & server,
   if (authorisationRealm.IsEmpty() ||
       !FindAuthorisations(((PHTTPDirRequest&)request).realPath.GetDirectory(), newRealm, authorisations) ||
       authorisations.GetSize() == 0)
-    return PTrue;
+    return true;
 
   PHTTPMultiSimpAuth authority(newRealm, authorisations);
   return PHTTPResource::CheckAuthority(authority, server, request, conInfo);
@@ -1610,7 +1613,7 @@ PBoolean PHTTPDirectory::LoadHeaders(PHTTPRequest & request)
   PFileInfo info;
   if (!PFile::GetInfo(realPath, info)) {
     request.code = PHTTP::NotFound;
-    return PFalse;
+    return false;
   }
 
   // if the resource is a file, and the file can't be opened, then return "not found"
@@ -1619,14 +1622,14 @@ PBoolean PHTTPDirectory::LoadHeaders(PHTTPRequest & request)
     if (!file.Open(realPath, PFile::ReadOnly) ||
         (!authorisationRealm.IsEmpty() && realPath.GetFileName() == accessFilename)) {
       request.code = PHTTP::NotFound;
-      return PFalse;
+      return false;
     }
   } 
 
   // resource is a directory - if index files disabled, then return "not found"
   else if (!allowDirectoryListing) {
     request.code = PHTTP::NotFound;
-    return PFalse;
+    return false;
   }
 
   // else look for index files
@@ -1645,7 +1648,7 @@ PBoolean PHTTPDirectory::LoadHeaders(PHTTPRequest & request)
                           PMIMEInfo::GetContentType(file.GetFilePath().GetType()));
     request.contentSize = file.GetLength();
     fakeIndex = PString();
-    return PTrue;
+    return true;
   }
 
   // construct a directory listing
@@ -1672,7 +1675,7 @@ PBoolean PHTTPDirectory::LoadHeaders(PHTTPRequest & request)
   reply << PHTML::Body();
   fakeIndex = reply;
 
-  return PTrue;
+  return true;
 }
 
 
