@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24410 $
+ * $Revision: 29742 $
  * $Author: rjongbloed $
- * $Date: 2010-05-27 18:43:58 -0500 (Thu, 27 May 2010) $
+ * $Date: 2013-05-19 22:27:45 -0500 (Sun, 19 May 2013) $
  */
 
 #ifdef __GNUC__
@@ -76,7 +76,7 @@ PHTTPServiceProcess::PHTTPServiceProcess(const Info & inf)
     copyrightHomePage(inf.copyrightHomePage != NULL ? inf.copyrightHomePage : (const char *)manufacturersHomePage),
     copyrightEmail(inf.copyrightEmail != NULL ? inf.copyrightEmail : (const char *)manufacturersEmail)
 {
-  ignoreSignatures = PFalse;
+  ignoreSignatures = false;
 
   if (inf.gifFilename != NULL) {
     PDirectory exeDir = GetFile().GetDirectory();
@@ -87,7 +87,7 @@ PHTTPServiceProcess::PHTTPServiceProcess(const Info & inf)
 #endif
     httpNameSpace.AddResource(new PServiceHTTPFile(inf.gifFilename, exeDir+inf.gifFilename));
     if (gifHTML.IsEmpty()) {
-      gifHTML = psprintf("<img src=\"/%s\" alt=\"%s!\"", inf.gifFilename, inf.productName);
+      gifHTML = psprintf("<img border=0 src=\"%s\" alt=\"%s!\"", inf.gifFilename, inf.productName);
       if (inf.gifWidth != 0 && inf.gifHeight != 0)
         gifHTML += psprintf(" width=%i height=%i", inf.gifWidth, inf.gifHeight);
       gifHTML += " align=absmiddle>";
@@ -95,7 +95,6 @@ PHTTPServiceProcess::PHTTPServiceProcess(const Info & inf)
   }
 
   restartThread = NULL;
-  httpListeningSocket = NULL;
   httpThreads.DisallowDeleteObjects();
 }
 
@@ -117,17 +116,16 @@ PHTTPServiceProcess & PHTTPServiceProcess::Current()
 PBoolean PHTTPServiceProcess::OnStart()
 {
   if (!Initialise("Started"))
-    return PFalse;
+    return false;
 
   OnConfigChanged();
-  return PTrue;
+  return true;
 }
 
 
 void PHTTPServiceProcess::OnStop()
 {
   ShutdownListener();
-  PSYSTEMLOG(Warning, GetName() << " stopped.");
   PServiceProcess::OnStop();
 }
 
@@ -135,7 +133,7 @@ void PHTTPServiceProcess::OnStop()
 PBoolean PHTTPServiceProcess::OnPause()
 {
   OnConfigChanged();
-  return PTrue;
+  return true;
 }
 
 
@@ -157,67 +155,117 @@ const char * PHTTPServiceProcess::GetServiceDependencies() const
 #endif
 
 
-PBoolean PHTTPServiceProcess::ListenForHTTP(WORD port,
+bool PHTTPServiceProcess::ListenForHTTP(WORD port,
                                         PSocket::Reusability reuse,
                                         PINDEX stackSize)
 {
-  if (httpListeningSocket != NULL &&
-      httpListeningSocket->GetPort() == port &&
-      httpListeningSocket->IsOpen())
-    return PTrue;
-
-  return ListenForHTTP(new PTCPSocket(port), reuse, stackSize);
+  return ListenForHTTP(PString::Empty(), port, reuse, stackSize);
 }
 
 
-PBoolean PHTTPServiceProcess::ListenForHTTP(PSocket * listener,
+bool PHTTPServiceProcess::ListenForHTTP(const PString & interfaces,
+                                        WORD port,
                                         PSocket::Reusability reuse,
                                         PINDEX stackSize)
 {
-  if (httpListeningSocket != NULL)
+  if (port == 0) {
+    PAssertAlways(PInvalidParameter);
+    return false;
+  }
+
+  if (!m_httpListeningSockets.IsEmpty()) {
+    if (m_httpListeningSockets.front().GetPort() == port)
+      return true;
+
+    ShutdownListener();
+  }
+
+  PStringArray ifaces = interfaces.Tokenise(',');
+  if (ifaces.IsEmpty()) {
+    ifaces.AppendString("0.0.0.0");
+#if P_HAS_IPV6
+    ifaces.AppendString("[::]");
+#endif
+  }
+
+  bool atLeastOne = false;
+  for (PINDEX i = 0; i < ifaces.GetSize(); ++i) {
+    PIPSocket::Address binding(ifaces[i]);
+    if (binding.IsValid()) {
+      PTCPSocket * listener = new PTCPSocket(port);
+      if (listener->Listen(binding, 5, 0, reuse)) {
+        PSYSTEMLOG(Debug, "HTTPSVC\tListening for HTTP on " << listener->GetLocalAddress());
+        m_httpListeningSockets.Append(listener);
+        atLeastOne = true;
+      }
+      else {
+        PSYSTEMLOG(Debug, "HTTPSVC\tListen on port " << binding << ':' << listener->GetPort() << " failed: " << listener->GetErrorText());
+        delete listener;
+      }
+    }
+    else {
+      PSYSTEMLOG(Debug, "HTTPSVC\tInvalid interface address \"" << ifaces[i] << '"');
+    }
+  }
+
+  if (atLeastOne && stackSize > 1000)
+    new PHTTPServiceThread(stackSize, *this);
+
+  return atLeastOne;
+}
+
+
+bool PHTTPServiceProcess::ListenForHTTP(PSocket * listener,
+                                        PSocket::Reusability reuse,
+                                        PINDEX stackSize)
+{
+  if (PAssertNULL(listener) == NULL)
+    return false;
+
+  if (!m_httpListeningSockets.IsEmpty())
     ShutdownListener();
 
-  httpListeningSocket = PAssertNULL(listener);
-  if (!httpListeningSocket->Listen(5, 0, reuse)) {
-    PSYSTEMLOG(Debug, "HTTPSVC\tListen on port " << httpListeningSocket->GetPort()
-                   << " failed: " << httpListeningSocket->GetErrorText());
-    return PFalse;
+  if (!listener->Listen(5, 0, reuse)) {
+    PSYSTEMLOG(Debug, "HTTPSVC\tListen on port " << listener->GetPort()
+                    << " failed: " << listener->GetErrorText());
+    return false;
   }
 
+  PSYSTEMLOG(Debug, "HTTPSVC\tListening for HTTP on " << *listener);
+  m_httpListeningSockets.Append(listener);
+
   if (stackSize > 1000)
     new PHTTPServiceThread(stackSize, *this);
 
-  return PTrue;
+  return true;
 }
 
 
 void PHTTPServiceProcess::ShutdownListener()
 {
-  if (httpListeningSocket == NULL)
-    return;
-
-  if (!httpListeningSocket->IsOpen())
+  if (m_httpListeningSockets.IsEmpty())
     return;
 
   PSYSTEMLOG(Debug, "HTTPSVC\tClosing listener socket on port "
-                 << httpListeningSocket->GetPort());
+                 << m_httpListeningSockets.front().GetPort());
 
-  httpListeningSocket->Close();
+  for (PSocketList::iterator it = m_httpListeningSockets.begin(); it != m_httpListeningSockets.end(); ++it)
+    it->Close();
 
   httpThreadsMutex.Wait();
-  for (ThreadList::iterator i = httpThreads.begin(); i != httpThreads.end(); i++)
-    i->Close();
+  for (ThreadList::iterator it = httpThreads.begin(); it != httpThreads.end(); it++)
+    it->Close();
 
   while (httpThreads.GetSize() > 0) {
     httpThreadsMutex.Signal();
-    Sleep(1);
+    SignalTimerChange();
+    Sleep(10);
     httpThreadsMutex.Wait();
   }
 
   httpThreadsMutex.Signal();
 
-  delete httpListeningSocket;
-  httpListeningSocket = NULL;
+  m_httpListeningSockets.RemoveAll();
 }
 
 
@@ -239,10 +287,6 @@ PString PHTTPServiceProcess::GetCopyrightText()
 
 PString PHTTPServiceProcess::GetPageGraphic()
 {
-  PFile header;
-  if (header.Open("header.html", PFile::ReadOnly))
-    return header.ReadString(header.GetLength());
-
   PHTML html(PHTML::InBody);
   html << PHTML::TableStart()
        << PHTML::TableRow()
@@ -257,7 +301,7 @@ PString PHTTPServiceProcess::GetPageGraphic()
 
        << PHTML::TableData()
        << GetOSClass() << ' ' << GetOSName()
-       << " Version " << GetVersion(PTrue) << PHTML::BreakLine()
+       << " Version " << GetVersion(true) << PHTML::BreakLine()
        << ' ' << GetCompilationDate().AsString("d MMMM yyyy")
        << PHTML::BreakLine()
        << "By "
@@ -287,38 +331,42 @@ void PHTTPServiceProcess::GetPageHeader(PHTML & html, const PString & title)
 
 PTCPSocket * PHTTPServiceProcess::AcceptHTTP()
 {
-  if (httpListeningSocket == NULL)
+  if (m_httpListeningSockets.IsEmpty())
     return NULL;
 
-  if (!httpListeningSocket->IsOpen())
-    return NULL;
+  PSocket::SelectList listeners;
+  for (PSocketList::iterator it = m_httpListeningSockets.begin(); it != m_httpListeningSockets.end(); ++it)
+    listeners += *it;
 
-  // get a socket when a client connects
-  PTCPSocket * socket = new PTCPSocket;
-  if (socket->Accept(*httpListeningSocket))
-    return socket;
+  PChannel::Errors error = PSocket::Select(listeners);
+  if (error == PChannel::NoError) {
+    // get a socket when a client connects
+    PTCPSocket * socket = new PTCPSocket;
+    if (socket->Accept(listeners.front()))
+      return socket;
 
-  if (socket->GetErrorCode() != PChannel::Interrupted) {
-    PSYSTEMLOG(Error, "Accept failed for HTTP: " << socket->GetErrorText());
+    if (socket->GetErrorCode() != PChannel::Interrupted) {
+      PSYSTEMLOG(Error, "Accept failed for HTTP: " << socket->GetErrorText());
+    }
+    delete socket;
+  }
+  else if (error != PChannel::Interrupted) {
+    PSYSTEMLOG(Error, "Select failed for HTTP: " << PSocket::GetErrorText(error));
   }
 
-  if (httpListeningSocket != NULL && httpListeningSocket->IsOpen())
-    return socket;
-
-  delete socket;
   return NULL;
 }
 
 
 PBoolean PHTTPServiceProcess::ProcessHTTP(PTCPSocket & socket)
 {
-  if (!socket.IsOpen())
-    return PTrue;
+  if (!socket.IsOpen() || m_httpListeningSockets.IsEmpty() || !m_httpListeningSockets.front().IsOpen())
+    return true;
 
   PHTTPServer * server = CreateHTTPServer(socket);
   if (server == NULL) {
     PSYSTEMLOG(Error, "HTTP server creation/open failed.");
-    return PTrue;
+    return true;
   }
 
   // process requests
@@ -329,10 +377,10 @@ PBoolean PHTTPServiceProcess::ProcessHTTP(PTCPSocket & socket)
   delete server;
 
   // if a restart was requested, then do it, but only if we are not shutting down
-  if (httpListeningSocket->IsOpen())
+  if (!m_httpListeningSockets.IsEmpty() && m_httpListeningSockets.front().IsOpen())
     CompleteRestartSystem();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -377,7 +425,7 @@ void PHTTPServiceProcess::AddUnregisteredText(PHTML &)
 
 PBoolean PHTTPServiceProcess::SubstituteEquivalSequence(PHTTPRequest &, const PString &, PString &)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -450,10 +498,10 @@ void PHTTPServiceThread::Main()
 //////////////////////////////////////////////////////////////
 
 PConfigPage::PConfigPage(PHTTPServiceProcess & app,
-                         const PString & title,
+                         const PString & url,
                          const PString & section,
                          const PHTTPAuthority & auth)
-  : PHTTPConfig(title, section, auth),
+  : PHTTPConfig(url, section, auth),
     process(app)
 {
 }
@@ -485,7 +533,7 @@ PBoolean PConfigPage::OnPOST(PHTTPServer & server,
                          const PHTTPConnectionInfo & connectInfo)
 {
   PHTTPConfig::OnPOST(server, url, info, data, connectInfo);
-  return PFalse;    // Make sure we break any persistent connections
+  return false;    // Make sure we break any persistent connections
 }
 
 
@@ -512,7 +560,7 @@ PBoolean PConfigPage::GetExpirationDate(PTime & when)
 {
   // Well and truly before now....
   when = ImmediateExpiryTime;
-  return PTrue;
+  return true;
 }
 
 
@@ -550,7 +598,7 @@ PBoolean PConfigSectionsPage::OnPOST(PHTTPServer & server,
                                  const PHTTPConnectionInfo & connectInfo)
 {
   PHTTPConfigSectionList::OnPOST(server, url, info, data, connectInfo);
-  return PFalse;    // Make sure we break any persistent connections
+  return false;    // Make sure we break any persistent connections
 }
 
 
@@ -569,7 +617,7 @@ PBoolean PConfigSectionsPage::GetExpirationDate(PTime & when)
 {
   // Well and truly before now....
   when = ImmediateExpiryTime;
-  return PTrue;
+  return true;
 }
 
 
@@ -594,7 +642,7 @@ PString PRegisterPage::LoadText(PHTTPRequest & request)
   if (process.GetHomePage() == HOME_PAGE) {
     orderURL = "https://home.equival.com.au/purchase.html";
     tempURL = "http://www.equival.com/" + process.GetName().ToLower() + "/register.html";
-    tempURL.Replace(" ", "", PTrue);
+    tempURL.Replace(" ", "", true);
   }
 
   PServiceHTML regPage(process.GetName() & "Registration", NULL);
@@ -707,7 +755,7 @@ static PBoolean FindSpliceBlock(const PRegularExpression & regex,
                             PINDEX & finish)
 {
   if (!text.FindRegEx(regex, pos, len, 0))
-    return PFalse;
+    return false;
 
   PINDEX endpos, endlen;
   static PRegularExpression EndBlock("<?!--#registration[ \t\n]*end[ \t\n]*[a-z]*[ \t\n]*-->?",
@@ -718,7 +766,7 @@ static PBoolean FindSpliceBlock(const PRegularExpression & regex,
     len = endpos - pos + endlen;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -841,7 +889,7 @@ PBoolean PRegisterPage::Post(PHTTPRequest & request,
 
   PBoolean retval = PHTTPConfig::Post(request, data, reply);
   if (request.code != PHTTP::RequestOK)
-    return PFalse;
+    return false;
 
   PSecureConfig sconf(process.GetProductKey(), process.GetSecuredKeys());
   switch (sconf.GetValidation()) {
@@ -1001,7 +1049,7 @@ PBoolean PServiceHTML::CheckSignature()
 PBoolean PServiceHTML::CheckSignature(const PString & html)
 {
   if (PHTTPServiceProcess::Current().ShouldIgnoreSignatures())
-    return PTrue;
+    return true;
 
   // extract the signature from the file
   PString out;
@@ -1010,7 +1058,7 @@ PBoolean PServiceHTML::CheckSignature(const PString & html)
   // calculate the signature on the data
   PString checkSignature = CalculateSignature(out);
 
-  // return PTrue or PFalse
+  // return true or false
   return checkSignature == signature;
 }
 
@@ -1019,7 +1067,7 @@ static PBoolean FindBrackets(const PString & args, PINDEX & open, PINDEX & close
 {
   open = args.FindOneOf("[{(", close);
   if (open == P_MAX_INDEX)
-    return PFalse;
+    return false;
 
   switch (args[open]) {
     case '[' :
@@ -1049,12 +1097,12 @@ static PBoolean ExtractVariables(const PString & args,
     close = P_MAX_INDEX-1;
   }
   if (variable.IsEmpty())
-    return PFalse;
+    return false;
 
   if (FindBrackets(args, open, close))
     value = args(open+1, close-1);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1143,7 +1191,7 @@ PCREATE_SERVICE_MACRO(Manufacturer,P_EMPTY,P_EMPTY)
 
 PCREATE_SERVICE_MACRO(Version,P_EMPTY,P_EMPTY)
 {
-  return PHTTPServiceProcess::Current().GetVersion(PTrue);
+  return PHTTPServiceProcess::Current().GetVersion(true);
 }
 
 
@@ -1309,7 +1357,7 @@ PCREATE_SERVICE_MACRO(MonitorInfo,request,P_EMPTY)
 
   PStringStream monitorText; 
   monitorText << "Program: "          << PHTTPServiceProcess::Current().GetProductName() << "\n"
-              << "Version: "          << PHTTPServiceProcess::Current().GetVersion(PTrue) << "\n"
+              << "Version: "          << PHTTPServiceProcess::Current().GetVersion(true) << "\n"
               << "Manufacturer: "     << PHTTPServiceProcess::Current().GetManufacturer() << "\n"
               << "OS: "               << PHTTPServiceProcess::Current().GetOSClass() << " " << PHTTPServiceProcess::Current().GetOSName() << "\n"
               << "OS Version: "       << PHTTPServiceProcess::Current().GetOSVersion() << "\n"
@@ -1408,9 +1456,9 @@ PCREATE_SERVICE_MACRO(InputsFromQuery,request,P_EMPTY)
 {
   PStringToString vars = request.url.GetQueryVars();
   PStringStream subs;
-  for (PINDEX i = 0; i < vars.GetSize(); i++)
-    subs << "<INPUT TYPE=hidden NAME=\"" << vars.GetKeyAt(i)
-         << "\" VALUE=\"" << vars.GetDataAt(i) << "\">\r\n";
+  for (PStringToString::iterator it = vars.begin(); it != vars.end(); ++it)
+    subs << "<INPUT TYPE=hidden NAME=\"" << it->first
+         << "\" VALUE=\"" << it->second << "\">\r\n";
   return subs;
 }
 
@@ -1613,7 +1661,7 @@ bool PServiceHTML::ProcessMacros(PHTTPRequest & request,
            << '"'
            << PHTML::Body();
       text = html;
-      return PFalse;
+      return false;
     }
   }
 
@@ -1630,7 +1678,7 @@ bool PServiceHTML::ProcessMacros(PHTTPRequest & request,
 
   PBoolean substitedMacro;
   do {
-    substitedMacro = PFalse;
+    substitedMacro = false;
 
     PINDEX pos = 0;
     PINDEX len;
@@ -1648,10 +1696,10 @@ bool PServiceHTML::ProcessMacros(PHTTPRequest & request,
       if (text.FindRegEx(EndBlockRegEx, endpos, endlen, pos+len)) {
         PINDEX startpos = pos+len;
         len = endpos-pos + endlen;
-        PINDEX idx = ServiceMacros.GetValuesIndex(PServiceMacro(cmd, PTrue));
+        PINDEX idx = ServiceMacros.GetValuesIndex(PServiceMacro(cmd, true));
         if (idx != P_MAX_INDEX) {
           substitution = ServiceMacros[idx].Translate(request, args, text(startpos, endpos-1));
-          substitedMacro = PTrue;
+          substitedMacro = true;
         }
       }
 
@@ -1666,10 +1714,10 @@ bool PServiceHTML::ProcessMacros(PHTTPRequest & request,
 
       PString substitution;
       if (!process.SubstituteEquivalSequence(request, cmd & args, substitution)) {
-        PINDEX idx = ServiceMacros.GetValuesIndex(PServiceMacro(cmd, PFalse));
+        PINDEX idx = ServiceMacros.GetValuesIndex(PServiceMacro(cmd, false));
         if (idx != P_MAX_INDEX) {
           substitution = ServiceMacros[idx].Translate(request, args, PString::Empty());
-          substitedMacro = PTrue;
+          substitedMacro = true;
         }
       }
 
@@ -1677,7 +1725,7 @@ bool PServiceHTML::ProcessMacros(PHTTPRequest & request,
     }
   } while (substitedMacro);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1708,15 +1756,15 @@ static void ServiceOnLoadedText(PString & text)
 
   PString manuf = "<!--Standard_" + process.GetManufacturer() + "_Header-->";
   if (text.Find(manuf) != P_MAX_INDEX)
-    text.Replace(manuf, process.GetPageGraphic(), PTrue);
+    text.Replace(manuf, process.GetPageGraphic(), true);
 
   static const char equiv[] = "<!--Standard_Equivalence_Header-->";
   if (text.Find(equiv) != P_MAX_INDEX)
-    text.Replace(equiv, process.GetPageGraphic(), PTrue);
+    text.Replace(equiv, process.GetPageGraphic(), true);
 
   static const char copy[] = "<!--Standard_Copyright_Header-->";
   if (text.Find(copy) != P_MAX_INDEX)
-    text.Replace(copy, process.GetCopyrightText(), PTrue);
+    text.Replace(copy, process.GetCopyrightText(), true);
 }
 
 
@@ -1733,7 +1781,7 @@ PBoolean PServiceHTTPString::GetExpirationDate(PTime & when)
 {
   // Well and truly before now....
   when = ImmediateExpiryTime;
-  return PTrue;
+  return true;
 }
 
 
@@ -1748,7 +1796,7 @@ PBoolean PServiceHTTPFile::GetExpirationDate(PTime & when)
 {
   // Well and truly before now....
   when = ImmediateExpiryTime;
-  return PTrue;
+  return true;
 }
 
 
@@ -1764,7 +1812,7 @@ PBoolean PServiceHTTPDirectory::GetExpirationDate(PTime & when)
 {
   // Well and truly before now....
   when = ImmediateExpiryTime;
-  return PTrue;
+  return true;
 }
 
 #endif // P_HTTPSVC
