@@ -23,9 +23,9 @@
  *
  * Contributor(s): Federico Pinna and Reitek S.p.A. (SASL authentication)
  *
- * $Revision: 24410 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2010-05-27 18:43:58 -0500 (Thu, 27 May 2010) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #ifdef __GNUC__
@@ -35,12 +35,12 @@
 #include <ptlib.h>
 #include <ptlib/sockets.h>
 #include <ptclib/inetmail.h>
-#if P_SASL2
+#if P_SASL
 #include <ptclib/psasl.h>
 #endif
 
-static const PString CRLF = "\r\n";
-static const PString CRLFdotCRLF = "\r\n.\r\n";
+static const PConstString CRLF("\r\n");
+static const PConstString CRLFdotCRLF("\r\n.\r\n");
 
 
 #define new PNEW
@@ -68,9 +68,9 @@ PSMTP::PSMTP()
 
 PSMTPClient::PSMTPClient()
 {
-  haveHello = PFalse;
-  extendedHello = PFalse;
-  eightBitMIME = PFalse;
+  haveHello = false;
+  extendedHello = false;
+  eightBitMIME = false;
 }
 
 
@@ -88,7 +88,7 @@ PBoolean PSMTPClient::OnOpen()
 
 PBoolean PSMTPClient::Close()
 {
-  PBoolean ok = PTrue;
+  PBoolean ok = true;
 
   if (sendingData)
     ok = EndMessage();
@@ -101,7 +101,7 @@ PBoolean PSMTPClient::Close()
 }
 
 
-#if P_SASL2
+#if P_SASL
 PBoolean PSMTPClient::LogIn(const PString & username,
                         const PString & password)
 {
@@ -113,12 +113,12 @@ PBoolean PSMTPClient::LogIn(const PString & username,
   }
 
   if (haveHello)
-    return PFalse; // Wrong state
+    return false; // Wrong state
 
   if (ExecuteCommand(EHLO, localHost)/100 != 2)
-    return PTrue; // EHLO not supported, therefore AUTH not supported
+    return true; // EHLO not supported, therefore AUTH not supported
 
-  haveHello = extendedHello = PTrue;
+  haveHello = extendedHello = true;
 
   PStringArray caps = lastResponseInfo.Lines();
   PStringArray serverMechs;
@@ -126,18 +126,18 @@ PBoolean PSMTPClient::LogIn(const PString & username,
 
   for (i = 0, max = caps.GetSize() ; i < max ; i++)
     if (caps[i].Left(5) == "AUTH ") {
-      serverMechs = caps[i].Mid(5).Tokenise(" ", PFalse);
+      serverMechs = caps[i].Mid(5).Tokenise(" ", false);
       break;
     }
 
   if (serverMechs.GetSize() == 0)
-    return PTrue; // No mechanisms, no login
+    return true; // No mechanisms, no login
 
   PSASLClient auth("smtp", username, username, password);
   PStringSet ourMechs;
 
   if (!auth.Init("", ourMechs))
-    return PFalse;
+    return false;
 
   PString mech;
 
@@ -148,19 +148,19 @@ PBoolean PSMTPClient::LogIn(const PString & username,
     }
 
   if (mech.IsEmpty())
-    return PTrue;  // No mechanism in common
+    return true;  // No mechanism in common
 
   PString output;
 
   // Ok, let's go...
   if (!auth.Start(mech, output))
-    return PFalse;
+    return false;
 
   if (!output.IsEmpty())
     mech = mech + " " + output;
 
   if (ExecuteCommand(AUTH, mech) <= 0)
-    return PFalse;
+    return false;
 
   PSASLClient::PSASLResult result;
   int response;
@@ -171,22 +171,22 @@ PBoolean PSMTPClient::LogIn(const PString & username,
     if (response == 2)
       break;
     else if (response != 3)
-      return PFalse;
+      return false;
 
     result = auth.Negotiate(lastResponseInfo, output);
       
     if (result == PSASLClient::Fail)
-      return PFalse;
+      return false;
 
     if (!output.IsEmpty()) {
       WriteLine(output);
       if (!ReadResponse())
-        return PFalse;
+        return false;
     }
   } while (result == PSASLClient::Continue);
   auth.End();
 
-  return PTrue;
+  return true;
 }
 
 #else
@@ -194,7 +194,7 @@ PBoolean PSMTPClient::LogIn(const PString & username,
 PBoolean PSMTPClient::LogIn(const PString &,
                         const PString &)
 {
-  return PTrue;
+  return true;
 }
 
 #endif
@@ -235,16 +235,16 @@ bool PSMTPClient::InternalBeginMessage()
 
   if (!haveHello) {
     if (ExecuteCommand(EHLO, localHost)/100 == 2)
-      haveHello = extendedHello = PTrue;
+      haveHello = extendedHello = true;
   }
 
   if (!haveHello) {
-    extendedHello = PFalse;
+    extendedHello = false;
     if (eightBitMIME)
-      return PFalse;
+      return false;
     if (ExecuteCommand(HELO, localHost)/100 != 2)
-      return PFalse;
-    haveHello = PTrue;
+      return false;
+    haveHello = true;
   }
 
   if (fromAddress[0] != '"' && fromAddress.Find(' ') != P_MAX_INDEX)
@@ -252,7 +252,7 @@ bool PSMTPClient::InternalBeginMessage()
   if (!localHost && fromAddress.Find('@') == P_MAX_INDEX)
     fromAddress += '@' + localHost;
   if (ExecuteCommand(MAIL, "FROM:<" + fromAddress + '>')/100 != 2)
-    return PFalse;
+    return false;
 
   for (PStringList::iterator i = toNames.begin(); i != toNames.end(); i++) {
     if (!peerHost && i->Find('@') == P_MAX_INDEX)
@@ -264,8 +264,10 @@ bool PSMTPClient::InternalBeginMessage()
   if (ExecuteCommand(DATA, PString())/100 != 3)
     return false;
 
+  flush();
+
   stuffingState = StuffIdle;
-  sendingData = PTrue;
+  sendingData = true;
   return true;
 }
 
@@ -275,10 +277,10 @@ PBoolean PSMTPClient::EndMessage()
   flush();
 
   stuffingState = DontStuff;
-  sendingData = PFalse;
+  sendingData = false;
 
   if (!WriteString(CRLFdotCRLF))
-    return PFalse;
+    return false;
 
   return ReadResponse() && lastResponseCode/100 == 2;
 }
@@ -289,8 +291,8 @@ PBoolean PSMTPClient::EndMessage()
 
 PSMTPServer::PSMTPServer()
 {
-  extendedHello = PFalse;
-  eightBitMIME = PFalse;
+  extendedHello = false;
+  eightBitMIME = false;
   messageBufferSize = 30000;
   ServerReset();
 }
@@ -298,7 +300,7 @@ PSMTPServer::PSMTPServer()
 
 void PSMTPServer::ServerReset()
 {
-  eightBitMIME = PFalse;
+  eightBitMIME = false;
   sendCommand = WasMAIL;
   fromAddress = PString();
   toNames.RemoveAll();
@@ -316,7 +318,7 @@ PBoolean PSMTPServer::ProcessCommand()
   PString args;
   PINDEX num;
   if (!ReadCommand(num, args))
-    return PFalse;
+    return false;
 
   switch (num) {
     case HELO :
@@ -327,7 +329,7 @@ PBoolean PSMTPServer::ProcessCommand()
       break;
     case QUIT :
       OnQUIT();
-      return PFalse;
+      return false;
     case NOOP :
       OnNOOP();
       break;
@@ -365,13 +367,13 @@ PBoolean PSMTPServer::ProcessCommand()
       return OnUnknown(args);
   }
 
-  return PTrue;
+  return true;
 }
 
 
 void PSMTPServer::OnHELO(const PCaselessString & remoteHost)
 {
-  extendedHello = PFalse;
+  extendedHello = false;
   ServerReset();
 
   PCaselessString peerHost;
@@ -394,7 +396,7 @@ void PSMTPServer::OnHELO(const PCaselessString & remoteHost)
 
 void PSMTPServer::OnEHLO(const PCaselessString & remoteHost)
 {
-  extendedHello = PTrue;
+  extendedHello = true;
   ServerReset();
 
   PCaselessString peerHost;
@@ -657,9 +659,9 @@ void PSMTPServer::OnDATA()
 
   endMIMEDetectState = eightBitMIME ? StuffIdle : DontStuff;
 
-  PBoolean ok = PTrue;
-  PBoolean completed = PFalse;
-  PBoolean starting = PTrue;
+  PBoolean ok = true;
+  PBoolean completed = false;
+  PBoolean starting = true;
 
   while (ok && !completed) {
     PCharArray buffer;
@@ -669,7 +671,7 @@ void PSMTPServer::OnDATA()
       ok = OnTextData(buffer, completed);
     if (ok) {
       ok = HandleMessage(buffer, starting, completed);
-      starting = PFalse;
+      starting = false;
     }
   }
 
@@ -683,7 +685,7 @@ void PSMTPServer::OnDATA()
 PBoolean PSMTPServer::OnUnknown(const PCaselessString & command)
 {
   WriteResponse(500, "Command \"" + command + "\" unrecognised.");
-  return PTrue;
+  return true;
 }
 
 
@@ -693,8 +695,8 @@ PBoolean PSMTPServer::OnTextData(PCharArray & buffer, PBoolean & completed)
   while (ReadLine(line)) {
     PINDEX len = line.GetLength();
     if (len == 1 && line[0] == '.') {
-      completed = PTrue;
-      return PTrue;
+      completed = true;
+      return true;
     }
 
     PINDEX start = (len > 1 && line[0] == '.' && line[1] == '.') ? 1 : 0;
@@ -706,9 +708,9 @@ PBoolean PSMTPServer::OnTextData(PCharArray & buffer, PBoolean & completed)
     buffer[size++] = '\r';
     buffer[size++] = '\n';
     if (size > messageBufferSize)
-      return PTrue;
+      return true;
   }
-  return PFalse;
+  return false;
 }
 
 
@@ -758,8 +760,8 @@ PBoolean PSMTPServer::OnMIMEData(PCharArray & buffer, PBoolean & completed)
 
       case StuffCRLFdotCR :
         if (c == '\n') {
-          completed = PTrue;
-          return PTrue;
+          completed = true;
+          return true;
         }
         buffer[count++] = '.';
         buffer[count++] = '\r';
@@ -771,11 +773,11 @@ PBoolean PSMTPServer::OnMIMEData(PCharArray & buffer, PBoolean & completed)
     }
     if (count > messageBufferSize) {
       buffer.SetSize(messageBufferSize);
-      return PTrue;
+      return true;
     }
   }
 
-  return PFalse;
+  return false;
 }
 
 
@@ -796,7 +798,7 @@ PSMTPServer::LookUpResult PSMTPServer::LookUpName(const PCaselessString &,
 
 PBoolean PSMTPServer::HandleMessage(PCharArray &, PBoolean, PBoolean)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -810,8 +812,8 @@ static char const * const POP3Commands[PPOP3::NumCommands] = {
 };
 
 
-PString PPOP3::okResponse = "+OK";
-PString PPOP3::errResponse = "-ERR";
+const PString & PPOP3::okResponse () { static const PConstString s("+OK");  return s; }
+const PString & PPOP3::errResponse() { static const PConstString s("-ERR"); return s; }
 
 
 PPOP3::PPOP3()
@@ -837,7 +839,7 @@ PINDEX PPOP3::ParseResponse(const PString & line)
 
 PPOP3Client::PPOP3Client()
 {
-  loggedIn = PFalse;
+  loggedIn = false;
 }
 
 
@@ -849,7 +851,7 @@ PPOP3Client::~PPOP3Client()
 PBoolean PPOP3Client::OnOpen()
 {
   if (!ReadResponse() || lastResponseCode <= 0)
-    return PFalse;
+    return false;
 
   // APOP login command supported?
   PINDEX i = lastResponseInfo.FindRegEx("<.*@.*>");
@@ -857,13 +859,13 @@ PBoolean PPOP3Client::OnOpen()
   if (i != P_MAX_INDEX)
     apopBanner = lastResponseInfo.Mid(i);
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PPOP3Client::Close()
 {
-  PBoolean ok = PTrue;
+  PBoolean ok = true;
   if (IsOpen() && loggedIn) {
     SetReadTimeout(60000);
     ok = ExecuteCommand(QUIT, "") > 0;
@@ -874,7 +876,7 @@ PBoolean PPOP3Client::Close()
 
 PBoolean PPOP3Client::LogIn(const PString & username, const PString & password, int options)
 {
-#if P_SASL2
+#if P_SASL
   PString mech;
   PSASLClient auth("pop", username, username, password);
 
@@ -893,9 +895,9 @@ PBoolean PPOP3Client::LogIn(const PString & username, const PString & password,
         ourMechs.Exclude("LOGIN");
       }
 
-      for (PINDEX i = 0, max = serverMechs.GetSize() ; i < max ; i++)
-        if (ourMechs.Contains(serverMechs.GetKeyAt(i))) {
-          mech = serverMechs.GetKeyAt(i);
+      for (PStringSet::iterator it = serverMechs.begin(); it != serverMechs.end(); ++it)
+        if (ourMechs.Contains(*it)) {
+          mech = *it;
           break;
         }
     }
@@ -906,7 +908,7 @@ PBoolean PPOP3Client::LogIn(const PString & username, const PString & password,
   if ((options & UseSASL) && !mech.IsEmpty() && auth.Start(mech, output)) {
 
     if (ExecuteCommand(AUTH, mech) <= 0)
-      return PFalse;
+      return false;
 
     PSASLClient::PSASLResult result;
 
@@ -914,19 +916,19 @@ PBoolean PPOP3Client::LogIn(const PString & username, const PString & password,
       result = auth.Negotiate(lastResponseInfo, output);
       
       if (result == PSASLClient::Fail)
-        return PFalse;
+        return false;
 
       if (!output.IsEmpty()) {
         WriteLine(output);
         if (!ReadResponse() || !lastResponseCode)
-          return PFalse;
+          return false;
       }
     } while (result == PSASLClient::Continue);
     auth.End();
   }
-  else {
+  else
 #endif
-
+  {
     if (!apopBanner.IsEmpty()) { // let's try with APOP
 
       PMessageDigest::Result bin_digest;
@@ -939,27 +941,24 @@ PBoolean PPOP3Client::LogIn(const PString & username, const PString & password,
         digest.sprintf("%02x", (unsigned)data[i]);
 
       if (ExecuteCommand(APOP, username + " " + digest) > 0)
-        return loggedIn = PTrue;
+        return loggedIn = true;
     }
 
     // No SASL and APOP didn't work for us
     // If we really have to, we'll go with the plain old USER/PASS scheme
 
     if (!(options & AllowUserPass))
-      return PFalse;
+      return false;
 
     if (ExecuteCommand(USER, username) <= 0)
-      return PFalse;
+      return false;
 
     if (ExecuteCommand(PASS, password) <= 0)
-      return PFalse;
-
-#if P_SASL2
+      return false;
   }
-#endif
 
-  loggedIn = PTrue;
-  return PTrue;
+  loggedIn = true;
+  return true;
 }
 
 
@@ -995,7 +994,7 @@ PStringArray PPOP3Client::GetMessageHeaders()
   for (int msgNum = 1; msgNum <= count; msgNum++) {
     if (ExecuteCommand(TOP, PString(PString::Unsigned,msgNum) + " 0") > 0) {
       PString headerLine;
-      while (ReadLine(headerLine, PTrue))
+      while (ReadLine(headerLine, true))
         headers[msgNum-1] += headerLine;
     }
   }
@@ -1025,7 +1024,7 @@ PPOP3Server::PPOP3Server()
 
 PBoolean PPOP3Server::OnOpen()
 {
-  return WriteResponse(okResponse, PIPSocket::GetHostName() +
+  return WriteResponse(okResponse(), PIPSocket::GetHostName() +
                      " POP3 server ready at " +
                       PTime(PTime::MediumDateTime).AsString());
 }
@@ -1036,7 +1035,7 @@ PBoolean PPOP3Server::ProcessCommand()
   PString args;
   PINDEX num;
   if (!ReadCommand(num, args))
-    return PFalse;
+    return false;
 
   switch (num) {
     case USER :
@@ -1047,7 +1046,7 @@ PBoolean PPOP3Server::ProcessCommand()
       break;
     case QUIT :
       OnQUIT();
-      return PFalse;
+      return false;
     case RSET :
       OnRSET();
       break;
@@ -1068,7 +1067,7 @@ PBoolean PPOP3Server::ProcessCommand()
       break;
     case TOP :
       if (args.Find(' ') == P_MAX_INDEX)
-        WriteResponse(errResponse, "Syntax error");
+        WriteResponse(errResponse(), "Syntax error");
       else
         OnTOP((PINDEX)args.AsInteger(),
               (PINDEX)args.Mid(args.Find(' ')).AsInteger());
@@ -1080,7 +1079,7 @@ PBoolean PPOP3Server::ProcessCommand()
       return OnUnknown(args);
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1089,18 +1088,18 @@ void PPOP3Server::OnUSER(const PString & name)
   messageSizes.SetSize(0);
   messageIDs.SetSize(0);
   username = name;
-  WriteResponse(okResponse, "User name accepted.");
+  WriteResponse(okResponse(), "User name accepted.");
 }
 
 
 void PPOP3Server::OnPASS(const PString & password)
 {
   if (username.IsEmpty())
-    WriteResponse(errResponse, "No user name specified.");
+    WriteResponse(errResponse(), "No user name specified.");
   else if (HandleOpenMailbox(username, password))
-    WriteResponse(okResponse, username + " mail is available.");
+    WriteResponse(okResponse(), username + " mail is available.");
   else
-    WriteResponse(errResponse, "No access to " + username + " mail.");
+    WriteResponse(errResponse(), "No access to " + username + " mail.");
   messageDeletions.SetSize(messageIDs.GetSize());
 }
 
@@ -1111,7 +1110,7 @@ void PPOP3Server::OnQUIT()
     if (messageDeletions[i])
       HandleDeleteMessage(i+1, messageIDs[i]);
 
-  WriteResponse(okResponse, PIPSocket::GetHostName() +
+  WriteResponse(okResponse(), PIPSocket::GetHostName() +
                      " POP3 server signing off at " +
                       PTime(PTime::MediumDateTime).AsString());
 
@@ -1122,14 +1121,14 @@ void PPOP3Server::OnQUIT()
 void PPOP3Server::OnRSET()
 {
   for (PINDEX i = 0; i < messageDeletions.GetSize(); i++)
-    messageDeletions[i] = PFalse;
-  WriteResponse(okResponse, "Resetting state.");
+    messageDeletions[i] = false;
+  WriteResponse(okResponse(), "Resetting state.");
 }
 
 
 void PPOP3Server::OnNOOP()
 {
-  WriteResponse(okResponse, "Doing nothing.");
+  WriteResponse(okResponse(), "Doing nothing.");
 }
 
 
@@ -1138,32 +1137,32 @@ void PPOP3Server::OnSTAT()
   DWORD total = 0;
   for (PINDEX i = 0; i < messageSizes.GetSize(); i++)
     total += messageSizes[i];
-  WriteResponse(okResponse, psprintf("%u %u", messageSizes.GetSize(), total));
+  WriteResponse(okResponse(), psprintf("%u %u", messageSizes.GetSize(), total));
 }
 
 
 void PPOP3Server::OnLIST(PINDEX msg)
 {
   if (msg == 0) {
-    WriteResponse(okResponse, psprintf("%u messages.", messageSizes.GetSize()));
+    WriteResponse(okResponse(), psprintf("%u messages.", messageSizes.GetSize()));
     for (PINDEX i = 0; i < messageSizes.GetSize(); i++)
       if (!messageDeletions[i])
         WriteLine(psprintf("%u %u", i+1, messageSizes[i]));
     WriteLine(".");
   }
   else if (msg < 1 || msg > messageSizes.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else
-    WriteResponse(okResponse, psprintf("%u %u", msg, messageSizes[msg-1]));
+    WriteResponse(okResponse(), psprintf("%u %u", msg, messageSizes[msg-1]));
 }
 
 
 void PPOP3Server::OnRETR(PINDEX msg)
 {
   if (msg < 1 || msg > messageDeletions.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else {
-    WriteResponse(okResponse,
+    WriteResponse(okResponse(),
                  PString(PString::Unsigned, messageSizes[msg-1]) + " octets.");
     stuffingState = StuffIdle;
     HandleSendMessage(msg, messageIDs[msg-1], P_MAX_INDEX);
@@ -1176,10 +1175,10 @@ void PPOP3Server::OnRETR(PINDEX msg)
 void PPOP3Server::OnDELE(PINDEX msg)
 {
   if (msg < 1 || msg > messageDeletions.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else {
-    messageDeletions[msg-1] = PTrue;
-    WriteResponse(okResponse, "Message marked for deletion.");
+    messageDeletions[msg-1] = true;
+    WriteResponse(okResponse(), "Message marked for deletion.");
   }
 }
 
@@ -1187,9 +1186,9 @@ void PPOP3Server::OnDELE(PINDEX msg)
 void PPOP3Server::OnTOP(PINDEX msg, PINDEX count)
 {
   if (msg < 1 || msg > messageDeletions.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else {
-    WriteResponse(okResponse, "Top of message");
+    WriteResponse(okResponse(), "Top of message");
     stuffingState = StuffIdle;
     HandleSendMessage(msg, messageIDs[msg-1], count);
     stuffingState = DontStuff;
@@ -1201,7 +1200,7 @@ void PPOP3Server::OnTOP(PINDEX msg, PINDEX count)
 void PPOP3Server::OnUIDL(PINDEX msg)
 {
   if (msg == 0) {
-    WriteResponse(okResponse,
+    WriteResponse(okResponse(),
               PString(PString::Unsigned, messageIDs.GetSize()) + " messages.");
     for (PINDEX i = 0; i < messageIDs.GetSize(); i++)
       if (!messageDeletions[i])
@@ -1209,7 +1208,7 @@ void PPOP3Server::OnUIDL(PINDEX msg)
     WriteLine(".");
   }
   else if (msg < 1 || msg > messageSizes.GetSize())
-    WriteResponse(errResponse, "No such message.");
+    WriteResponse(errResponse(), "No such message.");
   else
     WriteLine(PString(PString::Unsigned, msg) & messageIDs[msg-1]);
 }
@@ -1217,14 +1216,14 @@ void PPOP3Server::OnUIDL(PINDEX msg)
 
 PBoolean PPOP3Server::OnUnknown(const PCaselessString & command)
 {
-  WriteResponse(errResponse, "Command \"" + command + "\" unrecognised.");
-  return PTrue;
+  WriteResponse(errResponse(), "Command \"" + command + "\" unrecognised.");
+  return true;
 }
 
 
 PBoolean PPOP3Server::HandleOpenMailbox(const PString &, const PString &)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -1241,24 +1240,24 @@ void PPOP3Server::HandleDeleteMessage(PINDEX, const PString &)
 //////////////////////////////////////////////////////////////////////////////
 // PRFC822Channel
 
-const PString & PRFC822Channel::MimeVersionTag() { static PString s = "MIME-version"; return s; }
-const PString & PRFC822Channel::FromTag() { static PString s = "From"; return s; }
-const PString & PRFC822Channel::ToTag() { static PString s = "To"; return s; }
-const PString & PRFC822Channel::CCTag() { static PString s = "cc"; return s; }
-const PString & PRFC822Channel::BCCTag() { static PString s = "bcc"; return s; }
-const PString & PRFC822Channel::SubjectTag() { static PString s = "Subject"; return s; }
-const PString & PRFC822Channel::DateTag() { static PString s = "Date"; return s; }
-const PString & PRFC822Channel::ReturnPathTag() { static PString s = "Return-Path"; return s; }
-const PString & PRFC822Channel::ReceivedTag() { static PString s = "Received"; return s; }
-const PString & PRFC822Channel::MessageIDTag() { static PString s = "Message-ID"; return s; }
-const PString & PRFC822Channel::MailerTag() { static PString s = "X-Mailer"; return s; }
+const PCaselessString & PRFC822Channel::MimeVersionTag() { static const PConstCaselessString s("MIME-version"); return s; }
+const PCaselessString & PRFC822Channel::FromTag()        { static const PConstCaselessString s("From");         return s; }
+const PCaselessString & PRFC822Channel::ToTag()          { static const PConstCaselessString s("To");           return s; }
+const PCaselessString & PRFC822Channel::CCTag()          { static const PConstCaselessString s("cc");           return s; }
+const PCaselessString & PRFC822Channel::BCCTag()         { static const PConstCaselessString s("bcc");          return s; }
+const PCaselessString & PRFC822Channel::SubjectTag()     { static const PConstCaselessString s("Subject");      return s; }
+const PCaselessString & PRFC822Channel::DateTag()        { static const PConstCaselessString s("Date");         return s; }
+const PCaselessString & PRFC822Channel::ReturnPathTag()  { static const PConstCaselessString s("Return-Path");  return s; }
+const PCaselessString & PRFC822Channel::ReceivedTag()    { static const PConstCaselessString s("Received");     return s; }
+const PCaselessString & PRFC822Channel::MessageIDTag()   { static const PConstCaselessString s("Message-ID");   return s; }
+const PCaselessString & PRFC822Channel::MailerTag()      { static const PConstCaselessString s("X-Mailer");     return s; }
 
 
 
 PRFC822Channel::PRFC822Channel(Direction direction)
 {
   writeHeaders = direction == Sending;
-  writePartHeaders = PFalse;
+  writePartHeaders = false;
   base64 = NULL;
 }
 
@@ -1284,7 +1283,7 @@ PBoolean PRFC822Channel::Write(const void * buf, PINDEX len)
 
   if (writeHeaders) {
     if (!headers.Contains(FromTag()) || !headers.Contains(ToTag()))
-      return PFalse;
+      return false;
 
     if (!headers.Contains(MimeVersionTag()))
       headers.SetAt(MimeVersionTag(), "1.0");
@@ -1295,33 +1294,37 @@ PBoolean PRFC822Channel::Write(const void * buf, PINDEX len)
     if (writePartHeaders)
       headers.SetAt(ContentTypeTag(), "multipart/mixed; boundary=\""+boundaries.front()+'"');
     else if (!headers.Contains(ContentTypeTag()))
-      headers.SetAt(ContentTypeTag(), "text/plain");
+      headers.SetAt(ContentTypeTag(), PMIMEInfo::TextPlain());
 
     PStringStream hdr;
-    hdr << ::setfill('\r') << headers;
-    if (!PIndirectChannel::Write(hdr.GetPointer(), hdr.GetLength()))
-      return PFalse;
+    hdr << headers;
+    if (!PIndirectChannel::Write((const char *)hdr, hdr.GetLength()))
+      return false;
 
     if (base64 != NULL)
       base64->StartEncoding();
 
-    writeHeaders = PFalse;
+    writeHeaders = false;
+
+    flush();
   }
 
-  if (writePartHeaders) {
+  if (writePartHeaders && (boundaries.GetSize() > 0)) {
     if (!partHeaders.Contains(ContentTypeTag()))
-      partHeaders.SetAt(ContentTypeTag(), "text/plain");
+      partHeaders.SetAt(ContentTypeTag(), PMIMEInfo::TextPlain());
 
     PStringStream hdr;
     hdr << "\n--"  << boundaries.front() << '\n'
-        << ::setfill('\r') << partHeaders;
-    if (!PIndirectChannel::Write(hdr.GetPointer(), hdr.GetLength()))
-      return PFalse;
+        << partHeaders;
+    if (!PIndirectChannel::Write((const char *)hdr, hdr.GetLength()))
+      return false;
 
     if (base64 != NULL)
       base64->StartEncoding();
 
-    writePartHeaders = PFalse;
+    writePartHeaders = false;
+
+    flush();
   }
 
   PBoolean ok;
@@ -1330,9 +1333,11 @@ PBoolean PRFC822Channel::Write(const void * buf, PINDEX len)
   else {
     base64->ProcessEncoding(buf, len);
     PString str = base64->GetEncodedString();
-    ok = PIndirectChannel::Write(str.GetPointer(), str.GetLength());
+    ok = PIndirectChannel::Write((const char *)str, str.GetLength());
   }
 
+  flush();
+
   // Always return the lastWriteCount as the number of bytes expected to be
   // written, not teh actual number which with base64 encoding etc may be
   // significantly more.
@@ -1345,7 +1350,7 @@ PBoolean PRFC822Channel::Write(const void * buf, PINDEX len)
 PBoolean PRFC822Channel::OnOpen()
 {
   if (writeHeaders)
-    return PTrue;
+    return true;
 
   istream & this_stream = *this;
   this_stream >> headers;
@@ -1362,7 +1367,7 @@ void PRFC822Channel::NewMessage(Direction direction)
   headers.RemoveAll();
   partHeaders.RemoveAll();
   writeHeaders = direction == Sending;
-  writePartHeaders = PFalse;
+  writePartHeaders = false;
 }
 
 
@@ -1371,7 +1376,7 @@ PString PRFC822Channel::MultipartMessage()
   PString boundary;
 
   do {
-    boundary.sprintf("PWLib.%lu.%u", PTime().GetTimeInSeconds(), rand());
+    boundary.sprintf("PTLib.%lu.%u", PTime().GetTimeInSeconds(), rand());
   } while (!MultipartMessage(boundary));
 
   return boundary;
@@ -1380,7 +1385,7 @@ PString PRFC822Channel::MultipartMessage()
 
 PBoolean PRFC822Channel::MultipartMessage(const PString & boundary)
 {
-  writePartHeaders = PTrue;
+  writePartHeaders = true;
   for (PStringList::iterator i = boundaries.begin(); i != boundaries.end(); i++) {
     if (*i == boundary)
       return false;
@@ -1389,10 +1394,10 @@ PBoolean PRFC822Channel::MultipartMessage(const PString & boundary)
   if (boundaries.GetSize() > 0) {
     partHeaders.SetAt(ContentTypeTag(), "multipart/mixed; boundary=\""+boundary+'"');
     flush();
-    writePartHeaders = PTrue;
+    writePartHeaders = true;
   }
 
-  boundaries.InsertAt(0, new PString(boundary));
+  boundaries.Prepend(new PString(boundary));
   return true;
 }
 
@@ -1404,13 +1409,14 @@ void PRFC822Channel::NextPart(const PString & boundary)
     base64 = NULL;
     *this << oldBase64->CompleteEncoding() << '\n';
     delete oldBase64;
+    flush();
   }
 
   while (boundaries.GetSize() > 0) {
     if (boundaries.front() == boundary)
       break;
     *this << "\n--" << boundaries.front() << "--\n";
-    boundaries.RemoveAt(0);
+    boundaries.RemoveHead();
   }
 
   flush();
@@ -1499,10 +1505,10 @@ PBoolean PRFC822Channel::SendWithSMTP(const PString & hostname)
 PBoolean PRFC822Channel::SendWithSMTP(PSMTPClient * smtp)
 {
   if (!Open(smtp))
-    return PFalse;
+    return false;
 
   if (!headers.Contains(FromTag()) || !headers.Contains(ToTag()))
-    return PFalse;
+    return false;
 
   return smtp->BeginMessage(headers[FromTag()], headers[ToTag()]);
 }
