@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23902 $
+ * $Revision: 26962 $
  * $Author: rjongbloed $
- * $Date: 2010-01-06 18:30:15 -0600 (Wed, 06 Jan 2010) $
+ * $Date: 2012-02-08 20:55:46 -0600 (Wed, 08 Feb 2012) $
  */
 
 #ifdef __GNUC__
@@ -70,6 +70,14 @@ void PInternetProtocol::SetReadLineTimeout(const PTimeInterval & t)
 
 PBoolean PInternetProtocol::Read(void * buf, PINDEX len)
 {
+  if (unReadCount == 0) {
+    char readAhead[1000];
+    if (!PIndirectChannel::Read(readAhead, sizeof(readAhead)))
+      return false;
+
+    UnRead(readAhead, GetLastReadCount());
+  }
+
   lastReadCount = PMIN(unReadCount, len);
   const char * unReadPtr = ((const char *)unReadBuffer)+unReadCount;
   char * bufptr = (char *)buf;
@@ -79,9 +87,6 @@ PBoolean PInternetProtocol::Read(void * buf, PINDEX len)
     len--;
   }
 
-  if (unReadCount == 0)
-    unReadBuffer.SetSize(0);
-
   if (len > 0) {
     PINDEX saveCount = lastReadCount;
     PIndirectChannel::Read(bufptr, len);
@@ -92,6 +97,23 @@ PBoolean PInternetProtocol::Read(void * buf, PINDEX len)
 }
 
 
+int PInternetProtocol::ReadChar()
+{
+  if (unReadCount == 0) {
+    char readAhead[1000];
+    if (!PIndirectChannel::Read(readAhead, sizeof(readAhead)))
+      return -1;
+
+    UnRead(readAhead, GetLastReadCount());
+    if (unReadCount == 0)
+      return -1;
+  }
+
+  lastReadCount = 1;
+  return (unReadBuffer[--unReadCount]&0xff);
+}
+
+
 PBoolean PInternetProtocol::Write(const void * buf, PINDEX len)
 {
   if (len == 0 || stuffingState == DontStuff)
@@ -245,11 +267,11 @@ PBoolean PInternetProtocol::WriteLine(const PString & line)
 }
 
 
-PBoolean PInternetProtocol::ReadLine(PString & str, PBoolean allowContinuation)
+PBoolean PInternetProtocol::ReadLine(PString & line, PBoolean allowContinuation)
 {
-  str = PString();
+  if (!line.SetMinSize(1000))
+    return false;
 
-  PCharArray line(100);
   PINDEX count = 0;
   PBoolean gotEndOfLine = false;
 
@@ -261,13 +283,6 @@ PBoolean PInternetProtocol::ReadLine(PString & str, PBoolean allowContinuation)
   SetReadTimeout(readLineTimeout);
 
   while (c >= 0 && !gotEndOfLine) {
-    if (unReadCount == 0) {
-      char readAhead[1000];
-      SetReadTimeout(0);
-      if (PIndirectChannel::Read(readAhead, sizeof(readAhead)))
-        UnRead(readAhead, GetLastReadCount());
-      SetReadTimeout(readLineTimeout);
-    }
     switch (c) {
       case '\b' :
       case '\177' :
@@ -315,8 +330,10 @@ PBoolean PInternetProtocol::ReadLine(PString & str, PBoolean allowContinuation)
 
   SetReadTimeout(oldTimeout);
 
-  if (count > 0)
-    str = PString(line, count);
+  if (count < line.GetSize()-1)
+    line[count] = '\0';
+  line.MakeMinimumSize();
+
   return gotEndOfLine;
 }
 
@@ -345,22 +362,28 @@ void PInternetProtocol::UnRead(const void * buffer, PINDEX len)
 }
 
 
-PBoolean PInternetProtocol::WriteCommand(PINDEX cmdNumber)
-{
-  if (cmdNumber >= commandNames.GetSize())
-    return false;
-  return WriteLine(commandNames[cmdNumber]);
-}
-
-
 PBoolean PInternetProtocol::WriteCommand(PINDEX cmdNumber, const PString & param)
 {
   if (cmdNumber >= commandNames.GetSize())
     return false;
-  if (param.IsEmpty())
-    return WriteLine(commandNames[cmdNumber]);
-  else
-    return WriteLine(commandNames[cmdNumber] & param);
+
+  *this << commandNames[cmdNumber];
+
+  if (!param.IsEmpty())
+    *this << ' ' << param;
+
+  *this << CRLF << ::flush;
+  return good();
+}
+
+
+PBoolean PInternetProtocol::WriteCommand(PINDEX cmdNumber, const PString & param, const PMIMEInfo & mime)
+{
+  if (cmdNumber >= commandNames.GetSize())
+    return false;
+
+  *this << commandNames[cmdNumber] << ' ' << param << CRLF << setfill('\r') << mime << ::flush;
+  return good();
 }
 
 
@@ -384,6 +407,15 @@ PBoolean PInternetProtocol::ReadCommand(PINDEX & num, PString & args)
 }
 
 
+PBoolean PInternetProtocol::ReadCommand(PINDEX & num, PString & args, PMIMEInfo & mime)
+{
+  if (!ReadCommand(num, args))
+    return false;
+
+  return mime.Read(*this);
+}
+
+
 PBoolean PInternetProtocol::WriteResponse(unsigned code, const PString & info)
 {
   return WriteResponse(psprintf("%03u", code), info);
@@ -457,6 +489,15 @@ PBoolean PInternetProtocol::ReadResponse(int & code, PString & info)
 }
 
 
+PBoolean PInternetProtocol::ReadResponse(int & code, PString & info, PMIMEInfo & mime)
+{
+  if (!ReadResponse(code, info))
+    return false;
+
+  return mime.Read(*this);
+}
+
+
 PINDEX PInternetProtocol::ParseResponse(const PString & line)
 {
   PINDEX endCode = line.FindOneOf(" -");
@@ -517,10 +558,32 @@ PMIMEInfo::PMIMEInfo(PInternetProtocol & socket)
 }
 
 
+PMIMEInfo::PMIMEInfo(const PStringToString & dict)
+  : PStringOptions(dict)
+{
+}
+
+
+PMIMEInfo::PMIMEInfo(const PString & str)
+{
+  PStringStream strm(str);
+  ReadFrom(strm);
+}
+
+
+PString PMIMEInfo::AsString() const
+{
+  PStringStream strm;
+  PrintOn(strm);
+  return strm;
+}
+
+
 void PMIMEInfo::PrintOn(ostream &strm) const
 {
+  bool crlf = strm.fill() == '\r';
   PrintContents(strm);
-  if (strm.fill() == '\r')
+  if (crlf)
     strm << '\r';
   strm << '\n';
 }
@@ -529,9 +592,9 @@ ostream & PMIMEInfo::PrintContents(ostream &strm) const
 {
   PBoolean output_cr = strm.fill() == '\r';
   strm.fill(' ');
-  for (PINDEX i = 0; i < GetSize(); i++) {
-    PString name = GetKeyAt(i) + ": ";
-    PString value = GetDataAt(i);
+  for (const_iterator it = begin(); it != end(); ++it) {
+    PString name = it->first + ": ";
+    PString value = it->second;
     if (value.FindOneOf("\r\n") != P_MAX_INDEX) {
       PStringArray vals = value.Lines();
       for (PINDEX j = 0; j < vals.GetSize(); j++) {
@@ -558,7 +621,7 @@ void PMIMEInfo::ReadFrom(istream &strm)
 
   PString line;
   PString lastLine;
-  while (!strm.bad() && !strm.eof()) {
+  while (strm.good()) {
     strm >> line;
     if (line.IsEmpty())
       break;
@@ -570,9 +633,8 @@ void PMIMEInfo::ReadFrom(istream &strm)
     }
   }
 
-  if (!lastLine.IsEmpty()) {
+  if (!lastLine.IsEmpty())
     AddMIME(lastLine);
-  }
 }
 
 
@@ -597,14 +659,15 @@ bool PMIMEInfo::AddMIME(const PString & line)
   if (colonPos == P_MAX_INDEX)
     return false;
 
-  PCaselessString fieldName  = line.Left(colonPos).Trim();
-  PString fieldValue = line(colonPos+1, P_MAX_INDEX).Trim();
+  PINDEX dataPos = colonPos+1;
+  while (isspace(line[dataPos]))
+    ++dataPos;
 
-  return AddMIME(fieldName, fieldValue);
+  return AddMIME(line.Left(colonPos).Trim(), line.Mid(dataPos));
 }
 
 
-bool PMIMEInfo::AddMIME(const PString & fieldName, const PString & fieldValue)
+bool PMIMEInfo::InternalAddMIME(const PString & fieldName, const PString & fieldValue)
 {
   PString * str = GetAt(fieldName);
   if (str == NULL)
@@ -618,8 +681,8 @@ bool PMIMEInfo::AddMIME(const PString & fieldName, const PString & fieldValue)
 
 bool PMIMEInfo::AddMIME(const PMIMEInfo & mime)
 {
-  for (PINDEX i = 0; i < mime.GetSize(); ++i) {
-    if (!AddMIME(mime.GetKeyAt(i), mime.GetDataAt(i)))
+  for (const_iterator it = mime.begin(); it != mime.end(); ++it) {
+    if (!AddMIME(it->first, it->second))
       return false;
   }
 
@@ -629,9 +692,9 @@ bool PMIMEInfo::AddMIME(const PMIMEInfo & mime)
 
 PBoolean PMIMEInfo::Write(PInternetProtocol & socket) const
 {
-  for (PINDEX i = 0; i < GetSize(); i++) {
-    PString name = GetKeyAt(i) + ": ";
-    PString value = GetDataAt(i);
+  for (const_iterator it = begin(); it != end(); ++it) {
+    PString name = it->first + ": ";
+    PString value = it->second;
     if (value.FindOneOf("\r\n") != P_MAX_INDEX) {
       PStringArray vals = value.Lines();
       for (PINDEX j = 0; j < vals.GetSize(); j++) {
@@ -649,26 +712,6 @@ PBoolean PMIMEInfo::Write(PInternetProtocol & socket) const
 }
 
 
-PString PMIMEInfo::GetString(const PString & key, const PString & dflt) const
-{
-  PString * str = GetAt(PCaselessString(key));
-  return str != NULL ? *str : dflt;
-}
-
-
-long PMIMEInfo::GetInteger(const PString & key, long dflt) const
-{
-  PString * str = GetAt(PCaselessString(key));
-  return str != NULL ? str->AsInteger() : dflt;
-}
-
-
-void PMIMEInfo::SetInteger(const PCaselessString & key, long value)
-{
-  SetAt(key, PString(PString::Unsigned, value));
-}
-
-
 bool PMIMEInfo::ParseComplex(const PString & fieldValue, PStringToString & info)
 {
   info.RemoveAll();
@@ -753,7 +796,7 @@ bool PMIMEInfo::ParseComplex(const PString & fieldValue, PStringToString & info)
 }
 
 
-bool PMIMEInfo::DecodeMultiPartList(PMultiPartList & parts, const PString & body, const PString & key) const
+bool PMIMEInfo::DecodeMultiPartList(PMultiPartList & parts, const PString & body, const PCaselessString & key) const
 {
   PStringToString info;
   return GetComplex(key, info) && parts.Decode(body, info);
@@ -798,15 +841,15 @@ void PMIMEInfo::SetAssociation(const PStringToString & allTypes, PBoolean merge)
   PStringToString & types = GetContentTypes();
   if (!merge)
     types.RemoveAll();
-  for (PINDEX i = 0; i < allTypes.GetSize(); i++)
-    types.SetAt(allTypes.GetKeyAt(i), allTypes.GetDataAt(i));
+  for (const_iterator it = allTypes.begin(); it != allTypes.end(); ++it)
+    types.SetAt(it->first, it->second);
 }
 
 
 PString PMIMEInfo::GetContentType(const PString & fType)
 {
   if (fType.IsEmpty())
-    return "text/plain";
+    return PMIMEInfo::TextPlain();
 
   PStringToString & types = GetContentTypes();
   if (types.Contains(fType))
@@ -816,11 +859,13 @@ PString PMIMEInfo::GetContentType(const PString & fType)
 }
 
 
-const PString & PMIMEInfo::ContentTypeTag()             { static PString s = "Content-Type";              return s; }
-const PString & PMIMEInfo::ContentDispositionTag()      { static PString s = "Content-Disposition";       return s; }
-const PString & PMIMEInfo::ContentTransferEncodingTag() { static PString s = "Content-Transfer-Encoding"; return s; }
-const PString & PMIMEInfo::ContentDescriptionTag()      { static PString s = "Content-Description";       return s; }
-const PString & PMIMEInfo::ContentIdTag()               { static PString s = "Content-ID";                return s; }
+const PCaselessString & PMIMEInfo::ContentTypeTag()             { static const PConstCaselessString s("Content-Type");              return s; }
+const PCaselessString & PMIMEInfo::ContentDispositionTag()      { static const PConstCaselessString s("Content-Disposition");       return s; }
+const PCaselessString & PMIMEInfo::ContentTransferEncodingTag() { static const PConstCaselessString s("Content-Transfer-Encoding"); return s; }
+const PCaselessString & PMIMEInfo::ContentDescriptionTag()      { static const PConstCaselessString s("Content-Description");       return s; }
+const PCaselessString & PMIMEInfo::ContentIdTag()               { static const PConstCaselessString s("Content-ID");                return s; }
+
+const PCaselessString & PMIMEInfo::TextPlain()                  { static const PConstCaselessString s("text/plain");                return s; }
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -911,8 +956,8 @@ bool PMultiPartList::Decode(const PString & entityBody, const PStringToString &
     info->m_mime.ReadFrom(strm);
 
     // Skip over MIME
-    partPtr += strm.tellp();
-    partLen -= strm.tellp();
+    partPtr += (int)strm.tellp();
+    partLen -= (int)strm.tellp();
 
     // Check transfer encoding
     PStringToString typeInfo;
@@ -922,8 +967,10 @@ bool PMultiPartList::Decode(const PString & entityBody, const PStringToString &
     // save the entity body, being careful of binary files
     if (encoding == "base64")
       PBase64::Decode(PString(partPtr, partLen), info->m_binaryBody);
+#ifdef P_HAS_WCHAR
     else if (typeInfo("charset") *= "UCS-2")
       info->m_textBody = PString((const wchar_t *)partPtr, partLen/2);
+#endif
     else if (encoding == "7bit" || encoding == "8bit" || (typeInfo("charset") *= "UTF-8") || memchr(partPtr, 0, partLen) == NULL)
       info->m_textBody = PString(partPtr, partLen);
     else
@@ -938,7 +985,7 @@ bool PMultiPartList::Decode(const PString & entityBody, const PStringToString &
         info->m_mime.SetAt(PMIMEInfo::ContentTypeTag, startContentType);
 
       // Make sure "start" mime entry is at the beginning of list
-      InsertAt(0, info);
+      Prepend(info);
       startContentId.MakeEmpty();
     }
 
