@@ -23,9 +23,9 @@
  *
  * Copyright 2003 Equivalence Pty. Ltd.
  *
- * $Revision: 28359 $
- * $Author: rjongbloed $
- * $Date: 2012-09-14 18:44:41 -0500 (Fri, 14 Sep 2012) $
+ * $Revision: 28931 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 06:31:11 -0600 (Fri, 18 Jan 2013) $
  */
 
 #ifdef __GNUC__
@@ -39,45 +39,24 @@
 
 #define new PNEW
 
-#define USE_RESOLVER_CACHING    1
 #define RESOLVER_CACHE_TIMEOUT  30000
 
-#if P_DNS
+#if P_DNS_RESOLVER
 
-#ifdef _WIN32
+#ifdef _MSC_VER
   #pragma comment(lib, "DnsAPI.Lib")
   #pragma message("DNS support enabled")
-#else
-  enum { DnsFreeFlat = 0 };
 #endif
 
 
 /////////////////////////////////////////////////
 
-#if defined(P_HAS_RESOLVER) ||  defined(USE_RESOLVER_CACHING)
-
-void PDnsRecordListFree(PDNS_RECORD rec, int /* FreeType */)
-{
-  while (rec != NULL) {
-    PDNS_RECORD next = rec->pNext;
-    free(rec);
-    rec = next;
-  }
-}
-
-#endif
-
-#if defined(USE_RESOLVER_CACHING) || (defined(P_HAS_RESOLVER) && !defined(P_HAS_RES_NINIT))
-
 static PMutex & GetDNSMutex()
 {
   static PMutex mutex;
   return mutex;
 }
 
-#endif
-
-#if defined(USE_RESOLVER_CACHING) 
 
 struct DNSCacheInfo {
   DNSCacheInfo() : m_results(NULL), m_status(-1) { }
@@ -91,17 +70,16 @@ typedef std::map<std::string, DNSCacheInfo> DNSCache;
 static PTime g_lastAgeTime(0);
 static DNSCache g_dnsCache;
 
-#endif
 
-#ifdef P_HAS_RESOLVER
+#ifdef P_HAS_RESOLV_H
 
 static PBoolean GetDN(const BYTE * reply, const BYTE * replyEnd, BYTE * & cp, char * buff)
 {
   int len = dn_expand(reply, replyEnd, cp, buff, MAXDNAME);
   if (len < 0)
-    return PFalse;
+    return false;
   cp += len;
-  return PTrue;
+  return true;
 }
 
 static PBoolean ProcessDNSRecords(
@@ -133,7 +111,7 @@ static PBoolean ProcessDNSRecords(
     // get the name
     char pName[MAXDNAME];
     if (!GetDN(reply, replyEnd, cp, pName))
-      return PFalse;
+      return false;
 
     // get other common parts of the record
     WORD  type;
@@ -166,7 +144,7 @@ static PBoolean ProcessDNSRecords(
         GETSHORT(newRecord->Data.SRV.wPort, data);
         if (!GetDN(reply, replyEnd, data, newRecord->Data.SRV.pNameTarget)) {
           free(newRecord);
-          return PFalse;
+          return false;
         }
         break;
 
@@ -176,7 +154,7 @@ static PBoolean ProcessDNSRecords(
         GETSHORT(newRecord->Data.MX.wPreference,  data);
         if (!GetDN(reply, replyEnd, data, newRecord->Data.MX.pNameExchange)) {
           free(newRecord);
-          return PFalse;
+          return false;
         }
         break;
 
@@ -200,7 +178,7 @@ static PBoolean ProcessDNSRecords(
         memset(newRecord, 0, sizeof(DnsRecord));
         if (!GetDN(reply, replyEnd, data, newRecord->Data.NS.pNameHost)) {
           delete newRecord;
-          return PFalse;
+          return false;
         }
         break;
     }
@@ -223,13 +201,13 @@ static PBoolean ProcessDNSRecords(
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 DNS_STATUS DnsQuery_A(const char * service,
                               WORD requestType,
                              DWORD options,
-                            PIP4_ARRAY,
+                            void *,
                      PDNS_RECORD * results,
                             void *)
 {
@@ -294,7 +272,7 @@ DNS_STATUS DnsQuery_A(const char * service,
        ntohs(reply.hdr.nscount),
        ntohs(reply.hdr.arcount),
        results)) {
-    PDnsRecordListFree(*results, 0);
+    DnsRecordListFree(*results, DnsFreeRecordList);
     return -1;
   }
 
@@ -302,7 +280,40 @@ DNS_STATUS DnsQuery_A(const char * service,
 }
 
 
-#endif // P_HAS_RESOLVER
+PDNS_RECORD DnsRecordSetCopy(PDNS_RECORD src)
+{
+  PDNS_RECORD result = NULL;
+  PDNS_RECORD dst = NULL;
+  PDNS_RECORD rec;
+
+  while (src != NULL) {
+    rec = (PDNS_RECORD)malloc(sizeof(DNS_RECORD));
+    memcpy(rec, src, sizeof(DNS_RECORD));
+    if (result == NULL)
+      result = rec;
+    rec->pNext = NULL;
+    if (dst != NULL)
+      dst->pNext = rec;
+    src = src->pNext;
+    dst = rec;
+  }
+
+  return result;
+}
+
+
+void DnsRecordListFree(PDNS_RECORD rec, int /* FreeType */)
+{
+  while (rec != NULL) {
+    PDNS_RECORD next = rec->pNext;
+    free(rec);
+    rec = next;
+  }
+}
+
+
+#endif // P_HAS_RESOLV_H
+
 
 PObject::Comparison PDNS::SRVRecord::Compare(const PObject & obj) const
 {
@@ -397,9 +408,9 @@ PDNS::SRVRecord * PDNS::SRVRecordList::GetFirst()
     priList.SetSize(1);
     WORD lastPri = (*this)[0].priority;
     priList[0] = lastPri;
-    (*this)[0].used = PFalse;
+    (*this)[0].used = false;
     for (i = 1; i < GetSize(); i++) {
-      (*this)[i].used = PFalse;
+      (*this)[i].used = false;
       if ((*this)[i].priority != lastPri) {
         priPos++;
         priList.SetSize(priPos);
@@ -455,7 +466,7 @@ PDNS::SRVRecord * PDNS::SRVRecordList::GetNext()
         if (!(*this)[i].used) {
           totalWeight += (*this)[i].weight;
           if (totalWeight >= targetWeight) {
-            (*this)[i].used = PTrue;
+            (*this)[i].used = true;
             return &(*this)[i];
           }
         }
@@ -468,7 +479,7 @@ PDNS::SRVRecord * PDNS::SRVRecordList::GetNext()
     for (i = firstPos; i < GetSize() && ((*this)[i].priority == currentPri); i++) {
       if (!(*this)[i].used) {
         if (count == j) {
-          (*this)[i].used = PTrue;
+          (*this)[i].used = true;
           return &(*this)[i];
         }
         count++;
@@ -490,7 +501,7 @@ PBoolean PDNS::GetSRVRecords(
 )
 {
   if (_service.IsEmpty())
-    return PFalse;
+    return false;
 
   PStringStream service;
   if (_service[0] != '_')
@@ -501,6 +512,9 @@ PBoolean PDNS::GetSRVRecords(
   return GetSRVRecords(service, recordList);
 }
 
+
+#if P_URL
+
 PBoolean PDNS::LookupSRV(
            const PURL & url,
         const PString & service,
@@ -511,7 +525,7 @@ PBoolean PDNS::LookupSRV(
 
   if (!LookupSRV(url.GetHostName(), service, defaultPort, info)) {
     PTRACE(2,"DNS\tSRV Lookup Fail no domain " << url );
-    return PFalse;
+    return false;
   }
 
   PString user = url.GetUserName();
@@ -529,6 +543,9 @@ PBoolean PDNS::LookupSRV(
   return returnList.GetSize() != 0;;
 }
 
+#endif // P_URL
+
+
 PBoolean PDNS::LookupSRV(
          const PString & domain,            ///< domain to lookup
          const PString & service,           ///< service to use
@@ -538,7 +555,7 @@ PBoolean PDNS::LookupSRV(
 {
   if (domain.IsEmpty()) {
     PTRACE(1,"DNS\tSRV lookup failed - no domain specified");
-    return PFalse;
+    return false;
   }
 
   PString srvLookupStr = service;
@@ -560,7 +577,7 @@ PBoolean PDNS::LookupSRV(
   PDNS::SRVRecordList srvRecords;
   PBoolean found = PDNS::GetRecords(srvLookupStr, srvRecords);
   if (found) {
-    PTRACE(5,"DNS\tSRV Record found " << srvLookupStr);
+    PTRACE(5,"DNS\tSRV Record found \"" << srvLookupStr << '"');
     PDNS::SRVRecord * recPtr = srvRecords.GetFirst();
     while (recPtr != NULL) {
       PIPSocketAddressAndPort addrAndPort;
@@ -648,7 +665,7 @@ PDNS::MXRecord * PDNS::MXRecordList::GetFirst()
 {
   PINDEX i;
   for (i = 0; i < GetSize(); i++) 
-    (*this)[i].used = PFalse;
+    (*this)[i].used = false;
 
   lastIndex = 0;
 
@@ -668,52 +685,6 @@ PDNS::MXRecord * PDNS::MXRecordList::GetNext()
 
 /////////////////////////////////////////////////////////////////
 
-#ifndef USE_RESOLVER_CACHING
-
-DNS_STATUS PDNS::Cached_DnsQuery(
-    const char * name,
-    WORD       type,
-    DWORD      ,
-    void *     ,
-    PDNS_RECORD * queryResults,
-    void * )
-{
-  return DnsQuery_A((const char *)name, 
-                               type,
-                               DNS_QUERY_STANDARD, 
-                               (PIP4_ARRAY)NULL, 
-                               queryResults, 
-                               NULL);
-}
-
-void PDNS::Cached_DnsRecordListFree(PDNS_RECORD results, int)
-{
-  DnsRecordListFree(results, 0);
-}
-
-#else  // ifndef USE_RESOLVER_CACHING
-
-static PDNS_RECORD DnsRecordListClone(PDNS_RECORD src)
-{
-  PDNS_RECORD result = NULL;
-  PDNS_RECORD dst = NULL;
-  PDNS_RECORD rec;
-
-  while (src != NULL) {
-    rec = (PDNS_RECORD)malloc(sizeof(DNS_RECORD));
-    memcpy(rec, src, sizeof(DNS_RECORD));
-    if (result == NULL)
-      result = rec;
-    rec->pNext = NULL;
-    if (dst != NULL)
-      dst->pNext = rec;
-    src = src->pNext;
-    dst = rec;
-  }
-
-  return result;
-}
-
 DNS_STATUS PDNS::Cached_DnsQuery(
     const char * name,
     WORD       type,
@@ -735,54 +706,70 @@ DNS_STATUS PDNS::Cached_DnsQuery(
     while (r != g_dnsCache.end()) {
       if ((now - r->second.m_time) < RESOLVER_CACHE_TIMEOUT)
         ++r;
-      else { 
-        DnsRecordListFree(r->second.m_results, DnsFreeFlat);
+      else {
+        PTRACE(5, "DNS\tQuery aged \"" << r->first << '"');
+        DnsRecordListFree(r->second.m_results, DnsFreeRecordList);
         g_dnsCache.erase(r++);
       }
     }
   }
 
   // see if cache contains the entry we need
-  std::stringstream key;
-  key << name << '\n' << type << '\n' << options;
+  string key;
+  {
+    std::stringstream strm;
+    strm << name << '\t' << type << '\t' << options;
+    key = strm.str();
+  }
 
-  r = g_dnsCache.find(key.str());
+  r = g_dnsCache.find(key);
   if (r == g_dnsCache.end()) {
+    PTRACE(5, "DNS\tSRV physical lookup \"" << key << '"');
 
     // else do the lookup and put it into the cache
     DNSCacheInfo info;
     info.m_status = DnsQuery_A((const char *)name, 
                                type,
                                DNS_QUERY_STANDARD, 
-                               (PIP4_ARRAY)NULL, 
+                               NULL, 
                                &info.m_results, 
                                NULL);
-    PTRACE_IF(3, info.m_status != 0, "DNS\tSRV query failed, error=" << info.m_status);
+#if PTRACING
+    if (info.m_status != 0)
+      PTRACE(3, "DNS\tQuery failed: error=" << info.m_status);
+    else {
+      PTRACE(6, "DNS\tQuery success: " << info.m_results);
+      for (PDNS_RECORD rec = info.m_results; rec != NULL; rec = rec->pNext)
+        PTRACE(6, "DNS\tQuery: name=\"" << PString(rec->pName)
+               << "\", type=" << rec->wType << ", len=" << rec->wDataLength);
+      PTRACE(6, "DNS\tQuery done");
+    }
+#endif
 
-    r = g_dnsCache.insert(DNSCache::value_type(key.str(), info)).first;
+    r = g_dnsCache.insert(DNSCache::value_type(key, info)).first;
   }
 
-  *queryResults = DnsRecordListClone(r->second.m_results);
+  *queryResults = DnsRecordSetCopy(r->second.m_results);
   return r->second.m_status;
 }
 
-void PDNS::Cached_DnsRecordListFree(PDNS_RECORD results, int)
+
+PDNS::PDnsRecords::~PDnsRecords()
 {
-  PDnsRecordListFree(results, 0);
+  if (m_records != NULL)
+    DnsRecordListFree(m_records, DnsFreeRecordList);
 }
 
 
-#endif  // ifndef USE_RESOLVER_CACHING
-
 /////////////////////////////////////////////////////////////////
 
 #else
 
   #ifdef _MSC_VER
-    #pragma message("DNS support DISABLED")
+    #pragma message("DNS resolver support DISABLED")
   #endif
 
-#endif // P_DNS
+#endif // P_DNS_RESOLVER
 
 
 // End Of File ///////////////////////////////////////////////////////////////
