@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25075 $
- * $Author: willamowius $
- * $Date: 2011-01-19 12:22:45 -0600 (Wed, 19 Jan 2011) $
+ * $Revision: 28484 $
+ * $Author: rjongbloed $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #ifdef __GNUC__
@@ -84,28 +84,28 @@ PBoolean PLDAPSession::Open(const PString & server, WORD port)
 
   ldapContext = ldap_init(server, port);
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   SetOption(LDAP_OPT_PROTOCOL_VERSION, protocolVersion);
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PLDAPSession::Close()
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   ldap_unbind(ldapContext);
   ldapContext = NULL;
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PLDAPSession::SetOption(int optcode, int value)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   return ldap_set_option(ldapContext, optcode, &value);
 }
@@ -114,18 +114,27 @@ PBoolean PLDAPSession::SetOption(int optcode, int value)
 PBoolean PLDAPSession::SetOption(int optcode, void * value)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   return ldap_set_option(ldapContext, optcode, value);
 }
 
+PBoolean PLDAPSession::StartTLS()
+{
+#ifdef LDAP_EXOP_START_TLS
+  errorNumber = ldap_start_tls_s(ldapContext, NULL, NULL);
+  return errorNumber == LDAP_SUCCESS;
+#else
+  return LDAP_OPERATIONS_ERROR;
+#endif
+}
 
 PBoolean PLDAPSession::Bind(const PString & who,
                         const PString & passwd,
                         AuthenticationMethod authMethod)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   const char * whoPtr;
   if (who.IsEmpty())
@@ -210,7 +219,7 @@ void PLDAPSession::StringModAttrib::AddValue(const PString & value)
 
 PBoolean PLDAPSession::StringModAttrib::IsBinary() const
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -219,9 +228,9 @@ void PLDAPSession::StringModAttrib::SetLDAPModVars(struct ldapmod & mod)
   pointers.SetSize(values.GetSize()+1);
   PINDEX i;
   for (i = 0; i < values.GetSize(); i++)
-    pointers[i] = values[i].GetPointer();
+    pointers[i] = values[i];
   pointers[i] = NULL;
-  mod.mod_values = pointers.GetPointer();
+  mod.mod_values = (char **)pointers.GetPointer();
 }
 
 
@@ -265,7 +274,7 @@ void PLDAPSession::BinaryModAttrib::AddValue(const PBYTEArray & value)
 
 PBoolean PLDAPSession::BinaryModAttrib::IsBinary() const
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -307,9 +316,9 @@ static PArray<PLDAPSession::ModAttrib> AttribsFromDict(const PStringToString & a
 {
   PArray<PLDAPSession::ModAttrib> attrs(attributes.GetSize());
 
-  for (PINDEX i = 0; i < attributes.GetSize(); i++)
-    attrs.SetAt(i, new PLDAPSession::StringModAttrib(attributes.GetKeyAt(i),
-                                                     attributes.GetDataAt(i).Lines()));
+  PINDEX index = 0;
+  for (PStringToString::const_iterator it = attributes.begin(); it != attributes.end(); ++it)
+    attrs.SetAt(index++, new PLDAPSession::StringModAttrib(it->first, it->second.Lines()));
 
   return attrs;
 }
@@ -335,8 +344,8 @@ static PArray<PLDAPSession::ModAttrib> AttribsFromStruct(const PLDAPStructBase &
 {
   PArray<PLDAPSession::ModAttrib> attrs;
 
-  for (PINDEX i = 0; i < attributes.GetNumAttributes(); i++) {
-    PLDAPAttributeBase & attr = attributes.GetAttribute(i);
+  for (PLDAPStructBase::AttribDict::const_iterator it = attributes.GetAttributes().begin(); it != attributes.GetAttributes().end(); ++it) {
+    const PLDAPAttributeBase & attr = it->second;
     if (attr.IsBinary())
       attrs.Append(new PLDAPSession::BinaryModAttrib(attr.GetName(), attr.ToBinary()));
     else {
@@ -353,7 +362,7 @@ static PArray<PLDAPSession::ModAttrib> AttribsFromStruct(const PLDAPStructBase &
 PBoolean PLDAPSession::Add(const PString & dn, const PArray<ModAttrib> & attributes)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   PBYTEArray storage;
   int msgid;
@@ -364,13 +373,13 @@ PBoolean PLDAPSession::Add(const PString & dn, const PArray<ModAttrib> & attribu
                              NULL,
                              &msgid);
   if (errorNumber != LDAP_SUCCESS)
-    return PFalse;
+    return false;
 
   P_timeval tval = timeout;
   LDAPMessage * result = NULL;
   ldap_result(ldapContext, msgid, LDAP_MSG_ALL, tval, &result);
   if (result)
-    errorNumber = ldap_result2error(ldapContext, result, PTrue);
+    errorNumber = ldap_result2error(ldapContext, result, true);
 
   return errorNumber == LDAP_SUCCESS;
 }
@@ -397,7 +406,7 @@ PBoolean PLDAPSession::Add(const PString & dn, const PLDAPStructBase & attribute
 PBoolean PLDAPSession::Modify(const PString & dn, const PArray<ModAttrib> & attributes)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   PBYTEArray storage;
   int msgid;
@@ -408,13 +417,13 @@ PBoolean PLDAPSession::Modify(const PString & dn, const PArray<ModAttrib> & attr
                                 NULL,
                                 &msgid);
   if (errorNumber != LDAP_SUCCESS)
-    return PFalse;
+    return false;
 
   P_timeval tval = timeout;
   LDAPMessage * result = NULL;
   ldap_result(ldapContext, msgid, LDAP_MSG_ALL, tval, &result);
   if (result)
-    errorNumber = ldap_result2error(ldapContext, result, PTrue);
+    errorNumber = ldap_result2error(ldapContext, result, true);
 
   return errorNumber == LDAP_SUCCESS;
 }
@@ -441,18 +450,18 @@ PBoolean PLDAPSession::Modify(const PString & dn, const PLDAPStructBase & attrib
 PBoolean PLDAPSession::Delete(const PString & dn)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   int msgid;
   errorNumber = ldap_delete_ext(ldapContext, dn, NULL, NULL, &msgid);
   if (errorNumber != LDAP_SUCCESS)
-    return PFalse;
+    return false;
 
   P_timeval tval = timeout;
   LDAPMessage * result = NULL;
   ldap_result(ldapContext, msgid, LDAP_MSG_ALL, tval, &result);
   if (result)
-    errorNumber = ldap_result2error(ldapContext, result, PTrue);
+    errorNumber = ldap_result2error(ldapContext, result, true);
 
   return errorNumber == LDAP_SUCCESS;
 }
@@ -462,8 +471,8 @@ PLDAPSession::SearchContext::SearchContext()
 {
   result = NULL;
   message = NULL;
-  found = PFalse;
-  completed = PFalse;
+  found = false;
+  completed = false;
 }
 
 
@@ -484,7 +493,7 @@ PBoolean PLDAPSession::Search(SearchContext & context,
                           SearchScope scope)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   PCharArray storage;
   char ** attribs = attributes.ToCharArray(&storage);
@@ -504,7 +513,7 @@ PBoolean PLDAPSession::Search(SearchContext & context,
                                 ScopeCode[scope],
                                 filter,
                                 attribs,
-                                PFalse,
+                                false,
                                 NULL,
                                 NULL,
                                 tval,
@@ -512,16 +521,16 @@ PBoolean PLDAPSession::Search(SearchContext & context,
                                 &context.msgid);
 
   if (errorNumber != LDAP_SUCCESS)
-    return PFalse;
+    return false;
 
   if (ldap_result(ldapContext, context.msgid, LDAP_MSG_ONE, tval, &context.result) > 0)
     return GetNextSearchResult(context);
 
   if (context.result)
-    errorNumber = ldap_result2error(ldapContext, context.result, PTrue);
+    errorNumber = ldap_result2error(ldapContext, context.result, true);
   if (errorNumber == 0)
     errorNumber = LDAP_OTHER;
-  return PFalse;
+  return false;
 }
 
 
@@ -530,10 +539,10 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context, PStringToString
   data.RemoveAll();
 
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   if (context.result == NULL || context.message == NULL || context.completed)
-    return PFalse;
+    return false;
 
   // Extract the resulting data
 
@@ -564,7 +573,7 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context, PStringToString
   if (ber != NULL)
     ber_free (ber, 0);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -575,19 +584,19 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context,
   data.MakeEmpty();
 
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   if (context.result == NULL || context.message == NULL || context.completed)
-    return PFalse;
+    return false;
 
   if (attribute == "dn") {
     data = GetSearchResultDN(context);
-    return PTrue;
+    return true;
   }
 
   char ** values = ldap_get_values(ldapContext, context.message, attribute);
   if (values == NULL)
-    return PFalse;
+    return false;
 
   PINDEX count = ldap_count_values(values);
   for (PINDEX i = 0; i < count; i++) {
@@ -597,7 +606,7 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context,
   }
 
   ldap_value_free(values);
-  return PTrue;
+  return true;
 }
 
 
@@ -608,20 +617,20 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context,
   data.RemoveAll();
 
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   if (context.result == NULL || context.message == NULL || context.completed)
-    return PFalse;
+    return false;
 
   if (attribute == "dn") {
     data.SetSize(1);
     data[0] = GetSearchResultDN(context);
-    return PTrue;
+    return true;
   }
 
   char ** values = ldap_get_values(ldapContext, context.message, attribute);
   if (values == NULL)
-    return PFalse;
+    return false;
 
   PINDEX count = ldap_count_values(values);
   data.SetSize(count);
@@ -629,7 +638,7 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context,
     data[i] = values[i];
 
   ldap_value_free(values);
-  return PTrue;
+  return true;
 }
 
 
@@ -640,18 +649,18 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context,
   data.RemoveAll();
 
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   if (attribute == "dn") {
     char * dn = ldap_get_dn(ldapContext, context.message);
     data.Append(new PBYTEArray((const BYTE *)dn, ::strlen(dn)));
     ldap_memfree(dn);
-    return PTrue;
+    return true;
   }
 
   struct berval ** values = ldap_get_values_len(ldapContext, context.message, attribute);
   if (values == NULL)
-    return PFalse;
+    return false;
 
   PINDEX count = ldap_count_values_len(values);
   data.SetSize(count);
@@ -663,7 +672,7 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context,
   }
 
   ldap_value_free_len(values);
-  return PTrue;
+  return true;
 }
 
 
@@ -671,24 +680,24 @@ PBoolean PLDAPSession::GetSearchResult(SearchContext & context,
                                    PLDAPStructBase & data)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
-  PBoolean atLeastOne = PFalse;
+  PBoolean atLeastOne = false;
 
-  for (PINDEX i = 0; i < data.GetNumAttributes(); i++) {
-    PLDAPAttributeBase & attr = data.GetAttribute(i);
+  for (PLDAPStructBase::AttribDict::iterator it = data.GetAttributes().begin(); it != data.GetAttributes().end(); ++it) {
+    PLDAPAttributeBase & attr = it->second;
     if (attr.IsBinary()) {
       PArray<PBYTEArray> bin;
       if (GetSearchResult(context, attr.GetName(), bin)) {
         attr.FromBinary(bin);
-        atLeastOne = PTrue;
+        atLeastOne = true;
       }
     }
     else {
       PString str;
       if (GetSearchResult(context, attr.GetName(), str)) {
         attr.FromString(str);
-        atLeastOne = PTrue;
+        atLeastOne = true;
       }
     }
   }
@@ -716,10 +725,10 @@ PString PLDAPSession::GetSearchResultDN(SearchContext & context)
 PBoolean PLDAPSession::GetNextSearchResult(SearchContext & context)
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   if (context.result == NULL || context.completed)
-    return PFalse;
+    return false;
 
   P_timeval tval = timeout;
   do {
@@ -731,19 +740,19 @@ PBoolean PLDAPSession::GetNextSearchResult(SearchContext & context)
     if (context.message != NULL) {
       switch (ldap_msgtype(context.message)) {
         case LDAP_RES_SEARCH_ENTRY :
-          context.found = PTrue;
+          context.found = true;
           errorNumber = LDAP_SUCCESS;
-          return PTrue;
+          return true;
 
         case LDAP_RES_SEARCH_RESULT :
-          errorNumber = ldap_result2error(ldapContext, context.message, PFalse);
+          errorNumber = ldap_result2error(ldapContext, context.message, false);
           if (errorNumber == 0 && !context.found)
             errorNumber = LDAP_NO_RESULTS_RETURNED;
-          context.completed = PTrue;
-          return PFalse;
+          context.completed = true;
+          return false;
         case LDAP_RES_SEARCH_REFERENCE :
           errorNumber = LDAP_SUCCESS;
-          return PTrue;
+          return true;
         // Ignore other result message types for now ...
         default:
           PTRACE(3, "Unhandled LDAP message type " << ldap_msgtype(context.message));
@@ -754,10 +763,10 @@ PBoolean PLDAPSession::GetNextSearchResult(SearchContext & context)
   } while (ldap_result(ldapContext, context.msgid, LDAP_MSG_ONE, tval, &context.result) > 0);
 
   if (context.result)
-    errorNumber = ldap_result2error(ldapContext, context.result, PFalse);
+    errorNumber = ldap_result2error(ldapContext, context.result, false);
   if (errorNumber == 0)
     errorNumber = LDAP_OTHER;
-  return PFalse;
+  return false;
 }
 
 
@@ -820,7 +829,7 @@ void PLDAPAttributeBase::FromString(const PString & str)
 
 PBYTEArray PLDAPAttributeBase::ToBinary() const
 {
-  return PBYTEArray((const BYTE *)pointer, size, PFalse);
+  return PBYTEArray((const BYTE *)pointer, size, false);
 }
 
 
@@ -848,8 +857,11 @@ PLDAPStructBase::PLDAPStructBase()
 
 PLDAPStructBase & PLDAPStructBase::operator=(const PLDAPStructBase & other)
 {
-  for (PINDEX i = 0; i < attributes.GetSize(); i++)
-    attributes.GetDataAt(i).Copy(other.attributes.GetDataAt(i));
+  for (AttribDict::iterator it = attributes.begin(); it != attributes.end(); ++it) {
+    PLDAPAttributeBase * otherAttrib = other.attributes.GetAt(it->first);
+    if (otherAttrib != NULL)
+      it->second.Copy(*otherAttrib);
+  }
 
   return *this;
 }
@@ -872,10 +884,10 @@ PLDAPStructBase & PLDAPStructBase::operator=(const PStringArray & array)
 
 PLDAPStructBase & PLDAPStructBase::operator=(const PStringToString & dict)
 {
-  for (PINDEX i = 0; i < dict.GetSize(); i++) {
-    PLDAPAttributeBase * attr = GetAttribute(dict.GetKeyAt(i));
+  for (PStringToString::const_iterator it = dict.begin(); it != dict.end(); ++it) {
+    PLDAPAttributeBase * attr = GetAttribute(it->first);
     if (attr != NULL)
-      attr->FromString(dict.GetDataAt(i));
+      attr->FromString(it->second);
   }
   return *this;
 }
@@ -903,7 +915,7 @@ void PLDAPStructBase::EndConstructor()
 
 static const char PLDAPSchemaPluginBaseClass[] = "PLDAPSchema";
 
-template <> PLDAPSchema * PDevicePluginFactory<PLDAPSchema>::Worker::Create(const PString & type) const
+template <> PLDAPSchema * PDevicePluginFactory<PLDAPSchema>::Worker::Create(const PDefaultPFactoryKey & type) const
 {
   return PLDAPSchema::CreateSchema(type);
 }
@@ -958,11 +970,11 @@ PBoolean PLDAPSchema::SetAttribute(const PString & attribute, const PString & va
 		if ((r->m_name == attribute) &&  (r->m_type != AttributeBinary)) {
 	       attributes.insert(make_pair(attribute,value));
 		   PTRACE(4, "schema\tMatch " << attribute);
-		   return PTrue;
+		   return true;
 		}
 	}
 
-	return PFalse;
+	return false;
 }
 
 PBoolean PLDAPSchema::SetAttribute(const PString & attribute, const PBYTEArray & value)
@@ -971,11 +983,11 @@ PBoolean PLDAPSchema::SetAttribute(const PString & attribute, const PBYTEArray &
 		if ((r->m_name == attribute) && (r->m_type == AttributeBinary)) {
 	       binattributes.insert(make_pair(attribute,value));
 		   PTRACE(4, "schema\tMatch Binary " << attribute);
-		   return PTrue;
+		   return true;
 		}
 	}
 
-	return PFalse;
+	return false;
 }
 
 PBoolean PLDAPSchema::GetAttribute(const PString & attribute, PString & value)
@@ -983,10 +995,10 @@ PBoolean PLDAPSchema::GetAttribute(const PString & attribute, PString & value)
 	for (ldapAttributes::const_iterator r = attributes.begin(); r != attributes.end(); ++r) {
 		if (r->first == attribute) {
 			value = r->second;
-			return PTrue;
+			return true;
 		}
 	}
-	return PFalse; 
+	return false; 
 }
 
 PBoolean PLDAPSchema::GetAttribute(const PString & attribute, PBYTEArray & value)
@@ -994,10 +1006,10 @@ PBoolean PLDAPSchema::GetAttribute(const PString & attribute, PBYTEArray & value
 	for (ldapBinAttributes::const_iterator r = binattributes.begin(); r != binattributes.end(); ++r) {
 		if (r->first == attribute) {
 			value = r->second;
-			return PTrue;
+			return true;
 		}
 	}
-	return PFalse; 
+	return false; 
 }
 
 PStringList PLDAPSchema::GetAttributeList()
@@ -1016,17 +1028,17 @@ PBoolean PLDAPSchema::Exists(const PString & attribute)
 		  if (r->m_type == AttributeString) {
 	          for (ldapAttributes::const_iterator r = attributes.begin(); r != attributes.end(); ++r) {
                   if (r->first == attribute)
-			         return PTrue;
+			         return true;
 		      }
 		  } else if (r->m_type == AttributeBinary) {
 	          for (ldapBinAttributes::const_iterator r = binattributes.begin(); r != binattributes.end(); ++r) {
                   if (r->first == attribute)
-			         return PTrue;
+			         return true;
 			  }
 		  }
 	  }
 	}
-	return PFalse;
+	return false;
 }
 
 PLDAPSchema::AttributeType PLDAPSchema::GetAttributeType(const PString & attribute)
