@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24875 $
+ * $Revision: 29362 $
  * $Author: rjongbloed $
- * $Date: 2010-11-12 02:03:15 -0600 (Fri, 12 Nov 2010) $
+ * $Date: 2013-03-28 02:36:25 -0500 (Thu, 28 Mar 2013) $
  */
 
 #ifdef __GNUC__
@@ -41,12 +41,27 @@
 #include <ptclib/ptts.h>
 
 #include <ptlib/pipechan.h>
-#include <ptclib/ptts.h>
-
-
 
 #if P_SAPI
 
+// Following defined by both stdint.h and intsafe.h
+#undef INT8_MIN
+#undef INT16_MIN
+#undef INT32_MIN
+#undef INT64_MIN
+#undef INT8_MAX
+#undef UINT8_MAX
+#undef INT16_MAX
+#undef UINT16_MAX
+#undef INT32_MAX
+#undef UINT32_MAX
+#undef INT64_MAX
+#undef UINT64_MAX
+
+#include <intsafe.h>
+
+#include <ptlib/msos/ptlib/pt_atl.h>
+
 ////////////////////////////////////////////////////////////
 //
 // Text to speech using Microsoft's Speech API (SAPI)
@@ -56,73 +71,6 @@
   #pragma message("SAPI support enabled")
 #endif
 
-#ifndef _WIN32_DCOM
-  #define _WIN32_DCOM 1
-#endif
-
-#ifndef P_ATL
-
-  // We are using express edition of MSVC which does not come with ATL support
-  // So hand implement just enough for the SAPI code to work.
-  #define __ATLBASE_H__
-
-  #include <objbase.h>
-
-  typedef WCHAR OLECHAR;
-  typedef OLECHAR *LPOLESTR;
-  typedef const OLECHAR *LPCOLESTR;
-  typedef struct IUnknown IUnknown;
-  typedef IUnknown *LPUNKNOWN;
-
-  template <class T> class CComPtr
-  {
-    public:
-      CComPtr(T * pp = NULL) : p(pp) { }
-      ~CComPtr() { Release(); }
-
-      T *  operator-> () const { return  PAssertNULL(p); }
-           operator T*() const { return  PAssertNULL(p); }
-      T &  operator*  () const { return *PAssertNULL(p); }
-      T ** operator&  ()       { return &p; }
-      bool operator!() const   { return (p == NULL); }
-      bool operator<(__in_opt T* pT) const  { return p <  pT; }
-      bool operator==(__in_opt T* pT) const { return p == pT; }
-      bool operator!=(__in_opt T* pT) const { return p != pT; }
-
-      void Attach(T * p2)
-      {
-        if (p)
-	  p->Release();
-        p = p2;
-      }
-
-      T * Detach()
-      {
-        T * pt = p;
-        p = NULL;
-        return pt;
-      }
-
-      void Release()
-      {
-        T * pt = p;
-        if (pt != NULL) {
-	  p = NULL;
-	  pt->Release();
-        }
-      }
-
-      __checkReturn HRESULT CoCreateInstance(__in REFCLSID rclsid, __in_opt LPUNKNOWN pUnkOuter = NULL, __in DWORD dwClsContext = CLSCTX_ALL)
-      {
-        return ::CoCreateInstance(rclsid, pUnkOuter, dwClsContext, __uuidof(T), (void**)&p);
-      }
-
-    private:
-      T * p;
-  };
-
-#endif // P_ATL
-
 #include <sphelper.h>
 
 
@@ -153,6 +101,7 @@ class PTextToSpeech_SAPI : public PTextToSpeech
     CComPtr<ISpVoice>  m_cpVoice;
     CComPtr<ISpStream> m_cpWavStream;
     bool               m_opened;
+    PString            m_CurrentVoice;
 };
 
 PFACTORY_CREATE(PFactory<PTextToSpeech>, PTextToSpeech_SAPI, "Microsoft SAPI", false);
@@ -164,7 +113,8 @@ PFACTORY_CREATE(PFactory<PTextToSpeech>, PTextToSpeech_SAPI, "Microsoft SAPI", f
 PTextToSpeech_SAPI::PTextToSpeech_SAPI()
   : m_opened(false)
 {
-  ::CoInitializeEx(NULL, COINIT_MULTITHREADED);
+  PThread::Current()->CoInitialise();
+  PTRACE(5, "SAPI-TTS", "Constructed");
 }
 
 
@@ -179,22 +129,24 @@ PBoolean PTextToSpeech_SAPI::OpenFile(const PFilePath & fn)
 {
   Close();
 
-  HRESULT hr = m_cpVoice.CoCreateInstance(CLSID_SpVoice);
-  if (FAILED(hr))
+  PComResult hr = m_cpVoice.CoCreateInstance(CLSID_SpVoice);
+  if (hr.Failed()) {
+    PTRACE(2, "SAPI-TTS", "Could not start SAPI: " << hr);
     return false;
+  }
 
   CSpStreamFormat wavFormat;
   wavFormat.AssignFormat(SPSF_8kHz16BitMono);
 
   PWideString wfn = fn;
   hr = SPBindToFile(wfn, SPFM_CREATE_ALWAYS, &m_cpWavStream, &wavFormat.FormatId(), wavFormat.WaveFormatExPtr()); 
-  if (FAILED(hr)) {
+  if (hr.Failed()) {
     m_cpWavStream.Release();
     return false;
   }
 
   hr = m_cpVoice->SetOutput(m_cpWavStream, true);
-  m_opened = SUCCEEDED(hr);
+  m_opened = hr.Succeeded();
   return m_opened;
 }
 
@@ -228,48 +180,62 @@ PBoolean PTextToSpeech_SAPI::Speak(const PString & text, TextType hint)
       break;
   };
 
-  HRESULT hr = m_cpVoice->Speak(wtext, SPF_DEFAULT, NULL);
-  if (SUCCEEDED(hr))
-    return true;
+  if (m_CurrentVoice != NULL && !m_CurrentVoice.IsEmpty()) {
+    PTRACE(4, "SAPI-TTS", "Trying to set voice \"" << m_CurrentVoice << "\""
+              " of voices: " << setfill(',') << GetVoiceList());
 
-  PTRACE(4, "SAPI\tError speaking text: " << hr);
-  return false;
+    //Enumerate voice tokens with attribute "Name=<specified voice>"
+    CComPtr<IEnumSpObjectTokens> cpEnum;
+    if (PCOM_SUCCEEDED(SpEnumTokens,(SPCAT_VOICES, m_CurrentVoice.AsUCS2(), NULL, &cpEnum))) {
+      //Get the closest token
+      CComPtr<ISpObjectToken> cpVoiceToken;
+      if (PCOM_SUCCEEDED(cpEnum->Next,(1, &cpVoiceToken, NULL))) {
+        //set the voice
+        if (PCOM_SUCCEEDED(m_cpVoice->SetVoice,(cpVoiceToken))) {
+          PTRACE(4, "SAPI-TTS", "SetVoice(" << m_CurrentVoice << ") OK!");
+        }
+      }
+    } 
+  }
+
+  PTRACE(4, "SAPI-TTS", "Speaking...");
+  return PCOM_SUCCEEDED(m_cpVoice->Speak,(wtext, SPF_DEFAULT, NULL));
 }
 
 
 PStringArray PTextToSpeech_SAPI::GetVoiceList()
 {
+  //Enumerate the available voices 
   PStringArray voiceList;
 
-  CComPtr<ISpObjectToken> cpVoiceToken;
   CComPtr<IEnumSpObjectTokens> cpEnum;
   ULONG ulCount = 0;
-
-  //Enumerate the available voices 
-  HRESULT hr = SpEnumTokens(SPCAT_VOICES, NULL, NULL, &cpEnum);
+  PComResult hr;
 
   // Get the number of voices
-  if (SUCCEEDED(hr))
-    hr = cpEnum->GetCount(&ulCount);
+  if (PCOM_SUCCEEDED_EX(hr,SpEnumTokens,(SPCAT_VOICES, NULL, NULL, &cpEnum))) {
+    if (PCOM_SUCCEEDED_EX(hr,cpEnum->GetCount,(&ulCount))) {
+      PTRACE(4, "SAPI-TTS", "Found " << ulCount << " voices..");
 
-  // Obtain a list of available voice tokens, set the voice to the token, and call Speak
-  while (SUCCEEDED(hr) && ulCount--) {
-
-    cpVoiceToken.Release();
-
-    if (SUCCEEDED(hr))
-      hr = cpEnum->Next(1, &cpVoiceToken, NULL );
-
-    if (SUCCEEDED(hr))
-      voiceList.AppendString("voice");
-  } 
+      // Obtain a list of available voice tokens, set the voice to the token, and call Speak
+      while (ulCount-- > 0) {
+        CComPtr<ISpObjectToken> cpVoiceToken;
+        if (hr.Succeeded(cpEnum->Next(1, &cpVoiceToken, NULL))) {
+          voiceList.AppendString("voice");
+          PTRACE(4, "SAPI-TTS", "Found voice:" << cpVoiceToken);
+        }
+      } 
+    }
+  }
 
   return voiceList;
 }
 
-PBoolean PTextToSpeech_SAPI::SetVoice(const PString &)
+
+PBoolean PTextToSpeech_SAPI::SetVoice(const PString & voice)
 {
-  return false;
+  m_CurrentVoice = voice;
+  return true;
 }
 
 PBoolean PTextToSpeech_SAPI::SetRate(unsigned)
@@ -301,7 +267,7 @@ unsigned PTextToSpeech_SAPI::GetVolume()
 #endif // P_SAPI
 
 
-#ifndef _WIN32_WCE
+#if P_PIPECHAN
 
 ////////////////////////////////////////////////////////////
 //
@@ -329,115 +295,164 @@ class PTextToSpeech_Festival : public PTextToSpeech
 
     PBoolean OpenFile(const PFilePath & fn);
     PBoolean OpenChannel(PChannel * channel);
-    PBoolean IsOpen()    { return opened; }
+    PBoolean IsOpen()    { return m_opened; }
 
     PBoolean Close();
     PBoolean Speak(const PString & str, TextType hint);
 
   protected:
-    PBoolean Invoke(const PString & str, const PFilePath & fn);
-
-    PMutex mutex;
-    PBoolean opened;
-    PBoolean usingFile;
-    PString text;
-    PFilePath path;
-    unsigned volume, rate;
+    PMutex    m_mutex;
+    bool      m_opened;
+    PString   m_text;
+    PFilePath m_filePath;
+    unsigned  m_volume;
+    unsigned  m_sampleRate;
 };
 
 #define new PNEW
 
 PFACTORY_CREATE(PFactory<PTextToSpeech>, PTextToSpeech_Festival, "Festival", false);
 
+
 PTextToSpeech_Festival::PTextToSpeech_Festival()
+  : m_opened(false)
+  , m_volume(100)
+  , m_sampleRate(8000)
 {
-  PWaitAndSignal m(mutex);
-  usingFile = opened = false;
-  rate = 8000;
-  volume = 100;
+  PTRACE(5, "Festival-TTS", "Constructed");
 }
 
 
 PTextToSpeech_Festival::~PTextToSpeech_Festival()
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal mutex(m_mutex);
+  PTRACE(5, "Festival-TTS", "Destroyed");
 }
 
+
 PBoolean PTextToSpeech_Festival::OpenChannel(PChannel *)
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal mutex(m_mutex);
 
   Close();
-  usingFile = false;
-  opened = false;
 
-  return true;
+  m_text.MakeEmpty();
+  m_filePath.MakeEmpty();
+
+  return false;
 }
 
 
 PBoolean PTextToSpeech_Festival::OpenFile(const PFilePath & fn)
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal mutex(m_mutex);
 
   Close();
-  usingFile = true;
-  path = fn;
-  opened = true;
 
-  PTRACE(3, "TTS\tWriting speech to " << fn);
+  m_text.MakeEmpty();
+  m_filePath = fn;
+  m_opened = true;
+
+  PTRACE(4, "Festival-TTS", "Writing speech to \"" << fn << '"');
 
   return true;
 }
 
+
 PBoolean PTextToSpeech_Festival::Close()
 {
-  PWaitAndSignal m(mutex);
+  PWaitAndSignal mutex(m_mutex);
 
-  if (!opened)
+  if (!m_opened)
     return true;
 
-  PBoolean stat = false;
+  m_opened = false;
 
-  if (usingFile)
-    stat = Invoke(text, path);
-
-  text = PString();
-
-  opened = false;
-
-  return stat;
-}
-
-
-PBoolean PTextToSpeech_Festival::Speak(const PString & ostr, TextType hint)
-{
-  PWaitAndSignal m(mutex);
-
-  if (!IsOpen()) {
-    PTRACE(2, "TTS\tAttempt to speak whilst engine not open");
+  if (m_filePath.IsEmpty()) {
+    PTRACE(1, "Festival-TTS", "Stream mode not supported (yet)");
     return false;
   }
 
-  PString str = ostr;
+  if (m_text.IsEmpty()) {
+    PTRACE(1, "Festival-TTS", "Nothing spoken");
+    return false;
+  }
+
+  PFile wav;
+  if (!wav.Open(m_filePath, PFile::WriteOnly)) {
+    PTRACE(1, "Festival-TTS", "Could not create WAV file: \"" << m_filePath << '"');
+    return false;
+  }
+
+  PStringStream cmdLine;
+  cmdLine << "text2wave -scale " << std::fixed << m_volume/100.0 << " -F " << m_sampleRate;
+
+  PTRACE(4, "Festival-TTS", "Creating \"" << m_filePath << "\" from \"" << m_text << "\" using \"" << cmdLine << '"');
+  PPipeChannel cmd(cmdLine, PPipeChannel::ReadWrite, true, true);
+
+  cmd << m_text << '\n';
+  if (!cmd.Execute()) { // Flushes stream and sends EOF
+    PTRACE(1, "Festival-TTS", "Festival Generation failed: code=" << cmd.WaitForTermination());
+    wav.Remove();
+    return false;
+  }
+
+  char buf[1000];
+  while (cmd.Read(buf, sizeof(buf))) {
+    if (!wav.Write(buf, cmd.GetLastReadCount())) {
+      PTRACE(1, "Festival-TTS", "Could not write to WAV file: \"" << m_filePath << '"');
+      wav.Remove();
+      return false;
+    }
+  }
+
+  int result = cmd.WaitForTermination();
+
+#if PTRACING
+  PString error;
+  if (cmd.ReadStandardError(error, false)) {
+    PTRACE(2, "Festival-TTS", "Error: \"" << error.Trim() << '"');
+    result = 1;
+  }
+  else if (result != 0) {
+    PTRACE(2, "Festival-TTS", "Error from sub-process: result=" << result);
+  }
+  else {
+    PTRACE(5, "Festival-TTS", "Generation complete: " << wav.GetLength() << " bytes");
+  }
+#endif
+
+  return result == 0;
+}
+
+
+PBoolean PTextToSpeech_Festival::Speak(const PString & str, TextType hint)
+{
+  PWaitAndSignal mutex(m_mutex);
+
+  if (!IsOpen()) {
+    PTRACE(2, "Festival-TTS", "Attempt to speak whilst engine not open");
+    return false;
+  }
+
+  if (m_filePath.IsEmpty()) {
+    PTRACE(1, "Festival-TTS", "Stream mode not supported (yet)");
+    return false;
+  }
+
+  PTRACE(4, "Festival-TTS", "Speaking \"" << str << "\", hint=" << hint);
 
   // do various things to the string, depending upon the hint
   switch (hint) {
     case Digits:
     default:
-    ;
-  };
-
-  if (usingFile) {
-    PTRACE(3, "TTS\tSpeaking " << ostr);
-    text = text & str;
-    return true;
+      m_text = m_text & str;
   }
 
-  PTRACE(1, "TTS\tStream mode not supported for Festival");
-
-  return false;
+  return true;
 }
 
+
 PStringArray PTextToSpeech_Festival::GetVoiceList()
 {
   PStringArray voiceList;
@@ -445,63 +460,40 @@ PStringArray PTextToSpeech_Festival::GetVoiceList()
   return voiceList;
 }
 
+
 PBoolean PTextToSpeech_Festival::SetVoice(const PString & v)
 {
   return v == "default";
 }
 
+
 PBoolean PTextToSpeech_Festival::SetRate(unsigned v)
 {
-  rate = v;
+  m_sampleRate = v;
   return true;
 }
 
+
 unsigned PTextToSpeech_Festival::GetRate()
 {
-  return rate;
+  return m_sampleRate;
 }
 
+
 PBoolean PTextToSpeech_Festival::SetVolume(unsigned v)
 {
-  volume = v;
+  m_volume = v;
   return true;
 }
 
+
 unsigned PTextToSpeech_Festival::GetVolume()
 {
-  return volume;
+  return m_volume;
 }
 
-PBoolean PTextToSpeech_Festival::Invoke(const PString & otext, const PFilePath & fname)
-{
-  PString text = otext;
-  text.Replace('\n', ' ', true);
-  text.Replace('\"', '\'', true);
-  text.Replace('\\', ' ', true);
-  text = "\"" + text + "\"";
 
-  PString cmdLine = "echo " + text + " | ./text2wave -F " + PString(PString::Unsigned, rate) + " -otype riff > " + fname;
-
-  PPipeChannel cmd;
-  if (!cmd.Open(cmdLine, PPipeChannel::ReadWriteStd)) {
-    PTRACE(1, "TTS\tCannot execute command " << cmd);
-    return false;
-  }
-
-  PTRACE(3, "TTS\tCreating " << fname << " using " << cmdLine);
-  cmd.Execute();
-  int code = -1;
-  code = cmd.WaitForTermination();
-  if (code >= 0) {
-    PTRACE(4, "TTS\tdata generated");
-  } else {
-    PTRACE(1, "TTS\tgeneration failed");
-  }
-
-  return code == 0;
-}
-
-#endif // _WIN32_WCE
+#endif // P_PIPECHAN
 
 #endif // P_TTS
 
