@@ -27,9 +27,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23873 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2009-12-13 21:44:25 -0600 (Sun, 13 Dec 2009) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #ifdef __GNUC__
@@ -37,9 +37,12 @@
 #endif
 
 #include <ptlib.h>
-#include <ptlib/pfactory.h>
 #include <ptclib/pwavfile.h>
 
+#if P_WAVFILE
+
+#include <ptlib/pfactory.h>
+
 #define new PNEW
 
 
@@ -49,18 +52,12 @@ const char WAVLabelFMT_[4] = { 'f', 'm', 't', ' ' };
 const char WAVLabelFACT[4] = { 'F', 'A', 'C', 'T' };
 const char WAVLabelDATA[4] = { 'd', 'a', 't', 'a' };
 
-PINSTANTIATE_FACTORY(PWAVFileFormat, unsigned)
-PINSTANTIATE_FACTORY(PWAVFileConverter, unsigned)
 
 inline PBoolean ReadAndCheck(PWAVFile & file, void * buf, PINDEX len)
 {
   return file.FileRead(buf, len) && (file.PFile::GetLastReadCount() == len);
 }
 
-inline PBoolean WriteAndCheck(PWAVFile & file, void * buf, PINDEX len)
-{
-  return file.FileWrite(buf, len) && (file.GetLastWriteCount() == len);
-}
 
 #if PBYTE_ORDER==PBIG_ENDIAN
 #  if defined(USE_SYSTEM_SWAB)
@@ -87,40 +84,20 @@ static void SWAB(const void * void_from, void * void_to, register size_t len)
 // PWAVFile
 
 PWAVFile::PWAVFile(unsigned fmt)
-  : PFile(), origFmt(fmt)
+  : origFmt(fmt)
 {
   Construct();
   SelectFormat(fmt);
 }
 
-PWAVFile * PWAVFile::format(const PString & format)
-{
-  PWAVFile * file = new PWAVFile;
-  file->origFmt = 0xffffffff;
-  file->SelectFormat(format);
-  return file;
-}
-
-PWAVFile::PWAVFile(OpenMode mode, int opts, unsigned fmt)
+PWAVFile::PWAVFile(OpenMode mode, OpenOptions opts, unsigned fmt)
   : PFile(mode, opts), origFmt(fmt)
 {
   Construct();
   SelectFormat(fmt);
 }
 
-PWAVFile * PWAVFile::format(
-  const PString & format,
-  PFile::OpenMode mode,
-  int opts
-)
-{
-  PWAVFile * file = new PWAVFile(mode, opts);
-  file->origFmt = 0xffffffff;
-  file->SelectFormat(format);
-  return file;
-}
-
-PWAVFile::PWAVFile(const PFilePath & name, OpenMode mode, int opts, unsigned fmt)
+PWAVFile::PWAVFile(const PFilePath & name, OpenMode mode, OpenOptions opts, unsigned fmt)
   : origFmt(fmt)
 {
   Construct();
@@ -132,7 +109,7 @@ PWAVFile::PWAVFile(
   const PString & format,  
   const PFilePath & name,  
   OpenMode mode,
-  int opts 
+  OpenOptions opts 
 )
 {
   origFmt = 0xffffffff;
@@ -144,8 +121,9 @@ PWAVFile::PWAVFile(
 PWAVFile::~PWAVFile()
 { 
   Close(); 
-  if (formatHandler != NULL)
-    delete formatHandler;
+
+  delete formatHandler;
+  delete autoConverter;
 }
 
 
@@ -153,48 +131,73 @@ void PWAVFile::Construct()
 {
   lenData                 = 0;
   lenHeader               = 0;
-  isValidWAV              = PFalse;
-  header_needs_updating   = PFalse;
-  autoConvert             = PFalse;
+  isValidWAV              = false;
+  header_needs_updating   = false;
+  autoConvert             = false;
   autoConverter           = NULL;
 
   formatHandler           = NULL;
   wavFmtChunk.hdr.len     = sizeof(wavFmtChunk) - sizeof(wavFmtChunk.hdr);
 }
 
-void PWAVFile::SelectFormat(unsigned fmt)
+
+PWAVFile * PWAVFile::format(const PString & format)
 {
-  if (formatHandler != NULL) {
-    delete formatHandler;
-    formatHandler = NULL;
-  }
-  if (fmt != fmt_NotKnown) {
-    formatHandler       = PWAVFileFormatByIDFactory::CreateInstance(fmt);
-    wavFmtChunk.format  = (WORD)fmt;
-  }
+  PWAVFile * file = new PWAVFile;
+  file->origFmt = 0xffffffff;
+  file->SelectFormat(format);
+  return file;
 }
 
-void PWAVFile::SelectFormat(const PString & format)
+PWAVFile * PWAVFile::format(const PString & format, PFile::OpenMode mode, OpenOptions opts)
 {
-  if (formatHandler != NULL) {
-    delete formatHandler;
-    formatHandler = NULL;
-  }
-  if (!format.IsEmpty())
-    formatHandler = PWAVFileFormatByFormatFactory::CreateInstance(format);
-  if (formatHandler != NULL) {
-    wavFmtChunk.format = (WORD)formatHandler->GetFormat();
-    if (origFmt == 0xffffffff)
-      origFmt = wavFmtChunk.format;
-  }
+  PWAVFile * file = new PWAVFile(mode, opts);
+  file->origFmt = 0xffffffff;
+  file->SelectFormat(format);
+  return file;
 }
 
-PBoolean PWAVFile::Open(OpenMode mode, int opts)
+
+bool PWAVFile::SelectFormat(unsigned fmt)
+{
+  delete formatHandler;
+  formatHandler = NULL;
+
+  if (fmt == fmt_NotKnown)
+    return true;
+
+  formatHandler = PWAVFileFormatByIDFactory::CreateInstance(fmt);
+  if (formatHandler == NULL)
+    return false;
+
+  wavFmtChunk.format  = (WORD)fmt;
+  return true;
+}
+
+bool PWAVFile::SelectFormat(const PString & format)
+{
+  delete formatHandler;
+  formatHandler = NULL;
+
+  if (format.IsEmpty())
+    return false;
+
+  formatHandler = PWAVFileFormatByFormatFactory::CreateInstance(format);
+  if (formatHandler == NULL)
+    return SelectFormat(format.AsUnsigned());
+
+  wavFmtChunk.format = (WORD)formatHandler->GetFormat();
+  if (origFmt == 0xffffffff)
+    origFmt = wavFmtChunk.format;
+  return true;
+}
+
+PBoolean PWAVFile::Open(OpenMode mode, OpenOptions opts)
 {
   if (!(PFile::Open(mode, opts)))
-    return PFalse;
+    return false;
 
-  isValidWAV = PFalse;
+  isValidWAV = false;
 
   // Try and process the WAV file header information.
   // Either ProcessHeader() or GenerateHeader() must be called.
@@ -218,7 +221,7 @@ PBoolean PWAVFile::Open(OpenMode mode, int opts)
       GenerateHeader();
     }
     if (mode == ReadOnly) {
-      isValidWAV = PFalse; // ReadOnly on a zero length file
+      isValidWAV = false; // ReadOnly on a zero length file
     }
   }
 
@@ -226,14 +229,14 @@ PBoolean PWAVFile::Open(OpenMode mode, int opts)
   if (formatHandler == NULL) {
     Close();
     SetErrorValues(BadParameter, EINVAL);
-    return PFalse;
+    return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
-PBoolean PWAVFile::Open(const PFilePath & name, OpenMode mode, int opts)
+PBoolean PWAVFile::Open(const PFilePath & name, OpenMode mode, OpenOptions opts)
 {
   if (IsOpen())
     Close();
@@ -244,10 +247,11 @@ PBoolean PWAVFile::Open(const PFilePath & name, OpenMode mode, int opts)
 
 PBoolean PWAVFile::Close()
 {
-  autoConverter = NULL; // This is a pointer to a singleton, no need to delete
+  delete autoConverter;
+  autoConverter = NULL;
 
   if (!PFile::IsOpen())
-    return PTrue;
+    return true;
 
   if (header_needs_updating)
     UpdateHeader();
@@ -257,6 +261,7 @@ PBoolean PWAVFile::Close()
 
   delete formatHandler;
   formatHandler = NULL;
+
   if (origFmt != 0xffffffff)
     SelectFormat(origFmt);
 
@@ -265,15 +270,15 @@ PBoolean PWAVFile::Close()
 
 void PWAVFile::SetAutoconvert()
 { 
-  autoConvert = PTrue; 
+  autoConvert = true; 
 }
 
 
 // Performs necessary byte-order swapping on for big-endian platforms.
 PBoolean PWAVFile::Read(void * buf, PINDEX len)
 {
-	if (!IsOpen())
-		return PFalse;
+  if (!IsOpen())
+    return false;
 
   if (autoConverter != NULL)
     return autoConverter->Read(*this, buf, len);
@@ -288,10 +293,15 @@ PBoolean PWAVFile::RawRead(void * buf, PINDEX len)
   // We do not want to return this data by mistake.
   PINDEX readlen = len;
   off_t pos = PFile::GetPosition();
-  if (pos >= (lenHeader+lenData))
-    return PFalse;
-  
-  if ((pos + len) > (lenHeader+lenData))
+
+  // indicate eof (return false, but error=0, lastReadCount=0)
+  if (pos >= (lenHeader + lenData)) {
+    lastReadCount = 0;
+    ConvertOSError(0, LastReadError);
+    return false;
+  }
+
+  if ((pos + len) > (lenHeader + lenData))
     readlen = (lenHeader+lenData) - pos;
 
   if (formatHandler != NULL)
@@ -308,11 +318,11 @@ PBoolean PWAVFile::FileRead(void * buf, PINDEX len)
 // Performs necessary byte-order swapping on for big-endian platforms.
 PBoolean PWAVFile::Write(const void * buf, PINDEX len)
 {
-	if (!IsOpen())
-		return PFalse;
+  if (!IsOpen())
+    return false;
 
   // Needs to update header on close.
-  header_needs_updating = PTrue;
+  header_needs_updating = true;
 
   if (autoConverter != NULL)
     return autoConverter->Write(*this, buf, len);
@@ -323,7 +333,7 @@ PBoolean PWAVFile::Write(const void * buf, PINDEX len)
 PBoolean PWAVFile::RawWrite(const void * buf, PINDEX len)
 {
   // Needs to update header on close.
-  header_needs_updating = PTrue;
+  header_needs_updating = true;
 
   if (formatHandler != NULL)
     return formatHandler->Write(*this, buf, len);
@@ -406,13 +416,12 @@ unsigned PWAVFile::GetChannels() const
 
 void PWAVFile::SetChannels(unsigned v) 
 {
-  wavFmtChunk.numChannels = (WORD)v;
-  if (wavFmtChunk.numChannels == 1 || wavFmtChunk.numChannels == 2)
-  {
-      wavFmtChunk.bytesPerSample = (wavFmtChunk.bitsPerSample/8) * wavFmtChunk.numChannels;
-      wavFmtChunk.bytesPerSec = wavFmtChunk.sampleRate * wavFmtChunk.bytesPerSample;
+  if (formatHandler == NULL || formatHandler->CanSetChannels(v)) {
+    wavFmtChunk.numChannels = (WORD)v;
+    wavFmtChunk.bytesPerSample = (wavFmtChunk.bitsPerSample/8) * wavFmtChunk.numChannels;
+    wavFmtChunk.bytesPerSec = wavFmtChunk.sampleRate * wavFmtChunk.bytesPerSample;
+    header_needs_updating = true;
   }
-  header_needs_updating = PTrue;
 }
 
 unsigned PWAVFile::GetSampleRate() const
@@ -426,7 +435,7 @@ unsigned PWAVFile::GetSampleRate() const
 void PWAVFile::SetSampleRate(unsigned v) 
 {
   wavFmtChunk.sampleRate = (WORD)v;
-  header_needs_updating = PTrue;
+  header_needs_updating = true;
 }
 
 unsigned PWAVFile::GetSampleSize() const
@@ -440,7 +449,7 @@ unsigned PWAVFile::GetSampleSize() const
 void PWAVFile::SetSampleSize(unsigned v) 
 {
   wavFmtChunk.bitsPerSample = (WORD)v;
-  header_needs_updating = PTrue;
+  header_needs_updating = true;
 }
 
 unsigned PWAVFile::GetBytesPerSecond() const
@@ -454,7 +463,7 @@ unsigned PWAVFile::GetBytesPerSecond() const
 void PWAVFile::SetBytesPerSecond(unsigned v)
 {
   wavFmtChunk.bytesPerSec = (WORD)v;
-  header_needs_updating = PTrue;
+  header_needs_updating = true;
 }
 
 off_t PWAVFile::GetHeaderLength() const
@@ -489,21 +498,19 @@ off_t PWAVFile::RawGetDataLength()
 PBoolean PWAVFile::SetFormat(unsigned fmt)
 {
   if (IsOpen() || isValidWAV)
-    return PFalse;
+    return false;
 
   SelectFormat(fmt);
 
-  return PTrue;
+  return true;
 }
 
 PBoolean PWAVFile::SetFormat(const PString & format)
 {
   if (IsOpen() || isValidWAV)
-    return PFalse;
+    return false;
 
-  SelectFormat(format);
-
-  return PTrue;
+  return SelectFormat(format);
 }
 
 static inline PBoolean NeedsConverter(const PWAV::FMTChunk & fmtChunk)
@@ -513,7 +520,8 @@ static inline PBoolean NeedsConverter(const PWAV::FMTChunk & fmtChunk)
 
 PBoolean PWAVFile::ProcessHeader() 
 {
-  autoConverter = NULL; // This is a pointer to a singleton, no need to delete
+  delete autoConverter;
+  autoConverter = NULL;
 
   // Process the header information
   // This comes in 3 or 4 chunks, either RIFF, FORMAT and DATA
@@ -521,39 +529,39 @@ PBoolean PWAVFile::ProcessHeader()
 
   if (!IsOpen()) {
     PTRACE(1,"WAV\tProcessHeader: Not Open");
-    return (PFalse);
+    return (false);
   }
 
   // go to the beginning of the file
   if (!PFile::SetPosition(0)) {
     PTRACE(1,"WAV\tProcessHeader: Cannot Set Pos");
-    return (PFalse);
+    return (false);
   }
 
   // Read the RIFF chunk.
   struct PWAV::RIFFChunkHeader riffChunk;
   if (!ReadAndCheck(*this, &riffChunk, sizeof(riffChunk)))
-    return PFalse;
+    return false;
 
   // check if tags are correct
   if (strncmp(riffChunk.hdr.tag, WAVLabelRIFF, sizeof(WAVLabelRIFF)) != 0) {
     PTRACE(1,"WAV\tProcessHeader: Not RIFF");
-    return (PFalse);
+    return (false);
   }
 
   if (strncmp(riffChunk.tag, WAVLabelWAVE, sizeof(WAVLabelWAVE)) != 0) {
     PTRACE(1,"WAV\tProcessHeader: Not WAVE");
-    return (PFalse);
+    return (false);
   }
 
   // Read the known part of the FORMAT chunk.
   if (!ReadAndCheck(*this, &wavFmtChunk, sizeof(wavFmtChunk)))
-    return PFalse;
+    return false;
 
   // check if labels are correct
   if (strncmp(wavFmtChunk.hdr.tag, WAVLabelFMT_, sizeof(WAVLabelFMT_)) != 0) {
     PTRACE(1,"WAV\tProcessHeader: Not FMT");
-    return (PFalse);
+    return (false);
   }
 
   // if we opened the file without knowing the format, then try and set the format now
@@ -561,7 +569,7 @@ PBoolean PWAVFile::ProcessHeader()
     SelectFormat(wavFmtChunk.format);
     if (formatHandler == NULL) {
       Close();
-      return PFalse;
+      return false;
     }
   }
 
@@ -570,24 +578,24 @@ PBoolean PWAVFile::ProcessHeader()
   if ((unsigned)wavFmtChunk.hdr.len > (sizeof(wavFmtChunk) - sizeof(wavFmtChunk.hdr))) {
     extendedHeader.SetSize(wavFmtChunk.hdr.len - (sizeof(wavFmtChunk) - sizeof(wavFmtChunk.hdr)));
     if (!ReadAndCheck(*this, extendedHeader.GetPointer(), extendedHeader.GetSize()))
-      return PFalse;
+      return false;
   }
 
   // give format handler a chance to read extra chunks
   if (!formatHandler->ReadExtraChunks(*this))
-    return PFalse;
+    return false;
 
   PWAV::ChunkHeader chunkHeader;
 
   // ignore chunks until we see a DATA chunk
   for (;;) {
     if (!ReadAndCheck(*this, &chunkHeader, sizeof(chunkHeader)))
-      return PFalse;
+      return false;
     if (strncmp(chunkHeader.tag, WAVLabelDATA, sizeof(WAVLabelDATA)) == 0) 
       break;
     if (!PFile::SetPosition(PFile::GetPosition() + + chunkHeader.len)) {
       PTRACE(1,"WAV\tProcessHeader: Cannot set new position");
-      return PFalse;
+      return false;
     }
   }
 
@@ -603,7 +611,7 @@ PBoolean PWAVFile::ProcessHeader()
 
   formatHandler->OnStart();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -613,15 +621,16 @@ PBoolean PWAVFile::ProcessHeader()
 // b) G.723.1 data
 // When this function is called with lenData < 0, it will write the header
 // as if the lenData is LONG_MAX minus header length.
-// Note: If it returns PFalse, the file may be left in inconsistent state.
+// Note: If it returns false, the file may be left in inconsistent state.
 
 PBoolean PWAVFile::GenerateHeader()
 {
-  autoConverter = NULL; // This is a pointer to a singleton, no need to delete
+  delete autoConverter;
+  autoConverter = NULL;
 
   if (!IsOpen()) {
     PTRACE(1, "WAV\tGenerateHeader: Not Open");
-    return (PFalse);
+    return (false);
   }
 
   // length of audio data is set to a large value if lenData does not
@@ -630,7 +639,7 @@ PBoolean PWAVFile::GenerateHeader()
   int audioDataLen;
   if (lenData < 0) {
     audioDataLen = LONG_MAX - wavFmtChunk.hdr.len;
-    header_needs_updating = PTrue;
+    header_needs_updating = true;
   } else {
     audioDataLen = lenData;
   }
@@ -638,7 +647,7 @@ PBoolean PWAVFile::GenerateHeader()
   // go to the beginning of the file
   if (!PFile::SetPosition(0)) {
     PTRACE(1,"WAV\tGenerateHeader: Cannot Set Pos");
-    return (PFalse);
+    return (false);
   }
 
   // write the WAV file header
@@ -647,8 +656,8 @@ PBoolean PWAVFile::GenerateHeader()
   memcpy(riffChunk.tag,     WAVLabelWAVE, sizeof(WAVLabelWAVE));
   riffChunk.hdr.len = lenHeader + audioDataLen - sizeof(riffChunk.hdr);
 
-  if (!WriteAndCheck(*this, &riffChunk, sizeof(riffChunk)))
-    return PFalse;
+  if (!FileWrite(&riffChunk, sizeof(riffChunk)))
+    return false;
 
   // populate and write the WAV header with the default data
   memcpy(wavFmtChunk.hdr.tag,  WAVLabelFMT_, sizeof(WAVLabelFMT_));
@@ -657,29 +666,29 @@ PBoolean PWAVFile::GenerateHeader()
   // allow the format handler to modify the header and extra bytes
   if(formatHandler == NULL){
     PTRACE(1,"WAV\tGenerateHeader: format handler is null!");
-    return PFalse;
+    return false;
   }
   formatHandler->CreateHeader(wavFmtChunk, extendedHeader);
 
   // write the basic WAV header
-  if (
-      !WriteAndCheck(*this, &wavFmtChunk, sizeof(wavFmtChunk)) ||
-      ((extendedHeader.GetSize() > 0) && !WriteAndCheck(*this, extendedHeader.GetPointer(), extendedHeader.GetSize()))
-     )
-    return PFalse;
+  if (!FileWrite(&wavFmtChunk, sizeof(wavFmtChunk)))
+    return false;
+
+  if (extendedHeader.GetSize() > 0 && !FileWrite(extendedHeader.GetPointer(), extendedHeader.GetSize()))
+    return false;
 
   // allow the format handler to write additional chunks
   if (!formatHandler->WriteExtraChunks(*this))
-    return PFalse;
+    return false;
 
   // Write the DATA chunk.
   PWAV::ChunkHeader dataChunk;
   memcpy(dataChunk.tag, WAVLabelDATA, sizeof(WAVLabelDATA));
   dataChunk.len = audioDataLen;
-  if (!WriteAndCheck(*this, &dataChunk, sizeof(dataChunk)))
-    return PFalse;
+  if (!FileWrite(&dataChunk, sizeof(dataChunk)))
+    return false;
 
-  isValidWAV = PTrue;
+  isValidWAV = true;
 
   // get the length of the header
   lenHeader = PFile::GetPosition();
@@ -689,11 +698,11 @@ PBoolean PWAVFile::GenerateHeader()
     autoConverter = PWAVFileConverterFactory::CreateInstance(wavFmtChunk.format);
     if (autoConverter == NULL) {
       PTRACE(1, "PWAVFile\tNo format converter for type " << (int)wavFmtChunk.format);
-      return PFalse;
+      return false;
     }
   }
 
-  return (PTrue);
+  return (true);
 }
 
 // Update the WAV header according to the file length
@@ -702,13 +711,13 @@ PBoolean PWAVFile::UpdateHeader()
   // Check file is still open
   if (!IsOpen()) {
     PTRACE(1,"WAV\tUpdateHeader: Not Open");
-    return (PFalse);
+    return (false);
   }
 
   // Check there is already a valid header
   if (!isValidWAV) {
     PTRACE(1,"WAV\tUpdateHeader: File not valid");
-    return (PFalse);
+    return (false);
   }
 
   // Find out the length of the audio data
@@ -717,31 +726,31 @@ PBoolean PWAVFile::UpdateHeader()
   // rewrite the length in the RIFF chunk
   PInt32l riffChunkLen = (lenHeader - 8) + lenData; // size does not include first 8 bytes
   PFile::SetPosition(4);
-  if (!WriteAndCheck(*this, &riffChunkLen, sizeof(riffChunkLen)))
-    return PFalse;
+  if (!FileWrite(&riffChunkLen, sizeof(riffChunkLen)))
+    return false;
 
   // rewrite the data length field in the data chunk
   PInt32l dataChunkLen = lenData;
   PFile::SetPosition(lenHeader - 4);
-  if (!WriteAndCheck(*this, &dataChunkLen, sizeof(dataChunkLen)))
-    return PFalse;
+  if (!FileWrite(&dataChunkLen, sizeof(dataChunkLen)))
+    return false;
 
   if(formatHandler == NULL){
     PTRACE(1,"WAV\tGenerateHeader: format handler is null!");
-    return PFalse;
+    return false;
   }
   formatHandler->UpdateHeader(wavFmtChunk, extendedHeader);
 
   PFile::SetPosition(12);
-  if (!WriteAndCheck(*this, &wavFmtChunk, sizeof(wavFmtChunk)))
-    return PFalse;
+  if (!FileWrite(&wavFmtChunk, sizeof(wavFmtChunk)))
+    return false;
 
-  if (!WriteAndCheck(*this, extendedHeader.GetPointer(), extendedHeader.GetSize()))
-    return PFalse;
+  if (!FileWrite(extendedHeader.GetPointer(), extendedHeader.GetSize()))
+    return false;
 
-  header_needs_updating = PFalse;
+  header_needs_updating = false;
 
-  return PTrue;
+  return true;
 }
 
 
@@ -750,19 +759,19 @@ PBoolean PWAVFile::UpdateHeader()
 PBoolean PWAVFileFormat::Read(PWAVFile & file, void * buf, PINDEX & len)
 { 
   if (!file.FileRead(buf, len))
-    return PFalse;
+    return false;
 
   len = file.GetLastReadCount();
-  return PTrue;
+  return true;
 }
 
 PBoolean PWAVFileFormat::Write(PWAVFile & file, const void * buf, PINDEX & len)
 { 
   if (!file.FileWrite(buf, len))
-    return PFalse;
+    return false;
 
   len = file.GetLastWriteCount();
-  return PTrue;
+  return true;
 }
 
 //////////////////////////////////////////////////////////////////
@@ -776,6 +785,9 @@ class PWAVFileFormatPCM : public PWAVFileFormat
     unsigned GetFormat() const
     { return PWAVFile::fmt_PCM; }
 
+    bool CanSetChannels(unsigned channels) const
+    { return channels > 0 && channels < 7; }
+
     PString GetDescription() const
     { return "PCM"; }
 
@@ -786,7 +798,7 @@ class PWAVFileFormatPCM : public PWAVFileFormat
     PBoolean Write(PWAVFile & file, const void * buf, PINDEX & len);
 };
 
-PWAVFileFormatByIDFactory::Worker<PWAVFileFormatPCM> pcmIDWAVFormat(PWAVFile::fmt_PCM);
+PFACTORY_CREATE(PWAVFileFormatByIDFactory, PWAVFileFormatPCM, PWAVFile::fmt_PCM, false);
 PWAVFileFormatByFormatFactory::Worker<PWAVFileFormatPCM> pcmFormatWAVFormat("PCM-16");
 
 void PWAVFileFormatPCM::CreateHeader(PWAV::FMTChunk & wavFmtChunk, 
@@ -811,18 +823,18 @@ void PWAVFileFormatPCM::UpdateHeader(PWAV::FMTChunk & wavFmtChunk,
 PBoolean PWAVFileFormatPCM::Read(PWAVFile & file, void * buf, PINDEX & len)
 {
   if (!file.FileRead(buf, len))
-    return PFalse;
+    return false;
 
   len = file.GetLastReadCount();
 
   // WAV files are little-endian. So swap the bytes if this is
   // a big endian machine and we have 16 bit samples
   // Note: swab only works on even length buffers.
-  if (file.wavFmtChunk.bitsPerSample == 16) {
+  if (file.GetSampleSize() == 16) {
     SWAB(buf, buf, len);
   }
 
-  return PTrue;
+  return true;
 }
 
 PBoolean PWAVFileFormatPCM::Write(PWAVFile & file, const void * buf, PINDEX & len)
@@ -830,41 +842,31 @@ PBoolean PWAVFileFormatPCM::Write(PWAVFile & file, const void * buf, PINDEX & le
   // WAV files are little-endian. So swap the bytes if this is
   // a big endian machine and we have 16 bit samples
   // Note: swab only works on even length buffers.
-  if (file.wavFmtChunk.bitsPerSample == 16) {
+  if (file.GetSampleSize() == 16) {
     SWAB(buf, (void *)buf, len);
   }
 
   if (!file.FileWrite(buf, len))
-    return PFalse;
+    return false;
 
   len = file.GetLastWriteCount();
-  return PTrue;
+  return true;
 }
 
 //////////////////////////////////////////////////////////////////
 
-#ifdef __GNUC__
-#define P_PACKED    __attribute__ ((packed));
-#else
-#define P_PACKED
-#pragma pack(1)
-#endif
-
-struct G7231ExtendedInfo {
-  PInt16l data1      P_PACKED;      // 1
-  PInt16l data2      P_PACKED;      // 480
+struct G7231ExtendedInfo
+{
+  P_PACK_FIELD(PInt16l data1);      // 1
+  P_PACK_FIELD(PInt16l data2);      // 480
 };
 
-struct G7231FACTChunk {
+struct G7231FACTChunk
+{
   PWAV::ChunkHeader hdr;
-  PInt32l data1      P_PACKED;      // 0   Should be number of samples.
+  P_PACK_FIELD(PInt32l data1);      // 0   Should be number of samples.
 };
 
-#ifdef __GNUC__
-#undef P_PACKED
-#else
-#pragma pack()
-#endif
 
 
 class PWAVFileFormatG7231 : public PWAVFileFormat
@@ -878,6 +880,9 @@ class PWAVFileFormatG7231 : public PWAVFileFormat
     void CreateHeader(PWAV::FMTChunk & wavFmtChunk, PBYTEArray & extendedHeader);
     PBoolean WriteExtraChunks(PWAVFile & file);
 
+    bool CanSetChannels(unsigned channels) const
+    { return channels == 1; }
+
     PString GetFormatString() const
     { return "G.723.1"; }   // must match string in mediafmt.h
 
@@ -938,7 +943,7 @@ PBoolean PWAVFileFormatG7231::Read(PWAVFile & file, void * origData, PINDEX & or
     // keep reading until we find a 20 or 24 byte frame
     while (cachePos == cacheLen) {
       if (!file.FileRead(cacheBuffer, 24))
-        return PFalse;
+        return false;
 
       // calculate actual length of frame
       PINDEX frameLen = G7231FrameSizes[cacheBuffer[0] & 3];
@@ -958,7 +963,7 @@ PBoolean PWAVFileFormatG7231::Read(PWAVFile & file, void * origData, PINDEX & or
 
   origLen = bytesRead;
 
-  return PTrue;
+  return true;
 }
 
 PBoolean PWAVFileFormatG7231::Write(PWAVFile & file, const void * origData, PINDEX & len)
@@ -974,7 +979,7 @@ PBoolean PWAVFileFormatG7231::Write(PWAVFile & file, const void * origData, PIND
     // calculate actual length of frame
     PINDEX frameLen = G7231FrameSizes[(*(char *)origData) & 3];
     if (len < frameLen)
-      return PFalse;
+      return false;
 
     // we can write 24 byte frame straight out, 
     // 20 byte frames need to be reblocked
@@ -994,7 +999,7 @@ PBoolean PWAVFileFormatG7231::Write(PWAVFile & file, const void * origData, PIND
     }
 
     if (buf != NULL && !file.FileWrite(buf, 24))
-      return PFalse;
+      return false;
     else
       written += 24;
 
@@ -1004,7 +1009,7 @@ PBoolean PWAVFileFormatG7231::Write(PWAVFile & file, const void * origData, PIND
 
   len = written;
 
-  return PTrue;
+  return true;
 }
 
 class PWAVFileFormatG7231_vivo : public PWAVFileFormatG7231
@@ -1080,19 +1085,19 @@ off_t PWAVFileConverterPCM::GetDataLength(PWAVFile & file)
 
 PBoolean PWAVFileConverterPCM::Read(PWAVFile & file, void * buf, PINDEX len)
 {
-  if (file.wavFmtChunk.bitsPerSample == 16)
+  if (file.GetSampleSize() == 16)
     return file.PWAVFile::RawRead(buf, len);
 
-  if (file.wavFmtChunk.bitsPerSample != 8) {
-    PTRACE(1, "PWAVFile\tAttempt to read autoconvert PCM data with unsupported number of bits per sample " << (int)file.wavFmtChunk.bitsPerSample);
-    return PFalse;
+  if (file.GetSampleSize() != 8) {
+    PTRACE(1, "PWAVFile\tAttempt to read autoconvert PCM data with unsupported number of bits per sample " << file.GetSampleSize());
+    return false;
   }
 
   // read the PCM data with 8 bits per sample
   PINDEX samples = (len / 2);
   PBYTEArray pcm8;
   if (!file.PWAVFile::RawRead(pcm8.GetPointer(samples), samples))
-    return PFalse;
+    return false;
 
   // convert to PCM-16
   PINDEX i;
@@ -1103,19 +1108,22 @@ PBoolean PWAVFileConverterPCM::Read(PWAVFile & file, void * buf, PINDEX len)
   // fake the lastReadCount
   file.SetLastReadCount(len);
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PWAVFileConverterPCM::Write(PWAVFile & file, const void * buf, PINDEX len)
 {
-  if (file.wavFmtChunk.bitsPerSample == 16)
+  if (file.GetSampleSize() == 16)
     return file.PWAVFile::RawWrite(buf, len);
 
-  PTRACE(1, "PWAVFile\tAttempt to write autoconvert PCM data with unsupported number of bits per sample " << (int)file.wavFmtChunk.bitsPerSample);
-  return PFalse;
+  PTRACE(1, "PWAVFile\tAttempt to write autoconvert PCM data with unsupported number of bits per sample " << file.GetSampleSize());
+  return false;
 }
 
-PWAVFileConverterFactory::Worker<PWAVFileConverterPCM> pcmConverter(PWAVFile::fmt_PCM, true);
+PWAVFileConverterFactory::Worker<PWAVFileConverterPCM> pcmConverter(PWAVFile::fmt_PCM);
+
+
+#endif // P_WAVFILE
 
 //////////////////////////////////////////////////////////////////
