@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22203 $
+ * $Revision: 28974 $
  * $Author: rjongbloed $
- * $Date: 2009-03-17 05:23:12 -0500 (Tue, 17 Mar 2009) $
+ * $Date: 2013-01-23 19:55:23 -0600 (Wed, 23 Jan 2013) $
  */
 
 #ifdef __GNUC__
@@ -38,6 +38,7 @@
 #include <ptlib.h>
 #include <ptclib/pwavfiledev.h>
 
+#if P_WAVFILE
 
 class PSoundChannel_WAVFile_PluginServiceDescriptor : public PDevicePluginServiceDescriptor
 {
@@ -52,19 +53,28 @@ class PSoundChannel_WAVFile_PluginServiceDescriptor : public PDevicePluginServic
     }
     virtual bool ValidateDeviceName(const PString & deviceName, int userData) const
     {
-      PCaselessString adjustedDevice = deviceName;
-      PINDEX length = adjustedDevice.GetLength();
-      if (userData == PSoundChannel::Recorder && length > 5 && adjustedDevice.NumCompare(".wav*", 5, length-5) == PObject::EqualTo)
-        adjustedDevice.Delete(length-1, 1);
-      else if (length < 5 || adjustedDevice.NumCompare(".wav", 4, length-4) != PObject::EqualTo)
+      PFilePath pathname = deviceName;
+      if (pathname.GetTitle().IsEmpty())
         return false;
 
-      return PFile::Access(adjustedDevice, userData == PSoundChannel::Recorder ? PFile::ReadOnly : PFile::WriteOnly);
+      PINDEX last = pathname.GetLength()-1;
+      if (userData == PSoundChannel::Recorder && pathname[last] == '*')
+        pathname.Delete(last, 1);
+
+      if (pathname.GetType() != ".wav")
+        return false;
+
+      if (userData == PSoundChannel::Recorder)
+        return PFile::Access(pathname, PFile::ReadOnly);
+
+      if (PFile::Exists(pathname))
+        return PFile::Access(pathname, PFile::WriteOnly);
+
+      return PFile::Access(pathname.GetDirectory(), PFile::WriteOnly);
     }
 } PSoundChannel_WAVFile_descriptor;
 
 PCREATE_PLUGIN(WAVFile, PSoundChannel, &PSoundChannel_WAVFile_descriptor);
-PINSTANTIATE_FACTORY(PSoundChannel, WAVFile)
 
 
 #define new PNEW
@@ -73,9 +83,11 @@ PINSTANTIATE_FACTORY(PSoundChannel, WAVFile)
 ///////////////////////////////////////////////////////////////////////////////
 
 PSoundChannel_WAVFile::PSoundChannel_WAVFile()
-  : m_autoRepeat(false)
+  : m_Pacing(1000)
+  , m_autoRepeat(false)
   , m_sampleRate(8000)
   , m_bufferSize(2)
+  , m_samplePosition(P_MAX_INDEX)
 {
 }
 
@@ -166,7 +178,7 @@ PBoolean PSoundChannel_WAVFile::SetFormat(unsigned numChannels,
   m_WAVFile.SetSampleRate(sampleRate);
   m_WAVFile.SetSampleSize(bitsPerSample);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -195,7 +207,7 @@ PBoolean PSoundChannel_WAVFile::Close()
 
   m_WAVFile.Close();
   os_handle = -1;
-  return PTrue;
+  return true;
 }
 
 
@@ -225,19 +237,19 @@ PBoolean PSoundChannel_WAVFile::Write(const void * data, PINDEX size)
 
 PBoolean PSoundChannel_WAVFile::HasPlayCompleted()
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannel_WAVFile::WaitForPlayCompletion()
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannel_WAVFile::StartRecording()
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -255,7 +267,7 @@ PBoolean PSoundChannel_WAVFile::Read(void * data, PINDEX size)
       iDutyCycle += wavSampleRate;
       if (iDutyCycle >= m_sampleRate) {
         iDutyCycle -= m_sampleRate;
-        if (!ReadSamples(&iSample, sizeof(short)))
+        if (!ReadSample(iSample))
           return false;
       }
       *pPCM++ = iSample;
@@ -269,7 +281,7 @@ PBoolean PSoundChannel_WAVFile::Read(void * data, PINDEX size)
     short * pPCM = (short *)data;
     for (PINDEX count = 0; count < size; count += sizeof(short)) {
       do {
-        if (!ReadSamples(&iSample, sizeof(short)))
+        if (!ReadSample(iSample))
           return false;
         iDutyCycle += m_sampleRate;
       } while (iDutyCycle < wavSampleRate);
@@ -289,9 +301,23 @@ PBoolean PSoundChannel_WAVFile::Read(void * data, PINDEX size)
 }
 
 
+bool PSoundChannel_WAVFile::ReadSample(short & sample)
+{
+  if (m_samplePosition >= m_sampleBuffer.GetSize()) {
+    static const PINDEX BufferSize = 10000;
+    if (!ReadSamples(m_sampleBuffer.GetPointer(BufferSize), BufferSize*sizeof(short)))
+      return false;
+    m_sampleBuffer.SetSize(m_WAVFile.GetLastReadCount()/sizeof(short));
+    m_samplePosition = 0;
+  }
+  sample = m_sampleBuffer[m_samplePosition++];
+  return true;
+}
+
+
 bool PSoundChannel_WAVFile::ReadSamples(void * data, PINDEX size)
 {
-  if (m_WAVFile.Read(data, size) && m_WAVFile.GetLastReadCount() >= size)
+  if (m_WAVFile.Read(data, size))
     return true;
 
   if (!m_autoRepeat)
@@ -304,27 +330,31 @@ bool PSoundChannel_WAVFile::ReadSamples(void * data, PINDEX size)
 
 PBoolean PSoundChannel_WAVFile::IsRecordBufferFull()
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannel_WAVFile::AreAllRecordBuffersFull()
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannel_WAVFile::WaitForRecordBufferFull()
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannel_WAVFile::WaitForAllRecordBuffersFull()
 {
-  return PTrue;
+  return true;
 }
 
 
+#endif // P_WAVFILE
+
+
 // End of File ///////////////////////////////////////////////////////////////
 
+
