@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 29415 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2013-04-02 16:41:13 -0500 (Tue, 02 Apr 2013) $
  */
 
 #ifdef __GNUC__
@@ -80,14 +80,14 @@ PBoolean PQueueChannel::Open(PINDEX size)
     unfull.Signal();
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PQueueChannel::Close()
 {
   if (!IsOpen())
-    return PFalse;
+    return false;
 
   mutex.Wait();
   if (queueBuffer != NULL)
@@ -97,7 +97,7 @@ PBoolean PQueueChannel::Close()
   mutex.Signal();
   unempty.Signal();
   unfull.Signal();
-  return PTrue;
+  return true;
 }
 
 
@@ -109,7 +109,7 @@ PBoolean PQueueChannel::Read(void * buf, PINDEX count)
 
   if (!IsOpen()) {
     mutex.Signal();
-    return PFalse;
+    return false;
   }
 
   BYTE * buffer = (BYTE *)buf;
@@ -126,7 +126,7 @@ PBoolean PQueueChannel::Read(void * buf, PINDEX count)
     PTRACE_IF(6, readTimeout > 0, "QChan\tBlocking on empty queue");
     if (!unempty.Wait(readTimeout)) {
       PTRACE(6, "QChan\tRead timeout on empty queue");
-      return SetErrorValues(Timeout, EAGAIN, LastReadError);
+      return SetErrorValues(Timeout, ETIMEDOUT, LastReadError);
     }
 
     // relock the data
@@ -181,7 +181,7 @@ PBoolean PQueueChannel::Read(void * buf, PINDEX count)
   // unlock the buffer
   mutex.Signal();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -193,7 +193,7 @@ PBoolean PQueueChannel::Write(const void * buf, PINDEX count)
 
   if (!IsOpen()) {
     mutex.Signal();
-    return PFalse;
+    return false;
   }
 
   const BYTE * buffer = (BYTE *)buf;
@@ -207,7 +207,7 @@ PBoolean PQueueChannel::Write(const void * buf, PINDEX count)
     PTRACE_IF(6, writeTimeout > 0, "QChan\tBlocking on full queue");
     if (!unfull.Wait(writeTimeout)) {
       PTRACE(6, "QChan\tWrite timeout on full queue");
-      return SetErrorValues(Timeout, EAGAIN, LastWriteError);
+      return SetErrorValues(Timeout, ETIMEDOUT, LastWriteError);
     }
 
     mutex.Wait();
@@ -259,7 +259,7 @@ PBoolean PQueueChannel::Write(const void * buf, PINDEX count)
 
   mutex.Signal();
 
-  return PTrue;
+  return true;
 }
 
 
