@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23298 $
+ * $Revision: 29306 $
  * $Author: rjongbloed $
- * $Date: 2009-08-28 03:53:17 -0500 (Fri, 28 Aug 2009) $
+ * $Date: 2013-03-24 23:14:19 -0500 (Sun, 24 Mar 2013) $
  */
 
 #ifdef __GNUC__
@@ -63,21 +63,27 @@ class HTTP_PSSLChannel : public PSSLChannel
 
 PSecureHTTPServiceProcess::PSecureHTTPServiceProcess(const Info & inf)
   : PHTTPServiceProcess(inf)
+  , m_sslContext(NULL)
 {
-  sslContext = new PSSLContext;
-  disableSSL = PFalse;
 }
 
 
 PSecureHTTPServiceProcess::~PSecureHTTPServiceProcess()
 {
-  delete sslContext;
+  delete m_sslContext;
+}
+
+
+void PSecureHTTPServiceProcess::DisableSSL()
+{
+  delete m_sslContext;
+  m_sslContext = NULL;
 }
 
 
 PHTTPServer * PSecureHTTPServiceProcess::CreateHTTPServer(PTCPSocket & socket)
 {
-  if (disableSSL)
+  if (m_sslContext == NULL)
     return PHTTPServiceProcess::CreateHTTPServer(socket);
 
 #ifdef SO_LINGER
@@ -85,7 +91,7 @@ PHTTPServer * PSecureHTTPServiceProcess::CreateHTTPServer(PTCPSocket & socket)
   socket.SetOption(SO_LINGER, &ling, sizeof(ling));
 #endif
 
-  PSSLChannel * ssl = new HTTP_PSSLChannel(this, sslContext);
+  PSSLChannel * ssl = new HTTP_PSSLChannel(this, m_sslContext);
 
   if (!ssl->Accept(socket)) {
     PSYSTEMLOG(Error, "HTTPS\tAccept failed: " << ssl->GetErrorText());
@@ -104,10 +110,13 @@ PHTTPServer * PSecureHTTPServiceProcess::CreateHTTPServer(PTCPSocket & socket)
 }
 
 
-PBoolean PSecureHTTPServiceProcess::SetServerCertificate(const PFilePath & certificateFile,
-                                                     PBoolean create,
+bool PSecureHTTPServiceProcess::SetServerCertificate(const PFilePath & certificateFile,
+                                                     bool create,
                                                      const char * dn)
 {
+  if (m_sslContext == NULL)
+    m_sslContext = new PSSLContext;
+
   if (create && !PFile::Exists(certificateFile)) {
     PSSLPrivateKey key(1024);
     PSSLCertificate certificate;
@@ -120,14 +129,17 @@ PBoolean PSecureHTTPServiceProcess::SetServerCertificate(const PFilePath & certi
     }
     if (!certificate.CreateRoot(name, key)) {
       PTRACE(1, "MTGW\tCould not create certificate");
-      return PFalse;
+      return false;
     }
     certificate.Save(certificateFile);
-    key.Save(certificateFile, PTrue);
+    key.Save(certificateFile, true);
   }
 
-  return sslContext->UseCertificate(certificateFile) &&
-         sslContext->UsePrivateKey(certificateFile);
+  if (m_sslContext->UseCertificate(certificateFile) && m_sslContext->UsePrivateKey(certificateFile))
+    return true;
+
+  DisableSSL();
+  return false;
 }
 
 PBoolean PSecureHTTPServiceProcess::OnDetectedNonSSLConnection(PChannel * chan, const PString & line)
@@ -158,7 +170,7 @@ PBoolean PSecureHTTPServiceProcess::OnDetectedNonSSLConnection(PChannel * chan,
     else {
       PIPSocket::Address addr;
       PIPSocket::GetHostAddress(addr);
-      url = addr.AsString() + ":" + PString(PString::Unsigned, httpListeningSocket->GetPort());
+      url = addr.AsString() + ":" + PString(PString::Unsigned, m_httpListeningSockets.front().GetPort());
     }
   }
 
@@ -167,7 +179,7 @@ PBoolean PSecureHTTPServiceProcess::OnDetectedNonSSLConnection(PChannel * chan,
   chan->WriteString(str);
   chan->Close();
 
-  return PFalse; 
+  return false; 
 }
 
 PString PSecureHTTPServiceProcess::CreateNonSSLMessage(const PString & url)
@@ -222,7 +234,7 @@ PBoolean HTTP_PSSLChannel::RawSSLRead(void * buf, PINDEX & len)
         line += (char)ch;
 
       if (!svc->OnDetectedNonSSLConnection(chan, line))
-        return PFalse;
+        return false;
     }
   }
 
@@ -230,7 +242,7 @@ PBoolean HTTP_PSSLChannel::RawSSLRead(void * buf, PINDEX & len)
   len = PMIN(len, preReadLen);
   memcpy(buf, preRead, len);
   preReadLen -= len;
-  return PTrue;
+  return true;
 }
 
 #endif //P_SSL
