@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -43,20 +43,21 @@
 static const char defaultCommunity[] = "public";
 
 PSNMPServer::PSNMPServer(PIPSocket::Address binding, WORD localPort, PINDEX timeout, PINDEX rxSize, PINDEX txSize)
- : PThread(1000, AutoDeleteThread, NormalPriority, "SNMP Server"),
-   community(defaultCommunity),
-   version(SNMP_VERSION),
-   maxRxSize(rxSize),
-   maxTxSize(txSize)
+ : P_DISABLE_MSVC_WARNINGS(4355, m_thread(*this, &PSNMPServer::Main, true, "SNMP Server"))
+ , community(defaultCommunity)
+ , version(SNMP_VERSION)
+ , maxRxSize(rxSize)
+ , maxTxSize(txSize)
 {
   SetReadTimeout(PTimeInterval(0, timeout));
   baseSocket = new PUDPSocket;
 
   if (!baseSocket->Listen(binding, 0, localPort)) {
-	  PTRACE(4,"SNMPsrv\tError: Unable to Listen on port " << localPort);
-  } else {
-      Open(baseSocket);
-	  Resume();
+    PTRACE(4,"SNMPsrv\tError: Unable to Listen on port " << localPort);
+  }
+  else {
+    Open(baseSocket);
+    m_thread.Resume();
   }
 }
 
@@ -80,7 +81,7 @@ PBoolean PSNMPServer::HandleChannel()
 
   for (;;) {
    	if (!IsOpen())
-	  return PFalse;
+	  return false;
 
 		// Reading
 	    PINDEX rxSize = 0;
@@ -116,7 +117,7 @@ PBoolean PSNMPServer::HandleChannel()
 		  continue;
 		}
 		// process the request
-		if (ProcessPDU(readBuffer, sendBuffer) == PTrue) {
+		if (ProcessPDU(readBuffer, sendBuffer) == true) {
 			// send the packet
 			baseSocket->SetSendAddress(remoteAddress, remotePort);
 			PTRACE(4, "SNMPsrv\tWriting " << sendBuffer.GetSize() << " Bytes to basesocket");
@@ -133,7 +134,7 @@ PBoolean PSNMPServer::HandleChannel()
 
 PBoolean PSNMPServer::Authorise(const PIPSocket::Address & /*received*/)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -153,19 +154,19 @@ PSNMP::ErrorType PSNMPServer::SendGetResponse (PSNMPVarBindingList &)
 
 PBoolean PSNMPServer::OnGetRequest (PINDEX , PSNMP::BindingList &, PSNMP::ErrorType &)
 {
-	return PFalse;
+	return false;
 }
 
 
 PBoolean PSNMPServer::OnGetNextRequest (PINDEX , PSNMP::BindingList &, PSNMP::ErrorType &)
 {
-	return PFalse;
+	return false;
 }
 
 
 PBoolean PSNMPServer::OnSetRequest (PINDEX , PSNMP::BindingList &,PSNMP::ErrorType &)
 {
-	return PFalse;
+	return false;
 }
 
 
@@ -194,7 +195,7 @@ static void EncodeOID(PDUType & pdu, const PINDEX & reqID,
 	   // Build the response list
 		PSNMP_VarBindList & vars = pdu.m_variable_bindings;
 		PINDEX i = 0;
-		vars.SetSize(varlist.size());
+		vars.SetSize((int)varlist.size());
 		PSNMP::BindingList::const_iterator Iter = varlist.begin();
 		while (Iter != varlist.end()) {
 		  vars[i].m_name.SetValue(Iter->first);
@@ -207,7 +208,7 @@ static void EncodeOID(PDUType & pdu, const PINDEX & reqID,
 
 PBoolean PSNMPServer::MIB_LocalMatch(PSNMP_PDU & pdu)
 {
-  PBoolean found = PFalse;
+  PBoolean found = false;
   PSNMP_VarBindList & vars = pdu.m_variable_bindings;
   PINDEX size = vars.GetSize();
  
@@ -215,22 +216,23 @@ PBoolean PSNMPServer::MIB_LocalMatch(PSNMP_PDU & pdu)
     PRFC1155_ObjectSyntax *obj = (PRFC1155_ObjectSyntax*) objList.GetAt(vars[x].m_name);
     if (obj != NULL){
       vars[x].m_value = *obj;
-      found = PTrue;
-    }else{
+      found = true;
+    }
+    else{
       pdu.m_error_status = PSNMP::NoSuchName;
-	}
+    }
   }
-  return PTrue;
+  return found;
 }
 
 PBoolean PSNMPServer::ConfirmCommunity(PASN_OctetString & /*community*/)
 {
-	return PFalse;
+	return false;
 }
 
 PBoolean PSNMPServer::ConfirmVersion(PASN_Integer vers)
 {
-  return version == vers ? PTrue : PFalse;
+  return version == vers ? true : false;
 }
 
 PBoolean PSNMPServer::ProcessPDU(const PBYTEArray & readBuffer, PBYTEArray & sendBuffer)
@@ -240,19 +242,19 @@ PBoolean PSNMPServer::ProcessPDU(const PBYTEArray & readBuffer, PBYTEArray & sen
   
   if (!msg.Decode((PASN_Stream &)readBuffer)) {
 	  PTRACE(4,"SNMPsrv\tERROR DECODING PDU");
-	  return PFalse;
+	  return false;
   }
 
   PTRACE(4, "SNMPsrv\tEncoded message" << msg);
 
   if (!ConfirmVersion(msg.m_version)){
     PTRACE(4, "SNMPsrv\tVersion mismatch on request, ignoring");
-    return PFalse;
+    return false;
   }
 
   if (!ConfirmCommunity(msg.m_community)){
     PTRACE(4, "SNMPsrv\tCommunity string mismatch on request, ignoring");
-    return PFalse;
+    return false;
   }
 
   PSNMP::BindingList varlist;
@@ -261,13 +263,13 @@ PBoolean PSNMPServer::ProcessPDU(const PBYTEArray & readBuffer, PBYTEArray & sen
   PSNMP_Message resp;
   PSNMP_PDUs sendpdu;
 
-  PBoolean retval = PTrue;
+  PBoolean retval = true;
   PSNMP::ErrorType errCode = PSNMP::NoError;
   switch (msg.m_pdu.GetTag()) {
     case PSNMP_PDUs::e_get_request:
       DecodeOID<PSNMP_GetRequest_PDU>(msg.m_pdu, reqID, varlist);
       retval = OnGetRequest(reqID, varlist, errCode);
-      if (retval == PTrue){
+      if (retval == true){
         sendpdu.SetTag(PSNMP_PDUs::e_get_response);
         EncodeOID<PSNMP_GetResponse_PDU>(sendpdu, reqID,varlist,errCode);
         resp.m_pdu = sendpdu;
@@ -275,7 +277,7 @@ PBoolean PSNMPServer::ProcessPDU(const PBYTEArray & readBuffer, PBYTEArray & sen
         resp.m_community = msg.m_community;
         PSNMP_GetResponse_PDU & mpdu = (PSNMP_GetResponse_PDU &) resp.m_pdu;
         if (!MIB_LocalMatch(mpdu)){
-          retval = PFalse;
+          retval = false;
           break;
         }
         resp.Encode((PASN_Stream &)sendBuffer);
@@ -285,7 +287,7 @@ PBoolean PSNMPServer::ProcessPDU(const PBYTEArray & readBuffer, PBYTEArray & sen
     case PSNMP_PDUs::e_get_next_request:
       DecodeOID<PSNMP_GetNextRequest_PDU>(msg.m_pdu, reqID, varlist);
       retval = OnGetNextRequest(reqID,varlist,errCode);
-      if (retval == PTrue){
+      if (retval == true){
         sendpdu.SetTag(PSNMP_PDUs::e_get_response);   
         EncodeOID<PSNMP_GetResponse_PDU>(sendpdu, reqID,varlist,errCode);
         resp.m_pdu = sendpdu;
@@ -293,7 +295,7 @@ PBoolean PSNMPServer::ProcessPDU(const PBYTEArray & readBuffer, PBYTEArray & sen
         resp.m_community = msg.m_community;
         PSNMP_GetResponse_PDU & mpdu = (PSNMP_GetResponse_PDU &) resp.m_pdu;
         if (!MIB_LocalMatch(mpdu)){
-          retval = PFalse;
+          retval = false;
           break;
         }
         resp.Encode((PASN_Stream &)sendBuffer);
@@ -310,7 +312,7 @@ PBoolean PSNMPServer::ProcessPDU(const PBYTEArray & readBuffer, PBYTEArray & sen
     default:
       PTRACE(4,"SNMPsrv\tSNMP Request/Response not supported");
       errCode= PSNMP::GenErr;
-      retval = PFalse;
+      retval = false;
   }
 
   if (retval) {
