@@ -24,9 +24,9 @@
  *
  * Contributor(s): Miguel Rodriguez Perez.
  *
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -52,7 +52,7 @@ PAec::PAec(int _clock, int _sampletime)
   ref_buf = NULL;
   noise = NULL;
 
-  receiveReady = PFalse;
+  receiveReady = false;
 
   bufferTime = sampleTime*2;  // Indicating it takes sampletime to play and sampletime to record
   minbuffer = bufferTime - sampleTime;
@@ -113,7 +113,7 @@ void PAec::Receive(BYTE * buffer, unsigned & length)
 	  rectime.Enqueue(new PTimeInterval(PTimer::Tick()));
 
   if (!receiveReady)
-     receiveReady = PTrue;
+     receiveReady = true;
 
 }
 
