@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22983 $
- * $Author: csoutheren $
- * $Date: 2009-06-25 07:29:05 -0500 (Thu, 25 Jun 2009) $
+ * $Revision: 28090 $
+ * $Author: rjongbloed $
+ * $Date: 2012-07-22 23:27:41 -0500 (Sun, 22 Jul 2012) $
  */
 
 #ifdef __GNUC__
@@ -42,21 +42,26 @@
 #define new PNEW
 
 
-PThreadPoolBase::PThreadPoolBase(unsigned int maxWorkerCount, unsigned int maxWorkUnitCount)
-  : m_maxWorkerCount(maxWorkerCount), m_maxWorkUnitCount(maxWorkUnitCount)
+PThreadPoolBase::PThreadPoolBase(unsigned int maxWorkerCount,
+                                 unsigned int maxWorkUnitCount,
+                                 const char * threadName,
+                                 PThread::Priority priority)
+  : m_maxWorkerCount(maxWorkerCount)
+  , m_maxWorkUnitCount(maxWorkUnitCount)
+  , m_highWaterMark(0)
+  , m_threadName(threadName != NULL ? threadName : "Pool")
+  , m_priority(priority)
 {
 }
 
 PThreadPoolBase::~PThreadPoolBase()
 {
-  for (;;) {
-    PWaitAndSignal mutex(m_listMutex);
-    if (m_workers.size() == 0)
-      break;
-
-    WorkerThreadBase * worker = m_workers[0];
-    worker->Shutdown();
+  while (!m_workers.empty()) {
+    m_listMutex.Wait();
+    WorkerThreadBase * worker = m_workers.front();
     m_workers.erase(m_workers.begin());
+    m_listMutex.Signal();
+
     StopWorker(worker);
   }
 }
@@ -99,9 +104,18 @@ PThreadPoolBase::WorkerThreadBase * PThreadPoolBase::NewWorker()
 {
   // create a new worker thread
   WorkerThreadBase * worker = CreateWorkerThread();
-  worker->Resume();
+
+  m_listMutex.Wait();
   m_workers.push_back(worker);
 
+  if (m_workers.size() > m_highWaterMark) {
+    m_highWaterMark = m_workers.size();
+    PTRACE(4, "PTLib\tThread pool high water mark: " << m_highWaterMark);
+  }
+
+  m_listMutex.Signal();
+
+  worker->Resume();
   return worker;
 }
 
@@ -149,6 +163,7 @@ void PThreadPoolBase::StopWorker(WorkerThreadBase * worker)
   if (!worker->WaitForTermination(10000)) {
     PTRACE(4, "ThreadPool\tWorker did not terminate promptly");
   }
+
   PTRACE(4, "ThreadPool\tDestroying pool thread");
   delete worker;
 }
