@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25925 $
+ * $Revision: 29844 $
  * $Author: rjongbloed $
- * $Date: 2011-05-30 20:40:07 -0500 (Mon, 30 May 2011) $
+ * $Date: 2013-05-30 22:00:59 -0500 (Thu, 30 May 2013) $
  */
 
 #ifdef __GNUC__
@@ -44,8 +44,10 @@
 
 #if defined(_WIN32) && !defined(_WIN32_WCE)
 #include <shellapi.h>
+#ifdef _MSC_VER
 #pragma comment(lib,"shell32.lib")
 #endif
+#endif
 
 
 // RFC 1738
@@ -101,7 +103,6 @@ PURL_LEGACY_SCHEME(rtmp,      false, false, true,  false,  false,  false, false,
 PURL_LEGACY_SCHEME(sip,       true,  true,  true,  false,  false,  true,  true,  false, false, false, DEFAULT_SIP_PORT)
 PURL_LEGACY_SCHEME(sips,      true,  true,  true,  false,  false,  true,  true,  false, false, false, DEFAULT_SIPS_PORT)
 PURL_LEGACY_SCHEME(fax,       false, false, false, true,   false,  false, true,  false, false, false, 0)
-PURL_LEGACY_SCHEME(callto,    false, false, false, true,   false,  false, true,  false, false, false, 0)
 PURL_LEGACY_SCHEME(msrp,      false, false, true,  false,  false,  true,  true,  false, true,  false, DEFAULT_MSRP_PORT)
 
 #define DEFAULT_SCHEME "http"
@@ -111,31 +112,32 @@ PURL_LEGACY_SCHEME(msrp,      false, false, true,  false,  false,  true,  true,
 // PURL
 
 PURL::PURL()
-  : scheme(DEFAULT_SCHEME),
-    port(0),
-    portSupplied (PFalse),
-    relativePath(PFalse)
+  : schemeInfo(NULL)
+  , port(0)
+  , portSupplied (false)
+  , relativePath(false)
 {
 }
 
 
 PURL::PURL(const char * str, const char * defaultScheme)
 {
-  Parse(str, defaultScheme);
+  InternalParse(str, defaultScheme);
 }
 
 
 PURL::PURL(const PString & str, const char * defaultScheme)
 {
-  Parse(str, defaultScheme);
+  InternalParse((const char *)str, defaultScheme);
 }
 
 
 PURL::PURL(const PFilePath & filePath)
-  : scheme(FILE_SCHEME),
-    port(0),
-    portSupplied (PFalse),
-    relativePath(PFalse)
+  : schemeInfo(PURLSchemeFactory::CreateInstance(FILE_SCHEME))
+  , scheme(FILE_SCHEME)
+  , port(0)
+  , portSupplied(false)
+  , relativePath(false)
 {
   PStringArray pathArray = filePath.GetDirectory().GetPath();
   if (pathArray.IsEmpty())
@@ -169,6 +171,7 @@ PURL & PURL::operator=(const PURL & other)
 
 void PURL::CopyContents(const PURL & other)
 {
+  schemeInfo   = other.schemeInfo;
   urlString    = other.urlString;
   scheme       = other.scheme;
   username     = other.username;
@@ -228,15 +231,30 @@ PString PURL::TranslateString(const PString & str, TranslationType type)
       break;
 
     case PathTranslation :
-      safeChars += ":@&=+$,|";   // Section 3.3
+      safeChars += ":@&=$,|";   // Section 3.3
       break;
 
+    case ParameterTranslation :
+      /* By strict RFC2396/3.3 this should be as for PathTranslation, but many
+         URI schemes have parameters of the form key=value so we don't allow
+         '=' character in the allowed set. Also, including one of "@,|" is
+         incompatible with some schemes, leave those out too. */
+      safeChars += ":&+$";
+      break;
+
+    case QuotedParameterTranslation :
+      safeChars += "[]/:@&=+$,|";
+      return str.FindSpan(safeChars) != P_MAX_INDEX ? str.ToLiteral() : str;
+
     default :
       break;    // Section 3.4, no reserved characters may be used
   }
   PINDEX pos = (PINDEX)-1;
-  while ((pos = xlat.FindSpan(safeChars, pos+1)) != P_MAX_INDEX)
-    xlat.Splice(psprintf("%%%02X", (BYTE)xlat[pos]), pos, 1);
+  while ((pos = xlat.FindSpan(safeChars, pos+1)) != P_MAX_INDEX) {
+    char buf[10];
+    sprintf(buf, "%%%02X", (BYTE)xlat[pos]);
+    xlat.Splice(buf, pos, 1);
+  }
 
   return xlat;
 }
@@ -260,10 +278,12 @@ PString PURL::UntranslateString(const PString & str, TranslationType type)
     int digit1 = xlat[pos+1];
     int digit2 = xlat[pos+2];
     if (isxdigit(digit1) && isxdigit(digit2)) {
-      xlat[pos] = (char)(
+      char buf[2];
+      buf[0] = (char)(
             (isdigit(digit2) ? (digit2-'0') : (toupper(digit2)-'A'+10)) +
            ((isdigit(digit1) ? (digit1-'0') : (toupper(digit1)-'A'+10)) << 4));
-      xlat.Delete(pos+1, 2);
+      buf[1] = '\0';
+      xlat.Splice(buf, pos, 3);
     }
   }
 
@@ -281,14 +301,46 @@ void PURL::SplitVars(const PString & str, PStringToString & vars, char sep1, cha
     if (sep1next == P_MAX_INDEX)
       sep1next--; // Implicit assumption string is not a couple of gigabytes long ...
 
+    PCaselessString key, data;
+
     PINDEX sep2pos = str.Find(sep2, sep1prev);
     if (sep2pos > sep1next)
-      sep2pos = sep1next;
+      key = str(sep1prev, sep1next-1);
+    else {
+      key = str(sep1prev, sep2pos-1);
+      if (type != QuotedParameterTranslation)
+        data = str(sep2pos+1, sep1next-1);
+      else {
+        while (isspace(str[++sep2pos]))
+          ;
+        if (str[sep2pos] != '"')
+          data = str(sep2pos, sep1next-1);
+        else {
+          // find the end quote
+          PINDEX endQuote = sep2pos+1;
+          do {
+            endQuote = str.Find('"', endQuote+1);
+            if (endQuote == P_MAX_INDEX) {
+              PTRACE2(1, NULL, "URI\tNo closing double quote in parameter: " << str);
+              endQuote = str.GetLength()-1;
+              break;
+            }
+          } while (str[endQuote-1] == '\\');
 
-    PCaselessString key = PURL::UntranslateString(str(sep1prev, sep2pos-1), type);
+          data = PString(PString::Literal, str(sep2pos, endQuote));
+
+          if (sep1next < endQuote) {
+            sep1next = str.Find(sep1, endQuote);
+            if (sep1next == P_MAX_INDEX)
+              sep1next--; // Implicit assumption string is not a couple of gigabytes long ...
+          }
+        }
+      }
+    }
+
+    key = PURL::UntranslateString(key, type);
     if (!key) {
-      PString data = PURL::UntranslateString(str(sep2pos+1, sep1next-1), type);
-
+      data = PURL::UntranslateString(data, type);
       if (vars.Contains(key))
         vars.SetAt(key, vars[key] + '\n' + data);
       else
@@ -307,14 +359,18 @@ void PURL::OutputVars(ostream & strm,
                       char sep2,
                       TranslationType type)
 {
-  for (PINDEX i = 0; i < vars.GetSize(); i++) {
-    if (i > 0)
+  bool outputSeperator = false;
+  for (PStringOptions::const_iterator it = vars.begin(); it != vars.end(); ++it) {
+    if (outputSeperator)
       strm << sep1;
-    else if (sep0 != '\0')
-      strm << sep0;
+    else {
+      if (sep0 != '\0')
+        strm << sep0;
+      outputSeperator = true;
+    }
 
-    PString key  = TranslateString(vars.GetKeyAt (i), type);
-    PString data = TranslateString(vars.GetDataAt(i), type);
+    PString key  = TranslateString(it->first,  type);
+    PString data = TranslateString(it->second, type);
 
     if (key.IsEmpty())
       strm << data;
@@ -333,8 +389,8 @@ PBoolean PURL::InternalParse(const char * cstr, const char * defaultScheme)
   password.MakeEmpty();
   hostname.MakeEmpty();
   port = 0;
-  portSupplied = PFalse;
-  relativePath = PFalse;
+  portSupplied = false;
+  relativePath = false;
   path.SetSize(0);
   paramVars.RemoveAll();
   fragment.MakeEmpty();
@@ -347,119 +403,61 @@ PBoolean PURL::InternalParse(const char * cstr, const char * defaultScheme)
   // copy the string so we can take bits off it
   while (((*cstr & 0x80) == 0x00) && isspace(*cstr))
     cstr++;
-  PString url = cstr;
-  if (url.IsEmpty())
+
+  // if string is empty, return
+  if (*cstr == '\0')
     return false;
 
   // get information which tells us how to parse URL for this
   // particular scheme
-  PURLScheme * schemeInfo = NULL;
+  schemeInfo = NULL;
 
   // Character set as per RFC2396
   //    scheme        = alpha *( alpha | digit | "+" | "-" | "." )
-  if (isalpha(url[0])) {
+  if (isalpha(cstr[0])) {
     PINDEX pos = 1;
-    while (isalnum(url[pos]) || url[pos] == '+' || url[pos] == '-' || url[pos] == '.')
+    while (isalnum(cstr[pos]) || cstr[pos] == '+' || cstr[pos] == '-' || cstr[pos] == '.')
       ++pos;
 
     // Determine if the URL has an explicit scheme
-    if (url[pos] == ':') {
+    if (cstr[pos] == ':') {
+      scheme = PString(cstr, pos);
       // get the scheme information
-      schemeInfo = PURLSchemeFactory::CreateInstance(url.Left(pos));
+      schemeInfo = PURLSchemeFactory::CreateInstance(std::string(cstr, pos));
       if (schemeInfo != NULL)
-        url.Delete(0, pos+1);
+        cstr += pos+1;
     }
   }
 
   // if we could not match a scheme, then use the specified default scheme
-  if (schemeInfo == NULL && defaultScheme != NULL) {
+  if (schemeInfo == NULL && defaultScheme != NULL && *defaultScheme != '\0') {
+    scheme = defaultScheme;
     schemeInfo = PURLSchemeFactory::CreateInstance(defaultScheme);
-    PAssert(schemeInfo != NULL, "Default scheme " + PString(defaultScheme) + " not available");
+    PAssert(schemeInfo != NULL, "Default scheme " + scheme + " not available");
   }
 
   // if that still fails, then there is nowehere to go
   if (schemeInfo == NULL)
     return false;
 
-  scheme = schemeInfo->GetName();
-  return schemeInfo->Parse(url, *this) && !IsEmpty();
+  // Now parse using the the scheme info.
+  return schemeInfo->Parse(cstr, *this) && !IsEmpty();
 }
 
-PBoolean PURL::LegacyParse(const PString & _url, const PURLLegacyScheme * schemeInfo)
+
+bool PURL::LegacyParse(const char * cstr, const PURLLegacyScheme * schemeInfo)
 {
-  PString url = _url;
+  const PConstCaselessString str(cstr);
+  PINDEX start = 0;
+  PINDEX end = P_MAX_INDEX;
   PINDEX pos;
 
-  // Super special case!
-  if (scheme *= "callto") {
-
-    // Actually not part of MS spec, but a lot of people put in the // into
-    // the URL, so we take it out of it is there.
-    if (url.GetLength() > 2 && url[0] == '/' && url[1] == '/')
-      url.Delete(0, 2);
-
-    // For some bizarre reason callto uses + instead of ; for paramters
-    // We do a loop so that phone numbers of the form +61243654666 still work
-    do {
-      pos = url.Find('+');
-    } while (pos != P_MAX_INDEX && isdigit(url[pos+1]));
-
-    if (pos != P_MAX_INDEX) {
-      SplitVars(url(pos+1, P_MAX_INDEX), paramVars, '+', '=');
-      url.Delete(pos, P_MAX_INDEX);
-    }
-
-    hostname = paramVars("gateway");
-    if (!hostname)
-      username = UntranslateString(url, LoginTranslation);
-    else {
-      PCaselessString type = paramVars("type");
-      if (type == "directory") {
-        pos = url.Find('/');
-        if (pos == P_MAX_INDEX)
-          username = UntranslateString(url, LoginTranslation);
-        else {
-          hostname = UntranslateString(url.Left(pos), LoginTranslation);
-          username = UntranslateString(url.Mid(pos+1), LoginTranslation);
-        }
-      }
-      else {
-        // Now look for an @ and split user and host
-        pos = url.Find('@');
-        if (pos != P_MAX_INDEX) {
-          username = UntranslateString(url.Left(pos), LoginTranslation);
-          hostname = UntranslateString(url.Mid(pos+1), LoginTranslation);
-        }
-        else {
-          if (type == "ip" || type == "host")
-            hostname = UntranslateString(url, LoginTranslation);
-          else
-            username = UntranslateString(url, LoginTranslation);
-        }
-      }
-    }
-
-    // Allow for [ipv6] form
-    pos = hostname.Find(']');
-    if (pos == P_MAX_INDEX)
-      pos = 0;
-    pos = hostname.Find(':', pos);
-    if (pos != P_MAX_INDEX) {
-      port = (WORD)hostname.Mid(pos+1).AsUnsigned();
-      portSupplied = PTrue;
-      hostname.Delete(pos, P_MAX_INDEX);
-    }
-
-    password = paramVars("password");
-    return PTrue;
-  }
-
   // if the URL should have leading slash, then remove it if it has one
   if (schemeInfo != NULL && schemeInfo->hasHostPort && schemeInfo->hasPath) {
-    if (url.GetLength() > 2 && url[0] == '/' && url[1] == '/')
-      url.Delete(0, 2);
+    if (str.GetLength() > 2 && str[0] == '/' && str[1] == '/')
+      start = 2;
     else
-      relativePath = PTrue;
+      relativePath = true;
   }
 
   // parse user/password/host/port
@@ -473,25 +471,26 @@ PBoolean PURL::LegacyParse(const PString & _url, const PURLLegacyScheme * scheme
       endHostChars += ';';
     if (schemeInfo->hasFragments)
       endHostChars += '#';
+
     if (endHostChars.IsEmpty())
       pos = P_MAX_INDEX;
     else if (schemeInfo->hasUsername) {
       //';' showing in the username field should be valid.
       // Looking for ';' after the '@' for the parameters.
-      PINDEX posAt = url.Find('@');
+      PINDEX posAt = str.Find('@', start);
       if (posAt != P_MAX_INDEX)
-        pos = url.FindOneOf(endHostChars, posAt);
+        pos = str.FindOneOf(endHostChars, posAt);
       else 
-        pos = url.FindOneOf(endHostChars);
+        pos = str.FindOneOf(endHostChars, start);
     }
     else
-      pos = url.FindOneOf(endHostChars);
+      pos = str.FindOneOf(endHostChars, start);
 
-    PString uphp = url.Left(pos);
+    PString uphp = str(start, pos-1);
     if (pos != P_MAX_INDEX)
-      url.Delete(0, pos);
+      start = pos;
     else
-      url.MakeEmpty();
+      start = P_MAX_INDEX;
 
     // if the URL is of type UserPasswordHostPort, then parse it
     if (schemeInfo->hasUsername) {
@@ -534,16 +533,18 @@ PBoolean PURL::LegacyParse(const PString & _url, const PURLLegacyScheme * scheme
     else {
       // determine if the URL has a port number
       // Allow for [ipv6] form
-      pos = uphp.Find(']');
-      if (pos == P_MAX_INDEX)
-        pos = 0;
-      pos = uphp.Find(':', pos);
-      if (pos == P_MAX_INDEX)
-        hostname = UntranslateString(uphp, LoginTranslation);
+      if (uphp[0] == '[' && (pos = uphp.Find(']')) != P_MAX_INDEX) {
+        hostname = uphp.Left(pos+1); // No translation if inside []
+        pos = uphp.Find(':', pos);
+      }
       else {
+        pos = uphp.Find(':');
         hostname = UntranslateString(uphp.Left(pos), LoginTranslation);
+      }
+
+      if (pos != P_MAX_INDEX) {
         port = (WORD)uphp.Mid(pos+1).AsUnsigned();
-        portSupplied = PTrue;
+        portSupplied = true;
       }
 
       if (hostname.IsEmpty() && schemeInfo->defaultHostToLocal)
@@ -551,51 +552,44 @@ PBoolean PURL::LegacyParse(const PString & _url, const PURLLegacyScheme * scheme
     }
   }
 
-  if (schemeInfo->hasQuery) {
-    // chop off any trailing query
-    pos = url.Find('?');
-    if (pos != P_MAX_INDEX) {
-      SplitQueryVars(url(pos+1, P_MAX_INDEX), queryVars);
-      url.Delete(pos, P_MAX_INDEX);
-    }
+  // chop off any trailing query
+  if (schemeInfo->hasQuery && (pos = str.Find('?', start)) < end) {
+    SplitQueryVars(str(pos+1, end), queryVars);
+    end = pos-1;
   }
 
-  if (schemeInfo->hasParameters) {
-    // chop off any trailing parameters
-    pos = url.Find(';');
-    if (pos != P_MAX_INDEX) {
-      SplitVars(url(pos+1, P_MAX_INDEX), paramVars);
-      url.Delete(pos, P_MAX_INDEX);
-    }
+  // chop off any trailing parameters
+  if (schemeInfo->hasParameters && (pos = str.Find(';', start)) < end) {
+    SplitVars(str(pos+1, end), paramVars);
+    end = pos-1;
   }
 
-  if (schemeInfo->hasFragments) {
-    // chop off any trailing fragment
-    pos = url.Find('#');
-    if (pos != P_MAX_INDEX) {
-      fragment = UntranslateString(url(pos+1, P_MAX_INDEX), PathTranslation);
-      url.Delete(pos, P_MAX_INDEX);
-    }
+  // chop off any trailing fragment
+  if (schemeInfo->hasFragments && (pos = str.Find('#', start)) < end) {
+    fragment = UntranslateString(str(pos+1, end), PathTranslation);
+    end = pos-1;
   }
 
-  if (schemeInfo->hasPath)
-    SetPathStr(url);   // the hierarchy is what is left
-  else {
-    // if the rest of the URL isn't a path, then we are finished!
-    m_contents = UntranslateString(url, PathTranslation);
-    Recalculate();
-  }
-
-  if (port == 0 && schemeInfo->defaultPort != 0 && !relativePath) {
+  if (port == 0 && !relativePath) {
     // Yes another horrible, horrible special case!
     if (scheme == "h323" && paramVars("type") == "gk")
       port = DEFAULT_H323RAS_PORT;
     else
       port = schemeInfo->defaultPort;
+  }
+
+  if (schemeInfo->hasPath) {
+    if (str[start] == '/')
+      ++start;
+    SetPathStr(str(start, end));   // the hierarchy is what is left
+  }
+  else {
+    // if the rest of the URL isn't a path, then we are finished!
+    m_contents = UntranslateString(str(start, end), PathTranslation);
     Recalculate();
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -630,29 +624,24 @@ PString PURL::AsString(UrlFormat fmt) const
   if (fmt == FullURL)
     return urlString;
 
-  if (scheme.IsEmpty())
+  if (scheme.IsEmpty() || schemeInfo == NULL)
     return PString::Empty();
 
-  const PURLScheme * schemeInfo = PURLSchemeFactory::CreateInstance(scheme);
-  if (schemeInfo == NULL)
-    schemeInfo = PURLSchemeFactory::CreateInstance(DEFAULT_SCHEME);
-
   return schemeInfo->AsString(fmt, *this);
 }
 
+
 PString PURL::LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schemeInfo) const
 {
   PStringStream str;
 
-  if (fmt == HostPortOnly) {
+  if (fmt != RelativeOnly && !(relativePath && schemeInfo->relativeImpliesScheme))
     str << scheme << ':';
 
-    if (relativePath) {
-      if (schemeInfo->relativeImpliesScheme)
-        return PString::Empty();
-      return str;
-    }
+  if (fmt == LocationOnly && relativePath)
+    return str;
 
+  if (fmt != RelativeOnly && !relativePath) {
     if (schemeInfo->hasPath && schemeInfo->hasHostPort)
       str << "//";
 
@@ -661,15 +650,18 @@ PString PURL::LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schem
         str << TranslateString(username, LoginTranslation);
         if (schemeInfo->hasPassword && !password)
           str << ':' << TranslateString(password, LoginTranslation);
-        str << '@';
+        if (schemeInfo->hasHostPort && !hostname.IsEmpty())
+          str << '@';
       }
     }
 
     if (schemeInfo->hasHostPort) {
-      if (hostname.Find(':') != P_MAX_INDEX && hostname[0] != '[')
+      if (hostname[0] == '[') // Should be IPv6 address
+        str << hostname;
+      else if (hostname.Find(':') != P_MAX_INDEX) // Assume it is an IPv6 address
         str << '[' << hostname << ']';
       else
-        str << hostname;
+        str << TranslateString(hostname, LoginTranslation);
     }
 
     if (schemeInfo->defaultPort != 0) {
@@ -677,27 +669,36 @@ PString PURL::LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schem
         str << ':' << port;
     }
 
-    // Problem was fixed for handling legacy schema like tel URI.
-    // HostPortOnly format: if there is no default user and host fields, only the schema itself is being returned.
-    // URIOnly only format: the pathStr will be retruned.
-    // The Recalculate() will merge both HostPortOnly and URIOnly formats for the completed uri string creation.
-    if (schemeInfo->defaultToUserIfNoAt)
-      return str;
+    if (fmt == LocationOnly) {
+      // Problem was fixed for handling legacy schema like tel URI.
+      // LocationOnly format: if there is no default user and host fields, only the schema itself is being returned.
+      // RelativeOnly only format: the pathStr will be retruned.
+      // The Recalculate() will merge both LocationOnly and RelativeOnly formats for the completed uri string creation.
+      if (schemeInfo->defaultToUserIfNoAt)
+        return str;
 
-    if (str.GetLength() > scheme.GetLength()+1)
-      return str;
+      if (str.GetLength() > scheme.GetLength()+1)
+        return str;
 
-    // Cannot JUST have the scheme: ....
-    return PString::Empty();
+      // Cannot JUST have the scheme: ....
+      return PString::Empty();
+    }
   }
 
-  // URIOnly and PathOnly
-  if (schemeInfo->hasPath)
-    str << GetPathStr();
+  // RelativeOnly and PathOnly
+  if (schemeInfo->hasPath) {
+    for (PINDEX i = 0; i < path.GetSize(); i++) {
+      if (i > 0 || !relativePath)
+        str << '/';
+      str << TranslateString(path[i], PathTranslation);
+    }
+    if (!relativePath && str.IsEmpty())
+      str << '/';
+  }
   else
     str << TranslateString(m_contents, PathTranslation);
 
-  if (fmt == URIOnly) {
+  if (fmt == FullURL || fmt == RelativeOnly) {
     if (!fragment)
       str << "#" << TranslateString(fragment, PathTranslation);
 
@@ -709,10 +710,23 @@ PString PURL::LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schem
 }
 
 
-void PURL::SetScheme(const PString & s)
+bool PURL::SetScheme(const PString & newScheme)
 {
-  scheme = s;
+  const PURLScheme * newSchemeInfo = PURLSchemeFactory::CreateInstance(newScheme);
+  if (newSchemeInfo == NULL)
+    return false;
+
+  scheme = newScheme;
+  schemeInfo = newSchemeInfo;
+
+  if (!portSupplied) {
+    const PURLLegacyScheme * legacy = dynamic_cast<const PURLLegacyScheme *>(schemeInfo);
+    if (legacy != NULL)
+      port = legacy->defaultPort;
+  }
+
   Recalculate();
+  return true;
 }
 
 
@@ -739,15 +753,31 @@ void PURL::SetHostName(const PString & h)
 
 void PURL::SetPort(WORD newPort)
 {
-  port = newPort;
-  portSupplied = true;
+  if (newPort != 0) {
+    port = newPort;
+    portSupplied = true;
+  }
+  else {
+    port = schemeInfo != NULL ? schemeInfo->GetDefaultPort() : 0;
+    portSupplied = false;
+  }
   Recalculate();
 }
 
 
+PString PURL::GetHostPort() const
+{
+  PStringStream strm;
+  strm << hostname;
+  if (portSupplied)
+    strm << ':' << port;
+  return strm;
+}
+
+
 void PURL::SetPathStr(const PString & pathStr)
 {
-  path = pathStr.Tokenise("/", PTrue);
+  path = pathStr.Tokenise("/", true);
 
   if (path.GetSize() > 0 && path[0].IsEmpty()) 
     path.RemoveAt(0);
@@ -779,17 +809,40 @@ PString PURL::GetPathStr() const
 void PURL::SetPath(const PStringArray & p)
 {
   path = p;
+  path.MakeUnique();
   Recalculate();
 }
 
 
 void PURL::AppendPath(const PString & segment)
 {
+  path.MakeUnique();
   path.AppendString(segment);
   Recalculate();
 }
 
 
+void PURL::ChangePath(const PString & segment, PINDEX idx)
+{
+  path.MakeUnique();
+
+  if (path.IsEmpty()) {
+    if (!segment.IsEmpty())
+      path.AppendString(segment);
+  }
+  else {
+    if (idx >= path.GetSize())
+      idx = path.GetSize()-1;
+    if (segment.IsEmpty())
+      path.RemoveAt(idx);
+    else
+      path[idx] = segment;
+  }
+
+  Recalculate();
+}
+
+
 PString PURL::GetParameters() const
 {
   PStringStream strm;
@@ -861,17 +914,17 @@ void PURL::SetContents(const PString & str)
 }
 
 
-bool PURL::LoadResource(PString & str, const PString & requiredContentType)
+bool PURL::LoadResource(PString & str, const LoadParams & params) const
 {
   PURLLoader * loader = PURLLoaderFactory::CreateInstance(GetScheme());
-  return loader != NULL && loader->Load(*this, str, requiredContentType);
+  return loader != NULL && loader->Load(str, *this, params);
 }
 
 
-bool PURL::LoadResource(PBYTEArray & data, const PString & requiredContentType)
+bool PURL::LoadResource(PBYTEArray & data, const LoadParams & params) const
 {
   PURLLoader * loader = PURLLoaderFactory::CreateInstance(GetScheme());
-  return loader != NULL && loader->Load(*this, data, requiredContentType);
+  return loader != NULL && loader->Load(data, *this, params);
 }
 
 
@@ -899,13 +952,108 @@ bool PURL::OpenBrowser(const PString & url)
 
 void PURL::Recalculate()
 {
-  if (scheme.IsEmpty())
-    scheme = DEFAULT_SCHEME;
-
-  urlString = AsString(HostPortOnly) + AsString(URIOnly);
+  if (schemeInfo != NULL)
+    urlString = schemeInfo->AsString(FullURL, *this);
+  else
+    urlString.MakeEmpty();
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+
+// RFC3966 tel URI
+
+class PURL_CalltoScheme : public PURLScheme
+{
+    PCLASSINFO(PURL_CalltoScheme, PURLScheme);
+  public:
+    virtual bool Parse(const char * cstr, PURL & url) const
+    {
+      const PConstCaselessString str(cstr);
+
+      // Actually not part of MS spec, but a lot of people put in the // into
+      // the URL, so we take it out of it is there.
+      PINDEX start = 0;
+      if (str.GetLength() > 2 && str[0] == '/' && str[1] == '/')
+        start = 2;
+
+      // For some bizarre reason callto uses + instead of ; for paramters
+      PINDEX pos = str.Find('+', start);
+
+      // We also check for phone numbers of the form +61243654666 still work
+      if (pos != P_MAX_INDEX && isdigit(str[pos+1]))
+        pos = str.Find('+', pos+1);
+
+      PINDEX end = P_MAX_INDEX;
+      if (pos != P_MAX_INDEX) {
+        PStringToString paramVars;
+        PURL::SplitVars(str(start, ++pos), paramVars, '+', '=');
+        url.SetParamVars(paramVars);
+        end = pos-1;
+      }
+
+      PString hostname = url.GetParamVars()("gateway");
+      PString username;
+      if (!hostname)
+        username = PURL::UntranslateString(str(start, end), PURL::LoginTranslation);
+      else {
+        PCaselessString type = url.GetParamVars()("type");
+        if (type == "directory") {
+          pos = str.Find('/', start);
+          if (pos == P_MAX_INDEX)
+            username = PURL::UntranslateString(str(start, end), PURL::LoginTranslation);
+          else {
+            hostname = PURL::UntranslateString(str(start, pos), PURL::LoginTranslation);
+            username = PURL::UntranslateString(str(pos+1, end), PURL::LoginTranslation);
+          }
+        }
+        else {
+          // Now look for an @ and split user and host
+          pos = str.Find('@');
+          if (pos != P_MAX_INDEX) {
+            username = PURL::UntranslateString(str(start, pos), PURL::LoginTranslation);
+            hostname = PURL::UntranslateString(str(pos+1, end), PURL::LoginTranslation);
+          }
+          else {
+            if (type == "ip" || type == "host")
+              hostname = PURL::UntranslateString(str(start, end), PURL::LoginTranslation);
+            else
+              username = PURL::UntranslateString(str(start, end), PURL::LoginTranslation);
+          }
+        }
+      }
+
+      // Allow for [ipv6] form
+      pos = hostname.Find(']');
+      if (pos == P_MAX_INDEX)
+        pos = 0;
+      pos = hostname.Find(':', pos);
+      if (pos != P_MAX_INDEX) {
+        url.SetPort((WORD)hostname.Mid(pos+1).AsUnsigned());
+        hostname.Delete(pos, P_MAX_INDEX);
+      }
+
+      url.SetHostName(hostname);
+      url.SetUserName(username);
+      url.SetPassword(url.GetParamVars()("password"));
+      return true;
+    }
+
+    virtual PString AsString(PURL::UrlFormat fmt, const PURL & url) const
+    {
+      if (fmt == PURL::LocationOnly)
+        return PString::Empty();
+
+      PStringStream strm;
+      strm << "callto:" + url.GetUserName();
+      PURL::OutputVars(strm, url.GetParamVars(), '+', '+', '=', PURL::ParameterTranslation);
+      return strm;
+    }
+};
+
+static PURLSchemeFactory::Worker<PURL_CalltoScheme> calltoScheme("callto", true);
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 // RFC3966 tel URI
@@ -914,13 +1062,10 @@ class PURL_TelScheme : public PURLScheme
 {
     PCLASSINFO(PURL_TelScheme, PURLScheme);
   public:
-    virtual PString GetName() const
+    virtual PBoolean Parse(const char * cstr, PURL & url) const
     {
-      return "tel";
-    }
+      const PConstCaselessString str(cstr);
 
-    virtual PBoolean Parse(const PString & str, PURL & url) const
-    {
       PINDEX pos = str.FindSpan("0123456789*#", str[0] != '+' ? 0 : 1);
       if (pos == P_MAX_INDEX)
         url.SetUserName(str);
@@ -931,7 +1076,7 @@ class PURL_TelScheme : public PURLScheme
         url.SetUserName(str.Left(pos));
 
         PStringToString paramVars;
-        PURL::SplitVars(str(pos+1, P_MAX_INDEX), paramVars);
+        PURL::SplitVars(str.Mid(pos+1), paramVars);
         url.SetParamVars(paramVars);
 
         PString phoneContext = paramVars("phone-context");
@@ -952,7 +1097,7 @@ class PURL_TelScheme : public PURLScheme
 
     virtual PString AsString(PURL::UrlFormat fmt, const PURL & url) const
     {
-      if (fmt == PURL::HostPortOnly)
+      if (fmt == PURL::LocationOnly)
         return PString::Empty();
 
       PStringStream strm;
@@ -973,33 +1118,30 @@ class PURL_DataScheme : public PURLScheme
 {
     PCLASSINFO(PURL_DataScheme, PURLScheme);
   public:
-    virtual PString GetName() const
+    virtual bool Parse(const char * cstr, PURL & url) const
     {
-      return "data";
-    }
+      const PConstCaselessString str(cstr);
 
-    virtual PBoolean Parse(const PString & url, PURL & purl) const
-    {
-      PINDEX comma = url.Find(',');
+      PINDEX comma = str.Find(',');
       if (comma == P_MAX_INDEX)
         return false;
 
-      PINDEX semi = url.Find(';');
+      PINDEX semi = str.Find(';');
       if (semi > comma)
-        purl.SetParamVar("type", url.Left(comma));
+        url.SetParamVar("type", str.Left(comma));
       else {
-        purl.SetParameters(url(semi, comma-1));
-        purl.SetParamVar("type", url.Left(semi));
+        url.SetParameters(str(semi, comma-1));
+        url.SetParamVar("type", str.Left(semi));
       }
 
-      purl.SetContents(url.Mid(comma+1));
+      url.SetContents(str.Mid(comma+1));
 
       return true;
     }
 
     virtual PString AsString(PURL::UrlFormat fmt, const PURL & purl) const
     {
-      if (fmt == PURL::HostPortOnly)
+      if (fmt == PURL::LocationOnly)
         return PString::Empty();
 
       const PStringToString & params = purl.GetParamVars();
@@ -1008,8 +1150,8 @@ class PURL_DataScheme : public PURLScheme
       strm << "data:" + params("type", "text/plain");
 
       bool base64 = false;
-      for (PINDEX i = 0; i < params.GetSize(); i++) {
-        PCaselessString key = params.GetKeyAt(i);
+      for (PStringOptions::const_iterator it = params.begin(); it != params.end(); ++it) {
+        PCaselessString key = it->first;
         if (key == "type")
           continue;
         if (key == "base64") {
@@ -1019,7 +1161,7 @@ class PURL_DataScheme : public PURLScheme
 
         strm << ';' << PURL::TranslateString(key, PURL::ParameterTranslation);
 
-        PString data = params.GetDataAt(i);
+        PString data = it->second;
         if (!data)
           strm << '=' << PURL::TranslateString(data, PURL::ParameterTranslation);
       }
@@ -1043,17 +1185,16 @@ class PURL_FileLoader : public PURLLoader
 {
     PCLASSINFO(PURL_FileLoader, PURLLoader);
   public:
-    virtual bool Load(const PURL & url, PString & str, const PString &)
+    virtual bool Load(PString & str, const PURL & url, const PURL::LoadParams &) const
     {
-      PTextFile file;
+      PFile file;
       if (!file.Open(url.AsFilePath()))
         return false;
-      if (!str.SetSize(file.GetLength()+1))
-        return false;
-      return file.Read(str.GetPointer(), str.GetSize()-1);
+      str = file.ReadString(file.GetLength());
+      return true;
     }
 
-    virtual bool Load(const PURL & url, PBYTEArray & data, const PString &)
+    virtual bool Load(PBYTEArray & data, const PURL & url, const PURL::LoadParams &) const
     {
       PFile file;
       if (!file.Open(url.AsFilePath()))
@@ -1073,11 +1214,11 @@ class PURL_DataLoader : public PURLLoader
 {
     PCLASSINFO(PURL_FileLoader, PURLLoader);
   public:
-    virtual bool Load(const PURL & url, PString & str, const PString & requiredContentType)
+    virtual bool Load(PString & str, const PURL & url, const PURL::LoadParams & params) const
     {
-      if (!requiredContentType.IsEmpty()) {
+      if (!params.m_requiredContentType.IsEmpty()) {
         PCaselessString actualContentType = url.GetParamVars()("type");
-        if (!actualContentType.IsEmpty() && requiredContentType != requiredContentType)
+        if (!actualContentType.IsEmpty() && actualContentType != params.m_requiredContentType)
           return false;
       }
 
@@ -1085,11 +1226,11 @@ class PURL_DataLoader : public PURLLoader
       return true;
     }
 
-    virtual bool Load(const PURL & url, PBYTEArray & data, const PString & requiredContentType)
+    virtual bool Load(PBYTEArray & data, const PURL & url, const PURL::LoadParams & params) const
     {
-      if (!requiredContentType.IsEmpty()) {
+      if (!params.m_requiredContentType.IsEmpty()) {
         PCaselessString actualContentType = url.GetParamVars()("type");
-        if (!actualContentType.IsEmpty() && requiredContentType != requiredContentType)
+        if (!actualContentType.IsEmpty() && actualContentType != params.m_requiredContentType)
           return false;
       }
 
