@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26016 $
+ * $Revision: 28736 $
  * $Author: rjongbloed $
- * $Date: 2011-06-14 02:38:05 -0500 (Tue, 14 Jun 2011) $
+ * $Date: 2013-01-01 22:51:46 -0600 (Tue, 01 Jan 2013) $
  */
 
 #ifdef __GNUC__
@@ -36,6 +36,8 @@
 
 #include <ptclib/vcard.h>
 
+#if P_VCARD
+
 
 static long MaxLineLength = 72;
 
@@ -154,7 +156,7 @@ void PvCard::ParamValue::PrintOn(ostream & strm) const
   PINDEX lastPos = 0, pos;
   while ((pos = Find('"', lastPos)) != P_MAX_INDEX) {
     strm.iword(0) += pos - lastPos + 1;
-    strm << string(lastPos, pos-1) << "\\\"";
+    strm << Mid(lastPos, pos-1-lastPos) << "\\\"";
     lastPos = pos+1;
   }
 
@@ -730,4 +732,7 @@ PString PvCard::AsString(Format fmt)
 }
 
 
+#endif // P_VCARD
+
+
 // End of File ///////////////////////////////////////////////////////////////
