@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Revision: 28935 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:12:14 -0600 (Fri, 18 Jan 2013) $
  */
 
 #ifdef __GNUC__
@@ -42,12 +42,12 @@
 
 ///////////////////////////////////////////////////////
 
-const PString & XMPP::LanguageTag() { static PString s = "xml:lang"; return s; }
-const PString & XMPP::NamespaceTag() { static PString s = "xmlns"; return s; }
-const PString & XMPP::MessageStanzaTag() { static PString s = "message"; return s; }
-const PString & XMPP::PresenceStanzaTag() { static PString s = "presence"; return s; }
-const PString & XMPP::IQStanzaTag() { static PString s = "iq"; return s; }
-const PString & XMPP::IQQueryTag() { static PString s = "query"; return s; }
+const PCaselessString & XMPP::LanguageTag()       { static const PConstCaselessString s("xml:lang"); return s; }
+const PCaselessString & XMPP::NamespaceTag()      { static const PConstCaselessString s("xmlns");    return s; }
+const PCaselessString & XMPP::MessageStanzaTag()  { static const PConstCaselessString s("message");  return s; }
+const PCaselessString & XMPP::PresenceStanzaTag() { static const PConstCaselessString s("presence"); return s; }
+const PCaselessString & XMPP::IQStanzaTag()       { static const PConstCaselessString s("iq");       return s; }
+const PCaselessString & XMPP::IQQueryTag()        { static const PConstCaselessString s("query");    return s; }
 
 ///////////////////////////////////////////////////////
 
@@ -64,7 +64,7 @@ XMPP::JID::JID(const PString& jid)
 
 
 XMPP::JID::JID(const PString& user, const PString& server, const PString& resource)
-  : m_User(user), m_Server(server), m_Resource(resource), m_IsDirty(PTrue)
+  : m_User(user), m_Server(server), m_Resource(resource), m_IsDirty(true)
 {
   BuildJID();
 }
@@ -103,21 +103,21 @@ XMPP::JID::operator const PString&() const
 
 void XMPP::JID::SetUser(const PString& user)
 {
-  m_IsDirty = PTrue;
+  m_IsDirty = true;
   m_User = user;
 }
 
 
 void XMPP::JID::SetServer(const PString& server)
 {
-  m_IsDirty = PTrue;
+  m_IsDirty = true;
   m_Server = server;
 }
 
 
 void XMPP::JID::SetResource(const PString& resource)
 {
-  m_IsDirty = PTrue;
+  m_IsDirty = true;
   m_Resource = resource;
 }
 
@@ -164,7 +164,7 @@ void XMPP::JID::BuildJID() const
   if (!m_Resource.IsEmpty())
     m_JID += "/" + m_Resource;
 
-  m_IsDirty = PFalse;
+  m_IsDirty = false;
 }
 
 ///////////////////////////////////////////////////////
@@ -219,7 +219,7 @@ PBoolean XMPP::Stream::Close()
     return PIndirectChannel::Close();
   }
 
-  return PFalse;
+  return false;
 }
 
 
@@ -241,11 +241,11 @@ PBoolean XMPP::Stream::Write(const PXML& pdu)
   PXMLElement * root = pdu.GetRootElement();
 
   if (root == NULL)
-    return PFalse;
+    return false;
 
   PStringStream os;
   root->Output(os, pdu, 0);
-  return Write(os.GetPointer(), os.GetLength());
+  return Write((const char *)os, os.GetLength());
 }
 
 
@@ -266,7 +266,7 @@ void XMPP::Stream::Reset()
 XMPP::BaseStreamHandler::BaseStreamHandler()
   : PThread(0x1000, NoAutoDeleteThread, NormalPriority, "XMPP"),
     m_Stream(NULL),
-    m_AutoReconnect(PTrue),
+    m_AutoReconnect(true),
     m_ReconnectTimeout(1000)
 {
 }
@@ -284,11 +284,11 @@ PBoolean XMPP::BaseStreamHandler::Start(XMPP::Transport * transport)
     Stop();
 
   m_Stream = new XMPP::Stream();
-  m_Stream->OpenHandlers().Add(new PCREATE_NOTIFIER(OnOpen));
-  m_Stream->CloseHandlers().Add(new PCREATE_NOTIFIER(OnClose));
+  m_Stream->OpenHandlers().Add(PCREATE_NOTIFIER(OnOpen));
+  m_Stream->CloseHandlers().Add(PCREATE_NOTIFIER(OnClose));
 
   if (!transport->IsOpen() && !transport->Open())
-    return PFalse;
+    return false;
 
   if (m_Stream->Open(transport))
   {
@@ -296,17 +296,17 @@ PBoolean XMPP::BaseStreamHandler::Start(XMPP::Transport * transport)
       Resume();
     else
       Restart();
-    return PTrue;
+    return true;
   }
 
-  return PFalse;
+  return false;
 }
 
 
 PBoolean XMPP::BaseStreamHandler::Stop(const PString& _error)
 {
   if (m_Stream == NULL)
-    return PFalse;
+    return false;
 
   if (!_error.IsEmpty())
   {
@@ -324,16 +324,16 @@ PBoolean XMPP::BaseStreamHandler::Stop(const PString& _error)
   delete m_Stream;
   m_Stream = NULL;
 
-  return PFalse;
+  return false;
 }
 
 
-void XMPP::BaseStreamHandler::OnOpen(XMPP::Stream&, INT)
+void XMPP::BaseStreamHandler::OnOpen(XMPP::Stream&, P_INT_PTR)
 {
 }
 
 
-void XMPP::BaseStreamHandler::OnClose(XMPP::Stream&, INT)
+void XMPP::BaseStreamHandler::OnClose(XMPP::Stream&, P_INT_PTR)
 {
 }
 
@@ -348,7 +348,7 @@ void XMPP::BaseStreamHandler::SetAutoReconnect(PBoolean b, long t)
 PBoolean XMPP::BaseStreamHandler::Write(const void * buf, PINDEX len)
 {
   if (m_Stream == NULL)
-    return PFalse;
+    return false;
 
   return m_Stream->Write(buf, len);
 }
@@ -357,7 +357,7 @@ PBoolean XMPP::BaseStreamHandler::Write(const void * buf, PINDEX len)
 PBoolean XMPP::BaseStreamHandler::Write(const PString& data)
 {
   if (m_Stream == NULL)
-    return PFalse;
+    return false;
 
   return m_Stream->Write(data);
 }
@@ -366,7 +366,7 @@ PBoolean XMPP::BaseStreamHandler::Write(const PString& data)
 PBoolean XMPP::BaseStreamHandler::Write(const PXML& pdu)
 {
   if (m_Stream == NULL)
-    return PFalse;
+    return false;
 
   return m_Stream->Write(pdu);
 }
@@ -374,7 +374,7 @@ PBoolean XMPP::BaseStreamHandler::Write(const PXML& pdu)
 
 void XMPP::BaseStreamHandler::OnElement(PXML& pdu)
 {
-  m_ElementHandlers.Fire(pdu);
+  m_ElementHandlers(pdu, 0);
 }
 
 
@@ -393,7 +393,7 @@ void XMPP::BaseStreamHandler::Main()
     {
 #if PTRACING
       if (PTrace::CanTrace(5)) {
-        ostream& os = PTrace::Begin(5, __FILE__, __LINE__);
+        ostream& os = PTRACE_BEGIN(5);
         os << "XMPP\tRCV: ";
         pdu->GetRootElement()->Output(os, *pdu, 0);
         os << PTrace::End;
@@ -411,9 +411,9 @@ void XMPP::BaseStreamHandler::Main()
 
 ///////////////////////////////////////////////////////
 
-const PString & XMPP::Stanza::IDTag()   { static PString s = "id";   return s; }
-const PString & XMPP::Stanza::FromTag() { static PString s = "from"; return s; }
-const PString & XMPP::Stanza::ToTag()   { static PString s = "to";   return s; }
+const PCaselessString & XMPP::Stanza::IDTag()   { static const PConstCaselessString s("id");   return s; }
+const PCaselessString & XMPP::Stanza::FromTag() { static const PConstCaselessString s("from"); return s; }
+const PCaselessString & XMPP::Stanza::ToTag()   { static const PConstCaselessString s("to");   return s; }
 
 void XMPP::Stanza::SetID(const PString& id)
 { 
@@ -470,10 +470,10 @@ PString XMPP::Stanza::GenerateID()
 
 ///////////////////////////////////////////////////////
 
-const PString & XMPP::Message::TypeTag()    { static PString s = "type";    return s; }
-const PString & XMPP::Message::SubjectTag() { static PString s = "subject"; return s; }
-const PString & XMPP::Message::BodyTag()    { static PString s = "body";    return s; }
-const PString & XMPP::Message::ThreadTag()  { static PString s = "thread";  return s; }
+const PCaselessString & XMPP::Message::TypeTag()    { static const PConstCaselessString s("type");    return s; }
+const PCaselessString & XMPP::Message::SubjectTag() { static const PConstCaselessString s("subject"); return s; }
+const PCaselessString & XMPP::Message::BodyTag()    { static const PConstCaselessString s("body");    return s; }
+const PCaselessString & XMPP::Message::ThreadTag()  { static const PConstCaselessString s("thread");  return s; }
 
 XMPP::Message::Message()
 {
@@ -692,10 +692,10 @@ void XMPP::Message::SetThread(const PString& thrd)
 
 ///////////////////////////////////////////////////////
 
-const PString & XMPP::Presence::TypeTag()    { static PString s = "type";     return s; }
-const PString & XMPP::Presence::ShowTag()    { static PString s = "show";     return s; }
-const PString & XMPP::Presence::StatusTag()  { static PString s = "status";   return s; }
-const PString & XMPP::Presence::PriorityTag(){ static PString s = "priority"; return s; }
+const PCaselessString & XMPP::Presence::TypeTag()    { static const PConstCaselessString s("type");     return s; }
+const PCaselessString & XMPP::Presence::ShowTag()    { static const PConstCaselessString s("show");     return s; }
+const PCaselessString & XMPP::Presence::StatusTag()  { static const PConstCaselessString s("status");   return s; }
+const PCaselessString & XMPP::Presence::PriorityTag(){ static const PConstCaselessString s("priority"); return s; }
 
 XMPP::Presence::Presence()
 {
@@ -951,11 +951,11 @@ void XMPP::Presence::SetStatus(const PString& status, const PString& lang)
 
 ///////////////////////////////////////////////////////
 
-const PString & XMPP::IQ::TypeTag() { static PString s = "type"; return s; }
+const PCaselessString & XMPP::IQ::TypeTag() { static const PConstCaselessString s("type"); return s; }
 
 
 XMPP::IQ::IQ(XMPP::IQ::IQType type, PXMLElement * body)
-  : m_Processed(PFalse),
+  : m_Processed(false),
     m_OriginalIQ(NULL)
 {
   SetRootElement(new PXMLElement(NULL, XMPP::IQStanzaTag()));
@@ -967,7 +967,7 @@ XMPP::IQ::IQ(XMPP::IQ::IQType type, PXMLElement * body)
 
 
 XMPP::IQ::IQ(PXML& pdu)
-  : m_Processed(PFalse),
+  : m_Processed(false),
     m_OriginalIQ(NULL)
 {
   if (XMPP::IQ::IsValid(&pdu)) {
@@ -980,7 +980,7 @@ XMPP::IQ::IQ(PXML& pdu)
 
 
 XMPP::IQ::IQ(PXML * pdu)
-  : m_Processed(PFalse),
+  : m_Processed(false),
     m_OriginalIQ(NULL)
 {
   if (XMPP::IQ::IsValid(pdu)) {
@@ -1009,18 +1009,18 @@ PBoolean XMPP::IQ::IsValid(const PXML * pdu)
   PXMLElement * elem = PAssertNULL(pdu)->GetRootElement();
 
   if (elem == NULL || elem->GetName() != XMPP::IQStanzaTag())
-    return PFalse;
+    return false;
 
   PString s = elem->GetAttribute(XMPP::IQ::TypeTag());
 
   if (s.IsEmpty() || (s != "get" && s != "set" && s != "result" && s != "error"))
-    return PFalse;
+    return false;
 
   /* Appartently when a server sends a set to us there's no id...
   s = elem->GetAttribute(XMPP::IQ::ID);
   return !s.IsEmpty();
   */
-  return PTrue;
+  return true;
 }
 
 
@@ -1280,9 +1280,9 @@ PXMLElement * XMPP::Disco::Info::AsXML(PXMLElement * parent) const
 
   m_Identities.AsXML(info);
 
-  for (PINDEX i = 0, max = m_Features.GetSize() ; i < max ; i++) {
+  for (PStringSet::const_iterator it = m_Features.begin(); it != m_Features.end(); ++it) {
     PXMLElement * feature = info->AddChild(new PXMLElement(info, "feature"));
-    feature->SetAttribute("var", m_Features.GetKeyAt(i));
+    feature->SetAttribute("var", *it);
   }
 
   return info;
