@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24875 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2010-11-12 02:03:15 -0600 (Fri, 12 Nov 2010) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #ifdef __GNUC__
@@ -38,9 +38,7 @@
 
 #if P_EXPAT
 
-#if P_DNS
 #include <ptclib/pdns.h>
-#endif
 #include <ptclib/cypher.h>
 
 
@@ -52,7 +50,7 @@ XMPP::C2S::TCPTransport::TCPTransport(const PString& hostname)
   : m_Hostname(hostname),
     m_Port(5222)
 {
-#if P_DNS
+#if P_DNS_RESOLVER
   PDNS::SRVRecordList srvRecords;
 
   if (PDNS::GetSRVRecords(PString("_xmpp-client._tcp.") + hostname, srvRecords)) {
@@ -105,7 +103,7 @@ XMPP::C2S::StreamHandler::StreamHandler(const JID& jid, const PString& pwd, PBoo
 #if P_SASL
     m_SASL("xmpp", BareJID(m_JID), m_JID.GetUser(), m_Password),
 #endif
-    m_HasBind(PFalse), m_HasSession(PFalse),
+    m_HasBind(false), m_HasSession(false),
     m_State(XMPP::C2S::StreamHandler::Null)
 {
   m_PendingIQs.DisallowDeleteObjects();
@@ -116,7 +114,7 @@ XMPP::C2S::StreamHandler::~StreamHandler()
 {
   m_PendingIQsLock.Wait();
   while (m_PendingIQs.GetSize() > 0)
-    delete m_PendingIQs.RemoveAt(0);
+    delete m_PendingIQs.RemoveHead();
   m_PendingIQsLock.Signal();
 }
 
@@ -133,7 +131,7 @@ PBoolean XMPP::C2S::StreamHandler::Start(Transport * transport)
 PBoolean XMPP::C2S::StreamHandler::Send(XMPP::Stanza * stanza)
 {
   if (!stanza)
-    return PFalse;
+    return false;
 
   if (PIsDescendant(stanza, XMPP::IQ)) {
     XMPP::IQ * iq = (XMPP::IQ *)stanza;
@@ -144,11 +142,11 @@ PBoolean XMPP::C2S::StreamHandler::Send(XMPP::Stanza * stanza)
         m_PendingIQsLock.Wait();
         m_PendingIQs.Append(iq);
         m_PendingIQsLock.Signal();
-        return PTrue;
+        return true;
       }
       else {
         delete iq;
-        return PFalse;
+        return false;
       }
     }
   }
@@ -202,15 +200,18 @@ PNotifierList& XMPP::C2S::StreamHandler::MessageSenderHandlers(const JID& from)
 }
 
 
-PBoolean XMPP::C2S::StreamHandler::Discover(const PString& xmlns, const PString& jid, PNotifier * responseHandler, const PString& node)
+PBoolean XMPP::C2S::StreamHandler::Discover(const PString& xmlns,
+                                            const PString& jid,
+                                            const PNotifier & responseHandler,
+                                            const PString& node)
 {
   if (!IsEstablished()) {
     PTRACE(1, "XMPP\tDisco: invalid stream state");
-    return PFalse;
+    return false;
   }
-  else if (responseHandler == NULL) {
+  else if (responseHandler.IsNULL()) {
     PTRACE(1, "XMPP\tDisco: invalid response handler");
-    return PFalse;
+    return false;
   }
 
   PXMLElement * query = new PXMLElement(NULL, XMPP::IQQueryTag());
@@ -227,7 +228,7 @@ PBoolean XMPP::C2S::StreamHandler::Discover(const PString& xmlns, const PString&
 }
 
 
-PBoolean XMPP::C2S::StreamHandler::DiscoverItems(const PString& jid, PNotifier * responseHandler, const PString& node)
+PBoolean XMPP::C2S::StreamHandler::DiscoverItems(const PString& jid, const PNotifier & responseHandler, const PString& node)
 {
   if (node.IsEmpty())
     PTRACE(3, "XMPP\tDisco: discovering items for " << jid);
@@ -239,7 +240,7 @@ PBoolean XMPP::C2S::StreamHandler::DiscoverItems(const PString& jid, PNotifier *
 }
 
 
-PBoolean XMPP::C2S::StreamHandler::DiscoverInfo(const PString& jid, PNotifier * responseHandler, const PString& node)
+PBoolean XMPP::C2S::StreamHandler::DiscoverInfo(const PString& jid, const PNotifier & responseHandler, const PString& node)
 {
   if (node.IsEmpty())
     PTRACE(3, "XMPP\tDisco: discovering info for " << jid);
@@ -271,32 +272,33 @@ void XMPP::C2S::StreamHandler::OnOpen(XMPP::Stream& stream, INT extra)
      server support the new XMPP protocol or it is an old jabber server
   */
   PString data;
-  PString buf;
   PINDEX beg = P_MAX_INDEX, end = P_MAX_INDEX;
 
   while(beg == P_MAX_INDEX || end == P_MAX_INDEX) {
-    if (!((PChannel&)stream).Read(buf.GetPointer(256), 255)) {
+    char buffer[256];
+    buffer[sizeof(buffer)-1] = '\0';
+    if (!((PChannel&)stream).Read(buffer, sizeof(buffer)-1)) {
       // Error!!!
       stream.Close();
       return;
     }
-    data += buf;
+    data += buffer;
   
     if (beg == P_MAX_INDEX)
       beg = data.Find("<stream:stream ");
     if (beg != P_MAX_INDEX)
       end = data.Find('>', beg);
     if (end != P_MAX_INDEX) {
-      buf = data.Mid(beg, end - beg + 1);
-      PINDEX v = buf.Find("version='");
+      PString str = data.Mid(beg, end - beg + 1);
+      PINDEX v = str.Find("version='");
 
       if (v == P_MAX_INDEX) {
         m_VersionMajor = 0;
         m_VersionMinor = 9;
       } else {
-        buf = buf.Mid(v + 9);
+        str = str.Mid(v + 9);
         int maj, min;
-        if (sscanf(buf.GetPointer(), "%d.%d", &maj, &min) != 2) {
+        if (sscanf(str, "%d.%d", &maj, &min) != 2) {
           m_VersionMajor = 0;
           m_VersionMinor = 9;
         } else {
@@ -310,7 +312,7 @@ void XMPP::C2S::StreamHandler::OnOpen(XMPP::Stream& stream, INT extra)
   PXMLStreamParser * parser = stream.GetParser();
 
   // Now we have to feed to the parser whatever we read so far
-  if (parser == NULL || !parser->Parse(data, data.GetLength(), PFalse)) {
+  if (parser == NULL || !parser->Parse(data, data.GetLength(), false)) {
     // Error!!!
     stream.Close();
     return;
@@ -331,8 +333,8 @@ void XMPP::C2S::StreamHandler::OnOpen(XMPP::Stream& stream, INT extra)
 void XMPP::C2S::StreamHandler::OnClose(XMPP::Stream& stream, INT extra)
 {
   SetState(XMPP::C2S::StreamHandler::Null);
-  m_HasBind = PFalse;
-  m_HasSession = PFalse;
+  m_HasBind = false;
+  m_HasSession = false;
   PString streamOff("</stream:stream>");
 
   stream.Write(streamOff);
@@ -406,13 +408,13 @@ void XMPP::C2S::StreamHandler::StartAuthNegotiation()
 
 void XMPP::C2S::StreamHandler::OnSessionEstablished()
 {
-  m_SessionEstablishedHandlers.Fire(*this);
+  m_SessionEstablishedHandlers(*this, 0);
 }
 
 
 void XMPP::C2S::StreamHandler::OnSessionReleased()
 {
-  m_SessionReleasedHandlers.Fire(*this);
+  m_SessionReleasedHandlers(*this, 0);
 }
 
 
@@ -531,7 +533,7 @@ void XMPP::C2S::StreamHandler::HandleRegStartedState(PXML& pdu)
     return;
   }
 
-  m_NewAccount = PFalse;
+  m_NewAccount = false;
   StartAuthNegotiation();
 }
 
@@ -603,19 +605,19 @@ void XMPP::C2S::StreamHandler::HandleNonSASLStartedState(PXML& pdu)
 
     PINDEX i = 0;
     PXMLElement * item;
-    PBoolean uid = PFalse, pwd = PFalse, digest = PFalse, res = PFalse;
+    PBoolean uid = false, pwd = false, digest = false, res = false;
 
     while ((item = (PXMLElement *)elem->GetElement(i++)) != NULL) {
       PString name = item->GetName();
 
       if (name *= "username")
-        uid = PTrue;
+        uid = true;
       else if (name *= "password")
-        pwd = PTrue;
+        pwd = true;
       else if (name *= "digest")
-        digest = PTrue;
+        digest = true;
       else if (name *= "resource")
-        res = PTrue;
+        res = true;
     }
 
     if (uid)
@@ -775,7 +777,7 @@ void XMPP::C2S::StreamHandler::HandleEstablishedState(PXML& pdu)
 
 void XMPP::C2S::StreamHandler::OnError(PXML& pdu)
 {
-  m_ErrorHandlers.Fire(pdu);
+  m_ErrorHandlers(pdu, 0);
 }
 
 
@@ -786,14 +788,14 @@ void XMPP::C2S::StreamHandler::OnMessage(XMPP::Message& pdu)
   /* Fire the generic message handles only if there isn't a notifier list 
   for this particular originator or the list is empty
   */
-  if (!m_MessageSenderHandlers.Contains(from) || !m_MessageSenderHandlers[from].Fire(pdu))
-    m_MessageHandlers.Fire(pdu);
+  if (!m_MessageSenderHandlers.Contains(from) || !m_MessageSenderHandlers[from](pdu, 0))
+    m_MessageHandlers(pdu, 0);
 }
 
 
 void XMPP::C2S::StreamHandler::OnPresence(XMPP::Presence& pdu)
 {
-  m_PresenceHandlers.Fire(pdu);
+  m_PresenceHandlers(pdu, 0);
 }
 
 
@@ -815,17 +817,17 @@ void XMPP::C2S::StreamHandler::OnIQ(XMPP::IQ& pdu)
   }
 
   if (origMsg != NULL)
-    origMsg->GetResponseHandlers().Fire(pdu);
+    origMsg->GetResponseHandlers()(pdu, 0);
 
   // Let's see if someone is registered to handle this namespace
   PXMLElement * query = (PXMLElement *)pdu.GetRootElement()->GetElement(0);
   PString xmlns = query != NULL ? query->GetAttribute(XMPP::NamespaceTag()) : PString::Empty();
 
   if (!xmlns.IsEmpty() && m_IQNamespaceHandlers.Contains(xmlns))
-    m_IQNamespaceHandlers[xmlns].Fire(pdu);
+    m_IQNamespaceHandlers[xmlns](pdu, 0);
 
   // Now the "normal" handlers
-  m_IQHandlers.Fire(pdu);
+  m_IQHandlers(pdu, 0);
 
   // If it was a set or a get and nobody took care of it, we send and error back
   if ((type == XMPP::IQ::Set || type == XMPP::IQ::Get) && !pdu.HasBeenProcessed()) {
