@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 27322 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-03-26 16:57:17 -0500 (Mon, 26 Mar 2012) $
  */
 
 #include <ptlib.h>
@@ -36,12 +36,12 @@
 #include <ctype.h>
 #include <signal.h>
 
-void PAssertFunc (const char * msg)
+bool PAssertFunc (const char * msg)
 
 {
   PError << msg << endl;
   BREAKPOINT;
-  return;
+  return false;
 }
 
 // End Of File ///////////////////////////////////////////////////////////////
