@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21708 $
- * $Author: csoutheren $
- * $Date: 2008-12-07 23:47:50 +0000 (Sun, 07 Dec 2008) $
+ * $Revision: 28484 $
+ * $Author: rjongbloed $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #ifdef __GNUC__
@@ -51,9 +51,6 @@
 #include <signal.h>
 #endif
 
-#ifdef P_LINUX
-extern PString PX_GetThreadName(pthread_t);
-#endif
 
 PFactoryBase::FactoryMap & PFactoryBase::GetFactories()
 {
@@ -61,11 +58,6 @@ PFactoryBase::FactoryMap & PFactoryBase::GetFactories()
   return factories;
 }
 
-PMutex & PFactoryBase::GetFactoriesMutex()
-{
-  static PMutex mutex;
-  return mutex;
-}
 
 PFactoryBase::FactoryMap::~FactoryMap()
 {
@@ -76,13 +68,43 @@ PFactoryBase::FactoryMap::~FactoryMap()
   }  
 }
 
+
+
+void PFactoryBase::FactoryMap::DestroySingletons()
+{
+  Wait();
+  for (iterator it = begin(); it != end(); ++it)
+    it->second->DestroySingletons();
+  Signal();
+}
+
+
+PFactoryBase & PFactoryBase::InternalGetFactory(const std::string & className, PFactoryBase * (*createFactory)())
+{
+  FactoryMap & factories = GetFactories();
+  PWaitAndSignal mutex(factories);
+
+  FactoryMap::const_iterator entry = factories.find(className);
+  if (entry != factories.end()) {
+    PAssert(entry->second != NULL, "Factory map returned NULL for existing key");
+    return *entry->second;
+  }
+
+  PMEMORY_IGNORE_ALLOCATIONS_FOR_SCOPE;
+  PFactoryBase * factory = createFactory();
+  factories[className] = factory;
+
+  return *factory;
+}
+
+
 #if !P_USE_ASSERTS
 
 #define PAssertFunc(a, b, c, d) { }
 
 #else
 
-void PAssertFunc(const char * file,
+bool PAssertFunc(const char * file,
                  int line,
                  const char * className,
                  PStandardAssertMessage msg)
@@ -93,8 +115,7 @@ void PAssertFunc(const char * file,
     static const char fmt[] = "Out of memory at file %.100s, line %u, class %.30s";
     char msgbuf[sizeof(fmt)+100+10+30];
     sprintf(msgbuf, fmt, file, line, className);
-    PAssertFunc(msgbuf);
-    return;
+    return PAssertFunc(msgbuf);
   }
 
   static const char * const textmsg[PMaxStandardAssertMessage] = {
@@ -121,11 +142,11 @@ void PAssertFunc(const char * file,
     sprintf(msgbuf, "Assertion %i", msg);
     theMsg = msgbuf;
   }
-  PAssertFunc(file, line, className, theMsg);
+  return PAssertFunc(file, line, className, theMsg);
 }
 
 
-void PAssertFunc(const char * file, int line, const char * className, const char * msg)
+bool PAssertFunc(const char * file, int line, const char * className, const char * msg)
 {
 #if defined(_WIN32)
   DWORD err = GetLastError();
@@ -144,19 +165,31 @@ void PAssertFunc(const char * file, int line, const char * className, const char
     str << ", Error=" << err;
   str << ends;
   
-  PAssertFunc(str.str().c_str());
+  return PAssertFunc(str.str().c_str());
 }
 
 #endif // !P_USE_ASSERTS
 
-PObject::Comparison PObject::CompareObjectMemoryDirect(const PObject&obj) const
+PObject::Comparison PObject::CompareObjectMemoryDirect(const PObject & obj) const
 {
-  int retval = memcmp(this, &obj, sizeof(PObject));
+  return InternalCompareObjectMemoryDirect(this, &obj, sizeof(obj));
+}
+
+
+PObject::Comparison PObject::InternalCompareObjectMemoryDirect(const PObject * obj1,
+                                                               const PObject * obj2,
+                                                               PINDEX size)
+{
+  if (obj2 == NULL)
+    return PObject::LessThan;
+  if (obj1 == NULL)
+    return PObject::GreaterThan;
+  int retval = memcmp((const void *)obj1, (const void *)obj2, size);
   if (retval < 0)
-    return LessThan;
+    return PObject::LessThan;
   if (retval > 0)
-    return GreaterThan;
-  return EqualTo;
+    return PObject::GreaterThan;
+  return PObject::EqualTo;
 }
 
 
@@ -285,6 +318,7 @@ void operator delete[](void * ptr)
 
 DWORD PMemoryHeap::allocationBreakpoint = 0;
 char PMemoryHeap::Header::GuardBytes[NumGuardBytes];
+static const size_t MaxMemoryDumBytes = 16;
 
 
 PMemoryHeap::Wrapper::Wrapper()
@@ -329,7 +363,7 @@ PMemoryHeap::Wrapper::~Wrapper()
 
 PMemoryHeap::PMemoryHeap()
 {
-  isDestroyed = PFalse;
+  isDestroyed = false;
 
   listHead = NULL;
   listTail = NULL;
@@ -372,13 +406,14 @@ PMemoryHeap::PMemoryHeap()
 
 PMemoryHeap::~PMemoryHeap()
 {
+  isDestroyed = true;
+
   if (leakDumpStream != NULL) {
+    *leakDumpStream << "Final memory statistics:\n";
     InternalDumpStatistics(*leakDumpStream);
     InternalDumpObjectsSince(firstRealObject, *leakDumpStream);
   }
 
-  isDestroyed = PTrue;
-
 #if defined(_WIN32)
   DeleteCriticalSection(&mutex);
   PProcess::Current().WaitOnExitConsoleWindow();
@@ -430,15 +465,8 @@ void * PMemoryHeap::InternalAllocate(size_t nSize, const char * file, int line,
   if (firstRealObject == 0 && (flags&NoLeakPrint) == 0)
     firstRealObject = allocationRequest;
 
-  if (allocationBreakpoint != 0 && allocationRequest == allocationBreakpoint) {
-#ifdef _WIN32
-    __asm int 3;
-#elif defined(P_VXWORKS)
-    kill(taskIdSelf(), SIGABRT);
-#else
-    kill(getpid(), SIGABRT);
-#endif 
-  }
+  if (allocationBreakpoint != 0 && allocationRequest == allocationBreakpoint)
+    PBreakToDebugger();
 
   currentMemoryUsage += nSize;
   if (currentMemoryUsage > peakMemoryUsage)
@@ -456,9 +484,7 @@ void * PMemoryHeap::InternalAllocate(size_t nSize, const char * file, int line,
   obj->size      = nSize;
   obj->fileName  = file;
   obj->line      = (WORD)line;
-#ifdef P_LINUX
-   obj->thread    = pthread_self();
-#endif
+   obj->threadId = PThread::GetCurrentThreadId();
   obj->className = className;
   obj->request   = allocationRequest++;
   obj->flags     = flags;
@@ -502,15 +528,8 @@ void * PMemoryHeap::Reallocate(void * ptr, size_t nSize, const char * file, int
     return NULL;
   }
 
-  if (mem->allocationBreakpoint != 0 && mem->allocationRequest == mem->allocationBreakpoint) {
-#ifdef _WIN32
-    __asm int 3;
-#elif defined(P_VXWORKS)
-    kill(taskIdSelf(), SIGABRT);
-#else
-    kill(getpid(), SIGABRT);
-#endif 
-  }
+  if (mem->allocationBreakpoint != 0 && mem->allocationRequest == mem->allocationBreakpoint)
+    PBreakToDebugger();
 
   mem->currentMemoryUsage -= obj->size;
   mem->currentMemoryUsage += nSize;
@@ -599,11 +618,12 @@ PMemoryHeap::Validation PMemoryHeap::InternalValidate(const void * ptr,
 
   Header * obj = ((Header *)ptr)-1;
 
+  unsigned count = currentObjects;
   Header * link = listTail;  
-  while (link != NULL && link != obj) 
+  while (link != NULL && link != obj && count-- > 0) 
     link = link->prev;  
 
-  if (link == NULL) {
+  if (link != obj) {
     if (error != NULL)
       *error << "Block " << ptr << " not in heap!" << endl;
     return Trashed;
@@ -648,13 +668,13 @@ PBoolean PMemoryHeap::ValidateHeap(ostream * error)
     if (memcmp(obj->guard, obj->GuardBytes, sizeof(obj->guard)) != 0) {
       if (error != NULL)
         *error << "Underrun at " << (obj+1) << '[' << obj->size << "] #" << obj->request << endl;
-      return PFalse;
+      return false;
     }
   
     if (memcmp((char *)(obj+1)+obj->size, obj->GuardBytes, sizeof(obj->guard)) != 0) {
       if (error != NULL)
         *error << "Overrun at " << (obj+1) << '[' << obj->size << "] #" << obj->request << endl;
-      return PFalse;
+      return false;
     }
 
     obj = obj->next;
@@ -664,12 +684,12 @@ PBoolean PMemoryHeap::ValidateHeap(ostream * error)
   if (!_CrtCheckMemory()) {
     if (error != NULL)
       *error << "Heap failed MSVCRT validation!" << endl;
-    return PFalse;
+    return false;
   }
 #endif
   if (error != NULL)
     *error << "Heap passed validation." << endl;
-  return PTrue;
+  return true;
 }
 
 
@@ -755,7 +775,7 @@ void PMemoryHeap::DumpObjectsSince(const State & state, ostream & strm)
 
 void PMemoryHeap::InternalDumpObjectsSince(DWORD objectNumber, ostream & strm)
 {
-  PBoolean first = PTrue;
+  bool first = true;
   for (Header * obj = listHead; obj != NULL; obj = obj->next) {
     if (obj->request < objectNumber || (obj->flags&NoLeakPrint) != 0)
       continue;
@@ -765,7 +785,7 @@ void PMemoryHeap::InternalDumpObjectsSince(DWORD objectNumber, ostream & strm)
 #if !defined(_WIN32)
       cin.get();
 #endif
-      first = PFalse;
+      first = false;
     }
 
     BYTE * data = (BYTE *)&obj[1];
@@ -775,14 +795,19 @@ void PMemoryHeap::InternalDumpObjectsSince(DWORD objectNumber, ostream & strm)
 
     strm << '#' << obj->request << ' ' << (void *)data << " [" << obj->size << "] ";
 
-#ifdef P_LINUX
-    strm << '"' << PX_GetThreadName(obj->thread) << "\" ";
-#endif
-
     if (obj->className != NULL)
-      strm << '"' << obj->className << "\" ";
+      strm << "class=\"" << obj->className << "\" ";
 
-    strm << '\n' << hex << setfill('0') << PBYTEArray(data, PMIN(16, obj->size), PFalse)
+    if (PProcess::IsInitialised() && obj->threadId != PNullThreadIdentifier) {
+      strm << "thread=";
+      PThread * thread = PProcess::Current().GetThread(obj->threadId);
+      if (thread != NULL)
+        strm << '"' << thread->GetThreadName() << "\" ";
+      else
+        strm << "0x" << hex << obj->threadId << dec << ' ';
+    }
+
+    strm << '\n' << hex << setfill('0') << PBYTEArray(data, std::min(MaxMemoryDumBytes, obj->size), false)
                  << dec << setfill(' ') << endl;
   }
 }
@@ -823,30 +848,29 @@ PMemoryHeap::~PMemoryHeap()
 }
 
 
-static PMemoryHeap & MemoryHeapInstance()
+void PMemoryHeap::CreateInstance()
 {
   static PMemoryHeap instance;
-  return instance;
 }
 
 
 void * PMemoryHeap::Allocate(size_t nSize, const char * file, int line, const char * className)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   return _malloc_dbg(nSize, className != NULL ? P_CLIENT_BLOCK : _NORMAL_BLOCK, file, line);
 }
 
 
 void * PMemoryHeap::Allocate(size_t count, size_t iSize, const char * file, int line)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   return _calloc_dbg(count, iSize, _NORMAL_BLOCK, file, line);
 }
 
 
 void * PMemoryHeap::Reallocate(void * ptr, size_t nSize, const char * file, int line)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   return _realloc_dbg(ptr, nSize, _NORMAL_BLOCK, file, line);
 }
 
@@ -859,7 +883,7 @@ void PMemoryHeap::Deallocate(void * ptr, const char * className)
 
 PMemoryHeap::Validation PMemoryHeap::Validate(const void * ptr, const char * className, ostream * /*strm*/)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   if (!_CrtIsValidHeapPointer(ptr))
     return Bad;
 
@@ -873,14 +897,14 @@ PMemoryHeap::Validation PMemoryHeap::Validate(const void * ptr, const char * cla
 
 PBoolean PMemoryHeap::ValidateHeap(ostream * /*strm*/)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   return _CrtCheckMemory();
 }
 
 
 PBoolean PMemoryHeap::SetIgnoreAllocations(PBoolean ignore)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   int flags = _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
   if (ignore)
     _CrtSetDbgFlag(flags & ~_CRTDBG_ALLOC_MEM_DF);
@@ -892,7 +916,7 @@ PBoolean PMemoryHeap::SetIgnoreAllocations(PBoolean ignore)
 
 void PMemoryHeap::DumpStatistics()
 {
-  MemoryHeapInstance();
+  CreateInstance();
   _CrtMemState state;
   _CrtMemCheckpoint(&state);
   _CrtMemDumpStatistics(&state);
@@ -907,28 +931,28 @@ void PMemoryHeap::DumpStatistics(ostream & /*strm*/)
 
 void PMemoryHeap::GetState(State & state)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   _CrtMemCheckpoint(&state);
 }
 
 
 void PMemoryHeap::DumpObjectsSince(const State & state)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   _CrtMemDumpAllObjectsSince(&state);
 }
 
 
 void PMemoryHeap::DumpObjectsSince(const State & state, ostream & /*strm*/)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   DumpObjectsSince(state);
 }
 
 
 void PMemoryHeap::SetAllocationBreakpoint(DWORD objectNumber)
 {
-  MemoryHeapInstance();
+  CreateInstance();
   _CrtSetBreakAlloc(objectNumber);
 }
 
@@ -1054,9 +1078,9 @@ void PInt64__::ShiftRight(int bits)
 PBoolean PInt64::Lt(const PInt64 & v) const
 {
   if ((long)high < (long)v.high)
-    return PTrue;
+    return true;
   if ((long)high > (long)v.high)
-    return PFalse;
+    return false;
   if ((long)high < 0)
     return (long)low > (long)v.low;
   return (long)low < (long)v.low;
@@ -1066,9 +1090,9 @@ PBoolean PInt64::Lt(const PInt64 & v) const
 PBoolean PInt64::Gt(const PInt64 & v) const
 {
   if ((long)high > (long)v.high)
-    return PTrue;
+    return true;
   if ((long)high < (long)v.high)
-    return PFalse;
+    return false;
   if ((long)high < 0)
     return (long)low < (long)v.low;
   return (long)low > (long)v.low;
@@ -1078,9 +1102,9 @@ PBoolean PInt64::Gt(const PInt64 & v) const
 PBoolean PUInt64::Lt(const PUInt64 & v) const
 {
   if (high < v.high)
-    return PTrue;
+    return true;
   if (high > v.high)
-    return PFalse;
+    return false;
   return low < high;
 }
 
@@ -1088,9 +1112,9 @@ PBoolean PUInt64::Lt(const PUInt64 & v) const
 PBoolean PUInt64::Gt(const PUInt64 & v) const
 {
   if (high > v.high)
-    return PTrue;
+    return true;
   if (high < v.high)
-    return PFalse;
+    return false;
   return low > high;
 }
 
