@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23845 $
- * $Author: shorne $
- * $Date: 2009-12-07 21:23:45 -0600 (Mon, 07 Dec 2009) $
+ * $Revision: 28941 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 08:18:28 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -61,7 +61,7 @@ streambuf::int_type PChannelStreamBuffer::overflow(int_type c)
     setp(p, p+output.GetSize());
   }
 
-  int bufSize = pptr() - pbase();
+  size_t bufSize = pptr() - pbase();
   if (bufSize > 0) {
     setp(pbase(), epptr());
     if (!channel->Write(pbase(), bufSize))
@@ -102,11 +102,10 @@ streambuf::int_type PChannelStreamBuffer::underflow()
 
 int PChannelStreamBuffer::sync()
 {
-  int inAvail = egptr() - gptr();
-  if (inAvail > 0) {
+  if (egptr() > gptr()) {
     setg(eback(), egptr(), egptr());
     if (PIsDescendant(channel, PFile))
-      ((PFile *)channel)->SetPosition(-inAvail, PFile::Current);
+      ((PFile *)channel)->SetPosition((off_t)(gptr() - egptr()), PFile::Current);
   }
 
   if (pptr() > pbase())
@@ -148,13 +147,10 @@ PChannelStreamBuffer::pos_type PChannelStreamBuffer::seekpos(pos_type pos, ios_b
 }
 
 
-#ifdef _MSC_VER
-#pragma warning(disable:4355)
-#endif
-
 PChannel::PChannel()
-  : iostream(new PChannelStreamBuffer(this)),
-    readTimeout(PMaxTimeInterval), writeTimeout(PMaxTimeInterval)
+  : P_DISABLE_MSVC_WARNINGS(4355, P_IOSTREAM(new PChannelStreamBuffer(this)))
+  , readTimeout(PMaxTimeInterval)
+  , writeTimeout(PMaxTimeInterval)
 {
   os_handle = -1;
   memset(lastErrorCode, 0, sizeof(lastErrorCode));
@@ -163,10 +159,6 @@ PChannel::PChannel()
   Construct();
 }
 
-#ifdef _MSC_VER
-#pragma warning(default:4355)
-#endif
-
 
 PChannel::~PChannel()
 {
@@ -181,8 +173,8 @@ PChannel::~PChannel()
 PObject::Comparison PChannel::Compare(const PObject & obj) const
 {
   PAssert(PIsDescendant(&obj, PChannel), PInvalidCast);
-  int h1 = GetHandle();
-  int h2 = ((const PChannel&)obj).GetHandle();
+  P_INT_PTR h1 = GetHandle();
+  P_INT_PTR h2 = ((const PChannel&)obj).GetHandle();
   if (h1 < h2)
     return LessThan;
   if (h1 > h2)
@@ -251,23 +243,15 @@ PString PChannel::ReadString(PINDEX len)
   PString str;
 
   if (len == P_MAX_INDEX) {
-    PINDEX l = 0;
     for (;;) {
-      char * p = l + str.GetPointer(l+1000+1);
-      if (!Read(p, 1000))
+      char chunk[1000];
+      if (!Read(chunk, sizeof(chunk)))
         break;
-      l += lastReadCount;
+      str += PString(chunk, lastReadCount);
     }
-    str.SetSize(l+1);
-
-    /*Need to put in a null at the end to allow for MSDOS/Win32 text files
-      which returns fewer bytes than actually read as it shrinks the data into
-      the removed carriage returns, so it actually changes the buffer beyond
-      what it indicated. */
-    str[l] = '\0';
   }
   else {
-    if (!ReadBlock(str.GetPointer(len+1), len))
+    if (!ReadBlock(str.GetPointerAndSetLength(len), len))
       return PString::Empty();
   }
 
@@ -282,25 +266,12 @@ PBoolean PChannel::WriteString(const PString & str)
   while (written < len) {
     if (!Write((const char *)str + written, len - written)) {
       lastWriteCount += written;
-      return PFalse;
+      return false;
     }
     written += lastWriteCount;
   }
   lastWriteCount = written;
-  return PTrue;
-}
-
-
-PBoolean PChannel::ReadAsync(void * buf, PINDEX len)
-{
-  PBoolean retVal = Read(buf, len);
-  OnReadComplete(buf, lastReadCount);
-  return retVal;
-}
-
-
-void PChannel::OnReadComplete(void *, PINDEX)
-{
+  return true;
 }
 
 
@@ -312,19 +283,6 @@ PBoolean PChannel::WriteChar(int c)
 }
 
 
-PBoolean PChannel::WriteAsync(const void * buf, PINDEX len)
-{
-  PBoolean retVal = Write(buf, len);
-  OnWriteComplete(buf, lastWriteCount);
-  return retVal;
-}
-
-
-void PChannel::OnWriteComplete(const void *, PINDEX)
-{
-}
-
-
 PBoolean PChannel::SetBufferSize(PINDEX newSize)
 {
   return ((PChannelStreamBuffer *)rdbuf())->SetBufferSize(newSize);
@@ -454,7 +412,7 @@ PBoolean PChannel::ReceiveCommandString(int nextChar,
 {
   if (nextChar != GetNextChar(reply, pos)) {
     pos = start;
-    return PFalse;
+    return false;
   }
 
   PINDEX dummyPos = pos;
@@ -464,7 +422,7 @@ PBoolean PChannel::ReceiveCommandString(int nextChar,
 
 PBoolean PChannel::SendCommandString(const PString & command)
 {
-  abortCommandString = PFalse;
+  abortCommandString = false;
 
   int nextChar;
   PINDEX sendPosition = 0;
@@ -476,11 +434,11 @@ PBoolean PChannel::SendCommandString(const PString & command)
     switch (nextChar) {
       default :
         if (!WriteChar(nextChar))
-          return PFalse;
+          return false;
         break;
 
       case NextCharEndOfString :
-        return PTrue;  // Success!!
+        return true;  // Success!!
 
       case NextCharSend :
         break;
@@ -495,15 +453,15 @@ PBoolean PChannel::SendCommandString(const PString & command)
           SetReadTimeout(timeout);
           while (ReadChar() >= 0)
             if (abortCommandString) // aborted
-              return PFalse;
+              return false;
         }
         else {
           receivePosition = sendPosition;
           do {
             if (abortCommandString) // aborted
-              return PFalse;
+              return false;
             if ((nextChar = ReadCharWithTimeout(timeout)) < 0)
-              return PFalse;
+              return false;
           } while (!ReceiveCommandString(nextChar,
                                      command, receivePosition, sendPosition));
 //          nextChar = GetNextChar(command, receivePosition);
@@ -512,16 +470,26 @@ PBoolean PChannel::SendCommandString(const PString & command)
     }
   }
 
-  return PFalse;
+  return false;
 }
 
 
 PBoolean PChannel::Shutdown(ShutdownValue)
 {
-  return PFalse;
+  return false;
 }
 
 
+bool PChannel::SetLocalEcho(bool /*localEcho*/)
+{
+  return IsOpen();
+}
+
+bool PChannel::FlowControl(const void * /*flowData*/)
+{
+    return false;
+}
+
 PChannel * PChannel::GetBaseReadChannel() const
 {
   return (PChannel *)this;
@@ -540,7 +508,7 @@ PString PChannel::GetErrorText(ErrorGroup group) const
 }
 
 
-PBoolean PChannel::ConvertOSError(int status, ErrorGroup group)
+PBoolean PChannel::ConvertOSError(P_INT_PTR status, ErrorGroup group)
 {
   Errors lastError;
   int osError;
@@ -557,41 +525,71 @@ PBoolean PChannel::SetErrorValues(Errors errorCode, int errorNum, ErrorGroup gro
   return errorCode == NoError;
 }
 
-#ifndef P_HAS_RECVMSG
 
-PBoolean PChannel::Read(const VectorOfSlice & slices)
+PChannel::AsyncContext::AsyncContext(void * buf, PINDEX len, const AsyncNotifier & notifier)
+  : m_buffer(buf)
+  , m_length(len)
+  , m_notifier(notifier)
+  , m_errorCode(NoError)
+  , m_errorNumber(0)
+  , m_channel(NULL)
+  , m_onComplete(NULL)
 {
-  PINDEX length = 0;
-
-  VectorOfSlice::const_iterator r;
-  for (r = slices.begin(); r != slices.end(); ++r) {
-    PBoolean stat = Read(r->iov_base, r->iov_len);
-    length        += lastReadCount;
-    lastReadCount = length;
-    if (!stat)
-      return PFalse;
-  }
-
-  return PTrue;
+  memset(this, 0, sizeof(AsyncContextBase));
 }
 
-PBoolean PChannel::Write(const VectorOfSlice & slices)
+
+void PChannel::AsyncContext::OnIOComplete(PINDEX length, int errorNumber)
 {
-  PINDEX length = 0;
+  PTRACE(6, m_channel, "AsyncIO", "OnIOComplete: len=" << length << ", error=" << errorNumber);
 
-  VectorOfSlice::const_iterator r;
-  for (r = slices.begin(); r != slices.end(); ++r) {
-    PBoolean stat = Write(r->iov_base, r->iov_len);
-    length        += lastWriteCount;
-    lastWriteCount = length;
-    if (!stat)
-      return PFalse;
-  }
+  m_length = length;
+  m_errorNumber = errorNumber;
+  PChannel::ConvertOSError(-3, m_errorCode, m_errorNumber);
 
-  return PTrue;
+  PChannel * channel = m_channel;
+  m_channel = NULL;
+  (channel->*m_onComplete)(*this);
+}
+
+
+void PChannel::OnReadComplete(AsyncContext & context)
+{
+  if (!context.m_notifier.IsNULL())
+    context.m_notifier(*this, context);
+}
+
+
+void PChannel::OnWriteComplete(AsyncContext & context)
+{
+  if (!context.m_notifier.IsNULL())
+    context.m_notifier(*this, context);
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+// PNullChannel
+
+PNullChannel::PNullChannel()
+{
+  channelName = "null";
+  os_handle = 0;
+}
+
+
+PBoolean PNullChannel::Read(void *, PINDEX)
+{
+  lastReadCount = 0;
+  return false;
+}
+
+
+PBoolean PNullChannel::Write(const void *, PINDEX length)
+{
+  lastWriteCount = length;
+  return true;
 }
 
-#endif // P_HAS_RECVMSG
 
 ///////////////////////////////////////////////////////////////////////////////
 // PIndirectChannel
@@ -599,7 +597,7 @@ PBoolean PChannel::Write(const VectorOfSlice & slices)
 PIndirectChannel::PIndirectChannel()
 {
   readChannel = writeChannel = NULL;
-  writeAutoDelete = readAutoDelete = PFalse;
+  writeAutoDelete = readAutoDelete = false;
 }
 
 
@@ -635,7 +633,7 @@ PString PIndirectChannel::GetName() const
 
 PBoolean PIndirectChannel::Close()
 {
-  PBoolean retval = PTrue;
+  PBoolean retval = true;
 
   flush();
 
@@ -676,7 +674,7 @@ PBoolean PIndirectChannel::IsOpen() const
   if (readChannel != NULL && readChannel == writeChannel)
     return readChannel->IsOpen();
 
-  PBoolean returnValue = readChannel != NULL ? readChannel->IsOpen() : PFalse;
+  PBoolean returnValue = readChannel != NULL ? readChannel->IsOpen() : false;
 
   if (writeChannel != NULL)
     returnValue = writeChannel->IsOpen() || returnValue;
@@ -693,7 +691,7 @@ PBoolean PIndirectChannel::Read(void * buf, PINDEX len)
 
   if (readChannel == NULL) {
     SetErrorValues(NotOpen, EBADF, LastReadError);
-    return PFalse;
+    return false;
   }
 
   readChannel->SetReadTimeout(readTimeout);
@@ -716,7 +714,7 @@ PBoolean PIndirectChannel::Write(const void * buf, PINDEX len)
 
   if (writeChannel == NULL) {
     SetErrorValues(NotOpen, EBADF, LastWriteError);
-    return PFalse;
+    return false;
   }
 
   writeChannel->SetWriteTimeout(writeTimeout);
@@ -739,7 +737,7 @@ PBoolean PIndirectChannel::Shutdown(ShutdownValue value)
   if (readChannel != NULL && readChannel == writeChannel)
     return readChannel->Shutdown(value);
 
-  PBoolean returnValue = readChannel != NULL ? readChannel->Shutdown(value) : PFalse;
+  PBoolean returnValue = readChannel != NULL ? readChannel->Shutdown(value) : false;
 
   if (writeChannel != NULL)
     returnValue = writeChannel->Shutdown(value) || returnValue;
@@ -748,6 +746,13 @@ PBoolean PIndirectChannel::Shutdown(ShutdownValue value)
 }
 
 
+bool PIndirectChannel::SetLocalEcho(bool localEcho)
+{
+  PReadWaitAndSignal mutex(channelPointerMutex);
+  return readChannel != NULL && readChannel->SetLocalEcho(localEcho);
+}
+
+
 PString PIndirectChannel::GetErrorText(ErrorGroup group) const
 {
   if (readChannel != NULL)
@@ -762,7 +767,7 @@ PString PIndirectChannel::GetErrorText(ErrorGroup group) const
 
 PBoolean PIndirectChannel::Open(PChannel & channel)
 {
-  return Open(&channel, (PBoolean)PFalse);
+  return Open(&channel, (PBoolean)false);
 }
 
 
@@ -807,39 +812,47 @@ PBoolean PIndirectChannel::Open(PChannel * readChan,
 
 PBoolean PIndirectChannel::OnOpen()
 {
-  return PTrue;
+  return true;
 }
 
 
-PBoolean PIndirectChannel::SetReadChannel(PChannel * channel, PBoolean autoDelete)
+bool PIndirectChannel::SetReadChannel(PChannel * channel, bool autoDelete, bool closeExisting)
 {
-  if (readChannel != NULL)
-    return SetErrorValues(DeviceInUse, EEXIST);
+  PWriteWaitAndSignal mutex(channelPointerMutex);
 
-  channelPointerMutex.StartWrite();
+  if (closeExisting) {
+    if (readAutoDelete)
+      delete readChannel;
+  }
+  else {
+    if (readChannel != NULL)
+      return SetErrorValues(DeviceInUse, EEXIST);
+  }
 
   readChannel = channel;
   readAutoDelete = autoDelete;
 
-  channelPointerMutex.EndWrite();
-
-  return IsOpen();
+  return channel != NULL && channel->IsOpen();
 }
 
 
-PBoolean PIndirectChannel::SetWriteChannel(PChannel * channel, PBoolean autoDelete)
+bool PIndirectChannel::SetWriteChannel(PChannel * channel, bool autoDelete, bool closeExisting)
 {
-  if (writeChannel != NULL)
-    return SetErrorValues(DeviceInUse, EEXIST);
+  PWriteWaitAndSignal mutex(channelPointerMutex);
 
-  channelPointerMutex.StartWrite();
+  if (closeExisting) {
+    if (writeAutoDelete)
+      delete writeChannel;
+  }
+  else {
+    if (writeChannel != NULL)
+      return SetErrorValues(DeviceInUse, EEXIST);
+  }
 
   writeChannel = channel;
   writeAutoDelete = autoDelete;
 
-  channelPointerMutex.EndWrite();
-
-  return IsOpen();
+  return channel != NULL && channel->IsOpen();
 }
 
 
@@ -878,10 +891,10 @@ PBoolean PFile::Rename(const PString & newname, PBoolean force)
   Close();
 
   if (!ConvertOSError(Rename(path, newname, force) ? 0 : -1))
-    return PFalse;
+    return false;
 
   path = path.GetDirectory() + newname;
-  return PTrue;
+  return true;
 }
 
 
@@ -893,9 +906,9 @@ PBoolean PFile::Close()
   flush();
 
 #ifdef WOT_NO_FILESYSTEM
-  PBoolean ok = PTrue;
+  PBoolean ok = true;
 #else
-  PBoolean ok = ConvertOSError(_close(os_handle));
+  PBoolean ok = ConvertOSError(_close(GetOSHandleAsInt()));
 #endif
 
   os_handle = -1;
@@ -916,7 +929,7 @@ PBoolean PFile::Read(void * buffer, PINDEX amount)
 #ifdef WOT_NO_FILESYSTEM
   lastReadCount = 0;
 #else
-  lastReadCount = _read(GetHandle(), buffer, amount);
+  lastReadCount = _read(GetOSHandleAsInt(), buffer, amount);
 #endif
   return ConvertOSError(lastReadCount, LastReadError) && lastReadCount > 0;
 }
@@ -931,13 +944,13 @@ PBoolean PFile::Write(const void * buffer, PINDEX amount)
 #ifdef WOT_NO_FILESYSTEM
   lastWriteCount = amount;
 #else
-  lastWriteCount = _write(GetHandle(), buffer, amount);
+  lastWriteCount = _write(GetOSHandleAsInt(), buffer, amount);
 #endif
   return ConvertOSError(lastWriteCount, LastWriteError) && lastWriteCount >= amount;
 }
 
 
-PBoolean PFile::Open(const PFilePath & name, OpenMode  mode, int opts)
+PBoolean PFile::Open(const PFilePath & name, OpenMode  mode, OpenOptions opts)
 {
   Close();
   SetFilePath(name);
@@ -953,9 +966,9 @@ off_t PFile::GetLength() const
 #ifdef WOT_NO_FILESYSTEM
   return 0;
 #else
-  off_t pos = _lseek(GetHandle(), 0, SEEK_CUR);
-  off_t len = _lseek(GetHandle(), 0, SEEK_END);
-  PAssertOS(_lseek(GetHandle(), pos, SEEK_SET) != (off_t)-1);
+  off_t pos = _lseek(GetOSHandleAsInt(), 0, SEEK_CUR);
+  off_t len = _lseek(GetOSHandleAsInt(), 0, SEEK_END);
+  PAssertOS(_lseek(GetOSHandleAsInt(), pos, SEEK_SET) != (off_t)-1);
   return len;
 #endif
 }
@@ -974,12 +987,12 @@ PBoolean PFile::IsEndOfFile() const
 PBoolean PFile::SetPosition(off_t pos, FilePositionOrigin origin)
 {
 #ifdef WOT_NO_FILESYSTEM
-  return PTrue;
+  return true;
 #else
   if (!IsOpen())
     return SetErrorValues(NotOpen, EBADF);
 
-  return _lseek(GetHandle(), pos, origin) != (off_t)-1;
+  return _lseek(GetOSHandleAsInt(), pos, origin) != (off_t)-1;
 #endif
 }
 
@@ -988,31 +1001,47 @@ PBoolean PFile::Copy(const PFilePath & oldname, const PFilePath & newname, PBool
 {
   PFile oldfile(oldname, ReadOnly);
   if (!oldfile.IsOpen())
-    return PFalse;
+    return false;
 
   PFile newfile(newname,
                    WriteOnly, Create|Truncate|(force ? MustExist : Exclusive));
   if (!newfile.IsOpen())
-    return PFalse;
+    return false;
 
   PCharArray buffer(10000);
 
   off_t amount = oldfile.GetLength();
   while (amount > 10000) {
     if (!oldfile.Read(buffer.GetPointer(), 10000))
-      return PFalse;
+      return false;
     if (!newfile.Write((const char *)buffer, 10000))
-      return PFalse;
+      return false;
     amount -= 10000;
   }
 
   if (!oldfile.Read(buffer.GetPointer(), (int)amount))
-    return PFalse;
+    return false;
   if (!newfile.Write((const char *)buffer, (int)amount))
-    return PFalse;
+    return false;
 
   return newfile.Close();
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+// PTextFile
+
+PBoolean PTextFile::ReadLine(PString & str)
+{
+  str.ReadFrom(*this);
+  return !str.IsEmpty() || good();
+}
+
+
+PBoolean PTextFile::WriteLine(const PString & str)
+{
+  return WriteString(str) && WriteChar('\n');
+}
+
+
 // End Of File ///////////////////////////////////////////////////////////////
