@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25115 $
- * $Author: rjongbloed $
- * $Date: 2011-02-15 04:27:40 -0600 (Tue, 15 Feb 2011) $
+ * $Revision: 28931 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 06:31:11 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -80,7 +80,7 @@ void PTimeInterval::PrintOn(ostream & stream) const
   Formats fmt = NormalFormat;
   if ((stream.flags()&ios::scientific) != 0)
     fmt = SecondsOnly;
-  else if (precision < 0) {
+  else if (precision > -4 && precision < 0) {
     fmt = IncludeDays;
     precision = -precision;
   }
@@ -116,8 +116,12 @@ PString PTimeInterval::AsString(int precision, Formats format, int width) const
 
   if (precision > 3)
     precision = 3;
-  else if (precision < 0)
-    precision = 0;
+  else if (precision < -9)
+    precision = -9;
+  else if (precision < -6)
+    precision = -6;
+  else if (precision < -3)
+    precision = -3;
 
   PInt64 ms = GetMilliSeconds();
   if (ms < 0) {
@@ -146,14 +150,14 @@ PString PTimeInterval::AsString(int precision, Formats format, int width) const
     return str;
   }
 
-  PBoolean hadPrevious = PFalse;
+  PBoolean hadPrevious = false;
   long tmp;
 
   if (format == IncludeDays) {
     tmp = (long)(ms/86400000);
     if (tmp > 0 || width > (precision+10)) {
       str << tmp << 'd';
-      hadPrevious = PTrue;
+      hadPrevious = true;
     }
 
     tmp = (long)(ms%86400000)/3600000;
@@ -161,24 +165,30 @@ PString PTimeInterval::AsString(int precision, Formats format, int width) const
   else
     tmp = (long)(ms/3600000);
 
-  if (hadPrevious || tmp > 0 || width > (precision+7)) {
-    if (hadPrevious)
-      str.width(2);
-    str << tmp << ':';
-    hadPrevious = PTrue;
+  if (precision >= -9) {
+    if (hadPrevious || tmp > 0 || width > (precision+7)) {
+      if (hadPrevious)
+        str << ':' << setw(2);
+      str << tmp;
+      hadPrevious = true;
+    }
   }
 
-  tmp = (long)(ms%3600000)/60000;
-  if (hadPrevious || tmp > 0 || width > (precision+4)) {
-    if (hadPrevious)
-      str.width(2);
-    str << tmp << ':';
-    hadPrevious = PTrue;
+  if (precision >= -6) {
+    tmp = (long)(ms%3600000)/60000;
+    if (hadPrevious || tmp > 0 || width > (precision+4)) {
+      if (hadPrevious)
+        str << ':' << setw(2);
+      str << tmp;
+      hadPrevious = true;
+    }
   }
 
-  if (hadPrevious)
-    str.width(2);
-  str << (long)(ms%60000)/1000;
+  if (precision >= -3) {
+    if (hadPrevious)
+      str << ':' << setw(2);
+    str << (long)(ms%60000)/1000;
+  }
 
   switch (precision) {
     case 1 :
@@ -451,14 +461,14 @@ PString PTime::AsString(const char * format, int zone) const
   struct tm ts;
   struct tm * t = os_gmtime(&realTime, &ts);
 
-  PINDEX repeatCount;
-
   while (*format != '\0') {
-    repeatCount = 1;
-    switch (*format) {
+    char formatLetter = *format;
+    PINDEX repeatCount = 1;
+    while (*++format == formatLetter)
+      repeatCount++;
+
+    switch (formatLetter) {
       case 'a' :
-        while (*++format == 'a')
-          ;
         if (t->tm_hour < 12)
           str << GetTimeAM();
         else
@@ -466,26 +476,18 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case 'h' :
-        while (*++format == 'h')
-          repeatCount++;
         str << setw(repeatCount) << (is12hour ? (t->tm_hour+11)%12+1 : t->tm_hour);
         break;
 
       case 'm' :
-        while (*++format == 'm')
-          repeatCount++;
         str << setw(repeatCount) << t->tm_min;
         break;
 
       case 's' :
-        while (*++format == 's')
-          repeatCount++;
         str << setw(repeatCount) << t->tm_sec;
         break;
 
       case 'w' :
-        while (*++format == 'w')
-          repeatCount++;
         if (repeatCount != 3 || *format != 'e')
           str << GetDayName((Weekdays)t->tm_wday, repeatCount <= 3 ? Abbreviated : FullName);
         else {
@@ -498,8 +500,6 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case 'M' :
-        while (*++format == 'M')
-          repeatCount++;
         if (repeatCount < 3)
           str << setw(repeatCount) << (t->tm_mon+1);
         else if (repeatCount > 3 || *format != 'E')
@@ -516,14 +516,10 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case 'd' :
-        while (*++format == 'd')
-          repeatCount++;
         str << setw(repeatCount) << t->tm_mday;
         break;
 
       case 'y' :
-        while (*++format == 'y')
-          repeatCount++;
         if (repeatCount < 3)
           str << setw(2) << (t->tm_year%100);
         else
@@ -532,17 +528,13 @@ PString PTime::AsString(const char * format, int zone) const
 
       case 'z' :
       case 'Z' :
-        if (zone == 0) {
-          if (*format == 'Z')
+        if (repeatCount == 1 && zone == 0) {
+          if (formatLetter == 'Z')
             str << 'Z';
           else
             str << "GMT";
-          while (toupper(*++format) == 'z')
-            ;
         }
         else {
-          while (toupper(*++format) == 'z')
-            repeatCount++;
           str << (zone < 0 ? '-' : '+');
           zone = PABS(zone);
           str << setw(2) << (zone/60);
@@ -553,8 +545,6 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case 'u' :
-        while (*++format == 'u')
-          repeatCount++;
         switch (repeatCount) {
           case 1 :
             str << (microseconds/100000);
@@ -572,11 +562,10 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case '\\' :
-        format++;
         // Escaped character, put straight through to output string
 
       default :
-        str << *format++;
+        str << formatLetter;
     }
   }
 
@@ -591,9 +580,11 @@ PString PTime::AsString(const char * format, int zone) const
 extern "C" {
 
 #ifdef _WIN32
-#define STDAPICALLTYPE __stdcall
+#  ifndef STDAPICALLTYPE
+#  define STDAPICALLTYPE __stdcall
+#  endif
 #else
-#define STDAPICALLTYPE
+#  define STDAPICALLTYPE
 #endif
 
 // This funcctions implementation is in getdate_tab.c
@@ -636,7 +627,6 @@ int STDAPICALLTYPE PTimeIsDayName(const char * str, int day, int abbrev)
 };
 
 
-
 void PTime::ReadFrom(istream & strm)
 {
   time_t now;
@@ -648,6 +638,13 @@ void PTime::ReadFrom(istream & strm)
 }
 
 
+bool PTime::Parse(const PString & str)
+{
+  PStringStream strm(str);
+  ReadFrom(strm);
+  return IsValid();
+}
+
 
 PTime PTime::operator+(const PTimeInterval & t) const
 {
