@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24979 $
+ * $Revision: 29812 $
  * $Author: rjongbloed $
- * $Date: 2010-12-22 05:34:45 -0600 (Wed, 22 Dec 2010) $
+ * $Date: 2013-05-27 19:43:21 -0500 (Mon, 27 May 2013) $
  */
 
 #ifdef __GNUC__
@@ -43,7 +43,7 @@
 
 PSafeObject::PSafeObject(PSafeObject * indirectLock)
   : safeReferenceCount(0)
-  , safelyBeingRemoved(PFalse)
+  , safelyBeingRemoved(false)
   , safeInUse(indirectLock != NULL ? indirectLock->safeInUse : &safeInUseMutex)
 {
 }
@@ -52,27 +52,30 @@ PSafeObject::PSafeObject(PSafeObject * indirectLock)
 PBoolean PSafeObject::SafeReference()
 {
 #if PTRACING
-  unsigned tracedReferenceCount;
+  unsigned tracedReferenceCount = 0;
 #endif
 
   {
     PWaitAndSignal mutex(safetyMutex);
-    if (safelyBeingRemoved)
-      return PFalse;
+    if (safelyBeingRemoved) {
+      PTRACE(3, "SafeColl\tAttempt to reference object being removed: " << GetClass() << ' ' << (void *)this);
+      return false;
+    }
     safeReferenceCount++;
 #if PTRACING
     tracedReferenceCount = safeReferenceCount;
 #endif
   }
 
-  PTRACE(6, "SafeColl\tIncrement reference count to " << tracedReferenceCount << " for " << GetClass() << ' ' << (void *)this);
-  return PTrue;
+  PTRACE(m_traceContextIdentifier == 1234567890 ? 3 : 7,
+         "SafeColl\tIncrement reference count to " << tracedReferenceCount << " for " << GetClass() << ' ' << (void *)this);
+  return true;
 }
 
 
 PBoolean PSafeObject::SafeDereference()
 {
-  PBoolean mayBeDeleted = PFalse;
+  PBoolean mayBeDeleted = false;
 #if PTRACING
   unsigned tracedReferenceCount;
 #endif
@@ -87,7 +90,8 @@ PBoolean PSafeObject::SafeDereference()
 #endif
   safetyMutex.Signal();
 
-  PTRACE(6, "SafeColl\tDecrement reference count to " << tracedReferenceCount << " for " << GetClass() << ' ' << (void *)this);
+  PTRACE(m_traceContextIdentifier == 1234567890 ? 3 : 7,
+         "SafeColl\tDecrement reference count to " << tracedReferenceCount << " for " << GetClass() << ' ' << (void *)this);
 
   return mayBeDeleted;
 }
@@ -95,50 +99,56 @@ PBoolean PSafeObject::SafeDereference()
 
 PBoolean PSafeObject::LockReadOnly() const
 {
-  PTRACE(6, "SafeColl\tWaiting read ("<<(void *)this<<")");
+  PTRACE(m_traceContextIdentifier == 1234567890 ? 3 : 7,
+         "SafeColl\tWaiting read ("<<(void *)this<<")");
   safetyMutex.Wait();
 
   if (safelyBeingRemoved) {
     safetyMutex.Signal();
     PTRACE(6, "SafeColl\tBeing removed while waiting read ("<<(void *)this<<")");
-    return PFalse;
+    return false;
   }
 
   safetyMutex.Signal();
   safeInUse->StartRead();
-  PTRACE(6, "SafeColl\tLocked read ("<<(void *)this<<")");
-  return PTrue;
+  PTRACE(m_traceContextIdentifier == 1234567890 ? 3 : 7,
+         "SafeColl\tLocked read ("<<(void *)this<<")");
+  return true;
 }
 
 
 void PSafeObject::UnlockReadOnly() const
 {
-  PTRACE(6, "SafeColl\tUnlocked read ("<<(void *)this<<")");
+  PTRACE(m_traceContextIdentifier == 1234567890 ? 3 : 7,
+         "SafeColl\tUnlocked read ("<<(void *)this<<")");
   safeInUse->EndRead();
 }
 
 
 PBoolean PSafeObject::LockReadWrite()
 {
-  PTRACE(6, "SafeColl\tWaiting readWrite ("<<(void *)this<<")");
+  PTRACE(m_traceContextIdentifier == 1234567890 ? 3 : 7,
+         "SafeColl\tWaiting readWrite ("<<(void *)this<<")");
   safetyMutex.Wait();
 
   if (safelyBeingRemoved) {
     safetyMutex.Signal();
     PTRACE(6, "SafeColl\tBeing removed while waiting readWrite ("<<(void *)this<<")");
-    return PFalse;
+    return false;
   }
 
   safetyMutex.Signal();
   safeInUse->StartWrite();
-  PTRACE(6, "SafeColl\tLocked readWrite ("<<(void *)this<<")");
-  return PTrue;
+  PTRACE(m_traceContextIdentifier == 1234567890 ? 3 : 7,
+         "SafeColl\tLocked readWrite ("<<(void *)this<<")");
+  return true;
 }
 
 
 void PSafeObject::UnlockReadWrite()
 {
-  PTRACE(6, "SafeColl\tUnlocked readWrite ("<<(void *)this<<")");
+  PTRACE(m_traceContextIdentifier == 1234567890 ? 3 : 7,
+         "SafeColl\tUnlocked readWrite ("<<(void *)this<<")");
   safeInUse->EndWrite();
 }
 
@@ -146,7 +156,7 @@ void PSafeObject::UnlockReadWrite()
 void PSafeObject::SafeRemove()
 {
   safetyMutex.Wait();
-  safelyBeingRemoved = PTrue;
+  safelyBeingRemoved = true;
   safetyMutex.Signal();
 }
 
@@ -191,7 +201,7 @@ void PSafeLockReadOnly::Unlock()
 {
   if (locked) {
     safeObject.UnlockReadOnly();
-    locked = PFalse;
+    locked = false;
   }
 }
 
@@ -224,7 +234,7 @@ void PSafeLockReadWrite::Unlock()
 {
   if (locked) {
     safeObject.UnlockReadWrite();
-    locked = PFalse;
+    locked = false;
   }
 }
 
@@ -232,17 +242,18 @@ void PSafeLockReadWrite::Unlock()
 /////////////////////////////////////////////////////////////////////////////
 
 PSafeCollection::PSafeCollection(PCollection * coll)
+  : m_deleteObjectsTimer(NULL)
 {
-  collection = coll;
+  collection = PAssertNULL(coll);
   collection->DisallowDeleteObjects();
   toBeRemoved.DisallowDeleteObjects();
-  deleteObjects = PTrue;
+  deleteObjects = true;
 }
 
 
 PSafeCollection::~PSafeCollection()
 {
-  deleteObjectsTimer.Stop();
+  delete m_deleteObjectsTimer;
 
   RemoveAll();
 
@@ -267,14 +278,14 @@ PSafeCollection::~PSafeCollection()
 PBoolean PSafeCollection::SafeRemove(PSafeObject * obj)
 {
   if (obj == NULL)
-    return PFalse;
+    return false;
 
   PWaitAndSignal mutex(collectionMutex);
   if (!collection->Remove(obj))
-    return PFalse;
+    return false;
 
   SafeRemoveObject(obj);
-  return PTrue;
+  return true;
 }
 
 
@@ -283,10 +294,19 @@ PBoolean PSafeCollection::SafeRemoveAt(PINDEX idx)
   PWaitAndSignal mutex(collectionMutex);
   PSafeObject * obj = PDownCast(PSafeObject, collection->RemoveAt(idx));
   if (obj == NULL)
-    return PFalse;
+    return false;
 
   SafeRemoveObject(obj);
-  return PTrue;
+  return true;
+}
+
+
+void PSafeCollection::PrintOn(ostream &strm) const
+{
+  collectionMutex.Wait();
+  if (collection != NULL)
+    collection->PrintOn(strm);
+  collectionMutex.Signal();
 }
 
 
@@ -308,6 +328,21 @@ void PSafeCollection::RemoveAll(PBoolean synchronous)
 }
 
 
+bool PSafeCollection::SafeAddObject(PSafeObject * obj, PSafeObject * old)
+{
+  if (obj == old)
+    return false;
+
+  if (old != NULL)
+    old->SafeDereference();
+
+  if (obj == NULL)
+    return false;
+
+  return PAssert(collection->GetObjectsIndex(obj) == P_MAX_INDEX, "Cannot insert safe object twice") && obj->SafeReference();
+}
+
+
 void PSafeCollection::SafeRemoveObject(PSafeObject * obj)
 {
   if (obj == NULL)
@@ -365,15 +400,16 @@ void PSafeCollection::DeleteObject(PObject * object) const
 
 void PSafeCollection::SetAutoDeleteObjects()
 {
-  if (deleteObjectsTimer.IsRunning())
+  if (m_deleteObjectsTimer != NULL)
     return;
 
-  deleteObjectsTimer.SetNotifier(PCREATE_NOTIFIER(DeleteObjectsTimeout));
-  deleteObjectsTimer.RunContinuous(1000); // EVery second
+  m_deleteObjectsTimer = new PTimer();
+  m_deleteObjectsTimer->SetNotifier(PCREATE_NOTIFIER(DeleteObjectsTimeout));
+  m_deleteObjectsTimer->RunContinuous(1000); // Every second
 }
 
 
-void PSafeCollection::DeleteObjectsTimeout(PTimer &, INT)
+void PSafeCollection::DeleteObjectsTimeout(PTimer &, P_INT_PTR)
 {
   DeleteObjectsToBeRemoved();
 }
@@ -426,7 +462,7 @@ PSafePtrBase::PSafePtrBase(const PSafeCollection & safeCollection,
                            PSafetyMode mode,
                            PINDEX idx)
 {
-  collection = &safeCollection;
+  collection = safeCollection.CloneAs<PSafeCollection>();
   currentObject = NULL;
   lockMode = mode;
 
@@ -438,7 +474,7 @@ PSafePtrBase::PSafePtrBase(const PSafeCollection & safeCollection,
                            PSafetyMode mode,
                            PSafeObject * obj)
 {
-  collection = &safeCollection;
+  collection = safeCollection.CloneAs<PSafeCollection>();
   currentObject = NULL;
   lockMode = mode;
 
@@ -448,7 +484,7 @@ PSafePtrBase::PSafePtrBase(const PSafeCollection & safeCollection,
 
 PSafePtrBase::PSafePtrBase(const PSafePtrBase & enumerator)
 {
-  collection = enumerator.collection;
+  collection = enumerator.collection != NULL ? enumerator.collection->CloneAs<PSafeCollection>() : NULL;
   currentObject = enumerator.currentObject;
   lockMode = enumerator.lockMode;
 
@@ -459,6 +495,7 @@ PSafePtrBase::PSafePtrBase(const PSafePtrBase & enumerator)
 PSafePtrBase::~PSafePtrBase()
 {
   ExitSafetyMode(WithDereference);
+  delete collection;
 }
 
 
@@ -476,6 +513,12 @@ PObject::Comparison PSafePtrBase::Compare(const PObject & obj) const
 }
 
 
+void PSafePtrBase::PrintOn(ostream &strm) const
+{
+  strm << currentObject;
+}
+
+
 void PSafePtrBase::Assign(const PSafePtrBase & enumerator)
 {
   if (this == &enumerator)
@@ -484,7 +527,8 @@ void PSafePtrBase::Assign(const PSafePtrBase & enumerator)
   // lockCount ends up zero after this
   ExitSafetyMode(WithDereference);
 
-  collection = enumerator.collection;
+  delete collection;
+  collection = enumerator.collection != NULL ? enumerator.collection->CloneAs<PSafeCollection>() : NULL;
   currentObject = enumerator.currentObject;
   lockMode = enumerator.lockMode;
 
@@ -497,7 +541,8 @@ void PSafePtrBase::Assign(const PSafeCollection & safeCollection)
   // lockCount ends up zero after this
   ExitSafetyMode(WithDereference);
 
-  collection = &safeCollection;
+  delete collection;
+  collection = safeCollection.CloneAs<PSafeCollection>();
   currentObject = NULL;
   lockMode = PSafeReadWrite;
 
@@ -525,6 +570,7 @@ void PSafePtrBase::Assign(PSafeObject * newObj)
 
   if (collection->collection->GetObjectsIndex(newObj) == P_MAX_INDEX) {
     collection->collectionMutex.Signal();
+    delete collection;
     collection = NULL;
     lockMode = PSafeReference;
     if (!EnterSafetyMode(WithReference))
@@ -633,6 +679,7 @@ void PSafePtrBase::SetNULL()
   // lockCount ends up zero after this
   ExitSafetyMode(WithDereference);
 
+  delete collection;
   collection = NULL;
   currentObject = NULL;
   lockMode = PSafeReference;
@@ -642,7 +689,7 @@ void PSafePtrBase::SetNULL()
 PBoolean PSafePtrBase::SetSafetyMode(PSafetyMode mode)
 {
   if (lockMode == mode)
-    return PTrue;
+    return true;
 
   ExitSafetyMode(NoDereference);
   lockMode = mode;
@@ -653,31 +700,31 @@ PBoolean PSafePtrBase::SetSafetyMode(PSafetyMode mode)
 PBoolean PSafePtrBase::EnterSafetyMode(EnterSafetyModeOption ref)
 {
   if (currentObject == NULL)
-    return PFalse;
+    return false;
 
   if (ref == WithReference && !currentObject->SafeReference()) {
     currentObject = NULL;
-    return PFalse;
+    return false;
   }
 
   switch (lockMode) {
     case PSafeReadOnly :
       if (currentObject->LockReadOnly())
-        return PTrue;
+        return true;
       break;
 
     case PSafeReadWrite :
       if (currentObject->LockReadWrite())
-        return PTrue;
+        return true;
       break;
 
     case PSafeReference :
-      return PTrue;
+      return true;
   }
 
   currentObject->SafeDereference();
   currentObject = NULL;
-  return PFalse;
+  return false;
 }
 
 
@@ -723,12 +770,12 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(PSafeObject * obj, PSafetyMode mode
   : PSafePtrBase(NULL, mode)
   , m_objectToDelete(NULL)
 {
-  Lock();
+  LockPtr();
 
   currentObject = obj;
   EnterSafetyMode(WithReference);
 
-  Unlock();
+  UnlockPtr();
 }
 
 
@@ -738,12 +785,12 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafeCollection & safeCollect
   : PSafePtrBase(NULL, mode)
   , m_objectToDelete(NULL)
 {
-  Lock();
+  LockPtr();
 
-  collection = &safeCollection;
+  collection = safeCollection.CloneAs<PSafeCollection>();
   Assign(idx);
 
-  Unlock();
+  UnlockPtr();
 }
 
 
@@ -753,38 +800,38 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafeCollection & safeCollect
   : PSafePtrBase(NULL, mode)
   , m_objectToDelete(NULL)
 {
-  Lock();
+  LockPtr();
 
-  collection = &safeCollection;
+  collection = safeCollection.CloneAs<PSafeCollection>();
   Assign(obj);
 
-  Unlock();
+  UnlockPtr();
 }
 
 
 PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafePtrMultiThreaded & enumerator)
   : m_objectToDelete(NULL)
 {
-  Lock();
+  LockPtr();
   enumerator.m_mutex.Wait();
 
-  collection = enumerator.collection;
+  collection = enumerator.collection != NULL ? enumerator.collection->CloneAs<PSafeCollection>() : NULL;
   currentObject = enumerator.currentObject;
   lockMode = enumerator.lockMode;
 
   EnterSafetyMode(WithReference);
 
   enumerator.m_mutex.Signal();
-  Unlock();
+  UnlockPtr();
 }
 
 
 PSafePtrMultiThreaded::~PSafePtrMultiThreaded()
 {
-  Lock();
+  LockPtr();
   ExitSafetyMode(WithDereference);
   currentObject = NULL;
-  Unlock();
+  UnlockPtr();
 }
 
 
@@ -797,9 +844,9 @@ PObject::Comparison PSafePtrMultiThreaded::Compare(const PObject & obj) const
 
 void PSafePtrMultiThreaded::SetNULL()
 {
-  Lock();
+  LockPtr();
   PSafePtrBase::SetNULL();
-  Unlock();
+  UnlockPtr();
 }
 
 
@@ -812,59 +859,59 @@ PBoolean PSafePtrMultiThreaded::SetSafetyMode(PSafetyMode mode)
 
 void PSafePtrMultiThreaded::Assign(const PSafePtrMultiThreaded & ptr)
 {
-  Lock();
+  LockPtr();
   ptr.m_mutex.Wait();
   PSafePtrBase::Assign(ptr);
   ptr.m_mutex.Signal();
-  Unlock();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Assign(const PSafePtrBase & ptr)
 {
-  Lock();
+  LockPtr();
   PSafePtrBase::Assign(ptr);
-  Unlock();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Assign(const PSafeCollection & safeCollection)
 {
-  Lock();
+  LockPtr();
   PSafePtrBase::Assign(safeCollection);
-  Unlock();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Assign(PSafeObject * obj)
 {
-  Lock();
+  LockPtr();
   PSafePtrBase::Assign(obj);
-  Unlock();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Assign(PINDEX idx)
 {
-  Lock();
+  LockPtr();
   PSafePtrBase::Assign(idx);
-  Unlock();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Next()
 {
-  Lock();
+  LockPtr();
   PSafePtrBase::Next();
-  Unlock();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Previous()
 {
-  Lock();
+  LockPtr();
   PSafePtrBase::Previous();
-  Unlock();
+  UnlockPtr();
 }
 
 
@@ -874,7 +921,7 @@ void PSafePtrMultiThreaded::DeleteObject(PSafeObject * obj)
 }
 
 
-void PSafePtrMultiThreaded::Unlock()
+void PSafePtrMultiThreaded::UnlockPtr()
 {
   PSafeObject * obj = m_objectToDelete;
   m_objectToDelete = NULL;
