@@ -24,9 +24,9 @@
  * Contributor(s): Craig Southeren
  *                 Snark at GnomeMeeting
  *
- * $Revision: 28275 $
- * $Author: rjongbloed $
- * $Date: 2012-08-29 21:42:35 -0500 (Wed, 29 Aug 2012) $
+ * $Revision: 28941 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 08:18:28 -0600 (Fri, 18 Jan 2013) $
  */
 
 #ifdef __GNUC__
@@ -37,11 +37,13 @@
 
 #include <ptlib/sound.h>
 #include <ptlib/pluginmgr.h>
+#include <ptclib/delaychan.h>
+
 
 static const char soundPluginBaseClass[] = "PSoundChannel";
 
 
-template <> PSoundChannel * PDevicePluginFactory<PSoundChannel>::Worker::Create(const PString & type) const
+template <> PSoundChannel * PDevicePluginFactory<PSoundChannel>::Worker::Create(const PDefaultPFactoryKey & type) const
 {
   return PSoundChannel::CreateChannel(type);
 }
@@ -243,7 +245,7 @@ PBoolean PSoundChannel::Close()
 }
 
 
-int PSoundChannel::GetHandle() const
+P_INT_PTR PSoundChannel::GetHandle() const
 {
   PReadWaitAndSignal mutex(m_baseMutex);
   return m_baseChannel == NULL ? -1 : m_baseChannel->PChannel::GetHandle();
@@ -338,10 +340,6 @@ PBoolean PSoundChannel::Write(const void * buf, PINDEX len)
   return m_baseChannel != NULL && m_baseChannel->Write(buf, len);
 }
 
-PBoolean PSoundChannel::Write(const void * buf, PINDEX len, const void * /*mark*/)
-{
-  return Write(buf, len);
-}
 
 PINDEX PSoundChannel::GetLastWriteCount() const
 {
@@ -523,13 +521,13 @@ void PSound::SetFormat(unsigned channels,
 
 PBoolean PSound::Load(const PFilePath & /*filename*/)
 {
-  return PFalse;
+  return false;
 }
 
 
 PBoolean PSound::Save(const PFilePath & /*filename*/)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -544,9 +542,9 @@ PBoolean PSound::Play(const PString & device)
 
   PSoundChannel channel(device, PSoundChannel::Player);
   if (!channel.IsOpen())
-    return PFalse;
+    return false;
 
-  return channel.PlaySound(*this, PTrue);
+  return channel.PlaySound(*this, true);
 }
 
 
@@ -555,10 +553,129 @@ PBoolean PSound::PlayFile(const PFilePath & file, PBoolean wait)
   PSoundChannel channel(PSoundChannel::GetDefaultDevice(PSoundChannel::Player),
                         PSoundChannel::Player);
   if (!channel.IsOpen())
-    return PFalse;
+    return false;
 
   return channel.PlayFile(file, wait);
 }
 
 
 #endif //_WIN32
+
+///////////////////////////////////////////////////////////////////////////
+
+static const PConstString NullAudio("Null Audio");
+
+class PSoundChannelNull : public PSoundChannel
+{
+ PCLASSINFO(PSoundChannelNull, PSoundChannel);
+ public:
+    PSoundChannelNull()
+      : m_sampleRate(0)
+    {
+    }
+
+    PSoundChannelNull(
+      const PString &device,
+      PSoundChannel::Directions dir,
+      unsigned numChannels,
+      unsigned sampleRate,
+      unsigned bitsPerSample
+    ) : m_sampleRate(0)
+    {
+      Open(device, dir, numChannels, sampleRate, bitsPerSample);
+    }
+
+    static PStringArray GetDeviceNames(PSoundChannel::Directions = Player)
+    {
+      return NullAudio;
+    }
+
+    PBoolean Open(const PString &,
+                  Directions dir,
+                  unsigned numChannels,
+                  unsigned sampleRate,
+                  unsigned bitsPerSample)
+    {
+      activeDirection = dir;
+      return SetFormat(numChannels, sampleRate, bitsPerSample);
+    }
+
+    virtual PString GetName() const
+    {
+      return NullAudio;
+    }
+
+    PBoolean Close()
+    {
+      m_sampleRate = 0;
+      return true;
+    }
+
+    PBoolean IsOpen() const
+    {
+      return m_sampleRate > 0;
+    }
+
+    PBoolean Write(const void *, PINDEX len)
+    {
+      if (m_sampleRate <= 0)
+        return false;
+
+      lastWriteCount = len;
+      m_Pacing.Delay(len/2*1000/m_sampleRate);
+      return true;
+    }
+
+    PBoolean Read(void * buf, PINDEX len)
+    {
+      if (m_sampleRate <= 0)
+        return false;
+
+      memset(buf, 0, len);
+      lastReadCount = len;
+      m_Pacing.Delay(len/2*1000/m_sampleRate);
+      return true;
+    }
+
+    PBoolean SetFormat(unsigned numChannels,
+                   unsigned sampleRate,
+                   unsigned bitsPerSample)
+    {
+      m_sampleRate = sampleRate;
+      return numChannels == 1 && bitsPerSample == 16;
+    }
+
+    unsigned GetChannels() const
+    {
+      return 1;
+    }
+
+    unsigned GetSampleRate() const
+    {
+      return m_sampleRate;
+    }
+
+    unsigned GetSampleSize() const
+    {
+      return 16;
+    }
+
+    PBoolean SetBuffers(PINDEX, PINDEX)
+    {
+      return true;
+    }
+
+    PBoolean GetBuffers(PINDEX & size, PINDEX & count)
+    {
+      size = 2;
+      count = 1;
+      return true;
+    }
+
+protected:
+    unsigned       m_sampleRate;
+    PAdaptiveDelay m_Pacing;
+};
+
+
+PCREATE_SOUND_PLUGIN(NullAudio, PSoundChannelNull)
