@@ -23,9 +23,9 @@
  *
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
- * $Revision: 23891 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2009-12-22 01:57:21 -0600 (Tue, 22 Dec 2009) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -1315,7 +1315,7 @@ class PVideoInputDevice_FakeVideo : public PVideoInputDevice
       */
     PBoolean Open(
       const PString & deviceName,   /// Device name to open
-      PBoolean startImmediate = PTrue    /// Immediately start device
+      PBoolean startImmediate = true    /// Immediately start device
     );
 
     /**Determine of the device is currently open.
@@ -1458,7 +1458,7 @@ class PVideoInputDevice_FakeVideo : public PVideoInputDevice
     /**Get the minimum & maximum size of a frame on the device.
 
        Default behaviour returns the value 1 to UINT_MAX for both and returns
-       PFalse.
+       false.
     */
     virtual PBoolean GetFrameSizeLimits(
       unsigned & minWidth,   /// Variable to receive minimum width
@@ -1479,12 +1479,10 @@ class PVideoInputDevice_FakeVideo : public PVideoInputDevice
          
     void ClearMapping() { return ; }
 
-    /**Try all known video formats & see which ones are accepted by the video driver
-     */
-    virtual PBoolean TestAllFormats()
-      { return PTrue; }
 
  protected:
+   bool m_open;
+
     enum {
       eRGB32,
       eRGB24,
@@ -1507,6 +1505,8 @@ PCREATE_VIDINPUT_PLUGIN(FakeVideo);
 
 
 PVideoInputDevice_FakeVideo::PVideoInputDevice_FakeVideo()
+  : m_open(false)
+  , m_Pacing(500)
 {
   SetColourFormat("RGB24");
   channelNumber = 3; // Blank screen
@@ -1528,31 +1528,33 @@ PBoolean PVideoInputDevice_FakeVideo::Open(const PString & devName, PBoolean /*s
   }
 
   deviceName = FakeDeviceNames[i];
+  m_open = true;
   return true;
 }
 
 
 PBoolean PVideoInputDevice_FakeVideo::IsOpen() 
 {
-  return PTrue;
+  return m_open;
 }
 
 
 PBoolean PVideoInputDevice_FakeVideo::Close()
 {
-  return PTrue;
+  m_open = false;
+  return true;
 }
 
 
 PBoolean PVideoInputDevice_FakeVideo::Start()
 {
-  return PTrue;
+  return IsOpen();
 }
 
 
 PBoolean PVideoInputDevice_FakeVideo::Stop()
 {
-  return PTrue;
+  return IsOpen();
 }
 
 
@@ -1600,10 +1602,10 @@ PBoolean PVideoInputDevice_FakeVideo::SetColourFormat(const PString & newFormat)
   else if ((newFormat *= "YUV422") || (newFormat *= "YUY2"))
     m_internalColourFormat = eYUV422;
   else
-    return PFalse;
+    return false;
 
   if (!PVideoDevice::SetColourFormat(newFormat))
-    return PFalse;
+    return false;
 
   return SetFrameSize(frameWidth, frameHeight);
 }
@@ -1630,17 +1632,17 @@ PBoolean PVideoInputDevice_FakeVideo::GetFrameSizeLimits(unsigned & minWidth,
   maxWidth  = PVideoFrameInfo::MaxWidth;
   maxHeight = PVideoFrameInfo::MaxHeight;
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PVideoInputDevice_FakeVideo::SetFrameSize(unsigned width, unsigned height)
 {
   if (!PVideoDevice::SetFrameSize(width, height))
-    return PFalse;
+    return false;
 
   m_videoFrameSize = CalculateFrameBytes(frameWidth, frameHeight, colourFormat);
-  m_scanLineWidth = m_videoFrameSize/frameHeight;
+  m_scanLineWidth = (frameHeight == 0) ? 0 : m_videoFrameSize/frameHeight;
   return m_videoFrameSize > 0;
 }
 
@@ -1660,6 +1662,9 @@ PBoolean PVideoInputDevice_FakeVideo::GetFrameData(BYTE * buffer, PINDEX * bytes
  
 PBoolean PVideoInputDevice_FakeVideo::GetFrameDataNoDelay(BYTE *destFrame, PINDEX * bytesReturned)
 {
+  if (!IsOpen())
+    return false;
+
   m_grabCount++;
 
   // Make sure are NUM_PATTERNS cases here.
@@ -1686,18 +1691,18 @@ PBoolean PVideoInputDevice_FakeVideo::GetFrameDataNoDelay(BYTE *destFrame, PINDE
        GrabNTSCTestFrame(destFrame);
        break;
      default :
-       return PFalse;
+       return false;
   }
 
   if (NULL != converter) {
     if (!converter->Convert(destFrame, destFrame, bytesReturned))
-      return PFalse;
+      return false;
   }
 
   if (bytesReturned != NULL)
     *bytesReturned = m_videoFrameSize;
 
-  return PTrue;
+  return true;
 }
 
 
@@ -2127,7 +2132,7 @@ class PVideoOutputDevice_NULLOutput : public PVideoOutputDevice
       */
     virtual PBoolean Open(
       const PString & deviceName,   /// Device name to open
-      PBoolean startImmediate = PTrue    /// Immediately start device
+      PBoolean startImmediate = true    /// Immediately start device
     );
 
     /**Start the video device I/O.
@@ -2161,12 +2166,17 @@ class PVideoOutputDevice_NULLOutput : public PVideoOutputDevice
       unsigned width,
       unsigned height,
       const BYTE * data,
-      PBoolean endFrame = PTrue
+      PBoolean endFrame = true
     );
 
     /**Indicate frame may be displayed.
       */
     virtual PBoolean EndFrame();
+
+	/**Decide whether to disable Decode
+	  */
+	virtual PBoolean DisableDecode();
+
 };
 
 PCREATE_VIDOUTPUT_PLUGIN(NULLOutput);
@@ -2183,27 +2193,27 @@ PVideoOutputDevice_NULLOutput::PVideoOutputDevice_NULLOutput()
 PBoolean PVideoOutputDevice_NULLOutput::Open(const PString & /*deviceName*/,
                                   PBoolean /*startImmediate*/)
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean PVideoOutputDevice_NULLOutput::Close()
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean PVideoOutputDevice_NULLOutput::Start()
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean PVideoOutputDevice_NULLOutput::Stop()
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean PVideoOutputDevice_NULLOutput::IsOpen()
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -2224,13 +2234,19 @@ PBoolean PVideoOutputDevice_NULLOutput::SetFrameData(unsigned /*x*/, unsigned /*
                                           const BYTE * /*data*/,
                                           PBoolean /*endFrame*/)
 {
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PVideoOutputDevice_NULLOutput::EndFrame()
 {
-  return PTrue;
+  return true;
+}
+
+
+PBoolean PVideoOutputDevice_NULLOutput::DisableDecode()
+{
+  return true; 
 }
 
 
