@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28740 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2013-01-02 01:58:47 -0600 (Wed, 02 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -78,7 +78,7 @@ PMail::~PMail()
 
 void PMail::Construct()
 {
-  loggedOn = PFalse;
+  loggedOn = false;
   hUserInterface = NULL;
 }
 
@@ -96,13 +96,20 @@ PBoolean PMail::LogOn(const PString & username,
   return LogOnCommonInterface(username, password, service);
 }
 
+#ifdef _WIN64
+  #pragma warning(disable:4100)
+#endif
+
+#if _MSC_VER >= 1700
+  #pragma warning(disable:4100)
+#endif
 
 PBoolean PMail::LogOnCommonInterface(const char * username,
                                  const char * password,
                                  const char * service)
 {
   if (!LogOff())
-    return PFalse;
+    return false;
 
 #if P_HAS_CMC
   if (cmc.IsLoaded()) {
@@ -142,14 +149,14 @@ PBoolean PMail::LogOnCommonInterface(const char * username,
 #endif
 
   lastError = 1;
-  return PFalse;
+  return false;
 }
 
 
 PBoolean PMail::LogOff()
 {
   if (!loggedOn)
-    return PTrue;
+    return true;
 
 #if P_HAS_CMC
   if (cmc.IsLoaded()) {
@@ -162,7 +169,7 @@ PBoolean PMail::LogOff()
       case CMC_SUCCESS :
       case CMC_E_INVALID_SESSION_ID :
       case CMC_E_USER_NOT_LOGGED_ON :
-        loggedOn = PFalse;
+        loggedOn = false;
     }
     return lastError == CMC_SUCCESS;
   }
@@ -172,13 +179,13 @@ PBoolean PMail::LogOff()
   if (mapi.IsLoaded()) {
     lastError = mapi.Logoff(sessionId, (HWND)hUserInterface, 0, 0);
     if (lastError == SUCCESS_SUCCESS || lastError == MAPI_E_INVALID_SESSION)
-      loggedOn = PFalse;
+      loggedOn = false;
     return lastError == SUCCESS_SUCCESS;
   }
 #endif
 
   lastError = 1;
-  return PFalse;
+  return false;
 }
 
 
@@ -307,7 +314,7 @@ PBoolean PMail::SendNote(const PString & recipient,
 #endif
 
   lastError = 1;
-  return PFalse;
+  return false;
 }
 
 
@@ -345,7 +352,7 @@ PStringArray PMail::GetMessageIDs(PBoolean unreadOnly)
             sprintf(ptr, "%02x", messages[m].message_reference->string[c]);
             ptr += 2;
           }
-          msgIDs[m] = buf;
+          msgIDs[m] = PString(buf);
         }
       }
       cmc.free_buf(messages);
@@ -438,7 +445,7 @@ PBoolean PMail::GetMessageHeader(const PString & id,
     lastError = cmc.list(sessionId,
                     NULL, flags, seed, &count, hUserInterface, &message, NULL);
     if (lastError != CMC_SUCCESS)
-      return PFalse;
+      return false;
 
     hdrInfo.subject = message->subject;
     hdrInfo.received = PTime(0, message->time_sent.minute,
@@ -453,7 +460,7 @@ PBoolean PMail::GetMessageHeader(const PString & id,
       hdrInfo.originatorAddress = '"' + hdrInfo.originatorName + '"';
 
     cmc.free_buf(message);
-    return PTrue;
+    return true;
   }
 #endif
 
@@ -463,7 +470,7 @@ PBoolean PMail::GetMessageHeader(const PString & id,
     lastError = mapi.ReadMail(sessionId,
                     (HWND)hUserInterface, id, MAPI_ENVELOPE_ONLY, 0, &message);
     if (lastError != SUCCESS_SUCCESS)
-      return PFalse;
+      return false;
 
     PStringStream str = message->lpszDateReceived;
     int min, hr, day, mon, yr;
@@ -479,12 +486,12 @@ PBoolean PMail::GetMessageHeader(const PString & id,
       hdrInfo.originatorAddress = '"' + hdrInfo.originatorName + '"';
 
     mapi.FreeBuffer(message);
-    return PTrue;
+    return true;
   }
 #endif
 
   lastError = 1;
-  return PFalse;
+  return false;
 }
 
 
@@ -505,7 +512,7 @@ PBoolean PMail::GetMessageBody(const PString & id, PString & body, PBoolean mark
 
     lastError = cmc.read(sessionId,seed,flags,&message,hUserInterface,NULL);
     if (lastError != CMC_SUCCESS)
-      return PFalse;
+      return false;
 
     if (message->text_note != NULL)
       body = message->text_note;
@@ -528,16 +535,16 @@ PBoolean PMail::GetMessageBody(const PString & id, PString & body, PBoolean mark
     lastError = mapi.ReadMail(sessionId,
                                  (HWND)hUserInterface, id, flags, 0, &message);
     if (lastError != SUCCESS_SUCCESS)
-      return PFalse;
+      return false;
 
     body = message->lpszNoteText;
     mapi.FreeBuffer(message);
-    return PTrue;
+    return true;
   }
 #endif
 
   lastError = 1;
-  return PFalse;
+  return false;
 }
 
 
@@ -561,7 +568,7 @@ PBoolean PMail::GetMessageAttachments(const PString & id,
 
     lastError = cmc.read(sessionId,seed,flags,&message,hUserInterface,NULL);
     if (lastError != CMC_SUCCESS)
-      return PFalse;
+      return false;
 
     if (message->attachments != NULL) {
       PINDEX total = 1;
@@ -581,7 +588,7 @@ PBoolean PMail::GetMessageAttachments(const PString & id,
     }
 
     cmc.free_buf(message);
-    return PTrue;
+    return true;
   }
 #endif
 
@@ -597,7 +604,7 @@ PBoolean PMail::GetMessageAttachments(const PString & id,
     lastError = mapi.ReadMail(sessionId,
                                  (HWND)hUserInterface, id, flags, 0, &message);
     if (lastError != SUCCESS_SUCCESS)
-      return PFalse;
+      return false;
 
     filenames.SetSize(message->nFileCount);
     PINDEX i;
@@ -605,12 +612,12 @@ PBoolean PMail::GetMessageAttachments(const PString & id,
       filenames[i] = message->lpFiles[i].lpszPathName;
 
     mapi.FreeBuffer(message);
-    return PTrue;
+    return true;
   }
 #endif
 
   lastError = 1;
-  return PFalse;
+  return false;
 }
 
 
@@ -627,10 +634,10 @@ PBoolean PMail::MarkMessageRead(const PString & id)
 
     lastError = cmc.read(sessionId,seed,flags,&message,hUserInterface,NULL);
     if (lastError != CMC_SUCCESS)
-      return PFalse;
+      return false;
 
     cmc.free_buf(message);
-    return PTrue;
+    return true;
   }
 #endif
 
@@ -640,15 +647,15 @@ PBoolean PMail::MarkMessageRead(const PString & id)
     lastError = mapi.ReadMail(sessionId,
                     (HWND)hUserInterface, id, MAPI_ENVELOPE_ONLY, 0, &message);
     if (lastError != SUCCESS_SUCCESS)
-      return PFalse;
+      return false;
 
     mapi.FreeBuffer(message);
-    return PTrue;
+    return true;
   }
 #endif
 
   lastError = 1;
-  return PFalse;
+  return false;
 }
 
 
@@ -673,7 +680,7 @@ PBoolean PMail::DeleteMessage(const PString & id)
 #endif
 
   lastError = 1;
-  return PFalse;
+  return false;
 }
 
 
@@ -736,6 +743,9 @@ PMail::LookUpResult PMail::LookUp(const PString & name, PString * fullName)
   return LookUpError;
 }
 
+#ifdef _WIN64
+  #pragma warning(default:4100)
+#endif
 
 int PMail::GetErrorCode() const
 {
