@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22588 $
+ * $Revision: 29415 $
  * $Author: rjongbloed $
- * $Date: 2009-05-12 04:12:46 -0500 (Tue, 12 May 2009) $
+ * $Date: 2013-04-02 16:41:13 -0500 (Tue, 02 Apr 2013) $
  */
 
 #include <ptlib.h>
@@ -41,14 +41,13 @@
 
 PPipeChannel::PPipeChannel()
 {
-  hToChild = hFromChild = hStandardError = INVALID_HANDLE_VALUE;
 }
 
 
 #ifdef _WIN32_WCE
 PBoolean PPipeChannel::PlatformOpen(const PString &, const PStringArray &, OpenMode, PBoolean, PBoolean, const PStringToString *)
 {
-  return PFalse;
+  return false;
 }
 #else
 PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
@@ -81,8 +80,8 @@ PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
   char * envStr = NULL;
   if (environment != NULL) {
     PINDEX size = 0;
-    for (PINDEX e = 0; e < environment->GetSize(); e++) {
-      PString str = environment->GetKeyAt(e) + '=' + environment->GetDataAt(e);
+    for (PStringToString::const_iterator it = environment->begin(); it != environment->end(); ++it) {
+      PString str = it->first + '=' + it->second;
       PINDEX len = str.GetLength() + 1;
       envBuf.SetSize(size + len);
       memcpy(envBuf.GetPointer()+size, (const char *)str, len);
@@ -111,50 +110,45 @@ PBoolean PPipeChannel::PlatformOpen(const PString & subProgram,
   // ReadOnly means child has no stdin
   // otherwise create a pipe for us to send data to child
   if (mode == ReadOnly)
-    hToChild = INVALID_HANDLE_VALUE;
+    m_hToChild.Close();
   else {
     HANDLE writeEnd;
     PAssertOS(CreatePipe(&startup.hStdInput, &writeEnd, &security, 0));
     PAssertOS(SetHandleInformation(writeEnd, HANDLE_FLAG_INHERIT, 0));
-    PAssertOS(DuplicateHandle(GetCurrentProcess(), writeEnd,
-                              GetCurrentProcess(), &hToChild, 0, PFalse,
-                              DUPLICATE_CLOSE_SOURCE|DUPLICATE_SAME_ACCESS));
+    PAssertOS(m_hToChild.Duplicate(writeEnd, DUPLICATE_CLOSE_SOURCE|DUPLICATE_SAME_ACCESS));
   }
 
   // WriteOnly means child has no stdout
   // ReadWriteStd means child uses our stdout and stderr
   // otherwise, create a pipe to read stdout from child, and perhaps a seperate one for stderr too
   if (mode == WriteOnly)
-    hFromChild = INVALID_HANDLE_VALUE;
+    m_hFromChild.Close();
   else if (mode == ReadWriteStd) {
-    hFromChild = INVALID_HANDLE_VALUE;
+    m_hFromChild.Close();
     startup.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
     startup.hStdError = GetStdHandle(STD_ERROR_HANDLE);
   }
   else {
-    PAssertOS(CreatePipe(&hFromChild, &startup.hStdOutput, &security, 1));
-    PAssertOS(SetHandleInformation(hFromChild, HANDLE_FLAG_INHERIT, 0));
+    PAssertOS(CreatePipe(m_hFromChild.GetPointer(), &startup.hStdOutput, &security, 1));
+    PAssertOS(SetHandleInformation(m_hFromChild, HANDLE_FLAG_INHERIT, 0));
     if (stderrSeparate) {
-      PAssertOS(CreatePipe(&hStandardError, &startup.hStdError, &security, 1));
-      PAssertOS(SetHandleInformation(hStandardError, HANDLE_FLAG_INHERIT, 0));
+      PAssertOS(CreatePipe(m_hStandardError.GetPointer(), &startup.hStdError, &security, 1));
+      PAssertOS(SetHandleInformation(m_hStandardError, HANDLE_FLAG_INHERIT, 0));
     }
     else {
       startup.hStdError = startup.hStdOutput;
-      hStandardError = INVALID_HANDLE_VALUE;
+      m_hStandardError.Close();
     }
   }
 
-  if (ConvertOSError(CreateProcess(prog, cmdLine.GetPointer(),
-                                   NULL, NULL, PTrue, 0, envStr,
+  if (ConvertOSError(CreateProcess(prog, (LPSTR)(const char *)cmdLine,
+                                   NULL, NULL, true, 0, envStr,
                                    NULL, &startup, &info) ? 0 : -2))
     os_handle = info.dwProcessId;
   else {
-    if (hToChild != INVALID_HANDLE_VALUE)
-      CloseHandle(hToChild);
-    if (hFromChild != INVALID_HANDLE_VALUE)
-      CloseHandle(hFromChild);
-    if (hStandardError != INVALID_HANDLE_VALUE)
-      CloseHandle(hStandardError);
+    m_hToChild.Close();
+    m_hFromChild.Close();
+    m_hStandardError.Close();
   }
 
   if (startup.hStdInput != INVALID_HANDLE_VALUE)
@@ -198,7 +192,7 @@ int PPipeChannel::GetReturnCode() const
 
 PBoolean PPipeChannel::CanReadAndWrite()
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean PPipeChannel::IsRunning() const
@@ -244,14 +238,14 @@ PBoolean PPipeChannel::Read(void * buffer, PINDEX len)
   // So have all this hideous code. :-(
 
   if (readTimeout == PMaxTimeInterval) {
-    if (!ConvertOSError(ReadFile(hFromChild, buffer, 1, &count, NULL) ? 0 : -2, LastReadError))
+    if (!ConvertOSError(ReadFile(m_hFromChild, buffer, 1, &count, NULL) ? 0 : -2, LastReadError))
       return false;
 
     lastReadCount = 1;
     if (len == 1)
       return true;
 
-    if (!PeekNamedPipe(hFromChild, NULL, 0, NULL, &count, NULL))
+    if (!PeekNamedPipe(m_hFromChild, NULL, 0, NULL, &count, NULL))
       return ConvertOSError(-2, LastReadError);
 
     if (count == 0)
@@ -261,16 +255,16 @@ PBoolean PPipeChannel::Read(void * buffer, PINDEX len)
     --len;
   }
   else {
-    PTimeInterval startTick = PTimer::Tick();
+    PSimpleTimer timeout(readTimeout);
     for (;;) {
-      if (!PeekNamedPipe(hFromChild, NULL, 0, NULL, &count, NULL))
+      if (!PeekNamedPipe(m_hFromChild, NULL, 0, NULL, &count, NULL))
         return ConvertOSError(-2, LastReadError);
 
       if (count > 0)
         break;
 
-      if ((PTimer::Tick() - startTick) > readTimeout) {
-        SetErrorValues(Timeout, EAGAIN, LastReadError);
+      if (timeout.HasExpired()) {
+        SetErrorValues(Timeout, ETIMEDOUT, LastReadError);
         return false;
       }
 
@@ -282,7 +276,7 @@ PBoolean PPipeChannel::Read(void * buffer, PINDEX len)
     len = count;
 #endif
 
-  if (!ConvertOSError(ReadFile(hFromChild, buffer, len, &count, NULL) ? 0 : -2, LastReadError))
+  if (!ConvertOSError(ReadFile(m_hFromChild, buffer, len, &count, NULL) ? 0 : -2, LastReadError))
     return false;
 
   lastReadCount += count;
@@ -294,8 +288,8 @@ PBoolean PPipeChannel::Write(const void * buffer, PINDEX len)
 {
   lastWriteCount = 0;
   DWORD count;
-  if (!ConvertOSError(WriteFile(hToChild, buffer, len, &count, NULL) ? 0 : -2, LastWriteError))
-    return PFalse;
+  if (!ConvertOSError(WriteFile(m_hToChild, buffer, len, &count, NULL) ? 0 : -2, LastWriteError))
+    return false;
   lastWriteCount = count;
   return lastWriteCount >= len;
 }
@@ -305,16 +299,13 @@ PBoolean PPipeChannel::Close()
 {
   if (IsOpen()) {
     os_handle = -1;
-    if (hToChild != INVALID_HANDLE_VALUE)
-      CloseHandle(hToChild);
-    if (hFromChild != INVALID_HANDLE_VALUE)
-      CloseHandle(hFromChild);
-    if (hStandardError != INVALID_HANDLE_VALUE)
-      CloseHandle(hStandardError);
+    m_hToChild.Close();
+    m_hFromChild.Close();
+    m_hStandardError.Close();
     if (!TerminateProcess(info.hProcess, 1))
-      return PFalse;
+      return false;
   }
-  return PTrue;
+  return true;
 }
 
 
@@ -322,9 +313,7 @@ PBoolean PPipeChannel::Execute()
 {
   flush();
   clear();
-  if (hToChild != INVALID_HANDLE_VALUE)
-    CloseHandle(hToChild);
-  hToChild = INVALID_HANDLE_VALUE;
+  m_hToChild.Close();
   return IsRunning();
 }
 
@@ -332,37 +321,37 @@ PBoolean PPipeChannel::Execute()
 #ifdef _WIN32_WCE
 PBoolean PPipeChannel::ReadStandardError(PString &, PBoolean)
 {
-  return PFalse;
+  return false;
 }
 #else
 PBoolean PPipeChannel::ReadStandardError(PString & errors, PBoolean wait)
 {
   DWORD available, bytesRead;
-  if (!PeekNamedPipe(hStandardError, NULL, 0, NULL, &available, NULL))
+  if (!PeekNamedPipe(m_hStandardError, NULL, 0, NULL, &available, NULL))
     return ConvertOSError(-2, LastReadError);
 
   if (available != 0)
-    return ConvertOSError(ReadFile(hStandardError,
-                          errors.GetPointer(available+1), available,
+    return ConvertOSError(ReadFile(m_hStandardError,
+                          errors.GetPointerAndSetLength(available), available,
                           &bytesRead, NULL) ? 0 : -2, LastReadError);
 
   if (!wait)
-    return PFalse;
+    return false;
 
   char firstByte;
-  if (!ReadFile(hStandardError, &firstByte, 1, &bytesRead, NULL))
+  if (!ReadFile(m_hStandardError, &firstByte, 1, &bytesRead, NULL))
     return ConvertOSError(-2, LastReadError);
 
   errors = firstByte;
 
-  if (!PeekNamedPipe(hStandardError, NULL, 0, NULL, &available, NULL))
+  if (!PeekNamedPipe(m_hStandardError, NULL, 0, NULL, &available, NULL))
     return ConvertOSError(-2, LastReadError);
 
   if (available == 0)
-    return PTrue;
+    return true;
 
-  return ConvertOSError(ReadFile(hStandardError,
-                        errors.GetPointer(available+2)+1, available,
+  return ConvertOSError(ReadFile(m_hStandardError,
+                        errors.GetPointerAndSetLength(available+1)+1, available,
                         &bytesRead, NULL) ? 0 : -2, LastReadError);
 }
 #endif // !_WIN32_WCE
