@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23590 $
- * $Author: rjongbloed $
- * $Date: 2009-10-05 19:29:59 -0500 (Mon, 05 Oct 2009) $
+ * $Revision: 28935 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:12:14 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -72,7 +72,7 @@ ostream & operator<<(ostream & s, PUInt64 v)
 }
 
 
-const int MaxDigits = (64+2)/3+1; // Maximum is 22 digit octal number, plus sign
+const PINDEX MaxDigits = (64+2)/3+1; // Maximum is 22 digit octal number, plus sign
 
 static void GetDigits(PBoolean sign, istream & s, char * buffer)
 {
@@ -111,7 +111,7 @@ static void GetDigits(PBoolean sign, istream & s, char * buffer)
 istream & operator>>(istream & s, PInt64 & v)
 {
   char b[MaxDigits+1];
-  GetDigits(PTrue, s, b);
+  GetDigits(true, s, b);
   v = _atoi64(b);
   return s;
 }
@@ -120,7 +120,7 @@ istream & operator>>(istream & s, PInt64 & v)
 istream & operator>>(istream & s, PUInt64 & v)
 {
   char b[MaxDigits+1];
-  GetDigits(PFalse, s, b);
+  GetDigits(false, s, b);
   v = _atoi64(b);
   return s;
 }
@@ -229,32 +229,26 @@ void PChannel::Construct()
 }
 
 
-PString PChannel::GetName() const
-{
-  PAssertAlways(PUnimplementedFunction);
-  return PString();
-}
-
-
 PBoolean PChannel::Read(void *, PINDEX)
 {
   PAssertAlways(PUnimplementedFunction);
-  return PFalse;
+  return false;
 }
 
 
 PBoolean PChannel::Write(const void *, PINDEX)
 {
   PAssertAlways(PUnimplementedFunction);
-  return PFalse;
+  return false;
 }
 
 
 PBoolean PChannel::Close()
 {
-  return PFalse;
+  return false;
 }
 
+
 ///////////////////////////////////////////////////////////////////////////////
 // Directories
 
@@ -273,7 +267,7 @@ PBoolean PDirectory::Change(const PString & p)
 
   if (d[0] != '\\')
     if (_chdrive(toupper(d[0])-'A'+1) != 0)
-      return PFalse;
+      return false;
 
   return _chdir(d + ".") == 0;
 }
@@ -287,11 +281,11 @@ PBoolean PDirectory::Filtered()
   char * name = fileinfo.cFileName;
 #endif // UNICODE
   if (strcmp(name, ".") == 0)
-    return PTrue;
+    return true;
   if (strcmp(name, "..") == 0)
-    return PTrue;
+    return true;
   if (scanMask == PFileInfo::AllPermissions)
-    return PFalse;
+    return false;
 
   PFileInfo inf;
   PAssert(PFile::GetInfo(*this+name, inf), POperatingSystemError);
@@ -326,9 +320,9 @@ PStringArray PDirectory::GetPath() const
     return path;
 
   if ((*this)[1] == ':')
-    path = Tokenise("/\\", PFalse);
+    path = Tokenise("/\\", false);
   else {
-    path = Mid(2).Tokenise("/\\", PFalse);
+    path = Mid(2).Tokenise("/\\", false);
     path[0].Splice("\\\\", 0);
   }
 
@@ -346,17 +340,28 @@ PBoolean PDirectory::GetInfo(PFileInfo & info) const
 }
 
 
+PBoolean PDirectory::Exists(const PString & path)
+{
+  if (_access(path, 0) != 0)
+    return false;
+
+  // Could be a file, so actually need to try and open it
+  PDirectory dir(path);
+  return dir.Open();
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // File Path
 
 PFilePath::PFilePath(const PString & str)
-  : PCaselessString(PDirectory::CreateFullPath(str, PFalse))
+  : PCaselessString(PDirectory::CreateFullPath(str, false))
 {
 }
 
 
 PFilePath::PFilePath(const char * cstr)
-  : PCaselessString(PDirectory::CreateFullPath(cstr, PFalse))
+  : PCaselessString(PDirectory::CreateFullPath(cstr, false))
 {
 }
 
@@ -383,14 +388,14 @@ PFilePath::PFilePath(const char * prefix, const char * dir)
   else
     *this += "PW";
   *this += "XXXXXX";
-  PAssert(_mktemp(GetPointer()) != NULL, "Could not make temporary file");
+  PAssert(_mktemp(theArray) != NULL, "Could not make temporary file");
 }
 
 
 void PFilePath::AssignContents(const PContainer & cont)
 {
   PCaselessString::AssignContents(cont);
-  PCaselessString::AssignContents(PDirectory::CreateFullPath(*this, PFalse));
+  PCaselessString::AssignContents(PDirectory::CreateFullPath(*this, false));
 }
 
 
@@ -528,11 +533,11 @@ PBoolean PFile::Access(const PFilePath & name, OpenMode mode)
 PBoolean PFile::Remove(const PString & name, PBoolean force)
 {
   if (remove(name) == 0)
-    return PTrue;
+    return true;
   if (!force || errno != EACCES)
-    return PFalse;
+    return false;
   if (_chmod(name, _S_IWRITE) != 0)
-    return PFalse;
+    return false;
   return remove(name) == 0;
 }
 
@@ -545,15 +550,15 @@ PBoolean PFile::Rename(const PFilePath & oldname, const PString & newname, PBool
 #else
     errno = EINVAL;
 #endif // _WIN32_WCE
-    return PFalse;
+    return false;
   }
   PString fullname = oldname.GetDirectory() + newname;
   if (rename(oldname, fullname) == 0)
-    return PTrue;
+    return true;
   if (!force || errno == ENOENT || !Exists(fullname))
-    return PFalse;
-  if (!Remove(fullname, PTrue))
-    return PFalse;
+    return false;
+  if (!Remove(fullname, true))
+    return false;
   return rename(oldname, fullname) == 0;
 }
 
@@ -561,14 +566,14 @@ PBoolean PFile::Rename(const PFilePath & oldname, const PString & newname, PBool
 PBoolean PFile::Move(const PFilePath & oldname, const PFilePath & newname, PBoolean force)
 {
   if (rename(oldname, newname) == 0)
-    return PTrue;
+    return true;
   if (errno == ENOENT)
-    return PFalse;
+    return false;
   if (force && Exists(newname)) {
-    if (!Remove(newname, PTrue))
-      return PFalse;
+    if (!Remove(newname, true))
+      return false;
     if (rename(oldname, newname) == 0)
-      return PTrue;
+      return true;
   }
   return Copy(oldname, newname, force) && Remove(oldname);
 }
@@ -696,9 +701,11 @@ static int FileSecurityPermissions(const PFilePath & filename, int newPermission
       DWORD domainLen = 1000;
       SID_NAME_USE usage;
       if (LookupAccountSid(NULL, &ace->SidStart,
-                           account.GetPointer(1000), &accountLen,
-                           domain.GetPointer(1000), &domainLen,
+                           account.GetPointerAndSetLength(accountLen-1), &accountLen,
+                           domain.GetPointerAndSetLength(domainLen-1), &domainLen,
                            &usage)) {
+        account.MakeMinimumSize(accountLen);
+        domain.MakeMinimumSize(domainLen);
         if (account *= "None") {
           cygwinMask |= 2;
           TwiddleBits(newPermissions, oldPermissions, PFileInfo::WorldRead,
@@ -746,7 +753,7 @@ static int FileSecurityPermissions(const PFilePath & filename, int newPermission
 PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & info)
 {
   if (name.IsEmpty())
-    return PFalse;
+    return false;
 
   PString fn = name;
   PINDEX pos = fn.GetLength()-1;
@@ -756,7 +763,7 @@ PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & info)
 
   struct stat s;
   if (stat(fn, &s) != 0)
-    return PFalse;
+    return false;
 
   info.created =  (s.st_ctime < 0) ? 0 : s.st_ctime;
   info.modified = (s.st_mtime < 0) ? 0 : s.st_mtime;
@@ -799,7 +806,7 @@ PBoolean PFile::GetInfo(const PFilePath & name, PFileInfo & info)
   info.hidden = (attr & _A_HIDDEN) != 0;
 #endif
 
-  return PTrue;
+  return true;
 }
 
 #endif // _WIN32_WCE
@@ -816,17 +823,17 @@ PBoolean PFile::SetPermissions(const PFilePath & name, int permissions)
 
 PBoolean PFile::IsTextFile() const
 {
-  return PFalse;
+  return false;
 }
 
 
-PBoolean PFile::Open(OpenMode mode, int opts)
+PBoolean PFile::Open(OpenMode mode, OpenOptions opts)
 {
   Close();
   clear();
 
   if (path.IsEmpty())
-    path = PFilePath("PWL", NULL);
+    path = PFilePath("PTL", NULL);
 
   int oflags = IsTextFile() ? _O_TEXT : _O_BINARY;
   switch (mode) {
@@ -852,22 +859,22 @@ PBoolean PFile::Open(OpenMode mode, int opts)
       PAssertAlways(PInvalidParameter);
   }
 
-  if ((opts&Create) != 0)
+  if (opts & Create)
     oflags |= O_CREAT;
-  if ((opts&Exclusive) != 0)
+  if (opts & Exclusive)
     oflags |= O_EXCL;
-  if ((opts&Truncate) != 0)
+  if (opts & Truncate)
     oflags |= O_TRUNC;
 
-  if ((opts&Temporary) != 0)
-    removeOnClose = PTrue;
+  if (opts & Temporary)
+    removeOnClose = true;
 
   int sflags = _SH_DENYNO;
-  if ((opts&DenySharedRead) == DenySharedRead)
+  if (opts & DenySharedRead)
     sflags = _SH_DENYRD;
-  else if ((opts&DenySharedWrite) == DenySharedWrite)
+  else if (opts & DenySharedWrite)
     sflags = _SH_DENYWR;
-  else if ((opts&(DenySharedRead|DenySharedWrite)) != 0)
+  else if (opts & (DenySharedRead|DenySharedWrite))
     sflags = _SH_DENYWR;
 
   os_handle = _sopen(path, oflags, sflags, S_IREAD|S_IWRITE);
@@ -880,7 +887,7 @@ PBoolean PFile::Open(OpenMode mode, int opts)
 
 PBoolean PFile::SetLength(off_t len)
 {
-  return ConvertOSError(_chsize(GetHandle(), len));
+  return ConvertOSError(_chsize(GetOSHandleAsInt(), len));
 }
 
 
@@ -889,29 +896,7 @@ PBoolean PFile::SetLength(off_t len)
 
 PBoolean PTextFile::IsTextFile() const
 {
-  return PTrue;
-}
-
-
-PBoolean PTextFile::ReadLine(PString & str)
-{
-  char * ptr = str.GetPointer(100);
-  PINDEX len = 0;
-  int c;
-  while ((c = ReadChar()) >= 0 && c != '\n') {
-    *ptr++ = (char)c;
-    if (++len >= str.GetSize())
-      ptr = str.GetPointer(len + 100) + len;
-  }
-  *ptr = '\0';
-  PAssert(str.MakeMinimumSize(), POutOfMemory);
-  return c >= 0 || len > 0;
-}
-
-
-PBoolean PTextFile::WriteLine(const PString & str)
-{
-  return WriteString(str) && WriteChar('\n');
+  return true;
 }
 
 
@@ -919,13 +904,11 @@ PBoolean PTextFile::WriteLine(const PString & str)
 // PConsoleChannel
 
 PConsoleChannel::PConsoleChannel()
-  : m_hConsole(INVALID_HANDLE_VALUE)
 {
 }
 
 
 PConsoleChannel::PConsoleChannel(ConsoleType type)
-  : m_hConsole(INVALID_HANDLE_VALUE)
 {
   Open(type);
 }
@@ -946,9 +929,7 @@ PBoolean PConsoleChannel::Open(ConsoleType type)
   return false;
 #else
   static DWORD HandleNames[] = { STD_INPUT_HANDLE, STD_OUTPUT_HANDLE, STD_ERROR_HANDLE };
-  if (!DuplicateHandle(GetCurrentProcess(), GetStdHandle(HandleNames[type]),
-                       GetCurrentProcess(), &m_hConsole,
-                       NULL, FALSE, DUPLICATE_SAME_ACCESS))
+  if (!m_hConsole.Duplicate(GetStdHandle(HandleNames[type])))
     return ConvertOSError(-2);
 
   os_handle = type;
@@ -965,7 +946,7 @@ PString PConsoleChannel::GetName() const
 
 PBoolean PConsoleChannel::Read(void * buffer, PINDEX length)
 {
-  if (m_hConsole == INVALID_HANDLE_VALUE)
+  if (!m_hConsole.IsValid())
     return ConvertOSError(-2, LastReadError);
 
   flush();
@@ -982,7 +963,7 @@ PBoolean PConsoleChannel::Read(void * buffer, PINDEX length)
 
 PBoolean PConsoleChannel::Write(const void * buffer, PINDEX length)
 {
-  if (m_hConsole == INVALID_HANDLE_VALUE)
+  if (!m_hConsole.IsValid())
     return ConvertOSError(-2, LastReadError);
 
   flush();
@@ -999,16 +980,40 @@ PBoolean PConsoleChannel::Write(const void * buffer, PINDEX length)
 
 PBoolean PConsoleChannel::Close()
 {
-  if (m_hConsole == INVALID_HANDLE_VALUE)
+  if (!m_hConsole.IsValid())
     return false;
 
-  CloseHandle(m_hConsole);
-  m_hConsole = INVALID_HANDLE_VALUE;
+  m_hConsole.Close();
   os_handle = -1;
   return true;
 }
 
 
+#ifdef _WIN32_WCE
+bool PConsoleChannel::SetLocalEcho(bool)
+{
+  return false;
+}
+#else
+bool PConsoleChannel::SetLocalEcho(bool localEcho)
+{
+  if (!m_hConsole.IsValid())
+    return ConvertOSError(-2, LastReadError);
+
+  DWORD mode;
+  if (!GetConsoleMode(m_hConsole, &mode))
+    return false;
+
+  if (localEcho)
+    mode |= ENABLE_ECHO_INPUT;
+  else
+    mode &= ~ENABLE_ECHO_INPUT;
+
+  return SetConsoleMode(m_hConsole, mode);
+}
+#endif
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PProcess
 
@@ -1042,8 +1047,6 @@ void PProcess::Construct()
   _wsetsize(1, &ws);
 #endif
 
-  houseKeeper = NULL;
-
 #ifndef _WIN32_WCE 
   PreviousSigIntHandler = signal(SIGINT, SignalHandler);
   PreviousSigTermHandler = signal(SIGTERM, SignalHandler);
@@ -1054,7 +1057,7 @@ void PProcess::Construct()
 PBoolean PProcess::SetMaxHandles(int /*newLimit*/)
 {
   // Not applicable
-  return PTrue;
+  return true;
 }
 
 
