@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23292 $
- * $Author: rjongbloed $
- * $Date: 2009-08-28 01:13:05 -0500 (Fri, 28 Aug 2009) $
+ * $Revision: 29106 $
+ * $Author: csoutheren $
+ * $Date: 2013-02-14 23:08:54 -0600 (Thu, 14 Feb 2013) $
  */
 
 #include <ptlib.h>
@@ -53,35 +53,6 @@ class PSound;
 
 PCREATE_SOUND_PLUGIN(WindowsMultimedia, PSoundChannelWin32);
 
-class PMultiMediaFile
-{
-  public:
-    PMultiMediaFile();
-    ~PMultiMediaFile();
-
-    PBoolean CreateWaveFile(const PFilePath & filename,
-                        const PWaveFormat & waveFormat,
-                        DWORD dataSize);
-    PBoolean OpenWaveFile(const PFilePath & filename,
-                      PWaveFormat & waveFormat,
-                      DWORD & dataSize);
-
-    PBoolean Open(const PFilePath & filename, DWORD dwOpenFlags, LPMMIOINFO lpmmioinfo = NULL);
-    PBoolean Close(UINT wFlags = 0);
-    PBoolean Ascend(MMCKINFO & ckinfo, UINT wFlags = 0);
-    PBoolean Descend(UINT wFlags, MMCKINFO & ckinfo, LPMMCKINFO lpckParent = NULL);
-    PBoolean Read(void * data, PINDEX len);
-    PBoolean CreateChunk(MMCKINFO & ckinfo, UINT wFlags = 0);
-    PBoolean Write(const void * data, PINDEX len);
-
-    DWORD GetLastError() const { return dwLastError; }
-
-  protected:
-    HMMIO hmmio;
-    DWORD dwLastError;
-};
-
-
 #define new PNEW
 
 
@@ -104,7 +75,7 @@ PBoolean PMultiMediaFile::CreateWaveFile(const PFilePath & filename,
                                      DWORD dataSize)
 {
   if (!Open(filename, MMIO_CREATE|MMIO_WRITE))
-    return PFalse;
+    return false;
 
   MMCKINFO mmChunk;
   mmChunk.fccType = mmioFOURCC('W', 'A', 'V', 'E');
@@ -114,16 +85,16 @@ PBoolean PMultiMediaFile::CreateWaveFile(const PFilePath & filename,
 
   // Create a RIFF chunk
   if (!CreateChunk(mmChunk, MMIO_CREATERIFF))
-    return PFalse;
+    return false;
 
   // Save the format sub-chunk
   mmChunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
   mmChunk.cksize = waveFormat.GetSize();
   if (!CreateChunk(mmChunk))
-    return PFalse;
+    return false;
 
   if (!Write(waveFormat, waveFormat.GetSize()))
-    return PFalse;
+    return false;
 
   // Save the data sub-chunk
   mmChunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
@@ -138,7 +109,7 @@ PBoolean PMultiMediaFile::OpenWaveFile(const PFilePath & filename,
 {
   // Open wave file
   if (!Open(filename, MMIO_READ | MMIO_ALLOCBUF))
-    return PFalse;
+    return false;
 
   MMCKINFO mmParentChunk, mmSubChunk;
   dwLastError = MMSYSERR_NOERROR;
@@ -146,20 +117,20 @@ PBoolean PMultiMediaFile::OpenWaveFile(const PFilePath & filename,
   // Locate a 'RIFF' chunk with a 'WAVE' form type
   mmParentChunk.fccType = mmioFOURCC('W', 'A', 'V', 'E');
   if (!Descend(MMIO_FINDRIFF, mmParentChunk))
-    return PFalse;
+    return false;
 
   // Find the format chunk
   mmSubChunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
   if (!Descend(MMIO_FINDCHUNK, mmSubChunk, &mmParentChunk))
-    return PFalse;
+    return false;
 
   // Get the size of the format chunk, allocate memory for it
   if (!waveFormat.SetSize(mmSubChunk.cksize))
-    return PFalse;
+    return false;
 
   // Read the format chunk
   if (!Read(waveFormat.GetPointer(), waveFormat.GetSize()))
-    return PFalse;
+    return false;
 
   // Ascend out of the format subchunk
   Ascend(mmSubChunk);
@@ -167,16 +138,16 @@ PBoolean PMultiMediaFile::OpenWaveFile(const PFilePath & filename,
   // Find the data subchunk
   mmSubChunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
   if (!Descend(MMIO_FINDCHUNK, mmSubChunk, &mmParentChunk))
-    return PFalse;
+    return false;
 
   // Get the size of the data subchunk
   if (mmSubChunk.cksize == 0) {
     dwLastError = MMSYSERR_INVALPARAM;
-    return PFalse;
+    return false;
   }
 
   dataSize = mmSubChunk.cksize;
-  return PTrue;
+  return true;
 }
 
 
@@ -201,11 +172,11 @@ PBoolean PMultiMediaFile::Open(const PFilePath & filename,
 PBoolean PMultiMediaFile::Close(UINT wFlags)
 {
   if (hmmio == NULL)
-    return PFalse;
+    return false;
 
   mmioClose(hmmio, wFlags);
   hmmio = NULL;
-  return PTrue;
+  return true;
 }
 
 
@@ -225,7 +196,7 @@ PBoolean PMultiMediaFile::Descend(UINT wFlags, MMCKINFO & ckinfo, LPMMCKINFO lpc
 
 PBoolean PMultiMediaFile::Read(void * data, PINDEX len)
 {
-  return mmioRead(hmmio, (char *)data, len) == len;
+  return mmioRead(hmmio, (char *)data, len) == (int)len;
 }
 
 
@@ -238,7 +209,7 @@ PBoolean PMultiMediaFile::CreateChunk(MMCKINFO & ckinfo, UINT wFlags)
 
 PBoolean PMultiMediaFile::Write(const void * data, PINDEX len)
 {
-  return mmioWrite(hmmio, (char *)data, len) == len;
+  return mmioWrite(hmmio, (char *)data, len) == (int)len;
 }
 
 
@@ -352,7 +323,7 @@ PBoolean PWaveFormat::SetSize(PINDEX sz)
   if (sz == 0)
     waveFormat = NULL;
   else {
-    if (sz < sizeof(WAVEFORMATEX))
+    if (sz < (PINDEX)sizeof(WAVEFORMATEX))
       sz = sizeof(WAVEFORMATEX);
     waveFormat = (WAVEFORMATEX *)calloc(sz, 1);
     waveFormat->cbSize = (WORD)(sz - sizeof(WAVEFORMATEX));
@@ -417,7 +388,7 @@ PBoolean PSound::Load(const PFilePath & filename)
   DWORD dataSize;
   if (!mmio.OpenWaveFile(filename, waveFormat, dataSize)) {
     dwLastError = mmio.GetLastError();
-    return PFalse;
+    return false;
   }
 
   encoding = waveFormat->wFormatTag;
@@ -433,16 +404,16 @@ PBoolean PSound::Load(const PFilePath & filename)
   // Allocate and lock memory for the waveform data.
   if (!SetSize(dataSize)) {
     dwLastError = MMSYSERR_NOMEM;
-    return PFalse;
+    return false;
   }
 
   // Read the waveform data subchunk
   if (!mmio.Read(GetPointer(), GetSize())) {
     dwLastError = mmio.GetLastError();
-    return PFalse;
+    return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -460,15 +431,15 @@ PBoolean PSound::Save(const PFilePath & filename)
   PMultiMediaFile mmio;
   if (!mmio.CreateWaveFile(filename, waveFormat, GetSize())) {
     dwLastError = mmio.GetLastError();
-    return PFalse;
+    return false;
   }
 
   if (!mmio.Write(GetPointer(), GetSize())) {
     dwLastError = mmio.GetLastError();
-    return PFalse;
+    return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -477,9 +448,9 @@ PBoolean PSound::Play()
   PSoundChannel channel(PSoundChannel::GetDefaultDevice(PSoundChannel::Player),
                         PSoundChannel::Player);
   if (!channel.IsOpen())
-    return PFalse;
+    return false;
 
-  return channel.PlaySound(*this, PTrue);
+  return channel.PlaySound(*this, true);
 }
 
 PBoolean PSound::Play(const PString & device)
@@ -488,9 +459,9 @@ PBoolean PSound::Play(const PString & device)
   PSoundChannel channel(device,
                        PSoundChannel::Player);
   if (!channel.IsOpen())
-    return PFalse;
+    return false;
 
-  return channel.PlaySound(*this, PTrue);
+  return channel.PlaySound(*this, true);
 }
 
 PBoolean PSound::PlayFile(const PFilePath & file, PBoolean wait)
@@ -531,7 +502,7 @@ void PWaveBuffer::PrepareCommon(PINDEX count)
   memset(&header, 0, sizeof(header));
   header.lpData = (char *)GetPointer();
   header.dwBufferLength = count;
-  header.dwUser = (DWORD)this;
+  //header.dwUser = (DWORD)this;  // NOT USED
 }
 
 
@@ -610,7 +581,7 @@ void PSoundChannelWin32::Construct()
   direction = Player;
   hWaveOut = NULL;
   hWaveIn = NULL;
-  hEventDone = CreateEvent(NULL, PFalse, PFalse, NULL);
+  hEventDone = CreateEvent(NULL, false, false, NULL);
 
   waveFormat.SetFormat(1, 8000, 16);
 
@@ -692,6 +663,8 @@ PStringArray PSoundChannelWin32::GetDeviceNames(Directions dir)
           devices.AppendString(dev);
       } while (++id < numDevs);
       break;
+    case Closed:
+      break;
   }
 
   return devices;
@@ -718,6 +691,8 @@ PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir,
         if (id < waveInGetNumDevs())
           GetWaveInDeviceName(id, deviceName);
         break;
+      case Closed :
+        break;
     }
   }
   else {
@@ -745,13 +720,16 @@ PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir,
           }
         } while (++id < numDevs);
         break;
+
+      case Closed :
+        break;
     }
   }
 
   if (deviceName.IsEmpty())
     return SetErrorValues(NotFound, MMSYSERR_BADDEVICEID|PWIN32ErrorFlag);
 
-  return PTrue;
+  return true;
 }
 
 PBoolean PSoundChannelWin32::Open(const PString & device,
@@ -764,7 +742,7 @@ PBoolean PSoundChannelWin32::Open(const PString & device,
   unsigned id = 0;
 
   if( !GetDeviceID(device, dir, id) )
-    return PFalse;
+    return false;
 
   waveFormat.SetFormat(numChannels, sampleRate, bitsPerSample);
 
@@ -780,7 +758,7 @@ PBoolean PSoundChannelWin32::Open(const PString & device,
   unsigned id = 0;
 
   if( !GetDeviceID(device, dir, id) )
-    return PFalse;
+    return false;
 
   waveFormat = format;
 
@@ -789,7 +767,7 @@ PBoolean PSoundChannelWin32::Open(const PString & device,
 }
 
 
-PBoolean PSoundChannelWin32::OpenDevice(unsigned id)
+PBoolean PSoundChannelWin32::OpenDevice(P_INT_PTR id)
 {
   Close();
 
@@ -800,62 +778,188 @@ PBoolean PSoundChannelWin32::OpenDevice(unsigned id)
 
   WAVEFORMATEX* format = (WAVEFORMATEX*) waveFormat;
 
+  UINT mixerId = UINT_MAX;
   MIXERLINE line;
 
-  DWORD osError = MMSYSERR_BADDEVICEID;
+  MMRESULT osError = MMSYSERR_BADDEVICEID;
   switch (direction) {
     case Player :
-      osError = waveOutOpen(&hWaveOut, id, format, (DWORD)hEventDone, 0, CALLBACK_EVENT);
+      PTRACE(4, "WinSnd\twaveOutOpen, id=" << id);
+      osError = waveOutOpen(&hWaveOut, (UINT)id, format, (DWORD_PTR)hEventDone, 0, CALLBACK_EVENT);
       if (osError == MMSYSERR_NOERROR) {
-        mixerOpen(&hMixer, (UINT)hWaveOut, NULL, NULL, MIXER_OBJECTF_HWAVEOUT);
-        line.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT;
+        if (mixerGetID((HMIXEROBJ)hWaveOut, &mixerId, MIXER_OBJECTF_HWAVEOUT) == MMSYSERR_NOERROR)
+          line.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT;
       }
       break;
 
     case Recorder :
-      osError = waveInOpen(&hWaveIn, id, format, (DWORD)hEventDone, 0, CALLBACK_EVENT);
+      PTRACE(4, "WinSnd\twaveInOpen, id=" << id);
+      osError = waveInOpen(&hWaveIn, (UINT)id, format, (DWORD_PTR)hEventDone, 0, CALLBACK_EVENT);
       if (osError == MMSYSERR_NOERROR) {
-        mixerOpen(&hMixer, (UINT)hWaveIn, NULL, NULL, MIXER_OBJECTF_HWAVEIN);
-        line.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;
+        if (mixerGetID((HMIXEROBJ)hWaveIn, &mixerId, MIXER_OBJECTF_HWAVEIN) == MMSYSERR_NOERROR)
+          line.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;
       }
       break;
+
+    case Closed:
+      break;
   }
 
   if (osError != MMSYSERR_NOERROR)
     return SetErrorValues(NotFound, osError|PWIN32ErrorFlag);
 
-  if (hMixer != NULL) {
-    line.cbStruct = sizeof(line);
-    if (mixerGetLineInfo((HMIXEROBJ)hMixer, &line, MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE) != MMSYSERR_NOERROR) {
-      mixerClose(hMixer);
-      hMixer = NULL;
+  opened = true;
+  os_handle = id;
+
+  if (mixerId == UINT_MAX) {
+    PTRACE(2, "WinSnd\tNo mixer available");
+    return true; // Still return true as have actual device
+  }
+
+  if ((osError = mixerOpen(&hMixer, mixerId, 0, 0, MIXER_OBJECTF_MIXER)) != MMSYSERR_NOERROR) {
+    PTRACE(2, "WinSnd\tFailed to open mixer, error=" << osError);
+    return true; // Still return true as have actual device
+  }
+
+  line.cbStruct = sizeof(line);
+  if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line,
+            MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE)) != MMSYSERR_NOERROR) {
+    PTRACE(2, "WinSnd\tFailed to get mixer info, error=" << osError);
+  }
+  else {
+    bool haveControl = true;
+    MIXERLINECONTROLS controls;
+
+    if ((direction == Recorder)
+#ifndef _WIN32_WCE
+      && ((DWORD)(LOBYTE(LOWORD(GetVersion()))) < 6)
+#endif
+      ) { //5=XP/win2003
+      /* There is no "master" for the recording side, so need to find the
+         single selected input or at least individual microphone input
+         No need to do all of these on Vista/Win7/Win2008 as there is
+         a "master" for every input on those OS */
+
+      DWORD iConn = line.cConnections; //should save first
+      
+      //1)Attempt to find currently selected input
+      MIXERCONTROL muxControl;
+      muxControl.cbStruct = sizeof(muxControl);
+      controls.cbStruct = sizeof(controls);
+      controls.dwLineID = line.dwLineID;
+      controls.dwControlType = MIXERCONTROL_CONTROLTYPE_MUX;
+      controls.cControls = 1;
+      controls.pamxctrl = &muxControl;
+      controls.cbmxctrl = muxControl.cbStruct;
+      if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
+                                          MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) == MMSYSERR_NOERROR) {
+        MIXERCONTROLDETAILS mxcd;
+        mxcd.cbStruct = sizeof(mxcd);
+        mxcd.dwControlID = muxControl.dwControlID;
+        mxcd.cChannels = 1;
+        mxcd.cMultipleItems = muxControl.cMultipleItems;
+        mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_LISTTEXT);
+        std::vector<MIXERCONTROLDETAILS_LISTTEXT> mxcdlt(mxcd.cChannels * mxcd.cMultipleItems); 
+        mxcd.paDetails = &mxcdlt[0];
+        // Get the control list text items (input lines id's)
+        if ((osError = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
+                                              MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_LISTTEXT)) == MMSYSERR_NOERROR) {
+          mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
+          std::vector<MIXERCONTROLDETAILS_BOOLEAN> mxcdb(mxcd.cChannels * mxcd.cMultipleItems);
+          mxcd.paDetails = &mxcdb[0];
+          // Get the control values for line states (selected/not selected)
+          if ((osError = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
+                                                MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE)) == MMSYSERR_NOERROR) {
+            DWORD iMult = mxcd.cMultipleItems;
+            while (iMult > 0) {
+              line.dwLineID = mxcdlt[--iMult].dwParam1;
+              if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line,
+                                              MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_LINEID)) != MMSYSERR_NOERROR)
+                PTRACE(2, "WinSnd\tRecorder: Failed to get mixer info, error=" << osError);
+              else {
+                if (mxcdb[iMult].fValue == 1) {
+                  PTRACE(2, "WinSnd\tRecorder: Found selected input device: " << line.szName);
+                  break;
+                }
+
+                if (iMult == 0) {
+                  haveControl = false;
+                  PTRACE(2, "WinSnd\tRecorder: Failed to find selected input device");
+                }
+              }
+            }
+          }
+          else {
+            haveControl = false;
+            PTRACE(2, "WinSnd\tRecorder: failed to get info about states of input lines, error=" << osError);
+          }
+        }
+        else {
+          haveControl = false;
+          PTRACE(2, "WinSnd\tRecorder: failed to get info about input lines, error=" << osError);
+        }
+      }
+      else {
+        haveControl = false;
+        PTRACE(2, "WinSnd\tRecorder: failed to get mixer line mux control, error=" << osError);
+      }
+
+      //2) Attempt to find microphone input if not seccessfull with selected input
+      if (!haveControl) {
+        haveControl = true;
+        PTRACE(2, "WinSnd\tRecorder: trying to find microphone now...");
+        while (iConn > 0) {
+          line.dwSource = --iConn;
+          if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line, MIXER_GETLINEINFOF_SOURCE)) != MMSYSERR_NOERROR)
+            PTRACE(2, "WinSnd\tFailed to get mixer info, error=" << osError);
+          else if (line.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE) {
+            PTRACE(5, "WinSnd\tFfound microphone, source=" << iConn);
+            break;
+          }
+
+          if (iConn == 0) {
+            PTRACE(2, "WinSnd\tFailed to find microphone info");
+            haveControl = false;
+          }
+        }
+      }
     }
-    else {
+
+    if (haveControl) {
       volumeControl.cbStruct = sizeof(volumeControl);
 
-      MIXERLINECONTROLS controls;
       controls.cbStruct = sizeof(controls);
-      controls.dwLineID = line.dwLineID&0xffff;
+      controls.dwLineID = line.dwLineID;
       controls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
       controls.cControls = 1;
       controls.pamxctrl = &volumeControl;
       controls.cbmxctrl = volumeControl.cbStruct;
 
-      if (mixerGetLineControls((HMIXEROBJ)hMixer, &controls, MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE) != MMSYSERR_NOERROR) {
-        mixerClose(hMixer);
-        hMixer = NULL;
+      if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
+                MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) == MMSYSERR_NOERROR) {
+        muteControl.cbStruct = sizeof(muteControl);
+        controls.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
+        controls.pamxctrl = &muteControl;
+        controls.cbmxctrl = muteControl.cbStruct;
+        if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
+                MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) != MMSYSERR_NOERROR) {
+          PTRACE(2, "WinSnd\tFailed to get mixer line mute control ("
+                 << (direction == Recorder ? "Recorder" : "Player") << "), error=" << osError);
+        }
+        return true;
       }
+      PTRACE(2, "WinSnd\tFailed to get mixer line control, error=" << osError);
     }
   }
 
-  opened = true;
-  os_handle = id;
-  return PTrue;
+  mixerClose(hMixer);
+  hMixer = NULL;
+  return true; // Still return true as have actual device
 }
 
 PBoolean PSoundChannelWin32::IsOpen() const
 { 
-  return opened ? PTrue : PFalse;
+  return opened ? true : false;
 }
 
 PBoolean PSoundChannelWin32::SetFormat(unsigned numChannels,
@@ -926,7 +1030,7 @@ PBoolean PSoundChannelWin32::Close()
 
   opened = false;
   os_handle = -1;
-  return PTrue;
+  return true;
 }
 
 
@@ -941,18 +1045,18 @@ PBoolean PSoundChannelWin32::SetBuffers(PINDEX size, PINDEX count)
 
   PTRACE(3, "WinSnd\tSetting sounds buffers to " << count << " x " << size);
 
-  PBoolean ok = PTrue;
+  PBoolean ok = true;
 
   PWaitAndSignal mutex(bufferMutex);
 
   if (!buffers.SetSize(count))
-    ok = PFalse;
+    ok = false;
   else {
     for (PINDEX i = 0; i < count; i++) {
       if (buffers.GetAt(i) == NULL)
         buffers.SetAt(i, new PWaveBuffer(size));
       if (!buffers[i].SetSize(size))
-        ok = PFalse;
+        ok = false;
     }
   }
 
@@ -974,7 +1078,7 @@ PBoolean PSoundChannelWin32::GetBuffers(PINDEX & size, PINDEX & count)
   else
     size = buffers[0].GetSize();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1021,7 +1125,7 @@ PBoolean PSoundChannelWin32::Write(const void * data, PINDEX size)
   if (size != 0)
     return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastWriteError);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1029,7 +1133,7 @@ PBoolean PSoundChannelWin32::PlaySound(const PSound & sound, PBoolean wait)
 {
   Abort();
 
-  PBoolean ok = PFalse;
+  PBoolean ok = false;
 
   PINDEX bufferSize;
   PINDEX bufferCount;
@@ -1063,7 +1167,7 @@ PBoolean PSoundChannelWin32::PlaySound(const PSound & sound, PBoolean wait)
     }
     else {
       SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastWriteError);
-      ok = PFalse;
+      ok = false;
     }
 
     bufferMutex.Signal();
@@ -1092,7 +1196,7 @@ PBoolean PSoundChannelWin32::PlayFile(const PFilePath & filename, PBoolean wait)
   waveFormat = fileFormat;
   if (!OpenDevice(os_handle)) {
     SetFormat(numChannels, sampleRate, bitsPerSample);
-    return PFalse;
+    return false;
   }
 
   bufferMutex.Wait();
@@ -1141,7 +1245,7 @@ PBoolean PSoundChannelWin32::PlayFile(const PFilePath & filename, PBoolean wait)
     SetFormat(numChannels, sampleRate, bitsPerSample);
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1151,10 +1255,10 @@ PBoolean PSoundChannelWin32::HasPlayCompleted()
 
   for (PINDEX i = 0; i < buffers.GetSize(); i++) {
     if ((buffers[i].header.dwFlags&WHDR_DONE) == 0)
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1162,10 +1266,10 @@ PBoolean PSoundChannelWin32::WaitForPlayCompletion()
 {
   while (!HasPlayCompleted()) {
     if (WaitForSingleObject(hEventDone, INFINITE) != WAIT_OBJECT_0)
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1175,7 +1279,7 @@ PBoolean PSoundChannelWin32::StartRecording()
 
   // See if has started already.
   if (bufferByteOffset != P_MAX_INDEX)
-    return PTrue;
+    return true;
 
   DWORD osError;
 
@@ -1183,15 +1287,15 @@ PBoolean PSoundChannelWin32::StartRecording()
   for (PINDEX i = 0; i < buffers.GetSize(); i++) {
     PWaveBuffer & buffer = buffers[i];
     if ((osError = buffer.Prepare(hWaveIn)) != MMSYSERR_NOERROR)
-      return PFalse;
+      return false;
     if ((osError = waveInAddBuffer(hWaveIn, &buffer.header, sizeof(WAVEHDR))) != MMSYSERR_NOERROR)
-      return PFalse;
+      return false;
   }
 
   bufferByteOffset = 0;
 
   if ((osError = waveInStart(hWaveIn)) == MMSYSERR_NOERROR) // start recording
-    return PTrue;
+    return true;
 
   bufferByteOffset = P_MAX_INDEX;
   return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastReadError);
@@ -1206,13 +1310,13 @@ PBoolean PSoundChannelWin32::Read(void * data, PINDEX size)
     return SetErrorValues(NotOpen, EBADF, LastReadError);
 
   if (!WaitForRecordBufferFull())
-    return PFalse;
+    return false;
 
   PWaitAndSignal mutex(bufferMutex);
 
   // Check to see if Abort() was called in another thread
   if (bufferByteOffset == P_MAX_INDEX)
-    return PFalse;
+    return false;
 
   PWaveBuffer & buffer = buffers[bufferIndex];
 
@@ -1234,14 +1338,14 @@ PBoolean PSoundChannelWin32::Read(void * data, PINDEX size)
     bufferByteOffset = 0;
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannelWin32::RecordSound(PSound & sound)
 {
   if (!WaitForAllRecordBuffersFull())
-    return PFalse;
+    return false;
 
   sound.SetFormat(waveFormat->nChannels,
                   waveFormat->nSamplesPerSec,
@@ -1269,14 +1373,14 @@ PBoolean PSoundChannelWin32::RecordSound(PSound & sound)
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannelWin32::RecordFile(const PFilePath & filename)
 {
   if (!WaitForAllRecordBuffersFull())
-    return PFalse;
+    return false;
 
   PWaitAndSignal mutex(bufferMutex);
 
@@ -1294,7 +1398,7 @@ PBoolean PSoundChannelWin32::RecordFile(const PFilePath & filename)
       return SetErrorValues(Miscellaneous, mmio.GetLastError()|PWIN32ErrorFlag, LastReadError);
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1314,46 +1418,46 @@ PBoolean PSoundChannelWin32::AreAllRecordBuffersFull()
   for (PINDEX i = 0; i < buffers.GetSize(); i++) {
     if ((buffers[i].header.dwFlags&WHDR_DONE) == 0 ||
          buffers[i].header.dwBytesRecorded    == 0)
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannelWin32::WaitForRecordBufferFull()
 {
   if (!StartRecording())  // Start the first read, queue all the buffers
-    return PFalse;
+    return false;
 
   while (!IsRecordBufferFull()) {
     if (WaitForSingleObject(hEventDone, INFINITE) != WAIT_OBJECT_0)
-      return PFalse;
+      return false;
 
     PWaitAndSignal mutex(bufferMutex);
     if (bufferByteOffset == P_MAX_INDEX)
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
 PBoolean PSoundChannelWin32::WaitForAllRecordBuffersFull()
 {
   if (!StartRecording())  // Start the first read, queue all the buffers
-    return PFalse;
+    return false;
 
   while (!AreAllRecordBuffersFull()) {
     if (WaitForSingleObject(hEventDone, INFINITE) != WAIT_OBJECT_0)
-      return PFalse;
+      return false;
 
     PWaitAndSignal mutex(bufferMutex);
     if (bufferByteOffset == P_MAX_INDEX)
-      return PFalse;
+      return false;
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1366,12 +1470,12 @@ PBoolean PSoundChannelWin32::Abort()
 
     if (hWaveOut != NULL || hWaveIn != NULL) {
       for (PINDEX i = 0; i < buffers.GetSize(); i++) {
-        while (buffers[i].Release() == WAVERR_STILLPLAYING) {
+        do {
           if (hWaveOut != NULL)
             waveOutReset(hWaveOut);
           if (hWaveIn != NULL)
             waveInReset(hWaveIn);
-        }
+        } while (buffers[i].Release() == WAVERR_STILLPLAYING);
       }
     }
 
@@ -1386,24 +1490,24 @@ PBoolean PSoundChannelWin32::Abort()
   if (osError != MMSYSERR_NOERROR)
     return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag);
 
-  return PTrue;
+  return true;
 }
 
 
 PString PSoundChannelWin32::GetErrorText(ErrorGroup group) const
 {
-  PString str;
+  char str[256];
 
   if ((lastErrorNumber[group]&PWIN32ErrorFlag) == 0)
     return PChannel::GetErrorText(group);
 
   DWORD osError = lastErrorNumber[group]&~PWIN32ErrorFlag;
   if (direction == Recorder) {
-    if (waveInGetErrorText(osError, str.GetPointer(256), 256) != MMSYSERR_NOERROR)
+    if (waveInGetErrorText(osError, str, sizeof(str)) != MMSYSERR_NOERROR)
       return PChannel::GetErrorText(group);
   }
   else {
-    if (waveOutGetErrorText(osError, str.GetPointer(256), 256) != MMSYSERR_NOERROR)
+    if (waveOutGetErrorText(osError, str, sizeof(str)) != MMSYSERR_NOERROR)
       return PChannel::GetErrorText(group);
   }
 
@@ -1421,8 +1525,7 @@ PBoolean PSoundChannelWin32::SetVolume(unsigned newVolume)
     volume.dwValue = volumeControl.Bounds.dwMaximum;
   else
     volume.dwValue = volumeControl.Bounds.dwMinimum +
-            (DWORD)((volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum) *
-                                                   log10(9.0*newVolume/MaxVolume + 1.0));
+            (DWORD)((volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum)*newVolume/MaxVolume);
   PTRACE(5, "WinSnd\tVolume set to " << newVolume << " -> " << volume.dwValue);
 
   MIXERCONTROLDETAILS details;
@@ -1461,9 +1564,59 @@ PBoolean PSoundChannelWin32::GetVolume(unsigned & oldVolume)
   if (result != MMSYSERR_NOERROR)
     return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);
 
-  oldVolume = 100*(volume.dwValue - volumeControl.Bounds.dwMinimum)/(volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum);
+  oldVolume = MaxVolume*(volume.dwValue + 1 - volumeControl.Bounds.dwMinimum)/(volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum);
   return true;
 }
 
+
+bool PSoundChannelWin32::SetMute(bool newMute)
+{
+  if (!IsOpen() || hMixer == NULL)
+    return SetErrorValues(NotOpen, EBADF);
+
+  MIXERCONTROLDETAILS_UNSIGNED mute;
+  mute.dwValue = newMute;
+  PTRACE(5, "WinSnd\tMute set to " << newMute << " -> " << mute.dwValue);
+
+  MIXERCONTROLDETAILS details;
+  details.cbStruct = sizeof(details);
+  details.dwControlID = muteControl.dwControlID;
+  details.cChannels = 1;
+  details.cMultipleItems = 0;
+  details.cbDetails = sizeof(mute);
+  details.paDetails = &mute;
+
+  MMRESULT result = mixerSetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE);
+  if (result != MMSYSERR_NOERROR)
+    return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);
+
+  return true;
+}
+
+
+bool PSoundChannelWin32::GetMute(bool & oldMute)
+{
+  if (!IsOpen() || hMixer == NULL)
+    return SetErrorValues(NotOpen, EBADF);
+
+  MIXERCONTROLDETAILS_UNSIGNED mute;
+
+  MIXERCONTROLDETAILS details;
+  details.cbStruct = sizeof(details);
+  details.dwControlID = muteControl.dwControlID;
+  details.cChannels = 1;
+  details.cMultipleItems = 0;
+  details.cbDetails = sizeof(mute);
+  details.paDetails = &mute;
+
+  MMRESULT result = mixerGetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE);
+  if (result != MMSYSERR_NOERROR)
+    return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);
+
+  oldMute = mute.dwValue;
+  return true;
+}
+
+
 // End of File ///////////////////////////////////////////////////////////////
 
