@@ -24,9 +24,9 @@
  * Contributor(s): ______________________________________.
  *                 Walter H Whitlock (twohives@nc.rr.com)
  *
- * $Revision: 23962 $
- * $Author: rjongbloed $
- * $Date: 2010-01-21 22:39:06 -0600 (Thu, 21 Jan 2010) $
+ * $Revision: 28935 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 07:12:14 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -45,32 +45,13 @@
 #pragma comment(lib, "gdi32.lib")
 #endif
 
-    
+
 #include <vfw.h>
 
 #ifdef __MINGW32__
 
 #define VHDR_DONE       0x00000001
 #define VHDR_KEYFRAME   0x00000008
-#define WM_USER 1024
-#define WM_CAP_FIRSTA (WM_USER)
-
-#define capSetCallbackOnError(hwnd, fpProc)        ((BOOL)::SendMessage(hwnd, WM_CAP_SET_CALLBACK_ERROR, 0, (LPARAM)(LPVOID)(fpProc)))
-#define capSetCallbackOnFrame(hwnd, fpProc)        ((BOOL)::SendMessage(hwnd, WM_CAP_SET_CALLBACK_FRAME, 0, (LPARAM)(LPVOID)(fpProc)))
-#define capSetCallbackOnVideoStream(hwnd, fpProc)  ((BOOL)::SendMessage(hwnd, WM_CAP_SET_CALLBACK_VIDEOSTREAM, 0, (LPARAM)(LPVOID)(fpProc)))
-#define capGetUserData(hwnd)                       (::SendMessage(hwnd, WM_CAP_GET_USER_DATA, 0, 0))
-#define capSetUserData(hwnd, lUser)                ((BOOL)::SendMessage(hwnd, WM_CAP_SET_USER_DATA, 0, (LPARAM)lUser))
-#define capDriverConnect(hwnd, i)                  ((BOOL)::SendMessage(hwnd, WM_CAP_DRIVER_CONNECT, (WPARAM)(i), 0L))
-#define capDriverDisconnect(hwnd)                  ((BOOL)::SendMessage(hwnd, WM_CAP_DRIVER_DISCONNECT, (WPARAM)0, 0L))
-#define capDriverGetCaps(hwnd, s, wSize)           ((BOOL)::SendMessage(hwnd, WM_CAP_DRIVER_GET_CAPS, (WPARAM)(wSize), (LPARAM)(LPVOID)(LPCAPDRIVERCAPS)(s)))
-#define capGetVideoFormat(hwnd, s, wSize)          ((DWORD)::SendMessage(hwnd, WM_CAP_GET_VIDEOFORMAT, (WPARAM)(wSize), (LPARAM)(LPVOID)(s)))
-#define capGetVideoFormatSize(hwnd)                ((DWORD)::SendMessage(hwnd, WM_CAP_GET_VIDEOFORMAT, 0, NULL))
-#define capSetVideoFormat(hwnd, s, wSize)          ((BOOL)::SendMessage(hwnd, WM_CAP_SET_VIDEOFORMAT, (WPARAM)(wSize), (LPARAM)(LPVOID)(s)))
-#define capPreview(hwnd, f)                        ((BOOL)::SendMessage(hwnd, WM_CAP_SET_PREVIEW, (WPARAM)(BOOL)(f), 0L))
-#define capGetStatus(hwnd, s, wSize)               ((BOOL)::SendMessage(hwnd, WM_CAP_GET_STATUS, (WPARAM)(wSize), (LPARAM)(LPVOID)(LPCAPSTATUS)(s)))
-#define capGrabFrameNoStop(hwnd)                   ((BOOL)::SendMessage(hwnd, WM_CAP_GRAB_FRAME_NOSTOP, (WPARAM)0, (LPARAM)0L))
-#define capCaptureSetSetup(hwnd, s, wSize)         ((BOOL)::SendMessage(hwnd, WM_CAP_SET_SEQUENCE_SETUP, (WPARAM)(wSize), (LPARAM)(LPVOID)(LPCAPTUREPARMS)(s)))
-#define capCaptureGetSetup(hwnd, s, wSize)         ((BOOL)::SendMessage(hwnd, WM_CAP_GET_SEQUENCE_SETUP, (WPARAM)(wSize), (LPARAM)(LPVOID)(LPCAPTUREPARMS)(s)))
 
 extern "C" {
 #ifdef _MSC_VER
@@ -82,8 +63,6 @@ BOOL VFWAPI capGetDriverDescriptionA (WORD wDriverIndex, LPSTR lpszName,
 #endif // __MINGW32
 
 
-#define STEP_GRAB_CAPTURE 1
-
 
 /**This class defines a video input device.
  */
@@ -98,7 +77,8 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
 
     /**Close the video input device on destruction.
       */
-    ~PVideoInputDevice_VideoForWindows() { Close(); }
+    ~PVideoInputDevice_VideoForWindows();
+
 
     /** Is the device a camera, and obtain video
      */
@@ -107,18 +87,11 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
     virtual PStringArray GetDeviceNames() const
       { return GetInputDeviceNames(); }
 
-    /**Retrieve a list of Device Capabilities
-      */
-    static bool GetDeviceCapabilities(
-      const PString & /*deviceName*/, ///< Name of device
-      Capabilities * /*caps*/         ///< List of supported capabilities
-    ) { return false; }
-
     /**Open the device given the device name.
       */
     virtual PBoolean Open(
       const PString & deviceName,   /// Device name to open
-      PBoolean startImmediate = PTrue    /// Immediately start device
+      PBoolean startImmediate = true    /// Immediately start device
     );
 
     /**Determine if the device is currently open.
@@ -129,6 +102,13 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
       */
     virtual PBoolean Close();
 
+    /**Retrieve a list of Device Capabilities
+      */
+    bool GetDeviceCapabilities(
+      Capabilities * /*caps*/         ///< List of supported capabilities
+    );
+    static PBoolean GetDeviceCapabilities(const PString & deviceName, Capabilities * capabilities);
+
     /**Start the video device I/O.
       */
     virtual PBoolean Start();
@@ -142,14 +122,14 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
     virtual PBoolean IsCapturing();
 
     /**Set the colour format to be used.
-       Note that this function does not do any conversion. If it returns PTrue
+       Note that this function does not do any conversion. If it returns true
        then the video device does the colour format in native mode.
 
        To utilise an internal converter use the SetColourFormatConverter()
        function.
 
        Default behaviour sets the value of the colourFormat variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetColourFormat(
       const PString & colourFormat // New colour format for device.
@@ -158,7 +138,7 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
     /**Set the video frame rate to be used on the device.
 
        Default behaviour sets the value of the frameRate variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameRate(
       unsigned rate  /// Frames  per second
@@ -170,7 +150,7 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
        this function will fail.  See SetFrameSizeConverter().
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   /// New width of frame
@@ -199,17 +179,12 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
     );
 
 
-    /**Try all known video formats & see which ones are accepted by the video driver
-     */
-    virtual PBoolean TestAllFormats();
+    virtual bool SetCaptureMode(unsigned mode);
+    virtual int GetCaptureMode() const { return useVideoMode; }
+
 
   protected:
-
-   /**Check the hardware can do the asked for size.
-
-       Note that not all cameras can provide all frame sizes.
-     */
-    virtual PBoolean VerifyHardwareFrameSize(unsigned width, unsigned height);
+    PBoolean VerifyHardwareFrameSize(unsigned width, unsigned height);
 
     PDECLARE_NOTIFIER(PThread, PVideoInputDevice_VideoForWindows, HandleCapture);
 
@@ -226,7 +201,8 @@ class PVideoInputDevice_VideoForWindows : public PVideoInputDevice
     PMutex        operationMutex;
 
     PSyncPoint    frameAvailable;
-    LPBYTE        lastFramePtr;
+    bool          useVideoMode;
+    LPBYTE        lastFrameData;
     unsigned      lastFrameSize;
     PMutex        lastFrameMutex;
     bool          isCapturingNow;
@@ -360,7 +336,7 @@ PBoolean PVideoDeviceBitmap::ApplyFormat(HWND hWnd, const FormatTableEntry & for
             << PString(formatTableEntry.colourFormat) << ' '
             << bmi.bmiHeader.biWidth << "x" << bmi.bmiHeader.biHeight
             << " sz=" << bmi.bmiHeader.biSizeImage << " time=" << (PTimer::Tick() - startTime));
-    return PTrue;
+    return true;
   }
 
   if (formatTableEntry.negHeight) {
@@ -370,7 +346,7 @@ PBoolean PVideoDeviceBitmap::ApplyFormat(HWND hWnd, const FormatTableEntry & for
               << PString(formatTableEntry.colourFormat) << ' '
               << bmi.bmiHeader.biWidth << "x" << bmi.bmiHeader.biHeight
               << " sz=" << bmi.bmiHeader.biSizeImage << " time=" << (PTimer::Tick() - startTime));
-      return PTrue;
+      return true;
     }
   }
 
@@ -379,7 +355,7 @@ PBoolean PVideoDeviceBitmap::ApplyFormat(HWND hWnd, const FormatTableEntry & for
           << bmi.bmiHeader.biWidth << "x" << bmi.bmiHeader.biHeight
           << " sz=" << bmi.bmiHeader.biSizeImage << " time=" << (PTimer::Tick() - startTime)
           << " - lastError=" << ::GetLastError());
-  return PFalse;
+  return false;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -403,11 +379,71 @@ PVideoInputDevice_VideoForWindows::PVideoInputDevice_VideoForWindows()
 {
   captureThread = NULL;
   hCaptureWindow = NULL;
-  lastFramePtr = NULL;
   lastFrameSize = 0;
-  isCapturingNow = PFalse;
+  isCapturingNow = false;
+
+  useVideoMode    = false;
+  lastFrameData   = NULL;
 }
 
+
+PVideoInputDevice_VideoForWindows::~PVideoInputDevice_VideoForWindows()
+{
+  if(lastFrameData)
+    delete[] lastFrameData;
+  Close();
+}
+
+
+bool PVideoInputDevice_VideoForWindows::SetCaptureMode(unsigned mode)
+{
+  useVideoMode = mode != 0;
+
+  // Do nothing if we are currently capturing (we don't support switching between picture- and video-mode during a capture).
+  if(IsCapturing())
+    return false;
+
+  // Set the callback function for complete frames
+  BOOL result;
+  if (useVideoMode)
+    result = capSetCallbackOnVideoStream(hCaptureWindow, VideoHandler);
+  else
+    result = capSetCallbackOnFrame(hCaptureWindow, VideoHandler);
+
+  if (!result) {
+    lastError = ::GetLastError();
+    PTRACE(1, "PVidInp\tFailed to set callback on VfW - " << lastError);
+    return false;
+  }
+
+  CAPTUREPARMS parms;
+  memset(&parms, 0, sizeof(parms));
+  if (!capCaptureGetSetup(hCaptureWindow, &parms, sizeof(parms))) {
+    lastError = ::GetLastError();
+    PTRACE(1, "PVidInp\tcapCaptureGetSetup: failed - " << lastError);
+    return false;
+  }
+
+  // For video mode we must tell VfW to work in a separate background thread, or our application will lock otherwise.
+  if (useVideoMode) {
+    parms.fYield = TRUE;
+    parms.dwIndexSize = 324000;
+  }
+  else {
+    parms.fYield = FALSE;
+    parms.dwIndexSize = 0;
+  }
+
+  if (!capCaptureSetSetup(hCaptureWindow, &parms, sizeof(parms))) {
+    lastError = ::GetLastError();
+    PTRACE(1, "PVidInp\tcapCaptureSetSetup: failed - " << lastError);
+    return false;
+  }
+
+  return true;
+}
+
+
 PBoolean PVideoInputDevice_VideoForWindows::Open(const PString & devName, PBoolean startImmediate)
 {
   Close();
@@ -426,13 +462,13 @@ PBoolean PVideoInputDevice_VideoForWindows::Open(const PString & devName, PBoole
   if (hCaptureWindow == NULL) {
     delete captureThread;
     captureThread = NULL;
-    return PFalse;
+    return false;
   }
 
   if (startImmediate)
     return Start();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -447,7 +483,7 @@ PBoolean PVideoInputDevice_VideoForWindows::Close()
   PWaitAndSignal mutex(operationMutex);
 
   if (!IsOpen())
-    return PFalse;
+    return false;
  
   Stop();
 
@@ -470,7 +506,35 @@ PBoolean PVideoInputDevice_VideoForWindows::Close()
   delete captureThread;
   captureThread = NULL;
 
-  return PTrue;
+  return true;
+}
+
+
+PBoolean PVideoInputDevice_VideoForWindows::GetDeviceCapabilities(const PString & deviceName,
+                                                                  Capabilities * capabilities)
+{
+  PVideoInputDevice_VideoForWindows instance;
+  return instance.Open(deviceName, false) && instance.GetDeviceCapabilities(capabilities);
+}
+
+
+bool PVideoInputDevice_VideoForWindows::GetDeviceCapabilities(Capabilities * caps)
+{
+  for (PINDEX prefFormatIdx = 0; FormatTable[prefFormatIdx].colourFormat != NULL; prefFormatIdx++) {
+    PVideoDeviceBitmap bi(hCaptureWindow, FormatTable[prefFormatIdx].bitCount); 
+    bi->bmiHeader.biCompression = FormatTable[prefFormatIdx].compression;
+    for (PINDEX prefResizeIdx = 0; prefResizeIdx < PARRAYSIZE(winTestResTable); prefResizeIdx++) {
+      bi->bmiHeader.biWidth = winTestResTable[prefResizeIdx].device_width;
+      bi->bmiHeader.biHeight = winTestResTable[prefResizeIdx].device_height;
+      if (bi.ApplyFormat(hCaptureWindow, FormatTable[prefFormatIdx]) && caps != NULL) {
+        PVideoFrameInfo frameInfo;
+        frameInfo.SetFrameSize(winTestResTable[prefResizeIdx].device_width,
+                               winTestResTable[prefResizeIdx].device_height);
+        caps->framesizes.push_back(frameInfo);
+      }
+    }
+  }
+  return !caps->framesizes.empty();
 }
 
 
@@ -479,22 +543,26 @@ PBoolean PVideoInputDevice_VideoForWindows::Start()
   PWaitAndSignal mutex(operationMutex);
 
   if (IsCapturing())
-    return PTrue;
+    return true;
+
+  if (!useVideoMode) {
+    isCapturingNow = true;
+    return capGrabFrameNoStop(hCaptureWindow);
+  }
 
-#if STEP_GRAB_CAPTURE
-  isCapturingNow = PTrue;
-  return capGrabFrameNoStop(hCaptureWindow);
-#else
   if (capCaptureSequenceNoFile(hCaptureWindow)) {
     PCapStatus status(hCaptureWindow);
     isCapturingNow = status.fCapturingNow;
+
+    // As initializing the camera takes some time, and video-mode runs in a background thread, we need to wait for the first frame here.
+    // Otherwise "GetFrameDataNoDelay" might time-out.
+    frameAvailable.Wait();
     return isCapturingNow;
   }
 
   lastError = ::GetLastError();
   PTRACE(1, "PVidInp\tcapCaptureSequenceNoFile: failed - " << lastError);
-  return PFalse;
-#endif
+  return false;
 }
 
 
@@ -503,18 +571,20 @@ PBoolean PVideoInputDevice_VideoForWindows::Stop()
   PWaitAndSignal mutex(operationMutex);
 
   if (!IsCapturing())
-    return PFalse;
-  isCapturingNow = PFalse;
-#if STEP_GRAB_CAPTURE
-  return IsOpen() && frameAvailable.Wait(1000);
-#else
+    return false;
+  isCapturingNow = false;
+
+  // If using the picture mode, we just need to wait for the very next frame ...
+  if (!useVideoMode)
+    return IsOpen() && frameAvailable.Wait(1000);
+
+  // ... otherwise we need to explicitely stop capturing.
   if (capCaptureStop(hCaptureWindow))
-    return PTrue;
+    return true;
 
   lastError = ::GetLastError();
   PTRACE(1, "PVidInp\tcapCaptureStop: failed - " << lastError);
-  return PFalse;
-#endif
+  return false;
 }
 
 
@@ -538,7 +608,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetColourFormat(const PString & colo
   PString oldFormat = colourFormat;
 
   if (!PVideoDevice::SetColourFormat(colourFmt))
-    return PFalse;
+    return false;
 
   PINDEX i = 0;
   while (FormatTable[i].colourFormat != NULL && !(colourFmt *= FormatTable[i].colourFormat))
@@ -553,7 +623,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetColourFormat(const PString & colo
   else {
     bi->bmiHeader.biCompression = 0xffffffff; // Indicate invalid colour format
     PVideoDevice::SetColourFormat(oldFormat);
-    return PFalse;
+    return false;
   }
 
   // set frame width and height
@@ -562,7 +632,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetColourFormat(const PString & colo
   if (!bi.ApplyFormat(hCaptureWindow, FormatTable[i])) {
     lastError = ::GetLastError();
     PVideoDevice::SetColourFormat(oldFormat);
-    return PFalse;
+    return false;
   }
 
   // Didn't do top down, tell everything we are up side down
@@ -571,7 +641,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetColourFormat(const PString & colo
   if (running)
     return Start();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -580,10 +650,10 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameRate(unsigned rate)
   PWaitAndSignal mutex(operationMutex);
 
   if (!PVideoDevice::SetFrameRate(rate))
-    return PFalse;
+    return false;
 
   if (!IsOpen())
-    return PTrue; // Not open yet, just set internal variables
+    return true; // Not open yet, just set internal variables
 
   PBoolean running = IsCapturing();
   if (running)
@@ -595,29 +665,30 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameRate(unsigned rate)
   if (!capCaptureGetSetup(hCaptureWindow, &parms, sizeof(parms))) {
     lastError = ::GetLastError();
     PTRACE(1, "PVidInp\tcapCaptureGetSetup: failed - " << lastError);
-    return PFalse;
+    return false;
   }
 
   // keep current (default) framerate if 0==frameRate   
   if (0 != frameRate)
     parms.dwRequestMicroSecPerFrame = 1000000 / frameRate;
-  parms.fMakeUserHitOKToCapture = PFalse;
+
+  parms.fMakeUserHitOKToCapture = false;
   parms.wPercentDropForError = 100;
-  parms.fCaptureAudio = PFalse;
-  parms.fAbortLeftMouse = PFalse;
-  parms.fAbortRightMouse = PFalse;
-  parms.fLimitEnabled = PFalse;
+  parms.fCaptureAudio = false;
+  parms.fAbortLeftMouse = false;
+  parms.fAbortRightMouse = false;
+  parms.fLimitEnabled = false;
 
   if (!capCaptureSetSetup(hCaptureWindow, &parms, sizeof(parms))) {
     lastError = ::GetLastError();
     PTRACE(1, "PVidInp\tcapCaptureSetSetup: failed - " << lastError);
-    return PFalse;
+    return false;
   }
     
   if (running)
     return Start();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -644,7 +715,7 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameSize(unsigned width, unsigne
   bi->bmiHeader.biHeight = height;
   if (!bi.ApplyFormat(hCaptureWindow, FormatTable[i])) {
     lastError = ::GetLastError();
-    return PFalse;
+    return false;
   }
 
   // Didn't do top down, tell everything we are up side down
@@ -652,46 +723,23 @@ PBoolean PVideoInputDevice_VideoForWindows::SetFrameSize(unsigned width, unsigne
 
   // verify that the driver really took the frame size
   if (!VerifyHardwareFrameSize(width, height)) 
-    return PFalse; 
+    return false; 
 
   // frameHeight must be positive regardlesss of what the driver says
   if (0 > (int)height) 
     height = (unsigned)-(int)height;
 
   if (!PVideoDevice::SetFrameSize(width, height))
-    return PFalse;
+    return false;
 
   if (running)
     return Start();
 
-  return PTrue;
+  return true;
 }
 
 
-PBoolean PVideoInputDevice_VideoForWindows::TestAllFormats()
-{
-  PBoolean running = IsCapturing();
-  if (running)
-    Stop();
-
-  for (PINDEX prefFormatIdx = 0; FormatTable[prefFormatIdx].colourFormat != NULL; prefFormatIdx++) {
-    PVideoDeviceBitmap bi(hCaptureWindow, FormatTable[prefFormatIdx].bitCount); 
-    bi->bmiHeader.biCompression = FormatTable[prefFormatIdx].compression;
-    for (PINDEX prefResizeIdx = 0; prefResizeIdx < PARRAYSIZE(winTestResTable); prefResizeIdx++) {
-      bi->bmiHeader.biWidth = winTestResTable[prefResizeIdx].device_width;
-      bi->bmiHeader.biHeight = winTestResTable[prefResizeIdx].device_height;
-      bi.ApplyFormat(hCaptureWindow, FormatTable[prefFormatIdx]);
-    } // for prefResizeIdx
-  } // for prefFormatIdx
-
-  if (running)
-    return Start();
-
-  return PTrue;
-}
-
-
-//return PTrue if absolute value of height reported by driver 
+//return true if absolute value of height reported by driver 
 //  is equal to absolute value of current frame height AND
 //  width reported by driver is equal to current frame width
 PBoolean PVideoInputDevice_VideoForWindows::VerifyHardwareFrameSize(unsigned width, unsigned height)
@@ -699,10 +747,10 @@ PBoolean PVideoInputDevice_VideoForWindows::VerifyHardwareFrameSize(unsigned wid
   PCapStatus status(hCaptureWindow);
 
   if (!status.IsOK())
-    return PFalse;
+    return false;
 
   if (width != status.uiImageWidth)
-    return PFalse;
+    return false;
 
   if (0 > (int)height)
     height = (unsigned)-(int)height;
@@ -753,18 +801,18 @@ PBoolean PVideoInputDevice_VideoForWindows::GetFrameDataNoDelay(BYTE * buffer, P
 {
   if (!frameAvailable.Wait(1000)) {
     PTRACE(1, "PVidInp\tTimeout waiting for frame grab!");
-    return PFalse;
+    return false;
   }
 
   bool retval = false;
 
   lastFrameMutex.Wait();
 
-  if (lastFramePtr != NULL) {
+  if (lastFrameData != NULL) {
     if (NULL != converter)
-      retval = converter->Convert(lastFramePtr, buffer, bytesReturned);
+      retval = converter->Convert(lastFrameData, buffer, bytesReturned);
     else {
-      memcpy(buffer, lastFramePtr, lastFrameSize);
+      memcpy(buffer, lastFrameData, lastFrameSize);
       if (bytesReturned != NULL)
         *bytesReturned = lastFrameSize;
       retval = true;
@@ -773,10 +821,8 @@ PBoolean PVideoInputDevice_VideoForWindows::GetFrameDataNoDelay(BYTE * buffer, P
 
   lastFrameMutex.Signal();
 
-#if STEP_GRAB_CAPTURE
-  if (isCapturingNow)
+  if (!useVideoMode && isCapturingNow)
     capGrabFrameNoStop(hCaptureWindow);
-#endif
 
   return retval;
 }
@@ -785,7 +831,7 @@ PBoolean PVideoInputDevice_VideoForWindows::GetFrameDataNoDelay(BYTE * buffer, P
 LRESULT CALLBACK PVideoInputDevice_VideoForWindows::ErrorHandler(HWND hWnd, int id, LPCSTR err)
 {
   if (hWnd == NULL)
-    return PFalse;
+    return false;
 
   return ((PVideoInputDevice_VideoForWindows *)capGetUserData(hWnd))->HandleError(id, err);
 }
@@ -797,14 +843,14 @@ LRESULT PVideoInputDevice_VideoForWindows::HandleError(int id, LPCSTR PTRACE_PAR
     PTRACE(1, "PVidInp\tErrorHandler: [id="<< id << "] " << err);
   }
 
-  return PTrue;
+  return true;
 }
 
 
 LRESULT CALLBACK PVideoInputDevice_VideoForWindows::VideoHandler(HWND hWnd, LPVIDEOHDR vh)
 {
   if (hWnd == NULL || capGetUserData(hWnd) == NULL)
-    return PFalse;
+    return false;
 
   return ((PVideoInputDevice_VideoForWindows *)capGetUserData(hWnd))->HandleVideo(vh);
 }
@@ -814,15 +860,33 @@ LRESULT PVideoInputDevice_VideoForWindows::HandleVideo(LPVIDEOHDR vh)
 {
   if ((vh->dwFlags&(VHDR_DONE|VHDR_KEYFRAME)) != 0) {
     lastFrameMutex.Wait();
-    lastFramePtr = vh->lpData;
-    lastFrameSize = vh->dwBytesUsed;
-    if (lastFrameSize == 0)
-      lastFrameSize = vh->dwBufferLength;
+
+    /**
+    * As in video mode VfW captures in background, and hence might override the buffer of the current frame,
+    * we must copy the frame's data into a separate buffer.
+    */
+
+    // If the size of the current frame is same as of the old ...
+    //    -> ... simply copy the data of the new frame into the buffer ...
+    if(lastFrameSize == vh->dwBytesUsed)
+      memcpy(lastFrameData, vh->lpData, lastFrameSize);
+    else {
+      // ... otherwise delete the old buffer ...
+      if (lastFrameSize)
+        delete[] lastFrameData;
+
+      // ... and allocate a new one.
+      lastFrameSize = vh->dwBytesUsed;
+      lastFrameData = new BYTE[lastFrameSize];
+
+      memcpy(lastFrameData, vh->lpData, lastFrameSize);
+    }
+
     lastFrameMutex.Signal();
     frameAvailable.Signal();
   }
 
-  return PTrue;
+  return true;
 }
 
 
@@ -837,26 +901,28 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
                                                0)) == NULL) {
     lastError = ::GetLastError();
     PTRACE(1, "PVidInp\tcapCreateCaptureWindow failed - " << lastError);
-    return PFalse;
+    return false;
   }
 
   capSetCallbackOnError(hCaptureWindow, ErrorHandler);
 
-#if STEP_GRAB_CAPTURE
-  if (!capSetCallbackOnFrame(hCaptureWindow, VideoHandler)) { //} balance braces
-#else
-  if (!capSetCallbackOnVideoStream(hCaptureWindow, VideoHandler)) {
-#endif
+  BOOL result = FALSE;
+  if (useVideoMode)
+    result = capSetCallbackOnVideoStream(hCaptureWindow, VideoHandler);
+  else
+    result = capSetCallbackOnFrame(hCaptureWindow, VideoHandler);
+
+  if (!result) {
     lastError = ::GetLastError();
-    PTRACE(1, "PVidInp\tcapSetCallbackOnVideoStream failed - " << lastError);
-    return PFalse;
+    PTRACE(1, "PVidInp\tFailed to set callback on VfW - " << lastError);
+    return false;
   }
 
   WORD devId;
 
 #if PTRACING
   if (PTrace::CanTrace(4)) { // list available video capture drivers
-    ostream & trace = PTrace::Begin(5, __FILE__, __LINE__);
+    ostream & trace = PTRACE_BEGIN(5);
     trace << "PVidInp\tEnumerating available video capture drivers:\n";
     for (devId = 0; devId < 10; devId++) { 
       char name[100];
@@ -886,7 +952,7 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
   if (!capDriverConnect(hCaptureWindow, devId)) {
     lastError = ::GetLastError();
     PTRACE(1, "PVidInp\tcapDriverConnect failed - " << lastError);
-    return PFalse;
+    return false;
   }
 
   CAPDRIVERCAPS driverCaps;
@@ -894,7 +960,7 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
   if (!capDriverGetCaps(hCaptureWindow, &driverCaps, sizeof(driverCaps))) {
     lastError = ::GetLastError();
     PTRACE(1, "PVidInp\tcapGetDriverCaps failed - " << lastError);
-    return PFalse;
+    return false;
   }
 
   PTRACE(6, "PVidInp\tEnumerating CAPDRIVERCAPS values:\n"
@@ -908,25 +974,35 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
   
 /*
   if (driverCaps.fHasOverlay)
-    capOverlay(hCaptureWindow, PTrue);
+    capOverlay(hCaptureWindow, true);
   else {
     capPreviewRate(hCaptureWindow, 66);
-    capPreview(hCaptureWindow, PTrue);
+    capPreview(hCaptureWindow, true);
   }
 */
    
-  capPreview(hCaptureWindow, PFalse);
+  capPreview(hCaptureWindow, false);
 
 #if PTRACING
-  //if (PTrace::CanTrace(6))
-  //  TestAllFormats(); // list acceptable formats and frame resolutions for video capture driver
+  if (PTrace::CanTrace(6)) {
+    // Display log for every format set
+    for (PINDEX prefFormatIdx = 0; FormatTable[prefFormatIdx].colourFormat != NULL; prefFormatIdx++) {
+      PVideoDeviceBitmap bi(hCaptureWindow, FormatTable[prefFormatIdx].bitCount); 
+      bi->bmiHeader.biCompression = FormatTable[prefFormatIdx].compression;
+      for (PINDEX prefResizeIdx = 0; prefResizeIdx < PARRAYSIZE(winTestResTable); prefResizeIdx++) {
+        bi->bmiHeader.biWidth = winTestResTable[prefResizeIdx].device_width;
+        bi->bmiHeader.biHeight = winTestResTable[prefResizeIdx].device_height;
+        bi.ApplyFormat(hCaptureWindow, FormatTable[prefFormatIdx]);
+      }
+    }
+  }
 #endif
   
   return SetFrameRate(frameRate) && SetColourFormatConverter(colourFormat.IsEmpty() ? PString("YUV420P") : colourFormat);
 }
 
 
-void PVideoInputDevice_VideoForWindows::HandleCapture(PThread &, INT)
+void PVideoInputDevice_VideoForWindows::HandleCapture(PThread &, P_INT_PTR)
 {
   PBoolean initSucceeded = InitialiseCapture();
 
@@ -979,7 +1055,7 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
       */
     virtual PBoolean Open(
       const PString & deviceName,   /// Device name (filename base) to open
-      PBoolean startImmediate = PTrue    /// Immediately start device
+      PBoolean startImmediate = true    /// Immediately start device
     );
 
     /**Determine if the device is currently open.
@@ -1008,26 +1084,26 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
     { return GetOutputDeviceNames(); }
 
     /**Set the colour format to be used.
-       Note that this function does not do any conversion. If it returns PTrue
+       Note that this function does not do any conversion. If it returns true
        then the video device does the colour format in native mode.
 
        To utilise an internal converter use the SetColourFormatConverter()
        function.
 
        Default behaviour sets the value of the colourFormat variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetColourFormat(
       const PString & colourFormat // New colour format for device.
     );
 
     /**Get the video conversion vertical flip state.
-       Default action is to return PFalse.
+       Default action is to return false.
      */
     virtual PBoolean GetVFlipState();
 
     /**Set the video conversion vertical flip state.
-       Default action is to return PFalse.
+       Default action is to return false.
      */
     virtual PBoolean SetVFlipState(
       PBoolean newVFlipState    /// New vertical flip state
@@ -1039,7 +1115,7 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
        this function will fail.  See SetFrameSizeConverter().
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   /// New width of frame
@@ -1074,6 +1150,7 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
 
   protected:
     PDECLARE_NOTIFIER(PThread, PVideoOutputDevice_Window, HandleDisplay);
+    void CreateDisplayWindow();
     void SetWindowSize();
     void Draw(HDC hDC);
 
@@ -1147,7 +1224,7 @@ PVideoOutputDevice_Window::PVideoOutputDevice_Window()
 {
   m_hWnd = NULL;
   m_thread = NULL;
-  m_flipped = PFalse;
+  m_flipped = false;
   m_lastPosition.x = 0;
   m_lastPosition.y = 0;
   m_fixedSize.cx = 0;
@@ -1187,14 +1264,25 @@ PBoolean PVideoOutputDevice_Window::Open(const PString & name, PBoolean startImm
 
   deviceName = name;
 
-  m_thread = PThread::Create(PCREATE_NOTIFIER(HandleDisplay), "VidOut");
+  if (deviceName.Find("PARENT") == P_MAX_INDEX) {
+    m_thread = PThread::Create(PCREATE_NOTIFIER(HandleDisplay), "VidOut");
 
-  m_openCloseMutex.Signal();
-  m_started.Wait();
+    m_openCloseMutex.Signal();
+    m_started.Wait();     
+  }
+  else {    
+    /* Description: child windows should be created on the same thread as the
+      parent window (well, not necessarily, but it is much less error prone
+      that way) so I am adding another method to handle creating child
+      windows which will be created in the calling thread.
+      Date: 03/15/2012
+      Author: Jonathan M. Henson
+     */
+    CreateDisplayWindow();
+    m_openCloseMutex.Signal();
+  }
 
-  PWaitAndSignal m(m_openCloseMutex);
-
-  return startImmediate ? Start() : m_hWnd != NULL;
+  return startImmediate ? Start() : IsOpen();
 }
 
 
@@ -1209,13 +1297,17 @@ PBoolean PVideoOutputDevice_Window::Close()
   PWaitAndSignal m(m_openCloseMutex);
 
   if (m_hWnd == NULL)
-      return PFalse;
+    return false;
 
   SendMessage(m_hWnd, WM_CLOSE, 0, 0);
-  m_thread->WaitForTermination(3000);
-  delete m_thread;
-  m_thread = NULL;
-  return PTrue;
+
+  if (m_thread != NULL) {
+    m_thread->WaitForTermination(3000);
+    delete m_thread;
+    m_thread = NULL;
+  }
+
+  return true;
 }
 
 
@@ -1224,10 +1316,10 @@ PBoolean PVideoOutputDevice_Window::Start()
   PWaitAndSignal m(m_openCloseMutex);
 
   if (m_hWnd == NULL)
-    return PFalse;
+    return false;
   
   ShowWindow(m_hWnd, SW_SHOW);
-  return PTrue;
+  return true;
 }
 
 
@@ -1238,7 +1330,7 @@ PBoolean PVideoOutputDevice_Window::Stop()
   if (m_hWnd != NULL)
     return ShowWindow(m_hWnd, SW_HIDE);
 
-  return PFalse;
+  return false;
 }
 
 
@@ -1250,10 +1342,10 @@ PBoolean PVideoOutputDevice_Window::SetColourFormat(const PString & colourFormat
                 PVideoOutputDeviceRGB::SetColourFormat(colourFormat)) {
     m_bitmap.bmiHeader.biBitCount = (WORD)(bytesPerPixel*8);
     m_bitmap.bmiHeader.biSizeImage = frameStore.GetSize();
-    return PTrue;
+    return true;
   }
 
-  return PFalse;
+  return false;
 }
 
 
@@ -1267,7 +1359,7 @@ PBoolean PVideoOutputDevice_Window::SetVFlipState(PBoolean newVFlip)
 {
   m_flipped = newVFlip;
   m_bitmap.bmiHeader.biHeight = m_flipped ? frameHeight : -(int)frameHeight;
-  return PTrue;
+  return true;
 }
 
 
@@ -1277,10 +1369,10 @@ PBoolean PVideoOutputDevice_Window::SetFrameSize(unsigned width, unsigned height
     PWaitAndSignal m(mutex);
 
     if (width == frameWidth && height == frameHeight)
-      return PTrue;
+      return true;
 
     if (!PVideoOutputDeviceRGB::SetFrameSize(width, height))
-      return PFalse;
+      return false;
 
     m_bitmap.bmiHeader.biWidth = frameWidth;
     m_bitmap.bmiHeader.biHeight = m_flipped ? frameHeight : -(int)frameHeight;
@@ -1314,13 +1406,13 @@ PBoolean PVideoOutputDevice_Window::FrameComplete()
   PWaitAndSignal m(mutex);
 
   if (m_hWnd == NULL)
-    return PFalse;
+    return false;
 
   HDC hDC = GetDC(m_hWnd);
   Draw(hDC);
   ReleaseDC(m_hWnd, hDC);
 
-  return PTrue;
+  return true;
 }
 
 
@@ -1328,7 +1420,7 @@ PBoolean PVideoOutputDevice_Window::GetPosition(int & x, int & y) const
 {
   x = m_lastPosition.x;
   y = m_lastPosition.y;
-  return PTrue;
+  return true;
 }
 
 
@@ -1359,11 +1451,17 @@ void PVideoOutputDevice_Window::Draw(HDC hDC)
                                0, 0, frameWidth, frameHeight,
                                0, 0, 0, frameHeight,
                                frameStore.GetPointer(), &m_bitmap, DIB_RGB_COLORS);
-  else
+  else {
+#ifdef _WIN32_WCE
+    SetStretchBltMode(hDC, COLORONCOLOR);
+#else
+    SetStretchBltMode(hDC, STRETCH_DELETESCANS);
+#endif
     result = StretchDIBits(hDC,
                            0, 0, rect.right, rect.bottom,
                            0, 0, frameWidth, frameHeight,
                            frameStore.GetPointer(), &m_bitmap, DIB_RGB_COLORS, SRCCOPY);
+  }
 
   if (result == 0) {
     lastError = ::GetLastError();
@@ -1396,12 +1494,12 @@ static int GetTokenValue(const PString & deviceName, const char * token, int def
 }
 
 
-void PVideoOutputDevice_Window::HandleDisplay(PThread &, INT)
+void PVideoOutputDevice_Window::CreateDisplayWindow()
 {
 #ifndef _WIN32_WCE
-  static const char wndClassName[] = "PVideoOutputDevice_Window";
+  static char const wndClassName[] = "PVideoOutputDevice_Window";
 #else
-  static LPCWSTR wndClassName = L"PVideoOutputDevice_Window";
+  static LPCWSTR const wndClassName = L"PVideoOutputDevice_Window";
 #endif
 
   static bool needRegistration = true;
@@ -1410,7 +1508,7 @@ void PVideoOutputDevice_Window::HandleDisplay(PThread &, INT)
 
     WNDCLASS wndClass;
     memset(&wndClass, 0, sizeof(wndClass));
-    wndClass.style = CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS;
+    wndClass.style = CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS|CS_PARENTDC;
     wndClass.lpszClassName = wndClassName;
     wndClass.lpszClassName = wndClassName;
     wndClass.lpfnWndProc = ::WndProc;
@@ -1420,43 +1518,45 @@ void PVideoOutputDevice_Window::HandleDisplay(PThread &, INT)
 
   DWORD dwStyle;
   HWND hParent;
-  if (ParseWindowDeviceName(deviceName, &dwStyle, &hParent)) {
-    PString title = DEFAULT_TITLE;
 
-    PINDEX pos = deviceName.Find("TITLE=\"");
-    if (pos != P_MAX_INDEX) {
-      pos += 6;
-      PINDEX quote = deviceName.FindLast('"');
-      PString quotedTitle = deviceName(pos, quote > pos ? quote : P_MAX_INDEX);
-      title = PString(PString::Literal, quotedTitle);
+  if (!ParseWindowDeviceName(deviceName, &dwStyle, &hParent))
+    return;
+
+  PVarString title = DEFAULT_TITLE;
+  PINDEX pos = deviceName.Find("TITLE=\"");
+  if (pos != P_MAX_INDEX)
+    title = PString(PString::Literal, deviceName.Mid(pos+6));
+
+  m_lastPosition.x = GetTokenValue(deviceName, "X=", CW_USEDEFAULT);
+  m_lastPosition.y = GetTokenValue(deviceName, "Y=", CW_USEDEFAULT);
+  m_fixedSize.cx   = GetTokenValue(deviceName, "WIDTH=", 0);
+  m_fixedSize.cy   = GetTokenValue(deviceName, "HEIGHT=", 0);
+
+  if (m_lastPosition.x == CW_USEDEFAULT && m_lastPosition.y == CW_USEDEFAULT) {
+    if (hParent != NULL) {
+      RECT rect;
+      GetWindowRect(hParent, &rect);
+      m_lastPosition.x = (rect.right + rect.left - frameWidth)/2;
+      m_lastPosition.y = (rect.bottom + rect.top - frameHeight)/2;
     }
-
-    m_lastPosition.x = GetTokenValue(deviceName, "X=", CW_USEDEFAULT);
-    m_lastPosition.y = GetTokenValue(deviceName, "Y=", CW_USEDEFAULT);
-    m_fixedSize.cx   = GetTokenValue(deviceName, "WIDTH=", 0);
-    m_fixedSize.cy   = GetTokenValue(deviceName, "HEIGHT=", 0);
-
-    if (m_lastPosition.x == CW_USEDEFAULT && m_lastPosition.y == CW_USEDEFAULT) {
-      if (hParent != NULL) {
-        RECT rect;
-        GetWindowRect(hParent, &rect);
-        m_lastPosition.x = (rect.right + rect.left - frameWidth)/2;
-        m_lastPosition.y = (rect.bottom + rect.top - frameHeight)/2;
-      }
-      else {
-        m_lastPosition.x = (GetSystemMetrics(SM_CXSCREEN) - frameWidth)/2;
-        m_lastPosition.y = (GetSystemMetrics(SM_CYSCREEN) - frameHeight)/2;
-      }
+    else {
+      m_lastPosition.x = (GetSystemMetrics(SM_CXSCREEN) - frameWidth)/2;
+      m_lastPosition.y = (GetSystemMetrics(SM_CYSCREEN) - frameHeight)/2;
     }
-
-    PVarString windowTitle = title;
-    m_hWnd = CreateWindow(wndClassName,
-                          windowTitle, 
-                          dwStyle,
-                          m_lastPosition.x, m_lastPosition.y, frameWidth, frameHeight,
-                          hParent, NULL, GetModuleHandle(NULL), this);
-    SetWindowSize();
   }
+  
+  m_hWnd = CreateWindow(wndClassName,
+                        title, 
+                        dwStyle,
+                        m_lastPosition.x , m_lastPosition.y, frameWidth, frameHeight,
+                        hParent, NULL, GetModuleHandle(NULL), this);
+  SetWindowSize();
+}
+
+
+void PVideoOutputDevice_Window::HandleDisplay(PThread &, P_INT_PTR)
+{
+  CreateDisplayWindow();
 
   m_started.Signal();
 
@@ -1491,6 +1591,22 @@ LRESULT PVideoOutputDevice_Window::WndProc(UINT uMsg, WPARAM wParam, LPARAM lPar
       }
       break;
 
+    case WM_LBUTTONDBLCLK :
+      if (m_fixedSize.cx < 10000 && m_fixedSize.cy < 10000) {
+        m_fixedSize.cx = 2*(m_fixedSize.cx > 0 ? m_fixedSize.cx : frameWidth);
+        m_fixedSize.cy = 2*(m_fixedSize.cy > 0 ? m_fixedSize.cy : frameHeight);
+        SetWindowSize();
+      }
+      break;
+
+    case WM_RBUTTONDBLCLK :
+      if ((m_fixedSize.cx&1) == 0 && (m_fixedSize.cy&1) == 0) {
+        m_fixedSize.cx = (m_fixedSize.cx > 0 ? m_fixedSize.cx : frameWidth)/2;
+        m_fixedSize.cy = (m_fixedSize.cy > 0 ? m_fixedSize.cy : frameHeight)/2;
+        SetWindowSize();
+      }
+      break;
+
     case WM_CLOSE :
       DestroyWindow(m_hWnd);
       m_hWnd = NULL;
