@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Revision: 28931 $
+ * $Author: csoutheren $
+ * $Date: 2013-01-18 06:31:11 -0600 (Fri, 18 Jan 2013) $
  */
 
 #include <ptlib.h>
@@ -131,7 +131,7 @@ static DWORD SecureCreateKey(HKEY rootKey, const PString & subkey, HKEY & key)
   if (!InitializeSecurityDescriptor(&secdesc, SECURITY_DESCRIPTOR_REVISION))
     return GetLastError();
 
-  static SID_IDENTIFIER_AUTHORITY siaNTAuthority = SECURITY_NT_AUTHORITY;
+  static SID_IDENTIFIER_AUTHORITY siaNTAuthority = { SECURITY_NT_AUTHORITY };
   SecurityID adminID(&siaNTAuthority, 2,
                      SECURITY_BUILTIN_DOMAIN_RID,
                      DOMAIN_ALIAS_RID_ADMINS, 
@@ -139,14 +139,14 @@ static DWORD SecureCreateKey(HKEY rootKey, const PString & subkey, HKEY & key)
   if (!adminID.IsValid())
     return GetLastError();
 
-  static SID_IDENTIFIER_AUTHORITY siaSystemAuthority = SECURITY_NT_AUTHORITY;
+  static SID_IDENTIFIER_AUTHORITY siaSystemAuthority = { SECURITY_NT_AUTHORITY };
   SecurityID systemID(&siaSystemAuthority, 1,
                       SECURITY_LOCAL_SYSTEM_RID,
                       0, 0, 0, 0, 0, 0, 0);
   if (!systemID.IsValid())
     return GetLastError();
 
-  static SID_IDENTIFIER_AUTHORITY siaCreatorAuthority = SECURITY_CREATOR_SID_AUTHORITY;
+  static SID_IDENTIFIER_AUTHORITY siaCreatorAuthority = { SECURITY_CREATOR_SID_AUTHORITY };
   SecurityID creatorID(&siaCreatorAuthority, 1,
                        SECURITY_CREATOR_OWNER_RID,
                        0, 0, 0, 0, 0, 0, 0);
@@ -181,13 +181,13 @@ static DWORD SecureCreateKey(HKEY rootKey, const PString & subkey, HKEY & key)
   if (!AddAccessAllowedAce(dacl, ACL_REVISION2, KEY_ALL_ACCESS, userID))
     return GetLastError();
 
-  if (!SetSecurityDescriptorDacl(&secdesc, PTrue, dacl, PFalse))
+  if (!SetSecurityDescriptorDacl(&secdesc, true, dacl, false))
     return GetLastError();
 
   SECURITY_ATTRIBUTES secattr;
   secattr.nLength = sizeof(secattr);
   secattr.lpSecurityDescriptor = &secdesc;
-  secattr.bInheritHandle = PFalse;
+  secattr.bInheritHandle = false;
 
   DWORD disposition;
 
@@ -228,8 +228,8 @@ RegistryKey::RegistryKey(const PString & subkeyname, OpenMode mode)
 
     subkey = adjustedSubkey;
 
-    if (!proc.GetVersion(PFalse).IsEmpty()) {
-      adjustedSubkey.Replace("CurrentVersion", proc.GetVersion(PFalse));
+    if (!proc.GetVersion(false).IsEmpty()) {
+      adjustedSubkey.Replace("CurrentVersion", proc.GetVersion(false));
       PVarString keyname = adjustedSubkey;
 
       error = RegOpenKeyEx(HKEY_CURRENT_USER, keyname, 0, access, &key);
@@ -281,7 +281,8 @@ RegistryKey::RegistryKey(const PString & subkeyname, OpenMode mode)
     error = RegCreateKeyEx(basekey, subkey, 0, empty, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &key, &disposition);
     if (error != ERROR_SUCCESS) {
       PTRACE(1, "PTLib\tCould not create registry entry "
-             << (basekey != NULL ? "" : LocalMachineStr) << subkey);
+             << (basekey == HKEY_LOCAL_MACHINE ? LocalMachineStr :
+                (basekey == HKEY_CURRENT_USER ? CurrentUserStr : "")) << subkey);
       key = NULL;
     }
   }
@@ -298,201 +299,121 @@ RegistryKey::~RegistryKey()
 BOOL RegistryKey::EnumKey(PINDEX idx, PString & str)
 {
   if (key == NULL)
-    return PFalse;
+    return false;
 
-#ifndef _WIN32_WCE
-  if( RegEnumKey(key, idx, str.GetPointer(MAX_PATH),MAX_PATH) != ERROR_SUCCESS)
+  TCHAR buffer[_MAX_PATH];
+  if (RegEnumKey(key, idx, buffer, sizeof(buffer)) != ERROR_SUCCESS)
+    return false;
 
-#else // CE has only Unicode based API
-  WCHAR wcsKey[MAX_PATH] = {0};
-
-  LONG lResult = RegEnumKey(key, idx, wcsKey, MAX_PATH);
-  int size = wcslen(wcsKey);
-  if( size )
-  {
-	wcstombs( (char*) str.GetPointer(size), wcsKey, size );
-	str.SetSize(size + 1);
-	str.SetAt(size, 0);
-  }
-
-  if( lResult != ERROR_SUCCESS )
-#endif
-    return PFalse;
-
-  str.MakeMinimumSize();
-  return PTrue;
+  str = buffer;
+  return true;
 }
 
 
 BOOL RegistryKey::EnumValue(PINDEX idx, PString & str)
 {
   if (key == NULL)
-    return PFalse;
+    return false;
 
-  DWORD sizeofname = MAX_PATH;
-#ifndef _WIN32_WCE
-  if (RegEnumValue(key, idx, str.GetPointer(sizeofname),
-                         &sizeofname, NULL, NULL, NULL, NULL) != ERROR_SUCCESS)
+  TCHAR buffer[_MAX_PATH];
+  DWORD size = _MAX_PATH;
+  if (RegEnumValue(key, idx, buffer, &size, NULL, NULL, NULL, NULL) != ERROR_SUCCESS)
+    return false;
 
-#else  // CE has only Unicode based API
-  WCHAR wcsValue[MAX_PATH] = {0};
-
-  LONG lResult = RegEnumValue(key, idx, wcsValue,
-                         &sizeofname, NULL, NULL, NULL, NULL);
-  sizeofname = wcslen(wcsValue);
-  if( sizeofname )
-  {
-	wcstombs( (char*) str.GetPointer(sizeofname), wcsValue, sizeofname );
-	str.SetSize(sizeofname + 1);
-	str.SetAt(sizeofname, 0);
-  }
-
-  if( lResult != ERROR_SUCCESS )
-#endif
-    return PFalse;
-
-  str.MakeMinimumSize();
-  return PTrue;
+  str = buffer;
+  return true;
 }
 
 
 BOOL RegistryKey::DeleteKey(const PString & subkey)
 {
   if (key == NULL)
-    return PTrue;
+    return true;
 
   return RegDeleteKey(key, PVarString(subkey)) == ERROR_SUCCESS;
 }
 
 
-BOOL RegistryKey::DeleteValue(const PString & value)
+BOOL RegistryKey::DeleteValue(const PString & valueName)
 {
   if (key == NULL)
-    return PTrue;
+    return true;
 
-  return RegDeleteValue(key, PVarString(value)) == ERROR_SUCCESS;
+  return RegDeleteValue(key, PVarString(valueName)) == ERROR_SUCCESS;
 }
 
 
-BOOL RegistryKey::QueryValue(const PString & value, PString & str)
+BOOL RegistryKey::QueryValue(const PString & valueName, PString & str)
 {
   if (key == NULL)
-    return PFalse;
+    return false;
 
-  DWORD type, size;
-#ifndef _WIN32_WCE
-  if (RegQueryValueEx(key, (char *)(const char *)value,
-                                    NULL, &type, NULL, &size) != ERROR_SUCCESS)
-#else  
-    // WINCE has only Unicode based API
-    LONG lResult = ERROR_SUCCESS;
-    {   
-	  WCHAR wcsValue[MAX_PATH];
-	  mbstowcs(wcsValue, value, MAX_PATH);
-	  lResult = RegQueryValueEx(key, wcsValue, NULL,
-            &type, NULL, &size );
-    }
-  if( lResult != ERROR_SUCCESS )
-#endif
-    return PFalse;
+  PVarString varValueName = valueName;
 
-  switch (type) {
-    case REG_SZ :
-    case REG_MULTI_SZ :
-    case REG_EXPAND_SZ :
-    case REG_BINARY :
-#ifndef _WIN32_WCE
-      return RegQueryValueEx(key, (char *)(const char *)value, NULL,
-                  &type, (LPBYTE)str.GetPointer(size), &size) == ERROR_SUCCESS;
-#else  
-	{
-		// WINCE strings are Unicode
-		WCHAR wcsValue[MAX_PATH];
-		mbstowcs(wcsValue, value, MAX_PATH);
+  DWORD type, size, num;
+  if (RegQueryValueEx(key, varValueName, NULL, &type, NULL, &size) != ERROR_SUCCESS)
+    return false;
 
-		// Expand receiving buffer to accomodate wchars
-		str.SetSize( str.GetSize() * sizeof (short) );
-		if( RegQueryValueEx(key, wcsValue, NULL,
-			&type, (LPBYTE) str.GetPointer(size), &size ) != ERROR_SUCCESS )
-			return PFalse;
-		else
-		{
-			if( type != REG_BINARY )
-			{
-				// Convert to char* string
-				wcsncpy( wcsValue, (LPCWSTR) str.GetPointer(), MAX_PATH );
-				size = wcslen(wcsValue);
-				if( size )
-				{
-					wcstombs( (char*) str.GetPointer(size), wcsValue, size );
-					str.SetSize(size + 1);
-					str.SetAt(size, 0);
-				}
-			}
-			return PTrue;
-		}
-	}
-#endif // _WIN32_WCE
+  if (type == REG_DWORD) {
+    size = sizeof(num);
+    if (RegQueryValueEx(key, varValueName, NULL, &type, (LPBYTE)&num, &size) != ERROR_SUCCESS)
+      return false;
 
-    case REG_DWORD : {
-      DWORD num;
-      size = sizeof(num);
-#ifndef _WIN32_WCE
-      if (RegQueryValueEx(key, (char *)(const char *)value, NULL,
-                                &type, (LPBYTE)&num, &size) == ERROR_SUCCESS) {
-        str = PString(PString::Signed, num);
-#else
-	  // WINCE strings are Unicode
-	  WCHAR wcsValue[MAX_PATH];
-	  mbstowcs(wcsValue, value, MAX_PATH);
-      if (RegQueryValueEx(key, wcsValue, NULL,
-             &type, (LPBYTE)&num, &size) == ERROR_SUCCESS) {
-
-		str = PString(PString::Signed, num);
-#endif
-        return PTrue;
-      }
-    }
-    default :
-      PAssertAlways("Unsupported registry type.");
+    str = PString(PString::Signed, num);
+    return true;
   }
-  return PFalse;
+
+  if (type != REG_SZ && type != REG_MULTI_SZ && type != REG_EXPAND_SZ && type != REG_BINARY) {
+    PAssertAlways("Unsupported registry type.");
+    return false;
+  }
+
+  if (RegQueryValueEx(key, varValueName, NULL, &type,
+                      (LPBYTE)str.GetPointerAndSetLength(size), &size) != ERROR_SUCCESS)
+    return false;
+
+  if (type == REG_BINARY)
+    return true;
+
+  if (str[size-1] == '\0')
+    str.GetPointerAndSetLength(size-1);
+  return true;
 }
 
 
-BOOL RegistryKey::QueryValue(const PString & value, DWORD & num, BOOL boolean)
+BOOL RegistryKey::QueryValue(const PString & valueName, DWORD & num, BOOL boolean)
 {
   if (key == NULL)
-    return PFalse;
+    return false;
 
   DWORD type, size;
-  PVarString valueName = value;
+  PVarString varValueName = valueName;
 
-  if (RegQueryValueEx(key, valueName, NULL, &type, NULL, &size) != ERROR_SUCCESS)
-    return PFalse;
+  if (RegQueryValueEx(key, varValueName, NULL, &type, NULL, &size) != ERROR_SUCCESS)
+    return false;
 
 
   switch (type) {
     case REG_BINARY :
       if (size > sizeof(DWORD))
-        return PFalse;
+        return false;
 
       num = 0;
       // Do REG_DWORD case
 
     case REG_DWORD :
-      return RegQueryValueEx(key, valueName, NULL, &type, (LPBYTE)&num, &size) == ERROR_SUCCESS;
+      return RegQueryValueEx(key, varValueName, NULL, &type, (LPBYTE)&num, &size) == ERROR_SUCCESS;
 
     case REG_SZ : {
-      PVarString vstr;
-      if (RegQueryValueEx(key, valueName, NULL, &type, (LPBYTE)vstr.GetPointer(size), &size) == ERROR_SUCCESS) {
-        PString str = vstr;
+      TCHAR buffer[100];
+      if (RegQueryValueEx(key, varValueName, NULL, &type, (LPBYTE)buffer, &size) == ERROR_SUCCESS) {
+        PString str = buffer;
         num = str.AsInteger();
         if (num == 0 && boolean) {
           int c = toupper(str[0]);
           num = c == 'T' || c == 'Y';
         }
-        return PTrue;
+        return true;
       }
       break;
     }
@@ -500,25 +421,25 @@ BOOL RegistryKey::QueryValue(const PString & value, DWORD & num, BOOL boolean)
       PAssertAlways("Unsupported registry type.");
   }
 
-  return PFalse;
+  return false;
 }
 
 
-BOOL RegistryKey::SetValue(const PString & value, const PString & str)
+BOOL RegistryKey::SetValue(const PString & valueName, const PString & str)
 {
   if (key == NULL)
-    return PFalse;
+    return false;
 
   PVarString vstr(str);
-  return RegSetValueEx(key, PVarString(value), 0, REG_SZ,
-                       (LPBYTE)(const TCHAR *)vstr, str.GetLength()+1) == ERROR_SUCCESS;
+  return RegSetValueEx(key, PVarString(valueName), 0, REG_SZ,
+                       (LPBYTE)(const TCHAR *)vstr, vstr.GetLength()+1) == ERROR_SUCCESS;
 }
 
 
 BOOL RegistryKey::SetValue(const PString & value, DWORD num)
 {
   if (key == NULL)
-    return PFalse;
+    return false;
 
   return RegSetValueEx(key, PVarString(value), 0, REG_DWORD, (LPBYTE)&num, sizeof(num)) == ERROR_SUCCESS;
 }
@@ -575,9 +496,9 @@ void PConfig::Construct(Source src, const PString & appname, const PString & man
       if (IsRegistryPath(appname))
         location = appname;
       else {
-        PString dir;
-        GetWindowsDirectory(dir.GetPointer(_MAX_PATH), _MAX_PATH);
-        Construct(PDirectory(dir)+"WIN.INI");
+        TCHAR dir[_MAX_PATH];
+        GetWindowsDirectory(dir, sizeof(dir));
+        Construct(PDirectory(PString(dir))+"WIN.INI");
       }
       break;
 
@@ -638,9 +559,9 @@ static void RecurseRegistryKeys(const PString & location,
 
 
 static PString PGetPrivateProfileString(const char * lpAppName,
-                                        const char * lpKeyName,
-                                        const char * lpDefault,
-                                        const char * lpFileName)
+                                           const char * lpKeyName,
+                                           const char * lpDefault,
+                                           const char * lpFileName)
 {
   PString buffer;
 
@@ -648,7 +569,7 @@ static PString PGetPrivateProfileString(const char * lpAppName,
   DWORD size = 100;
   while (size <= 100000 &&
                 ::GetPrivateProfileString(lpAppName, lpKeyName, lpDefault,
-                                          buffer.GetPointer(size+numNulls), size+numNulls,
+                                          buffer.GetPointerAndSetLength(size+numNulls), size+numNulls,
                                           lpFileName) == size)
     size *= 10;
 
@@ -666,13 +587,19 @@ PStringArray PConfig::GetSections() const
       break;
 
     case NumSources :
-      PString buffer = PGetPrivateProfileString(NULL, NULL, "", location);
-      char * ptr = buffer.GetPointer();
-      while (*ptr != '\0') {
-        sections.AppendString(ptr);
-        ptr += strlen(ptr)+1;
+      {
+        PCharArray buffer = PGetPrivateProfileString(NULL, NULL, "", location);
+        char * ptr = buffer.GetPointer();
+        while (*ptr != '\0') {
+          sections.AppendString(ptr);
+          ptr += strlen(ptr)+1;
+        }
       }
       break;
+
+    case Environment :
+    case System :
+      break;
   }
 
   return sections;
@@ -704,12 +631,18 @@ PStringArray PConfig::GetKeys(const PString & section) const
 
     case NumSources :
       PAssert(!section.IsEmpty(), PInvalidParameter);
-      PString buffer = PGetPrivateProfileString(section, NULL, "", location);
-      char * ptr = buffer.GetPointer();
-      while (*ptr != '\0') {
-        keys.AppendString(ptr);
-        ptr += strlen(ptr)+1;
+      {
+        PCharArray buffer = PGetPrivateProfileString(section, NULL, "", location);
+        char * ptr = buffer.GetPointer();
+        while (*ptr != '\0') {
+          keys.AppendString(ptr);
+          ptr += strlen(ptr)+1;
+        }
       }
+      break;
+
+    case System :
+      break;
   }
 
   return keys;
@@ -729,6 +662,11 @@ void PConfig::DeleteSection(const PString & section)
     case NumSources :
       PAssert(!section.IsEmpty(), PInvalidParameter);
       PAssertOS(WritePrivateProfileString(section, NULL, NULL, location));
+      break;
+
+    case Environment :
+    case System :
+      break;
   }
 }
 
@@ -752,6 +690,10 @@ void PConfig::DeleteKey(const PString & section, const PString & key)
       PAssert(!key.IsEmpty(), PInvalidParameter);
       PAssert(!section.IsEmpty(), PInvalidParameter);
       PAssertOS(WritePrivateProfileString(section, key, NULL, location));
+      break;
+
+    case System :
+      break;
   }
 }
 
@@ -771,12 +713,17 @@ PBoolean PConfig::HasKey(const PString & section, const PString & key) const
     }
 
     case NumSources :
-      PAssert(!key.IsEmpty() && !section.IsEmpty(), PInvalidParameter);
-      static const char dflt[] = "<<<<<====---PConfig::DefaultValueString---====>>>>>";
-      return PGetPrivateProfileString(section, key, dflt, location) != dflt;
+      {
+        PAssert(!key.IsEmpty() && !section.IsEmpty(), PInvalidParameter);
+        static const char dflt[] = "<<<<<====---PConfig::DefaultValueString---====>>>>>";
+        return PGetPrivateProfileString(section, key, dflt, location) != dflt;
+      }
+
+    case System :
+      break;
   }
 
-  return PFalse;
+  return false;
 }
 
 
@@ -806,8 +753,12 @@ PString PConfig::GetString(const PString & section,
 
     case NumSources :
       PAssert(!key.IsEmpty() && !section.IsEmpty(), PInvalidParameter);
-      str = PGetPrivateProfileString(section, key, dflt, location);
+      str = PString(PGetPrivateProfileString(section, key, dflt, location));
       str.MakeMinimumSize();
+      break;
+
+    case System :
+      break;
   }
 
   return str;
@@ -833,6 +784,10 @@ void PConfig::SetString(const PString & section,
     case NumSources :
       PAssert(!key.IsEmpty() && !section.IsEmpty(), PInvalidParameter);
       PAssertOS(WritePrivateProfileString(section, key, value, location));
+      break;
+
+    case System :
+      break;
   }
 }
 
@@ -850,7 +805,7 @@ PBoolean PConfig::GetBoolean(const PString & section,
   RegistryKey registry(location + section, RegistryKey::ReadOnly);
 
   DWORD value;
-  if (!registry.QueryValue(key, value, PTrue))
+  if (!registry.QueryValue(key, value, true))
     return dflt;
 
   return value != 0;
@@ -881,7 +836,7 @@ long PConfig::GetInteger(const PString & section,
   RegistryKey registry(location + section, RegistryKey::ReadOnly);
 
   DWORD value;
-  if (!registry.QueryValue(key, value, PFalse))
+  if (!registry.QueryValue(key, value, false))
     return dflt;
 
   return value;
