@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21137 $
- * $Author: ms30002000 $
- * $Date: 2008-09-22 11:57:19 -0500 (Mon, 22 Sep 2008) $
+ * $Revision: 28484 $
+ * $Author: rjongbloed $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 
@@ -56,15 +56,19 @@ static PBoolean PAssertAction(int c, const char * msg)
     case 'T' :
       PError << "\nThrowing exception\n";
       throw std::runtime_error(msg);
-      return PTrue;
+      return true;
 #endif
         
 #ifdef _DEBUG
     case 'd' :
     case 'D' :
       {
-        PString cmd = "gdb " + PProcess::Current().GetFile();
+        PString cmd = ::getenv("PTLIB_ASSERT_DEBUGGER");
+        if (cmd.IsEmpty())
+          cmd = "gdb";
+        cmd &= PProcess::Current().GetFile();
         cmd.sprintf(" %d", getpid());
+        PError << "\nStarting debugger \"" << cmd << "\"\n";
         system((const char *)cmd);
       }
       break;
@@ -73,22 +77,27 @@ static PBoolean PAssertAction(int c, const char * msg)
     case 'c' :
     case 'C' :
       PError << "\nDumping core.\n";
-      kill(getpid(), SIGABRT);
+      raise(SIGABRT);
 
     case 'i' :
     case 'I' :
     case EOF :
       PError << "\nIgnoring.\n";
-      return PTrue;
+      return true;
   }
-  return PFalse;
+  return false;
 }
 #endif
 #endif
 
-void PAssertFunc(const char * msg)
+bool PAssertFunc(const char * msg)
 
 {
+  static PBoolean inAssert;
+  if (inAssert)
+    return false;
+  inAssert = true;
+
 #ifdef P_BEOS
   // Print location in Eddie-compatible format
   PError << msg << endl;
@@ -99,58 +108,58 @@ void PAssertFunc(const char * msg)
   // start the Be Debugger.
   debugger(msg);
 #else
-  static PBoolean inAssert;
-  if (inAssert)
-    return;
-  inAssert = PTrue;
-
 #if PTRACING
-  ostream & trace = PTrace::Begin(0, __FILE__, __LINE__);
-  trace << "PWLib\t" << msg << PTrace::End;
-
-  if (&trace != &PError)
-    PError << msg << endl;
+  if (PTrace::GetStream() != &PError) {
+    ostream & trace = PTrace::Begin(0, __FILE__, __LINE__);
+    trace << "PTLib\t" << msg << PTrace::End;
+  }
 #endif
 
+  PError << msg << endl;
+
   char *env;
 
 #if P_EXCEPTIONS
   //Throw a runtime exception if the environment variable PWLIB_ASSERT_EXCEPTION is set
-  env = ::getenv("PWLIB_ASSERT_EXCEPTION");
+  env = ::getenv("PTLIB_ASSERT_EXCEPTION");
+  if (env == NULL)
+    env = ::getenv("PWLIB_ASSERT_EXCEPTION");
   if (env != NULL){
     throw std::runtime_error(msg);
   }
 #endif
   
 #ifndef P_VXWORKS
-  env = ::getenv("PWLIB_ASSERT_ACTION");
+  env = ::getenv("PTLIB_ASSERT_ACTION");
+  if (env == NULL)
+    env = ::getenv("PWLIB_ASSERT_ACTION");
   if (env != NULL && *env != EOF && PAssertAction(*env, msg)) {
-    inAssert = PFalse;
-    return;
+    inAssert = false;
+    return false;
   }
 
   // Check for if stdin is not a TTY and just ignore the assert if so.
-  if (!isatty(STDIN_FILENO)) {
-    inAssert = PFalse;
-    return;
+  if (isatty(STDIN_FILENO) != 1) {
+    inAssert = false;
+    return false;
   }
 
   for(;;) {
     PError << "\n<A>bort, <C>ore dump"
 #if P_EXCEPTIONS
-           << ", <I>gnore <T>hrow exception"
+           << ", <T>hrow exception"
 #endif
 #ifdef _DEBUG
            << ", <D>ebug"
 #endif
-           << "? " << flush;
+           << ", <I>gnore? " << flush;
 
     int c = getchar();
 
     if (PAssertAction(c, msg))
       break;
    }
-   inAssert = PFalse;
+   inAssert = false;
 
 #else // P_VXWORKS
 
@@ -160,6 +169,8 @@ void PAssertFunc(const char * msg)
 
 #endif // P_VXWORKS
 #endif // P_BEOS
+
+  return false;
 }
 
 // End Of File ///////////////////////////////////////////////////////////////
