@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25444 $
+ * $Revision: 29834 $
  * $Author: rjongbloed $
- * $Date: 2011-03-29 17:17:31 -0500 (Tue, 29 Mar 2011) $
+ * $Date: 2013-05-30 01:12:29 -0500 (Thu, 30 May 2013) $
  */
 
 #define _CONFIG_CXX
@@ -57,48 +57,17 @@
 extern char **environ;
 #endif
 
-//
-//  a single key/value pair
-//
-PDECLARE_CLASS(PXConfigValue, PCaselessString)
-  public:
-    PXConfigValue(const PString & theKey, const PString & theValue = "") 
-      : PCaselessString(theKey), value(theValue) { }
-    PString GetValue() const { return value; }
-    void    SetValue(const PString & theValue) { value = theValue; }
-
-  protected:
-    PString  value;
-};
-
-//
-//  a list of key/value pairs
-//
-PLIST(PXConfigSectionList, PXConfigValue);
-
-//
-//  a list of key value pairs, with a section name
-//
-PDECLARE_CLASS(PXConfigSection, PCaselessString)
-  public:
-    PXConfigSection(const PCaselessString & theName) 
-      : PCaselessString(theName) { list.AllowDeleteObjects(); }
-
-    PXConfigSectionList & GetList() { return list; }
-
-  protected:
-    PXConfigSectionList list;
-};
 
 //
 // a list of sections
 //
 
-class PXConfig : public PList<PXConfigSection>
+class PXConfig : public PDictionary<PCaselessString, PStringToString>
 {
-    PCLASSINFO(PXConfig, PList<PXConfigSection>);
+    typedef PDictionary<PCaselessString, PStringToString> ParentClass;
+    PCLASSINFO(PXConfig, ParentClass);
   public:
-    PXConfig();
+    PXConfig(const PString & key, const PFilePath & filename);
     ~PXConfig();
 
     void Wait()   { mutex.Wait(); }
@@ -108,24 +77,23 @@ class PXConfig : public PList<PXConfigSection>
     void ReadFromEnvironment (char **envp);
 
     PBoolean WriteToFile(const PFilePath & filename);
-    PBoolean Flush(const PFilePath & filename);
+    void Flush();
 
     void SetDirty()
     {
       PTRACE_IF(4, !dirty, "PTLib\tSetting PXConfig dirty.");
-      dirty = PTrue;
+      dirty = true;
     }
 
-    PBoolean      AddInstance();
-    PBoolean      RemoveInstance(const PFilePath & filename);
-
-    PINDEX    GetSectionsIndex(const PString & theSection) const;
-
   protected:
-    int       instanceCount;
-    PMutex    mutex;
-    PBoolean      dirty;
-    PBoolean      canSave;
+    PString        m_key;
+    PFilePath      m_filename;
+    PAtomicInteger instanceCount;
+    PMutex         mutex;
+    bool           dirty;
+    bool           canSave;
+
+  friend class PXConfigDictionary;
 };
 
 
@@ -138,14 +106,14 @@ static PBoolean IsComment(const PString& str)
 //
 // a dictionary of configurations, keyed by filename
 //
-typedef PDictionary<PFilePath, PXConfig> PXConfigDictionaryBase;
-class PXConfigDictionary : public PXConfigDictionaryBase
+class PXConfigDictionary : public PDictionary<PString, PXConfig>
 {
-    PCLASSINFO(PXConfigDictionary, PXConfigDictionaryBase)
+    typedef PDictionary<PString, PXConfig> ParentClass;
+    PCLASSINFO(PXConfigDictionary, ParentClass)
   public:
     PXConfigDictionary();
     ~PXConfigDictionary();
-    PXConfig * GetFileConfigInstance(const PFilePath & key, const PFilePath & readKey);
+    PXConfig * GetFileConfigInstance(const PString & key, const PFilePath & filename);
     PXConfig * GetEnvironmentInstance();
     void RemoveInstance(PXConfig * instance);
     void WriteChangedInstances();
@@ -161,14 +129,13 @@ class PXConfigDictionary : public PXConfigDictionaryBase
 PDECLARE_CLASS(PXConfigWriteThread, PThread)
   public:
     PXConfigWriteThread(PSyncPointAck & stop);
-    ~PXConfigWriteThread();
     void Main();
   private:
     PSyncPointAck & stop;
 };
 
 
-PXConfigDictionary * configDict;
+typedef PSingleton<PXConfigDictionary, PAtomicInteger> PXConfigData;
 
 #define	new PNEW
 
@@ -187,82 +154,54 @@ PXConfigWriteThread::PXConfigWriteThread(PSyncPointAck & s)
   Resume();
 }
 
-PXConfigWriteThread::~PXConfigWriteThread()
-{
-}
 
 void PXConfigWriteThread::Main()
 {
   PTRACE(4, "PTLib\tConfig file cache write back thread started.");
-  while (!stop.Wait(30000))  // if stop.Wait() returns PTrue, we are shutting down
-    configDict->WriteChangedInstances();   // check dictionary for items that need writing
+  while (!stop.Wait(30000))  // if stop.Wait() returns true, we are shutting down
+    PXConfigData()->WriteChangedInstances();   // check dictionary for items that need writing
 
-  configDict->WriteChangedInstances();
+  PXConfigData()->WriteChangedInstances();
 
   stop.Acknowledge();
 }
 
 
+//////////////////////////////////////////////////////
 
-PXConfig::PXConfig()
+PXConfig::PXConfig(const PString & key, const PFilePath & filename)
+  : m_key(key)
+  , m_filename(filename)
 {
   // make sure content gets removed
   AllowDeleteObjects();
 
-  // no instances, initially
-  instanceCount = 0;
-
   // we start off clean
-  dirty = PFalse;
+  dirty = false;
 
   // normally save on exit (except for environment configs)
-  canSave = PTrue;
+  canSave = true;
 
   PTRACE(4, "PTLib\tCreated PXConfig " << this);
 }
 
+
 PXConfig::~PXConfig()
 {
   PTRACE(4, "PTLib\tDestroyed PXConfig " << this);
 }
 
 
-PBoolean PXConfig::AddInstance()
+void PXConfig::Flush()
 {
   mutex.Wait();
-  PBoolean stat = instanceCount++ == 0;
-  mutex.Signal();
-
-  return stat;
-}
-
-PBoolean PXConfig::RemoveInstance(const PFilePath & /*filename*/)
-{
-  mutex.Wait();
-
-  PAssert(instanceCount != 0, "PConfig instance count dec past zero");
-
-  PBoolean stat = --instanceCount == 0;
-
-  mutex.Signal();
-
-  return stat;
-}
-
-PBoolean PXConfig::Flush(const PFilePath & filename)
-{
-  mutex.Wait();
-
-  PBoolean stat = instanceCount == 0;
 
   if (canSave && dirty) {
-    WriteToFile(filename);
-    dirty = PFalse;
+    WriteToFile(m_filename);
+    dirty = false;
   }
 
   mutex.Signal();
-
-  return stat;
 }
 
 PBoolean PXConfig::WriteToFile(const PFilePath & filename)
@@ -274,7 +213,7 @@ PBoolean PXConfig::WriteToFile(const PFilePath & filename)
                                    PFileInfo::UserWrite |
                                    PFileInfo::UserRead)) {
     PProcess::PXShowSystemWarning(2000, "Cannot create PWLIB config directory");
-    return PFalse;
+    return false;
   }
 
   PTextFile file;
@@ -283,28 +222,25 @@ PBoolean PXConfig::WriteToFile(const PFilePath & filename)
 
   if (!file.IsOpen()) {
     PProcess::PXShowSystemWarning(2001, "Cannot create PWLIB config file: " + file.GetErrorText());
-    return PFalse;
+    return false;
   }
 
-  for (PINDEX i = 0; i < GetSize(); i++) {
-    PXConfigSectionList & section = (*this)[i].GetList();
-
+  for (iterator it = begin(); it != end(); ++it) {
     // If the line is a comment, output it as is
-    if (IsComment((*this)[i])) {
-      file << (*this)[i] << endl;
+    if (IsComment(it->first)) {
+      file << it->first << endl;
       continue;
     }
 
-    file << "[" << (*this)[i] << "]" << endl;
-    for (PINDEX j = 0; j < section.GetSize(); j++) {
-      PXConfigValue & value = section[j];
-      PStringArray lines = value.GetValue().Tokenise('\n', PTrue);
+    file << "[" << it->first << "]" << endl;
+    for (PStringToString::iterator it2 = it->second.begin(); it2 != it->second.end(); ++it2) {
+      PStringArray lines = it2->second.Tokenise('\n', true);
       // Preserve name/value pairs with no value, i.e. of the form "name="
       if (lines.IsEmpty())
-	    file << value << "=" << endl;
+	file << it2->first << "=" << endl;
       else {
         for (PINDEX k = 0; k < lines.GetSize(); k++) 
-          file << value << "=" << lines[k] << endl;
+          file << it2->first << "=" << lines[k] << endl;
       }
     }
     file << endl;
@@ -315,14 +251,14 @@ PBoolean PXConfig::WriteToFile(const PFilePath & filename)
   file.Close();
 
   if (file.GetFilePath() != filename) {
-    if (!file.Rename(file.GetFilePath(), filename.GetFileName(), PTrue)) {
+    if (!file.Rename(file.GetFilePath(), filename.GetFileName(), true)) {
       PProcess::PXShowSystemWarning(2001, "Cannot rename config file: " + file.GetErrorText());
-      return PFalse;
+      return false;
     }
   }
 
   PTRACE(4, "PTLib\tSaved config file: " << filename);
-  return PTrue;
+  return true;
 }
 
 
@@ -338,43 +274,38 @@ PBoolean PXConfig::ReadFromFile(const PFilePath & filename)
   // attempt to open file
   PTextFile file;
   if (!file.Open(filename, PFile::ReadOnly))
-    return PFalse;
+    return false;
 
-  PXConfigSection * currentSection = NULL;
+  PStringToString * currentSection = NULL;
 
   // read lines in the file
   while (file.good()) {
     PString line;
     file >> line;
-    line = line.Trim();
+    line = line.LeftTrim();
     if ((len = line.GetLength()) > 0) {
       // Preserve comments
       if (IsComment(line))
-        Append(new PXConfigSection(line));
+        SetAt(line, new PStringToString());
       else {
         if (line[0] == '[') {
-          PCaselessString sectionName = (line.Mid(1,len-(line[len-1]==']'?2:1))).Trim();
-          PINDEX  index;
-          if ((index = GetValuesIndex(sectionName)) != P_MAX_INDEX)
-            currentSection = &(*this )[index];
+          PCaselessString sectionName = line(1, line.Find(']')-1).Trim();
+          iterator iter;
+          if ((iter = find(sectionName)) != end())
+            currentSection = &iter->second;
           else {
-            currentSection = new PXConfigSection(sectionName);
-            Append(currentSection);
+            currentSection = new PStringToString();
+            SetAt(sectionName, currentSection);
           }
-        } else if (currentSection != NULL) {
-          PINDEX equals = line.Find('=');
-          if (equals > 0 && equals != P_MAX_INDEX) {
-            PString keyStr = line.Left(equals).Trim();
-            PString valStr = line.Right(len - equals - 1).Trim();
-
-            PINDEX index;
-            if ((index = currentSection->GetList().GetValuesIndex(keyStr)) != P_MAX_INDEX)  {
-              PXConfigValue & value = currentSection->GetList()[index];
-              value.SetValue(value.GetValue() + '\n' + valStr);
-            } else {
-              PXConfigValue * value = new PXConfigValue(keyStr, valStr);
-              currentSection->GetList().Append(value);
-            }
+        }
+        else if (currentSection != NULL) {
+          PString keyStr, valStr;
+          if (line.Split('=', keyStr, valStr, false) && !(keyStr = keyStr.Trim()).IsEmpty()) {
+            PStringToString::iterator iter = currentSection->find(keyStr);
+            if (iter != currentSection->end())
+              iter->second += '\n' + valStr;
+            else
+              currentSection->SetAt(keyStr, valStr);
           }
         }
       }
@@ -383,7 +314,7 @@ PBoolean PXConfig::ReadFromFile(const PFilePath & filename)
   
   // close the file and return
   file.Close();
-  return PTrue;
+  return true;
 }
 
 void PXConfig::ReadFromEnvironment (char **envp)
@@ -391,11 +322,11 @@ void PXConfig::ReadFromEnvironment (char **envp)
   // clear out all information
   RemoveAll();
 
-  PXConfigSection * currentSection = new PXConfigSection("Options");
-  Append(currentSection);
+  PStringToString * currentSection = new PStringToString();
+  SetAt("Options", currentSection);
 
   // can't save environment configs
-  canSave = PFalse;
+  canSave = false;
 
   if (envp == NULL)
     return;
@@ -403,23 +334,12 @@ void PXConfig::ReadFromEnvironment (char **envp)
   while (*envp != NULL && **envp != '\0') {
     PString line(*envp);
     PINDEX equals = line.Find('=');
-    if (equals > 0) {
-      PXConfigValue * value = new PXConfigValue(line.Left(equals), line.Right(line.GetLength() - equals - 1));
-      currentSection->GetList().Append(value);
-    }
+    if (equals > 0)
+      currentSection->SetAt(line.Left(equals), line.Mid(equals+1));
     envp++;
   }
 }
 
-PINDEX PXConfig::GetSectionsIndex(const PString & theSection) const
-{
-  PINDEX len = theSection.GetLength()-1;
-  if (theSection[len] != '\\')
-    return GetValuesIndex(theSection);
-  else
-    return GetValuesIndex(theSection.Left(len));
-}
-
 
 static PBoolean LocateFile(const PString & baseName,
                        PFilePath & readFilename,
@@ -428,7 +348,7 @@ static PBoolean LocateFile(const PString & baseName,
   // check the user's home directory first
   filename = readFilename = PProcess::Current().GetConfigurationFile();
   if (PFile::Exists(filename))
-    return PTrue;
+    return true;
 
   // otherwise check the system directory for a file to read,
   // and then create 
@@ -441,7 +361,7 @@ static PBoolean LocateFile(const PString & baseName,
 PString PProcess::GetConfigurationFile()
 {
   if (configurationPaths.IsEmpty()) {
-    configurationPaths.AppendString(PXGetHomeDir() + APP_CONFIG_DIR);
+    configurationPaths.AppendString(GetHomeDirectory() + APP_CONFIG_DIR);
     configurationPaths.AppendString(SYS_CONFIG_DIR);
   }
 
@@ -470,7 +390,6 @@ PXConfigDictionary::PXConfigDictionary()
 {
   environmentInstance = NULL;
   writeThread = NULL;
-  configDict = this;
 }
 
 
@@ -489,7 +408,7 @@ PXConfig * PXConfigDictionary::GetEnvironmentInstance()
 {
   mutex.Wait();
   if (environmentInstance == NULL) {
-    environmentInstance = new PXConfig;
+    environmentInstance = new PXConfig(PString::Empty(), PString::Empty());
     environmentInstance->ReadFromEnvironment(environ);
   }
   mutex.Signal();
@@ -497,7 +416,7 @@ PXConfig * PXConfigDictionary::GetEnvironmentInstance()
 }
 
 
-PXConfig * PXConfigDictionary::GetFileConfigInstance(const PFilePath & key, const PFilePath & readKey)
+PXConfig * PXConfigDictionary::GetFileConfigInstance(const PString & key, const PFilePath & filename)
 {
   mutex.Wait();
 
@@ -506,14 +425,12 @@ PXConfig * PXConfigDictionary::GetFileConfigInstance(const PFilePath & key, cons
     writeThread = new PXConfigWriteThread(stopConfigWriteThread);
 
   PXConfig * config = GetAt(key);
-  if (config != NULL) 
-    config->AddInstance();
-  else {
-    config = new PXConfig;
-    config->ReadFromFile(readKey);
-    config->AddInstance();
+  if (config == NULL) {
+    config = new PXConfig(key, filename);
+    config->ReadFromFile(filename);
     SetAt(key, config);
   }
+  ++config->instanceCount;
 
   mutex.Signal();
   return config;
@@ -524,14 +441,10 @@ void PXConfigDictionary::RemoveInstance(PXConfig * instance)
   mutex.Wait();
 
   if (instance != environmentInstance) {
-    PINDEX index = GetObjectsIndex(instance);
-    PAssert(index != P_MAX_INDEX, "Cannot find PXConfig instance to remove");
-
-    // decrement the instance count and remove it if this was the last instance
-    PFilePath key = GetKeyAt(index);
-    if (instance->RemoveInstance(key)) {
-      instance->Flush(key);
-      RemoveAt(key);
+    iterator it = find(instance->m_key);
+    if (it != end() && --instance->instanceCount == 0) {
+      instance->Flush();
+      erase(it);
     }
   }
 
@@ -542,11 +455,8 @@ void PXConfigDictionary::WriteChangedInstances()
 {
   mutex.Wait();
 
-  PINDEX i;
-  for (i = 0; i < GetSize(); i++) {
-    PFilePath key = GetKeyAt(i);
-    GetAt(key)->Flush(key);
-  }
+  for (iterator it = begin(); it != end(); ++it)
+    it->second.Flush();
 
   mutex.Signal();
 }
@@ -565,7 +475,7 @@ void PConfig::Construct(Source src,
 {
   // handle cnvironment configs differently
   if (src == PConfig::Environment)  {
-    config = configDict->GetEnvironmentInstance();
+    config = PXConfigData()->GetEnvironmentInstance();
     return;
   }
   
@@ -579,7 +489,7 @@ void PConfig::Construct(Source src,
     filename = readFilename = PProcess::Current().GetConfigurationFile();
 
   // get, or create, the configuration
-  config = configDict->GetFileConfigInstance(filename, readFilename);
+  config = PXConfigData()->GetFileConfigInstance(filename, readFilename);
 }
 
 PConfig::PConfig(int, const PString & name)
@@ -587,19 +497,19 @@ PConfig::PConfig(int, const PString & name)
 {
   PFilePath readFilename, filename;
   LocateFile(name, readFilename, filename);
-  config = configDict->GetFileConfigInstance(filename, readFilename);
+  config = PXConfigData()->GetFileConfigInstance(filename, readFilename);
 }
 
 
 void PConfig::Construct(const PFilePath & theFilename)
 {
-  config = configDict->GetFileConfigInstance(theFilename, theFilename);
+  config = PXConfigData()->GetFileConfigInstance(theFilename, theFilename);
 }
 
 
 PConfig::~PConfig()
 {
-  configDict->RemoveInstance(config);
+  PXConfigData()->RemoveInstance(config);
 }
 
 
@@ -619,8 +529,9 @@ PStringArray PConfig::GetSections() const
   PINDEX sz = config->GetSize();
   PStringArray sections(sz);
 
-  for (PINDEX i = 0; i < sz; i++)
-    sections[i] = (*config)[i];
+  PINDEX index = 0;
+  for (PXConfig::iterator it = config->begin(); it != config->end(); ++it)
+    sections[index++] = it->first;
 
   config->Signal();
 
@@ -642,14 +553,15 @@ PStringArray PConfig::GetKeys(const PString & theSection) const
   PAssert(config != NULL, "config instance not set");
   config->Wait();
 
-  PINDEX index;
   PStringArray keys;
 
-  if ((index = config->GetSectionsIndex(theSection)) != P_MAX_INDEX) {
-    PXConfigSectionList & section = (*config)[index].GetList();
+  PXConfig::iterator it;
+  if ((it = config->find(theSection)) != config->end()) {
+    PStringToString & section = it->second;
     keys.SetSize(section.GetSize());
-    for (PINDEX i = 0; i < section.GetSize(); i++)
-      keys[i] = section[i];
+    PINDEX index = 0;
+    for (PStringToString::iterator it2 = section.begin(); it2 != section.end(); ++it2)
+      keys[index++] = it2->first;
   }
 
   config->Signal();
@@ -672,9 +584,9 @@ void PConfig::DeleteSection(const PString & theSection)
   PAssert(config != NULL, "config instance not set");
   config->Wait();
 
-  PINDEX index;
-  if ((index = config->GetSectionsIndex(theSection)) != P_MAX_INDEX) {
-    config->RemoveAt(index);
+  PXConfig::iterator it;
+  if ((it = config->find(theSection)) != config->end()) {
+    config->RemoveAt(it->first);
     config->SetDirty();
   }
 
@@ -695,12 +607,12 @@ void PConfig::DeleteKey(const PString & theSection, const PString & theKey)
   PAssert(config != NULL, "config instance not set");
   config->Wait();
 
-  PINDEX index;
-  if ((index = config->GetSectionsIndex(theSection)) != P_MAX_INDEX) {
-    PXConfigSectionList & section = (*config)[index].GetList();
-    PINDEX index_2;
-    if ((index_2 = section.GetValuesIndex(theKey)) != P_MAX_INDEX) {
-      section.RemoveAt(index_2);
+  PXConfig::iterator it;
+  if ((it = config->find(theSection)) != config->end()) {
+    PStringToString & section = it->second;
+    PStringToString::iterator it2;
+    if ((it2 = section.find(theKey)) != section.end()) {
+      section.RemoveAt(it2->first);
       config->SetDirty();
     }
   }
@@ -723,12 +635,10 @@ PBoolean PConfig::HasKey(const PString & theSection, const PString & theKey) con
   PAssert(config != NULL, "config instance not set");
   config->Wait();
 
-  PBoolean present = PFalse;
-  PINDEX index;
-  if ((index = config->GetSectionsIndex(theSection)) != P_MAX_INDEX) {
-    PXConfigSectionList & section = (*config)[index].GetList();
-    present = section.GetValuesIndex(theKey) != P_MAX_INDEX;
-  }
+  bool present = false;
+  PXConfig::iterator it;
+  if ((it = config->find(theSection)) != config->end())
+    present = it->second.Contains(theKey);
 
   config->Signal();
   return present;
@@ -744,19 +654,18 @@ PBoolean PConfig::HasKey(const PString & theSection, const PString & theKey) con
 //
 ////////////////////////////////////////////////////////////
 
-PString PConfig::GetString(const PString & theSection,
-                                    const PString & theKey, const PString & dflt) const
+PString PConfig::GetString(const PString & theSection, const PString & theKey, const PString & dflt) const
 {
   PAssert(config != NULL, "config instance not set");
   config->Wait();
 
   PString value = dflt;
-  PINDEX index;
-  if ((index = config->GetSectionsIndex(theSection)) != P_MAX_INDEX) {
-
-    PXConfigSectionList & section = (*config)[index].GetList();
-    if ((index = section.GetValuesIndex(theKey)) != P_MAX_INDEX) 
-      value = section[index].GetValue();
+  PXConfig::iterator it;
+  if ((it = config->find(theSection)) != config->end()) {
+    PStringToString & section = it->second;
+    PStringToString::iterator it2;
+    if ((it2 = section.find(theKey)) != section.end()) 
+      value = it2->second;
   }
 
   config->Signal();
@@ -779,28 +688,20 @@ void PConfig::SetString(const PString & theSection,
   PAssert(config != NULL, "config instance not set");
   config->Wait();
 
-  PINDEX index;
-  PXConfigSection * section;
-  PXConfigValue   * value;
+  PStringToString * section;
 
-  if ((index = config->GetSectionsIndex(theSection)) != P_MAX_INDEX) 
-    section = &(*config)[index];
+  PXConfig::iterator it;
+  if ((it = config->find(theSection)) != config->end())
+    section = &it->second;
   else {
-    section = new PXConfigSection(theSection);
-    config->Append(section);
+    section = new PStringToString;
+    config->SetAt(theSection, section);
     config->SetDirty();
   } 
 
-  if ((index = section->GetList().GetValuesIndex(theKey)) != P_MAX_INDEX) 
-    value = &(section->GetList()[index]);
-  else {
-    value = new PXConfigValue(theKey);
-    section->GetList().Append(value);
-    config->SetDirty();
-  }
-
-  if (theValue != value->GetValue()) {
-    value->SetValue(theValue);
+  PStringToString::iterator it2;
+  if ((it2 = section->find(theKey)) == section->end() || it2->second != theValue) {
+    section->SetAt(theKey, theValue);
     config->SetDirty();
   }
 
