@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <ptlib.h>
@@ -62,7 +62,7 @@
 #endif
 #endif
 
-#ifdef __BEOS__
+#ifdef P_BEOS
 #define	SET_STACK	context[0].__jmpbuf[JB_SP] = (int)stackTop-16;
 #endif
 
@@ -131,7 +131,7 @@ void PThread::SwitchContext(PThread * from)
   }
 
   //  switch to the new thread
-  longjmp(context, PTrue);
+  longjmp(context, true);
   PAssertAlways("Return from longjmp not allowed");
 
 #else
