@@ -26,14 +26,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23336 $
+ * $Revision: 29919 $
  * $Author: rjongbloed $
- * $Date: 2009-09-05 01:02:14 +0000 (Sat, 05 Sep 2009) $
+ * $Date: 2013-06-11 22:34:15 -0500 (Tue, 11 Jun 2013) $
  */
 
 #define _OSUTIL_CXX
 
-//#define SIGNALS_DEBUG
+#define SIGNALS_DEBUG(fmt,...) //fprintf(stderr, fmt, ##__VA_ARGS__)
+
 
 #pragma implementation "args.h"
 #pragma implementation "pprocess.h"
@@ -64,13 +65,13 @@
 #include <sys/wait.h>
 #include <errno.h>
 
-#if defined(P_LINUX)
+#if defined(P_LINUX) || defined(P_GNU_HURD)
 #include <sys/cdefs.h>
 #include <sys/types.h>
 #include <sys/mman.h>
 #endif
 
-#if defined(P_LINUX) || defined(P_SUN4) || defined(P_SOLARIS) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined (P_AIX) || defined(P_BEOS) || defined(P_IRIX) || defined(P_QNX)
+#if defined(P_LINUX) || defined(P_SUN4) || defined(P_SOLARIS) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined (P_AIX) || defined(P_BEOS) || defined(P_IRIX) || defined(P_QNX) || defined(P_GNU_HURD)
 #include <sys/utsname.h>
 #define  HAS_UNAME
 #elif defined(P_RTEMS)
@@ -101,8 +102,6 @@ extern "C" int select(int width,
 #include "OS.h"
 #endif
 
-#include "../common/pglobalstatic.cxx"
-
 
 PString PProcess::GetOSClass()
 {
@@ -165,6 +164,38 @@ PString PProcess::GetOSVersion()
 #endif
 }
 
+bool PProcess::IsOSVersion(unsigned major, unsigned minor, unsigned build)
+{
+#if defined(HAS_UNAME)
+  struct utsname info;
+  uname(&info);
+  unsigned maj, min, bld;
+  sscanf(
+#ifdef P_SOLARIS
+         info.version
+#else
+         info.release
+#endif
+         , "%u.%u.%u", &maj, &min, &bld);
+  if (maj < major)
+    return false;
+  if (maj > major)
+    return true;
+
+  if (min < minor)
+    return false;
+  if (min > minor)
+    return true;
+
+  return bld >= build;
+#elif defined(P_VXWORKS)
+  return sysBspRev() >= major;
+#else
+  return true;
+#endif
+}
+
+
 PDirectory PProcess::GetOSConfigDir()
 {
 #ifdef P_VXWORKS
@@ -174,23 +205,18 @@ PDirectory PProcess::GetOSConfigDir()
 #endif // P_VXWORKS
 }
 
-PDirectory PProcess::PXGetHomeDir ()
 
+PString PProcess::GetUserName() const
 {
-#ifdef P_VXWORKS
-  return "./";
-#else
-  PString dest;
-  char *ptr;
-  struct passwd *pw = NULL;
+#ifndef P_VXWORKS
+
+  struct passwd * pw = NULL;
 
 #if defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)
   struct passwd pwd;
   char buffer[1024];
-#if defined (P_LINUX) || defined(P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS)
-  ::getpwuid_r(geteuid(), &pwd,
-               buffer, 1024,
-               &pw);
+#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS) || defined(P_GNU_HURD)
+  ::getpwuid_r(geteuid(), &pwd, buffer, 1024, &pw);
 #else
   pw = ::getpwuid_r(geteuid(), &pwd, buffer, 1024);
 #endif
@@ -198,63 +224,16 @@ PDirectory PProcess::PXGetHomeDir ()
   pw = ::getpwuid(geteuid());
 #endif
 
-  if (pw != NULL && pw->pw_dir != NULL) 
-    dest = pw->pw_dir;
-  else if ((ptr = getenv ("HOME")) != NULL) 
-    dest = ptr;
-  else 
-    dest = ".";
-
-  if (dest.GetLength() > 0 && dest[dest.GetLength()-1] != '/')
-    dest += "/";
-
-  return dest;
-#endif
-}
-
-///////////////////////////////////////////////////////////////////////////////
-//
-// PProcess
-//
-// Return the effective user name of the process, eg "root" etc.
-
-PString PProcess::GetUserName() const
-
-{
-#ifdef P_VXWORKS
-
-  char pnamebuf[1024];
-  int len = 1024;
-  STATUS gethostresult;
-  gethostresult =::gethostname(pnamebuf,len);  
-  if (gethostresult == OK)
-    return PString(pnamebuf,len);
-  else
-    return PString("VxWorks");
-
-#else
-
-#if defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)
-  struct passwd pwd;
-  char buffer[1024];
-  struct passwd * pw = NULL;
-#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS)
-  ::getpwuid_r(geteuid(), &pwd, buffer, 1024, &pw);
-#else
-  pw = ::getpwuid_r(geteuid(), &pwd, buffer, 1024);
-#endif
-#else
-  struct passwd * pw = ::getpwuid(geteuid());
-#endif
-
-  char * ptr;
   if (pw != NULL && pw->pw_name != NULL)
-    return PString(pw->pw_name);
-  else if ((ptr = getenv("USER")) != NULL)
-    return PString(ptr);
-  else
-    return PString("user");
+    return pw->pw_name;
+
+  const char * user;
+  if ((user = getenv("USER")) != NULL)
+    return user;
+
 #endif // P_VXWORKS
+
+  return GetName();
 }
 
 
@@ -262,7 +241,7 @@ PBoolean PProcess::SetUserName(const PString & username, PBoolean permanent)
 {
 #ifdef P_VXWORKS
   PAssertAlways("PProcess::SetUserName - not implemented for VxWorks");
-  return PFalse;
+  return false;
 #else
   if (username.IsEmpty())
     return seteuid(getuid()) != -1;
@@ -279,7 +258,7 @@ PBoolean PProcess::SetUserName(const PString & username, PBoolean permanent)
     struct passwd pwd;
     char buffer[1024];
     struct passwd * pw = NULL;
-#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS)
+#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS) || defined(P_GNU_HURD)
     ::getpwnam_r(username, &pwd, buffer, 1024, &pw);
 #else
     pw = ::getpwnam_r(username, &pwd, buffer, 1024);
@@ -297,7 +276,7 @@ PBoolean PProcess::SetUserName(const PString & username, PBoolean permanent)
   }
 
   if (uid < 0)
-    return PFalse;
+    return false;
 
   if (permanent)
     return setuid(uid) != -1;
@@ -307,6 +286,37 @@ PBoolean PProcess::SetUserName(const PString & username, PBoolean permanent)
 }
 
 
+PDirectory PProcess::GetHomeDirectory() const
+{
+#ifndef P_VXWORKS
+
+  const char * home = getenv("HOME");
+  if (home != NULL)
+    return home;
+
+  struct passwd * pw = NULL;
+
+#if defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB)
+  struct passwd pwd;
+  char buffer[1024];
+#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS) || defined(P_GNU_HURD)
+  ::getpwuid_r(geteuid(), &pwd, buffer, 1024, &pw);
+#else
+  pw = ::getpwuid_r(geteuid(), &pwd, buffer, 1024);
+#endif
+#else
+  pw = ::getpwuid(geteuid());
+#endif
+
+  if (pw != NULL && pw->pw_dir != NULL)
+    return pw->pw_dir;
+
+#endif // P_VXWORKS
+
+  return ".";
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // PProcess
@@ -326,7 +336,7 @@ PString PProcess::GetGroupName() const
   struct group grp;
   char buffer[1024];
   struct group * gr = NULL;
-#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS)
+#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS) || defined(P_GNU_HURD)
   ::getgrgid_r(getegid(), &grp, buffer, 1024, &gr);
 #else
   gr = ::getgrgid_r(getegid(), &grp, buffer, 1024);
@@ -350,7 +360,7 @@ PBoolean PProcess::SetGroupName(const PString & groupname, PBoolean permanent)
 {
 #ifdef P_VXWORKS
   PAssertAlways("PProcess::SetGroupName - not implemented for VxWorks");
-  return PFalse;
+  return false;
 #else
   if (groupname.IsEmpty())
     return setegid(getgid()) != -1;
@@ -367,7 +377,7 @@ PBoolean PProcess::SetGroupName(const PString & groupname, PBoolean permanent)
     struct group grp;
     char buffer[1024];
     struct group * gr = NULL;
-#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS)
+#if defined (P_LINUX) || defined (P_AIX) || defined(P_IRIX) || (__GNUC__>=3 && defined(P_SOLARIS)) || defined(P_RTEMS) || defined(P_GNU_HURD)
     ::getgrnam_r(groupname, &grp, buffer, 1024, &gr);
 #else
     gr = ::getgrnam_r(groupname, &grp, buffer, 1024);
@@ -385,7 +395,7 @@ PBoolean PProcess::SetGroupName(const PString & groupname, PBoolean permanent)
   }
 
   if (gid < 0)
-    return PFalse;
+    return false;
 
   if (permanent)
     return setgid(gid) != -1;
@@ -394,18 +404,42 @@ PBoolean PProcess::SetGroupName(const PString & groupname, PBoolean permanent)
 #endif // P_VXWORKS
 }
 
-extern PProcess * PProcessInstance;;
 
-PString PX_GetThreadName(pthread_t id)
+void PProcess::HostSystemURLHandlerInfo::SetIcon(const PString & _icon)
 {
-  if (PProcessInstance != NULL) {
-    PWaitAndSignal m(PProcessInstance->activeThreadMutex);
-    PThread & thread = PProcessInstance->activeThreads[_hptr(id)];
-    return thread.GetThreadName();
-  }
-  return psprintf("%08x", id);
 }
 
+PString PProcess::HostSystemURLHandlerInfo::GetIcon() const 
+{
+  return PString();
+}
+
+void PProcess::HostSystemURLHandlerInfo::SetCommand(const PString & key, const PString & _cmd)
+{
+}
+
+PString PProcess::HostSystemURLHandlerInfo::GetCommand(const PString & key) const
+{
+  return PString();
+}
+
+bool PProcess::HostSystemURLHandlerInfo::GetFromSystem()
+{
+  return false;
+}
+
+bool PProcess::HostSystemURLHandlerInfo::CheckIfRegistered()
+{
+  return false;
+}
+
+bool PProcess::HostSystemURLHandlerInfo::Register()
+{
+  return false;
+}
+
+
+
 void PProcess::PXShowSystemWarning(PINDEX num)
 {
   PXShowSystemWarning(num, "");
@@ -423,35 +457,22 @@ void PProcess::_PXShowSystemWarning(PINDEX code, const PString & str)
 
 void PXSignalHandler(int sig)
 {
-#ifdef SIGNALS_DEBUG
-  fprintf(stderr,"\nSIGNAL<%u>\n",sig);
-#endif
-
-  PProcess & process = PProcess::Current();
-  process.pxSignals |= 1 << sig;
-  process.PXOnAsyncSignal(sig);
-#if defined(P_MAC_MPTHREADS)
-  process.SignalTimerChange();
-#elif defined(P_PTHREADS)
-  // Inform house keeping thread we have a signal to be processed
-  process.SignalTimerChange();
-#endif
+  SIGNALS_DEBUG("\nSIGNAL<%u>\n",sig);
+  PProcess::Current().PXOnAsyncSignal(sig);
   signal(sig, PXSignalHandler);
 }
 
 void PProcess::PXCheckSignals()
 {
-  if (pxSignals == 0)
+  if (m_pxSignals == 0)
     return;
 
-#ifdef SIGNALS_DEBUG
-  fprintf(stderr,"\nCHKSIG<%x>\n",pxSignals);
-#endif
+  PTRACE(3, "PTLib", "Checking signals: 0x" << hex << m_pxSignals << dec);
 
-  for (int sig = 0; sig < 32; sig++) {
+  for (int sig = 0; sig < 32; ++sig) {
     int bit = 1 << sig;
-    if ((pxSignals&bit) != 0) {
-      pxSignals &= ~bit;
+    if (m_pxSignals&bit) {
+      m_pxSignals &= ~bit;
       PXOnSignal(sig);
     }
   }
@@ -460,9 +481,7 @@ void PProcess::PXCheckSignals()
 
 void SetSignals(void (*handler)(int))
 {
-#ifdef SIGNALS_DEBUG
-  fprintf(stderr,"\nSETSIG<%x>\n",(INT)handler);
-#endif
+  SIGNALS_DEBUG("\nSETSIG<%p>\n",handler);
 
   if (handler == NULL)
     handler = SIG_DFL;
@@ -496,66 +515,60 @@ void SetSignals(void (*handler)(int))
 
 void PProcess::PXOnAsyncSignal(int sig)
 {
-#ifdef SIGNALS_DEBUG
-  fprintf(stderr,"\nASYNCSIG<%u>\n",sig);
-#endif
+  SIGNALS_DEBUG("\nASYNCSIG<%u>\n",sig);
 
   switch (sig) {
     case SIGINT:
+    case SIGHUP:
     case SIGTERM:
       if (OnInterrupt(sig == SIGTERM))
-        break;
-      // Do next case
-
-    case SIGHUP:
-      raise(SIGKILL);
-      break;
-    default:
-      return;
+        return;
   }
+
+  m_pxSignals |= 1 << sig;
+  SignalTimerChange(); // Inform house keeping thread we have a signal to be processed
 }
 
 void PProcess::PXOnSignal(int sig)
 {
+  SIGNALS_DEBUG("\nSYNCSIG<%u>\n",sig);
+  PTRACE(3, "PTLib", "Handling signal " << sig);
+
+  switch (sig) {
+    case SIGINT:
+    case SIGHUP:
+    case SIGTERM:
+      raise(SIGKILL);
+
 #ifdef _DEBUG
-#ifdef SIGNALS_DEBUG
-  fprintf(stderr,"\nSYNCSIG<%u>\n",sig);
-#endif
-  if (sig == 28) {
-#if PMEMORY_CHECK
-    PBoolean oldIgnore = PMemoryHeap::SetIgnoreAllocations(PTrue);
-    static PMemoryHeap::State state;
-    PMemoryHeap::GetState(state);
-#endif
-    PStringStream strm;
-    activeThreadMutex.Wait();
-    PINDEX i;
-    strm << "===============\n";
-    strm << activeThreads.GetSize() << " active threads\n";
-    for (i = 0; i < activeThreads.GetSize(); ++i) {
-      POrdinalKey key = activeThreads.GetKeyAt(i);
-      PThread & thread = activeThreads[key];
-      strm << "  " << thread << "\n";
-    }
-#if PMEMORY_CHECK
-    strm << "---------------\n";
-    PMemoryHeap::DumpObjectsSince(state, strm);
-    PMemoryHeap::GetState(state);
-#endif
-    strm << "===============\n";
-    activeThreadMutex.Signal();
-    fprintf(stderr, "%s", strm.GetPointer());
-#if PMEMORY_CHECK
-    PMemoryHeap::SetIgnoreAllocations(oldIgnore);
-#endif
+    case 28 :
+      #if PMEMORY_CHECK
+        PMEMORY_IGNORE_ALLOCATIONS_FOR_SCOPE;
+        static PMemoryHeap::State state;
+        PMemoryHeap::GetState(state);
+      #endif // PMEMORY_CHECK
+      PStringStream strm;
+      m_threadMutex.Wait();
+      strm << "===============\n"
+           << m_activeThreads.size() << " active threads\n";
+      for (ThreadMap::iterator it = m_activeThreads.begin(); it != m_activeThreads.end(); ++it)
+        strm << "  " << *it->second << "\n";
+      #if PMEMORY_CHECK
+        strm << "---------------\n";
+        PMemoryHeap::DumpObjectsSince(state, strm);
+        PMemoryHeap::GetState(state);
+      #endif // PMEMORY_CHECK
+      strm << "===============\n";
+      m_threadMutex.Signal();
+      fprintf(stderr, "%s", (const char *)strm);
+#endif // _DEBUG
   }
-#endif
 }
 
 void PProcess::CommonConstruct()
 {
   // Setup signal handlers
-  pxSignals = 0;
+  m_pxSignals = 0;
 
   if (!m_library)
     SetSignals(&PXSignalHandler);
@@ -579,6 +592,8 @@ void PProcess::CommonDestruct()
 
   if (!m_library)
     SetSignals(NULL);
+
+  m_keepingHouse = false;
 }
 
 // rtems fixes
@@ -774,12 +789,19 @@ void* POSIX_Init(void*);
 //  Non-PTHREAD based routines
 //
 
+#if defined(P_PTHREADS)
+
+#include "tlibthrd.cxx"
+
+#else
+
 #if defined(P_MAC_MPTHREADS)
 #include "tlibmpthrd.cxx"
-#elif defined(P_PTHREADS)
-#include "tlibthrd.cxx"
 #elif defined(BE_THREADS)
 #include "tlibbe.cxx"
 #elif defined(VX_TASKS)
 #include "tlibvx.cxx"
 #endif
+
+#endif // P_PTHREADS
+
