@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21219 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-10-02 03:52:00 -0500 (Thu, 02 Oct 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 #include <sys/resource.h>
@@ -39,19 +39,6 @@
 extern int debug_mpthreads;
 #endif
 
-PDECLARE_CLASS(PHouseKeepingThread, PThread)
-  public:
-    PHouseKeepingThread()
-      : PThread(1000, NoAutoDeleteThread, NormalPriority, "Housekeeper")
-      { closing = PFalse; Resume(); }
-
-    void Main();
-    void SetClosing() { closing = PTrue; }
-
-  protected:
-    PBoolean closing;
-};
-
 
 #define new PNEW
 
@@ -215,13 +202,11 @@ static PBoolean PollNotificationQueue(Duration timeout)
     return err == noErr;
 }
 
-void PHouseKeepingThread::Main()
+void PProcess::HouseKeeping()
 {
-    PProcess & process = PProcess::Current();
-
     SetUpTermQueue();
 
-    while (!closing) {
+    while (m_keepingHouse) {
         PTimeInterval waitTime = process.timers.Process();
 
         Duration timeout;
@@ -262,36 +247,30 @@ void PProcess::Construct()
 
   SetUpTermQueue();
 
-  // initialise the housekeeping thread
-  housekeepingThread = NULL;
-
   CommonConstruct();
 }
 
 
 PProcess::~PProcess()
 {
-  // Don't wait for housekeeper to stop if Terminate() is called from it.
-  if (housekeepingThread != NULL && PThread::Current() != housekeepingThread) {
-    housekeepingThread->SetClosing();
-    SignalTimerChange();
-    housekeepingThread->WaitForTermination();
-    delete housekeepingThread;
-    housekeepingThread = 0;
-  }
+  PreShutdown();
+
   // XXX try to gracefully handle shutdown transient where the housekeeping
   // XXX thread hasn't managed to clean up all the threads
   while (PollNotificationQueue(kDurationImmediate)) ;
   
   CommonDestruct();
+
+  PostShutdown();
 }
 
 
-PThread::PThread()
+PThread::PThread(bool isProcess)
+  : m_isProcess(isProcess)
+  , m_autoDelete(!isProcess)
 {
   OSStatus err        = 0;
   PX_origStackSize    = 0;
-  autoDelete          = PFalse;
   PX_threadId         = MPCurrentTaskID();
   PX_suspendCount     = 0;
 
@@ -307,18 +286,10 @@ PThread::PThread()
       throw std::bad_alloc();
   }
 
-  if (!PProcess::IsInitialised())
+  if (isProcess)
     return;
 
-  autoDelete = true;
-
-  PProcess & process = PProcess::Current();
-
-  process.activeThreadMutex.Wait();
-  process.activeThreads.SetAt(PX_threadId, this);
-  process.activeThreadMutex.Signal();
-
-  process.SignalTimerChange();
+  PProcess::Current().InternalThreadStarted(this);
 }
 
 
@@ -347,15 +318,12 @@ PThread::PThread(PINDEX stackSize,
   ::pipe(unblockPipe);
 
   // throw the new thread
-  PX_NewThread(PTrue);
+  PX_NewThread(true);
 }
 
 
-PThread::~PThread()
+void PThread::InternalDestroy()
 {
-  if (!IsTerminated()) 
-    Terminate();
-
   ::close(unblockPipe[0]);
   ::close(unblockPipe[1]);
 
@@ -368,6 +336,7 @@ PThread::~PThread()
   PX_signature = kMPDeadSig;
 }
 
+
 void PThread::PX_NewThread(PBoolean startSuspended)
 {
   OSErr err;
@@ -439,23 +408,6 @@ long PThread::PX_ThreadStart(void * arg)
 }
 
 
-void PProcess::SignalTimerChange()
-{
-  if (housekeepingThread == NULL) {
-#if PMEMORY_CHECK
-    PBoolean oldIgnoreAllocations = PMemoryHeap::SetIgnoreAllocations(PTrue);
-#endif
-    housekeepingThread = new PHouseKeepingThread;
-#if PMEMORY_CHECK
-    PMemoryHeap::SetIgnoreAllocations(oldIgnoreAllocations);
-#endif
-  }
-
-  SetUpTermQueue();
-  MPNotifyQueue(terminationNotificationQueue, 0, 0, 0);
-}
-
-
 void PThread::PX_ThreadEnd(void * arg)
 {
   PThread * thread = (PThread *)arg;
@@ -492,7 +444,7 @@ void PThread::Restart()
   if (IsTerminated())
     return;
 
-  PX_NewThread(PFalse);
+  PX_NewThread(false);
 }
 
 
@@ -535,7 +487,7 @@ PBoolean PThread::IsTerminated() const
 {
   if (PX_threadId == 0) {
     //PTRACE(1, "tlibthrd\tIsTerminated(" << (void *)this << ") = 0");
-    return PTrue;
+    return true;
   }
 
 #ifdef _not_def_ // Sigh.  no MPGetNextTaskID on MOSX
@@ -548,14 +500,14 @@ PBoolean PThread::IsTerminated() const
       if (sometask == 0) break;
       if (sometask == PX_threadId) {
           //PTRACE(1, "tlibthrd\tIsTerminated(" << (void *)this << ") not dead yet");
-          return PFalse;
+          return false;
       }
   }
   // didn't find it, it's dead
   //PTRACE(1, "tlibthrd\tIsTerminated(" << (void *)this << ") = 0");
-  return PTrue;
+  return true;
 #else
-  return PFalse; // ENOCLUE
+  return false; // ENOCLUE
 #endif
 }
 
@@ -564,8 +516,8 @@ PBoolean PThread::IsTerminated() const
 // partial implementation using a Semaphore.
 // As a result, we can create a thread in a suspended state and then 'resume'
 // it, but once it is going, we can no longer suspend it.
-// So, for Mac OS X, we will accept Resume() calls (or Suspend(PFalse))
-// but reject Suspend(PTrue) calls with an Assertion. This will indicate
+// So, for Mac OS X, we will accept Resume() calls (or Suspend(false))
+// but reject Suspend(true) calls with an Assertion. This will indicate
 // to a user that we cannot Suspend threads on Mac OS X
 
 void PThread::Suspend(PBoolean susp)
@@ -593,7 +545,7 @@ void PThread::Suspend(PBoolean susp)
 
 void PThread::Resume()
 {
-  Suspend(PFalse);
+  Suspend(false);
 }
 
 
@@ -602,7 +554,7 @@ PBoolean PThread::IsSuspended() const
   OSStatus err;
 
   if (IsTerminated())
-    return PFalse;
+    return false;
 
   err = MPWaitOnSemaphore(PX_suspendMutex, kDurationForever);
   PAssert(err == 0, "MPWaitOnSemaphore failed");
@@ -613,13 +565,6 @@ PBoolean PThread::IsSuspended() const
 }
 
 
-void PThread::SetAutoDelete(AutoDeleteFlag deletion)
-{
-  PAssert(deletion != AutoDeleteThread || this != &PProcess::Current(), PLogicError);
-  autoDelete = deletion == AutoDeleteThread;
-}
-
-
 void PThread::SetPriority(Priority /*priorityLevel*/)
 {
 }
@@ -679,10 +624,10 @@ PBoolean PThread::WaitForTermination(const PTimeInterval & maxWait) const
   PTimer timeout = maxWait;
   while (!IsTerminated()) {
     if (timeout == 0)
-      return PFalse;
+      return false;
     Current()->Sleep(10);
   }
-  return PTrue;
+  return true;
 }
 
 
@@ -722,16 +667,16 @@ PBoolean PSemaphore::Wait(const PTimeInterval & waitTime)
     
   if (waitTime == PMaxTimeInterval) {
     Wait();
-    return PTrue;
+    return true;
   }
 
   Duration timeout = waitTime.GetMilliSeconds();
   if ((err = MPWaitOnSemaphore(semId, timeout)) == noErr)
-      return PTrue;
+      return true;
   if (err == kMPTimeoutErr)
-      return PFalse;
+      return false;
   PAssert(err == 0, psprintf("timed wait error = %i", err));
-  return PFalse;
+  return false;
 }
 
 void PSemaphore::Signal()
@@ -743,16 +688,6 @@ void PSemaphore::Signal()
 }
 
 
-PBoolean PSemaphore::WillBlock() const
-{
-    OSStatus err = MPWaitOnSemaphore(semId, kDurationImmediate);
-    if (err == kMPTimeoutErr)
-        return PTrue;
-    PAssert(err == 0, psprintf("timed wait error = %i", err));
-    (void)MPSignalSemaphore(semId);
-    return PFalse;
-}
-
 // Ideally, a PMutex would contain an MPCriticalSection instead of a
 // semaphore, but the class derivation is outside the machine-specific
 // code, and I'm unwilling to do something gross like implement a bogus
@@ -778,11 +713,6 @@ void PMutex::Signal()
 	PSemaphore::Signal();
 }
 
-PBoolean PMutex::WillBlock() const 
-{
-	return PSemaphore::WillBlock();
-}
-
 PSyncPoint::PSyncPoint()
   : PSemaphore(0, 1)
 {
