@@ -26,9 +26,9 @@
  *
  * Contributor(s):  ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 28484 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2012-10-05 03:53:11 -0500 (Fri, 05 Oct 2012) $
  */
 
 
@@ -83,7 +83,7 @@ CCriticalSection::~CCriticalSection()
 void CCriticalSection::Lock()
 {
   if (locked == false) {
-    if (::intContext() == PFalse)
+    if (::intContext() == false)
       taskLocked = ::taskLock();
     else
       taskLocked = ERROR;
@@ -114,13 +114,11 @@ static int const priorities[] = {
 
 int PThread::ThreadFunction(void *threadPtr)
 {
-	PAssertNULL(threadPtr);
+  PAssertNULL(threadPtr);
   PThread * thread = (PThread *)threadPtr;
+
   PProcess & process = PProcess::Current();
-  process.activeThreadMutex.Wait();
-  process.activeThreads.SetAt(thread->PX_threadId, thread);
-  process.activeThreadMutex.Signal();
-  process.SignalTimerChange();
+  process.InternalThreadStarted(this);
 
   if (::semTake(thread->syncPoint, WAIT_FOREVER) == OK) {
     if (::semDelete(thread->syncPoint) == OK)
@@ -159,11 +157,19 @@ void PThread::Trace(PThreadIdentifer threadId)
   ::checkStack(0);
 }
 
-PThread::PThread()
- : PX_threadId(ERROR),
-   priority(VX_NORMAL_PRIORITY),
-   originalStackSize(0)
+PThread::PThread(bool isProcess)
+  : m_isProcess(isProcess)
+  , m_autoDelete(!isProcess)
+  , m_originalStackSize(0) // 0 indicates external thread
+  , PX_threadId(::taskIdSelf())
+  , priority(VX_NORMAL_PRIORITY)
 {
+  PAssertOS((PX_threadId != ERROR) && (PX_threadId != 0));
+
+  if (isProcess)
+    return;
+
+  PProcess::Current().InternalThreadStarted(this);
 }
 
 PThread::PThread(PINDEX stackSize,
@@ -197,13 +203,6 @@ PThread::PThread(PINDEX stackSize,
 
         if (taskLocked == OK) 
           ::taskUnlock();
-
-        if (autoDelete) {
-          PProcess & process = PProcess::Current();
-          process.deleteThreadMutex.Wait();
-          process.autoDeleteThreads.Append(this);
-          process.deleteThreadMutex.Signal();
-        }
       }
       else {
         if (taskLocked == OK) 
@@ -223,13 +222,8 @@ PThread::PThread(PINDEX stackSize,
 }
 
 
-PThread::~PThread()
+void PThread::InternalDestroy()
 {
-  if (originalStackSize <= 0)
-    return;
-
-  if (!IsTerminated())
-    Terminate();
 }
 
 
@@ -292,15 +286,15 @@ void PThread::WaitForTermination() const
 PBoolean PThread::WaitForTermination(const PTimeInterval & maxWait) const
 {
   if (PX_threadId == 0)
-    return PTrue;
+    return true;
 
   PTimer timeout = maxWait;
   while (!IsTerminated()) {
     if (timeout == 0)
-      return PFalse;
+      return false;
     Current()->Sleep(100);
   }
- return PTrue;
+ return true;
 }
 
 
@@ -334,7 +328,7 @@ void PThread::Resume()
 
 PBoolean PThread::IsSuspended() const
 {
-  PBoolean isSuspended = PFalse;
+  PBoolean isSuspended = false;
   if (!IsTerminated())
     isSuspended = ::taskIsSuspended(PX_threadId);
   else
@@ -343,11 +337,6 @@ PBoolean PThread::IsSuspended() const
   return isSuspended;
 }
 
-void PThread::SetAutoDelete(AutoDeleteFlag deletion)
-{
-  PAssert(deletion != AutoDeleteThread || this != &PProcess::Current(), PLogicError);
-  autoDelete = deletion == AutoDeleteThread;
-}
 
 void PThread::SetPriority(Priority priorityLevel)
 {
@@ -401,36 +390,12 @@ void PThread::Sleep( const PTimeInterval & delay ) // Time interval to sleep for
   ::taskDelay(delay.GetInterval()*sysClkRateGet()/1000);
 }
 
-void PThread::InitialiseProcessThread()
-{
-  originalStackSize = 0;
-  autoDelete = PFalse;
-
-  PX_threadId = ::taskIdSelf();
-  PAssertOS((PX_threadId != ERROR) && (PX_threadId != 0));
-
-  ((PProcess *)this)->activeThreads.DisallowDeleteObjects();
-  ((PProcess *)this)->activeThreads.SetAt(PX_threadId, this);
-}
-
-
-PThread * PThread::Current()
-{
-  PProcess & process = PProcess::Current();
-  process.activeThreadMutex.Wait();
-  
-  PThread * thread = process.activeThreads.GetAt(::taskIdSelf());
-
-  process.activeThreadMutex.Signal();
-  return thread;
-}
-
 int PThread::PXBlockOnChildTerminate(int pid, const PTimeInterval & /*timeout*/) // Fix timeout
 {
   while (!IsTerminated()) {
     Current()->Sleep(100);
   }
-  return PTrue;
+  return true;
 }
 
 int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
@@ -470,59 +435,28 @@ void PThread::PXAbortBlock() const
 {
 }
 
-///////////////////////////////////////////////////////////////////////////////
-// PProcess::HouseKeepingThread
 
 void PProcess::Construct()
 {
   // hard coded value, change this to handle more sockets at once with the select call
   maxHandles = 1024; 
-  houseKeeper=NULL;
   CommonConstruct();
 }
 
-PProcess::HouseKeepingThread::HouseKeepingThread()
-  : PThread(10000, NoAutoDeleteThread, HighPriority, PString("HKeeping"))
-{
-  Resume();
-}
 
-void PProcess::HouseKeepingThread::Main()
+void PProcess::HouseKeeping()
 {
-	PProcess & process = PProcess::Current();
-
-	while(1) {
-		process.deleteThreadMutex.Wait();
-    for (PINDEX i = 0; i < process.autoDeleteThreads.GetSize(); i++) {
-			PThread * pThread = (PThread *) process.autoDeleteThreads.GetAt(i);
-			if( pThread->IsTerminated() )
-				process.autoDeleteThreads.RemoveAt(i--);
-			}
-		process.deleteThreadMutex.Signal();
-		PTimeInterval nextTimer = process.timers.Process();
+  while (m_signalHouseKeeper) {
+    PTimeInterval nextTimer = process.timers.Process();
     if (nextTimer != PMaxTimeInterval) {
-			if ( nextTimer.GetInterval() > 10000 )
-				nextTimer = 10000;
-			}
-		breakBlock.Wait( nextTimer );
-	}
+      if ( nextTimer.GetInterval() > 10000 )
+        nextTimer = 10000;
+    }
+    m_signalHouseKeeper.Wait( nextTimer );
+    InternalCleanAutoDeleteThreads();
+  }
 }
 
-void PProcess::SignalTimerChange()
-{
-  if (houseKeeper == NULL) {
-    // Prevent reentrance before the following assignment is done
-    // Placed after above if-statement due to efficiency, and so 
-    // requires an another NULL-test.
-    CCriticalSection section;
-    section.Lock();
-  if (houseKeeper == NULL)
-     houseKeeper = new HouseKeepingThread;  
-    section.Unlock();
-  }
-  else
-    houseKeeper->breakBlock.Signal();
-}
 
 ///////////////////////////////////////////////////////////////////////////////
 // PProcess
@@ -647,10 +581,6 @@ void PSemaphore::Signal()
   section.Unlock();
 }
 
-PBoolean PSemaphore::WillBlock() const
-{
-  return initialVar == 0;
-}
 
 ///////////////////////////////////////////////////////////////////////////////
 // PMutex  
@@ -703,14 +633,6 @@ void PMutex::Signal()
 	::semGive(semId);
 }
 
-PBoolean PMutex::WillBlock() const 
-{
-  STATUS result = ::semTake(semId, NO_WAIT);
-  PAssertOS((result != ERROR) || (errno == S_objLib_OBJ_UNAVAILABLE));
-  if (result != ERROR)
-    ::semGive(semId);
-		return result == ERROR;
-}
 
 ///////////////////////////////////////////////////////////////////////////////
 // PSyncPoint
